!===============================================================================
! CVS $Id: advance.F90,v 1.6 2001/08/01 22:56:45 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/atm/datm5/advance.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE advance()

      use calendar_mod  ! calendar routines
      use shr_timer_mod ! shared code: timing routines

      implicit none

      !----- global -----
#include <dims.h>      ! model resolution
#include <data_raw.h>  ! data read from data file
#include <data_msg.h>  ! data sent/received from coupler
#include <control.h>   ! control info, date/time

      !----- local  ------
      integer   i,j          ! generic loop indicies
      integer   k1,k2        ! raw data time-slice indicies
      real      eday1        ! elapsed days for 1st data sample
      real      eday2        ! elapsed days for 2nd data sample
      real      edaym        ! elapsed days for model date
      integer   edayn        ! elapsed days temporary variable
      real      f1,f2        ! raw data fraction
      real      alb          ! albedo used wrt albedo feedback option
      real      f            ! factor used wrt sw-down components
      integer   timer_n      ! timer number
      logical   first_call   ! flags 1st envocation of this routine

      data      first_call /.true./
      save

!-------------------------------------------------------------------------------
! PURPOSE:
!   advance the model state to the current date
!   
! METHOD:
!   fill atm output state & flux field arrays appropriately
!   (these arrays will subsequently be sent to the coupler)
!-------------------------------------------------------------------------------

      if (first_call) call shr_timer_get(timer_n,'advance')
      call shr_timer_start(timer_n)

      !-----------------------------------------------------------------
      ! get raw data time slices that bracket current model date & time 
      !-----------------------------------------------------------------
      call data_in()

      !-----------------------------------------------------------------
      ! determine time interpolation factors
      !-----------------------------------------------------------------
      if (info_dbug.gt.2) write(6,9001) 'advancing to date = ',cdate,sec
      k1 = r_kold  ! k1 must be either 1 or 2
      k2 = 3 - k1

      call date2eday(mod(r_cdate(k1),10000),edayn)
      eday1 = float(edayn) + r_sec(k1)/86400.0
      
      call date2eday(mod(r_cdate(k2),10000),edayn)
      eday2 = float(edayn) + r_sec(k2)/86400.0 
      
      call date2eday( mod(cdate,10000)     ,edayn) 
      edaym = float(edayn) + sec/86400.0

      !--- want eday1 < edaym < eday2 ---
      if (eday1 .gt. edaym) eday1 = eday1 - 365.
      if (eday2 .lt. edaym) eday2 = eday2 + 365.
      f1 = (eday2-edaym)/(eday2-eday1)
      f2 = 1.0 - f1

      !-----------------------------------------------------------------
      ! time interpolate raw data into model data fields
      !-----------------------------------------------------------------
      DO j=1,ny
      DO i=1,nx
         z    (i,j) = f1*r_z    (i,j,k1) + f2*r_z    (i,j,k2)
         u    (i,j) = f1*r_u    (i,j,k1) + f2*r_u    (i,j,k2)
         v    (i,j) = f1*r_v    (i,j,k1) + f2*r_v    (i,j,k2)
         t    (i,j) = f1*r_t    (i,j,k1) + f2*r_t    (i,j,k2)
         ptem (i,j) = f1*r_ptem (i,j,k1) + f2*r_ptem (i,j,k2)
         pres (i,j) = f1*r_pres (i,j,k1) + f2*r_pres (i,j,k2)
         pslv (i,j) = f1*r_pslv (i,j,k1) + f2*r_pslv (i,j,k2)
         shum (i,j) = f1*r_shum (i,j,k1) + f2*r_shum (i,j,k2)
         dens (i,j) = f1*r_dens (i,j,k1) + f2*r_dens (i,j,k2)
         precc(i,j) = f1*r_precc(i,j,k1) + f2*r_precc(i,j,k2)
         precl(i,j) = f1*r_precl(i,j,k1) + f2*r_precl(i,j,k2)
         snowc(i,j) = f1*r_snowc(i,j,k1) + f2*r_snowc(i,j,k2)
         snowl(i,j) = f1*r_snowl(i,j,k1) + f2*r_snowl(i,j,k2)
         lwdn (i,j) = f1*r_lwdn (i,j,k1) + f2*r_lwdn (i,j,k2)
         swnet(i,j) = f1*r_swnet(i,j,k1) + f2*r_swnet(i,j,k2)
         swvdr(i,j) = f1*r_swvdr(i,j,k1) + f2*r_swvdr(i,j,k2)
         swndr(i,j) = f1*r_swndr(i,j,k1) + f2*r_swndr(i,j,k2)
         swvdf(i,j) = f1*r_swvdf(i,j,k1) + f2*r_swvdf(i,j,k2)
         swndf(i,j) = f1*r_swndf(i,j,k1) + f2*r_swndf(i,j,k2)
      END DO
      END DO

      !-----------------------------------------------------------------
      ! albedo-feedback is on => alter short-wave wrt surface albedo
      !-----------------------------------------------------------------
      if (flux_albfb.eq.0) then
        if (first_call) write(6,9000) 'NOTE: albedo feedback is OFF'
      else
        if (first_call .and. rest_type == 'initial') then
          write(6,9000) 'NOTE: albedo feedback is ON'
          !--- cannot apply FB on 1st call because albedos are not set ---
        else
          !--- compute swnet consisten with (1-albedo)*swdn ---
          !--- note: need albedos from cpl before doing this ---
          do j=1,ny
          do i=1,nx
            f =     ( 1.0 - alvdr(i,j) )*swvdr(i,j)
            f = f + ( 1.0 - alndr(i,j) )*swndr(i,j)
            f = f + ( 1.0 - alvdf(i,j) )*swvdf(i,j)
            f = f + ( 1.0 - alndf(i,j) )*swndf(i,j)
            swnet(i,j) = f
          enddo
          enddo
        end if
      end if

      if (info_sleep > 0.0) call shr_timer_sleep(info_sleep)
      call shr_timer_stop(timer_n)

      !-----------------------------------------------------------------
      ! optional debug info
      !-----------------------------------------------------------------
      if ( (info_dbug.ge.2 .and. sec.eq.0) .or. info_dbug.ge.3) then
        do j=1,ny
        do i=1,nx
           if (alb.lt.0 .or. alb.gt.1) write(6,9010) xc(i,j),yc(i,j),alb
        enddo
        enddo
        write(6,9008) 't1: ',r_t(5,5,k1),r_cdate(k1)+r_sec(k1)/86400.,f1
        write(6,9008) 't : ',  t(5,5)   ,  cdate    +  sec    /86400.
        write(6,9008) 't2: ',r_t(5,5,k2),r_cdate(k2)+r_sec(k2)/86400.,f2
      end if

      first_call = .false.
      RETURN
 9000 format('(advance) ',a,a)
 9001 format('(advance) ',a,i8.8,i5,'sec')
 9008 format('(advance) ',a,f8.3,1x,f12.2,f6.3)
 9010 format('(advance) WARNING: x, y, albedo = ',2f8.3,1x,e11.3)
      END

!=======================================================================
