!===============================================================================
! CVS $Id: data_in.F90,v 1.2 2001/01/09 22:53:38 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/atm/datm5/data_in.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE data_in()

      use calendar_mod ! calendar routines

      implicit none

      !----- global -----
#include <dims.h>            ! model dimensions
#include <data_raw.h>        ! raw data arrays
#include <control.h>         ! control flags

      !----- local  ------
      integer      i,j,k,n   ! generic indicies
      integer      year0     ! 1st year in raw data sequence
      integer      nyear     ! number of years in sequence
      integer      oyear     ! model year matched to raw data year0

      integer      dyear     ! data  year  of input data
      integer      dmonth    ! data  month of input data
      integer      dday      ! data  day   of input data

      integer      r_knew    ! k-index for newer data slice

      integer      reday     ! model eday  assigned to input data
      integer      ryear     ! model year  assigned to input data
      integer      rmonth    ! model month assigned to input data
      integer      rday      ! model day   assigned to input data

      logical      firstcall ! flags one-time startup operations
 
      data         firstcall /.true./

      save

!-------------------------------------------------------------------------------
! PURPOSE:
!   update raw data fields
! ASSUMPTIONS:
!   o input data is in the form of daily mean fields
!   o input data is assigned to noon of any given day
!-------------------------------------------------------------------------------

      if (data_dir(1)(1:4) .eq. 'null') then
        !-----------------------------------------------------------------------
        ! make bogus data that brackets model date & time 
        !-----------------------------------------------------------------------
        r_kold     = 1
        r_cdate(1) = cdate
        r_sec  (1) = sec
        r_cdate(2) = cdate
        r_sec  (2) = sec + 86400/2
        if (firstcall) then
          write(6,9000) 'null data dir => fields constant in (x,y,t)'
          DO k=1,2
          DO j=1,ny
          DO i=1,nx
            r_z    (i,j,k) =  55.0 ! bottom level height       ~ m
            r_u    (i,j,k) =   0.0 ! wind, zonal               ~ m/s
            r_v    (i,j,k) =   2.0 ! wind, meridional          ~ m/s
            r_ptem (i,j,k) = 301.0 ! potential temperature     ~ Kelvin
            r_t    (i,j,k) = 300.0 ! temperature               ~ Kelvin
            r_shum (i,j,k) = 1.e-2 ! specific humidity         ~ kg/kg
            r_dens (i,j,k) =   1.0 ! density                   ~ kg/m^3
            r_pres (i,j,k) = 1.e+6 ! pressure                  ~ Pa
            r_pslv (i,j,k) = 1.e+6 ! pressure, sea level       ~ Pa
            r_precc(i,j,k) = 1.e-6 ! precip, convective        ~ kg/s/m^2
            r_precl(i,j,k) = 1.e-6 ! precip, large-scale       ~ kg/s/m^2
            r_lwdn (i,j,k) = 100.0 ! longwave, downward        ~    W/m^2
            r_swnet(i,j,k) = 100.0 ! shortwave, net            ~    W/m^2
            r_swvdr(i,j,k) = 100.0 ! shortwave, down, vis-dir  ~    W/m^2
            r_swndr(i,j,k) = 100.0 ! shortwave, down, nir-dir  ~    W/m^2
            r_swvdf(i,j,k) = 100.0 ! shortwave, down, vis-diff ~    W/m^2
            r_swndf(i,j,k) = 100.0 ! shortwave, down, nir-diff ~    W/m^2
          END DO
          END DO
          END DO
        end if
      else
        !-----------------------------------------------------------------------
        ! get real data that brackets model date & time 
        !-----------------------------------------------------------------------

        if (firstcall) then
          !------------------------------------------
          ! document the data cycle
          !------------------------------------------
          n=1                   ! n=1 => state data
          year0 = data_year0(n) ! 1st year in data sequence
          nyear = data_nyear(n) ! # of years in data sequence
          oyear = data_oyear(n) ! coupler year corresponding to year0
          dyear = year0 + modulo(year-oyear,nyear) ! current data year

          write(6,9001) 'first year in data sequence = '  ,year0
          write(6,9001) 'last  year in data sequence = '  ,year0+nyear-1
          write(6,9001) 'model year ',oyear,' => data year ',year0
          write(6,9001) 'model year ', year,' => data year ',dyear

          !-----------------------------------------------------
          ! set new/old dates to get two data slices on startup
          !-----------------------------------------------------
          r_kold = 1
          r_cdate(r_kold) = -999 
          r_sec(r_kold)   = 0
         
          !-----------------------------------------------
          ! compute date of data that brackets from below
          !-----------------------------------------------
          r_knew = 3-r_kold ! put data in this k-index

          reday  = eday - 1 ! we want data corresponding to this date
          if (reday .ge. 0) then
            !--- normal situation ---
            call eday2ymd (reday,ryear,rmonth,rday)
            call eday2date(reday,r_cdate(r_knew))
            r_sec(r_knew) = 86400/2  ! mid-day as per daily avg
          else if (reday .eq. -1) then
            !--- calendar routines can't handle negative years ---
            ryear  =  0
            rmonth =  1
            rday   =  1
            r_sec  (r_knew) = 0  
            r_cdate(r_knew) = 0101
          else 
            write(6,9000) 'can''t handle dates with negative years'
            stop
          end if
          write(6,9002) 'model date assigned to data = '  &
          &             ,r_cdate(r_knew),r_sec(r_knew),'sec'

          !----------------------------------------------
          ! get the data that brakets from below
          !----------------------------------------------
          dyear  = year0 + modulo(ryear-oyear,nyear)
          dmonth = rmonth
          dday   = rday  
          call data_in_nc(dyear,dmonth,dday,r_knew)

        endif

        r_knew = 3-r_kold ! k index of newer data

        if ( (cdate.eq.r_cdate(r_knew) .and. sec.ge.r_sec(r_knew)) &
        &  .or.  cdate.gt.r_cdate(r_knew) ) then
          !-----------------------------------------------
          ! advance raw data time slices
          !-----------------------------------------------
          r_kold = r_knew    ! toggle new/old k-index
          r_knew = 3-r_kold  ! toggle new/old k-index

          !-----------------------------------------------
          ! compute date of data that brackets from above
          !-----------------------------------------------
          if (sec.lt.43200) reday = eday    ! we want data corresponding
          if (sec.ge.43200) reday = eday + 1 ! to this date
          call eday2ymd (reday,ryear,rmonth,rday)
          call eday2date(reday,r_cdate(r_knew))
          r_sec(r_knew) = 86400/2  ! mid-day as per daily avg
          if ((info_dbug.eq.1.and.day.eq.0) .or. info_dbug.ge.2) then
            write(6,9002) 'model date assigned to data = '  &
            &             ,r_cdate(r_knew),r_sec(r_knew),'sec'
          end if
          !----------------------------------------------
          ! get the data that brakets from above
          !----------------------------------------------
          dyear  = year0 + modulo(ryear-oyear,nyear)
          dmonth = rmonth
          dday   = rday  
          call data_in_nc(dyear,dmonth,dday,r_knew)

        else
          if (info_dbug.ge.3) write(6,9000) 'don''t need new data'
        endif

      endif

      firstcall = .false.

      RETURN
 9000 format('(data_in) ',a,a)
 9001 format('(data_in) ',a,i4,a,i4)
 9002 format('(data_in) ',a,i8.8,' ',i5,a)
      END
 
!===============================================================================
