!===============================================================================
! CVS $Id: data_in_nc.F90,v 1.4 2001/07/06 18:06:13 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/atm/datm5/data_in_nc.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      subroutine data_in_nc(dyear,dmonth,dday,k)

      use shr_sys_mod  ! shared code: system calls
      use shr_file_mod ! shared code: file archive
      use const_mod    ! constants

      implicit none

      !----- global ------
#include <dims.h>               ! model dimensions
#include <data_raw.h>           ! raw data arrays
#include <control.h>            ! control flags
#include <netcdf.inc>           ! netCDF stuff

      !----- arguments ------
      integer       dyear       ! (in) year to read
      integer       dmonth      ! (in) month to read
      integer       dday        ! (in) day to read
      integer       k           ! (in) raw data k index to be replaced

      !----- local  ------
      integer       i,j,n        ! generic indicies
      character*800 str          ! data file ID string
      character*120 mss_fn       ! MSS file name
      character*80  loc_fn       ! local file name
      integer       fid          ! nc file ID
      integer       vid          ! nc variable ID
      integer       beg(3)       ! nc hyperslab begin vector
      integer       cnt(3)       ! nc hyperslab count vector
      integer       fdate        ! time coordinate (date) from data file
      integer       rcode        ! nc return code
      integer       last_dmonth  ! month of last nc data read in
      logical       missing_pslv ! flags that pslv data is not in input file
      logical       missing_swdn ! flags that swdn data is not in input file
      logical       missing_snow ! flags that snow data is not in input file
      real          Tatm         ! used for converting rain to snow
      real          frac         ! used for converting rain to snow
      logical       first_call   ! flags 1st envocation of this routine

      data          last_dmonth  / -999 /
      data          first_call   /.true./
      data          missing_swdn /.false./
      data          missing_snow /.false./
      data          missing_pslv /.false./

      save    

!-------------------------------------------------------------------------------
! PURPOSE:
!   acquire a raw data file & read the file
!-------------------------------------------------------------------------------

      if (dmonth .ne. last_dmonth) then
        !-----------------------------------------------------------------------
        ! acquire & open a new data file
        !-----------------------------------------------------------------------

        !----- close & remove an old file -----
        if (.not. first_call) then
           rcode = nf_close(fid)
           if (data_rmlf(1).ne.0) then
             call shr_sys_system('rm '//loc_fn,rcode)
             write(6,9000) 'rm ',loc_fn(1:index(loc_fn,' '))
           end if
        end if
        last_dmonth= dmonth
        if ((info_dbug.eq.1.and.day.eq.1) .or. info_dbug.ge.2) then
           write(6,9001) 'acquire file for data year,month = ',dyear,dmonth
        endif

        !----- msread & open a new file -----
        loc_fn = "yyyy-mm.nc "
        write(loc_fn(1:4),'(i4.4)') dyear
        write(loc_fn(6:7),'(i2.2)') dmonth
        j=index(loc_fn,' ')-1

        i = index(data_dir(1),' ')-1
        if (data_dir(1)(i:i) .eq. '/') i = i-1
        mss_fn = data_dir(1)(1:i) // '/' // loc_fn
        i=index(mss_fn,' ')-1
 
        call shr_file_get (rcode,loc_fn,mss_fn)
        write(6,9005) rcode,'acquire',trim(loc_fn),trim(mss_fn)
 
        write(6,9000) 'open netCDF file ',trim(loc_fn)
        rcode = nf_open(loc_fn,NF_NOWRITE,fid)
        if (rcode.ne.NF_NOERR) write(6,9000) nf_strerror(rcode)
 
        !----- print title info from nc file -----
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'title', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'title'  , str)
        else
          str = '<too long to print>   '
        end if
        write(6,9000) 'o title   = ',trim(str)
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'history', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'history', str)
        else
          str = '<too long to print>   '
        end if
        write(6,9000) 'o history = ',trim(str)
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'source', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'source' , str)
        else
          str = '<too long to print>   '
        end if
        write(6,9000) 'o source  = ',trim(str)
 
        last_dmonth = dmonth
      endif
      
      !-------------------------------------------------------------------------
      ! read data from an open file
      !-------------------------------------------------------------------------
      if ((info_dbug.eq.1.and.day.eq.1) .or. info_dbug.ge.2) then
        write(6,9002) 'desired data date = ',dyear*10000+dmonth*100+dday
      endif

      beg(1) = dday
      cnt(1) = 1
      rcode = nf_inq_varid(fid,'date',vid)
      rcode = nf_get_vara_int(fid,vid,beg,cnt,fdate)
      if ((info_dbug.eq.1.and.day.eq.1) .or. info_dbug.ge.2) then
        write(6,9002) 'file''s  data date = ',fdate
      endif

      beg(1) = 1
      beg(2) = 1
      beg(3) = dday
      cnt(1) = nx
      cnt(2) = ny
      cnt(3) = 1

      rcode = nf_inq_varid(fid,'z'    ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_z    (1,1,k))
      rcode = nf_inq_varid(fid,'v'    ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_v    (1,1,k))
      rcode = nf_inq_varid(fid,'u'    ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_u    (1,1,k))
      rcode = nf_inq_varid(fid,'t'    ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_t    (1,1,k))
      rcode = nf_inq_varid(fid,'ptem' ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_ptem (1,1,k))
      rcode = nf_inq_varid(fid,'shum' ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_shum (1,1,k))
      rcode = nf_inq_varid(fid,'dens' ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_dens (1,1,k))
      rcode = nf_inq_varid(fid,'pres' ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_pres (1,1,k))
      rcode = nf_inq_varid(fid,'precc',vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_precc(1,1,k))
      rcode = nf_inq_varid(fid,'precl',vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_precl(1,1,k))
      rcode = nf_inq_varid(fid,'swnet',vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_swnet(1,1,k))
      rcode = nf_inq_varid(fid,'swdn' ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_swdn (1,1,k))
      rcode = nf_inq_varid(fid,'lwdn' ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_lwdn (1,1,k))

      !------------------------------------------------
      ! the fields above MUST BE in the data file 
      ! the fields below will be fabricated if missing
      !------------------------------------------------

      if (nf_inq_varid(fid,'psl'  ,vid) /= 0) missing_pslv = .true.
      if ( .not. missing_pslv) then
        rcode = nf_get_vara_double(fid,vid,beg,cnt,r_pslv (1,1,k))
      endif
      if (nf_inq_varid(fid,'snowl',vid) /= 0) missing_snow = .true.
      if (nf_inq_varid(fid,'snowc',vid) /= 0) missing_snow = .true.
      if ( .not. missing_snow) then
        rcode = nf_get_vara_double(fid,vid,beg,cnt,r_snowc(1,1,k))
        rcode = nf_get_vara_double(fid,vid,beg,cnt,r_snowl(1,1,k))
      end if
      if (nf_inq_varid(fid,'swvdr',vid) /= 0) missing_swdn = .true.
      if (nf_inq_varid(fid,'swvdf',vid) /= 0) missing_swdn = .true.
      if (nf_inq_varid(fid,'swndr',vid) /= 0) missing_swdn = .true.
      if (nf_inq_varid(fid,'swndf',vid) /= 0) missing_swdn = .true.
      if ( .not. missing_swdn) then
        rcode = nf_get_vara_double(fid,vid,beg,cnt,r_swvdr(1,1,k))
        rcode = nf_get_vara_double(fid,vid,beg,cnt,r_swvdf(1,1,k))
        rcode = nf_get_vara_double(fid,vid,beg,cnt,r_swndr(1,1,k))
        rcode = nf_get_vara_double(fid,vid,beg,cnt,r_swndf(1,1,k))
      end if

      !-------------------------------------------------------------------------
      ! fabricate any missing data
      !-------------------------------------------------------------------------
      if ( missing_swdn) then
        if (first_call) then
           write(6,9000) 'WARNING: sw-down components are not in data file'
           write(6,9000) 'WARNING: using swvdr=0.28*swdn'
           write(6,9000) 'WARNING: using swndr=0.31*swdn'
           write(6,9000) 'WARNING: using swvdf=0.24*swdn'
           write(6,9000) 'WARNING: using swndf=0.17*swdn'
         end if
         r_swvdr(:,:,k) = r_swdn(:,:,k)*0.28
         r_swndr(:,:,k) = r_swdn(:,:,k)*0.31
         r_swvdf(:,:,k) = r_swdn(:,:,k)*0.24
         r_swndf(:,:,k) = r_swdn(:,:,k)*0.17
      end if

      if ( missing_pslv) then
        if (first_call) then
           write(6,9000) 'WARNING: sea-level pressure is not in data file'
           write(6,9000) 'WARNING: using pslv = pres'
         end if
        r_pslv (:,:,k) = r_pres(:,:,k) 
      end if

      if ( missing_snow) then
        if (first_call) then
          write(6,9000) 'WARNING: snow field is not in data file'
        end if
        if (.not. flux_rain2snow) then
          if (first_call) write(6,9000) 'WARNING: using snow = 0'
          r_snowc(i,j,k) = 0.0
          r_snowl(i,j,k) = 0.0
        else
          if (first_call) &
          &  write(6,9000) 'WARNING: converting prec to snow where T < zero C'
          do j=1,ny
          do i=1,nx
            Tatm =  r_t(i,j,k)
            if ( Tatm > (Tzro + 2.0) ) then
              r_snowc(i,j,k) = 0.0
              r_snowl(i,j,k) = 0.0
            else if ( Tatm < Tzro) then
              r_snowc(i,j,k) = r_precc(i,j,k)
              r_snowl(i,j,k) = r_precl(i,j,k)
              r_precc(i,j,k) = 0.0
              r_precl(i,j,k) = 0.0
            else
              frac = ( (Tzro+2.0) - Tatm)/2.0
              r_snowc(i,j,k) = r_precc(i,j,k)*frac
              r_snowl(i,j,k) = r_precl(i,j,k)*frac
              r_precc(i,j,k) = r_precc(i,j,k)*(1.0 - frac)
              r_precl(i,j,k) = r_precl(i,j,k)*(1.0 - frac)
            endif
          end do
          end do
        end if
      end if

      !-------------------------------------------------------------------------
      ! adjust short-wave by a user supplied fudge factor?
      !-------------------------------------------------------------------------
      if (flux_swfact  /= 1.0) then
        if (first_call) then
          write(6,9003)'NOTE: user supplied sw factor =',flux_swfact
          write(6,9003)'NOTE: adjusting *all* short-wave fields by this factor'
        endif
        r_swdn (:,:,k) = r_swdn (:,:,k)*flux_swfact
        r_swnet(:,:,k) = r_swnet(:,:,k)*flux_swfact
        r_swvdr(:,:,k) = r_swvdr(:,:,k)*flux_swfact
        r_swndr(:,:,k) = r_swndr(:,:,k)*flux_swfact
        r_swvdf(:,:,k) = r_swvdf(:,:,k)*flux_swfact
        r_swndf(:,:,k) = r_swndf(:,:,k)*flux_swfact
      endif

      !-------------------------------------------------------------------------
      ! optional debug info
      !-------------------------------------------------------------------------
      if (info_dbug.ge.3) then
        write(6,9010) '<dbug> t    :',r_t    (5,5,k)
        write(6,9010) '<dbug> v    :',r_u    (5,5,k)
        write(6,9010) '<dbug> u    :',r_u    (5,5,k)
        write(6,9010) '<dbug> ptem :',r_ptem (5,5,k)
        write(6,9010) '<dbug> shum :',r_shum (5,5,k)
        write(6,9010) '<dbug> dens :',r_dens (5,5,k)
        write(6,9010) '<dbug> pres :',r_pres (5,5,k)
        write(6,9010) '<dbug> pslv :',r_pslv (5,5,k)
        write(6,9010) '<dbug> swnet:',r_swnet(5,5,k)
        write(6,9010) '<dbug> swdn :',r_swdn (5,5,k)
        write(6,9010) '<dbug> lwdn :',r_lwdn (5,5,k)
      end if

      first_call = .false.
      RETURN
 9000 format('(data_in_nc) ',4a)
 9001 format('(data_in_nc) ',a,i4.4,', ',i2)
 9002 format('(data_in_nc) ',a,i8.8)
 9003 format('(data_in_nc) ',a,f9.6)
 9005 format('(data_in_nc) return code ',i3, ' on ',4(a,1x))
 9010 format('(data_in_nc) ',a,e12.3)
      END

!===============================================================================
