!===============================================================================
! CVS $Id: data_msg.h,v 1.2 2001/01/09 22:53:38 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/atm/datm5/data_msg.h,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================
!  Sign convention:
!    positive value <=> downward flux
!
!  Unit convention:
!    T             ~ Kelvin
!    u,v           ~ m/s
!    pressure      ~ N/m^2 = Pa
!    heat flux     ~ W/m^2
!    momentum flux ~ N/m^2
!    water flux    ~ (kg/s)/m^2
!    salt  flux    ~ (kg/s)/m^2
!    coordinates   ~ degrees north or degrees east
!-------------------------------------------------------------------------------

      integer,parameter :: nibuff =100 ! size of info/flag-buffer

      !-------------------------------------------------------------------------
      ! output: data sent to coupler  
      !-------------------------------------------------------------------------

      !----- control flags ---------------------------------------------
      integer   ibuffs(nibuff) ! control flags sent to coupler

      !----- domain -----
      real    xc  (nx,ny)     ! x-grid center coordinates ~ deg east
      real    yc  (nx,ny)     ! y-grid center coordinates ~ deg north
      real    xv(4,nx,ny)     ! x-grid vertex coordinates ~ deg east
      real    yv(4,nx,ny)     ! y-grid vertex coordinates ~ deg north
      real    area(nx,ny)     ! area of grid cells        ~ radians squared
      integer mask(nx,ny)     ! mask: 0 <=> not a active grid cell

      !----- states -----
      real   z    (nx,ny)      ! bottom level height       ~ m
      real   u    (nx,ny)      ! wind, zonal               ~ m/s
      real   v    (nx,ny)      ! wind, meridional          ~ m/s
      real   t    (nx,ny)      ! temperature               ~ Kelvin
      real   ptem (nx,ny)      ! potential temperature     ~ Kelvin
      real   pres (nx,ny)      ! pressure, bottom layer    ~ Pa
      real   pslv (nx,ny)      ! pressure, sea level       ~ Pa
      real   shum (nx,ny)      ! specific humidity         ~ kg/kg
      real   dens (nx,ny)      ! density                   ~ kg/m^3
  
      !----- fluxes -----
      real   precc(nx,ny)      ! precip, liquid, convective  ~ kg/s/m^2
      real   precl(nx,ny)      ! precip, liquid, large-scale ~ kg/s/m^2
      real   snowc(nx,ny)      ! precip, frozen, convective  ~ kg/s/m^2
      real   snowl(nx,ny)      ! precip, frozen, large-scale ~ kg/s/m^2
      real   lwdn (nx,ny)      ! longwave, downward        ~    W/m^2
      real   swnet(nx,ny)      ! shortwave, net            ~    W/m^2
      real   swvdr(nx,ny)      ! shortwave, down, vis-dir  ~    W/m^2
      real   swndr(nx,ny)      ! shortwave, down, nir-dir  ~    W/m^2
      real   swvdf(nx,ny)      ! shortwave, down, vis-diff ~    W/m^2
      real   swndf(nx,ny)      ! shortwave, down, nir-diff ~    W/m^2

      !-------------------------------------------------------------------------
      ! input: data received from coupler 
      !-------------------------------------------------------------------------

      !----- control flags ---------------------------------------------
      integer   ibuffr(nibuff) ! control flags received from coupler
      real      rbuffr(nibuff) ! control flags received from coupler

      !----- fluxes -----
      real   taux (nx,ny)      ! wind stress, zonal        ~    N/m^2
      real   tauy (nx,ny)      ! wind stress, meridional   ~    N/m^2
      real   lat  (nx,ny)      ! latent                    ~    W/m^2
      real   sen  (nx,ny)      ! sensible                  ~    W/m^2
      real   lwup (nx,ny)      ! longwave, upward          ~    W/m^2
      real   evap (nx,ny)      ! evaporation               ~ kg/s/m^2

      !----- states -----
      real   alvdr(nx,ny)      ! albedo: vis-dir           ~ [0,1]
      real   alndr(nx,ny)      ! albedo: nir-dir           ~ [0,1]
      real   alvdf(nx,ny)      ! albedo: vis-diff          ~ [0,1]
      real   alndf(nx,ny)      ! albedo: nir-diff          ~ [0,1]
      real   st   (nx,ny)      ! surface temperature       ~ Kelvin
      real   snowh(nx,ny)      ! surface snow height       ~ m
      real   fi   (nx,ny)      ! fraction of ice           ~ [0,1]
      real   fo   (nx,ny)      ! fraction of ocn           ~ [0,1]
      real   fl   (nx,ny)      ! fraction of lnd           ~ [0,1]

      !----- diagnostics -----
      real   tref (nx,ny)      ! 2m reference temperature  ~ Kelvin

      !-------------------------------------------------------------------------
      ! make data global
      !-------------------------------------------------------------------------
      COMMON /data_msg/                                             &
     &    ibuffs                                                    &
     &  , xc    , yc    , xv    , yv    , area  , mask              &
     &  , z     , u     , v     , ptem  , t     , pres  , pslv      &
     &  , shum  , dens  , precc , precl , snowc , snowl             &
     &  , lwdn  , swnet , swvdr , swndr , swvdf , swndf             &
     &  , ibuffr, rbuffr                                            &
     &  , taux  , tauy  , lat   , sen   , lwup  , evap              &
     &  , alvdr , alndr , alvdf , alndf                             &
     &  , snowh , st    , fi    , fo    , fl                        &
     &  , tref

      SAVE   /data_msg/
