!===============================================================================
! CVS $Id: domain.F90,v 1.3 2001/07/06 18:06:14 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/atm/datm5/domain.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE domain()

      use shr_file_mod ! shared code: file archive
      use const_mod    ! physical constants

      implicit none

      !----- global -----
#include <dims.h>
#include <data_msg.h>
#include <data_raw.h>
#include <control.h>
#include <netcdf.inc>

      !----- local  ------
      real          xt(nx)   ! cell center x-coords
      real          yt(ny)   ! cell center y-coords
      real          xe(nx+1) ! cell edge   x-coords
      real          ye(ny+1) ! cell edge   y-coords
      real          dth      ! d(theta) for grid cell (radians)
      real          dph      ! d(phi)   for grid cell (radians)
      
      integer       i,j,n   ! generic indicies
      character*240 str     ! data file ID string
      character*120 mss_fn  ! MSS   file name
      character*80  loc_fn  ! local file name
      integer       fid     ! nc file ID
      integer       vid     ! nc variable ID
      integer       did     ! nc dimension ID
      integer       rcode   ! return code
      
!-------------------------------------------------------------------------------
! PURPOSE:
!   get and/or set the model domain info
!-------------------------------------------------------------------------------

      if (data_dir(1)(1:4).eq.'null') then
        !-----------------------------------------------------------------------
        ! fabricate grid data suitable for a code test
        !-----------------------------------------------------------------------
        write(6,9000) 'null data file => make bogus domain data'
        call unifgrid(0.0,360.0,nx+1,xe)
        do i=1,nx
          xt(i)= (xe(i)+xe(i+1))*0.5
        end do
        call unifgrid(-90.0,90.0,ny+1,ye)
        do j=1,ny
          yt(j)= (ye(j)+ye(j+1))*0.5
        end do

      else 
        !-----------------------------------------------------------------------
        ! read domain data from a time-variant data file
        !-----------------------------------------------------------------------
        write(6,9000) 'acquire domain data file from MSS'

        loc_fn = "yyyy-01.nc "
        write(loc_fn(1:4),'(i4.4)') data_year0(1)
        i = index(data_dir(1),' ')-1
        j = index(loc_fn     ,' ')-1
        if (data_dir(1)(i:i) .eq. '/') i = i-1
        mss_fn = data_dir(1)(1:i) // '/' // loc_fn(1:j) // ' '
        i = index(mss_fn     ,' ')-1

        call shr_file_get (rcode,loc_fn,mss_fn)
        write(6,9010) rcode,'acquire',trim(loc_fn),trim(mss_fn)

        write(6,9000) 'open netCDF file ',loc_fn
        rcode = nf_open(loc_fn,NF_NOWRITE,fid)
        if (rcode.ne.NF_NOERR) write(6,9000) nf_strerror(rcode)

        !--- print title info from nc file ---
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'title', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'title'  , str)
        else
          str = '<too long to print>   '
        end if
        write(6,9000) 'o title   = ',trim(str)
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'history', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'history', str)
        else
          str = '<too long to print>   '
        end if
        write(6,9000) 'o history = ',trim(str)
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'source', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'source' , str)
        else
          str = '<too long to print>   '
        end if
        write(6,9000) 'o source  = ',trim(str)

        rcode = nf_inq_varid(fid,'lon',vid)
        rcode = nf_get_var_double(fid,vid,xt)
        rcode = nf_inq_varid(fid,'lat',vid)
        rcode = nf_get_var_double(fid,vid,yt)
        if (rcode.ne.NF_NOERR) write(6,9000) nf_strerror(rcode)

        rcode = nf_close(fid)

        !--- construct edge grids from center grids ---
        xe(   1) = xt( 1) - (xt(2) - xt(1))/2.
        xe(nx+1) = xt(nx) + (xt(2) - xt(1))/2.
        do i=2,nx
           xe(i) = (xt(i-1)+xt(i))/2.
        enddo
  
        ye(   1) = -90.
        ye(ny+1) = +90.
        do j=2,ny
           ye(j) = (yt(j-1)+yt(j))/2.
        enddo

      end if

      write(6,9001) 'nx,ny = ',nx,ny
      write(6,9002) 'xt: ',(xt(i),i=1,4),' ...',(xt(i),i=nx-2,nx)
      write(6,9002) 'xe: ',(xe(i),i=1,4),' ...',(xe(i),i=nx-2,nx+1)
      write(6,9002) 'yt: ',(yt(j),j=1,4),' ...',(yt(j),j=ny-2,ny)
      write(6,9002) 'ye: ',(ye(j),j=1,4),' ...',(ye(j),j=ny-2,ny+1)

      !--- reformat domain data to format required by cpl ---
      do j=1,ny
      do i=1,nx
        xc(  i,j)=xt(i)
        yc(  i,j)=yt(j)
        xv(1,i,j)=xe(i  )
        xv(2,i,j)=xe(i+1)
        xv(3,i,j)=xe(i+1)
        xv(4,i,j)=xe(i  )
        yv(1,i,j)=ye(j  )
        yv(2,i,j)=ye(j  )
        yv(3,i,j)=ye(j+1)
        yv(4,i,j)=ye(j+1)
        mask(i,j) = 1        ! assume all cells are active
        dph = ( xe(i+1) - xe(i) )*DEGtoRAD
        dth = sin(ye(j+1)*DEGtoRAD)-sin(ye(j)*DEGtoRAD)
        area(i,j) = dth*dph
      end do
      end do

      RETURN
 9000 format('(domain) ',4a)
 9001 format('(domain) ',a,2i4)
 9002 format('(domain) ',2(a,4f9.3))
 9010 format('(domain) return code ',i3, ' on ',4(a,1x))
      END
 
!===============================================================================
 
      SUBROUTINE UnifGrid(x1,xn,n,x)
 
      implicit  none
 
      !----- arguments -----
      integer   n
      real      x1,xn,x(n)
 
      !----- local -----
      integer   i
      real      dx
 
!-------------------------------------------------------------------------------
!  PURPOSE: make a uniform grid st. x(1)=x1 & x(n)=xn
!-------------------------------------------------------------------------------
 
      dx   = (xn-x1)/float(n-1)
      x(1) = x1
      do i=2,n-1
        x(i) = x1 + float(i-1)*dx
      end do
      x(n) = xn
 
      return
      end
 
!===============================================================================
