!===============================================================================
! CVS $Id: initial.F90,v 1.5 2001/07/06 20:37:44 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/atm/datm5/initial.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE initial()

      use shr_sys_mod   ! shared system routines

      implicit none

      !----- global -----
#include <dims.h>
#include <data_msg.h>
#include <control.h>

      NAMELIST / inparm /   &
     &  rest_type   ,       &
     &  rest_pfile  ,       &
     &  rest_bfile  ,       &
     &  rest_freq   ,       &
     &  rest_date   ,       &
     &  rest_nday   ,       &
     &  rest_odate  ,       &
     &  case_name   ,       &
     &  case_desc   ,       &
     &  data_dir    ,       &
     &  data_rmlf   ,       &
     &  data_year0  ,       &
     &  data_nyear  ,       &
     &  data_oyear  ,       &
     &  flux_albic  ,       &
     &  flux_albfb  ,       &
     &  flux_swfact ,       &
     &  flux_rain2snow ,    &
     &  mss_dir     ,       &
     &  mss_pass    ,       &
     &  mss_opts    ,       &
     &  mss_rtpd    ,       &
     &  mss_rmlf    ,       &
     &  info_dbug   ,       &
     &  info_time   ,       &
     &  info_sleep  ,       &
     &  ncpl        

      !----- local -----
      integer      n         ! generic loop index
      integer    rcode       ! return code

      !----- constants -----
      integer,parameter :: true =1
      integer,parameter :: false=0
      integer,parameter :: unset=-999999

      !----- formats -----
      character(len=*),parameter :: F00 = "('(initial) ',2a)"
      character(len=*),parameter :: F01 = "('(initial) ',a,i7)"

!-------------------------------------------------------------------------------
! PURPOSE:
!  o set default control variable values
!  o read ice input parameter namelist file
!  o read ice coordinate grids & domain mask
!  o read sst climatology
!  o convert sst climatology from celsius to kelvin
!-------------------------------------------------------------------------------

      !-----------------------------------------------
      ! set initial namelist values
      !-----------------------------------------------
      write(6,'(a)') '(initial) setting namelist defaults.'
      rest_type   = 'continue'  ! run type: initial, cont, regen, branch
      rest_pfile  = 'null'      ! IC data pointer file
      rest_bfile  = 'null'      ! IC data file for branch runs
      rest_freq   = 'never'     ! restart file frequency
      rest_date   = 10101       ! start date for initial or branch runs
      rest_nday   = 365         ! restart file interval    (nday option
      rest_odate  = 0101        ! restart file offset date (nday option)
      case_name   = ' '         ! case name
      case_desc   = ' '         ! case description
      do n=1,3                  ! 1=>state, 2=>radiation, 3=>precip
        data_dir  (n) = 'null'  ! data: directory
        data_year0(n) = 1       ! data: 1st  year  in sequence
        data_nyear(n) = 1       ! data: # of years in sequence
        data_oyear(n) = 1       ! data: cpl year matched to year0
        data_rmlf (n) = false   ! data: remove file after reading
      end do
      flux_albic  = unset       ! T => do optional albedo initialization
      flux_albfb  = false       ! T => do optional albedo feedback
      flux_swfact = 1.0         ! Solar forcing factor
      flux_rain2snow = .false.  ! T => convert rain to snow where T < 0
      mss_dir     = 'null'      ! MSS file directory
      mss_pass    = ' '         ! MSS file password
      mss_opts = 'nowait,nomail'! MSS file mswrite options
      mss_rtpd    = 180         ! MSS file MS retention period
      mss_rmlf    = true        ! T => remove local file after mswrite
      info_dbug   = 1           ! dbug level: 0=lowest, 3=highest
      info_time   = false       ! T => print extra timing info
      info_sleep  = 0.0         ! simulate time to advance an active model
      ncpl        = 24          ! number of msg pairs per day

      !-----------------------------------------------
      ! read input namelist
      !-----------------------------------------------
      write(6,F00) 'reading input namelist.'
      read (5,inparm,iostat=rcode)
      if (rcode > 0) then
         write(6,F01) 'ERROR: reading input namelist, iostat=',rcode
	 call shr_sys_abort("datm namelist read error")
      end if

      !-----------------------------------------------
      ! re-set some default namelist values
      !-----------------------------------------------
      do n=2,3
        if (data_dir(n)(1:4) .eq. 'null') then
          data_year0(n) = data_year0(1)
          data_nyear(n) = data_nyear(1)
          data_oyear(n) = data_oyear(1)
          data_rmlf (n) = data_rmlf (1)
        end if
      end do
      if ( flux_albfb .ne. 0) flux_albic = true
      if ( flux_albfb .eq. 0) flux_albic = false
      if ( rest_type == 'continue') flux_albic = false

      !-----------------------------------------------
      ! record initial namelist values
      !-----------------------------------------------
      write(6,'(a)') '(initial) namelist values:'
      write(6,inparm)
      write(6,*) 

      !-----------------------------------------------
      ! inititial values for control flags
      !-----------------------------------------------
      ierr     = 0  ! error code
      stop_now = 0  ! T => stop now
      stop_eod = 0  ! T => stop at end-of-day
      rest_now = 0  ! T => create restart file now
      rest_eod = 0  ! T => create restart file at end-of-day
      hist_now = 0  ! T => create history file now
      hist_eod = 0  ! T => create history file at end-of-day
      hist_tav = 0  ! T => create monthly avg data
      diag_now = 0  ! T => create diagnostic data now
      diag_eod = 0  ! T => create diagnostic data at end-of-day

      RETURN
      END

!===============================================================================
