!===============================================================================
! CVS $Id: msg.F90,v 1.5 2001/08/01 22:56:45 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/atm/datm5/msg.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE msg_pass(cOption)

      use shr_msg_mod   ! shared code: msg passing
      use shr_sys_mod   ! shared code: system calls
      use shr_timer_mod ! shared code: timing routines

      implicit none

      !----- arguments -----
      character*(*) cOption

      !----- global -----
#include <dims.h>               ! model resolution
#include <data_msg.h>           ! data sent/received from coupler
#include <control.h>            ! control info, date/time

      !----- local -----
      integer,parameter :: nsnd = 19   ! number of 2D fields to send
      integer,parameter :: nrcv = 15   ! number of 2D fields to receive
      real        :: buffs(nx,ny,nsnd) ! contiguous array for sending data
      real        :: buffr(nx,ny,nrcv) ! contiguous array for recving data
      integer,save:: timer_n           ! timer number
      integer,save:: timer_m           ! timer number
      integer     :: i,j               ! generic loop indicies
      integer*8   :: irtc_w            ! rtc tics when waiting for msg
      integer*8   :: irtc_r            ! rtc tics when msg received
      integer*8   :: irtc_u            ! rtc tics when msg unpacked
      integer*8   :: irtc_p            ! rtc tics when packing msg
      integer*8   :: irtc_s            ! rtc tics when sending msg 

!-------------------------------------------------------------------------------
! PURPOSE:
!   Implement various message passing tasks
!
! FUCTIONALITY:
!   cOption=
!     'connect'    => connect to the msg passing system
!     'init'       => recv/send initial info
!     'send'       => send model state & output fluxes
!     'recv'       => receive input fluxes & stop/continue flags
!     'disconnect' => disconnect from the msg passing system
!-------------------------------------------------------------------------------

      IF (cOption .eq. 'connect') THEN 
        !-----------------------------------------------------------------------
        ! connect to msg passing system
        !-----------------------------------------------------------------------
        write(6,9000) '(connect) connecting to cpl'
        call shr_msg_init('atm')
        call shr_msg_groups('atm')
     
        call shr_timer_get(timer_n,'msg: atm  -> cpl')
        call shr_timer_get(timer_m,'msg: atm <-  cpl')

      ELSE IF (cOption .eq. 'init') THEN 
        !-----------------------------------------------------------------------
        ! recv/send initial data
        !-----------------------------------------------------------------------
        write(6,9000) '(init) recv initial msg'

        call shr_msg_recv_i(ibuffr, nibuff  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2AI)
        call shr_msg_recv_r(rbuffr, nibuff  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2AI)

        ierr      = ibuffr( 1) ! error code
        cdate     = ibuffr( 4) ! current date
        sec       = ibuffr( 5) ! elapsed sec on current date
        info_time = ibuffr(11) ! T => turn on msg-passing timing
        !nfo_dbug = ibuffr(12) ! cpl debug level
 
        !--- send grid & domain mask ---
        write(6,9000) '(init) send initial msg'

        do i=1,nibuff
          ibuffs(i)=0          ! unused
        end do
        ibuffs( 1)=ierr        ! error code
        ibuffs( 7)=nx          ! number of grid points in x-direction
        ibuffs( 8)=ny          ! number of grid points in y-direction
        ibuffs( 9)=ncpl        ! number of msg-pairs per day
        ibuffs(34)=flux_albic  ! request extra albedo/solar init msg
        
        call shr_msg_send_i(ibuffs, nibuff  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_A2CI)
        call shr_msg_send_r(xc    , nx*ny   ,SHR_MSG_TID_CPL,SHR_MSG_TAG_A2CI)
        call shr_msg_send_r(yc    , nx*ny   ,SHR_MSG_TID_CPL,SHR_MSG_TAG_A2CI)
        call shr_msg_send_r(xv    , nx*ny*4 ,SHR_MSG_TID_CPL,SHR_MSG_TAG_A2CI)
        call shr_msg_send_r(yv    , nx*ny*4 ,SHR_MSG_TID_CPL,SHR_MSG_TAG_A2CI)
        call shr_msg_send_r(area  , nx*ny   ,SHR_MSG_TID_CPL,SHR_MSG_TAG_A2CI)
        call shr_msg_send_i(mask  , nx*ny   ,SHR_MSG_TID_CPL,SHR_MSG_TAG_A2CI)
        
      ELSE IF (cOption .eq. 'send') THEN 
        !-----------------------------------------------------------------------
        ! send output field msg
        !-----------------------------------------------------------------------
        if (info_dbug.ge.3) write(6,9000) '(send) send data to cpl'
        call shr_timer_start(timer_n)

        !--- set info buffer flags ---
        irtc_p = shr_sys_irtc()   ! packing msg
        do i=1,nibuff
          ibuffs(i)= 0         ! unused
        end do
        ibuffs( 1)=ierr        ! error code
        ibuffs( 3)=0           ! stop flag: 0 <=> able to continue
        ibuffs( 4)=cdate       ! model date, coded: yyyymmdd
        ibuffs( 5)=sec         ! elapsed seconds on model date
        ibuffs( 7)=nx          ! number of grid points in x-direction
        ibuffs( 8)=ny          ! number of grid points in y-direction
        ibuffs( 9)=ncpl        ! number of msg-pairs per day
        ibuffs(32)=0           ! albedo time shift in seconds
        ibuffs(34)=flux_albic  ! request extra albedo/solar init msg

        !--- pack & send msg buffer ---
        do j=1,ny
        do i=1,nx
          !--- atm states ---
          buffs(i,j, 1) = z    (i,j) ! layer height
          buffs(i,j, 2) = u    (i,j) ! velocity - zonal
          buffs(i,j, 3) = v    (i,j) ! velocity - merid
          buffs(i,j, 4) = t    (i,j) ! temperature
          buffs(i,j, 5) = ptem (i,j) ! potential temperature
          buffs(i,j, 6) = pres (i,j) ! pressure, bottom layer
          buffs(i,j, 7) = pslv (i,j) ! pressure, sea level
          buffs(i,j, 8) = shum (i,j) ! specific humidity
          buffs(i,j, 9) = dens (i,j) ! densisity
          !--- atm output fluxes ---
          buffs(i,j,10) = swnet(i,j) ! sw net 
          buffs(i,j,11) = lwdn (i,j) ! lw downward
          buffs(i,j,12) = precc(i,j) ! precip, liquid, convective
          buffs(i,j,13) = precl(i,j) ! precip, liquid, large scale
          buffs(i,j,14) = snowc(i,j) ! precip, frozen, convective
          buffs(i,j,15) = snowl(i,j) ! precip, frozen, large scale
          buffs(i,j,16) = swndr(i,j) ! sw down, near-infra, direct
          buffs(i,j,17) = swvdr(i,j) ! sw down, visible     direct
          buffs(i,j,18) = swndf(i,j) ! sw down, near-infra, diffuse
          buffs(i,j,19) = swvdf(i,j) ! sw down, visible     duffuse
        end do
        end do

        irtc_s = shr_sys_irtc()   ! sending msg
        call shr_msg_send_i(ibuffs,nibuff    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_A2C)
        call shr_msg_send_r( buffs,nx*ny*nsnd,SHR_MSG_TID_CPL,SHR_MSG_TAG_A2C)
        call shr_timer_stop(timer_n)

        if (info_time.ne.0) then
          write(6,9099) irtc_p,'a->c packing'
          write(6,9099) irtc_s,'a->c sending'
        endif

      ELSE IF (cOption .eq. 'recv') THEN
        !-----------------------------------------------------------------------
        ! recv input field msg
        !-----------------------------------------------------------------------
        if (info_dbug.ge.3) write(6,9000) '(recv) recv data to cpl'
        call shr_timer_start(timer_m)
        
        !--- wait for message ---
        irtc_w = shr_sys_irtc()   ! waiting to recv msg
        call shr_msg_recv_i(ibuffr,nibuff    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2A)
        irtc_r = shr_sys_irtc()   ! received msg
        call shr_msg_recv_r( buffr,nx*ny*nrcv,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2A)
        
        do j=1,ny
        do i=1,nx
           !--- atm/srf fluxes ---
           taux  (i,j) = buffr(i,j, 1) ! momentum flux - zonal
           tauy  (i,j) = buffr(i,j, 2) ! momentum flux - meridional
           lat   (i,j) = buffr(i,j, 3) ! heat flux - latent
           sen   (i,j) = buffr(i,j, 4) ! heat flux - sensible
           lwup  (i,j) = buffr(i,j, 5) ! heat flux - longwave, upward
           evap  (i,j) = buffr(i,j, 6) ! water flux - evaporation
           !--- merged srf states ---
           alvdr (i,j) = buffr(i,j, 7) ! albedo, visible   , direct
           alndr (i,j) = buffr(i,j, 8) ! albedo, near-infra, direct
           alvdf (i,j) = buffr(i,j, 9) ! albedo, visible   , diffuse
           alndf (i,j) = buffr(i,j,10) ! albedo, near-infra, diffuse
           st    (i,j) = buffr(i,j,11) ! surface temperature
           snowh (i,j) = buffr(i,j,12) ! snow depth
           fi    (i,j) = buffr(i,j,13) ! ice fraction
           fo    (i,j) = buffr(i,j,14) ! ocn fraction
           fl    (i,j) = max(1.0 - fi(i,j) - fo(i,j),0.0) ! lnd frac
           !--- diagnostic data ---
           tref  (i,j) = buffr(i,j,15) ! 2m reference T
        end do
        end do

        call shr_timer_stop(timer_m)
        irtc_u = shr_sys_irtc()   ! unpacked msg
 
        ierr      = ibuffr( 1) ! cpl error code (0 => no error)
        stop_eod  = ibuffr( 2) ! T => stop at end-of-day
        stop_now  = ibuffr( 3) ! T => stop now
        rest_eod  = ibuffr(21) ! T => create restart data at end-of-day
        rest_now  = ibuffr(22) ! T => create restart data now
        hist_eod  = ibuffr(23) ! T => create history data at end-of-day
        hist_now  = ibuffr(24) ! T => create history data now
        hist_tav  = ibuffr(25) ! T => create create monthly avg data
        diag_eod  = ibuffr(26) ! T => create diagnotic data at eod
        diag_now  = ibuffr(27) ! T => create diagnotic data now

        if (info_time.ne.0) then
          write(6,9099) irtc_w,'c->a waiting'
          write(6,9099) irtc_r,'c->a received'
          write(6,9099) irtc_u,'c->a unpacked'
        endif

      ELSE IF (cOption .eq. 'disconnect') THEN 
        !-----------------------------------------------------------------------
        ! disconnect from msg passing
        !-----------------------------------------------------------------------
        write(6,9000) '(disconnect) disconnect from cpl'
        call shr_msg_finalize()

      ELSE 
        !-----------------------------------------------------------------------
        ! this should never happen
        !-----------------------------------------------------------------------
        write(6,*) 'invalid cOption: ',cOption
        ierr=1
      END IF

      RETURN
 9000 format('(msg_pass) ',a)
 9099 format('[mp timing]  irtc = ',i20,' ',a)
      END

!=======-=======================================================================
