!===============================================================================
! CVS $Id: advance.F90,v 1.6 2002/04/30 17:44:41 tcraig Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/atm/latm5/advance.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE advance()

      use calendar_mod  ! calendar routines
      use const_mod     ! shared physical constants

      implicit none

      !----- global -----
#include <dims.h>      ! model resolution
#include <data_raw.h>  ! data read from data file
#include <data_msg.h>  ! data sent/received from coupler
#include <control.h>   ! control info, date/time

      !----- local  ------
      integer i,j,k,ivar,iftype   ! loop indices
      integer iprtinfo,inew       ! input flags for subroutines
      integer k1,k2,kold,knew     ! raw data time-slice indices

      real f1,f2               ! raw data fraction
      real a_edays             ! elapsed atmosphere days 
                               ! (incl. fraction of day)

      real,parameter :: em=1.0             ! emissivity factor
      real,save      :: shummin = 1.E-8    ! minimum specific humidity (kg/kg)
      logical,save   :: firstcall = .true. ! flags 1-time initialization

!-------------------------------------------------------------------------------
! PURPOSE:
!   advance the atmosphere state to the current date
!   
! METHOD:
!   o ensure that "older" and "newer" data time slices bracket 
!     atmospheric model time
!   o interpolate data input fields to model time
!   o derive (if necessary) additional fields
!   o set all fields required by coupler
!
! REMARKS:
!   o there are a lot of "magic numbers" here, this should be addressed -BK
!-------------------------------------------------------------------------------

      IF (.not.NOREADDAT) THEN

      !-----------------------------------------------------------------
      ! elapsed model days, incl. fraction of day
      !-----------------------------------------------------------------

      a_edays = real(eday) + real(sec)/86400.

      !--------------------------------------------------------------
      ! read initial "older" data slices
      ! (as close as possible before atmosphere model time) 
      ! and set "newer" = "older" data
      !--------------------------------------------------------------

      if (firstcall) then

         do iftype=1,maxftype

            r_kold(iftype) = 1
            r_knew(iftype) = 2
            kold = r_kold(iftype)
            knew = r_knew(iftype)

            ! determine atmosphere year of left bracket file (might be
            ! different for each forcing type); note: at the very 
            ! beginning of a model year, data from the previous year  
            ! need to be read to allow correct interpolation 

            af_year(iftype) = year         
            if (month.eq.1 .and. day.eq.1 .and. sec.eq.0) &
            &      af_year(iftype) = year - 1  
                                      
            ! determine correct year in data file sequence

 111        f_year(iftype) = data_year0(iftype) + &
            &                modulo( af_year(iftype)-data_oyear(iftype), &
            &                        data_nyear(iftype) )

            ! initialize data files:
            ! print complete netcdf header information on this call

            iprtinfo = 1 
            do ivar=1,nvar(iftype)
               call initfdat(data_dir(iftype),ivar,iftype,iprtinfo)
            enddo

            ! read "older" data slices
            ! (and initialize "newer" data as "newer"="older")

            inew = 0
            call readfdat(a_edays,iftype,inew)
            
            ! is current date too close to beginning of year,
            ! so that an older year needs to be opened instead?

            if (d_edays(iftype,kold).gt.a_edays) then
               call closefdat(iftype,nvar(iftype),data_rmlf(iftype))
               af_year(iftype) = af_year(iftype) - 1
               goto 111
            endif

         enddo
      endif  ! firstcall

      !--------------------------------------------------------------
      ! are "newer" data slices up-to-date ?
      !--------------------------------------------------------------

      do iftype=1,maxftype

         if (d_edays(iftype,r_knew(iftype)).lt.a_edays) then

            r_kold(iftype) = r_knew(iftype)
            r_knew(iftype) = 3 - r_kold(iftype)

            kold = r_kold(iftype)
            knew = r_knew(iftype)
            
            irecord(iftype) = irecord(iftype) + 1


            !-----------------------------------------------------------
            ! read next "newer" data 
            !-----------------------------------------------------------

 222        if (irecord(iftype).le.nrecord(iftype)) then

               !--------------------------------------------------------
               ! read next record in current file 
               ! (determine new "irecord" and "d_edays")
               !--------------------------------------------------------

               inew = 1
               call readfdat(a_edays,iftype,inew)
               if (d_edays(iftype,knew).lt.a_edays) then
                  irecord(iftype) = irecord(iftype) + 1
                  goto 222
               endif
   
            else

               !--------------------------------------------------------
               ! reached end-of-file in current file
               !--------------------------------------------------------

               af_year(iftype) = af_year(iftype) + 1

               irecord(iftype) = 1

               ! if more than one year in data sequence: 
               ! - close current files, and
               ! - initialize next data files in sequence.
               ! otherwise: simply go back to beginning of file

               if (data_nyear(iftype).gt.1) then

                  f_year(iftype) = f_year(iftype) + 1
                  if ( f_year(iftype) .gt.  &
                  &          data_year0(iftype)+data_nyear(iftype)-1 ) &
                  &    f_year(iftype) = data_year0(iftype)

                  call closefdat(iftype,nvar(iftype),data_rmlf(iftype))
                 
                  iprtinfo = 0
                  do ivar=1,nvar(iftype)
                    call initfdat(data_dir(iftype),ivar,iftype,iprtinfo)
                  enddo
          

               endif       ! several years in data sequence
               goto 222
            endif          ! reached end-of-file

         endif             ! need to read next record: d_edays < a_edays
      enddo                ! all maxftype categories


      !-----------------------------------------------------------------
      ! interpolate raw data
      !-----------------------------------------------------------------

      do iftype=1,maxftype

         k1 = r_kold(iftype)
         k2 = r_knew(iftype)

         f1 = ( d_edays(iftype,k2)-a_edays            ) /  &
         &    ( d_edays(iftype,k2)-d_edays(iftype,k1) )
         f2 = 1.0 - f1

         do ivar=1,nvar(iftype)
            do j=1,ny
            do i=1,nx
               if (iftype.eq.1) r_states(i,j,ivar,3) = &
               &   f1*r_states(i,j,ivar,k1)  +f2*r_states(i,j,ivar,k2)
               if (iftype.eq.2) r_precip(i,j,ivar,3) =  &
               &   f1*r_precip(i,j,ivar,k1)  +f2*r_precip(i,j,ivar,k2)
               if (iftype.eq.3) r_radflx(i,j,ivar,3) = &
               &   f1*r_radflx(i,j,ivar,k1)  +f2*r_radflx(i,j,ivar,k2)
            enddo
            enddo
         enddo

      enddo ! all maxftype categories

      !-----------------------------------------------------------------
      ! put raw data into data for coupler
      !-----------------------------------------------------------------

      do j=1,ny
      do i=1,nx
         if (NCEPSTATDAT) then
            u    (i,j) = r_states(i,j,k_u    ,3)
            v    (i,j) = r_states(i,j,k_v    ,3)
            ptem (i,j) = r_states(i,j,k_t    ,3)
            shum (i,j) = r_states(i,j,k_shum ,3) 
            dens (i,j) = r_states(i,j,k_dens ,3)
            t    (i,j) = ptem(i,j)
            pres (i,j) = 1.0e30
            pslv (i,j) = pres(i,j)
         endif
         if (CCMSTATDAT) then
            u    (i,j) = r_states(i,j,k_u    ,3)
            v    (i,j) = r_states(i,j,k_v    ,3)
            ptem (i,j) = r_states(i,j,k_ptem ,3)
            shum (i,j) = r_states(i,j,k_shum ,3)
            dens (i,j) = r_states(i,j,k_dens ,3)
            t    (i,j) = r_states(i,j,k_t    ,3)
            pres (i,j) = r_states(i,j,k_pres ,3)
            pslv (i,j) = pres(i,j)
         endif           
         !-------------------------------------------
         if (CCMPRECDAT) then
            precc(i,j) = r_precip(i,j,k_precc,3)
            precl(i,j) = r_precip(i,j,k_precl,3)
            snowc(i,j) = 0.0
            snowl(i,j) = 0.0
         else if (NCEPPRECDAT) then
            precc(i,j) = 0.0
            precl(i,j) = r_precip(i,j,k_precl,3)
            snowc(i,j) = 0.0
            snowl(i,j) = 0.0
         endif
         if (t(i,j) < Tfrz) then
            snowc(i,j) = precc(i,j)
            snowl(i,j) = precl(i,j)
            precl(i,j) = 0.0
            precc(i,j) = 0.0
         endif
         !-------------------------------------------
         if (NCEPRADFDAT) then
             lwdn (i,j) = r_radflx(i,j,k_lwdn ,3)
             swnet(i,j) = r_radflx(i,j,k_swdn ,3) - r_radflx(i,j,k_swup ,3)
             swnet(i,j) = swnet(i,j) 
             swvdr(i,j) = r_radflx(i,j,k_swdn ,3)*(.28)
             swndr(i,j) = r_radflx(i,j,k_swdn ,3)*(.31)
             swvdf(i,j) = r_radflx(i,j,k_swdn ,3)*(.24)
             swndf(i,j) = r_radflx(i,j,k_swdn ,3)*(.17)
         else if (CCMRADFDAT) then
              lwdn (i,j) = r_radflx(i,j,k_lwdn ,3)
              swnet(i,j) = r_radflx(i,j,k_swnet,3)
              swvdr(i,j) = r_radflx(i,j,k_swvdr,3)
              swndr(i,j) = r_radflx(i,j,k_swndr,3)
              swvdf(i,j) = r_radflx(i,j,k_swvdf,3)
              swndf(i,j) = r_radflx(i,j,k_swndf,3)
              if (flux_albfb .ne. false) then
                 if (firstcall) then
                    swnet(i,j) = MissVal
                 else
                    swnet(i,j) = (1.-alvdr(i,j)) * swvdr(i,j) &
                    &           +(1.-alndr(i,j)) * swndr(i,j) &
                    &           +(1.-alvdf(i,j)) * swvdf(i,j) &
                    &           +(1.-alndf(i,j)) * swndf(i,j)
                 endif
              endif
         endif
      enddo
      enddo

      ENDIF  ! not NOREADDAT


      firstcall = .false.

      RETURN
 9000 format('(advance) ',a,a)
 9001 format('(advance) ',a,i8.8)
      END

!===============================================================================
