!===============================================================================
! CVS $Id: data_nc.F90,v 1.3 2002/01/30 23:59:08 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/atm/latm5/data_nc.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE readfdat(a_edays,iftype,inew)

      use shr_sys_mod   ! shared code: system calls
      use calendar_mod  ! calendar routines
      
      implicit none

      !----- global -----
#include <netcdf.inc>  ! netCDF defs
#include <dims.h>      ! model resolution
#include <data_raw.h>  ! data read from data file
#include <data_nc.h>   ! netcdf file info
      
      !----- input  ------
      real    a_edays  ! atmosphere elapsed days including fraction of day
      integer iftype   ! forcing type categorry
      integer inew     ! flag to read newer  or older (=0) data

      !----- local ------
      integer r_cdateall(maxrec)  ! all dates from data file
      integer r_secall(maxrec)    ! all seconds in dates
      integer r_cdate
      integer r_year
      integer r_month
      integer r_day
      integer r_sec
      integer d_year
      integer d_eday
      integer i,j,ivar            ! loop indices
      integer kold,knew,kindex
      integer ircode
      integer istart(3),icount(3)
      integer iprtinfo            ! flag to print netcdf info (1=true)
      real    dat_edays           ! local d_edays

!-------------------------------------------------------------------------------
! PURPOSE:
!    read data time slices
!    o inew=0 : read "older" data as close as possible before
!               atmosphere date (set irecord(iftype)), and
!               initialize "newer" data  by setting "newer" = "older"
!    o inew=1 : read "newer" data (with irecord(iftype) set 
!               in "advance.f".
!    o read at irecord(iftype) from input files:
!               (inew=0 : kindex=old, inew=1 : kindex=new)
!               r_states(1:nx,1:ny,1:nvar(iftype),kindex),
!               r_precip(1:nx,1:ny,1:nvar(iftype),kindex),
!               r_radflx(1:nx,1:ny,1:nvar(iftype),kindex), and compute
!               d_edays(iftype,kindex)
!
!-------------------------------------------------------------------------------

      kold = r_kold(iftype)
      knew = r_knew(iftype)

      !-------------------------------------------------------------------------
      ! find correct record number for "older" data slices
      ! from currently open files 
      !-------------------------------------------------------------------------

      if (inew.eq.0) then

         ! get all dates from first file of forcing category
         
         istart(1) = 1
         icount(1) = nrecord(iftype)
         ircode = nf_get_vara_int(nc_file_id(1,iftype), &
         &        nc_date_id(1,iftype),istart,icount,r_cdateall)
         if (ircode.ne.0) then
            write(6,9000) 'readfdat', 'int', ' "older" date' 
            write(6,*) '      nc_file_ID=',nc_file_id(1,iftype)
            write(6,*) '      icount(1) =',icount(1)
            write(6,*) '      ircode=',ircode
            call shr_sys_flush(6)
         endif

         ircode = nf_get_vara_int(nc_file_id(1,iftype), &
         &        nc_datesec_id(1,iftype),istart,icount,r_secall)
         if (ircode.ne.0) then
            write(6,9000) 'readfdat', 'int', ' "older" datesec'
            write(6,*) '      nc_file_ID=',nc_file_id(1,iftype)
            write(6,*) '      icount(1) =',icount(1)
            write(6,*) '      ircode=',ircode
            call shr_sys_flush(6)
         endif

         ! find correct record number in file to read data as "older"
         ! data slice just before atm date

         irecord(iftype) = 0
         do i=1,nrecord(iftype)

            ! determine date of data in atmosphere time frame
            ! for interpolation

            call date2ymd(r_cdateall(i),r_year,r_month,r_day)
            d_year = r_year - f_year(iftype) + af_year(iftype)

            call ymd2eday(d_year,r_month,r_day,d_eday)
            dat_edays = real(d_eday) + real(r_secall(i))/86400.

            if (dat_edays .lt. a_edays) irecord(iftype) = i
            if (dat_edays .ge. a_edays) goto 999

         enddo
 999     continue

         ! cannot find correct date: need to get data 
         ! from previous year ?

         if (irecord(iftype).eq.0) then
            d_edays(iftype,kold) = dat_edays
            ! new data files will be selected in "advance.f"
            return
         endif

      endif ! inew=0

      !-------------------------------------------------------------------------
      ! read data slices at irecord(iftype)
      !-------------------------------------------------------------------------

      kindex = knew
      if (inew.ne.1) kindex= kold

      istart(1) = 1               ! starting longitude
      istart(2) = 1               ! starting latitude
      istart(3) = irecord(iftype) ! starting record
      icount(1) = nx
      icount(2) = ny
      icount(3) = 1
         
      do ivar=1,nvar(iftype)
         if (iftype.eq.1) then
            ircode = nf_get_vara_double(nc_file_id(ivar,iftype), &
            &        nc_var_id(ivar,iftype), &
            &        istart,icount,r_states(1,1,ivar,kindex) )
         else if (iftype.eq.2) then
            ircode = nf_get_vara_double(nc_file_id(ivar,iftype), &
            &        nc_var_id(ivar,iftype), &
            &        istart,icount,r_precip(1,1,ivar,kindex) )
         else if (iftype.eq.3) then
            ircode = nf_get_vara_double(nc_file_id(ivar,iftype), &
            &        nc_var_id(ivar,iftype), &
            &        istart,icount,r_radflx(1,1,ivar,kindex) )
         endif
         if (ircode.ne.0) then
            write(6,9000) 'readfdat', 'double', '  "newer" date'
            write(6,*) '      iftype=',iftype,' ivar=',ivar
            write(6,*) '      nc_file_ID=',nc_file_id(ivar,iftype)
            write(6,*) '      istart=',istart
            write(6,*) '      icount=',icount
            write(6,*) '      ircode=',ircode
            call shr_sys_flush(6)
         endif
      enddo

      ! read date of this record

      istart(1) = irecord(iftype)
      icount(1) = 1
      ircode = nf_get_vara_int (nc_file_id(1,iftype), &
      &         nc_date_id(1,iftype),istart,icount, r_cdate)
      if (ircode.ne.0) then
         write(6,9000) 'readfdat', 'int', '  to get record date'
         write(6,*) '      nc_file_id=',nc_file_id(1,iftype)
         write(6,*) '      istart = ',istart(1),' ircode=',ircode
         call shr_sys_flush(6)
      endif
      ircode = nf_get_vara_int (nc_file_id(1,iftype), &
      &         nc_datesec_id(1,iftype),istart,icount,r_sec)
      if (ircode.ne.0) then
         write(6,9000) 'readfdat', 'int', '  to get record date'
         write(6,*) '      nc_file_id=',nc_file_id(1,iftype)
         write(6,*) '      istart = ',istart(1),' ircode=',ircode
         call shr_sys_flush(6)
      endif

      call date2ymd(r_cdate,r_year,r_month,r_day)
      d_year = r_year - f_year(iftype) + af_year(iftype)

      call ymd2eday(d_year,r_month,r_day,d_eday)
      d_edays(iftype,kindex) = real(d_eday) + real(r_sec)/86400.

      !----------------------------------------------------------------------
      ! initialize "newer" data
      !----------------------------------------------------------------------

      if (inew.eq.0) then

         d_edays(iftype,knew) = d_edays(iftype,kold)

         do ivar=1,nvar(iftype)
            do j=1,ny
            do i=1,nx
              if (iftype.eq.1) r_states(i,j,ivar,knew) = r_states(i,j,ivar,kold)
              if (iftype.eq.2) r_precip(i,j,ivar,knew) = r_precip(i,j,ivar,kold)
              if (iftype.eq.3) r_radflx(i,j,ivar,knew) = r_radflx(i,j,ivar,kold)
            enddo
            enddo
         enddo

      endif ! inew=0

      return
9000     format (1x, 'ERROR in ', a, 'from nf_get_vara_',a,a)
      end

!===============================================================================

      SUBROUTINE initfdat(datadir,ivar,iftype,iprtinfo)

      use shr_sys_mod   ! shared code: system calls
      use shr_file_mod  ! shared code: file archive

      implicit none

      !----- global -----
#include <netcdf.inc>  ! netCDF defs
#include <dims.h>      ! model resolution
#include <data_raw.h>  ! data read from data file
#include <data_nc.h>   ! netcdf file info
      
      !----- input  ------
      character*64 datadir  ! masstore data directory for input files
      integer      ivar     ! index for variable in this category
      integer      iftype   ! forcing type category
      integer      iprtinfo ! print netcdf header information (1=true)

      !----- local ------
      integer i,j          ! generic loop indices
      integer ier,ircode   ! error return code from msread and netcdf calls
      integer found3dvar   ! check that there is only 1 3D variable in data file
      integer icdate1,icdate2       ! coded date: yyyymmdd
      integer icdatesec1,icdatesec2 ! seconds in date
      integer ifound3dvar           ! flag to identify 3D data

      character*64 mfname  ! masstore input file name

      !----- local netcdf ------
      integer maxvds

      parameter( maxvds = 3)  ! maximum variable dimensions (lon,lat,time)

! nf_open                ! OPEN NETCDF FILE FOR ACCESS
      integer   ncid     !  returned netcdf file ID
! nf_inq                 ! INQUIRE ABOUT A NETCDF FILE
      integer  ndims     !  returned number of dimensions in netcdf file
      integer  nvars     !  returned number of variables  .. ..  .. ..
      integer  ngatts    !  returned number of global attributes .. ..
      integer  irecdimid !  returned ID of the unlimited (or record) dimension
! nf_inq_dim             ! INQUIRE ABOUT A DIMENSION
      integer  idimid    !  dimension ID (1 to ndims; or returned from
                         !  previous call to ncdid)
      integer  idimsize  !  returned size of dimensions: either lon,lat, or time
! nf_inq_var             ! INQUIRE ABOUT A VARIABLE
      integer  ivarid    !  variable ID (1 to nvars; or returned from
                         !  previous call to ncvid)
      integer  ivartype  !  returned variable type: ncbyte=1, ncchar=2,
                         !  ncshort=3, nclong=4, ncfloat=5, ncdouble=6
      integer  nvdims    !  returned number of dimensions for the 
                         !  variable: e.g. lon=1, .., data=3
      integer  ivdimids(maxvds) ! returned vector of nvdims dimension IDs 
                         !  corresponding to the variable dimensions
      integer  nvatts    !  returned number of variable attributes 
                         !  assigned to this variable
! nf_inq_atttname        ! INQUIRE ABOUT AN ATTRIBUTE NAME
      integer  ivatt     !  number of the variable attribute(1 to nvatts)
      integer  igatt     !  number of the global   attribute(1 to ngatts)
! nf_inq_att             ! INQUIRE ABOUT AN ATTRIBUTE
      integer  iattype   !  returned attribute type: ncbyte=1,ncchar=2,..
      integer  nattlen   !  returned number of values stored in the 
                         !  attribute (for a string-valued attribute,this
                         !  is the number of characters in the string)
! nf_get_vara_double     ! READ HYPERSLAB OF VALUES FROM A NETCDF FILE  
      integer  istart(3) !  starting index for record to be read
      integer  icount(3) !  edge lengths of record to be read 

      character*30   dimnam    ! returned dimension name    (nf_inq_dim)
      character*30   varnam    ! returned variable name     (nf_inq_var)
      character*30   attnam    ! returned attribute name    (nf_inq_attname)
      character*1024 attstring ! returned attribute string  (nf_get_att_text)

!-------------------------------------------------------------------------------
! PURPOSE:
!    initialize netcdf input file and determine:
!      nc_file_id   (ivar,iftype), 
!      nc_var_id    (ivar,iftype),
!      nc_date_id   (ivar,iftype), 
!      nc_datesec_id(ivar,iftype),
!      nc_lon_id    (ivar,iftype), 
!      nc_lat_id    (ivar,iftype), and
!      nrecord(iftype).
!
! METHOD:
!  o copy input data file from masstore
!  o initialize netcdf file: get file, date, datesec, and variable IDs 
!    (data variable is assumed to be the only 3D variable in data file)
!  o check dimensions: nx and ny; and get number of records in files
!  o check date in file
!  o print netcdf header (if "iprtinfo"=1=true)
!
!-------------------------------------------------------------------------------

      !-------------------------------------------------------------------------
      ! get netcdf data file from masstore
      !-------------------------------------------------------------------------

      i = index(r_fname(ivar,iftype),'.nc') - 5
      write(r_fname(ivar,iftype),'(a,i4.4,a)')  &
      &      r_fname(ivar,iftype)(1:i) , f_year(iftype) , '.nc'

      mfname = datadir
      i = index(mfname,' ') - 1
      write(mfname,'(a,"/",a)') mfname(1:i),r_fname(ivar,iftype)

!!!   call msread(ier, r_fname(ivar,iftype),mfname,' ',' ')   
      call shr_file_get(ier,r_fname(ivar,iftype),mfname)

      if (ier.ne.0 .and. ier.ne.3) then
         write(6,*) 'ERROR in initfdat from shr_file_get'
         write(6,*) '      local file  =',r_fname(ivar,iftype)
         write(6,*) '      mss   file  =',mfname
         write(6,*) '      ier=',ier
         call shr_sys_flush(6)
      endif

      !-------------------------------------------------------------------------
      ! open netcdf file (for reading only), and
      ! and get file ID 
      !-------------------------------------------------------------------------
       
      ircode = nf_open(r_fname(ivar,iftype),NCNOWRIT, &
      &                  nc_file_id(ivar,iftype) )
 
      ncid = nc_file_id(ivar,iftype)

      write(6,1010) r_fname(ivar,iftype),datadir(1:i),ier, &
      &              f_year(iftype),af_year(iftype),ncid
 1010 format(/,'NEW INPUT DATA FILE =',a20,/, &
      &         4x,'from ',a64,' ier=',i3,/, &
      &         4x,'for data year=',i6,' atm year=',i6, &
      &            ' netcdf file ID=',i8)

      if (ircode.ne.0) then
         write(6,*) 'ERROR CODE in initfdat from ncopn= ',ircode
         write(6,*) '           file  =',r_fname(ivar,iftype)
         write(6,*) '           nc_id =',ncid
         call shr_sys_flush (6)
         stop 998
      endif

      !-------------------------------------------------------------------------
      ! check dimension sizes: lon, lat, and time
      !-------------------------------------------------------------------------
 
      ircode =  nf_inq(ncid,ndims,nvars,ngatts,irecdimid )
 
         
      if (ircode.ne.0) then
         write(6,*) 'ERROR in initfdat from nf_inq'
         write(6,*) '      ircode=',ircode
         call shr_sys_flush(6)
      endif
      if (iprtinfo.eq.1) then
         write(6,1020) ndims,nvars,ngatts,irecdimid
         call shr_sys_flush(6)
 1020    format(4x,'dimensions=',i2,' variables=',i2,' global attr.=', &
         &      i2,' rec.dim.ID(-1=not def)=',i3)
      endif

      ! inquire about all dimensions

      do idimid=1,ndims

         ircode = nf_inq_dim (ncid,idimid, dimnam,idimsize)

         if (ircode.ne.0) then
            write(6,*) 'ERROR in initfdat from nf_inq_dim'
            write(6,*) '      dimension=',idimid,' =',dimnam
            write(6,*) '      idimsize =',idimsize,' ircode=',ircode
            call shr_sys_flush(6)
         endif
         if (iprtinfo.eq.1) then
            write(6,1030) idimid,idimsize
 1030       format(2x,i3,'.dim=',i6,$)
            write(6,*) dimnam
            call shr_sys_flush(6)
         endif

         if (dimnam.eq.'lon' .and. idimsize.ne.nx) then
            write(6,*) 'ERROR in initfdat: dimensions do not match'
            write(6,*) '      read size=',idimsize,' nx=',nx
            call shr_sys_flush (6)
            stop 991
         endif
         if (dimnam.eq.'lat' .and. idimsize.ne.ny) then
            write(6,*) 'ERROR in initfdat: dimensions do not match'
            write(6,*) '      read size=',idimsize,' ny=',ny
            call shr_sys_flush (6)
            stop 991
         endif
         if (dimnam.eq.'time') then
            nrecord(iftype) = idimsize
            if (idimsize.gt.maxrec) then
               write(6,*) 'ERROR in initfdat: dimensions do not match'
               write(6,*) '      read record size=',idimsize
               write(6,*) '      maxrec=',maxrec
               call shr_sys_flush (6)
               stop 991
            endif
         endif
      enddo

      !-------------------------------------------------------------------------
      ! get 3D varible ID, date ID;
      ! and inquire about variable type, attributes, etc.
      !-------------------------------------------------------------------------

      ifound3dvar = 0  ! check that there is only one 3D variable

      do 100 ivarid=1,nvars

         ircode = nf_inq_var(ncid,ivarid,varnam,ivartype,nvdims,ivdimids,nvatts)

         if (ircode.ne.0) then
            write(6,*) 'ERROR in initfdat from nf_inq_var'
            write(6,*) '      var=',varnam,' ID=',ivarid
            write(6,*) '      ircode=',ircode
            call shr_sys_flush(6)
         endif

         if (nvdims.eq.3) then
            if (ifound3dvar.gt.0) then
               write(6,*) 'ERROR in initfdat:more than one 3D variables'
               write(6,*) '      in file=',r_fname(ivar,iftype)
               write(6,*) '      second 3D variable=',varnam
               call shr_sys_flush(6)
            endif
            ifound3dvar = 1
            nc_var_id(ivar,iftype) = ivarid
         endif
         ! get date ID from first file in each category
         if (varnam.eq.'date') then
            nc_date_id(ivar,iftype) = ivarid
         endif
         if (varnam.eq.'datesec') then
            nc_datesec_id(ivar,iftype) = ivarid
         endif
         if (varnam.eq.'lon') then
            nc_lon_id(ivar,iftype) = ivarid
         endif
         if (varnam.eq.'lat') then
            nc_lat_id(ivar,iftype) = ivarid
         endif

         if (iprtinfo.eq.1) then
            write(6,1040) ivarid,varnam,nvdims,ivartype,(ivdimids(i),i=1,nvdims)
            call shr_sys_flush(6)
 1040       format(2x,i3,'.var=',a14,'dim=',i2,' type=',i2, ' dimIDs=',3i3)
         endif
                        
         !----------------------------------------------------------------------
         ! print additional netcdf header information
         !----------------------------------------------------------------------

         if (iprtinfo.eq.1) then

            ! inquire about all variable attributes

            do 150 ivatt=1,nvatts

               ! get name of attribute

               ircode = nf_inq_attname (ncid,ivarid, ivatt, attnam)
               if (ircode.ne.0) then
                  write(6,*) 'ERROR in initfdat from nf_inq_attname'
                  write(6,*) '      ivarid=',ivarid,' ivatt=',ivatt
                  write(6,*) '      atname=',attnam,' ircode=',ircode
                  call shr_sys_flush(6)
               endif

                ! get attribute type and number of values in attribute:
                ! for a string-valued attribute, this is the number of
                ! characters in the string

               ircode = nf_inq_att (ncid,ivarid,attnam, iattype,nattlen)
               if (ircode.ne.0) then
                  write(6,*) 'ERROR in initfdat from nf_inq_att'
                  write(6,*) '      attnam =',attnam,' iattype=',iattype
                  write(6,*) '      nattlen=',nattlen
                  write(6,*) '      ircode =',ircode
                  call shr_sys_flush(6)
               endif
               if (iattype.eq.2) then
                  ! attribute is a character: get values
                  ircode = nf_get_att_text (ncid,ivarid,attnam, attstring)
                  if (ircode.ne.0) write(6,*) 'ERROR in initfdat from '&
                  &  //            'nf_get_att_text = ',ircode
                  write(6,1050) attnam
                  call shr_sys_flush(6)
 1050             format(10x,a17,'=',$)
                  write(6,*) attstring(1:nattlen)
               else
                  write(6,1060)
                  call shr_sys_flush(6)
 1060             format(10x,' attribute is not character type')
               endif
 150        continue  ! for all attributes 
         endif        ! if iprtinfo

 100  continue        ! for all variables

      !----------------------------------------------------------------------
      ! check first date in data file, and print first and last date
      !----------------------------------------------------------------------

      istart(1) = 1
      icount(1) = 1 

      ircode = nf_get_vara_int(ncid,nc_date_id(ivar,iftype), &
      & istart,icount, icdate1)
      if (ircode.ne.0) then
         write(6,*) 'ERROR in initfdat from mf_get_vara_int to get first date'
         write(6,*) '      istart=',istart(1)
         write(6,*) '      ncid=',ncid,' ircode=',ircode
         call shr_sys_flush(6)
      endif
      ircode = nf_get_vara_int(ncid,nc_datesec_id(ivar,iftype), &
      & istart,icount, icdatesec1)
      if (ircode.ne.0) then
         write(6,*) 'ERROR in initfdat from nf_get_vara_int to get 1st datesec'
         write(6,*) '      istart=',istart(1)
         write(6,*) '      ncid=',ncid,' ircode=',ircode
         call shr_sys_flush(6)
      endif

      istart(1) = nrecord(iftype)
      ircode = nf_get_vara_int (ncid,nc_date_id(ivar,iftype), &
      & istart,icount, icdate2)
      if (ircode.ne.0) then
         write(6,9000) 'initfdat', 'int', ' to get last date'
         write(6,*) '      istart=',istart(1)
         write(6,*) '      ncid=',ncid,' ircode=',ircode
         call shr_sys_flush(6)
      endif
      ircode = nf_get_vara_int(ncid,nc_datesec_id(ivar,iftype), &
      & istart,icount,icdatesec2)
      if (ircode.ne.0) then
         write(6,9000) 'initfdat', 'int', ' to get last datesec'
         write(6,*) '      istart=',istart(1)
         write(6,*) '      ncid=',ncid,' ircode=',ircode
         call shr_sys_flush(6)
      endif

      write(6,1080) icdate1,icdatesec1,icdate2,icdatesec2
 1080 format(4x,'dates from ',i10,i6,'sec to ',i10,i6,'sec (yyyymmdd)')

      if (icdate1/10000.ne.f_year(iftype)) then
         write(6,*) 'ERROR in initfdat: date mismatch'
         write(6,*) '      supposed to read year=',f_year(iftype)
         write(6,*) '      from file=',r_fname(ivar,iftype)
         write(6,*) '      but first date in file=',icdate1
         call shr_sys_flush (6)
         stop 992
      endif
       
      !-------------------------------------------------------------------------
      ! print all global attributes
      !-------------------------------------------------------------------------

      if (iprtinfo.eq.1) then

         write(6,1070) ngatts
 1070    format(4x,'global attributes      =',i3)

         do igatt=1,ngatts

           ! get name of global attribute

               ircode = nf_inq_attname (ncid,NCGLOBAL, igatt, attnam)
            if (ircode.ne.0) &
            &  write(6,*) 'ERROR: initfdat from nf_inq_attname=',ircode
               call shr_sys_flush(6)

           ! get attribute type and number of values in attribute

            ircode = &
            & nf_inq_att (ncid,NCGLOBAL,attnam, iattype,nattlen)
            if (ircode.ne.0) &
            &  write(6,*) 'ERROR in initfdat from nf_inq_att=',ircode
               call shr_sys_flush(6)
            if (iattype.eq.2) then
               ! attribute is a character: get values
               ircode = nf_get_att_text (ncid,NCGLOBAL,attnam, attstring)
               if (ircode.ne.0) &
               & write(6,*) 'ERROR in initfdat from nf_get_att_text = ',ircode
               call shr_sys_flush(6)
               write(6,1050) attnam
               write(6,*) '    ',attstring(1:nattlen)
            else
               write(6,1060)
            endif
               call shr_sys_flush(6)
         enddo ! for all global attributes
      endif    ! if iprtinfo
  
      return
9000     format (1x, 'ERROR in ', a, 'from nf_get_vara_',a,a)
      end

!===============================================================================
      
      SUBROUTINE closefdat(iftype,nfile,iremove)

      use shr_sys_mod   ! shared code: system calls

      implicit none

      !----- global -----
#include <netcdf.inc>  ! netCDF defs
#include <dims.h>      ! model resolution
#include <data_raw.h>  ! data read from data file
#include <data_nc.h>   ! netcdf file info
      
      !----- input  ------
      integer iftype              ! forcing type categorry
      integer nfile               ! number of files in this category to remove
      integer iremove             ! flag to remove local file: true=1

      !----- local  ------
      integer ivar                ! loop index
      integer ircode              ! error return code from netcdf and ishell
     
      character*30 char           ! character string for ishell

!-------------------------------------------------------------------------------
! PURPOSE:
!    close and remove input data files
!-------------------------------------------------------------------------------

      do ivar=1,nfile
         ircode = nf_close(nc_file_id(ivar,iftype) )
         if (ircode.ne.0) then
            write(6,*) 'ERROR in closefdat from nf_close'
            write(6,*) '      nc_file_id=',nc_file_id(ivar,iftype)
            write(6,*) '      ircode=',ircode
            call shr_sys_flush(6)
         endif
         if (iremove.eq.1) then
            write(char,'("rm ",a15 )') r_fname(ivar,iftype)
            call shr_sys_system(char,ircode)
         endif
      enddo

      return
      end

!===============================================================================
      
      SUBROUTINE domainfdat

      use shr_sys_mod   ! shared code: system calls
      use const_mod     ! physical constants

      implicit none

      !----- global -----
#include <netcdf.inc>  ! netCDF defs
#include <dims.h>      ! model resolution
#include <control.h>   ! control info, date/time
#include <data_msg.h>  ! data sent/received from coupler
#include <data_raw.h>  ! data read from data file
#include <data_nc.h>   ! netcdf file info
      
      !----- local  ------
      integer i,j            ! loop indices
      integer ivar,iftype    ! variable indices
      integer iprtinfo,ifile ! input flags for subroutines
      integer ircode         ! error return code from netcdf calls
      integer istart(3)      ! starting index of record to be read
      integer icount(3)      ! length of record to be read
      real    xt(nx)         ! cell center x-coords read in
      real    yt(ny)         ! cell center y-coords read in
      real    xe(nx+1)       ! cell edge   x-coords read in
      real    ye(ny+1)       ! cell edge   y-coords read in
      real    dth            ! d(theta) for grid cell (radians)
      real    dph            ! d(phi)   for grid cell (radians)

!-------------------------------------------------------------------------------
! PURPOSE:
!    read grid information from netcdf file
!-------------------------------------------------------------------------------

      !-------------------------------------------------------------------------
      ! initialize first file in data sequence
      !-------------------------------------------------------------------------

      ivar     = 1
      iftype   = 1

      f_year (iftype) = data_year0(iftype)
      af_year(iftype) = data_oyear(iftype)

      iprtinfo = 0
      write(6,*) '(domainfdat) : opening file for grid information'
      write(6,*) '             : at year=',f_year(iftype)
      call shr_sys_flush(6)

      call initfdat(data_dir(iftype),ivar,iftype,iprtinfo)

      !-------------------------------------------------------------------------
      ! read longitude and latitude at center of grid
      !-------------------------------------------------------------------------

      istart(1) = 1
      icount(1) = nx
      ircode = nf_get_vara_double(nc_file_id(ivar,iftype), &
      & nc_lon_id(ivar,iftype), istart,icount,xt)
      if (ircode.ne.0) then
         write(6,9000) 'domainfdat', 'double', ' lon data'     
         write(6,*) '      nc_file_ID=',nc_file_id(ivar,iftype)
         write(6,*) '      icount(1) =',icount(1)
         write(6,*) '      ircode=',ircode
         call shr_sys_flush(6)
      endif

      icount(1) = ny
      ircode = nf_get_vara_double(nc_file_id(ivar,iftype), &
      & nc_lat_id(ivar,iftype),istart,icount,yt)
      if (ircode.ne.0) then
         write(6,9000) 'domainfdat', 'double', ' lat data'     
         write(6,*) '      nc_file_ID=',nc_file_id(ivar,iftype)
         write(6,*) '      icount(1) =',icount(1)
         write(6,*) '      ircode=',ircode
         call shr_sys_flush(6)
      endif
      
      ifile   = 1
      call closefdat(iftype,ifile,data_rmlf(iftype))

      !-------------------------------------------------------------------------
      ! compute edges
      !-------------------------------------------------------------------------

      xe(   1) = xt( 1) - (xt( 2) - xt(1))/2.
      xe(nx+1) = xt(nx) + (xt( 2) - xt(1))/2.
      do i=2,nx
         xe(i) = (xt(i-1)+xt(i))/2.
      enddo

      ye(   1) = -90. 
      ye(ny+1) = +90. 
      do j=2,ny
         ye(j) = (yt(j-1)+yt(j))/2.
      enddo

      !-------------------------------------------------------------------------
      ! reformat domain data to format required by cpl
      !-------------------------------------------------------------------------
      do j=1,ny
      do i=1,nx
        xc(  i,j)=xt(i)
        yc(  i,j)=yt(j)
        xv(1,i,j)=xe(i  )
        xv(2,i,j)=xe(i+1)
        xv(3,i,j)=xe(i+1)
        xv(4,i,j)=xe(i  )
        yv(1,i,j)=ye(j  )
        yv(2,i,j)=ye(j  )
        yv(3,i,j)=ye(j+1)
        yv(4,i,j)=ye(j+1)
        mask(i,j) = 1        ! assume all cells are active
        dph = ( xe(i+1) - xe(i) )*DEGtoRAD
        dth = sin(ye(j+1)*DEGtoRAD)-sin(ye(j)*DEGtoRAD)
        area(i,j) = dth*dph
      end do
      end do

      return
9000     format (1x, 'ERROR in ', a, 'from nf_get_vara_',a,a)
      end

!===============================================================================
