!===============================================================================
! CVS $Id: data_raw.h,v 1.3 2002/04/30 17:44:41 tcraig Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/atm/latm5/data_raw.h,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      !-------------------------------------------------------------------------
      ! raw data: 
      ! two time slices read in from data files (note, that only one 
      ! file per variable is open for reading);
      ! there are 3 forcing categories; within each category there can 
      ! be several data files; 
      ! the data input and interpolation is handled in the exact same 
      ! way for all data files in each category, but may differ
      ! between one category and another;
      ! all fields sent to the flux coupler are either interpolated 
      ! raw data, or fields derived from interpolated raw data 
      ! these fields are not to be derived first and then time 
      ! interpolated.
      ! 
      !
      !-------------------------------------------------------------------------
      !
      ! data loop:
      ! the data loop cycles from data_year0 to 
      ! (data_year0+data_nyear-1):
      ! 
      !                         data_nyear
      !                   +-----------------------+
      !                   |                       |
      ! data :  ....  data_year0  ....  ....  data_year0+  .... .... 
      !                                       data_nyear
      !                                               -1
      !   eg :    83    1980      1981  1982     1983       80   81 
      ! 
      !   eg :                    0004  0005     0006     0007 0008
      ! 
      ! model:                    year
      ! 
      !-------------------------------------------------------------------------
      ! 
      ! time:
      ! 
      ! -------+----------+--------------+-----------+------------------
      ! model  | atm year | file name    | data      | date of data used
      !        |    of    |              | (read from| to interpolate
      !        | file name|              | file)     |
      ! -------+----------+--------------+-----------+------------------
      ! cdate  |          |              | r_cdate   |
      ! year   | af_year  |   f_year     | r_year    | d_year
      ! month  | (iftype) |  (iftype)    | r_month   |
      ! day    |          |              | r_day     |
      ! sec    |          |              | r_sec     |
      ! eday   |          |              |           | d_eday
      ! -------+----------+--------------+-----------+------------------
      ! a_edays|          |              |           | d_edays(iftype,k)
      ! -------+----------+--------------+-----------+------------------
      !
      ! where:
      ! af_year = (year-1) or year or (year+1)
      ! f_year  = data_0year + modulo(af_year-data_oyear,data_nyear)
      ! r_year  = f_year or (f_year+1)
      !           (NCEP 6-hourly state variables start and end with:
      !           1980/01/01  6hr, 1980/01/01 12hr, 1980/01/01 18hr,
      !           1980/01/02  0hr, ..
      !           ..             , 1980/01/01 18hr, 1981/01/01  0hr)
      ! d_year  = r_year - f_year + af_year
      ! a_edays = real(eday  ) + real(sec  )/86400.
      ! d_edays = real(d_eday) + real(r_sec)/86400.
      ! 
      !-------------------------------------------------------------------------

      !----- configuration -----

      integer maxftype,maxs,maxp,maxr,maxvar,maxtlev,maxrec

      parameter (maxftype=3)   ! max num of forcing types: states,precip,radflx
      parameter (maxs    =7)   ! maximum number of states fields
      parameter (maxp    =2)   ! maximum number of precip fields
      parameter (maxr    =6)   ! maximum number of radflx fields
      parameter (maxvar  =7)   ! maxvar = max(maxs,maxp,maxr) )
      parameter (maxtlev =2)   ! maximum number of time levels: old and new
      parameter (maxrec=365*4) ! maxium number of records in data files

      !----- timing info -----

      integer r_kold (maxftype)   ! =1 or 2: k index of older data slice
                                  !          for each forcing type
      integer r_knew (maxftype)   ! =1 or 2: k index of newer data slice

      integer  f_year(maxftype)   ! year in file names of data
      integer af_year(maxftype)   ! atmosphere year corresponding to f_year

      integer nvar(maxftype)      ! number of variable fields currently
                                  ! in use in each forcing type
      integer irecord(maxftype)   ! number of records read in all files
                                  ! for each forcing type 
      integer nrecord(maxftype)   ! maximum number of records in all 
                                  ! files for each forcing type 
      character*16 r_fname(maxvar,maxftype)! local input filenames of raw data
                               
      !----- data fields -----

      real d_edays(maxftype,maxtlev) ! elapsed days incl. day fraction of
                               ! data converted to atmospheric years
                               ! in order to use for interpolation 

      real r_states(nx,ny,maxs,maxtlev+1) ! raw input and interpolated 
                               ! fields; possible input:
                               ! u    = wind, zonal           ~ m/s
                               ! v    = wind, meridional      ~ m/s
                               ! ptem = potential temperature ~ Kelvin
                               ! shum = specific humidity     ~ kg/kg
                               ! dens = density               ~ kg/m^3
                               ! t    = temperature           ~ Kelvin
                               ! pres = pressure              ~ Pa
  
      real r_precip(nx,ny,maxp,maxtlev+1) ! possible input:
                               ! prec = precip                ~ kg/s/m^2
                               ! precc= precip, convective    ~ kg/s/m^2
                               ! precl= precip, large-scale   ~ kg/s/m^2

      real r_radflx(nx,ny,maxr,maxtlev+1) ! possible input:
                               ! lwdn = longwave,  downward     ~  W/m^2
                               ! swnet= shortwave, net          ~  W/m^2
                               ! swup = shortwave, up           ~  W/m^2
                               ! swdn = shortwave, down         ~  W/m^2
                               ! swvdr= shortwave, down,vis-dir ~  W/m^2
                               ! swndr= shortwave, down,nir-dir ~  W/m^2
                               ! swvdf= shortwave, down,vis-diff~  W/m^2
                               ! swndf= shortwave, down,nir-diff~  W/m^2
                               ! cldf = cloud fraction          ~  0.-1.
 

      !--- pointers to data arrays 
      integer k_u    , k_v     
      integer k_ptem , k_t    , k_shum , k_dens , k_pres
      integer k_precc, k_precl 
      integer k_lwdn , k_swnet, k_swup , k_swdn  
      integer k_swvdr, k_swndr, k_swvdf, k_swndf, k_cldf

      !----- forcing switches -----
      !----- example provided with NCEP and CCM, user may add other options -----

      logical NCEPSTATDAT       ! states from NCEP
      logical CCMSTATDAT        !          or CCM
      logical NCEPPRECDAT       ! precip from NCEP
      logical CCMPRECDAT        !          or CCM
      logical NCEPRADFDAT       ! radflx from NCEP
      logical CCMRADFDAT        !          or CCM
      logical NOREADDAT         ! no input data files read (using 
                                ! constant fields set in "initial.f")

      !-------------------------------------------------------------------------
      ! make data global
      !-------------------------------------------------------------------------

      COMMON /data_raw/                                               &
     &  r_kold     , r_knew      ,                                    &
     &  f_year     , af_year     ,                                    &
     &  nvar       , irecord     , nrecord   ,                        &
     &  d_edays    , r_fname     ,                                    &
     &  r_states   , r_precip    , r_radflx  ,                        &
     &  k_u        , k_v         ,                                    &
     &  k_ptem     , k_t         , k_shum    , k_dens  , k_pres ,     &
     &  k_precc    , k_precl     ,                                    &
     &  k_lwdn     , k_swnet     , k_swup    , k_swdn  ,              &
     &  k_swvdr    , k_swndr     , k_swvdf   , k_swndf , k_cldf ,     &
     &  NCEPSTATDAT, CCMSTATDAT  ,                                    &
     &  NCEPPRECDAT, CCMPRECDAT,                                      &
     &  NCEPRADFDAT, CCMRADFDAT,                                      &
     &  NOREADDAT 

      SAVE   /data_raw/

