!===============================================================================
! CVS $Id: initial.F90,v 1.7 2002/05/13 18:24:12 tcraig Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/atm/latm5/initial.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE initial()

      use const_mod     ! shared physical constants
      use shr_sys_mod   ! shared system routines
    
      implicit none

      !----- global -----
#include <dims.h>      ! model resolution
#include <data_raw.h>  ! data read from data file
#include <data_msg.h>  ! data sent/received from coupler
#include <control.h>   ! control info, date/time

      NAMELIST / inparm /  &
     &  rest_type   , &
     &  rest_pfile  , &
     &  rest_bfile  , &
     &  rest_freq   , &
     &  rest_date   , &
     &  rest_nday   , &
     &  rest_odate  , &
     &  case_name   , &
     &  case_desc   , &
     &  data_dir    , &
     &  data_rmlf   , &
     &  data_year0  , &
     &  data_nyear  , &
     &  data_oyear  , &
     &  flux_albfb  , &
     &  flux_lwfb   , &
     &  mss_dir     , &
     &  mss_pass    , &
     &  mss_opts    , &
     &  mss_rtpd    , &
     &  mss_rmlf    , &
     &  info_dbug   , &
     &  info_time   , &
     &  ncpl        

      !----- local -----
      integer rcode            ! return code
      integer i,j,k,n          ! loop indices
      integer ivar,iftype      ! loop indices

      !----- formats -----
      character(len=*),parameter :: F00 = "('(initial) ',2a)"
      character(len=*),parameter :: F01 = "('(initial) ',a,i7)"

!-------------------------------------------------------------------------------
! PURPOSE:
!  o initialize namelist values
!  o read input namelist
!  o set initial control flags
!  o set data forcing flags
!  o set default coupler data
!  o set horizontal grid for data
!-------------------------------------------------------------------------------

      !-----------------------------------------------
      ! set initial namelist values
      !-----------------------------------------------
      write(6,'(a)') '(initial) setting namelist defaults.'
      rest_type   = 'continue'  ! run type: initial, cont, regen, branch
      rest_pfile  = 'null'      ! IC data pointer file
      rest_bfile  = 'null'      ! IC data file for branch runs
      rest_freq   = 'never'     ! restart file frequency
      rest_date   = 10101       ! start date for initial or branch runs
      rest_nday   = 365         ! restart file interval    (nday option
      rest_odate  = 0101        ! restart file offset date (nday option)
      case_name   = ' '         ! case name
      case_desc   = ' '         ! case description
      do n=1,3                  ! 1=>state, 2=>radiation, 3=>precip
        data_dir  (n) = 'null'  ! data: directory
        data_year0(n) = 1       ! data: 1st  year  in sequence
        data_nyear(n) = 1       ! data: # of years in sequence
        data_oyear(n) = 1       ! data: cpl year matched to year0
        data_rmlf (n) = false   ! data: remove file after reading
      end do
      flux_albfb  = false       ! T => do optional albedo/sw feedback
      flux_lwfb   = false       ! T => do optional lw feedback (use SST)
      flux_init   = unset       ! T => do optional flux initialization
      mss_dir     = 'null'      ! MSS file root directory
      mss_pass    = ' '         ! MSS file password
      mss_opts = 'nowait,nomail'! MSS file mswrite options
      mss_rtpd    = 180         ! MSS file MS retention period
      mss_rmlf    = true        ! T => remove local file after mswrite
      info_dbug   = 1           ! dbug level: 0=lowest, 3=highest
      info_time   = false       ! T => print extra timing info
      ncpl        = 24          ! number of msg pairs per day

      !-----------------------------------------------
      ! read input namelist
      !-----------------------------------------------
      write(6,F00) 'reading input namelist.'
      read (5,inparm,iostat=rcode)
      if (rcode > 0) then
         write(6,F01) 'ERROR: reading input namelist, iostat=',rcode
	 call shr_sys_abort("XXXX namelist read error")
      end if

      !-----------------------------------------------
      ! re-set some default namelist values
      !-----------------------------------------------
      do n=2,3
        if (data_dir(n)(1:4) .eq. 'null') then
          data_year0(n) = data_year0(1)
          data_nyear(n) = data_nyear(1)
          data_oyear(n) = data_oyear(1)
          data_rmlf (n) = data_rmlf (1)
        end if
      end do

      !-----------------------------------------------
      ! set default value for flux_init 
      !-----------------------------------------------
      if ( flux_init .eq. unset ) then
        flux_init = false
        if ( rest_type(1:4) .eq. 'init' ) then
          if ( flux_albfb .ne. 0) flux_init = true
          if ( flux_lwfb  .ne. 0) flux_init = true
        end if
      end if

      !-----------------------------------------------
      ! record initial namelist values
      !-----------------------------------------------
      write(6,'(a)') '(initial) namelist values:'
      write(6,inparm)
      write(6,*) 

      !-----------------------------------------------
      ! inititial values for control flags
      !-----------------------------------------------
      ierr     = 0  ! error code
      stop_now = 0  ! T => stop now
      stop_eod = 0  ! T => stop at end-of-day
      rest_now = 0  ! T => create restart file now
      rest_eod = 0  ! T => create restart file at end-of-day
      hist_now = 0  ! T => create history file now
      hist_eod = 0  ! T => create history file at end-of-day
      hist_tav = 0  ! T => create monthly avg data
      diag_now = 0  ! T => create diagnostic data now
      diag_eod = 0  ! T => create diagnostic data at end-of-day

      !-----------------------------------------------
      ! set data forcing flags, 
      ! dependent on data directory names
      !-----------------------------------------------

        NCEPSTATDAT = .false.
         CCMSTATDAT = .false.
      !---------------------
           NCEPPRECDAT = .false.
            CCMPRECDAT = .false.
      !---------------------
       NCEPRADFDAT = .false.
        CCMRADFDAT = .false.
      !---------------------
         NOREADDAT = .false.

      !-----------------------------------------------
      ! determine number of variables in each category,
      ! set indices of forcing data arrays, and 
      ! define templates of data file names.
      !-----------------------------------------------

      if (data_dir(1)(1:4).eq.'null') then
         NOREADDAT = .true.
      else
         !--------------------------------------------
         ! states
         !--------------------------------------------
         if   (index(data_dir(1),'NCEP' ).gt.0) then
            NCEPSTATDAT = .true.
            k_u     = 1
            k_v     = 2
            k_t     = 3
            k_shum  = 4
            k_dens  = 5
            nvar(1) = 5
            r_fname(1,1) = 'u_10.1997.nc'
            r_fname(2,1) = 'v_10.1997.nc'
            r_fname(3,1) = 't_10.1997.nc'
            r_fname(4,1) = 'q_10.1997.nc'
            r_fname(5,1) = 'dn10.1997.nc'
         else if (index(data_dir(1),'CCM'  ).gt.0) then
            CCMSTATDAT  = .true.
            k_u     = 1
            k_v     = 2
            k_t     = 3
            k_shum  = 4
            k_dens  = 5
            k_ptem  = 6
            k_pres  = 7
            nvar(1) = 7
            r_fname(1,1) = 'u_10.1997.nc'
            r_fname(2,1) = 'v_10.1997.nc'
            r_fname(3,1) = 'p_10.1997.nc'
            r_fname(4,1) = 'q_10.1997.nc'
            r_fname(5,1) = 'dn10.1997.nc'
            r_fname(6,1) = 'pt10.1997.nc'
            r_fname(7,1) = 'pres.1997.nc'
         else
            write(6,*) 'ERROR in initial:'
            write(6,*) '      cannot identify states'
            write(6,*) '      for data directory=',data_dir(1)
            stop 993
         endif
         !--------------------------------------------
         ! precipitation
         !--------------------------------------------
         if      (index(data_dir(2),'NCEP' ).gt.0) then 
            NCEPPRECDAT = .true.
            k_precl = 1
            nvar(2) = 1
            r_fname(1,2) = 'prec.1997.nc'
         else if (index(data_dir(2),'CCM'  ).gt.0) then
            CCMPRECDAT  = .true.
            k_precc = 1
            k_precl = 2
            nvar(2) = 2
            r_fname(1,2) = 'precc.1997.nc'
            r_fname(2,2) = 'precl.1997.nc'
         else
            write(6,*) 'ERROR in initial:'
            write(6,*) '      cannot identify precip'
            write(6,*) '      for data directory=',data_dir(2)
            stop 994
         endif
         !--------------------------------------------
         ! radiation fluxes
         !--------------------------------------------
         if      (index(data_dir(3),'NCEP' ).gt.0) then
            NCEPRADFDAT = .true.
            k_lwdn  = 1
            k_swdn  = 2
            k_swup  = 3
            nvar(3) = 3
            r_fname(1,3) = 'lwdn.1997.nc'
            r_fname(2,3) = 'swdn.1997.nc'
            r_fname(3,3) = 'swup.1997.nc'
         else if (index(data_dir(3),'CCM'  ).gt.0) then
            CCMRADFDAT  = .true.
            k_lwdn  = 1
            k_swnet = 2
            k_swvdr = 3
            k_swndr = 4
            k_swvdf = 5
            k_swndf = 6
            nvar(3) = 6
            r_fname(1,3) = 'lwdn.1997.nc'
            r_fname(2,3) = 'swnet.1997.nc'
            r_fname(3,3) = 'swvdr.1997.nc'
            r_fname(4,3) = 'swndr.1997.nc'
            r_fname(5,3) = 'swvdf.1997.nc'
            r_fname(6,3) = 'swndf.1997.nc'
         else
            write(6,*) 'ERROR in initial:'
            write(6,*) '      cannot identify radflx'
            write(6,*) '      for data directory=',data_dir(3)
            stop 995
         endif

      endif ! if not NOREADDAT
        
      !--------------------------------------------------------------
      ! initialize all fields to be sent to coupler. 
      ! note: constant height of 10m of forcing state variables;
      ! note: only the land model needs t, pres, precc, precl, and
      !       swvdr,swndr,swvdf,swndf;
      ! note: ocn and ice models use (precc+precl).
      !--------------------------------------------------------------
      
      do j=1,ny
      do i=1,nx
         
         z(i,j)        =  10.0 ! height level for          ~ m 

         if (NOREADDAT) then
            u    (i,j) =   0.0 ! wind, zonal               ~ m/s
            v    (i,j) =   2.0 ! wind, meridional          ~ m/s 
            ptem (i,j) = 301.0 ! potential temperature     ~ Kelvin
            t    (i,j) = 300.0 ! temperature               ~ Kelvin
            shum (i,j) = 1.e-2 ! specific humidity         ~ kg/kg
            dens (i,j) =   1.0 ! density                   ~ kg/m^3
            pres (i,j) = 1.e+6 ! pressure                  ~ Pa
            pslv (i,j) = 1.e+6 ! pressure                  ~ Pa
            precc(i,j) = 1.e-6 ! precip, convective        ~ kg/s/m^2
            precl(i,j) = 1.e-6 ! precip, large-scale       ~ kg/s/m^2
            lwdn (i,j) = 100.0 ! longwave, downward        ~    W/m^2
            swnet(i,j) = 100.0 ! shortwave, net            ~    W/m^2
            swvdr(i,j) = 100.0 ! shortwave, down, vis-dir  ~    W/m^2
            swndr(i,j) = 100.0 ! shortwave, down, nir-dir  ~    W/m^2
            swvdf(i,j) = 100.0 ! shortwave, down, vis-diff ~    W/m^2
            swndf(i,j) = 100.0 ! shortwave, down, nir-diff ~    W/m^2
         else
            if (NCEPSTATDAT) then
               t    (i,j) = MissVal
               pres (i,j) = MissVal
               pslv (i,j) = MissVal
            endif
            if (.not. CCMPRECDAT) then
               precc(i,j) = 0.
            endif
            if (.not. CCMRADFDAT) then
               swvdr(i,j) = MissVal
               swndr(i,j) = MissVal
               swvdf(i,j) = MissVal
               swndf(i,j) = MissVal
            endif
         endif
      enddo
      enddo

      !-----------------------------------------------
      ! set the atmospheric domain
      !-----------------------------------------------

      if (NOREADDAT) then
         call domain
      else                ! read grid from netcdf file
         call domainfdat      
      endif

      RETURN
      END

!===============================================================================
