!===============================================================================
! CVS $Id: main.F90,v 1.1 2001/03/28 22:11:26 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/atm/latm5/main.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      PROGRAM main

!===============================================================================
! 
!    The Community Climate System Model (CCSM) Copyright and Release Notice
! 
!              +------------------------------------------+
!              | DISTRIBUTION TERMS AND CONDITIONS NOTICE |
!              +------------------------------------------+
! 
! 
! 
! CCSM was developed in cooperation with the National Science Foundation,
! the Department of Energy Los Alamos National Laboratory (LANL), the
! National Aeronautics and Space Administration Data Assimilation Office,
! and the University Corporation for Atmospheric Research's National
! Center for Atmospheric Research.*  Except for POP, SCRIP, and CICE,
! which are segregable components of the model, CCSM is public domain
! software.  
! 
! POP, SCRIP, and CICE were developed at LANL and are protected by the
! following copyright notice. 
! 
! Copyright (c)2002 The Regents of the University of California. All Rights
! Reserved. 
! 
! Permission to use, copy, modify, and distribute this software and its
! documentation for educational, research and non-profit purposes, without
! fee, and without a written agreement is hereby granted, provided that
! the above copyright notice, this paragraph and the following three
! paragraphs appear in all copies. 
! 
! Permission to incorporate this software into commercial products may be
! obtained by contacting the University of California, Charles Rzeszutko,
! Campus Liaison Officer Office of Technology Transfer, 1111 Franklin
! Street, 5th Floor, Oakland, CA 94607, (510) 587-6063,
! charles.rzeszutko@ucop.edu.
! 
! THIS SOFTWARE PROGRAM AND DOCUMENTATION ARE COPYRIGHTED BY 
! THE REGENTS OF THE UNIVERSITY OF CALIFORNIA. THE SOFTWARE 
! PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS", WITHOUT ANY 
! ACCOMPANYING SERVICES FROM THE REGENTS. THE REGENTS DOES NOT 
! WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE 
! UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS THAT 
! THE PROGRAM WAS DEVELOPED FOR RESEARCH PURPOSES AND IS 
! ADVISED NOT TO RELY EXCLUSIVELY ON THE PROGRAM FOR ANY 
! REASON. 
! 
! IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY 
! PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
! CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF 
! THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE 
! UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF 
! SUCH DAMAGE.  THE UNIVERSITY OF CALIFORNIA SPECIFICALLY 
! DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
! IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
! PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN 
! "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO 
! OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, 
! ENHANCEMENTS, OR MODIFICATIONS. 
! 
! *The National Center for Atmospheric Research is sponsored by the
! National Science Foundation.  Any opinions, findings and conclusions or
! recommendations expressed in this publication are those of the author(s)
! and do not necessarily reflect the views of the National Science
! Foundation.
!===============================================================================

      use calendar_mod  ! calendar routines
      use shr_msg_mod   ! shared code: msg passing
      use shr_sys_mod   ! shared code: system calls
      use shr_timer_mod ! shared code: timing routines

      implicit none

      !----- global -----
#include <control.h>

      !----- local -----
      integer      n        ! inner loop index
      integer      timer_n  ! timer number
      character*8  dstr     ! real world date string
      character*10 tstr     ! real world time string

!-------------------------------------------------------------------------------
! PURPOSE:
!   this is a climatological-data atm model
!   for use as an atm component of a CSM coupled model system
!-------------------------------------------------------------------------------

      call shr_msg_stdio('atm')
      call shr_timer_init
      call shr_timer_get(timer_n,'main ~ startup overhead')
      call shr_timer_start(timer_n)

      !-------------------------------------------------------------------------
      ! initialize
      !-------------------------------------------------------------------------
      call date_and_time(dstr,tstr)
      write(6,9000)
      write(6,9001) 'CSM climatology data atm model'
      write(6,9004) dstr(1:4),dstr(5:6),dstr(7:8),tstr(1:2),tstr(3:4),tstr(5:6)
      write(6,9001) 'CVS Tag: $Name: ccsm2_0_beta46 $'
      write(6,9000)

      call initial()              !--- read input namelist ---
      call domain ()              !--- read in domain data ---

      call msg_pass('connect')    !--- enable msg passing ---
      call msg_pass('init')       !--- send domain, get IC t ---
      call control()              !--- set control flags & IC t---

      call advance()              !--- advance the data ---

      if ( flux_init .ne.0 ) then !--- auxiliary IC msgs   ---
        write(6,9002) 'initial solar correction (optional)'
        call msg_pass('send')     !--- send output fields  ---
        call msg_pass('recv')     !--- recv input fields   ---
        call control()            !--- set control flags   ---
        call advance()            !--- recompute IC fields --- 
      end if

      call shr_timer_stop (timer_n)
      call shr_timer_print(timer_n)
      call shr_timer_free (timer_n)
      call shr_timer_get  (timer_n,'main ~ model integration')
      call shr_timer_zero_all
      call shr_timer_start(timer_n)

      !-------------------------------------------------------------------------
      ! advance the model: outer loop corresponds to 1 day
      !-------------------------------------------------------------------------
      DO WHILE ( stop_now .eq. 0 )
         DO n=1,ncpl
           if (sec.le.0 .or. info_dbug.ge.2) then
              write(6,9003) 'date: ',year,month,day,sec
              call shr_sys_flush(6)
           end if

           call msg_pass('send')     !--- send output fields ---
           call msg_pass('recv')     !--- recv input fields ---

           call control()            !--- set control flags ---

           call diagnos()            !--- write diagnostics? ---
           call history()            !--- make history file? ---

           if (stop_now .ne. 0) GOTO 900

           if (n.lt.ncpl) then       !--- advance the date ---
             sec  = sec + 86400/ncpl
           else
             eday = eday + 1
             call eday2ymd (eday,year,month,day)
             call eday2date(eday,cdate)
             sec  = 0
           end if
           call advance()            !--- advance the data ---

         END DO
      END DO
  900 CONTINUE
      call shr_timer_stop(timer_n)

      !-------------------------------------------------------------------------
      ! shut-down
      !-------------------------------------------------------------------------
      call msg_pass('disconnect')    !--- disable msg passing ---
      call shr_timer_print_all

      call date_and_time(dstr,tstr)
      write(6,9000)
      write(6,9001) 'Terminated model '
      write(6,9004) dstr(1:4),dstr(5:6),dstr(7:8),tstr(1:2),tstr(3:4),tstr(5:6)
      write(6,9000)

      STOP
 9000 format(80('='))
 9001 format(4a)
 9002 format('(main) ',2a)
 9003 format('(main) ',a,i4.4,'-',i2.2,'-',i2.2,' ',i5,'s')
 9004 format('date & time:',1x,a4,'-',a2,'-',a2,1x,a2,':',a2,':',a2)
 9999 format(' NCAR CSM Data Atm Model,  Copyright (C) 2000'/,     &
      &      ' University Corporation for Atmospheric Research, ', &
      &      ' All Rights Reserved')

      END

!===============================================================================
