!===============================================================================
! CVS $Id: buf_mod.F90,v 1.15 2001/08/03 21:25:36 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/buf_mod.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

MODULE buf_mod

   IMPLICIT none

   !----------------------------------------------------------------------------
   ! information buffers passed to & from models
   !----------------------------------------------------------------------------
   INTEGER,ALLOCATABLE :: ibuff_c(:)    ! integer info-buffer from cpl
   REAL   ,ALLOCATABLE :: rbuff_c(:)    ! real    info-buffer from cpl
   INTEGER,ALLOCATABLE :: ibuff_a(:)    ! integer info-buffer from atm
   INTEGER,ALLOCATABLE :: ibuff_i(:)    ! integer info-buffer from ice
   INTEGER,ALLOCATABLE :: ibuff_l(:)    ! integer info-buffer from lnd
   INTEGER,ALLOCATABLE :: ibuff_o(:)    ! integer info-buffer from ocn

   !----------------------------------------------------------------------------
   ! domain information
   !----------------------------------------------------------------------------
   INTEGER,ALLOCATABLE :: mask_a(:)     ! atm: domain mask
   REAL   ,ALLOCATABLE ::   xc_a(:)     ! atm: x-coord of grid cell center
   REAL   ,ALLOCATABLE ::   yc_a(:)     ! atm: y-coord of grid cell center
   REAL   ,ALLOCATABLE ::   xv_a(:,:)   ! atm: x-coord of grid cell vertex
   REAL   ,ALLOCATABLE ::   yv_a(:,:)   ! atm: y-coord of grid cell vertex
   REAL   ,ALLOCATABLE :: area_a(:)     ! atm: area of grid cell
   INTEGER,ALLOCATABLE :: mask_i(:)     ! ice: domain mask
   REAL   ,ALLOCATABLE ::   xc_i(:)     ! ice: x-coord of grid cell center
   REAL   ,ALLOCATABLE ::   yc_i(:)     ! ice: y-coord of grid cell center
   REAL   ,ALLOCATABLE ::   xv_i(:,:)   ! ice: x-coord of grid cell vertex
   REAL   ,ALLOCATABLE ::   yv_i(:,:)   ! ice: y-coord of grid cell vertex
   REAL   ,ALLOCATABLE :: area_i(:)     ! ice: area of grid cell
   INTEGER,ALLOCATABLE :: mask_l(:)     ! lnd: domain mask
   REAL   ,ALLOCATABLE ::   xc_l(:)     ! lnd: x-coord of grid cell center
   REAL   ,ALLOCATABLE ::   yc_l(:)     ! lnd: y-coord of grid cell center
   REAL   ,ALLOCATABLE ::   xv_l(:,:)   ! lnd: x-coord of grid cell vertex
   REAL   ,ALLOCATABLE ::   yv_l(:,:)   ! lnd: y-coord of grid cell vertex
   REAL   ,ALLOCATABLE :: area_l(:)     ! lnd: area of grid cell
   INTEGER,ALLOCATABLE :: mask_o(:)     ! ocn: domain mask
   REAL   ,ALLOCATABLE ::   xc_o(:)     ! ocn: x-coord of grid cell center
   REAL   ,ALLOCATABLE ::   yc_o(:)     ! ocn: y-coord of grid cell center
   REAL   ,ALLOCATABLE ::   xv_o(:,:)   ! ocn: x-coord of grid cell vertex
   REAL   ,ALLOCATABLE ::   yv_o(:,:)   ! ocn: y-coord of grid cell vertex
   REAL   ,ALLOCATABLE :: area_o(:)     ! ocn: area of grid cell
   INTEGER,ALLOCATABLE :: mask_r(:)     ! ocn: domain mask
   REAL   ,ALLOCATABLE ::   xc_r(:)     ! roff: x-coord of grid cell center
   REAL   ,ALLOCATABLE ::   yc_r(:)     ! roff: y-coord of grid cell center
   REAL   ,ALLOCATABLE ::   xv_r(:,:)   ! roff: x-coord of grid cell vertex
   REAL   ,ALLOCATABLE ::   yv_r(:,:)   ! roff: y-coord of grid cell vertex
   REAL   ,ALLOCATABLE :: area_r(:)     ! roff: area of grid cell

   !----------------------------------------------------------------------------
   ! output state buffers 
   !----------------------------------------------------------------------------
   REAL,ALLOCATABLE ::    Sa_a(:,:)     ! atm states on atm grid
   REAL,ALLOCATABLE ::    Sa_i(:,:)     ! atm states on ice grid
   REAL,ALLOCATABLE ::    Sa_l(:,:)     ! atm states on lnd grid
   REAL,ALLOCATABLE ::    Sa_o(:,:)     ! atm states on ocn grid

   REAL,ALLOCATABLE ::    Si_a(:,:)     ! ice states on atm grid
   REAL,ALLOCATABLE ::    Si_i(:,:)     ! ice states on ice grid
   REAL,ALLOCATABLE ::    Si_o(:,:)     ! ice states on ocn grid

   REAL,ALLOCATABLE ::    Sl_a(:,:)     ! lnd states on atm grid 
   REAL,ALLOCATABLE ::    Sl_l(:,:)     ! lnd states on lnd grid   

   REAL,ALLOCATABLE ::    So_a(:,:)     ! ocn states on atm grid
   REAL,ALLOCATABLE ::    So_i(:,:)     ! ocn states on ice grid
   REAL,ALLOCATABLE ::    So_o(:,:)     ! ocn states on ocn grid

   !----------------------------------------------------------------------------
   ! merged input state buffers  
   !----------------------------------------------------------------------------
   REAL,ALLOCATABLE ::    Ss_a(:,:)     ! merged surface states, atm grid

   !----------------------------------------------------------------------------
   ! merged input flux buffers 
   !----------------------------------------------------------------------------
   REAL,ALLOCATABLE ::  Faxx_a(:,:)     ! merged atm input flux
   REAL,ALLOCATABLE ::  Flxx_l(:,:)     ! merged lnd input flux
   REAL,ALLOCATABLE ::  Fixx_i(:,:)     ! merged ice input flux
   REAL,ALLOCATABLE ::  Foxx_o(:,:)     ! merged ocn input flux
   REAL,ALLOCATABLE :: aFoxx_o(:,:)     ! merged ocn input flux (time-averaged)
!
   !----------------------------------------------------------------------------
   ! component flux buffers 
   !----------------------------------------------------------------------------
   REAL,ALLOCATABLE ::  Faxa_a(:,:)     ! a/x flux, from atm, atm grid

   REAL,ALLOCATABLE ::  Faia_a(:,:)     ! a/i flux, from atm, atm grid
   REAL,ALLOCATABLE ::  Faia_i(:,:)     ! a/i flux, from atm, ice grid
   REAL,ALLOCATABLE ::  Faii_a(:,:)     ! a/i flux, from ice, atm grid
   REAL,ALLOCATABLE ::  Faii_i(:,:)     ! a/i flux, from ice, ice grid
   REAL,ALLOCATABLE ::  Faic_a(:,:)     ! a/i flux, from cpl, atm grid
   REAL,ALLOCATABLE ::  Faic_i(:,:)     ! a/i flux, from cpl, ice grid

   REAL,ALLOCATABLE ::  Fala_a(:,:)     ! a/l flux, from atm, atm grid
   REAL,ALLOCATABLE ::  Fala_l(:,:)     ! a/l flux, from atm, lnd grid
   REAL,ALLOCATABLE ::  Fall_a(:,:)     ! a/l flux, from lnd, atm grid
   REAL,ALLOCATABLE ::  Fall_l(:,:)     ! a/l flux, from lnd, lnd grid
   REAL,ALLOCATABLE ::  Falc_a(:,:)     ! a/l flux, from cpl, atm grid
   REAL,ALLOCATABLE ::  Falc_l(:,:)     ! a/l flux, from cpl, lnd grid

   REAL,ALLOCATABLE ::  Faoa_a(:,:)     ! a/o flux, from atm, atm grid
   REAL,ALLOCATABLE ::  Faoa_o(:,:)     ! a/o flux, from atm, ocn grid
   REAL,ALLOCATABLE ::  Faoo_a(:,:)     ! a/o flux, from ocn, atm grid 
   REAL,ALLOCATABLE ::  Faoo_o(:,:)     ! a/o flux, from ocn, ocn grid
   REAL,ALLOCATABLE ::  Faoc_a(:,:)     ! a/o flux, from cpl, atm grid
   REAL,ALLOCATABLE ::  Faoc_o(:,:)     ! a/o flux, from cpl, ocn grid

   REAL,ALLOCATABLE ::  Fioi_i(:,:)     ! i/o flux, from ice, ice grid
   REAL,ALLOCATABLE ::  Fioi_o(:,:)     ! i/o flux, from ice, ocn grid
   REAL,ALLOCATABLE ::  Fioo_i(:,:)     ! i/o flux, from ocn, ice grid
   REAL,ALLOCATABLE ::  Fioo_o(:,:)     ! i/o flux, from ocn, ocn grid
   REAL,ALLOCATABLE ::  Fioc_i(:,:)     ! i/o flux, from cpl, ice grid
   REAL,ALLOCATABLE ::  Fioc_o(:,:)     ! i/o flux, from cpl, ocn grid

   !--- special case: a one-field buffer specifically for compressed runoff ---
   REAL,ALLOCATABLE ::  roff_rc(:,:)    ! r/o flux, from lnd, runoff vector
   real,ALLOCATABLE ::  roff_o (:,:)    ! r/o flux, from lnd, ocn grid

!
   !----- information buffers passed to & from models ---------------
   INTEGER,PARAMETER :: nibuff= 100 ! size of integer info-buffers
   INTEGER,PARAMETER :: nrbuff= 100 ! size of real    info-buffers

   !----- computational domain dimensions -------------------------------
   INTEGER ::   n_a, nx_a, ny_a   ! # of atm grid points
   INTEGER ::   n_i, nx_i, ny_i   ! # of ice grid points
   INTEGER ::   n_l, nx_l, ny_l   ! # of lnd grid points
   INTEGER ::   n_o, nx_o, ny_o   ! # of ocn grid points

   INTEGER ::   n_r, nx_r, ny_r   ! # of roff grid points
   INTEGER ::   n_rc              ! # of roff grid points in compressed form

   !----- output state buffers dimensions -------------------------------
   INTEGER,PARAMETER ::   nSa = 9 ! # of atm state fields
   INTEGER,PARAMETER ::   nSi = 7 ! # of ice state fields
   INTEGER,PARAMETER ::   nSl = 6 ! # of lnd state fields
   INTEGER,PARAMETER ::   nSo = 8 ! # of ocn state fields

   !----- input state buffer dimensions ---------------------------------
   INTEGER,PARAMETER ::   nSs = 6 ! number of merged surface state fields

   !----- input flux buffers dimensions ---------------------------------
   INTEGER,PARAMETER :: nFaxx = 7 ! # of atm input flux fields
   INTEGER,PARAMETER :: nFlxx = 0 ! # of lnd input flux fields - NONE
   INTEGER,PARAMETER :: nFixx = 0 ! # of ice input flux fields - NONE
   INTEGER,PARAMETER :: nFoxx =16 ! # of ocn input flux fields

   !----- flux component buffers dimensions -----------------------------
   INTEGER,PARAMETER :: nFaxa = 1 ! # of a/x flux fields computed by atm

   INTEGER,PARAMETER :: nFaia = 7 ! # of a/i flux fields computed by atm
   INTEGER,PARAMETER :: nFaii = 8 ! # of a/i flux fields computed by ice
   INTEGER,PARAMETER :: nFaic = 0 ! # of a/i flux fields computed by cpl

   INTEGER,PARAMETER :: nFala = 9 ! # of a/l flux fields computed by atm
   INTEGER,PARAMETER :: nFall = 8 ! # of a/l flux fields computed by lnd
   INTEGER,PARAMETER :: nFalc = 0 ! # of a/l flux fields computed by cpl - NONE

   INTEGER,PARAMETER :: nFaoa = 7 ! # of a/o flux fields computed by atm
   INTEGER,PARAMETER :: nFaoo = 0 ! # of a/o flux fields computed by ocn - NONE
   INTEGER,PARAMETER :: nFaoc = 8 ! # of a/o flux fields computed by cpl

   INTEGER,PARAMETER :: nFioi = 6 ! # of i/o flux fields computed by ice
   INTEGER,PARAMETER :: nFioo = 3 ! # of i/o flux fields computed by ocn
   INTEGER,PARAMETER :: nFioc = 0 ! # of i/o flux fields computed by cpl

!

!===============================================================================
! Sign convention:
!    positive value <=> downward flux
! Unit convention:
!    T             ~ Kelvin
!    u,v           ~ m/s
!    pressure      ~ N/m^2 = Pa
!    heat flux     ~ W/m^2 
!    momentum flux ~ N/m^2
!    water flux    ~ (kg/s)/m^2
!===============================================================================

   !----------------------------------------------------------------------------
   ! atm/srf flux buffer indicies
   !----------------------------------------------------------------------------

   !----- fluxes computed by atm ----
   INTEGER,PARAMETER :: kFaxa_swnet =  1 ! shortwave absorbed by srf

   !----------------------------------------------------------------------------
   ! atm/ice flux buffer indicies
   !----------------------------------------------------------------------------

   !----- computed by cpl -----
   ! Note: empty set -- there are no such fluxes
          
   !----- fluxes computed by atm ----
   INTEGER,PARAMETER :: kFaia_lwdn  =  1 ! downward longwave heat flux
   INTEGER,PARAMETER :: kFaia_rain  =  2 ! precip: liquid
   INTEGER,PARAMETER :: kFaia_snow  =  3 ! precip: frozen
   INTEGER,PARAMETER :: kFaia_swndr =  4 ! shortwave: nir direct  downward
   INTEGER,PARAMETER :: kFaia_swvdr =  5 ! shortwave: vis direct  downward
   INTEGER,PARAMETER :: kFaia_swndf =  6 ! shortwave: nir diffuse downward
   INTEGER,PARAMETER :: kFaia_swvdf =  7 ! shortwave: vis diffuse downward

   !----- computed by ice -----
   INTEGER,PARAMETER :: kFaii_taux  =  1 ! wind stress, zonal
   INTEGER,PARAMETER :: kFaii_tauy  =  2 ! wind stress, meridional
   INTEGER,PARAMETER :: kFaii_lat   =  3 ! latent          heat flux
   INTEGER,PARAMETER :: kFaii_sen   =  4 ! sensible        heat flux
   INTEGER,PARAMETER :: kFaii_lwup  =  5 ! upward longwave heat flux
   INTEGER,PARAMETER :: kFaii_evap  =  6 ! evaporation    water flux
   INTEGER,PARAMETER :: kFaii_tref  =  7 ! 2m reference temperature
   INTEGER,PARAMETER :: kFaii_swnet =  8 ! shortwave: net absorbed
!
   !----------------------------------------------------------------------------
   ! atm/lnd flux buffer indicies
   !----------------------------------------------------------------------------

   !----- computed by cpl -----
   ! Note: empty set -- there are no such fluxes

   !----- computed by atm -----
   INTEGER,PARAMETER :: kFala_lwdn  =  1 ! downward longwave heat flux
   INTEGER,PARAMETER :: kFala_rainc =  2 ! precip: liquid, convective  
   INTEGER,PARAMETER :: kFala_rainl =  3 ! precip: liquid, large-scale
   INTEGER,PARAMETER :: kFala_snowc =  4 ! precip: frozen, convective
   INTEGER,PARAMETER :: kFala_snowl =  5 ! precip: frozen, large-scale
   INTEGER,PARAMETER :: kFala_swndr =  6 ! shortwave: nir direct  downward
   INTEGER,PARAMETER :: kFala_swvdr =  7 ! shortwave: vis direct  downward
   INTEGER,PARAMETER :: kFala_swndf =  8 ! shortwave: nir diffuse downward
   INTEGER,PARAMETER :: kFala_swvdf =  9 ! shortwave: vis diffuse downward

   !----- computed by lnd -----
   INTEGER,PARAMETER :: kFall_taux  =  1 ! wind stress, zonal
   INTEGER,PARAMETER :: kFall_tauy  =  2 ! wind stress, meridional
   INTEGER,PARAMETER :: kFall_lat   =  3 ! latent          heat flux
   INTEGER,PARAMETER :: kFall_sen   =  4 ! sensible        heat flux
   INTEGER,PARAMETER :: kFall_lwup  =  5 ! upward longwave heat flux
   INTEGER,PARAMETER :: kFall_evap  =  6 ! evaporation    water flux
   INTEGER,PARAMETER :: kFall_tref  =  7 ! 2m reference temperature
   INTEGER,PARAMETER :: kFall_swnet =  8 ! 2m reference temperature

   !----------------------------------------------------------------------------
   ! atm/ocn flux buffer indicies
   !----------------------------------------------------------------------------

   !----- computed by cpl -----
   INTEGER,PARAMETER :: kFaoc_taux  =  1 ! wind stress, zonal
   INTEGER,PARAMETER :: kFaoc_tauy  =  2 ! wind stress, meridional
   INTEGER,PARAMETER :: kFaoc_lat   =  3 ! latent          heat flux
   INTEGER,PARAMETER :: kFaoc_sen   =  4 ! sensible        heat flux
   INTEGER,PARAMETER :: kFaoc_lwup  =  5 ! upward longwave heat flux
   INTEGER,PARAMETER :: kFaoc_evap  =  6 ! evaporation    water flux
   INTEGER,PARAMETER :: kFaoc_tref  =  7 ! 2m reference temperature
   INTEGER,PARAMETER :: kFaoc_swnet =  8 ! shortwave: net  heat flux
          
   !----- computed by atm -----
   INTEGER,PARAMETER :: kFaoa_swndr =  1 ! shortwave: nir direct  downward
   INTEGER,PARAMETER :: kFaoa_swvdr =  2 ! shortwave: vis direct  downward
   INTEGER,PARAMETER :: kFaoa_swndf =  3 ! shortwave: nir diffuse downward
   INTEGER,PARAMETER :: kFaoa_swvdf =  4 ! shortwave: vis diffuse downward
   INTEGER,PARAMETER :: kFaoa_lwdn  =  5 ! downward longwave heat flux
   INTEGER,PARAMETER :: kFaoa_rain  =  6 ! water flux: rain
   INTEGER,PARAMETER :: kFaoa_snow  =  7 ! water flux: snow

   !----- computed by ocn -----
   ! Note: empty set -- there are no such fluxes
!
   !----------------------------------------------------------------------------
   ! ice/ocn flux buffer indicies
   !----------------------------------------------------------------------------

   !----- computed by cpl -----
   ! Note: empty set -- there are no such fluxes

   !----- computed by ice -----
   INTEGER,PARAMETER :: kFioi_swnet =  1 ! shortwave: net penetrating ice
   INTEGER,PARAMETER :: kFioi_melth =  2 ! heat  flux from melting ice
   INTEGER,PARAMETER :: kFioi_meltw =  3 ! water flux from melting ice
   INTEGER,PARAMETER :: kFioi_salt  =  4 ! salt  flux from melting ice
   INTEGER,PARAMETER :: kFioi_taux  =  5 ! ice/ocn stress, zonal
   INTEGER,PARAMETER :: kFioi_tauy  =  6 ! ice/ocn stress, meridional

   !----- computed by ocn -----
   INTEGER,PARAMETER :: kFioo_dhdx  =  1 ! surface slope, zonal
   INTEGER,PARAMETER :: kFioo_dhdy  =  2 ! surface slope, meridional
   INTEGER,PARAMETER :: kFioo_q     =  3 ! heat of fusion (q>0), melt pot (q<0)

   !----------------------------------------------------------------------------
   ! model state buffer indicies
   !----------------------------------------------------------------------------

   !----- atm states -----
   INTEGER,PARAMETER :: kSa_z       =  1 ! bottom atm level height
   INTEGER,PARAMETER :: kSa_u       =  2 ! bottom atm level wind, zonal
   INTEGER,PARAMETER :: kSa_v       =  3 ! bottom atm level wind, meridional
   INTEGER,PARAMETER :: kSa_tbot    =  4 ! bottom atm level temperature
   INTEGER,PARAMETER :: kSa_ptem    =  5 ! bottom atm level potential temp
   INTEGER,PARAMETER :: kSa_shum    =  6 ! bottom atm level specific humidity
   INTEGER,PARAMETER :: kSa_dens    =  7 ! bottom atm level air density
   INTEGER,PARAMETER :: kSa_pbot    =  8 ! bottom atm level pressure
   INTEGER,PARAMETER :: kSa_pslv    =  9 ! sea level atm pressure

   !----- ice states -----
   INTEGER,PARAMETER :: kSi_t       =  1 ! temperature
   INTEGER,PARAMETER :: kSi_ifrac   =  2 ! fractional ice coverage
   INTEGER,PARAMETER :: kSi_avsdr   =  3 ! albedo: visible      , direct
   INTEGER,PARAMETER :: kSi_anidr   =  4 ! albedo: near infrared, direct
   INTEGER,PARAMETER :: kSi_avsdf   =  5 ! albedo: visible      , diffuse
   INTEGER,PARAMETER :: kSi_anidf   =  6 ! albedo: near infrared, diffuse
   INTEGER,PARAMETER :: kSi_index   =  7 ! global data compression index

   !----- lnd states -----
   INTEGER,PARAMETER :: kSl_t       =  1 ! temperature
   INTEGER,PARAMETER :: kSl_avsdr   =  2 ! albedo: direct , visible
   INTEGER,PARAMETER :: kSl_anidr   =  3 ! albedo: direct , near-infrared
   INTEGER,PARAMETER :: kSl_avsdf   =  4 ! albedo: diffuse, visible
   INTEGER,PARAMETER :: kSl_anidf   =  5 ! albedo: diffuse, near-infrared
   INTEGER,PARAMETER :: kSl_snowh   =  6 ! snow height

   !----- ocn states -----
   INTEGER,PARAMETER :: kSo_u       =  1 ! velocity, zonal
   INTEGER,PARAMETER :: kSo_v       =  2 ! velocity, meridional
   INTEGER,PARAMETER :: kSo_t       =  3 ! temperature
   INTEGER,PARAMETER :: kSo_s       =  4 ! salinity
   INTEGER,PARAMETER :: kSo_avsdr   =  5 ! albedo: visible      , direct
   INTEGER,PARAMETER :: kSo_anidr   =  6 ! albedo: near infrared, direct
   INTEGER,PARAMETER :: kSo_avsdf   =  7 ! albedo: visible      , diffuse
   INTEGER,PARAMETER :: kSo_anidf   =  8 ! albedo: near infrared, diffuse

   !----- merged lnd/ice/ocn surface states -----
   INTEGER,PARAMETER :: kSs_avsdr   =  1 ! albedo: visible      , direct
   INTEGER,PARAMETER :: kSs_anidr   =  2 ! albedo: near infrared, direct
   INTEGER,PARAMETER :: kSs_avsdf   =  3 ! albedo: visible      , diffuse
   INTEGER,PARAMETER :: kSs_anidf   =  4 ! albedo: near infrared, diffuse
   INTEGER,PARAMETER :: kSs_t       =  5 ! temperature
   INTEGER,PARAMETER :: kSs_snowh   =  6 ! snow depth over land
!
   !----------------------------------------------------------------------------
   ! model input flux buffer indicies
   !----------------------------------------------------------------------------

   !----- atm model input -----
   INTEGER,PARAMETER :: kFaxx_taux  =  1 ! wind stress, zonal
   INTEGER,PARAMETER :: kFaxx_tauy  =  2 ! wind stress, meridional
   INTEGER,PARAMETER :: kFaxx_lat   =  3 ! latent          heat flux
   INTEGER,PARAMETER :: kFaxx_sen   =  4 ! sensible        heat flux
   INTEGER,PARAMETER :: kFaxx_lwup  =  5 ! upward longwave heat flux
   INTEGER,PARAMETER :: kFaxx_evap  =  6 ! evaporation    water flux
   INTEGER,PARAMETER :: kFaxx_tref  =  7 ! 2m reference temperature

   !----- lnd model input -----
   ! Note: empty set -- there are no such fluxes

   !----- ice model input -----
   ! Note: empty set -- there are no such fluxes

   !----- ocn model input -----
   INTEGER,PARAMETER :: kFoxx_taux  =  1 ! wind stress: zonal
   INTEGER,PARAMETER :: kFoxx_tauy  =  2 ! wind stress: meridional
   INTEGER,PARAMETER :: kFoxx_swnet =  3 ! heat flux: shortwave net
   INTEGER,PARAMETER :: kFoxx_lat   =  4 ! heat flux: latent
   INTEGER,PARAMETER :: kFoxx_sen   =  5 ! heat flux: sensible
   INTEGER,PARAMETER :: kFoxx_lwup  =  6 ! heat flux: long-wave up
   INTEGER,PARAMETER :: kFoxx_lwdn  =  7 ! heat flux: long-wave down
   INTEGER,PARAMETER :: kFoxx_melth =  8 ! heat flux: melt
   INTEGER,PARAMETER :: kFoxx_salt  =  9 ! salt flux
   INTEGER,PARAMETER :: kFoxx_rain  = 10 ! water flux: rain
   INTEGER,PARAMETER :: kFoxx_snow  = 11 ! water flux: snow
   INTEGER,PARAMETER :: kFoxx_evap  = 12 ! water flux: evaporation
   INTEGER,PARAMETER :: kFoxx_meltw = 13 ! water flux: melt
   INTEGER,PARAMETER :: kFoxx_roff  = 14 ! water flux: runoff
   INTEGER,PARAMETER :: kFoxx_ifrac = 15 ! state: ice fraction
   INTEGER,PARAMETER :: kFoxx_pslv  = 16 ! state: atm sea level pressure

!
!===============================================================================
! module routines
!===============================================================================

   CONTAINS

SUBROUTINE buf_alloc()

   IMPLICIT none

   CHARACTER(LEN=*),PARAMETER :: F00 = "('(buf_alloc) ',3a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   allocate and initialize memory for all data in module buf_data
!-------------------------------------------------------------------------------

   write(6,F00) 'allocating memory for field buffers'

   !----------------------------------------------------------------------------
   ! information buffers passed to & from models
   !----------------------------------------------------------------------------
   allocate( ibuff_c(nibuff) )  ! integer info-buffer from cpl
   allocate( rbuff_c(nrbuff) )  ! real    info-buffer from cpl
   allocate( ibuff_a(nibuff) )  ! integer info-buffer from atm
   allocate( ibuff_i(nibuff) )  ! integer info-buffer from ice
   allocate( ibuff_l(nibuff) )  ! integer info-buffer from lnd
   allocate( ibuff_o(nibuff) )  ! integer info-buffer from ocn

   ibuff_c = 0
   rbuff_c = 0
   ibuff_a = 0
   ibuff_i = 0
   ibuff_l = 0
   ibuff_o = 0
   !----------------------------------------------------------------------------
   ! domain information
   !----------------------------------------------------------------------------
   allocate( mask_a(  n_a) )  ! atm domain mask
   allocate(   xc_a(  n_a) )  ! atm x-coords, grid centers
   allocate(   yc_a(  n_a) )  ! atm y-coords, grid centers
   allocate(   xv_a(4,n_a) )  ! atm x-coords, grid verticies
   allocate(   yv_a(4,n_a) )  ! atm y-coords, grid verticies
   allocate( area_a(  n_a) )  ! atm grid cell area

   allocate( mask_i(  n_i) )  ! ice domain mask
   allocate(   xc_i(  n_i) )  ! ice x-coords, grid centers
   allocate(   yc_i(  n_i) )  ! ice y-coords, grid centers
   allocate(   xv_i(4,n_i) )  ! ice x-coords, grid verticies
   allocate(   yv_i(4,n_i) )  ! ice y-coords, grid verticies
   allocate( area_i(  n_i) )  ! ice grid cell area

   allocate( mask_l(  n_l) )  ! lnd domain mask
   allocate(   xc_l(  n_l) )  ! lnd x-coords, grid centers
   allocate(   yc_l(  n_l) )  ! lnd y-coords, grid centers
   allocate(   xv_l(4,n_l) )  ! lnd x-coords, grid verticies
   allocate(   yv_l(4,n_l) )  ! lnd y-coords, grid verticies
   allocate( area_l(  n_l) )  ! lnd grid cell area

   allocate( mask_o(  n_o) )  ! ocn domain mask
   allocate(   xc_o(  n_o) )  ! ocn x-coords, grid centers
   allocate(   yc_o(  n_o) )  ! ocn y-coords, grid centers
   allocate(   xv_o(4,n_o) )  ! ocn x-coords, grid verticies
   allocate(   yv_o(4,n_o) )  ! ocn y-coords, grid verticies
   allocate( area_o(  n_o) )  ! ocn grid cell area

   allocate( mask_r(  n_r) )  ! ocn domain mask
   allocate(   xc_r(  n_r) )  ! ocn x-coords, grid centers
   allocate(   yc_r(  n_r) )  ! ocn y-coords, grid centers
   allocate(   xv_r(4,n_r) )  ! ocn x-coords, grid verticies
   allocate(   yv_r(4,n_r) )  ! ocn y-coords, grid verticies
   allocate( area_r(  n_r) )  ! ocn grid cell area

   mask_a = 0
   xc_a = 0.0
   yc_a = 0.0
   xv_a = 0.0
   yv_a = 0.0

   mask_i = 0
   xc_i = 0.0
   yc_i = 0.0
   xv_i = 0.0
   yv_i = 0.0

   mask_l = 0
   xc_l = 0.0
   yc_l = 0.0
   xv_l = 0.0
   yv_l = 0.0

   mask_o = 0
   xc_o = 0.0
   yc_o = 0.0
   xv_o = 0.0
   yv_o = 0.0

   mask_r = 0
   xc_r = 0.0
   yc_r = 0.0
   xv_r = 0.0
   yv_r = 0.0

   !----------------------------------------------------------------------------
   ! model state buffers 
   !----------------------------------------------------------------------------
   if (nSa   > 0) then
     allocate(    Sa_a(n_a,nSa  ) ) ! atm states on atm grid
     allocate(    Sa_i(n_i,nSa  ) ) ! atm states on ice grid
     allocate(    Sa_l(n_l,nSa  ) ) ! atm states on lnd grid
     allocate(    Sa_o(n_o,nSa  ) ) ! atm states on ocn grid
     Sa_a = 0.0
     Sa_i = 0.0
     Sa_l = 0.0
     Sa_o = 0.0
   endif

   if (nSi   > 0) then
     allocate(    Si_a(n_a,nSi  ) ) ! ice states on atm grid
     allocate(    Si_i(n_i,nSi  ) ) ! ice states on ice grid
     allocate(    Si_o(n_o,nSi  ) ) ! ice states on ocn grid
     Si_a = 0.0
     Si_i = 0.0
     Si_o = 0.0
   endif

   if (nSl   > 0) then
     allocate(    Sl_a(n_a,nSl  ) ) ! lnd states on atm grid 
     allocate(    Sl_l(n_l,nSl  ) ) ! lnd states on lnd grid   
     Sl_a = 0.0
     Sl_l = 0.0
   endif

   if (nSo   > 0) then
     allocate(    So_a(n_a,nSo  ) ) ! ocn states on atm grid
     allocate(    So_i(n_i,nSo  ) ) ! ocn states on ice grid
     allocate(    So_o(n_o,nSo  ) ) ! ocn states on ocn grid
     So_a = 0.0
     So_i = 0.0
     So_o = 0.0
   endif

   !----------------------------------------------------------------------------
   ! merged input state buffers  
   !----------------------------------------------------------------------------
   if (nSs   > 0) then
     allocate(    Ss_a(n_a,nSs  ) ) ! merged surface states
     Ss_a = 0.0
   endif

   !----------------------------------------------------------------------------
   ! merged input flux buffers 
   !----------------------------------------------------------------------------
   if (nFaxx > 0) then
     allocate(  Faxx_a(n_a,nFaxx) ) ! merged atm input flux
     Faxx_a = 0.0
   endif
   if (nFlxx > 0) then
     allocate(  Flxx_l(n_l,nFlxx) ) ! merged lnd input flux
     Flxx_l = 0.0
   endif
   if (nFixx > 0) then
     allocate(  Fixx_i(n_i,nFixx) ) ! merged lnd input flux
     Fixx_i = 0.0
   endif
   if (nFoxx > 0) then
     allocate(  Foxx_o(n_o,nFoxx) ) ! merged ocn input flux
     allocate( aFoxx_o(n_o,nFoxx) ) ! merged ocn input flux (t-avg)
      Foxx_o = 0.0
     aFoxx_o = 0.0
   endif

   !----------------------------------------------------------------------------
   ! component flux buffers 
   !----------------------------------------------------------------------------
   if (nFaxa > 0) then
     allocate(  Faxa_a(n_a,nFaxa) ) ! a/x flux, from atm, atm grid
     Faxa_a = 0.0
   endif
   if (nFaia > 0) then
     allocate(  Faia_a(n_a,nFaia) ) ! a/i flux, from atm, atm grid
     allocate(  Faia_i(n_i,nFaia) ) ! a/i flux, from atm, ice grid
     Faia_a = 0.0
     Faia_i = 0.0
   endif
   if (nFaii > 0) then
     allocate(  Faii_a(n_a,nFaii) ) ! a/i flux, from ice, atm grid
     allocate(  Faii_i(n_i,nFaii) ) ! a/i flux, from ice, ice grid
     Faii_a = 0.0
     Faii_i = 0.0
   endif
   if (nFaic > 0) then
     allocate(  Faic_a(n_a,nFaic) ) ! a/i flux, from cpl, atm grid
     allocate(  Faic_i(n_i,nFaic) ) ! a/i flux, from cpl, ice grid
     Faic_a = 0.0
     Faic_i = 0.0
   endif

   if (nFala > 0) then
     allocate(  Fala_a(n_a,nFala) ) ! a/l flux, from atm, atm grid
     allocate(  Fala_l(n_l,nFala) ) ! a/l flux, from atm, lnd grid
     Fala_a = 0.0
     Fala_l = 0.0
   endif
   if (nFall > 0) then
     allocate(  Fall_a(n_a,nFall) ) ! a/l flux, from lnd, atm grid
     allocate(  Fall_l(n_l,nFall) ) ! a/l flux, from lnd, lnd grid
     Fall_a = 0.0
     Fall_l = 0.0
   endif
   if (nFalc > 0) then
     allocate(  Falc_a(n_a,nFalc) ) ! a/l flux, from cpl, atm grid
     allocate(  Falc_l(n_l,nFalc) ) ! a/l flux, from cpl, lnd grid
     Falc_a = 0.0
     Falc_l = 0.0
   endif

   if (nFaoa > 0) then
     allocate(  Faoa_a(n_a,nFaoa) ) ! a/o flux, from atm, atm grid
     allocate(  Faoa_o(n_o,nFaoa) ) ! a/o flux, from atm, ocn grid
     Faoa_a = 0.0
     Faoa_o = 0.0
   endif
   if (nFaoo > 0) then
     allocate(  Faoo_a(n_a,nFaoo) ) ! a/o flux, from ocn, atm grid
     allocate(  Faoo_o(n_o,nFaoo) ) ! a/o flux, from ocn, ocn grid
     Faoo_a = 0.0
     Faoo_o = 0.0
   endif
   if (nFaoc > 0) then
     allocate(  Faoc_a(n_a,nFaoc) ) ! a/o flux, from cpl, atm grid
     allocate(  Faoc_o(n_o,nFaoc) ) ! a/o flux, from cpl, ocn grid
     Faoc_a = 0.0
     Faoc_o = 0.0
   endif

   if (nFioi > 0) then
     allocate(  Fioi_i(n_i,nFioi) ) ! i/o flux, from ice, ice grid
     allocate(  Fioi_o(n_o,nFioi) ) ! i/o flux, from ice, ocn grid
     Fioi_i = 0.0
     Fioi_o = 0.0
   endif
   if (nFioo > 0) then
     allocate(  Fioo_i(n_i,nFioo) ) ! i/o flux, from ocn, ice grid
     allocate(  Fioo_o(n_o,nFioo) ) ! i/o flux, from ocn, ocn grid
     Fioo_i = 0.0
     Fioo_o = 0.0
   endif
   if (nFioc > 0) then
     allocate(  Fioc_i(n_i,nFioc) ) ! i/o flux, from cpl, ice grid
     allocate(  Fioc_o(n_o,nFioc) ) ! i/o flux, from cpl, ocn grid
     Fioc_i = 0.0
     Fioc_o = 0.0
   endif

   !----- lnd/ocn flux components -----
   allocate( roff_rc(n_rc,1) )       ! lnd runoff vector, compressed 2d array
   allocate( roff_o (n_o ,1) )       ! lnd runoff, ocn grid

END SUBROUTINE buf_alloc

!===============================================================================

END MODULE buf_mod
