!===============================================================================
! CVS $Id: main.F90,v 1.27 2001/09/18 21:03:00 tcraig Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/main.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

PROGRAM main 

!===============================================================================
! 
!    The Community Climate System Model (CCSM) Copyright and Release Notice
! 
!              +------------------------------------------+
!              | DISTRIBUTION TERMS AND CONDITIONS NOTICE |
!              +------------------------------------------+
! 
! 
! 
! CCSM was developed in cooperation with the National Science Foundation,
! the Department of Energy Los Alamos National Laboratory (LANL), the
! National Aeronautics and Space Administration Data Assimilation Office,
! and the University Corporation for Atmospheric Research's National
! Center for Atmospheric Research.*  Except for POP, SCRIP, and CICE,
! which are segregable components of the model, CCSM is public domain
! software.  
! 
! POP, SCRIP, and CICE were developed at LANL and are protected by the
! following copyright notice. 
! 
! Copyright (c)2002 The Regents of the University of California. All Rights
! Reserved. 
! 
! Permission to use, copy, modify, and distribute this software and its
! documentation for educational, research and non-profit purposes, without
! fee, and without a written agreement is hereby granted, provided that
! the above copyright notice, this paragraph and the following three
! paragraphs appear in all copies. 
! 
! Permission to incorporate this software into commercial products may be
! obtained by contacting the University of California, Charles Rzeszutko,
! Campus Liaison Officer Office of Technology Transfer, 1111 Franklin
! Street, 5th Floor, Oakland, CA 94607, (510) 587-6063,
! charles.rzeszutko@ucop.edu.
! 
! THIS SOFTWARE PROGRAM AND DOCUMENTATION ARE COPYRIGHTED BY 
! THE REGENTS OF THE UNIVERSITY OF CALIFORNIA. THE SOFTWARE 
! PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS", WITHOUT ANY 
! ACCOMPANYING SERVICES FROM THE REGENTS. THE REGENTS DOES NOT 
! WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE 
! UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS THAT 
! THE PROGRAM WAS DEVELOPED FOR RESEARCH PURPOSES AND IS 
! ADVISED NOT TO RELY EXCLUSIVELY ON THE PROGRAM FOR ANY 
! REASON. 
! 
! IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY 
! PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
! CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF 
! THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE 
! UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF 
! SUCH DAMAGE.  THE UNIVERSITY OF CALIFORNIA SPECIFICALLY 
! DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
! IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
! PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN 
! "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO 
! OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, 
! ENHANCEMENTS, OR MODIFICATIONS. 
! 
! *The National Center for Atmospheric Research is sponsored by the
! National Science Foundation.  Any opinions, findings and conclusions or
! recommendations expressed in this publication are those of the author(s)
! and do not necessarily reflect the views of the National Science
! Foundation.
!===============================================================================


   USE map_mod        ! mapping data
   USE buf_mod        ! model   data
   USE control_mod    ! control data
   USE calendar_mod   ! calendar routines
   USE histavg_mod    ! time-avg history data file routines
   USE shr_msg_mod    ! shared code: msg passing
   USE shr_sys_mod    ! shared code: system calls
   USE msg_atm_mod    ! msg passing module ~ atm specific
   USE msg_ice_mod    ! msg passing module ~ ice specific
   USE msg_lnd_mod    ! msg passing module ~ lnd specific
   USE msg_ocn_mod    ! msg passing module ~ ocn specific
   USE diag_mod       ! diagnostics module

   IMPLICIT none

   !----- local -----
   integer           :: nsFoxx       ! number of Foxx   tavg samples
   real   ,pointer   :: sFoxx_o(:,:) ! array to forming Foxx   tavg's
   real   ,pointer   :: junk1 (:)    ! temporary/dummy/discard arrays
   real   ,pointer   :: junk2 (:)    ! for unwanted model IC states
   real   ,pointer   :: junk3 (:)
   real   ,pointer   :: junk4 (:)
   real   ,pointer   :: junk5 (:)
   integer           :: n            ! inner time-stepping loop index
   character(len= 8) :: dstr         ! wall clock date string yyyymmdd
   character(len=10) :: tstr         ! wall clock time string hhmmss.sss

   integer    :: t00,t01,t02,t03,t04,t05,t06,t07,t08,t09 
   integer    :: t10,t11,t12,t13,t14,t15,t16,t17,t18,t19
   integer    :: t20,t21,t22,t23,t24,t25,t26,t27,t28,t29
   integer    :: s00,s01,s02,s03,s04,s05,s06,s07,s08,s09 
   integer    :: s10,s11,s12,s13,s14,s15,s16,s17,s18,s19 
   integer    :: s20,s21,s22,s23,s24,s25,s26,s27,s28,s29 

   !----- i/o formats -----
   character(len=*), parameter :: F00 = "('(main) ',16a)"
   character(len=*), parameter :: F01 = "('(main) ',a,i4)"
   character(len=*), parameter :: F02 = "(16a)"
   character(len=*), parameter :: F06 = "(80('='))"
   character(len=*), parameter :: F07 = "(80('=')/)"
   character(len=*), parameter :: F08 = "(80('-'))"
   character(len=*), parameter :: F09 = "(80('-')/)"
   character(len=*), parameter :: F10 = &
   &   "('(main) wall clock:',1x,a4,'-',a2,'-',a2,1x,a2,':',a2,':',a2)"
   character(len=*), parameter :: F12 = &
   &   "(      'date & time:',1x,a4,'-',a2,'-',a2,1x,a2,':',a2,':',a2)"
   character(len=*), parameter :: F99 = &
   & "('NCAR CSM Flux Coupler, (C) 2001 UCAR, All Rights Reserved')"

!-------------------------------------------------------------------------------
! PURPOSE:
!   This is the CSM Flux Coupler component which coordinates the 
!   flux calculations & time integration of all component models: 
!   o cpl - this flux coupler
!   o atm - the atmosphere model
!   o ice - the sea ice model
!   o lnd - the land surface model
!   o ocn - the global ocn model
!-------------------------------------------------------------------------------

   call shr_msg_stdio('cpl') ! redirect stdin/stdout

   call date_and_time(dstr,tstr)
   write(6,F06)
   write(6,F00) 'Executing CSM Flux Coupler'
   write(6,F12) dstr(1:4),dstr(5:6),dstr(7:8) ,tstr(1:2),tstr(3:4),tstr(5:6)
   write(6,F00) 'CVS tag $Name: ccsm2_0_beta46 $'
   write(6,F06)

   call shr_timer_init()
   call shr_timer_get(t00,'t00 - startup overhead')
   call shr_timer_get(t01,'t01')
   call shr_timer_get(t02,'t02')
   call shr_timer_get(t03,'t03')
   call shr_timer_get(t04,'t04')
   call shr_timer_get(t05,'t05')
   call shr_timer_get(t06,'t06')
   call shr_timer_get(t07,'t07')
   call shr_timer_get(t08,'t08')
   call shr_timer_get(t09,'t09')
   call shr_timer_get(t10,'t10')
   call shr_timer_get(t11,'t11')
   call shr_timer_get(t12,'t12')
   call shr_timer_get(t13,'t13')
   call shr_timer_get(t14,'t14')
   call shr_timer_get(t15,'t15')
   call shr_timer_get(t16,'t16')
   call shr_timer_get(t17,'t17')
   call shr_timer_get(t18,'t18')
   call shr_timer_get(t19,'t19')
   call shr_timer_get(t20,'t20')
   call shr_timer_get(t21,'t21')
   call shr_timer_get(t22,'t22')
   call shr_timer_get(t23,'t23')
   call shr_timer_get(t24,'t24')
   call shr_timer_get(t25,'t25')
   call shr_timer_get(t26,'t26')
   call shr_timer_get(t27,'t27')
   call shr_timer_get(t28,'t28')
   call shr_timer_get(s01,'s01')
   call shr_timer_get(s02,'s02')
   call shr_timer_get(s03,'s03')
   call shr_timer_get(s11,'s11')
   call shr_timer_get(s12,'s12')
   call shr_timer_get(s13,'s13')
   call shr_timer_get(s21,'s21')
   call shr_timer_get(s22,'s22')
   call shr_timer_get(s23,'s23')

   call shr_timer_start(t00)

   !----------------------------------------------------------------------------
   ! initialize
   !----------------------------------------------------------------------------
   call msg_cplc ()             ! connect to msg passing environment
   call initial ()              ! initialize & input namelist & related
   call buf_alloc ()            ! allocate model msg buffers
   allocate(sFoxx_o(n_o,nFoxx)) ! alloc sFoxx    (wrt ocn daily-avgs)
   call restart ('read')        ! get restart data (input or fabricate)
   call control()               ! set control flags: stop, rest, hist, etc
   call map_init ()             ! input/initialize mapping data

   !----------------------------------------------------------------------------
   ! exchange time invariant msgs with component models
   !----------------------------------------------------------------------------
   call msg_atmi ()
   call msg_icei ()
   call msg_ocni ()
   call msg_lndi ()

   !----------------------------------------------------------------------------
   ! verify communication intervals are acceptable
   !----------------------------------------------------------------------------
   write(6,F01) 'number of atm communications/day = ',ncpl_a
   write(6,F01) 'number of lnd communications/day = ',ncpl_l
   write(6,F01) 'number of ice communications/day = ',ncpl_i
   write(6,F01) 'number of ocn communications/day = ',ncpl_o
   if (ncpl_a /= ncpl_l) then
      write(6,F02) 'ERROR: communication interval ncpl_a /= ncpl_l'
      call shr_sys_abort("(main) comm interval mis-match")
   else if (ncpl_a /= ncpl_i) then
      write(6,F02) 'ERROR: communication interval ncpl_a /= ncpl_i '
      call shr_sys_abort("(main) comm interval mis-match")
   else if (ncpl_a < ncpl_o) then
      write(6,F02) 'ERROR: communication interval ncpl_a < ncpl_o '
      call shr_sys_abort("(main) comm interval mis-match")
   else if (mod(ncpl_a,ncpl_o) /= 0) then
      write(6,F02) 'ERROR: communication interval mod(ncpl_a,ncpl_o) /= 0 '
      call shr_sys_abort("(main) comm interval mis-match")
   end if
   call shr_sys_flush(6)

   !----------------------------------------------------------------------------
   ! optional initialization: cpl tells lnd what domain spec to use
   !----------------------------------------------------------------------------
   if ( init_maskl.ne.0 ) then
      write(6,F00) 'send lnd domain spec to lnd (optional, requested by lnd)'
      call shr_sys_flush(6)
      xc_l   = xc_a            ! coordinates & cell area same as for atm
      yc_l   = yc_a
      xv_l   = xv_a
      yv_l   = yv_a
      area_l = area_a 
      mask_l = domain_l%mask   ! lnd mask as determined by cpl (wrt map data)
      allocate(junk1(n_i))
      junk1   = 0.0            ! use bogus ice fraction = zero to ...
      call map_frac(junk1)     ! ... compute all surface fractions (esp. fl_l)
      deallocate(junk1)
      call msg_lndm()          ! send this domain spec to lnd
   end if

   !----------------------------------------------------------------------------
   ! receive IC data from models (& decide whether to use it or not)
   !----------------------------------------------------------------------------
   if (rest_type(1:4) .eq. 'init') then
      !--- initial run => use model's IC data ---
      write(6,F00) 'using IC data from models'
      call msg_icer ( Si_i , Fioi_i , Faii_i)
      call msg_ocnr ( So_o , Fioo_o )
      call msg_lndr ( Sl_l , Fall_l , roff_rc)
      call msg_atmr ( Sa_a , Faxa_a , Faia_a , Fala_a , Faoa_a)
   else
      !--- continue run => use model or cpl IC data? ---
      if (init_rica .ne. 0) then
         write(6,F00) 'using atm IC data provided by atm model'
         call msg_atmr (Sa_a , Faxa_a , Faia_a , Fala_a , Faoa_a)
      else           
         write(6,F00) 'using atm IC data from cpl restart file'
         allocate(junk1(n_a*nSa  ),junk2(n_a*nFaxa),junk3(n_a*nFaia))
         allocate(junk4(n_a*nFala),junk5(n_a*nFaoa)                 )
         call msg_atmr (junk1,junk2,junk3,junk4,junk5) 
         deallocate    (junk1,junk2,junk3,junk4,junk5)
      end if
      if (init_rici .ne. 0) then
         write(6,F00) 'using ice IC data provided by ice model'
         call msg_icer ( Si_i ,  Fioi_i, Faii_i )
         call flux_albi(Si_i)
         call map( map_i2af, Faii_i, Faii_a ,fi_i,fi_a)
      else           
         write(6,F00) 'using ice IC data from cpl restart file'
         allocate   (junk1(n_i*nSi),junk2(n_i*nFioi),junk3(n_i*nFaii))
         call msg_icer (junk1,junk2,junk3)
         deallocate    (junk1,junk2,junk3)
      end if
      if (init_ricl .ne. 0) then
         write(6,F00) 'using lnd IC data provided by lnd model'
         call msg_lndr ( Sl_l ,  Fall_l ,  roff_rc)
      else           
         write(6,F00) 'using lnd IC data from cpl restart file'
         allocate   (junk1(n_l*nSl),junk2(n_l*nFall),junk3(n_rc))
         call msg_lndr (junk1,junk2,junk3)
         deallocate    (junk1,junk2,junk3)
      end if
      if (init_rico .ne. 0) then
         write(6,F00) 'using ocn IC data provided by ocn model'
         call msg_ocnr ( So_o ,  Fioo_o )
         call flux_albo(So_o)
      else           
         write(6,F00) 'using ocn IC data from cpl restart file'
         allocate   (junk1(n_o*nSo),junk2(n_o*nFioo))
         call msg_ocnr (junk1,junk2)
         deallocate    (junk1,junk2)
      end if
   end if
   call shr_sys_flush(6)

   !----------------------------------------------------------------------------
   ! consistency check of model domains & mapping data
   !----------------------------------------------------------------------------
   call map_frac (Si_i(:,kSi_ifrac))  ! compute surface fractions
   call map_check()
   call shr_sys_flush(6)

   !----------------------------------------------------------------------------
   ! optional initialization: recompute atm solar radiation wrt surface albedo
   !----------------------------------------------------------------------------
   if (init_solar.ne.0 .and. stop_now.eq.0) then
      write(6,F00) 're-initialize atm solar (optional, requested by atm)'
      call shr_sys_flush(6)

      !--- prepare atm input *states* & send to atm model ---
      call map( map_i2as , Si_i, Si_a ,fi_i,fi_a)
      call map( map_o2as , So_o, So_a ,fa_o,fo_a)
      call map( map_l2as , Sl_l, Sl_a )
      call mrgs_atm (  Si_a ,  Sl_a ,  So_a ,  Ss_a)
      call msg_atms (Faxx_a ,  Ss_a )  ! -> only Ss_a is valid <-

      !--- receive atm outputs ---
      call msg_atmr (Sa_a ,Faxa_a, Faia_a ,Fala_a ,Faoa_a)
   end if

   !----------------------------------------------------------------------------
   ! check time coordination of IC data
   !----------------------------------------------------------------------------
   if (init_tavg.eq.0) then
      !--- in this case, all model should be coordinated in time ---
      write(6,F08)
      write(6,F01) ' Start of time coordinated integration'
      write(6,F08)
      call tcheck(1)  ! 1 => enforce time coordination
   else
      !--- in this case, ocn is initially ahead of other models ---
      write(6,F08)
      write(6,F01) ' Integrate to form initial tavg ocn input flux'
      write(6,F08)
      call tcheck(0)  ! 0 <=> don't enforce time coordination
   end if
   call shr_sys_flush(6)

   !----------------------------------------------------------------------------
   ! recreate data needed for 1st iteration of main event loop
   !----------------------------------------------------------------------------

   !--- need So_i, Fioo_i for 1st ice-send ---
   call map( map_o2is,   So_o,   So_i )
   call map( map_o2if, Fioo_o, Fioo_i )

   !--- need Fioi_o for 1st ocn mrg ---
   call map( map_i2of, Fioi_i, Fioi_o )

   !--- need Ss_a for 1st diagnostic call ---
   call map( map_i2as, Si_i, Si_a ,fi_i,fi_a)
   call map( map_l2as, Sl_l, Sl_a )
   call map( map_o2as, So_o, So_a ,fa_o,fo_a)
   call mrgs_atm ( Si_a, Sl_a, So_a, Ss_a)

   !--- need Faxx_a for 1st diagnostic call ---
   call map( map_l2af, Fall_l, Fall_a )
   call map( map_i2af, Faii_i, Faii_a ,fi_i, fi_a)
   call map( map_o2af, Faoc_o, Faoc_a ,fa_o, fo_a)
   call mrgf_atm (Faii_a, Fall_a ,Faoc_a ,Faxx_a)

   call shr_timer_stop (t00)
   call shr_timer_print(t00)
   call shr_timer_free (t00)
   call shr_timer_get  (t00,'t00 - main integration loop')
   call shr_timer_start(t00)

   !----------------------------------------------------------------------------
   ! start of model integration
   !----------------------------------------------------------------------------
   DO WHILE ( stop_now .eq. 0 )

      DO n=0,ncpl_a-1  ! advance one day

         call shr_timer_start(t01)

         !--- record wall clock time ---
         if (day.eq.1 .and. sec.eq.0) then
            call date_and_time(dstr,tstr)
            write(6,F10) dstr(1:4),dstr(5:6),dstr(7:8) &
            &           ,tstr(1:2),tstr(3:4),tstr(5:6)
         end if
         call shr_timer_stop (t01) ; call shr_timer_start(t02)

         !--- full precision result test ---
         call bitcheck()
         call shr_timer_stop (t02) ; call shr_timer_start(t03)

         !--- send msg to ocean ---
         if (mod(n,ncpl_a/ncpl_o) == 0 ) then
            call shr_timer_start(s11)
            if (init_tavg == 0) call msg_ocns (aFoxx_o)
            call shr_timer_stop (s11) ; call shr_timer_start(s12)

            !--- re-initialize ocn tavg data ---
            call t_avg('zero',n_o*nFoxx,Foxx_o(1,1),sFoxx_o(1,1),nsFoxx  )
            call shr_timer_stop (s12)
         endif
         call shr_timer_stop (t03) ; call shr_timer_start(t04)

         !--- map atm states & fluxes to lnd ---
         call map( map_a2ls,   Sa_a,   Sa_l )
         call map( map_a2lf, Fala_a, Fala_l )
         call shr_timer_stop (t04) ; call shr_timer_start(t05)

!lnds    !--- send data to lnd model ---
         call msg_lnds (Fala_l , Sa_l   )
         call shr_timer_stop (t05) ; call shr_timer_start(t06)

         !--- compute and map ice inputs ----
         call map( map_a2is,   Sa_a,   Sa_i ) ; call map_npfix(Sa_a,Sa_i)
         call map( map_a2if, Faia_a, Faia_i )
         call shr_timer_stop (t06) ; call shr_timer_start(t07)

         !--- force zero net water flux into ocn+ice ? ----
         if (flx_epbal(1:3) /= 'off') then
             call map( map_rc2o,  roff_rc, roff_o )
             call map( map_a2of,   Faoa_a, Faoa_o )
             call map( map_a2os,     Sa_a,   Sa_o ) ; call map_npfix(Sa_a,Sa_o)
             call flux_epbal(Faia_i,Faii_i,Faoa_o,Faoc_o,roff_o,fa_o,fi_o)
         endif
         call shr_timer_stop (t07) ; call shr_timer_start(t08)

!ices    !--- send msg to ice (tavg data) ---
         call msg_ices (Sa_i, So_i, Faia_i, Fioo_i)
         call shr_timer_stop (t08) ; call shr_timer_start(t09)

         !--- map atm & lnd output fluxes to ocn ---
         if (flx_epbal(1:3) == 'off') then
             call map( map_rc2o,  roff_rc, roff_o )
             call map( map_a2of,   Faoa_a, Faoa_o )
             call map( map_a2os,     Sa_a,   Sa_o ) ; call map_npfix(Sa_a,Sa_o)
         endif
         call flux_solar(Faoa_o, So_o, Faoc_o )
         call shr_timer_stop (t09) ; call shr_timer_start(t10)

!ocns    !--- merge/"send" ocn inputs ---
         call mrgf_ocn ( Faoa_o, Faoc_o, Fioi_o, roff_o, Sa_o, fi_o, Foxx_o)
         call shr_timer_stop (t10) ; call shr_timer_start(t11)

         !--- form tavg partial sum of ocn inputs ---
         call t_avg('sum',n_o*nFoxx,Foxx_o(1,1),sFoxx_o(1,1),nsFoxx  )
         call shr_timer_stop (t11) ; call shr_timer_start(t12)

         !--- do diagnostics ---
         if (init_tavg == 0 .or. n /= 0) call diagnos ()
         call shr_timer_stop (t12) ; call shr_timer_start(t13)

!ocnr    !--- "recv" new ocn data ---
         call flux_albo(So_o) ! compute ocn albedos
         call shr_timer_stop (t13) ; call shr_timer_start(t14)

         !--- compute cpl's atm/ocn fluxes ---
         call flux_ao( Sa_o, So_o, Faoc_o )
         call shr_timer_stop (t14) ; call shr_timer_start(t15)

!icer    !--- recv new ice data ---
         call msg_icer (Si_i ,Fioi_i ,Faii_i)
         call shr_timer_stop (t15) ; call shr_timer_start(t16)
         call flux_albi(Si_i) ! modify ice albedos?

         !--- update surface fracs wrt new ice frac ---
         call map_frac (Si_i(:,kSi_ifrac))
         call shr_timer_stop (t16) ; call shr_timer_start(t17)

         !--- map states/fluxes to atm & ocn grid ---
         call map( map_i2af, Faii_i, Faii_a ,fi_i, fi_a)
         call map( map_i2as,   Si_i,   Si_a ,fi_i, fi_a)
         call map( map_i2of, Fioi_i, Fioi_o )
         call shr_timer_stop (t17) ; call shr_timer_start(t18)

         !--- map ocn states/fluxes to atm ---
         call map( map_o2af, Faoc_o, Faoc_a, fa_o, fo_a)
         call map( map_o2as,   So_o,   So_a, fa_o, fo_a)
         call shr_timer_stop (t18) ; call shr_timer_start(t19)

!lndr    !--- recv new lnd data ---
         call msg_lndr (Sl_l, Fall_l, roff_rc )
         call shr_timer_stop (t19) ; call shr_timer_start(t20)

         !--- verify net solar calcs are coordinated ---
         call diag_solar ()
         call shr_timer_stop (t20) ; call shr_timer_start(t21)

         !--- map lnd states & fluxes to atm ---
         call map( map_l2af, Fall_l, Fall_a )
         call map( map_l2as,   Sl_l,   Sl_a )
         call shr_timer_stop (t21) ; call shr_timer_start(t22)

         !--- merge atm states & fluxes ---
         call mrgf_atm (Faii_a, Fall_a, Faoc_a, Faxx_a)
         call mrgs_atm (  Si_a,   Sl_a,   So_a,   Ss_a)
         call shr_timer_stop (t22) ; call shr_timer_start(t23)
       
!atms    !--- send data to atm ---
         call msg_atms (Faxx_a, Ss_a )
         call shr_timer_stop (t23) ; call shr_timer_start(t24)

         !--- add data to history file ---
         call history ()
         call histavg ()
         call shr_timer_stop (t24) ; call shr_timer_start(t25)

         if (mod(n+1,ncpl_a/ncpl_o) == 0 ) then
            call shr_timer_start(s21)
            !--- form tavg of ocn inputs ---
            call t_avg('avg',n_o*nFoxx,sFoxx_o(1,1),aFoxx_o(1,1),nsFoxx)
 
            call shr_timer_stop (s21) ; call shr_timer_start(s22)
            !--- recv msg from ocn ---
            if (init_tavg == 0) call msg_ocnr(So_o, Fioo_o)
            call shr_timer_stop (s22) ; call shr_timer_start(s23)
           
            !--- map ocn to ice ---
            call map( map_o2is,   So_o,   So_i )
            call map( map_o2if, Fioo_o, Fioo_i )

            call shr_timer_stop (s23) 

            !--- start normal interaction with ocn ---
            if (init_tavg /= 0) then
               write(6,F08)
               write(6,F01) ' Start of time coordinated integration'
               write(6,F08)
               init_tavg=0   ! done forming initial tavg ocn forcing
            end if
         end if
         call shr_timer_stop (t25) ; call shr_timer_start(t26)
 
!atmr    !--- recv msg from atm ---
         call msg_atmr (Sa_a, Faxa_a, Faia_a, Fala_a, Faoa_a)
         call shr_timer_stop (t26) ; call shr_timer_start(t27)

         !--- advanced model time ---
         sec = sec + 86400/ncpl_a
         if (sec.ge.86400) then
            sec = 0       ! it's a new day!
            eday = eday + 1  
            call eday2date(eday,cdate)
            call eday2ymd (eday,year,month,day)
         end if
         call control() ! reset stop, rest, hist control flags

         call shr_timer_stop (t27)
      END DO

      call tcheck(1)   ! 1 => enforce time coordination

      !--- make a cpl restart file ---
      call shr_timer_start(t28)
      call restart ('write')
      call shr_timer_stop (t28) 

      call shr_sys_flush(6)
   END DO

   !--- wall-clock time-stamp ---
   write(6,F10) dstr(1:4),dstr(5:6),dstr(7:8) ,tstr(1:2),tstr(3:4),tstr(5:6)

   !----------------------------------------------------------------------------
   ! send final msg to models, dis-connect msg-passing, shutdown
   !----------------------------------------------------------------------------
   call shr_timer_stop (t00)
   call shr_timer_print_all()
   call shr_sys_flush(6)
   call history  ()  ! last chance to form/save/archive data 
   call histavg  ()  ! last chance to form/save/archive data 
   call msg_atmf ()
   call msg_lndf ()
   call msg_icef ()
   call msg_ocnf ()
   call msg_cpld ()

 999 CONTINUE

   call date_and_time(dstr,tstr)
   write(6,F06)
   write(6,F00) 'Terminating CSM coupler'
   write(6,F12) dstr(1:4),dstr(5:6),dstr(7:8) ,tstr(1:2),tstr(3:4),tstr(5:6)
   write(6,F07)
   call shr_sys_flush(6)
STOP
END

!===============================================================================
