!===============================================================================
! CVS $Id: map_check.F90,v 1.6 2001/12/07 22:13:35 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/map_check.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

SUBROUTINE map_check()

   USE buf_mod       ! data exchanged with models
   USE map_mod       ! data for mapping between domains
   use const_mod     ! provides physical constants

   implicit none

   !--- local ---
   integer         :: i,j,k,n      ! generic indicies
   integer         :: n0,n1,n2,n3  ! counts rows that add up to 0,1,>1
   integer         :: n_s          ! size of mapping matrix
   real   ,pointer :: s  (:)       ! mapping matrix
   integer,pointer :: row(:)       ! rows corresponding to s elements
   integer,pointer :: col(:)       ! cols corresponding to s elements
   integer,pointer :: sn1(:)       ! the column of the matrix elements
   integer         :: ncol         ! number of non-zero elements in one row of s
   integer         :: mincol       ! min number of non-zero elements in any row
   integer         :: maxcol       ! max number of non-zero elements in any row
   real            :: maxsum       ! max sum of any one row
   real            :: sum          ! sum    of  the     elements in one row of s
   real            :: suma         ! sum of atm area
   real            :: suml         ! sum of lnd area
   real            :: sumi         ! sum of ice area
   real            :: sumo         ! sum of ocn area
   integer,pointer :: mask(:)      ! domain mask
   real   ,pointer :: area(:)      ! domain area
   real   ,pointer :: xc  (:)      ! x-coords
   real   ,pointer :: yc  (:)      ! y-coords
   real ,parameter :: eps = 1.0e-9 ! epsilon

   !--- formats ---
   character(len=*),parameter :: F00 = "('(map_check) ',4a         )"
   character(len=*),parameter :: F01 = "('(map_check) ',a,i7, es14.6,2i7 )"
   character(len=*),parameter :: F02 = "('(map_check) ',a,i6,2(a,f15.12) )"
   character(len=*),parameter :: F03 = "('(map_check) ',a,f8.3,a )"
   character(len=*),parameter :: F05 = "('(map_check) ',a,i6,a,i6,a )"
   character(len=*),parameter :: F06 = "('(map_check) ',a,i6,a,i6,a,es14.6)"

!-------------------------------------------------------------------------------
! PURPOSE:
! o check for errors in mapping data
!-------------------------------------------------------------------------------

   if ( map_rc2o%n /= 0) then
     !--------------------------------------------------------------------------
     write(6,F00) 'checking rc2o matrix...'
     !--------------------------------------------------------------------------
     n_s  =  map_rc2o%n
     s    => map_rc2o%s
     row  => map_rc2o%row
     col  => map_rc2o%col
     sn1  => map_rc2o%sn1
     mask => domain_o%mask

     !--- examine sum of matrix rows ---
     n0=0       ! number of rows that add up to zero
     n1=0       ! number of rows that add up to one
     n2=0       ! number of rows that add up to more than one
     n3=0       ! number of other rows 
     maxcol=0   ! max number of non-zero elements in any one row
     mincol=n_s ! min number of non-zero elements in any one row
     maxsum=0.0 ! max sum of any one row
     do i=1,domain_o%n
        !----------------------------------
        sum=0.0
        ncol=0
        do n=sn1(i),sn1(i+1)-1
           if ( row(n) == i) then
              ncol = ncol + 1
              sum  = sum  + s(n)
           end if
        enddo
        !----------------------------------
        if ( mask(i) /= 0 ) then
           maxcol = max(maxcol,ncol)
           mincol = min(mincol,ncol)
           maxsum = max(maxsum, sum)
        end if
        !----------------------------------
        if      ( abs(sum)     < eps ) then  ! this row adds up to 0.0
           n0=n0+1 
        else if ( abs(1.0-sum) < eps ) then  ! this row adds up to 1.0
           n1=n1+1 
        else if (   sum        > 1.0 ) then  ! this row adds up to > 1.0
           n2=n2+1 
        else if (   sum        > 0.0 ) then  ! this row adds up to in (0,1)
           n3=n3+1 
        else 
           write(6,F02) '  WARNING: row =',i,', mask /= 0, sum of row = ',sum
        endif
        !----------------------------------
     enddo
     write(6,F03) '  ',100.0*n0/domain_o%n, '% of matix rows add up to 0.0'
     write(6,F03) '  ',100.0*n1/domain_o%n, '% of matix rows add up to 1.0'
     write(6,F03) '  ',100.0*n2/domain_o%n, '% of matix rows add up to > 1.0'
     write(6,F03) '  ',100.0*n3/domain_o%n, '% of matix rows add up to in (0,1)'
     write(6,F05) '  ', maxcol     , ' = max non-zero elements per row'
     write(6,F05) '  ', mincol     , ' = min non-zero elements per row'
     write(6,F03) '  max sum of any one row = ', maxsum

     call shr_sys_flush(6)

   endif
!
   if ( map_a2os%n /= 0) then
     !--------------------------------------------------------------------------
     write(6,F00) 'checking a2os matrix...'
     !--------------------------------------------------------------------------
     n_s  =  map_a2os%n
     s    => map_a2os%s
     row  => map_a2os%row
     col  => map_a2os%col
     sn1  => map_a2os%sn1
     mask => domain_o%mask

     !--- examine individual matrix elements ---
     do n=1,n_s
       if (s(n) > 1.0 .or. s(n) < 0.0 ) then
          write(6,F01) '  ERROR: n,s(n),row(n),col(n) = ',n,s(n),row(n),col(n)
       end if
     end do

     !--- examine sum of matrix rows ---
     j=0        ! number of rows that add up to one
     maxcol=0   ! max number of non-zero elements in any one row
     mincol=n_a ! min number of non-zero elements in any one row
     do i=1,n_o
        sum=0.0
        ncol=0
        do n=sn1(i),sn1(i+1)-1
           if ( row(n) == i) then
             ncol = ncol + 1
             sum = sum + s(n)
           end if
        enddo
        if ( mask(i) /= 0 ) then
           maxcol = max(maxcol,ncol)
           mincol = min(mincol,ncol)
           if ( abs(1.0-sum) < eps ) then
              j=j+1 ! this row adds up to 1.0 (as it should)
           else 
             if ( sum /= 0.0 ) write(6,F02) &
             & '  ERROR: row =',i,', mask /= 0, yet sum of row = ',sum
           endif
        else 
           if ( sum /= 0.0 ) write(6,F02) &
           &  '  FYI: row =',i,', mask == 0, yet sum of row = ',sum
        endif
     enddo
     write(6,F03) '  ',100.0*j/n_o, '% of matrix rows add up to 1.0'
     write(6,F05) '  ',maxcol     , ' = max non-zero elements per row'
     write(6,F05) '  ',mincol     , ' = min non-zero elements per row'
   endif
!
   if (map_a2of%n /= 0) then
     !--------------------------------------------------------------------------
     write(6,F00) 'checking a2of matrix...'
     !--------------------------------------------------------------------------
     n_s  =  map_a2of%n
     s    => map_a2of%s
     row  => map_a2of%row
     col  => map_a2of%col
     sn1  => map_a2of%sn1
     mask => domain_o%mask

     !--- examine individual matrix elements ---
     do n=1,n_s
       if (s(n) > 1.0 .or. s(n) < 0.0 ) then
          write(6,F01) '  ERROR: n,s(n),row(n),col(n) = ',n,s(n),row(n),col(n)
       end if
     end do

     !--- examine sum of matrix rows ---
     j=0        ! number of rows that add up to one
     maxcol=0   ! max number of non-zero elements in any one row
     mincol=n_a ! min number of non-zero elements in any one row
     do i=1,n_o
        sum=0.0
        ncol=0
        do n=sn1(i),sn1(i+1)-1
           if ( row(n) == i) then
             ncol = ncol + 1
             sum = sum + s(n)
           end if
        enddo
        if ( mask(i) /= 0 ) then
           maxcol = max(maxcol,ncol)
           mincol = min(mincol,ncol)
           if ( abs(1.0-sum) < eps ) then
              j=j+1 ! this row adds up to 1.0 (as it should)
           else 
             if ( sum /= 0.0 ) write(6,F02) &
             & '  ERROR: row =',i,', mask /= 0, yet sum of row = ',sum
           endif
        else 
           if ( sum /= 0.0 ) write(6,F02) &
           &  '  FYI: row =',i,', mask == 0, yet sum of row = ',sum
        endif
     enddo
     write(6,F03) '  ',100.0*j/n_o, '% of matrix rows add up to 1.0'
     write(6,F05) '  ',maxcol     , ' = max non-zero elements per row'
     write(6,F05) '  ',mincol     , ' = min non-zero elements per row'
   endif
!
   if (map_o2af%n /= 0) then
     !--------------------------------------------------------------------------
     write(6,F00) 'checking o2af matrix...'
     !--------------------------------------------------------------------------
     n_s  =  map_o2af%n
     s    => map_o2af%s
     row  => map_o2af%row
     col  => map_o2af%col
     sn1  => map_o2af%sn1
     mask => domain_a%mask
     area => domain_a%area
     xc   => domain_a%xc
     yc   => domain_a%yc

     !--- examine individual matrix elements ---
     do n=1,n_s
       if (s(n) > 1.0 .or. s(n) < 0.0 ) then
          write(6,F01) '  ERROR: n,s(n),row(n),col(n) = ',n,s(n),row(n),col(n)
       end if
     end do

     !--- examine sum of matrix rows ---
     j=0        ! number of rows that add up to one
     k=0        ! number of rows that add up to zero
     maxcol=0   ! max number of non-zero elements in any one row
     mincol=n_o ! min number of non-zero elements in any one row
     do i=1,n_a
       sum=0.0
       ncol=0
       do n=sn1(i),sn1(i+1)-1
          if ( row(n) == i) then
             ncol = ncol + 1
             sum = sum + s(n)
          end if
       enddo
       maxcol = max(maxcol,ncol)
       mincol = min(mincol,ncol)
       if ( abs(1.0-sum) < eps ) j = j+1
       if ( abs(    sum) < eps ) k = k+1
       if ( sum < 0.0     ) write(6,F02) 'ERROR: matrix row ',i,', sum =',sum
       if ( sum > 1.0+eps ) write(6,F02) 'ERROR: matrix row ',i,', sum =',sum
       if ( abs(xc(i)-180.0)<10.0 .and. abs(yc(i))< 10.0) then
         if ( abs(sum-1.0) > eps ) then
           write(6,F02) 'NOTE: in central Pacific, row = ',i,'sum of row = ',sum
         endif
       endif
     enddo
     write(6,F03) '  ', 100.0*k/n_a, '% of matix rows add up to 0.0'
     write(6,F03) '  ', 100.0*j/n_a, '% of matix rows add up to 1.0'
     write(6,F05) '  ', maxcol     , ' = max non-zero elements per row'
     write(6,F05) '  ', mincol     , ' = min non-zero elements per row'

     !--- compare sum of rows vs. surface fractions ---
     do i=1,n_a
       sum=0.0
       do n=sn1(i),sn1(i+1)-1
          if ( row(n) == i) sum = sum + s(n)
       enddo
       if (      fl_a(i) == 0.0  .and.  abs(1.0-sum) > eps ) then
          write(6,F02) '  NOTE: row = ',i,', fl_a = 0.0, sum of row =',sum
       else if ( fl_a(i) == 1.0  .and.  abs(    sum) > eps ) then
          write(6,F02) '  NOTE: row = ',i,', fl_a = 0.0, sum of row =',sum
       endif
     enddo
   endif
!
   !----------------------------------------------------------------------------
   write(6,F00) 'checking cell areas...'
   !----------------------------------------------------------------------------
   area => domain_a%area
   suma = 0.0
   suml = 0.0
   sumi = 0.0
   sumo = 0.0
   k=0
   j=0
   do n=1,n_a
     if ( fl_a(n) == 1.0 ) j = j+1 ! atm cell with all lnd
     if ( fl_a(n) == 0.0 ) k = k+1 ! atm cell with no  lnd
     suma = suma + area(n)
     suml = suml + area(n)*fl_a(n)
     sumi = sumi + area(n)*fi_a(n)
     sumo = sumo + area(n)*fo_a(n)
   end do
   write(6,F03) '* atm grid:',100.0*suma/(4*pi),' % of global area is atm'
   write(6,F03) '  atm grid:',100.0*suml/(4*pi),' % of global area is lnd'
   write(6,F03) '  atm grid:',100.0*sumi/(4*pi),' % of global area is sea ice'
   write(6,F03) '  atm grid:',100.0*sumo/(4*pi),' % of global area is open ocn'
   write(6,F03) '  atm grid:',100.0 *  j/n_a   ,' % of cells have fl_a = 1.0'
   write(6,F03) '  atm grid:',100.0 *  k/n_a   ,' % of cells have fl_a = 0.0'

   area => domain_l%area
   mask => domain_l%mask
   suml = 0.0
   do n=1,n_l
     if (mask(n) /= 0) suml = suml + area(n)*fl_l(n)
   end do
   write(6,F03) '* lnd grid:',100.0*suml/(4*pi),' % of globe is lnd'

   area => domain_i%area
   mask => domain_i%mask
   sumi = 0.0
   suma = 0.0
   do n=1,n_i
     if (mask(n) /= 0) sumi = sumi + area(n)*fi_i(n)
     if (mask(n) /= 0) suma = suma + area(n)
   end do
   write(6,F03) '* ice grid:',100*sumi/(4*pi),' % of global area is ice'
   write(6,F03) '  ice grid:',100*suma/(4*pi),' % of global area is ice domain'

   area => domain_o%area
   mask => domain_o%mask
   sumi = 0.0
   suma = 0.0
   sumo = 0.0
   do n=1,n_o
     if (mask(n) /= 0) sumi = sumi + area(n)*fi_o(n)
     if (mask(n) /= 0) suma = suma + area(n)*fa_o(n)
     if (mask(n) /= 0) sumo = sumo + area(n)
   end do
   write(6,F03)'* ocn grid:',100*suma/(4*pi),' % of global area is atm over ocn'
   write(6,F03)'  ocn grid:',100*sumi/(4*pi),' % of global area is ice over ocn'
   write(6,F03)'  ocn grid:',100*sumo/(4*pi),' % of global area is ocn '

   !----------------------------------------------------------------------------
   write(6,F00) 'checking fl_l vs. mask_l'
   !----------------------------------------------------------------------------
   i=0 ; j=0
   do n=1,n_l
     if ( mask_l(n)==0 .and. fl_l(n)> 0.0 ) i=i+1
     if ( mask_l(n)/=0 .and. fl_l(n)==0.0 ) j=j+1
   enddo
   if (i>0) write(6,F05)'  ERROR: fl_l > 0, mask_l==0 in ',i,' cells, n_l =',n_l
   if (j>0) write(6,F05)'  NOTE : fl_l = 0, mask_l/=0 in ',j,' cells, n_l =',n_l

   !----------------------------------------------------------------------------
   write(6,F00) 'checking sum of fi_a + fl_a + fo_a'
   !----------------------------------------------------------------------------
   do n=1,n_a
     if ( abs(1.0-fi_a(n)-fl_a(n)-fo_a(n)) > .001 ) then
       write(6,F02) 'at n =',n,' fi_a + fl_a + fo_a = ',fi_a(n)+fl_a(n)+fo_a(n)
     end if
   enddo
 
   !----------------------------------------------------------------------------
   write(6,F00) 'checking sum of fi_o + fa_o'
   !----------------------------------------------------------------------------
   mask => domain_o%mask
   do n=1,n_o
     if ( mask(n)/=0 .and. abs(1.0-fi_o(n)-fa_o(n)) > .001 ) then
       write(6,F02) 'ocn grid: n = ',n,', fi_o + fa_o = ',fi_o(n)+fa_o(n)
     endif
   enddo
!
   !----------------------------------------------------------------------------
   write(6,F00) 'checking map masks vs. buf masks'
   !----------------------------------------------------------------------------
   mask => domain_o%mask
   i=0 ; j=0 ; k=0
   do n=1,n_o
     if ( mask_o(n)       ==   mask(n)    ) i=i+1
     if ( mask_o(n)/=0.0 .and. mask(n)==0 ) j=j+1
     if ( mask_o(n)==0.0 .and. mask(n)/=0 ) k=k+1
   enddo
   write(6,F03) '  ocn grid: in',100.*i/n_o,' % of cells, the two masks agree'
   if ( i /= n_o ) then
     write(6,F03)'ocn grid:',100.*j/n_o,' % of cells, only map   grid is active'
     write(6,F03)'ocn grid:',100.*k/n_o,' % of cells, only model grid is active'
   endif

   mask => domain_i%mask
   i=0 ; j=0 ; k=0
   do n=1,n_i
     if ( mask_i(n)       ==   mask(n)    ) i=i+1
     if ( mask_i(n)/=0.0 .and. mask(n)==0 ) j=j+1
     if ( mask_i(n)==0.0 .and. mask(n)/=0 ) k=k+1
   enddo
   write(6,F03)'  ice grid: in',100.*i/n_i,' % of cells, the two masks agree'
   if ( i /= n_i ) then
     write(6,F03)'ice grid:',100.*j/n_i,' % of cells, only map   grid is active'
     write(6,F03)'ice grid:',100.*k/n_i,' % of cells, only model grid is active'
   endif

   mask => domain_l%mask
   i=0 ; j=0 ; k=0
   do n=1,n_l
     if ( mask_l(n)       ==   mask(n)    ) i=i+1
     if ( mask_l(n)/=0.0 .and. mask(n)==0 ) j=j+1
     if ( mask_l(n)==0.0 .and. mask(n)/=0 ) k=k+1
   enddo
   write(6,F03)'  lnd grid: in',100.*i/n_l,' % of cells, the two masks agree'
   if ( i /= n_l ) then
     write(6,F03)'lnd grid:',100.*j/n_l,' % of cells, only map   grid is active'
     write(6,F03)'lnd grid:',100.*k/n_l,' % of cells, only model grid is active'
   endif

END SUBROUTINE map_check

!===============================================================================


