!===============================================================================
! CVS $Id: mrgf_atm.F90,v 1.1 2000/01/03 22:56:20 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/mrgf_atm.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

SUBROUTINE mrgf_atm(Faii,Fall,Faoc,Faxx)

   USE buf_mod
   USE map_mod

   implicit none

   !----- arguments -----
   real,intent(in)  :: Faii(n_a,nFaii) ! atm/ice fluxes computed by ice
   real,intent(in)  :: Fall(n_a,nFall) ! atm/lnd fluxes computed by atm
   real,intent(in)  :: Faoc(n_a,nFaoc) ! atm/ocn fluxes computed by cpl
   real,intent(out) :: Faxx(n_a,nFaxx) ! merged atm/surface fluxes together

   !----- local -----
   integer          :: ki,kl,ko,kx     ! indicies to buffer fields

!-------------------------------------------------------------------------------
! PURPOSE:
!   gather various flux component fields (all on the atm grid) and
!   merge these atm/surface component fluxes (via geographical weights),
!   forming the complete set of atm input fluxes
!-------------------------------------------------------------------------------

   !--- merge taux -------------------
   ki=kFaii_taux
   kl=kFall_taux
   ko=kFaoc_taux
   kx=kFaxx_taux
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)

   !--- merge tauy -------------------
   ki=kFaii_tauy
   kl=kFall_tauy
   ko=kFaoc_tauy
   kx=kFaxx_tauy
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)

   !--- merge latent heat flux -------
   ki=kFaii_lat
   kl=kFall_lat
   ko=kFaoc_lat
   kx=kFaxx_lat
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)

   !--- merge sensible heat flux -----
   ki=kFaii_sen
   kl=kFall_sen
   ko=kFaoc_sen
   kx=kFaxx_sen
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)

   !--- merge longwave upward flux ---
   ki=kFaii_lwup
   kl=kFall_lwup
   ko=kFaoc_lwup
   kx=kFaxx_lwup
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)

   !--- merge evaporative flux -------
   ki=kFaii_evap
   kl=kFall_evap
   ko=kFaoc_evap
   kx=kFaxx_evap
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)

   !--- merge 2m reference temp-------
   ki=kFaii_tref
   kl=kFall_tref
   ko=kFaoc_tref
   kx=kFaxx_tref
   Faxx(:,kx) = fi_a*Faii(:,ki) + fl_a*Fall(:,kl) + fo_a*Faoc(:,ko)

END

!===============================================================================
