!===============================================================================
! CVS $Id: mrgf_ocn.F90,v 1.7 2001/10/12 17:33:12 tcraig Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/mrgf_ocn.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

SUBROUTINE mrgf_ocn(Faoa,Faoc,Fioi,roff, Sa, fi, Foxx)

   USE buf_mod   ! contains field buffers & buffer indicies
   USE map_mod   ! contains cell fractions

   implicit none

   !----- arguments -----
   real,intent(in)  :: Faoa(n_o,nFaoa)  ! (in) atm/ocn flux, computed by atm
   real,intent(in)  :: Faoc(n_o,nFaoc)  ! (in) atm/ocn flux, computed by cpl
   real,intent(in)  :: Fioi(n_o,nFioi)  ! (in) ice/ocn flux, computed by ice
   real,intent(in)  :: roff(n_o,1)      ! (in) lnd/ocn flux, computed by lnd
   real,intent(in)  :: Sa  (n_o,nSa)    ! (in) atm state
   real,intent(in)  :: fi  (n_o)        ! (in) fraction of ice
   real,intent(out) :: Foxx(n_o,nFoxx)  ! (out) merged ocn/x flux: sent to ocn

   !----- local variables -----
   integer :: n                         ! counter
   real    :: sfa_o, sfi_o              ! scalars of array values

!-------------------------------------------------------------------------------
! PURPOSE:
!   gather various flux component fields (all on the ocn grid),
!   and then merge these ocn/comp component fluxes 
!   forming the complete set of ocn input fluxes
! NOTE:
!   The presence of Sa and fi is somewhat of a hack, sending such data was not
!   in the original requirements/design of cpl5.   Adding them to the list of
!   time avg'd input to the ocn was most easily accomplished by adding them to
!   the list of ocn-input fluxes.
!-------------------------------------------------------------------------------

!$OMP PARALLEL DO PRIVATE(n,sfa_o,sfi_o)
   do n=1,n_o
   sfa_o = fa_o(n)
   sfi_o = fi_o(n)
   !--- merge tau ---------------------------------------------------
   Foxx(n,kFoxx_taux ) = sfa_o*Faoc(n,kFaoc_taux ) + sfi_o*Fioi(n,kFioi_taux)
   Foxx(n,kFoxx_tauy ) = sfa_o*Faoc(n,kFaoc_tauy ) + sfi_o*Fioi(n,kFioi_tauy)

   !--- merge heat flux ---------------------------------------------
   Foxx(n,kFoxx_swnet) = sfa_o*Faoc(n,kFaoc_swnet) + sfi_o*Fioi(n,kFioi_swnet)
   Foxx(n,kFoxx_lat  ) = sfa_o*Faoc(n,kFaoc_lat )
   Foxx(n,kFoxx_sen  ) = sfa_o*Faoc(n,kFaoc_sen )
   Foxx(n,kFoxx_lwup ) = sfa_o*Faoc(n,kFaoc_lwup)
   Foxx(n,kFoxx_lwdn ) = sfa_o*Faoa(n,kFaoa_lwdn)
   Foxx(n,kFoxx_melth) = sfi_o*Fioi(n,kFioi_melth)

   !--- merge water flux --------------------------------------------
   Foxx(n,kFoxx_rain ) = sfa_o*Faoa(n,kFaoa_rain)
   Foxx(n,kFoxx_snow ) = sfa_o*Faoa(n,kFaoa_snow)
   Foxx(n,kFoxx_evap ) = sfa_o*Faoc(n,kFaoc_evap)
   Foxx(n,kFoxx_meltw) = sfi_o*Fioi(n,kFioi_meltw)
   Foxx(n,kFoxx_roff ) =      roff(n,1)

   !--- merge salt flux --------------------------------------------
   Foxx(n,kFoxx_salt ) = sfi_o*Fioi(n,kFioi_salt)

   !--- merge input states -----------------------------------------
   Foxx(n,kFoxx_pslv ) = Sa(n,kSa_pslv)
   Foxx(n,kFoxx_ifrac) = fi(n)
   enddo

END

!===============================================================================
