!===============================================================================
! CVS $Id: mrgs_atm.F90,v 1.1 2000/01/03 22:56:20 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/mrgs_atm.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

SUBROUTINE mrgs_atm(Si,Sl,So, Ss)

   USE buf_mod
   USE map_mod

   IMPLICIT none

   !----- arguments----
   real,intent(in)  :: Si(n_a,nSi)   ! ice state
   real,intent(in)  :: Sl(n_a,nSl)   ! lnd state
   real,intent(in)  :: So(n_a,nSo)   ! ocn state
   real,intent(out) :: Ss(n_a,nSs)   ! merged surface states

   !----- local -----
   integer          :: ki,kl,ko,kx   ! indicies to buffer fields

!-------------------------------------------------------------------------------
! PURPOSE:
!   merge surface states, forming the complete set of atm input states
!   (eg. surface temperature &  surface albedo)
!-------------------------------------------------------------------------------
   
   !--- 3-way merge of T ----------------------
   ki=kSi_t
   kl=kSl_t
   ko=kSo_t
   kx=kSs_t
   Ss(:,kx) = fi_a*Si(:,ki) + fl_a*Sl(:,kl) + fo_a*So(:,ko)

   !--- 3-way merge of visible/direct albedo ---
   ki=kSi_avsdr
   kl=kSl_avsdr
   ko=kSo_avsdr
   kx=kSs_avsdr
   Ss(:,kx) = fi_a*Si(:,ki) + fl_a*Sl(:,kl) + fo_a*So(:,ko)

   !--- 3-way merge of near-infrared/direct albedo ---
   ki=kSi_anidr
   kl=kSl_anidr
   ko=kSo_anidr
   kx=kSs_anidr
   Ss(:,kx) = fi_a*Si(:,ki) + fl_a*Sl(:,kl) + fo_a*So(:,ko)

   !--- 3-way merge of visible/diffuse albedo ---
   ki=kSi_avsdf
   kl=kSl_avsdf
   ko=kSo_avsdf
   kx=kSs_avsdf
   Ss(:,kx) = fi_a*Si(:,ki) + fl_a*Sl(:,kl) + fo_a*So(:,ko)

   !--- 3-way merge of near-infrared/diffuse albedo ---
   ki=kSi_anidf
   kl=kSl_anidf
   ko=kSo_anidf
   kx=kSs_anidf
   Ss(:,kx) = fi_a*Si(:,ki) + fl_a*Sl(:,kl) + fo_a*So(:,ko)

   !--- snow depth, no merge, set equal to land value-------------
   Ss(:,kSs_snowh) = fl_a*Sl(:,kSl_snowh)

END

!===============================================================================
