!===============================================================================
! CVS $Id: msg_ocn_mod.F90,v 1.15 2001/12/07 22:13:36 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/msg_ocn_mod.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

MODULE msg_ocn_mod

   use control_mod      ! control data
   use buf_mod          ! model   data
   use const_mod        ! physical constants
   use map_mod          ! map data
   use shr_msg_mod      ! msg constants, data, routines
   use shr_timer_mod    ! timing routines

   implicit none

   private  ! resticted access
   public  :: msg_ocni,msg_ocnr,msg_ocns,msg_ocnf

   integer,parameter   :: nsflds=14        ! number of fields sent
   integer,parameter   :: nrflds= 7        ! number of fields recv'd
   real   ,allocatable :: buffs(:,:)       ! msg buffer: send
   real   ,allocatable :: buffr(:,:)       ! msg buffer: recv
   real   ,allocatable :: area_fact_c2o(:) ! cpl->ocn area correction factor
   real   ,allocatable :: area_fact_o2c(:) ! ocn->cpl area correction factor

   !--- aggregate timing info ---
   integer      :: timer_send    ! shr_timer timer number for data send
   integer      :: timer_recv    ! shr_timer timer number for data recv

   !--- detailed timing info ---
!  integer(8)   :: irtc_w        ! rtc tics when waiting for msg
!  integer(8)   :: irtc_r        ! rtc tics when msg received
!  integer(8)   :: irtc_u        ! rtc tics when msg unpacked
!  integer(8)   :: irtc_p        ! rtc tics when packing msg
!  integer(8)   :: irtc_s        ! rtc tics when sending msg 

   !--- formats ---
   character(len=*),parameter :: F98 = "('comm_diag ',a3,1x,a4,1x,i3,es26.19,1x,a)"
   character(len=*),parameter :: F99 = "('[mp timing]  irtc = ',i20,' ',a)"

   save

CONTAINS

!===============================================================================

SUBROUTINE msg_ocni ()

   implicit none

   !--- local ---
   integer :: n             ! generic loop index
   real    :: minf,maxf     ! min/max cell area correction factors

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_ocni) ',a,i4)"
   character(len=*),parameter :: F02 = "('(msg_ocni) ',a,2f12.9)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   send/recv initial msg to/from ocn 
! INPUT:
!   o flags contained in control.h
! OUTPUT:
!   o ocn info-buffer data         : ibuff_o
!   o ocn spatial coordinate arrays: xv_o,xc_o,yv_o,yc_o
!   o ocn model domain mask        : mask_o
! MESSAGES:
!   o data  sent : cpl info-buffer flags
!   o data recv'd: ocn info-buffer flags, grids, domain mask
!-------------------------------------------------------------------------------
 
   !--- allocate timer numbers ---
   call shr_timer_get(timer_send,'msg: cpl  -> ocn')
   call shr_timer_get(timer_recv,'msg: cpl <-  ocn')
   
   !--- allocate local buffers/arrays ---
   allocate(buffr(n_o,nrflds))   ! recv buffer
   allocate(buffs(n_o,nsflds))   ! send buffer
   allocate(area_fact_c2o(n_o))  ! area factment factor
   allocate(area_fact_o2c(n_o))  ! area factment factor

   !-----------------------------------------------------------------
   ! send initial msg
   !-----------------------------------------------------------------
   ibuff_c     = 0
   ibuff_c( 1) = ierr              ! error code, 0 => no error
   ibuff_c( 2) = stop_eod          ! flag: stop at end of day
   ibuff_c( 3) = stop_now          ! flag: stop now
   ibuff_c( 4) = cdate             ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec               ! seconds
   ibuff_c(11) = info_time         ! flag: mp timing
   ibuff_c(12) = info_dbug         ! dbug diagnostic level
   ibuff_c(40) = SHR_MSG_O_MAJ_V04 ! cpl/ocn interface: major version #
   ibuff_c(41) = SHR_MSG_O_MIN_V00 ! cpl/ocn interface: minor version #

   write(6,F01) 'send ocn msg (initial)'
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_C2OI)
   
   !-----------------------------------------------------------------
   ! receive initial msg
   !-----------------------------------------------------------------
   write(6,F01) 'recv ocn msg (initial)'
   call shr_msg_recv_i(ibuff_o, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(  xc_o , n_o       , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(  yc_o , n_o       , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(  xv_o , n_o*4     , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(  yv_o , n_o*4     , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_r(area_o , n_o       , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
   call shr_msg_recv_i(mask_o , n_o       , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2CI)
  
   !--- extract info-buffer values ---
   if (ibuff_o(7) .ne. nx_o) stop 'nx'
   if (ibuff_o(8) .ne. ny_o) stop 'ny'
   ncpl_o    = ibuff_o( 9) ! number of msg pairs per day
   flx_epfac = ibuff_o(31) ! E~P adjustment factor
   init_rico = ibuff_o(35) ! T => use model's restart IC data

   !-----------------------------------------------------------------
   ! compute factors that account for inconsistent cell area values 
   !-----------------------------------------------------------------
   area_fact_c2o = 1.0
   area_fact_o2c = 1.0
   minf =  1.0e30
   maxf = -1.0e30
   do n=1,n_o
     if (mask_o(n) /= 0) then
       area_fact_c2o(n) = domain_o%area(n)/area_o(n)
       area_fact_o2c(n) = area_o(n)/domain_o%area(n)
       minf = min(minf,area_fact_c2o(n))
       maxf = max(maxf,area_fact_c2o(n))
     end if
   end do
   write(6,F02) 'min c2o area factor:',minf
   write(6,F02) 'max c2o area factor:',maxf

   !-----------------------------------------------------------------
   ! KLUDGE: scrip & ocn xc & yc should be the same but aren't
   !-----------------------------------------------------------------
   write(6,F01) 'WARNING: replacing scrip xc,yc with values provided by ocn'
   domain_o%xc = xc_o
   domain_o%yc = yc_o

END SUBROUTINE msg_ocni

!===============================================================================

SUBROUTINE msg_ocnr (So,Fioo)

   implicit none

   !--- arguments ---
   real         :: So  (n_o,nSo  )  ! ocn state
   real         :: Fioo(n_o,nFioo)  ! ice/ocn fluxes from ocn

   !--- local ---
   integer      :: n                ! generic loop index
   integer      :: m                ! generic loop index
   real         :: f                ! factor wrt area correction
   real         :: gsum             ! global sum for diagnostics
   integer(8)   :: irtc_w        ! rtc tics when waiting for msg
   integer(8)   :: irtc_r        ! rtc tics when msg received
   integer(8)   :: irtc_u        ! rtc tics when msg unpacked
   integer(8)   :: irtc_p        ! rtc tics when packing msg
   integer(8)   :: irtc_s        ! rtc tics when sending msg

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_ocnr) ',a,i4)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   receive msg from ocn model 
! INPUT:
!   o none
! OUTPUT:
!   o ibuff_o, So_o, Fioo_o
! MESSAGES:
!   o data received: ocn info-buffer, state variables, output fluxes
!-------------------------------------------------------------------------------
   
   if (info_dbug.ge.3) write(6,F01) 'recv ocn msg'

   irtc_w = shr_sys_irtc()
   call shr_timer_start(timer_recv)
   call shr_msg_recv_i(ibuff_o, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_O2C)
   irtc_r = shr_sys_irtc()
   call shr_msg_recv_r( buffr ,size(buffr), SHR_MSG_TID_OCN, SHR_MSG_TAG_O2C)
   
!$OMP PARALLEL DO PRIVATE(n,f)
   do n=1,n_o
      f = area_fact_o2c(n) ! wrt flux conservation & varying cell area calcs

      !--- ocn states ---
      So (n,kSo_t      ) = buffr(n,1)
      So (n,kSo_s      ) = buffr(n,2)
      So (n,kSo_u      ) = buffr(n,3)
      So (n,kSo_v      ) = buffr(n,4)

      !--- output fluxes: ice/ocn ---
      Fioo(n,kFioo_dhdx) = buffr(n,5)
      Fioo(n,kFioo_dhdy) = buffr(n,6)
      Fioo(n,kFioo_q   ) = buffr(n,7) * f
   end do

   if (info_dbug >= 2) then
      do m=1,nrflds
         gsum = 0.
         do n=1,n_o
            if (mask_o(n) /= 0) gsum = gsum + buffr(n,m)*area_o(n)*rEarth*rEarth
         enddo
         write(6,F98)'ocn','send',m,gsum,'in_cpl'
      enddo
   endif

   irtc_u = shr_sys_irtc()
   call shr_timer_stop(timer_recv)

   !--- extract info-buffer values ---
   flx_epfac = ibuff_o(31) ! E~P factment factor

   !--- print timing info? ---
   if (info_time.ne.0) then
      write(6,F99) irtc_w,'o->c waiting'
      write(6,F99) irtc_r,'o->c received'
      write(6,F99) irtc_u,'o->c unpacked'
   end if

END SUBROUTINE msg_ocnr

!===============================================================================

SUBROUTINE msg_ocns (Foxx)

   implicit none

   !--- input ---
   real         :: Foxx(n_o,nFoxx)  ! ocn input fluxes

   !--- local ---
   integer      :: n                ! generic loop index
   integer      :: m                ! generic loop index
   real         :: f                ! factor wrt area correction
   real         :: gsum             ! global sum for diagnostics
   integer(8)   :: irtc_w        ! rtc tics when waiting for msg
   integer(8)   :: irtc_r        ! rtc tics when msg received
   integer(8)   :: irtc_u        ! rtc tics when msg unpacked
   integer(8)   :: irtc_p        ! rtc tics when packing msg
   integer(8)   :: irtc_s        ! rtc tics when sending msg

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_ocns) ',a,i4)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   send msg to ocn model 
! INPUT:
!   o Foxx_o & various flags contained in control.h 
! OUTPUT:
!   o none
! MESSAGES:
!   o data sent: coupler info-buffer, ocn input fluxes
!-------------------------------------------------------------------------------
   
   if (info_dbug.ge.3) write(6,F01) 'send ocn msg'

   !--- fill info-buffer array ---
   ibuff_c     = 0
   ibuff_c( 1) = ierr      ! error code, 0 => no error
   ibuff_c( 2) = stop_eod  ! flag: stop at end of day
   ibuff_c( 3) = stop_now  ! flag: stop now
   ibuff_c( 4) = cdate     ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec       ! seconds
   ibuff_c(11) = info_time ! flag: mp timing
   ibuff_c(12) = info_dbug ! dbug diagnostic level
   ibuff_c(21) = rest_eod  ! flag: make restart data at eod
   ibuff_c(22) = rest_now  ! flag: make restart data now
   ibuff_c(23) = hist_eod  ! flag: make history data at eod
   ibuff_c(24) = hist_now  ! flag: make history data now
   ibuff_c(25) = hist_tavg ! flag: make monthly average data
   ibuff_c(26) = diag_eod  ! flag: make diagnostic at eod
   ibuff_c(27) = diag_now  ! flag: make diagnostic data now
   ibuff_c(32) = ashift    ! albedo time shift

   !--- pack msg buffer ---
   irtc_p = shr_sys_irtc()
   call shr_timer_start(timer_send)

!$OMP PARALLEL DO PRIVATE(n,f)
   do n=1,n_o
      f = area_fact_c2o(n) ! wrt flux conservation & varying cell area calcs

      !--- state variables ---
      buffs(n, 1) = Foxx(n,kFoxx_ifrac )   ! ice fraction
      buffs(n, 2) = Foxx(n,kFoxx_pslv  )   ! sea level pressure

      !--- merged ocn/[atm|ice] fluxes ---
      buffs(n, 3) = Foxx(n,kFoxx_taux ) * f
      buffs(n, 4) = Foxx(n,kFoxx_tauy ) * f
      buffs(n, 5) = Foxx(n,kFoxx_swnet) * f
      !!!!!(n, -) = Foxx(n,kFoxx_lat  ) * f ~ assume lat = k*evap 
      buffs(n, 6) = Foxx(n,kFoxx_sen  ) * f
      buffs(n, 7) = Foxx(n,kFoxx_lwup ) * f
      buffs(n, 8) = Foxx(n,kFoxx_lwdn ) * f
      buffs(n, 9) = Foxx(n,kFoxx_melth) * f
      buffs(n,10) = Foxx(n,kFoxx_salt ) * f
      buffs(n,11) =(Foxx(n,kFoxx_rain ) + Foxx(n,kFoxx_snow)) * f
      buffs(n,12) = Foxx(n,kFoxx_evap ) * f
      buffs(n,13) = Foxx(n,kFoxx_meltw) * f
      buffs(n,14) = Foxx(n,kFoxx_roff ) * f
   end do

   if (info_dbug >= 2) then
      do m=1,nsflds
         gsum = 0.
         do n=1,n_o
            if (mask_o(n) /= 0) gsum = gsum + buffs(n,m)*area_o(n)*rEarth*rEarth
         enddo
         write(6,F98)'ocn','recv',m,gsum,'in_cpl'
      enddo
   endif

   !--- send msg ---
   irtc_s = shr_sys_irtc()
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_C2O)
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_OCN, SHR_MSG_TAG_C2O)
   call shr_timer_stop(timer_send)

   !--- print timing info? ---
   if (info_time.ne.0) then
      write(6,F99) irtc_p,'c->o packing'
      write(6,F99) irtc_s,'c->o sending'
   end if

END SUBROUTINE msg_ocns

!===============================================================================

SUBROUTINE msg_ocnf ()

   implicit none

   !--- local ---
   integer      :: n         ! generic loop index

   !--- formats ---
   character(len=*),parameter :: F01 = "('(msg_ocnf) ',a,i4)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   send final msg to ocn model
! INPUT:
!   o various flags contained in control.h 
! OUTPUT:
!   o none
! MESSAGES:
!   o data sent: coupler info-buffer
!-------------------------------------------------------------------------------
   
   write(6,F01) 'send ocn msg (final)'

   !--- fill info-buffer array ---
   ibuff_c     = 0
   ibuff_c( 1) = ierr      ! error code, 0 => no error
   ibuff_c( 2) = stop_eod  ! flag: stop at end of day
   ibuff_c( 3) = 1         ! flag: stop now
   ibuff_c( 4) = cdate     ! date (encoded: yyyymmdd)
   ibuff_c( 5) = sec       ! seconds
   ibuff_c(11) = info_time ! flag: mp timing
   ibuff_c(12) = info_dbug ! dbug diagnostic level
   ibuff_c(21) = rest_eod  ! flag: make restart data at eod
   ibuff_c(22) = rest_now  ! flag: make restart data now
   ibuff_c(23) = hist_eod  ! flag: make history data at eod
   ibuff_c(24) = hist_now  ! flag: make history data now
   ibuff_c(25) = hist_tavg ! flag: make monthly average data
   ibuff_c(26) = diag_eod  ! flag: make diagnostic at eod
   ibuff_c(27) = diag_now  ! flag: make diagnostic data now
   ibuff_c(32) = ashift    ! albedo time shift

   !--- pack msg buffer (with special-values) ---
   buffs = spval

   !--- send msg ---
   call shr_msg_send_i(ibuff_c, nibuff    , SHR_MSG_TID_OCN, SHR_MSG_TAG_C2O)
   call shr_msg_send_r( buffs ,size(buffs), SHR_MSG_TID_OCN, SHR_MSG_TAG_C2O)
   
END SUBROUTINE msg_ocnf

!===============================================================================

END MODULE msg_ocn_mod
