!===============================================================================
! CVS $Id: nc_fields2.F90,v 1.6 2002/01/05 00:07:19 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/nc_fields2.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================
! o These routines can define and add time-variant data to a netCDF file
! o fid must be the file id of an existing netCDF file 
!   1) that is open
!   2) that has all dimensions defined
!   3) is     in define-mode iff op="define"
!      is NOT in define-mode iff op="put"
! o data may be instantanious or time-averaged, global attributes 
!   of the time dimension are supposed to specify which is the case.
!===============================================================================

SUBROUTINE nc_Faxx(op,fid,F,n,tavg,spv,code)

   USE buf_mod                          ! model data buffers

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_a,ny_a,*)  ! (in) data buffer
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: code            ! (out) return code

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_nc_Faxx) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Faxx_a buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     code = nf_inq_dimid(fid,'ni_a'  ,vdims(1))
     code = nf_inq_dimid(fid,'nj_a'  ,vdims(2))
     code = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Faxx_a_taux'
     lname = 'surface stress: zonal'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faxx_a_tauy'
     lname = 'surface stress: meridional'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faxx_a_lat'
     lname = 'heat flux: latent'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faxx_a_sen'
     lname = 'heat flux: sensible'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faxx_a_lwup'
     lname = 'heat flux: long wave up'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faxx_a_evap'
     lname = 'water flux: evaporation'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Faxx_a_tref'
     lname = 'diagnostic: 2m reference air temperature'
     units = 'K'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_a
     cnt(2) = ny_a
     cnt(3) = 1
     call nc_pvar(fid,'Faxx_a_taux' ,beg,cnt,F(1,1,kFaxx_taux ),code)
     call nc_pvar(fid,'Faxx_a_tauy' ,beg,cnt,F(1,1,kFaxx_tauy ),code)
     call nc_pvar(fid,'Faxx_a_lat'  ,beg,cnt,F(1,1,kFaxx_lat  ),code)
     call nc_pvar(fid,'Faxx_a_sen'  ,beg,cnt,F(1,1,kFaxx_sen  ),code)
     call nc_pvar(fid,'Faxx_a_lwup' ,beg,cnt,F(1,1,kFaxx_lwup ),code)
     call nc_pvar(fid,'Faxx_a_evap' ,beg,cnt,F(1,1,kFaxx_evap ),code)
     call nc_pvar(fid,'Faxx_a_tref' ,beg,cnt,F(1,1,kFaxx_tref ),code)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================

SUBROUTINE nc_Foxx(op,fid,F,n,tavg,spv,code)

   USE buf_mod                          ! model data buffers

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_o,ny_o,*)  ! (in) data buffer
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: code            ! (out) return code

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_Foxx) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add Foxx_o buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     code = nf_inq_dimid(fid,'ni_o'  ,vdims(1))
     code = nf_inq_dimid(fid,'nj_o'  ,vdims(2))
     code = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'Foxx_o_taux'
     lname = 'surface stress: zonal'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Foxx_o_tauy'
     lname = 'surface stress: meridional'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Foxx_o_swnet'
     lname = 'heat flux: net short wave'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Foxx_o_lat'
     lname = 'heat flux: latent'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Foxx_o_sen'
     lname = 'heat flux: sensible'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Foxx_o_lwup'
     lname = 'heat flux: long-wave up'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Foxx_o_lwdn'
     lname = 'heat flux: long-wave down'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Foxx_o_melth'
     lname = 'heat flux: melting'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Foxx_o_salt'
     lname = 'salt flux: melting'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Foxx_o_rain'
     lname = 'water flux: rain'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Foxx_o_snow'
     lname = 'water flux: snow'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Foxx_o_evap'
     lname = 'water flux: evaporation'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Foxx_o_meltw'
     lname = 'water flux: melting'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Foxx_o_roff'
     lname = 'water flux: runoff'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)

     name  = 'Foxx_o_pslv'
     lname = 'state: atm sea level pressure'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'Foxx_o_ifrac'
     lname = 'state: ice fraction'
     units = '1'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)

   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_o
     cnt(2) = ny_o
     cnt(3) = 1
     call nc_pvar(fid,'Foxx_o_taux' ,beg,cnt,F(1,1,kFoxx_taux ),code)
     call nc_pvar(fid,'Foxx_o_tauy' ,beg,cnt,F(1,1,kFoxx_tauy ),code)
     call nc_pvar(fid,'Foxx_o_swnet',beg,cnt,F(1,1,kFoxx_swnet),code)
     call nc_pvar(fid,'Foxx_o_lat'  ,beg,cnt,F(1,1,kFoxx_lat  ),code)
     call nc_pvar(fid,'Foxx_o_sen'  ,beg,cnt,F(1,1,kFoxx_sen  ),code)
     call nc_pvar(fid,'Foxx_o_lwup' ,beg,cnt,F(1,1,kFoxx_lwup ),code)
     call nc_pvar(fid,'Foxx_o_lwdn' ,beg,cnt,F(1,1,kFoxx_lwdn ),code)
     call nc_pvar(fid,'Foxx_o_melth',beg,cnt,F(1,1,kFoxx_melth),code)
     call nc_pvar(fid,'Foxx_o_salt' ,beg,cnt,F(1,1,kFoxx_salt ),code)
     call nc_pvar(fid,'Foxx_o_rain' ,beg,cnt,F(1,1,kFoxx_rain ),code)
     call nc_pvar(fid,'Foxx_o_snow' ,beg,cnt,F(1,1,kFoxx_snow ),code)
     call nc_pvar(fid,'Foxx_o_evap' ,beg,cnt,F(1,1,kFoxx_evap ),code)
     call nc_pvar(fid,'Foxx_o_meltw',beg,cnt,F(1,1,kFoxx_meltw),code)
     call nc_pvar(fid,'Foxx_o_roff' ,beg,cnt,F(1,1,kFoxx_roff ),code)
     call nc_pvar(fid,'Foxx_o_pslv' ,beg,cnt,F(1,1,kFoxx_pslv ),code)
     call nc_pvar(fid,'Foxx_o_ifrac',beg,cnt,F(1,1,kFoxx_ifrac),code)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================

SUBROUTINE nc_aFoxx(op,fid,F,n,tavg,spv,code)

   USE buf_mod                          ! model data buffers

   implicit none

   !----- global -----
#include <netcdf.inc>

   !----- arguments -----
   character(len= *) :: op              ! (in) operation: define or add?
   integer           :: fid             ! (in) nc file ID
   real              :: F(nx_o,ny_o,*)  ! (in) data buffer
   integer           :: n               ! (in) index of time dimension
   logical           :: tavg            ! (in) flags tavg data
   real              :: spv             ! (in) special value
   integer           :: code            ! (out) return code

   !----- local -----
   integer           :: ndims           ! number of dimensions
   integer           :: vdims(3)        ! vector of dimensions
   integer           :: beg  (3)        ! vector of beginning indicies
   integer           :: cnt  (3)        ! vector of edge lengths
   character(len=64) :: name            ! variable's name
   character(len=64) :: lname           ! variable's long-name
   character(len=64) :: units           ! variable's units

   !----- formats -----
   character(len=*),parameter :: F00 = "('(nc_aFoxx) ',2a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   define or add aFoxx_o buffer to netCDF file
!-------------------------------------------------------------------------------

   if (op .eq. 'define') then
     !-----------------------------------------------------------------------
     ! define data
     !-----------------------------------------------------------------------
     code = nf_inq_dimid(fid,'ni_o'  ,vdims(1))
     code = nf_inq_dimid(fid,'nj_o'  ,vdims(2))
     code = nf_inq_dimid(fid,'time'  ,vdims(3))
     ndims = 3

     !-----------------------------------------------------------------------
     name  = 'aFoxx_o_taux'
     lname = 'surface stress: zonal'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'aFoxx_o_tauy'
     lname = 'surface stress: meridional'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'aFoxx_o_swnet'
     lname = 'heat flux: net short wave'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'aFoxx_o_lat'
     lname = 'heat flux: latent'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'aFoxx_o_sen'
     lname = 'heat flux: sensible'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'aFoxx_o_lwup'
     lname = 'heat flux: long-wave up'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'aFoxx_o_lwdn'
     lname = 'heat flux: long-wave down'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'aFoxx_o_melth'
     lname = 'heat flux: melting'
     units = 'W/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'aFoxx_o_salt'
     lname = 'salt flux: melting'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'aFoxx_o_rain'
     lname = 'water flux: rain'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'aFoxx_o_snow'
     lname = 'water flux: snow'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'aFoxx_o_evap'
     lname = 'water flux: evaporation'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'aFoxx_o_meltw'
     lname = 'water flux: melting'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'aFoxx_o_roff'
     lname = 'water flux: runoff'
     units = '(kg/s)/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)

     name  = 'aFoxx_o_pslv'
     lname = 'state: atm sea level pressure'
     units = 'N/m^2'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)
     name  = 'aFoxx_o_ifrac'
     lname = 'state: ice fraction'
     units = '[0,1]'
     call nc_dvar(fid,name,ndims,vdims,lname,units,tavg,spv,code)

   else if (op .eq. 'put') then
     !-----------------------------------------------------------------------
     ! put data
     !-----------------------------------------------------------------------
     beg(1) = 1
     beg(2) = 1
     beg(3) = n
     cnt(1) = nx_o
     cnt(2) = ny_o
     cnt(3) = 1
     call nc_pvar(fid,'aFoxx_o_taux' ,beg,cnt,F(1,1,kFoxx_taux ),code)
     call nc_pvar(fid,'aFoxx_o_tauy' ,beg,cnt,F(1,1,kFoxx_tauy ),code)
     call nc_pvar(fid,'aFoxx_o_swnet',beg,cnt,F(1,1,kFoxx_swnet),code)
     call nc_pvar(fid,'aFoxx_o_lat'  ,beg,cnt,F(1,1,kFoxx_lat  ),code)
     call nc_pvar(fid,'aFoxx_o_sen'  ,beg,cnt,F(1,1,kFoxx_sen  ),code)
     call nc_pvar(fid,'aFoxx_o_lwup' ,beg,cnt,F(1,1,kFoxx_lwup ),code)
     call nc_pvar(fid,'aFoxx_o_lwdn' ,beg,cnt,F(1,1,kFoxx_lwdn ),code)
     call nc_pvar(fid,'aFoxx_o_melth',beg,cnt,F(1,1,kFoxx_melth),code)
     call nc_pvar(fid,'aFoxx_o_salt' ,beg,cnt,F(1,1,kFoxx_salt ),code)
     call nc_pvar(fid,'aFoxx_o_rain' ,beg,cnt,F(1,1,kFoxx_rain ),code)
     call nc_pvar(fid,'aFoxx_o_snow' ,beg,cnt,F(1,1,kFoxx_snow ),code)
     call nc_pvar(fid,'aFoxx_o_evap' ,beg,cnt,F(1,1,kFoxx_evap ),code)
     call nc_pvar(fid,'aFoxx_o_meltw',beg,cnt,F(1,1,kFoxx_meltw),code)
     call nc_pvar(fid,'aFoxx_o_roff' ,beg,cnt,F(1,1,kFoxx_roff ),code)
     call nc_pvar(fid,'aFoxx_o_pslv' ,beg,cnt,F(1,1,kFoxx_pslv ),code)
     call nc_pvar(fid,'aFoxx_o_ifrac',beg,cnt,F(1,1,kFoxx_ifrac),code)
   else 
     write(6,F00) 'unrecognized operation: ',op
   end if

END

!===============================================================================
