!===============================================================================
! CVS $Id: t_avg.F90,v 1.4 2001/10/12 17:33:13 tcraig Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/t_avg.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

SUBROUTINE t_avg (cFlag,nx,A,B,ns)

   USE shr_timer_mod  ! timing routines

   IMPLICIT none

   !----- arguments -----
   character(len=*) :: cFlag          ! operation request flag
   integer          :: nx             ! size of arrays
   real             :: A(nx),B(nx)    ! data arrays
   integer          :: ns             ! number of samples in summation

   !----- local -----
   real             :: fact           ! = 1/ns
   integer          :: i              ! generic loop index
   integer,save     :: timer_n = 0    ! timer number

!-------------------------------------------------------------------------------
! PURPOSE:
!   form the average, B, of a series of input arrays A
!
!   cFlag='zero' => B=0   , ns=0
!   cFlag='sum'  => B=B+A , ns=ns+1
!   cFlag='avg'  => B=A/ns
!
! ASSUMPTIONS:
!   there are no undefined array elements
!-------------------------------------------------------------------------------
   
   if (timer_n < 1) call shr_timer_get(timer_n,'time averaging/accumulation')
   call shr_timer_start(timer_n)

   SELECT CASE (cFlag)
      CASE ( 'zero' ) 
!$OMP PARALLEL DO PRIVATE(i)
         do i=1,nx
           B(i)=0.0
         enddo
         ns=0
      CASE ( 'sum' ) 
!$OMP PARALLEL DO PRIVATE(i)
         do i=1,nx
           B(i)=B(i)+A(i)
         enddo
         ns=ns+1
      CASE ( 'avg' ) 
         IF (ns .le. 0) THEN
           write(6,*) '(t_avg) ERROR: ns = ',ns
           STOP 't_avg'
         ELSE IF (ns == 1) THEN
!$OMP PARALLEL DO PRIVATE(i)
           do i=1,nx
             B(i)=A(i)
           enddo
         ELSE
           fact=1.0/float(ns)
!$OMP PARALLEL DO PRIVATE(i)
           do i=1,nx
             B(i)=A(i)*fact
           enddo
         END IF
      CASE DEFAULT
         write(6,*) '(t_avg) ERROR: invalid cFlag=',cFlag
         STOP 'cFlag'
   END SELECT

   call shr_timer_stop (timer_n)

END

!===============================================================================
