c CVS: $Id: ice.F,v 1.22 2002/04/30 16:01:17 schramm Exp $
c CVS: $Source: /fs/cgd/csm/models/CVS.REPOS/ice/csim4_src/source/ice.F,v $
c CVS: $Name: ccsm2_0_beta46 $

!===============================================================================
! 
!    The Community Climate System Model (CCSM) Copyright and Release Notice
! 
!              +------------------------------------------+
!              | DISTRIBUTION TERMS AND CONDITIONS NOTICE |
!              +------------------------------------------+
! 
! 
! 
! CCSM was developed in cooperation with the National Science Foundation,
! the Department of Energy Los Alamos National Laboratory (LANL), the
! National Aeronautics and Space Administration Data Assimilation Office,
! and the University Corporation for Atmospheric Research's National
! Center for Atmospheric Research.*  Except for POP, SCRIP, and CICE,
! which are segregable components of the model, CCSM is public domain
! software.  
! 
! POP, SCRIP, and CICE were developed at LANL and are protected by the
! following copyright notice. 
! 
! Copyright (c)2002 The Regents of the University of California. All Rights
! Reserved. 
! 
! Permission to use, copy, modify, and distribute this software and its
! documentation for educational, research and non-profit purposes, without
! fee, and without a written agreement is hereby granted, provided that
! the above copyright notice, this paragraph and the following three
! paragraphs appear in all copies. 
! 
! Permission to incorporate this software into commercial products may be
! obtained by contacting the University of California, Charles Rzeszutko,
! Campus Liaison Officer Office of Technology Transfer, 1111 Franklin
! Street, 5th Floor, Oakland, CA 94607, (510) 587-6063,
! charles.rzeszutko@ucop.edu.
! 
! THIS SOFTWARE PROGRAM AND DOCUMENTATION ARE COPYRIGHTED BY 
! THE REGENTS OF THE UNIVERSITY OF CALIFORNIA. THE SOFTWARE 
! PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS", WITHOUT ANY 
! ACCOMPANYING SERVICES FROM THE REGENTS. THE REGENTS DOES NOT 
! WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE 
! UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS THAT 
! THE PROGRAM WAS DEVELOPED FOR RESEARCH PURPOSES AND IS 
! ADVISED NOT TO RELY EXCLUSIVELY ON THE PROGRAM FOR ANY 
! REASON. 
! 
! IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY 
! PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
! CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF 
! THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE 
! UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF 
! SUCH DAMAGE.  THE UNIVERSITY OF CALIFORNIA SPECIFICALLY 
! DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
! IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
! PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN 
! "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO 
! OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, 
! ENHANCEMENTS, OR MODIFICATIONS. 
! 
! *The National Center for Atmospheric Research is sponsored by the
! National Science Foundation.  Any opinions, findings and conclusions or
! recommendations expressed in this publication are those of the author(s)
! and do not necessarily reflect the views of the National Science
! Foundation.
!===============================================================================

!BOP
!
! !MODULE: icemodel - main ice model program
!
! !DESCRIPTION:
!
! The NCAR Community Climate System Model Version 2 - Sea Ice (CCSM2-SI)
! is intended for use by the climate modeling community, and its
! development has been a true community effort.  The list of physics to
! be included in CCSM2-SI was developed by the CCSM Polar Climate
! Working Group (PCWG) in a series of meetings during 1997-1999.  The
! specific parameterizations and codes to be implemented were identified
! and agreed upon at a meeting of ice model developers from the PCWG,
! held at NCAR in September, 1999. \\
! 
! A brief description of the development of the CCSM2-SI code is as
! follows: \\
! 
!   o The code originated from the LANL CICE model (version of
!   September, 1999).  CCSM2-SI retains the following elements from this
!   original code: Elastic-viscous-plastic dynamics (EVP), MPDATA
!   transport scheme, and much of the original code structure. \\
!   
!   o Later in autumn of 1999, the code was changed as follows: (1) the
!   MPI parallelization routines were updated; (2) a multi-category ice
!   thickness distribution was added; (3) a new thermodynamic code
!   with explicit treatment of brine pockets was added. \\ 
!  
!   o In late 1999/early 2000, changes were made to the history output. \\
! 
!   o During winter-spring 2000, the Active Ice Only (AIO) framework was
!   developed, together with datasets to be used as forcing within
!   this framework.  The model was tested within this AIO framework. \\
! 
!   o In the fall of 2000, the code was streamlined, checked, 
!   standardized, and updated to Fortran 90. Documentation is currently 
!   being written.  \\
! 
! The development team for the model described above is: \\
! 
! Elizabeth Hunke, LANL \\
! Cecilia Bitz, U. WA.  \\
! Bruce Briegleb, NCAR  \\
! Tony Craig, NCAR      \\
! Marika Holland, NCAR  \\
! Bill Lipscomb, LANL   \\
! Julie Schramm, NCAR   \\
!
! Numerous others have contributed to this effort also--thanks to all! 
!
! !INTERFACE:
!
      program icemodel
!
! !USES:
! 
      use ice_timers
      use ice_mpi_internal
      use ice_albedo
      use ice_vthermo
      use ice_mechred
      use ice_scaling
      use ice_diagnostics
      use ice_history
      use ice_coupling
      use ice_grid
      use ice_calendar
      use ice_dyn_evp
      use ice_itd
      use ice_itd_linear
      use ice_transport
      use ice_therm_driver
      use ice_init
      use shr_msg_mod,only:shr_msg_chdir,shr_msg_dirio,shr_sys_flush
      use ice_prescribed
      use ice_ocean
      use ice_prnpnt
!
! !REVISION HISTORY:
!
!    February 2001 - E.Hunke (LANL) provided original 
!    May 2001      - Final form settled (M.Holland, J.Schramm and 
!                    B.Briegleb, NCAR)
!
!EOP
!
      implicit none

      !-----------------------------------------------------------------
      ! initializations
      !-----------------------------------------------------------------
      call setup_mpi           ! initial setup for message passing

      call shr_msg_chdir('ice')    ! change cwd
#if coupled
      if (my_task == master_task) then
        call shr_msg_dirio('ice')    ! redirect stdin/stdout
      endif
#endif
      ! timing devices
      call ice_timer_clear(-1)     ! initialize all timers
      call ice_timer_start(0)      ! begin timing entire run

      ! parameters, grid, parameterizations, etc
      call init_constants
      call input_data          ! namelist variables
      call init_grid(oceanmixed_ice) ! grid variables
      call init_calendar       ! initialize some calendar stuff
      call init_hist           ! initialize output history file
      call init_evp            ! define evp dynamics parameters, variables
      call init_flux           ! initialize coupler fluxes
      call init_itd            ! initialize ice thickness distribution
      call init_thermo         ! initialize ice salinity and melting temp.
      call init_mechred        ! initialize ridging parameters
      call init_state          ! initialize the ice state

      if(prescribed_ice) then
        call init_prescribed_ice      ! initialize prescribed ice
      endif

      if (restart) call restartfile   ! start from restart data

      call albedos             ! albedo based on initial ice distribution
      call calendar(time)      ! determine the initial date

      if(oceanmixed_ice) then
        call init_oceanmixed_ice         ! initialize ocean mixed layer forcing data
      endif

#if coupled
      call init_cpl            ! initialize message passing 
#endif
      call init_diags          ! initialize diagnostic output
      if( print_points ) call intpnt  ! initialize grid point output

      write_ic = .true.        ! write initial conditions
      if(.not.prescribed_ice) call ice_write_hist
      write_ic = .false.

      !-----------------------------------------------------------------
      ! timestep loop 
      !-----------------------------------------------------------------
      call ice_timer_start(1)  ! begin timing entire timestepping loop
      istep = 0 

      do while (stop_now .lt. 1)

      call ice_timer_start(14) ! begin timing between snd-recv
      istep  = istep  + 1
      istep1 = istep1 + 1
      time = time + dt         ! determine the time and date
      call calendar(time)      !   at the end of the timestep
      call ice_timer_stop(14)  ! end timing between snd-recv

#if coupled
      call ice_timer_start(13) ! begin timing MPI-recv
      call from_coupler        ! get updated info from flux coupler
      call ice_timer_stop(13)  ! end timing MPI-recv
#endif
      if( stop_now == 1 ) goto 200   ! coupler called it quits

      if(oceanmixed_ice) then
        call set_oceanmixed_ice      ! set sst and frzmlt from ocean mixed layer
      endif

      call ice_timer_start(15) ! begin timing between recv-snd

      if(prescribed_ice) then
        if(prescribed_ice_climatology) then
          call read_prescribed_ice_climate ! read prescribed climatology ice
        else
          call read_prescribed_ice         ! read prescribed ice
        endif
        call set_prescribed_ice_state  ! set the ice state
        call set_prescribed_ice_flux   ! set appropriate fluxes
      endif
      if (mod(istep,diagfreq).eq.0) call init_mass_diags ! diagnostic
      call thermo_rates        ! thermodynamic growth rates and fluxes

#ifndef fcd_coupled
      call scale_fluxes        ! divide fluxes by ice area for coupler
#endif
      call ice_timer_stop(15)  ! end timing between recv-snd

#if coupled
      call ice_timer_start(12) ! begin timing MPI-send
      call to_coupler          ! collect/send data to flux coupler
      call ice_timer_stop(12)  ! end timing MPI-send
#endif

      if( oceanmixed_ice ) then
        call time_intrplt_ocean_forcing  ! update ocean forcing data 
        call compute_oceanmixed_ice      ! update ocean sst and frzmlt
      endif

      call ice_timer_start(14)  ! begin timing between snd-recv

      if(.not.prescribed_ice) then
        call thermo_itd          ! thermodynamics and associated itd changes
        if (kdyn.eq.1) then
          call evp(kstrength) ! elastic-viscous-plastic dynamics 
          call transport           ! horizontal advection
          call mechanical_redistr  ! ridging
        endif
        call distr_check         ! corrections to thickness distribution
      endif
      call aggregate           ! merge categories together
      call albedos             ! compute ice albedos

      !-----------------------------------------------------------------
      ! write data 
      !-----------------------------------------------------------------
      call ice_timer_start(9)  ! reading/writing
#ifndef fcd_coupled
      call scale_hist_fluxes
#endif
      if (mod(istep,diagfreq).eq.0) call runtime_diags  ! standard out
      call ice_write_hist

      if (write_restart==1) call dumpfile   ! dumps for restarting

      call ice_timer_stop(9)   ! reading/writing
      call ice_timer_stop(14)  ! end timing between snd-recv

      enddo ! while

      !-----------------------------------------------------------------
      ! end of timestep loop
      ! print timing information and exit
      !-----------------------------------------------------------------

 200  continue

      call ice_timer_stop(1)       ! end timestepping loop timer
      call ice_timer_stop(0)       ! end total time timer
      call ice_timer_print(-1)     ! print all timing information

      if(prescribed_ice) call exit_prescribed_ice  ! close prescribed ice file


#ifndef coupled
      call end_run        
#else
      call exit_coupler        ! disconnect cice from the coupled system  
#endif

      stop
!
      end program icemodel

