c CVS: $Id: ice_diagnostics.F,v 1.10 2002/04/15 14:24:05 schramm Exp $
c CVS: $Source: /fs/cgd/csm/models/CVS.REPOS/ice/csim4_src/source/ice_diagnostics.F,v $
c CVS: $Name: ccsm2_0_beta46 $
!
!BOP
!
! !MODULE: ice_diagnostics - diagnostic information output during run
!
! !DESCRIPTION:
!
! Diagnostic information output during run
!
! !REVISION HISTORY:
!
! authors: Elizabeth C. Hunke, LANL 
!          Bruce P. Briegleb, NCAR
!
! !INTERFACE:
!
      module ice_diagnostics
!
! !USES:
!
      use ice_domain
      use ice_constants
!
!EOP
!
      implicit none

      ! for water and heat budgets
      real (kind=dbl_kind) ::
     &   totmn   ! total ice/snow water mass (nh)
     &,  totms   ! total ice/snow water mass (sh)
     &,  toten   ! total ice/snow enthalpy (J)
     &,  totes   ! total ice/snow enthalpy (J)

      ! printing info for routine print_state
      character (len=20) :: plabel
      integer (kind=int_kind), parameter ::
     &  check_step = 100000
     &, ip = 51
     &, jp = 37
     &, mtask = 2
!
      contains
!
!BOP
!
! !IROUTINE: runtime_diags - writes max,min,global sums to standard out
!
! !INTERFACE:
!
      subroutine runtime_diags
!
! !DESCRIPTION:
!
! Writes diagnostic info (max, min, global sums, etc) to standard out
!
! !REVISION HISTORY:
!
! authors: Elizabeth C. Hunke, LANL
!          Bruce P. Briegleb, NCAR
!          Cecilia M. Bitz, UW
!
! !USES:
!
      use ice_model_size
      use ice_flux
      use ice_albedo
      use ice_mpi_internal
      use ice_history
      use ice_grid
      use ice_calendar
      use ice_state
      use ice_dyn_evp
      use shr_sys_mod, only : shr_sys_flush
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      real (kind=dbl_kind) ::
     &   umaxn, hmaxn, shmaxn, arean, snwmxn, rnn, snn, frzn
     &,  umaxs, hmaxs, shmaxs, areas, snwmxs, rns, sns, frzs
     &,  etotn, hnetn, fhocnn, fhatmn, fhfrzn, atotn
     &,  etots, hnets, fhocns, fhatms, fhfrzs, atots
     &,  sfreshn, evpn, mtotn, micen, msnwn, fluxn
     &,  sfreshs, evps, mtots, mices, msnws, fluxs
     &,  delmin, delein, werrn, herrn
     &,  delmis, deleis, werrs, herrs
     &,  workn(imt_local,jmt_local), works(imt_local,jmt_local)
     &,  work1(ilo:ihi,jlo:jhi), work2(ilo:ihi,jlo:jhi), ftmp
      integer (kind=int_kind) :: n, layer, i,j
      logical (kind=log_kind) :: prwtbd   ! if true, print water budget terms
c      prwtbd = .false.
      prwtbd = .true.

      !-----------------------------------------------------------------
      ! state of the ice
      !-----------------------------------------------------------------

      ! maximum velocity  
      do j=1,jmt_local
       do i=1,imt_local
       workn(i,j) = max(abs(mask_n(i,j)*u(i,j)),abs(mask_n(i,j)*v(i,j)))
       works(i,j) = max(abs(mask_s(i,j)*u(i,j)),abs(mask_s(i,j)*v(i,j)))
       enddo
      enddo
      umaxn = ice_global_real_maxval(imt_local*jmt_local,WORKN)
      umaxs = ice_global_real_maxval(imt_local*jmt_local,WORKS)
      ! maximum effective thickness
      do j=1,jmt_local
       do i=1,imt_local
       workn(i,j) = mask_n(i,j)*vice(i,j)
       works(i,j) = mask_s(i,j)*vice(i,j)
       enddo
      enddo
      hmaxn = ice_global_real_maxval(imt_local*jmt_local,WORKN)
      hmaxs = ice_global_real_maxval(imt_local*jmt_local,WORKS)
      ! total ice area
      call get_sum(0,tarean,one,aice,arean)
      call get_sum(0,tareas,one,aice,areas)
      arean = arean * m2_to_km2
      areas = areas * m2_to_km2
      ! total ice volume
      call get_sum(0,tarean,one,vice,shmaxn)
      call get_sum(0,tareas,one,vice,shmaxs)
      ! total snow volume
      call get_sum(0,tarean,one,vsno,snwmxn)
      call get_sum(0,tareas,one,vsno,snwmxs)
      ! total ice/snow enthalpy
       do j=1,jmt_local
        do i=1,imt_local
         works(i,j) = c0
        enddo
       enddo
      do layer = 1,ntilay
       do j=1,jmt_local
        do i=1,imt_local
         works(i,j) = works(i,j) + eicen(i,j,layer) ! ice enthalpy
        enddo
       enddo
      enddo
      call get_sum(0,tarean,one,works,etotn)
      call get_sum(0,tareas,one,works,etots)
      etotn = etotn - snwmxn * rLfs   ! plus snow enthalpy
      etots = etots - snwmxs * rLfs 
      ! average ice albedo
       do j=jlo,jhi
        do i=ilo,ihi
         work1(i,j) = alvdr(i,j)*awtvdr + alidr(i,j)*awtidr 
     $              + alvdf(i,j)*awtvdf + alidf(i,j)*awtidf
        enddo
       enddo
      call get_sum(1,tarean,work1,aice,atotn)
      call get_sum(1,tareas,work1,aice,atots)
      if (arean .gt. 0.) then 
        atotn = atotn / arean * m2_to_km2  ! convert arean back to m2
      else
        atotn = c0
      endif
      if (areas .gt. 0.) then 
        atots = atots / areas * m2_to_km2
      else
        atots = c0
      endif

      !-----------------------------------------------------------------
      ! various fluxes
      !-----------------------------------------------------------------
      ! evap, Fresh, Fhnet, Fswthr, Fsensible, Flwout 
      ! need to be multiplied by aice because 
      ! regretfully they have been divided by aice for the coupler
      !-----------------------------------------------------------------

      ! evaporation
      call get_sum(1,tarean,evap,aice_init,evpn)	
      call get_sum(1,tareas,evap,aice_init,evps)	
      evpn = evpn*dt
      evps = evps*dt
      ! fresh water flux
      call get_sum(1,tarean,Frsh_hist,aice_init,sfreshn) 
      call get_sum(1,tareas,Frsh_hist,aice_init,sfreshs) 
      sfreshn = sfreshn*dt
      sfreshs = sfreshs*dt
      ! ocean heat

      call get_sum(1,tarean,Foht_hist+Fswthru_hist,aice_init,fhocnn)
      call get_sum(1,tareas,Foht_hist+Fswthru_hist,aice_init,fhocns)

      ! latent heat
      ! you may be wondering, where is the latent heat flux? 
      ! I do not include it here because it cancels with 
      ! the evaporative flux times the enthalpy of the  
      ! ice/snow that evaporated

      ! atmo heat flux
      ! Note: Flwout includes the reflected longwave down, needed by the
      ! atmosphere as an upwards radiative boundary condition.
      do j=jlo,jhi
       do i=ilo,ihi
        ftmp = Fhdiag(i,j) + (Fsensible(i,j) + Flwout(i,j))*
     &		aice_init(i,j)
        work1(i,j) = tarean(i,j)*ftmp
        work2(i,j) = tareas(i,j)*ftmp
       enddo
      enddo
      fhatmn = ice_global_real_sum((jhi-jlo+1)*(ihi-ilo+1),WORK1)
      fhatms = ice_global_real_sum((jhi-jlo+1)*(ihi-ilo+1),WORK2)
      ! freezing potential
      do j=jlo,jhi
       do i=ilo,ihi
        ftmp = max(c0,frzmlt(i,j))
        work1(i,j) = tarean(i,j)*ftmp
        work2(i,j) = tareas(i,j)*ftmp
       enddo
      enddo
      fhfrzn = ice_global_real_sum((jhi-jlo+1)*(ihi-ilo+1),WORK1)
      fhfrzs = ice_global_real_sum((jhi-jlo+1)*(ihi-ilo+1),WORK2)
      ! rain
      call get_sum(1,tarean,Frain,aice_init,rnn)	
      call get_sum(1,tareas,Frain,aice_init,rns)	
      rnn = rnn*dt
      rns = rns*dt
      ! snow
      call get_sum(1,tarean,Fsnow,aice_init,snn)	
      call get_sum(1,tareas,Fsnow,aice_init,sns)	
      snn = snn*dt
      sns = sns*dt
      ! frazil ice growth !! should not be multiplied by aice
      do j=jlo,jhi
       do i=ilo,ihi
        work1(i,j) = tarean(i,j)*frazil(i,j)
        work2(i,j) = tareas(i,j)*frazil(i,j)
       enddo
      enddo
      frzn = ice_global_real_sum((jhi-jlo+1)*(ihi-ilo+1),WORK1)
      frzs = ice_global_real_sum((jhi-jlo+1)*(ihi-ilo+1),WORK2)
      frzn = frzn*dt
      frzs = frzs*dt

      ! ice and snow mass
      micen = rhoi*shmaxn
      msnwn = rhos*snwmxn
      mices = rhoi*shmaxs
      msnws = rhos*snwmxs

      mtotn = micen + msnwn
      mtots = mices + msnws

      delmin = mtotn - totmn
      delmis = mtots - totms

      ! total water flux
      fluxn  = c0
      fluxs  = c0
      if( arean .gt. 0. ) then
        fluxn = rnn + snn + evpn - sfreshn + frzn
      endif
      if( areas .gt. 0. ) then
        fluxs = rns + sns + evps - sfreshs + frzs
      endif
      
      werrn = (fluxn-delmin)/(mtotn+c1)
      werrs = (fluxs-delmis)/(mtots+c1)

      ! enthalpy change
      delein = etotn - toten
      deleis = etots - totes

      fhatmn = fhatmn + ( - snn * Lfus + evpn * Lvap ) / dt 
      fhatms = fhatms + ( - sns * Lfus + evps * Lvap ) / dt 

      hnetn = (fhatmn - fhocnn - fhfrzn) * dt
      hnets = (fhatms - fhocns - fhfrzs) * dt

      herrn = (hnetn - delein) / (etotn - c1)
      herrs = (hnets - deleis) / (etots - c1)

      !-----------------------------------------------------------------
      ! start spewing
      !-----------------------------------------------------------------
      if (my_task.eq.master_task) then

        write (6,899) 'Arctic','Antarctic'

        write (6,900) 'max cH (m)             = ',hmaxn,hmaxs
        write (6,901) 'total area (km^2)      = ',arean,areas
        write (6,901) 'total ice volume (m^3) = ',shmaxn,shmaxs
        write (6,901) 'total snw volume (m^3) = ',snwmxn,snwmxs
        write (6,900) 'max u, v (m/s)         = ',umaxn,umaxs
        write (6,900) 'average albedo         = ',atotn,atots

        call shr_sys_flush(6)

        if( prwtbd ) then  
  
          write (6,902) 'arwt rain h2o kg in dt = ',rnn,rns
          write (6,902) 'arwt snow h2o kg in dt = ',snn,sns
          write (6,902) 'arwt evap h2o kg in dt = ',evpn,evps
          write (6,902) 'arwt frzl h2o kg in dt = ',frzn,frzs
          write (6,902) 'arwt frsh h2o kg in dt = ',sfreshn,sfreshs

          write (6,902) 'arwt ice mass (kg)     = ',micen,mices
          write (6,902) 'arwt snw mass (kg)     = ',msnwn,msnws

          write (6,902) 'arwt tot mass (kg)     = ',mtotn,mtots
          write (6,902) 'arwt tot mass chng(kg) = ',delmin,delmis
          write (6,902) 'arwt water flux        = ',fluxn,fluxs
          write (6,*) '(=rain+snow+evap+frzl-fresh)  '
          write (6,902) 'water flux error       = ',werrn,werrs

          write (6,*) '----------------------------'
          write (6,902) 'arwt atm heat flux (W) = ',fhatmn,fhatms
          write (6,902) 'arwt ocn heat flux (W) = ',fhocnn,fhocns
          write (6,902) 'arwt frzl heat flux(W) = ',fhfrzn,fhfrzs
          write (6,902) 'arwt tot enthalpy  (J) = ',etotn,etots
          write (6,902) 'arwt net heat      (J) = ',hnetn,hnets
          write (6,902) 'arwt tot enth. chng(J) = ',delein,deleis
          write (6,902) 'arwt heat error        = ',herrn,herrs
          write (6,*) '----------------------------'

          call shr_sys_flush(6)

        endif
      endif

 899  format (27x,a24,2x,a24)
 900  format (a25,2x,f24.17,2x,f24.17)
 901  format (a25,2x,1pe24.17,2x,1pe24.17)
 902  format (a25,2x,1pe24.17,2x,1pe24.17)

      end subroutine runtime_diags
!
!BOP
!
! !IROUTINE: init_mass_diags - computes global combined ice and snow mass sum
!
! !INTERFACE:
!
      subroutine init_mass_diags
!
! !DESCRIPTION:
!
! Computes global combined ice and snow mass sum
!
! !REVISION HISTORY:
!
! author: Elizabeth C. Hunke, LANL
!
! !USES:
!
      use ice_mpi_internal
      use ice_grid
      use ice_state
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      real (kind=dbl_kind) ::
     &   shmaxn, snwmxn,  shmaxs, snwmxs

      real (kind=dbl_kind) ::
     &   works(imt_local,jmt_local)

      integer (kind=int_kind) :: i,j,layer

      ! total ice volume
      call get_sum(0,tarean,one,vice,shmaxn)
      call get_sum(0,tareas,one,vice,shmaxs)
      ! total snow volume
      call get_sum(0,tarean,one,vsno,snwmxn)
      call get_sum(0,tareas,one,vsno,snwmxs)

      ! north/south ice+snow volume
      totmn = rhoi*shmaxn + rhos*snwmxn
      totms = rhoi*shmaxs + rhos*snwmxs

      ! north/south ice+snow enthalpy
      ! total ice/snow enthalpy
       do j=1,jmt_local
        do i=1,imt_local
         works(i,j) = c0
        enddo
       enddo
      do layer = 1,ntilay
       do j=1,jmt_local
        do i=1,imt_local
         works(i,j) = works(i,j) + eicen(i,j,layer) ! ice enthalpy
        enddo
       enddo
      enddo
      call get_sum(0,tarean,one,works,toten)
      call get_sum(0,tareas,one,works,totes)
      toten = toten - snwmxn * rLfs   ! plus snow enthalpy
      totes = totes - snwmxs * rLfs 

      end subroutine init_mass_diags
!
!BOP
!
! !IROUTINE: init_diags - initialize diagnostic output
!
! !INTERFACE:
!
      subroutine init_diags
!
! !DESCRIPTION:
!
! Initialize diagnostic output
!
! !REVISION HISTORY:
!
! author: Elizabeth C. Hunke, LANL
!
! !USES:
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      totmn  = c0      ! initial water budget term
      totms  = c0      ! initial water budget term

      end subroutine init_diags
!
!BOP
!
! !IROUTINE: print_state - print ice state for specified grid point
!
! !INTERFACE:
!
      subroutine print_state(plabel,i,j)
!
! !DESCRIPTION:
!
! This routine is useful for debugging call to it should be inserted 
! in the form (after thermo, for example)
!
!      do j=jlo,jhi    \\
!      -do i=ilo,ihi   \\ 
!      ---call to\_column(i,j)    \\
!      ---plabel = 'post thermo' \\
!      ---if (istep1.ge.check\_step.and.i.eq.ip.and.j.eq.jp  \\
!      ------.and.my\_task.eq.mtask)  \\
!      ---call print\_state(plabel,i,j) \\
!      -enddo  \\
!      enddo   \\
!
! 'use ice\_diagnostics' may need to be inserted also, and \\
! 'use ice\_calendar' if it is not already being used
!
! !REVISION HISTORY:
!
! author: Elizabeth C. Hunke, LANL
!
! !USES:
!
      use ice_model_size
      use ice_kinds_mod
      use ice_calendar
      use ice_state
      use ice_grid
      use ice_itd
      use ice_flux
!
! !INPUT/OUTPUT PARAMETERS:
!
      character (len=20), intent(in) :: plabel
      integer (kind=int_kind), intent(in) :: i,j
!
!EOP
!
      real (kind=dbl_kind) :: edebug
      integer (kind=int_kind) :: nc, layer
      integer (kind=int_kind) :: nunit

      nunit = 55

      call to_column(i,j)

      write (nunit,*) plabel,' istep1 = ',istep1
      write (nunit,*) 'my_task',my_task,' i, j=',i,j 
      write (nunit,*) 'aice  =',aice(i,j)
      write (nunit,*) 'tmask =',tmask(i,j)
      do nc=1,ncat

        write (nunit,*) ' '
        write (nunit,*) 'nc=',nc
        write (nunit,*) 'ain', ain(nc)
        write (nunit,*) 'vin',vin(nc)
        if (ain(nc).gt.puny) write (nunit,*) 'hin',vin(nc)/ain(nc)
        write (nunit,*) 'vsn',vsn(nc)
        write (nunit,*) 'Tsfn',Tsfn(nc)
        write (nunit,*) ' '

      enddo

      edebug = c0
      do nc = 1,ncat
        do layer = 1,nilay(nc)
          write (nunit,*) 'ein, cat ',nc,' layer ',layer,ein(layer,nc)
          edebug = edebug+ein(layer,nc)
        enddo
      enddo
      write (nunit,*) 'eice(i,j)',edebug 
      write (nunit,*) ' '
      write (nunit,*) 'atm states and fluxes'
      write (nunit,*) '            zlvl  = ',zlvl(i,j)
      write (nunit,*) '            uatm  = ',uatm(i,j)
      write (nunit,*) '            vatm  = ',vatm(i,j)
      write (nunit,*) '            potT  = ',potT(i,j)
      write (nunit,*) '            Tair  = ',Tair(i,j)
      write (nunit,*) '            Qa    = ',Qa(i,j)
      write (nunit,*) '            rhoa  = ',rhoa(i,j)
      write (nunit,*) '            swvdr = ',swvdr(i,j)
      write (nunit,*) '            swvdf = ',swvdf(i,j)
      write (nunit,*) '            swidr = ',swidr(i,j)
      write (nunit,*) '            swidf = ',swidf(i,j)
      write (nunit,*) '            Flw   = ',Flw(i,j)
      write (nunit,*) '            Frain = ',Frain(i,j)
      write (nunit,*) '            Fsnow = ',Fsnow(i,j)
      write (nunit,*) ' '
      write (nunit,*) 'ocn states and fluxes'
      write (nunit,*) '            frzmlt = ',frzmlt(i,j)
      write (nunit,*) '            sst    = ',sst(i,j)
      write (nunit,*) '            sss    = ',sss(i,j)
      write (nunit,*) '            Tf     = ',Tf(i,j)
      write (nunit,*) ' '
      write (nunit,*) 'srf states and fluxes'
      write (nunit,*) '            Tref      = ',Tref(i,j)
      write (nunit,*) '            Fsensible = ',Fsensible(i,j)
      write (nunit,*) '            Flatent   = ',Flatent(i,j)
      write (nunit,*) '            evap      = ',evap(i,j)
      write (nunit,*) '            Flwout    = ',Flwout(i,j)
      write (nunit,*) ' '

      end subroutine print_state
!
      end module ice_diagnostics
!
