c CVS: $Id: ice_history.F,v 1.22 2002/04/30 16:01:18 schramm Exp $ 
c CVS: $Source: /fs/cgd/csm/models/CVS.REPOS/ice/csim4_src/source/ice_history.F,v $
c CVS: $Name: ccsm2_0_beta46 $
!
!BOP
!
! !MODULE: ice_history - ice model history and restart files
!
! !DESCRIPTION:
!
! Output files: netCDF data, Fortran unformatted dumps
!
! !REVISION HISTORY:
!
! authors Tony Craig, NCAR
!         Elizabeth C. Hunke, LANL
!         C.M. Bitz, UW
!         Bruce P. Briegleb, NCAR
!
! !INTERFACE:
!
      module ice_history
!
! !USES:
!
      use ice_kinds_mod
      use ice_domain
      use ice_read_write
!
!EOP
!
      implicit none

      logical (kind=log_kind) ::
     &   restart ! if true, initialize using restart file instead of defaults
     &,  hist_avg  ! if true, write averaged data instead of snapshots

      character (len=char_len) :: 
     &   dump_file         ! output file for restart dump
     &,  history_file      ! output file for history 
     &,  incond_file       ! output file for snapshot initial conditions
     &,  runtype           ! initial, continue, or branch

      character (len=char_len_long) :: 
     &   restart_dir         ! directory name for restart dump
     &,  history_dir         ! directory name for history file
     &,  incond_dir          ! directory for snapshot initial conditions

      character (char_len_long) :: 
     &   runid             ! identifier for the run
     &,  pointer_file      !  input pointer file for restarts

      ! extra arrays used only for holding history information
      real (kind=dbl_kind), dimension (ilo:ihi,jlo:jhi) ::
     &   growb     ! basal ice growth         (m)
     &,  frazil    ! frazil ice growth        (m)
     &,  snoice    ! snow-ice formation       (m)
     &,  meltt     ! top ice melt             (m)
     &,  meltb     ! basal ice melt           (m)
     &,  meltl     ! lateral ice melt         (m)
     &,  sabs      ! ice/ocn solar absrbd flx (W/m**2)
     &,  daidtt    ! ice area tendency thermo. (s^-1)
     &,  daidtd    ! ice area tendency advect. (s^-1)
     &,  dvidtt    ! ice vol. tendency thermo. (m/s)
     &,  dvidtd    ! ice vol. tendency advect. (m/s)

      !---------------------------------------------------------------
      ! primary info for the history file
      !---------------------------------------------------------------

      !----- mass store mswrite options ------------------------------
      character(char_len_long) :: mss_dir     ! MSS archive directory name
      character(len=16) :: mss_pass    ! MSS password
      integer           :: mss_rtpd    ! MSS retention period
      integer           :: mss_rmlf    ! T => remove local files after mswrite
      integer           :: rcode       ! mswrite error code

      integer (kind=int_kind), parameter ::
     &   avgsiz = 64  ! number of fields to be averaged

      real (kind=real_kind) :: time_beg, time_end ! bounds for averaging

      real (kind=dbl_kind) ::
     &   avgct    ! average sample counter
     &,  aa(ilo:ihi,jlo:jhi,avgsiz) ! field accumulations and averages
     &,  ice_present(ilo:ihi,jlo:jhi) ! fraction of avg any ice is present
     &,  cona(avgsiz)
     &,  conb(avgsiz)

      logical (kind=log_kind) ::
     &   iout(avgsiz)     ! logical if field should be written

      character (len=16) ::
     &   vname(avgsiz)     ! variable names
     &,  vunit(avgsiz)     ! variable units
     &,  vcoord(avgsiz)='TLON TLAT time'    ! variable coordinates
      character (len=55) :: 
     &   vdesc(avgsiz)     ! variable descriptions
     &,  vcomment(avgsiz)  ! variable comments

      data vname / 
     &     'hi',      'hs',         'Tsfc',    'aice',
     &     'aice1',   'aice2',      'aice3',   'aice4',   'aice5',
     &     'u',       'v',          'Fswdn',   'Flwdn',   
     &     'snow',    'snow_aice',  'rain',    'rain_aice', 
     &     'sst',     'sss',        'uocn',    'vocn',    'frzmlt',
     &     'Fswabs',  'Fswabs_aice','albsni',  'Flat',    'Flat_aice',
     &     'Fsens',   'Fsens_aice', 'Flwout',  'Flwout_aice',
     &     'evap',    'evap_aice',  'Tref',    'growb',   'frazil', 
     &     'snoice',  'meltb',      'meltt',   'meltl',  
     &     'Fresh',   'Fresh_aice', 'Fhnet',   'Fhnet_aice', 
     &     'strairx', 'strairy',    'strtltx', 'strtlty',     
     &     'strcorx', 'strcory',    'strocnx', 'strocny',    
     &     'strintx', 'strinty',    'strength','divu',    
     &     'shear',   'opening',    'sig1',    'sig2',    
     &     'dvidtt',  'dvidtd',     'daidtt',  'daidtd'/       
      data vdesc / 
     &  'grid box mean ice thickness'  ,'grid box mean snow thickness'
     & ,'snow/ice surface temperature'      
     & ,'aggregate ice area'       ,'ice area (category 1)' 
     & ,'ice area (category 2)'    ,'ice area (category 3)' 
     & ,'ice area (category 4)'    ,'ice area (category 5)' 
     & ,'zonal ice velocity'       ,'meridional ice velocity'
     & ,'down solar flux'          ,'down longwave flux'
     & ,'snow fall rate received from coupler' 
     & ,'snow fall rate on ice cover' 
     & ,'rain fall rate received from coupler'
     & ,'rain fall rate on ice cover'
     & ,'sea surface temperature'  ,'sea surface salinity'
     & ,'zonal ocean current'      ,'meridional ocean current' 
     & ,'negative surface ocean heat content'    
     & ,'snow/ice/ocn absorbed solar flux sent to coupler'
     & ,'snow/ice/ocn absorbed solar flux'
     & ,'snw/ice broad band albedo'
     & ,'latent heat flux sent to coupler' ,'ice/atm latent heat flux'
     & ,'sensible heat flux sent to coupler' ,'ice/atm sensible heat flux'
     & ,'outgoing long wave flux sent to coupler'
     & ,'ice/atm outgoing long wave flux'
     & ,'evaporative water flux sent to coupler'   
     & ,'ice/atm evaporative water flux'   
     & ,'reference temperature'
     & ,'basal ice growth'         ,'frazil ice growth'
     & ,'snow-ice conversion'      ,'basal ice melt'
     & ,'top ice melt'             ,'lateral ice melt'
     & ,'ice/ocn fresh water flux sent to coupler'
     & ,'ice/ocn fresh water flux'
     & ,'ice/ocn net heat flux sent to coupler'       
     & ,'ice/ocn net heat flux'       
     & ,'zonal atm/ice stress'     ,'meridional atm/ice stress'
     & ,'zonal sea surface tilt'   ,'meridional sea surface tilt'
     & ,'zonal coriolis stress'    ,'meridional coriolis stress'
     & ,'zonal ocean/ice stress'   ,'meridional ocean/ice stress' 
     & ,'zonal divergence of internal ice stress tensor'
     & ,'meridional divergence of internal ice stress tensor'  
     & ,'compressive ice strength' ,'strain rate (divergence)' 
     & ,'strain rate (shear)'      ,'lead opening rate'        
     & ,'normalized principal stress component' 
     & ,'normalized principal stress component' 
     & ,'ice volume tendency due to thermodynamics' 
     & ,'ice volume tendency due to dynamics' 
     & ,'area tendency due to thermodynamics'
     & ,'area tendency due to dynamics' /

      data vcomment / 
     &     'Ice volume distributed evenly over grid cell',   ! hi
     &     'Snow volume distributed evenly over grid cell',  ! hs 
     &     'Tf is averaged in if no ice is present',         ! Tsfc
     &     'none',                                           ! aice
     &     'Ice range:', 'Ice range:', 'Ice range:', 'Ice range:', 'Ice range:',
     &     'positive is x direction on U grid',              !u                     
     &     'positive is y direction on U grid',              !v                    
     &     'positive downward',   'positive downward',       ! Fswdn Flwdn
     &     'none', 'weighted by ice area',                   !snow snow_aice
     &     'none', 'weighted by ice area',                   !rain rain_aice
     &     'none', 'none',                                   ! sst sss
     &     'positive is x direction on U grid',              !uocn                     
     &     'positive is y direction on U grid',              !vocn                  
     &     'negative (sst<Tf) represents ice formation using frzmlt',     !frzmlt
     &     'positive downward',   'weighted by ice area',    ! Fswabs Fswabs_aice
     &     'none',                                           ! albsni
     &     'positive downward',   'weighted by ice area',    ! Flat Flat_aice
     &     'positive downward',   'weighted by ice area',    ! Fsens Fsens_aice
     &     'positive downward',   'weighted by ice area',    ! Flwout Flwout_aice
     &     'none', 'weighted by ice area',                   ! evap evap_aice
     &     'none', 'none', 'none', 'none',                   ! Tref growb frazil snoice
     &     'none', 'none', 'none',                           ! meltb meltt meltl
     &     'positive represents ocean gaining fresh water',  ! Fresh
     &     'weighted by ice area',                           ! Fresh_aice
     &     'positive represents ocean gaining heat',         ! Fhnet
     &     'weighted by ice area',                           ! Fhnet_aice
     &     'positive is x direction on U grid',              ! strairx
     &     'positive is y direction on U grid',              ! strairy
     &     'none', 'none',                                   ! strtltx strtlty
     &     'positive is x direction on U grid',              ! strcorx
     &     'positive is y direction on U grid',              ! strcory
     &     'positive is x direction on U grid',              ! strocnx
     &     'positive is y direction on U grid',              ! strocny
     &     'positive is x direction on U grid',              ! strintx
     &     'positive is y direction on U grid',              ! strinty
     &     'none', 'none', 'none', 'none',                   ! strength divu shear opening
     &     'sig1 is instantaneous','sig2 is instantaneous',  ! sig1 sig2
     &     'none', 'none', 'none', 'none'/                   ! dvidtt dvidtd daidtt daidtd

      data vunit / 
     &  'm',      'm',      'Celsius', '%',               ! hi hs Tsfc aice
     &  '%',      '%',      '%',       '%',      '%',     ! aice1-5
     &  'cm/s',   'cm/s',   'W/m^2',   'W/m^2',           ! u v Fswdn Flwdn
     &  'cm/day', 'cm/day', 'cm/day',  'cm/day',          ! snow snow_aice rain rain_aice
     &  'Celsius','g/kg',   'cm/s',    'cm/s',   'W/m^2', ! sst sss uocn vocn frzmlt 
     &  'W/m^2',  'W/m^2',  '%',       'W/m^2',  'W/m^2',
     &  'W/m^2',  'W/m^2',  'W/m^2',   'W/m^2',           ! Fsens Fsens_aice Flwout Flwout_aice
     &  'cm/day', 'cm/day', 'Celsius', 'cm/day',  'cm/day', ! evap evap_aice Tref growb frazil
     &  'cm/day', 'cm/day', 'cm/day',  'cm/day',          ! snoice meltb meltt meltl
     &  'cm/day', 'cm/day', 'W/m^2',   'W/m^2',   ! Fresh Fresh_aice Fhnet Fhnet_aice
     &  'N/m^2',  'N/m^2',  'm/m',     'm/m',     
     &  'N/m^2',  'N/m^2',  'N/m^2',   'N/m^2',     
     &  'N/m^2',  'N/m^2',  'N/m',     '%/day',    
     &  '%/day',  '%/day',  '1 ',      '1 ',    
     &  'cm/day', 'cm/day', '%/day',   '%/day' /

      logical
     &     f_hi,      f_hs,          f_Tsfc,     f_aice,
     &     f_aice1,   f_aice2,       f_aice3,    f_aice4,    f_aice5,
     &     f_u,       f_v,           f_Fswdn,    f_Flwdn,   
     &     f_snow,    f_snow_aice,   f_rain,     f_rain_aice,
     &     f_sst,     f_sss,         f_uocn,     f_vocn,     f_frzmlt,  
     &     f_Fswabs,  f_Fswabs_aice, f_albsni,   f_Flat,     f_Flat_aice,
     &     f_Fsens,   f_Fsens_aice , f_Flwout,   f_Flwout_aice,
     &     f_evap,    f_evap_aice,   f_Tref,     f_growb,    f_frazil, 
     &     f_snoice,  f_meltb,       f_meltt,    f_meltl,
     &     f_Fresh,   f_Fresh_aice,  f_Fhnet,    f_Fhnet_aice, 
     &     f_strairx, f_strairy,     f_strtltx,  f_strtlty,     
     &     f_strcorx, f_strcory,     f_strocnx,  f_strocny,    
     &     f_strintx, f_strinty,     f_strength, f_divu,    
     &     f_shear,   f_opening,     f_sig1,     f_sig2,    
     &     f_dvidtt,  f_dvidtd,      f_daidtt,   f_daidtd

      namelist / icefields_nml /
     &     f_hi,      f_hs,          f_Tsfc,     f_aice,
     &     f_aice1,   f_aice2,       f_aice3,    f_aice4,    f_aice5,
     &     f_u,       f_v,           f_Fswdn,    f_Flwdn,   
     &     f_snow,    f_snow_aice,   f_rain,     f_rain_aice,
     &     f_sst,     f_sss,         f_uocn,     f_vocn,     f_frzmlt,  
     &     f_Fswabs,  f_Fswabs_aice, f_albsni,   f_Flat,     f_Flat_aice,
     &     f_Fsens,   f_Fsens_aice , f_Flwout,   f_Flwout_aice,
     &     f_evap,    f_evap_aice,   f_Tref,     f_growb,    f_frazil, 
     &     f_snoice,  f_meltb,       f_meltt,    f_meltl,
     &     f_Fresh,   f_Fresh_aice,  f_Fhnet,    f_Fhnet_aice, 
     &     f_strairx, f_strairy,     f_strtltx,  f_strtlty,     
     &     f_strcorx, f_strcory,     f_strocnx,  f_strocny,    
     &     f_strintx, f_strinty,     f_strength, f_divu,    
     &     f_shear,   f_opening,     f_sig1,     f_sig2,    
     &     f_dvidtt,  f_dvidtd,      f_daidtt,   f_daidtd

      equivalence (iout(1),f_hi)
      equivalence (iout(2),f_hs)
      equivalence (iout(3),f_Tsfc)
      equivalence (iout(4),f_aice)
      equivalence (iout(5),f_aice1)
      equivalence (iout(6),f_aice2)
      equivalence (iout(7),f_aice3)
      equivalence (iout(8),f_aice4)
      equivalence (iout(9),f_aice5)
      equivalence (iout(10),f_u)
      equivalence (iout(11),f_v)

      equivalence (iout(12),f_Fswdn)
      equivalence (iout(13),f_Flwdn)
      equivalence (iout(14),f_snow)
      equivalence (iout(15),f_snow_aice)
      equivalence (iout(16),f_rain)
      equivalence (iout(17),f_rain_aice)

      equivalence (iout(18),f_sst)
      equivalence (iout(19),f_sss)
      equivalence (iout(20),f_uocn)
      equivalence (iout(21),f_vocn)
      equivalence (iout(22),f_frzmlt)

      equivalence (iout(23),f_Fswabs)
      equivalence (iout(24),f_Fswabs_aice)
      equivalence (iout(25),f_albsni)
      equivalence (iout(26),f_Flat)
      equivalence (iout(27),f_Flat_aice)
      equivalence (iout(28),f_Fsens)
      equivalence (iout(29),f_Fsens_aice)
      equivalence (iout(30),f_Flwout)
      equivalence (iout(31),f_Flwout_aice)
      equivalence (iout(32),f_evap)
      equivalence (iout(33),f_evap_aice)
      equivalence (iout(34),f_Tref)

      equivalence (iout(35),f_growb)
      equivalence (iout(36),f_frazil)
      equivalence (iout(37),f_snoice)
      equivalence (iout(38),f_meltb)
      equivalence (iout(39),f_meltt)
      equivalence (iout(40),f_meltl)
      equivalence (iout(41),f_Fresh)
      equivalence (iout(42),f_Fresh_aice)
      equivalence (iout(43),f_Fhnet)
      equivalence (iout(44),f_Fhnet_aice)

      equivalence (iout(45),f_strairx)
      equivalence (iout(46),f_strairy)
      equivalence (iout(47),f_strtltx)
      equivalence (iout(48),f_strtlty)
      equivalence (iout(49),f_strcorx)
      equivalence (iout(50),f_strcory)
      equivalence (iout(51),f_strocnx)
      equivalence (iout(52),f_strocny)
      equivalence (iout(53),f_strintx)
      equivalence (iout(54),f_strinty)
      equivalence (iout(55),f_strength)
      equivalence (iout(56),f_divu)
      equivalence (iout(57),f_shear)
      equivalence (iout(58),f_opening)
      equivalence (iout(59),f_sig1)
      equivalence (iout(60),f_sig2)

      equivalence (iout(61),f_dvidtt)
      equivalence (iout(62),f_dvidtd)
      equivalence (iout(63),f_daidtt)
      equivalence (iout(64),f_daidtd)
!
      contains
!
!BOP
!
! !IROUTINE: init_hist - initialize history files
!
! !INTERFACE:
!
      subroutine init_hist
!
! !DESCRIPTION:
!
! Initialize history files
!
! !REVISION HISTORY:
!
! authors Tony Craig, NCAR
!         Elizabeth C. Hunke, LANL
!         C.M. Bitz, UW
!         Bruce P. Briegleb, NCAR
!
! !USES:
!
      use ice_constants
      use ice_calendar
      use shr_sys_mod, only : shr_sys_flush, shr_sys_abort
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) :: n, k
      integer (kind=int_kind) :: nml_error ! namelist i/o error flag

      iout=.true.

      !-----------------------------------------------------------------
      ! read namelist
      !-----------------------------------------------------------------
      if (my_task == master_task) then
        open (21, file='ice_in', status='old')
   10   continue  !*** keep reading until right namelist is found
        read(21, nml=icefields_nml, iostat=nml_error)
        if (nml_error > 0) goto 10          ! An error occurred
        if (nml_error < 0) goto 20          ! End of file condition
        close(21)
   20   continue
      endif
      call ice_bcast_iscalar(nml_error)
      if (nml_error /= 0) then
        call shr_sys_abort ('ice: Namelist read error in ice_history.F')
      endif

      call ice_bcast_logical (f_hi)
      call ice_bcast_logical (f_hs)
      call ice_bcast_logical (f_Tsfc)
      call ice_bcast_logical (f_aice)
      call ice_bcast_logical (f_aice1)
      call ice_bcast_logical (f_aice2)
      call ice_bcast_logical (f_aice3)
      call ice_bcast_logical (f_aice4)
      call ice_bcast_logical (f_aice5)
      call ice_bcast_logical (f_u)
      call ice_bcast_logical (f_v)
      call ice_bcast_logical (f_Fswdn)
      call ice_bcast_logical (f_Flwdn)
      call ice_bcast_logical (f_snow)
      call ice_bcast_logical (f_snow_aice)
      call ice_bcast_logical (f_rain)
      call ice_bcast_logical (f_rain_aice)
      call ice_bcast_logical (f_sst)
      call ice_bcast_logical (f_sss)
      call ice_bcast_logical (f_uocn)
      call ice_bcast_logical (f_vocn)
      call ice_bcast_logical (f_frzmlt)
      call ice_bcast_logical (f_Fswabs)
      call ice_bcast_logical (f_Fswabs_aice)
      call ice_bcast_logical (f_albsni)
      call ice_bcast_logical (f_Flat)
      call ice_bcast_logical (f_Flat_aice)
      call ice_bcast_logical (f_Fsens)
      call ice_bcast_logical (f_Fsens_aice)
      call ice_bcast_logical (f_Flwout)
      call ice_bcast_logical (f_Flwout_aice)
      call ice_bcast_logical (f_evap)
      call ice_bcast_logical (f_evap_aice)
      call ice_bcast_logical (f_Tref)
      call ice_bcast_logical (f_growb)
      call ice_bcast_logical (f_frazil)
      call ice_bcast_logical (f_snoice)
      call ice_bcast_logical (f_meltb)
      call ice_bcast_logical (f_meltt)
      call ice_bcast_logical (f_meltl)
      call ice_bcast_logical (f_Fresh)
      call ice_bcast_logical (f_Fresh_aice)
      call ice_bcast_logical (f_Fhnet)
      call ice_bcast_logical (f_Fhnet_aice)
      call ice_bcast_logical (f_strairx)
      call ice_bcast_logical (f_strairy)
      call ice_bcast_logical (f_strtltx)
      call ice_bcast_logical (f_strtlty)
      call ice_bcast_logical (f_strcorx)
      call ice_bcast_logical (f_strcory)
      call ice_bcast_logical (f_strocnx)
      call ice_bcast_logical (f_strocny)
      call ice_bcast_logical (f_strintx)
      call ice_bcast_logical (f_strinty)
      call ice_bcast_logical (f_strength)
      call ice_bcast_logical (f_divu)
      call ice_bcast_logical (f_shear)
      call ice_bcast_logical (f_opening)
      call ice_bcast_logical (f_sig1)
      call ice_bcast_logical (f_sig2)
      call ice_bcast_logical (f_dvidtt)
      call ice_bcast_logical (f_dvidtd)
      call ice_bcast_logical (f_daidtt)
      call ice_bcast_logical (f_daidtd)

      if (my_task.eq.master_task) then
        write (6,*) 'The following variables will be',
     &    ' written to the history tape: '
        write (6,*) ' Description                             Units',
     &    '     netCDF variable'
        do n=1,avgsiz
           if (iout(n))   write (6,1000) vdesc(n), vunit(n), vname(n)
        enddo
        write (6,*) ' '
        call shr_sys_flush(6)
      endif
1000  format (1x,A55,2x,A10,2x,A10)

      !-----------------------------------------------------------------
      ! initialize the history arrays
      !-----------------------------------------------------------------
      aa = c0
      avgct = c0
      ice_present = c0

      do k=1,avgsiz
        cona(k) = c1
        conb(k) = c0
      enddo

      ! set up modified factors   
      cona( 4) = c100                    ! ai  fraction to %
      cona( 5) = c100                    ! ai1 fraction to %
      cona( 6) = c100                    ! ai2 fraction to %
      cona( 7) = c100                    ! ai3 fraction to %
      cona( 8) = c100                    ! ai4 fraction to %
      cona( 9) = c100                    ! ai5 fraction to %
      cona(10) = m_to_cm                 ! u m/s to cm/s
      cona(11) = m_to_cm                 ! v m/s to cm/s

      cona(14) = mps_to_cmpdy/rhofresh   ! snow kg/m2/s to cm/day
      cona(15) = mps_to_cmpdy/rhofresh   ! snow_aice kg/m2/s to cm/day
      cona(16) = mps_to_cmpdy/rhofresh   ! rain kg/m2/s to cm/day
      cona(17) = mps_to_cmpdy/rhofresh   ! rain_aice kg/m2/s to cm/day

      cona(20) = m_to_cm                 ! uocn m/s to cm/s
      cona(21) = m_to_cm                 ! vocn m/s to cm/s

      cona(25) = c100                    ! avg of spectral 4 albedos to %
      cona(32) = mps_to_cmpdy/rhofresh   ! evap kg/m2/s to cm/day 
      cona(33) = mps_to_cmpdy/rhofresh   ! evap_aice kg/m2/s to cm/day 
      conb(34) = -Tffresh                ! Tref K to C

      cona(35) = mps_to_cmpdy/dt         ! growb  m per step to cm/day
      cona(36) = mps_to_cmpdy/rhoi       ! frazil kg/m2/s to cm/day
      cona(37) = mps_to_cmpdy/dt         ! snoice m per step to cm/day
      cona(38) = mps_to_cmpdy/dt         ! meltb  m per step to cm/day
      cona(39) = mps_to_cmpdy/dt         ! meltt  m per step to cm/day
      cona(40) = mps_to_cmpdy/dt         ! meltl  m per step to cm/day
      cona(41) = mps_to_cmpdy/rhofresh   ! fresh water flux kg/m2/s to cm/day 
      cona(42) = mps_to_cmpdy/rhofresh   ! Fresh_aice  flux kg/m2/s to cm/day 

      cona(56) = secday*c100             ! divu from 1/s to %/day
      cona(57) = secday*c100             ! shear from 1/s to %/day
      cona(58) = secday*c100             ! opening from 1/s to %/day

      cona(61) = mps_to_cmpdy            ! dvidtt m/s to cm/day
      cona(62) = mps_to_cmpdy            ! dvidtd m/s to cm/day
      cona(63) = secday*c100             ! daidtt frac/s to %/day
      cona(64) = secday*c100             ! daidtd frac/s to %/day

!-------------------------------------------------------------------
! Change coordinates of variables printed out on u grid
!-------------------------------------------------------------------

      if (my_task == master_task) then
        do k=1,avgsiz
          if (TRIM(vname(k)) == 'u') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'v') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'uocn') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'vocn') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'strairx') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'strairy') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'strtltx') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'strtlty') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'strcorx') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'strcory') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'strocnx') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'strocny') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'strintx') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'strinty') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'sig1') vcoord(k) = 'ULON ULAT time'
          if (TRIM(vname(k)) == 'sig2') vcoord(k) = 'ULON ULAT time'
        enddo
      endif

      end subroutine init_hist
!
!BOP
!
! !IROUTINE: ice_write_hist - write average ice quantities or snapshots
!
! !INTERFACE:
!
      subroutine ice_write_hist
!
! !DESCRIPTION:
!
! write average ice quantities or snapshots
!
! !REVISION HISTORY:
!
! author:   E.C.Hunke, LANL
! 
! !USES:
!
      use ice_flux
      use ice_albedo
      use ice_mechred
      use ice_grid
      use ice_calendar
      use ice_state
      use ice_dyn_evp
      use ice_constants
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) :: i,j,k

      ! ice vol. tendency for history, due to dynamics
      do j=jlo,jhi
        do i=ilo,ihi
           dvidtd(i,j) = (vice(i,j)-dvidtd(i,j))/dt
           daidtd(i,j) = (aice(i,j)-daidtd(i,j))/dt
        enddo
      enddo

      if (.not. hist_avg) then  ! write snapshots
        aa = c0
        avgct = c1
      else                      ! write averages over time histfreq
        avgct = avgct + c1
        if (avgct == c1) time_beg = time/int(secday)
      endif

      do j=jlo,jhi
       do i=ilo,ihi
        if (aice(i,j).gt.puny) ice_present(i,j) = ice_present(i,j) + c1
        if (iout( 1)) aa(i,j, 1) = aa(i,j, 1) + vice(i,j)
        if (iout( 2)) aa(i,j, 2) = aa(i,j, 2) + vsno(i,j)
        if (iout( 3)) aa(i,j, 3) = aa(i,j, 3) + Tsfc(i,j)
        if (iout( 4)) aa(i,j, 4) = aa(i,j, 4) + aice(i,j)

        if (ncat .ge. 5 ) then
          if (iout( 5)) aa(i,j, 5) = aa(i,j, 5) + aicen(i,j,1)
          if (iout( 6)) aa(i,j, 6) = aa(i,j, 6) + aicen(i,j,2)
          if (iout( 7)) aa(i,j, 7) = aa(i,j, 7) + aicen(i,j,3)
          if (iout( 8)) aa(i,j, 8) = aa(i,j, 8) + aicen(i,j,4)
          if (iout( 9)) aa(i,j, 9) = aa(i,j, 9) + aicen(i,j,5)
        else
          if (iout( 5)) aa(i,j, 5) = 0.0
          if (iout( 6)) aa(i,j, 6) = 0.0
          if (iout( 7)) aa(i,j, 7) = 0.0
          if (iout( 8)) aa(i,j, 8) = 0.0
          if (iout( 9)) aa(i,j, 9) = 0.0
        endif 

        if (iout(10)) aa(i,j,10) = aa(i,j,10) + u(i,j)
        if (iout(11)) aa(i,j,11) = aa(i,j,11) + v(i,j)

        if (iout(12)) aa(i,j,12) = aa(i,j,12) + Fsw(i,j)
        if (iout(13)) aa(i,j,13) = aa(i,j,13) + Flw(i,j)
        if (iout(14)) aa(i,j,14) = aa(i,j,14) + Fsnow(i,j)
        if (iout(15)) aa(i,j,15) = aa(i,j,15) + Fsnow(i,j)*aice_init(i,j)
        if (iout(16)) aa(i,j,16) = aa(i,j,16) + Frain(i,j)
        if (iout(17)) aa(i,j,17) = aa(i,j,17) + Frain(i,j)*aice_init(i,j)

        if (iout(18)) aa(i,j,18) = aa(i,j,18) + sst(i,j)
        if (iout(19)) aa(i,j,19) = aa(i,j,19) + sss(i,j)
        if (iout(20)) aa(i,j,20) = aa(i,j,20) + uocn(i,j)
        if (iout(21)) aa(i,j,21) = aa(i,j,21) + vocn(i,j)
        if (iout(22)) aa(i,j,22) = aa(i,j,22) + Frzmlt(i,j)

        if (iout(23)) aa(i,j,23) = aa(i,j,23) + sabs(i,j)
        if (iout(24)) aa(i,j,24) = aa(i,j,24) + sabs(i,j)*aice_init(i,j)
        if (iout(25)) aa(i,j,25) = aa(i,j,25) + 
     &                             alvdr(i,j)*awtvdr +
     &                             alidr(i,j)*awtidr +
     &                             alvdf(i,j)*awtvdf +
     &                             alidf(i,j)*awtidf
        if (iout(26)) aa(i,j,26) = aa(i,j,26) + Flatent(i,j)
        if (iout(27)) aa(i,j,27) = aa(i,j,27) + Flatent(i,j)*aice_init(i,j)
        if (iout(28)) aa(i,j,28) = aa(i,j,28) + Fsensible(i,j)
        if (iout(29)) aa(i,j,29) = aa(i,j,29) + Fsensible(i,j)*aice_init(i,j)
        if (iout(30)) aa(i,j,30) = aa(i,j,30) + Flwout(i,j)
        if (iout(31)) aa(i,j,31) = aa(i,j,31) + Flwout(i,j)*aice_init(i,j)
        if (iout(32)) aa(i,j,32) = aa(i,j,32) + evap(i,j)
        if (iout(33)) aa(i,j,33) = aa(i,j,33) + evap(i,j)*aice_init(i,j)
        if (iout(34)) aa(i,j,34) = aa(i,j,34) + Tref(i,j)
        if (iout(35)) aa(i,j,35) = aa(i,j,35) + growb(i,j)
        if (iout(36)) aa(i,j,36) = aa(i,j,36) + frazil(i,j)
        if (iout(37)) aa(i,j,37) = aa(i,j,37) + snoice(i,j)
        if (iout(38)) aa(i,j,38) = aa(i,j,38) + meltb(i,j)
        if (iout(39)) aa(i,j,39) = aa(i,j,39) + meltt(i,j)
        if (iout(40)) aa(i,j,40) = aa(i,j,40) + meltl(i,j)
        if (iout(41)) aa(i,j,41) = aa(i,j,41) + Frsh_hist(i,j)
        if (iout(42)) aa(i,j,42) = aa(i,j,42) + Frsh_hist(i,j)*aice_init(i,j)
        if (iout(43)) aa(i,j,43) = aa(i,j,43) + Foht_hist(i,j)
        if (iout(44)) aa(i,j,44) = aa(i,j,44) + Foht_hist(i,j)*aice_init(i,j)


        if (iout(45)) aa(i,j,45) = aa(i,j,45) + strairx(i,j)
        if (iout(46)) aa(i,j,46) = aa(i,j,46) + strairy(i,j)
        if (iout(47)) aa(i,j,47) = aa(i,j,47) + strtltx(i,j)
        if (iout(48)) aa(i,j,48) = aa(i,j,48) + strtlty(i,j)
        if (iout(49)) aa(i,j,49) = aa(i,j,49) + 
     &                             fm(i,j)*v(i,j)
        if (iout(50)) aa(i,j,50) = aa(i,j,50) -
     &                             fm(i,j)*u(i,j)
        if (iout(51)) aa(i,j,51) = aa(i,j,51) + strocnx(i,j)
        if (iout(52)) aa(i,j,52) = aa(i,j,52) + strocny(i,j)
        if (iout(53)) aa(i,j,53) = aa(i,j,53) + strintx(i,j)
        if (iout(54)) aa(i,j,54) = aa(i,j,54) + strinty(i,j)
        if (iout(55)) aa(i,j,55) = aa(i,j,55) + strength(i,j)
        if (iout(56)) aa(i,j,56) = aa(i,j,56) + divu(i,j)
        if (iout(57)) aa(i,j,57) = aa(i,j,57) + shear(i,j)
        if (iout(58)) aa(i,j,58) = aa(i,j,58) + divu(i,j)
     &                                        + closing(i,j)
        if (iout(61)) aa(i,j,61) = aa(i,j,61) + dvidtt(i,j)
        if (iout(62)) aa(i,j,62) = aa(i,j,62) + dvidtd(i,j)
        if (iout(63)) aa(i,j,63) = aa(i,j,63) + daidtt(i,j)
        if (iout(64)) aa(i,j,64) = aa(i,j,64) + daidtd(i,j)

       enddo
      enddo

      if (write_history .or. write_ic) then

        do k=1,avgsiz
         do j=jlo,jhi
          do i=ilo,ihi
           if (.not. tmask(i,j)) then  ! mask out land points
            aa(i,j,k) = 1.0e30
           elseif (iout(k)) then       ! convert units
            aa(i,j,k) = (cona(k)*aa(i,j,k)/avgct + conb(k))
           endif
          enddo
         enddo  
        enddo

      ! snapshots 
      call principal_stress
      do j=jlo,jhi
       do i=ilo,ihi
        if (iout(59)) aa(i,j,59) = sig1(i,j) 
        if (iout(60)) aa(i,j,60) = sig2(i,j) 
        ice_present(i,j) = ice_present(i,j)/avgct
       enddo
      enddo

        time_end = time/int(secday)
        call icecdf
        aa = c0
        avgct = c0
        ice_present = c0

      endif  ! write_history or write_ic

      end subroutine ice_write_hist
!
!BOP
!
! !IROUTINE: icecdf - write netCDF history file
!
! !INTERFACE:
!
      subroutine icecdf
!
! !DESCRIPTION:
!
! write netCDF history file
!
! !REVISION HISTORY:
!
! authors:   E.C.Hunke, LANL
!            Bruce P. Briegleb, NCAR
! 
! !USES:
!
      use ice_model_size
      use ice_constants
      use ice_mpi_internal
      use ice_grid
      use ice_calendar
      use ice_itd, only : c_hi_range
      use shr_file_mod
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      include "netcdf.inc"

      integer (kind=int_kind) :: i,ind,i_aice,j,length,n
     &,  ncid,status,imtid,jmtid,timid,boundid,varid
      integer (kind=int_kind) :: iyear, imonth, iday

      integer (kind=int_kind), dimension(3) :: dimid,start,count
      real (kind=real_kind) :: ltime
      character (char_len) :: title,start_time,current_date,current_time
      character (len=320) :: time_op
      character (char_len) :: ncfile       = 'null'    !Local history fname only
      character (char_len_long) :: dir_ncfile = 'null' !Local dir+fname
      character (char_len) :: rem_fn               ! Remote file name on mss
      character (char_len) :: loc_fn_old = 'null'  ! Previous history file
      character (len=5) :: c_aice
      character(len=*),parameter :: F02="('(history) code',i4,4(1x,a))"
      real (kind=real_kind), parameter :: spval=1.0e30

      real (kind=dbl_kind),dimension(ilo:ihi,jlo:jhi) :: work
      real (kind=dbl_kind),dimension(imt_global,jmt_global) :: arg
      real (kind=real_kind),dimension(imt_global,jmt_global) :: ar

! Info for lat, lon and time invariant variables

      INTEGER (kind=int_kind), PARAMETER :: ncoord = 4, nvar = 10
      TYPE coord_attributes         ! netcdf coordinate attributes
        character (len=10)    :: short_name
        character (len=45)   :: long_name
        character (len=20)   :: units
      END TYPE coord_attributes

      TYPE req_attributes         ! req'd CSM/netcdf attributes
        type (coord_attributes) :: req
        character (len=20)   :: coordinates
      END TYPE req_attributes

      TYPE(req_attributes), dimension(nvar) :: var
      TYPE(coord_attributes), dimension(ncoord) :: coord_var

      if (my_task.eq.master_task) then

        ltime=time/int(secday)

        iyear = nyr
        imonth = month
        iday = mday

        ! construct filename
        if (write_ic) then
           write(ncfile,'(a,i4.4,a,i2.2,a,i2.2,a,i5.5,a)') 
     &        incond_file(1:lenstr(incond_file)),iyear,'-',
     &        imonth,'-',iday,'-',sec,'.nc'

        else

         if (hist_avg) then    ! write averaged data

          if (new_year) then  ! This doesn't work so well with instantaneous
            iyear = nyr - 1   ! files .
            imonth = 12
            iday = 31
          elseif (new_month) then
            imonth = month - 1
            iday = daymo(imonth)
          elseif (new_day) then
            iday = iday - 1
          endif

          if (histfreq.eq.'d'.or.histfreq.eq.'D') then     ! daily
           write(ncfile,'(a,a,i4.4,a,i2.2,a,i2.2,a)') 
     &      history_file(1:lenstr(history_file)),'.',iyear,'-',
     &       imonth,'-',iday,'.nc'
          elseif (histfreq.eq.'w'.or.histfreq.eq.'W') then ! weekly
           write(ncfile,'(a,a,i4.4,a,i2.2,a,i2.2,a)') 
     &      history_file(1:lenstr(history_file)),'w.',iyear,'-',
     &       imonth,'-',iday,'.nc'
          elseif (histfreq.eq.'m'.or.histfreq.eq.'M') then ! monthly
           write(ncfile,'(a,a,i4.4,a,i2.2,a)') 
     &      history_file(1:lenstr(history_file)),'.',iyear,'-',
     &       imonth,'.nc'
          elseif (histfreq.eq.'y'.or.histfreq.eq.'Y') then ! yearly
           write(ncfile,'(a,a,i4.4,a)')
     &      history_file(1:lenstr(history_file)),'.',iyear,'.nc'
          endif

         else                      ! instantaneous
          if (histfreq.eq.'w'.or.histfreq.eq.'W') then ! weekly
           write(ncfile,'(a,a,i4.4,a,i2.2,a,i2.2,a,i5.5,a)') 
     &      history_file(1:lenstr(history_file)),'w.',iyear,'-',
     &       imonth,'-',iday,'-',sec,'.nc'
          else
           write(ncfile,'(a,a,i4.4,a,i2.2,a,i2.2,a,i5.5,a)') 
     &      history_file(1:lenstr(history_file)),'.',iyear,'-',
     &       imonth,'-',iday,'-',sec,'.nc'
          endif
         endif
        endif

        ! create file, add local directory path name to ncfile
        if (write_ic) then
          dir_ncfile = trim(incond_dir)//ncfile
        else
          dir_ncfile = trim(history_dir)//ncfile
        endif

        status = nf_create(dir_ncfile, nf_clobber, ncid)
        call nf_stat_check (status,'Error creating netCDF file')

      !-----------------------------------------------------------------
      ! define dimensions
      !-----------------------------------------------------------------

        status = nf_def_dim(ncid,'nlon',imt_global,imtid)
        call nf_stat_check (status,'Error defining dimension nlon')

        status = nf_def_dim(ncid,'nlat',jmt_global,jmtid)
        call nf_stat_check (status,'Error defining dimension nlat')

        status = nf_def_dim(ncid,'time',NF_UNLIMITED,timid)
        call nf_stat_check (status,'Error defining dimension time')

        if (hist_avg) then
          status = nf_def_dim(ncid,'d2',2,boundid)
          call nf_stat_check (status,'Error defining dimension d2')
        endif

        dimid(1) = timid
        status = nf_def_var(ncid,'time',nf_float,1,dimid,varid)
        call nf_stat_check (status,'Error defining variable time')

        status = nf_put_att_text(ncid,varid,'long_name',10,'model time')
        call nf_stat_check (status,'Error assigning long_name for time')

        status = nf_put_att_text(ncid,varid,'units',
     $                  30,'days since 0001-01-01 00:00:00')  ! for now
        call nf_stat_check (status,'Error assigning units for time')

        status = nf_put_att_text(ncid,varid,'calendar', 6,'noleap')
        call nf_stat_check (status,'Error assigning calendar for time')

        if (hist_avg) then
          status = nf_put_att_text(ncid,varid,'bounds', 11,'time_bounds')
          call nf_stat_check (status,'Error assigning bounds for time')
        endif
      !------------------------------------------
      ! Define attributes for time bounds if hist_avg is true
      !------------------------------------------
        if (hist_avg) then
          dimid(1) = boundid
          dimid(2) = timid
          status = nf_def_var(ncid,'time_bounds',nf_float,2,dimid,varid)
          call nf_stat_check (status,'Error defining variable time_bounds')
          status = nf_put_att_text(ncid,varid,'long_name',
     $             38,'boundaries for time-averaging interval')
          call nf_stat_check (status,'Error with long_name to time_bounds')
          status = nf_put_att_text(ncid,varid,'units',
     $                  30,'days since 0001-01-01 00:00:00')  ! for now
          call nf_stat_check (status,'Error assigning units to time_bounds')
        endif

      !-----------------------------------------------------------------
      ! define information for the creation of time-invariant variables
      !-----------------------------------------------------------------
      ind = 0
      ind = ind + 1
      coord_var(ind) = coord_attributes('TLON',
     &                 'T grid center longitude', 'degrees_east')
      ind = ind + 1
      coord_var(ind) = coord_attributes('TLAT', 
     &                 'T grid center latitude',  'degrees_north')
      ind = ind + 1
      coord_var(ind) = coord_attributes('ULON',
     &                 'U grid center longitude', 'degrees_east')
      ind = ind + 1
      coord_var(ind) = coord_attributes('ULAT', 
     &                 'U grid center latitude',  'degrees_north')

      ind = 0
      ind = ind + 1
      var(ind)%req = coord_attributes('tarea', 'area of T grid cells',
     &                          'm^2')
      var(ind)%coordinates = 'TLON TLAT'
      ind = ind + 1
      var(ind)%req = coord_attributes('uarea', 'area of U grid cells',
     &                          'm^2')
      var(ind)%coordinates = 'ULON ULAT'
      ind = ind + 1
      var(ind)%req = coord_attributes('dxt',
     &               'T cell width through middle', 'm')
      var(ind)%coordinates = 'TLON TLAT'
      ind = ind + 1
      var(ind)%req = coord_attributes('dyt',
     &               'T cell height through middle', 'm')
      var(ind)%coordinates = 'TLON TLAT'
      ind = ind + 1
      var(ind)%req = coord_attributes('dxu',
     &               'U cell width through middle', 'm')
      var(ind)%coordinates = 'ULON ULAT'
      ind = ind + 1
      var(ind)%req = coord_attributes('dyu',
     &               'U cell height through middle', 'm')
      var(ind)%coordinates = 'ULON ULAT'
      ind = ind + 1
      var(ind)%req = coord_attributes('HTN',
     &               'T cell width on North side','m')
      var(ind)%coordinates = 'TLON TLAT'
      ind = ind + 1
      var(ind)%req = coord_attributes('HTE',
     &               'T cell width on East side', 'm')
      var(ind)%coordinates = 'TLON TLAT'
      ind = ind + 1
      var(ind)%req = coord_attributes('ANGLET',
     &               'angle grid makes with latitude line on T grid',
     &               'radians')
      var(ind)%coordinates = 'TLON TLAT'
      ind = ind + 1
      var(ind)%req = coord_attributes('ANGLE',
     &               'angle grid makes with latitude line on U grid',
     &               'radians')
      var(ind)%coordinates = 'ULON ULAT'

      !-----------------------------------------------------------------
      ! define attributes for time-invariant variables
      !-----------------------------------------------------------------

        dimid(1) = imtid
        dimid(2) = jmtid
        dimid(3) = timid

        do i = 1, ncoord
          status = nf_def_var(ncid, coord_var(i)%short_name, nf_float,
     &                       2, dimid, varid)
          call nf_stat_check (status,
     &            'Error defining short_name for'//coord_var(i)%short_name)
          length = lenstr(coord_var(i)%long_name)
          status = nf_put_att_text(ncid,varid, 'long_name', length,
     &                       coord_var(i)%long_name)
          call nf_stat_check (status,
     &            'Error defining long_name for'//coord_var(i)%short_name)
          length = lenstr(coord_var(i)%units)
          status = nf_put_att_text(ncid, varid, 'units', length,
     &                       coord_var(i)%units)
          call nf_stat_check (status,
     &            'Error defining units for'//coord_var(i)%short_name)
          if (coord_var(i)%short_name == 'ULAT') then
            status = nf_put_att_text(ncid,varid,'comment', 36,
     $            'Latitude of NE corner of T grid cell')
            call nf_stat_check (status,
     &            'Error defining comment for'//coord_var(i)%short_name)
          endif
        enddo

! Attributes for tmask defined separately, since it has no units
        status = nf_def_var(ncid, 'tmask', nf_float, 2, dimid, varid)
        call nf_stat_check (status,'Error defining variable tmask')
        status = nf_put_att_text(ncid,varid, 'long_name', 15,
     &                      'ocean grid mask') 
        call nf_stat_check (status,'Error defining long_name for tmask')
        status = nf_put_att_text(ncid, varid, 'coordinates', 9,
     &              'TLON TLAT')
        call nf_stat_check (status,'Error defining coordinates for tmask')
        status = nf_put_att_text(ncid,varid,'comment', 19,
     $            '0 = land, 1 = ocean')
        call nf_stat_check (status,'Error defining comment for tmask')

        do i = 1, nvar
          status = nf_def_var(ncid, var(i)%req%short_name, nf_float,
     &                        2, dimid, varid)
          call nf_stat_check (status,
     &             'Error defining variable'//var(i)%req%short_name)
          length = lenstr(var(i)%req%long_name)
          status = nf_put_att_text(ncid,varid, 'long_name', length,
     &              var(i)%req%long_name)
          call nf_stat_check (status,
     &             'Error defining long_name for'//var(i)%req%short_name)
          length = lenstr(var(i)%req%units)
          status = nf_put_att_text(ncid, varid, 'units', length,
     &              var(i)%req%units)
          call nf_stat_check (status,
     &             'Error defining units for'//var(i)%req%short_name)
          length = lenstr(var(i)%coordinates)
          status = nf_put_att_text(ncid, varid, 'coordinates', length,
     &              var(i)%coordinates)
          call nf_stat_check (status,
     &             'Error defining coordinates for'//var(i)%req%short_name)
         enddo

      !-----------------------------------------------------------------
      ! define attributes for ice_present
      !-----------------------------------------------------------------
         status  = nf_def_var(ncid, 'ice_present', nf_float, 3, dimid, varid)
         call nf_stat_check (status,'Error defining variable ice_present')
         status = nf_put_att_text(ncid,varid, 'units',1,'1')
         call nf_stat_check (status,'Error defining units for ice_present')
         status = nf_put_att_text(ncid,varid, 'long_name',59,
     &     'fraction of time-averaging interval that any ice is present')
         call nf_stat_check (status,
     &                    'Error defining long_name for ice_present')
         status = nf_put_att_text(ncid,varid,'coordinates',
     &                          14,'TLON TLAT time')
         call nf_stat_check (status,
     &                    'Error defining coordinates for ice_present')

      !-----------------------------------------------------------------
      ! define variables and variable attributes
      !-----------------------------------------------------------------

        do n=1,avgsiz
          if (iout(n)) then
            status  = nf_def_var(ncid, vname(n), nf_float, 3, dimid, varid)
            call nf_stat_check (status,'Error defining variable'//vname(n))
            length = lenstr(vunit(n))
            status = nf_put_att_text(ncid,varid, 'units',length,vunit(n))
            call nf_stat_check (status,'Error defining units for'//vname(n))
            length = lenstr(vdesc(n))
            status = nf_put_att_text(ncid,varid, 'long_name',length,vdesc(n))
            call nf_stat_check (status,
     &                       'Error defining long_name for'//vname(n))
            length = lenstr(vcoord(n))
            status = nf_put_att_text(ncid,varid,'coordinates',
     $                             length,vcoord(n))
            call nf_stat_check (status,
     &                       'Error defining coordinates for'//vname(n))
            status = nf_put_att_real(ncid,varid,'missing_value',
     $                             nf_float,1,spval)
            call nf_stat_check (status,
     &                       'Error defining mising_value for'//vname(n))

            status = nf_put_att_real(ncid,varid,'_FillValue',
     $                             nf_float,1,spval)
            call nf_stat_check (status,
     &                       'Error defining _FillValue for'//vname(n))
      !-----------------------------------------------------------------
      ! Add comment attribute to some variables
      !-----------------------------------------------------------------
            if (TRIM(vcomment(n))/='none') then
              !---------------------------------------------------------
              ! Append ice thickness range to aice comments
              !---------------------------------------------------------
              if (TRIM(vname(n))=='aice1'.or.TRIM(vname(n))=='aice2'
     $          .or.TRIM(vname(n))=='aice3'.or.TRIM(vname(n))=='aice4'
     $          .or.TRIM(vname(n))=='aice5') then
                c_aice = TRIM(vname(n))
                read(c_aice(5:5), '(i1)') i_aice
                vcomment(n) = c_hi_range(i_aice)
              endif
              length = lenstr(vcomment(n))
              status = nf_put_att_text(ncid,varid,'comment', length, vcomment(n))
              call nf_stat_check (status,
     &                       'Error defining comment for'//vname(n))
            endif
      !-----------------------------------------------------------------
      ! Add cell_methods attribute to variables if averaged
      !-----------------------------------------------------------------
            if (hist_avg) then
              if (TRIM(vname(n))/='sig1'.and.TRIM(vname(n))/='sig2') then
                status = nf_put_att_text(ncid,varid,'cell_methods',
     $                               9,'time:mean')
                call nf_stat_check (status,
     &                         'Error defining cell methods for'//vname(n))
              endif
            endif

          endif
        enddo

      !-----------------------------------------------------------------
      ! global attributes
      !-----------------------------------------------------------------
        length = lenstr(runid)
        status = nf_put_att_text(ncid,nf_global,'title',length,runid)
        call nf_stat_check (status,'Error in global attribute title')

        title = 'Diagnostic and Prognostic Variables'
        length = lenstr(title)
        status = nf_put_att_text(ncid,nf_global,'contents',length,title)
        call nf_stat_check (status,'Error in global attribute contents')

        title = 'CSIM4, the NCAR/CCSM2 Sea Ice Component'
        length = lenstr(title)
        status = nf_put_att_text(ncid,nf_global,'source',length,title)
        call nf_stat_check (status,'Error in global attribute source')

        title = 'CVS Tag: $Name: ccsm2_0_beta46 $'
        length = lenstr(title)
        status = nf_put_att_text(ncid,nf_global,'revision',length,title)
        call nf_stat_check (status,'Error in global attribute revision')

        title = 'All years have exactly 365 days'
        length = lenstr(title)
        status = nf_put_att_text(ncid,nf_global,'comment',length,title)
        call nf_stat_check (status,'Error in global attribute comment')

        title = 'CF-1.0'
        length = lenstr(title)
        status = 
     &       nf_put_att_text(ncid,nf_global,'conventions',length,title)
        call nf_stat_check (status,'Error in global attribute conventions')

        call date_and_time(date=current_date, time=current_time)
        write(start_time,1000) current_date(1:4), current_date(5:6)
     &,                        current_date(7:8), current_time(1:2)
     &,                        current_time(3:4), current_time(5:8)
1000    format('This dataset was created on '
     &,         a,'-',a,'-',a,' at ',a,':',a,':',a)

        length = lenstr(start_time)
        status = nf_put_att_text(ncid,nf_global,'history',length,start_time)
        call nf_stat_check (status,'Error in global attribute history')

      !-----------------------------------------------------------------
      ! end define mode
      !-----------------------------------------------------------------

        status = nf_enddef(ncid)
        call nf_stat_check (status,'Error in nf_enddef')

      !-----------------------------------------------------------------
      ! write time variable
      !-----------------------------------------------------------------

        status = nf_inq_varid(ncid,'time',varid)
        call nf_stat_check (status,'Error getting time varid')
        status = nf_put_vara_real(ncid,varid,1,1,ltime)
        call nf_stat_check (status,'Error writing time variable')

      !-----------------------------------------------------------------
      ! write time_bounds info
      !-----------------------------------------------------------------

        if (hist_avg) then
          start(1) = 1
          start(2) = 1
          count(1)=1
          count(2)=1
          status = nf_inq_varid(ncid,'time_bounds',varid)
          call nf_stat_check (status,'Error getting time_bounds id')
          status = nf_put_vara_real(ncid,varid,start,count,time_beg)
          call nf_stat_check (status,'Error writing time_beg')
          start(1) = 2
          status = nf_put_vara_real(ncid,varid,start,count,time_end)
          call nf_stat_check (status,'Error writing time_end')
        endif

        do i = 1,ncoord
          SELECT CASE (coord_var(i)%short_name)
            CASE ('TLON')
! Convert T grid longitude from -180 -> 180 to 0 to 360
              ar = TLON_G + c360    ! single precision
              where (ar > c360) ar = ar - c360
              where (ar < c0 )  ar = ar + c360
            CASE ('TLAT')
              ar = TLAT_G
            CASE ('ULON')
              ar = ULON_G*rad_to_deg
            CASE ('ULAT')
              ar = ULAT_G*rad_to_deg
          END SELECT
          status = nf_inq_varid(ncid, coord_var(i)%short_name, varid)
          call nf_stat_check (status,'Error getting varid for'
     &                                //coord_var(i)%short_name)
          status = nf_put_var_real(ncid,varid,ar)
          call nf_stat_check (status,'Error writing'
     &                                //coord_var(i)%short_name)
        enddo

      endif          ! master_task

      !-----------------------------------------------------------------
      ! write grid mask, area and rotation angle
      !-----------------------------------------------------------------

      work(:,:) = hm(ilo:ihi,jlo:jhi)    ! tmask, no units
      call global_gather(arg,work)
      if (my_task == master_task) then
        ar = arg  ! single precision
        status = nf_inq_varid(ncid, 'tmask', varid)
        call nf_stat_check (status,'Error getting varid for tmask')
        status = nf_put_var_real(ncid,varid,ar)
        call nf_stat_check (status,'Error writing variable tmask')
      endif

      do i = 1,nvar
        call ice_bcast_char(var(i)%req%short_name)
        SELECT CASE (var(i)%req%short_name)
          CASE ('tarea')
            work(:,:) = tarea(ilo:ihi,jlo:jhi)
          CASE ('uarea')
            work(:,:) = uarea(ilo:ihi,jlo:jhi)
          CASE ('dxu')
            work(:,:) = dxu(ilo:ihi,jlo:jhi)
          CASE ('dyu')
            work(:,:) = dyu(ilo:ihi,jlo:jhi)
          CASE ('dxt')
            work(:,:) = dxt(ilo:ihi,jlo:jhi)
          CASE ('dyt')
            work(:,:) = dyt(ilo:ihi,jlo:jhi)
          CASE ('HTN')
            work(:,:) = HTN(ilo:ihi,jlo:jhi)
          CASE ('HTE')
            work(:,:) = HTE(ilo:ihi,jlo:jhi)
          CASE ('ANGLE')
            work(:,:) = ANGLE(ilo:ihi,jlo:jhi)
          CASE ('ANGLET')
            work(:,:) = ANGLET(ilo:ihi,jlo:jhi)
        END SELECT

        call global_gather(arg,work)

        if (my_task == master_task) then
          ar = arg  ! single precision
          status = nf_inq_varid(ncid, var(i)%req%short_name, varid)
          call nf_stat_check (status,'Error getting varid for'
     &                          //var(i)%req%short_name)
          status = nf_put_var_real(ncid,varid,ar)
          call nf_stat_check (status,'Error writing variable'
     &                          //var(i)%req%short_name)
        endif
      enddo
      !-----------------------------------------------------------------
      ! write ice_present
      !-----------------------------------------------------------------
      work(:,:) = ice_present(ilo:ihi,jlo:jhi)    ! tmask, no units
      call global_gather(arg,work)
      if (my_task == master_task) then
        ar = arg  ! single precision
        status = nf_inq_varid(ncid, 'ice_present', varid)
        call nf_stat_check (status,'Error getting varid for ice_present')
        status = nf_put_var_real(ncid,varid,ar)
        call nf_stat_check (status,'Error writing variable ice_present')
      endif

      !-----------------------------------------------------------------
      ! write variable data 
      !-----------------------------------------------------------------

      start=1
      count(1)=imt_global
      count(2)=jmt_global
      count(3)=1
      do n=1,avgsiz
        if (iout(n)) then
          work=aa(:,:,n)
          call global_gather(arg,work)
          if (my_task.eq.master_task) then
            ar=arg
            status = nf_inq_varid(ncid,vname(n),varid)
            call nf_stat_check (status,'Error getting varid for'
     &                          //vname(n))
            status = nf_put_vara_real(ncid,varid,start,count,ar)
            call nf_stat_check (status,'Error writing variable'
     &                          //vname(n))
          endif
        endif
      enddo

      !-----------------------------------------------------------------
      ! close output dataset
      !-----------------------------------------------------------------
      if (my_task.eq.master_task) then
        status = nf_close(ncid)
        call nf_stat_check (status,'Error closing netCDF history file')
        write(6,*) ' '
        write(6,*) 'Finished writing ',trim(dir_ncfile)
        write(6,*) ' '
        !---------------------------------------------------------------
        ! write history and incond files to MSS
        !---------------------------------------------------------------
        if (mss_dir(1:4) /= 'null' .and. ncfile(1:4) /= 'null') then
          if (write_ic) then
            rem_fn = trim(mss_dir)//'init/'//trim(ncfile)//' '
          else
            rem_fn = trim(mss_dir)//'hist/'//trim(ncfile)//' '
          endif
          call shr_file_put (rcode, dir_ncfile,rem_fn,mss_pass,mss_rtpd)
          write(6,*)
          write(6,F02) rcode,'archiving',trim(ncfile),'to',trim(rem_fn)
          write(6,*)

          !--- remove older history file ---
          if (mss_rmlf.ne.0 .and. loc_fn_old(1:4).ne.'null') then
            call shr_sys_system('rm '//loc_fn_old,rcode)
          end if
          loc_fn_old = dir_ncfile
          ncfile     = 'null'
        end if
      endif

      end subroutine icecdf
!
!BOP
!
! !IROUTINE: nf_stat_check - check error code returned from netCDF call
!
! !INTERFACE:
!
      subroutine nf_stat_check (status,msg)

!
! !DESCRIPTION:
!
! Check error code returned from netCDF calls \\
! If bad, abort
!
! !REVISION HISTORY:
!
! author:   ?
! 
! !USES:
!
      use shr_sys_mod, only : shr_sys_abort
      include "netcdf.inc"
!
! !INPUT/OUTPUT PARAMETERS:
!
      integer (kind=int_kind), intent(in) :: status
      character (len = *), intent(in) :: msg
!
!EOP
!
      if (status /= NF_NOERR) then
        write (6,*) msg//nf_strerror(status)
        call shr_sys_abort ('writing netCDF file in ice_history.F')
      endif

      end subroutine nf_stat_check
!
!BOP
!
! !IROUTINE: integer function lenstr(label) - compute length string
!
! !INTERFACE:
!
      integer function lenstr(label)
!
! !DESCRIPTION:
!
! Check error code returned from netCDF calls.
! If bad, abort
!
! !REVISION HISTORY:
!
! author:   ?
!
! !INPUT/OUTPUT PARAMETERS:
!
      character*(*) label
!
!EOP
!
      integer (kind=int_kind) :: 
     &   length  ! length of character string 
     &,  n       ! loop index

      length = len(label)
      do n=length,1,-1
        if( label(n:n) .ne. ' ' ) goto 10 
      enddo
 10   lenstr = n

      end function lenstr
!
!BOP
!
! !IROUTINE: dumpfile - dumps all fields required for restart
!
! !INTERFACE:
!
      subroutine dumpfile
!
! !DESCRIPTION:
!
! Dumps all values needed for a restart 
!
! !REVISION HISTORY:
!
! author Elizabeth C. Hunke, LANL
!
! !USES:
!
      use ice_model_size
      use ice_flux
      use ice_grid
      use ice_calendar
      use ice_state
      use ice_dyn_evp
      use shr_file_mod
      use ice_ocean
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) :: i, j, k, n, iyear, imonth, iday
      character(len=80) :: filename, rem_fn
      character(char_len_long) :: dir_filename  ! dir and filename
      character(len=*),parameter :: F02="('(history) code',i4,4(1x,a))"
      logical (kind=log_kind) :: gather
      real (kind=dbl_kind) :: work(ilo:ihi,jlo:jhi)

      iyear = nyr
      imonth = month
      iday = mday
      write(filename,'(a,i4.4,a,i2.2,a,i2.2,a,i5.5)') 
     &  dump_file(1:lenstr(dump_file)),iyear,'-',imonth,'-',iday,'-',
     &          sec
      dir_filename = trim(restart_dir)//filename

      if (my_task.eq.master_task) then
        open(50,file=pointer_file, form='formatted')
        write(50,'(a)') trim(dir_filename)
        close(50)
      endif
      call ice_open(50,dir_filename,0)
      if (my_task.eq.master_task) then
        write (50) istep1,time,time_forc
        write(6,*) 'Writing ',filename(1:lenstr(filename))
        write(6,*) 'Restart written ',istep1,time,time_forc
      endif

      gather = .true.

      !-----------------------------------------------------------------
      ! state variables
      !-----------------------------------------------------------------
      do n=1,ncat
        work(ilo:ihi,jlo:jhi)=aicen(ilo:ihi,jlo:jhi,n)
        call ice_write(50,0,work,'ruf8',gather)
        work(ilo:ihi,jlo:jhi)=vicen(ilo:ihi,jlo:jhi,n)
        call ice_write(50,0,work,'ruf8',gather)
        work(ilo:ihi,jlo:jhi)=vsnon(ilo:ihi,jlo:jhi,n)
        call ice_write(50,0,work,'ruf8',gather)
        work(ilo:ihi,jlo:jhi)=Tsfcn(ilo:ihi,jlo:jhi,n)
        call ice_write(50,0,work,'ruf8',gather)
      enddo

      do k=1,ntilay
        work(ilo:ihi,jlo:jhi)=eicen(ilo:ihi,jlo:jhi,k)
        call ice_write(50,0,work,'ruf8',gather)
      enddo

      !-----------------------------------------------------------------
      ! velocity
      !-----------------------------------------------------------------
      work(ilo:ihi,jlo:jhi)=u(ilo:ihi,jlo:jhi)
      call ice_write(50,0,work,'ruf8',gather)
      work(ilo:ihi,jlo:jhi)=v(ilo:ihi,jlo:jhi)
      call ice_write(50,0,work,'ruf8',gather)

      !-----------------------------------------------------------------
      ! fresh water and heat flux info 
      !-----------------------------------------------------------------
      work(ilo:ihi,jlo:jhi)=Fresh(ilo:ihi,jlo:jhi)      
      call ice_write(50,0,work,'ruf8',gather)           
      work(ilo:ihi,jlo:jhi)=Fhnet(ilo:ihi,jlo:jhi)      
      call ice_write(50,0,work,'ruf8',gather)          

      !-----------------------------------------------------------------
      ! ice strength 
      !-----------------------------------------------------------------
      work(ilo:ihi,jlo:jhi)=strength(ilo:ihi,jlo:jhi)   
      call ice_write(50,0,work,'ruf8',gather)           

      !-----------------------------------------------------------------
      ! ocean stress
      !-----------------------------------------------------------------
      work(ilo:ihi,jlo:jhi)=strocnxT(ilo:ihi,jlo:jhi)
      call ice_write(50,0,work,'ruf8',gather)
      work(ilo:ihi,jlo:jhi)=strocnyT(ilo:ihi,jlo:jhi)
      call ice_write(50,0,work,'ruf8',gather)

      !-----------------------------------------------------------------
      ! internal stress
      !-----------------------------------------------------------------
      do k=1,4
      work(ilo:ihi,jlo:jhi)=stressp(k,ilo:ihi,jlo:jhi)
      call ice_write(50,0,work,'ruf8',gather)
      work(ilo:ihi,jlo:jhi)=stressm(k,ilo:ihi,jlo:jhi)
      call ice_write(50,0,work,'ruf8',gather)
      work(ilo:ihi,jlo:jhi)=stress12(k,ilo:ihi,jlo:jhi)
      call ice_write(50,0,work,'ruf8',gather)
      enddo

      !-----------------------------------------------------------------
      ! ice mask for dynamics
      !-----------------------------------------------------------------
      do j=jlo,jhi
       do i=ilo,ihi
        work(i,j) = 0.
        if (iceumask(i,j)) work(i,j) = 1.
       enddo
      enddo
      call ice_write(50,0,work,'ruf8',gather)

      if( oceanmixed_ice ) then
        ! for mixed layer model
        work(ilo:ihi,jlo:jhi)=sst_mixedlayer(ilo:ihi,jlo:jhi)
        call ice_write(50,0,work,'ruf8',gather)
        work(ilo:ihi,jlo:jhi)=frzmlt_mixedlayer(ilo:ihi,jlo:jhi)
        call ice_write(50,0,work,'ruf8',gather)
      endif 

      !-----------------------------------------------------------------
      ! close output restart file
      !-----------------------------------------------------------------
       if (my_task.eq.master_task) then
         close(50)
         !--------------------------------------------------------------
         ! write restart file to MSS
         !--------------------------------------------------------------
         if ( mss_dir(1:4) /= 'null' ) then
           rem_fn = trim(mss_dir)//'rest/'//trim(filename)//' '
           call shr_file_put (rcode, dir_filename,rem_fn,mss_pass,mss_rtpd)
           write(6,*)
           write(6,F02) rcode,'archiving',trim(filename),'to',trim(rem_fn)
           write(6,*)
         endif
       endif

      end subroutine dumpfile
!
!BOP
!
! !IROUTINE: restartfile  - restarts from a dumpfile
!
! !INTERFACE:
!
      subroutine restartfile
!
! !DESCRIPTION:
!
! Restarts from a dump
!
! !REVISION HISTORY:
!
! author Elizabeth C. Hunke, LANL
!
! !USES:
!
      use ice_model_size
      use ice_flux
      use ice_mpi_internal
      use ice_grid
      use ice_calendar
      use ice_state
      use ice_dyn_evp
      use ice_itd
      use ice_ocean
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) :: i, j, k, n, ios
      character(char_len_long) :: filename
      logical (kind=log_kind) :: scatter
      real (kind=dbl_kind) :: work(ilo:ihi,jlo:jhi)

      if (my_task.eq.master_task) then
        open(50,file=pointer_file, form='formatted')
        read(50,'(a)') filename
        close(50)
      endif
      call ice_open(50,filename,0)
      if (my_task.eq.master_task) then
        read (50) istep0,time,time_forc
        write(6,*) 'read ',pointer_file(1:lenstr(pointer_file))
        write(6,*) 'dump read at istep=',istep0,time,time_forc
      endif
      call ice_bcast_iscalar(istep0)
      istep1=istep0
      call ice_bcast_rscalar(time)
      call ice_bcast_rscalar(time_forc)

      scatter = .true.

      !-----------------------------------------------------------------
      ! state variables
      !-----------------------------------------------------------------
      do n=1,ncat
        if (my_task.eq.master_task) 
     &   write (6,*) 'cat ',n,' min/max area, vol ice, vol snow, Tsfc'
        call ice_read(50,0,work,'ruf8',scatter)
        aicen(ilo:ihi,jlo:jhi,n)=work(ilo:ihi,jlo:jhi)
        call ice_read(50,0,work,'ruf8',scatter)
        vicen(ilo:ihi,jlo:jhi,n)=work(ilo:ihi,jlo:jhi)
        call ice_read(50,0,work,'ruf8',scatter)
        vsnon(ilo:ihi,jlo:jhi,n)=work(ilo:ihi,jlo:jhi)
        call ice_read(50,0,work,'ruf8',scatter)
        Tsfcn(ilo:ihi,jlo:jhi,n)=work(ilo:ihi,jlo:jhi)
      enddo

      if (my_task.eq.master_task)
     & write (6,*) 'min/max eicen for each layer'
      do k=1,ntilay
        call ice_read(50,0,work,'ruf8',scatter)
        eicen(ilo:ihi,jlo:jhi,k)=work(ilo:ihi,jlo:jhi)
      enddo

      !-----------------------------------------------------------------
      ! velocity
      !-----------------------------------------------------------------
      if (my_task.eq.master_task) 
     & write (6,*) 'min/max velocity components'
      call ice_read(50,0,work,'ruf8',scatter)
      u(ilo:ihi,jlo:jhi)=work(ilo:ihi,jlo:jhi)
      call ice_read(50,0,work,'ruf8',scatter)
      v(ilo:ihi,jlo:jhi)=work(ilo:ihi,jlo:jhi)

      !-----------------------------------------------------------------
      ! fresh water and heat flux       
      !-----------------------------------------------------------------
      if (my_task.eq.master_task)
     & write (6,*) 'min/max fresh water and heat flux components'
      call ice_read(50,0,work,'ruf8',scatter)
      Fresh(ilo:ihi,jlo:jhi)=work(ilo:ihi,jlo:jhi)
      call ice_read(50,0,work,'ruf8',scatter)
      Fhnet(ilo:ihi,jlo:jhi)=work(ilo:ihi,jlo:jhi)

      !-----------------------------------------------------------------
      ! ice strength 
      !-----------------------------------------------------------------
      if (my_task.eq.master_task)
     & write (6,*) 'min/max ice strength'
      call ice_read(50,0,work,'ruf8',scatter)
      strength(ilo:ihi,jlo:jhi)=work(ilo:ihi,jlo:jhi)

      !-----------------------------------------------------------------
      ! ocean stress
      !-----------------------------------------------------------------
      if (my_task.eq.master_task) 
     & write (6,*) 'min/max ocean stress components'
      call ice_read(50,0,work,'ruf8',scatter)
      strocnxT(ilo:ihi,jlo:jhi)=work(ilo:ihi,jlo:jhi)
      call ice_read(50,0,work,'ruf8',scatter)
      strocnyT(ilo:ihi,jlo:jhi)=work(ilo:ihi,jlo:jhi)

      !-----------------------------------------------------------------
      ! internal stress
      !-----------------------------------------------------------------
      if (my_task.eq.master_task) 
     & write (6,*) 'min/max internal stress components (3 per triangle)'
      do k=1,4
      call ice_read(50,0,work,'ruf8',scatter)
      stressp(k,ilo:ihi,jlo:jhi)=work(ilo:ihi,jlo:jhi)
      call ice_read(50,0,work,'ruf8',scatter)
      stressm(k,ilo:ihi,jlo:jhi)=work(ilo:ihi,jlo:jhi)
      call ice_read(50,0,work,'ruf8',scatter)
      stress12(k,ilo:ihi,jlo:jhi)=work(ilo:ihi,jlo:jhi)
      enddo

      !-----------------------------------------------------------------
      ! ice mask for dynamics
      !-----------------------------------------------------------------
      if (my_task.eq.master_task) 
     & write (6,*) 'ice mask for dynamics'
      call ice_read(50,0,work,'ruf8',scatter)
      do j=jlo,jhi
       do i=ilo,ihi
        iceumask(i,j) = .false.
        if (work(i,j).gt.p5) iceumask(i,j) = .true.
       enddo
      enddo

      if( oceanmixed_ice ) then
        ! for mixed layer model if present
        if( .not. oceanmixed_ice_sst_init ) then
          if (my_task.eq.master_task) 
     &    write (6,*) 'min/max sst, frzmlt'
          call ice_read(50,0,work,'ruf8',scatter)
          sst_mixedlayer(ilo:ihi,jlo:jhi)=work(ilo:ihi,jlo:jhi)
          call ice_read(50,0,work,'ruf8',scatter)
          frzmlt_mixedlayer(ilo:ihi,jlo:jhi)=work(ilo:ihi,jlo:jhi)
        endif
      endif

      if (my_task.eq.master_task) close(50)

      !-----------------------------------------------------------------
      ! update boundary conditions
      !-----------------------------------------------------------------
      do n = 1,ncat
        call bound(aicen(:,:,n))
        call bound(vicen(:,:,n))
        call bound(vsnon(:,:,n))
        call bound(Tsfcn(:,:,n))
      end do
      do k = 1,ntilay
        call bound(eicen(:,:,k))
      end do
      call bound(u)      
      call bound(v)      

c      !-----------------------------------------------------------------
c      ! ensure ice is binned in correct categories 
c      ! should not be necessary unless restarting from a run with
c      ! different category boundaries
c      !-----------------------------------------------------------------
c      ! These calls do not allow for exact restart capability, so they
c      ! have been commented out.  This may be a problem for a run that
c      ! uses different category boundaries.
c      do j=jlo,jhi
c        do i=ilo,ihi
c          call to_column(i,j) 
c          call rebin_ice(i,j)
c          call from_column(i,j) 
c        enddo
c      enddo
c      call distr_check

      !-----------------------------------------------------------------
      ! compute aggregate ice state and open water area
      !-----------------------------------------------------------------
      call aggregate 

      end subroutine restartfile
!
      end module ice_history
!
