c CVS: $Id: ice_init.F,v 1.16 2002/04/15 14:24:06 schramm Exp $
c CVS: $Source: /fs/cgd/csm/models/CVS.REPOS/ice/csim4_src/source/ice_init.F,v $
c CVS: $Name: ccsm2_0_beta46 $
!
!BOP
!
! !MODULE:   ice_init - parameter and variable initializations
!
! !DESCRIPTION:
! 
! parameter and variable initializations
!
! !REVISION HISTORY:
! 
! authors Elizabeth C. Hunke, LANL
!         C. M. Bitz
!
! !INTERFACE:
!
      module ice_init
!
! !USES:
!
      use ice_domain
!
!EOP
!
      implicit none
!
      contains
!
!BOP
!
! !IROUTINE: input_data - namelist variables
!
! !INTERFACE:
!
      subroutine input_data
!
! !DESCRIPTION:
!
! Namelist variables, set to default values; may be altered
! at run time
!
! !REVISION HISTORY:
! 
! author Elizabeth C. Hunke, LANL
!
! !USES:
!
      use ice_mechred
      use ice_diagnostics
      use ice_history
      use ice_calendar
      use ice_dyn_evp
      use ice_itd
      use ice_transport
      use ice_prescribed
      use ice_ocean
      use shr_sys_mod, only : shr_sys_abort
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) :: nml_error ! namelist i/o error flag

      namelist /ice_nml/ runid, runtype, istep0, dt, ndte, npt, 
     & diagfreq, histfreq, dumpfreq, restart, print_points,
     & kcolumn, kitd, kdyn, kstrength, evp_damping, snow_into_ocn,
     & grid_file, kmt_file, incond_dir, incond_file, restart_dir,
     & dump_file, pointer_file, history_dir, history_file, grid_type,
     & advection, hist_avg, mss_dir, mss_rtpd, mss_pass, mss_rmlf,
     & prescribed_ice, prescribed_ice_file, prescribed_ice_climatology,
     & oceanmixed_ice, oceanmixed_ice_file, oceanmixed_ice_sst_init,
     & prntdiag_oceanmixed

      !-----------------------------------------------------------------
      ! default values
      !-----------------------------------------------------------------
      runid   = 'unknown_runid'   ! default run ID
      runtype = 'unknown_runtype'   ! default runtype
      istep0 = 0        ! number of steps taken in previous integrations,
                        ! real (dumped) or imagined (use to set calendar)
      dt = 3600.0_dbl_kind  ! time step, s 
      ndte = 120        ! number of subcycles:  ndte=dt/dte
      npt = 99999       ! total number of time steps (dt) 
      diagfreq = 24      ! how often diag output is written
      histfreq='m'      ! output frequency
      dumpfreq='y'      ! restart frequency
      hist_avg = .true. ! write snapshots rather than time-averages
      print_points = .false.     ! if true, print point data
      kcolumn = 0       ! 1 = column model
      kitd = 1          ! type of itd conversions (0 = delta f, 1 = linear)
      kdyn = 1          ! type of dynamics (1 = evp)
      restart = .false. ! do not read restart files for initialization
      kstrength = 1     ! 1 = use Rothrock 1975 pressure formulation
      evp_damping = .false. ! if true, use damping procedure in evp dynamics
      snow_into_ocn = .false. ! if true, throw snow into ocn during ridging
      advection  = 'mpdata2'  ! second order advection using mpdata
      grid_type    = 'displaced_pole'  ! define rectangular grid internally

      grid_file    = 'data.domain.grid'
      kmt_file     = 'data.domain.kmt'
      incond_dir  = ' '             ! Write to executable dir for default
      incond_file = 'incond'
      restart_dir  = ' '             ! Write to executable dir for default
      dump_file    = 'iced'          ! This is the restart file name prefix
      pointer_file = 'ice.restart_file'
      history_dir  = ' '             ! Write to executable dir for default
      history_file = 'iceh'
      mss_dir      = 'null'          ! MSS archive directory name
      mss_rtpd     = 365             ! MSS retention period
      mss_pass     = ' '             ! MSS password
      mss_rmlf     = 0               ! T => remove local files after mswrite
      prescribed_ice             = .false.               ! if true, prescribe ice
      prescribed_ice_file        = 'prescribed_ice.nc'   ! local prescribed ice file name
      prescribed_ice_climatology = .true.                ! if true, use climatology
      oceanmixed_ice             = .false.               ! if true, use internal ocean mixed layer
      oceanmixed_ice_file        = 'oceanmixed_ice.nc'   ! local ocean forcing file name
      oceanmixed_ice_sst_init    = .false.               ! if true, initialize sst from ocn frc data
      prntdiag_oceanmixed        = .false.               ! if true, print out some diagnostics

      !-----------------------------------------------------------------
      ! read from input file
      !-----------------------------------------------------------------
      if (my_task == master_task) then
        open (21, file='ice_in', status='old')
   10   continue  !*** keep reading until right namelist is found
        read(21, nml=ice_nml, iostat=nml_error)
        if (nml_error > 0) goto 10    ! An error occurred
        if (nml_error < 0) goto 20    ! End of file condition
        close(21)
   20   continue
      endif
      call ice_bcast_iscalar(nml_error)
      if (nml_error /= 0) then
        call shr_sys_abort('ice: Namelist read error in ice_init.F')
      endif
        call ice_bcast_char(runid)
        call ice_bcast_char(runtype)
        call ice_bcast_iscalar(istep0)
        call ice_bcast_rscalar(dt)
        call ice_bcast_iscalar(ndte)
        call ice_bcast_iscalar(npt)
        call ice_bcast_iscalar(diagfreq)
        call ice_bcast_char(histfreq)
        call ice_bcast_char(dumpfreq)
        call ice_bcast_logical(hist_avg)
        call ice_bcast_logical(print_points)
        call ice_bcast_iscalar(kcolumn)
        call ice_bcast_iscalar(kitd)
        call ice_bcast_iscalar(kdyn)
        call ice_bcast_logical(restart)
        call ice_bcast_iscalar(kstrength)
        call ice_bcast_logical(evp_damping)
        call ice_bcast_logical(snow_into_ocn)
        call ice_bcast_char(advection)
        call ice_bcast_char(grid_type)
        call ice_bcast_char(grid_file)
        call ice_bcast_char(kmt_file)
        call ice_bcast_char(restart_dir)
        call ice_bcast_char(dump_file)
        call ice_bcast_char(pointer_file)
        call ice_bcast_char(history_dir)
        call ice_bcast_char(history_file)
        call ice_bcast_char(mss_dir)
        call ice_bcast_iscalar(mss_rtpd)
        call ice_bcast_char(mss_pass)
        call ice_bcast_iscalar(mss_rmlf)
        call ice_bcast_logical(prescribed_ice)
        call ice_bcast_char(prescribed_ice_file)
        call ice_bcast_logical(prescribed_ice_climatology)
        call ice_bcast_logical(oceanmixed_ice)
        call ice_bcast_char(oceanmixed_ice_file)
        call ice_bcast_logical(oceanmixed_ice_sst_init)
        call ice_bcast_logical(prntdiag_oceanmixed)

        if (histfreq.eq.'1') hist_avg = .false. ! potential conflict
        if (kcolumn.eq.1) grid_type    = 'column' 

      !-----------------------------------------------------------------
      ! spew
      !-----------------------------------------------------------------
      if (my_task.eq.master_task) then
        write (6,900) dt, ndte, npt, istep0
        write (6,901) diagfreq, histfreq, dumpfreq
        write (6,902) kitd, kdyn, kstrength, kcolumn
        write (6,*) ' '
        if (evp_damping) then
          write (6,*) 'Damping procedure used in evp dynamics'
        else
          write (6,*) 'Damping procedure NOT used in evp dynamics'
        endif
        write (6,*) ' '
        if (snow_into_ocn) then
          write (6,*) 'Snow will be thrown into ocn during ridging'
        else
          write (6,*) 'Snow will NOT be thrown into ocn during ridging'
        endif
        write (6,*) ' '
#ifdef coupled
       if( oceanmixed_ice ) then
         write (6,*) 'WARNING:  coupled and oceanmixed flags are BOTH ON'
         write (6,*) 'Ocean data received from coupler will be altered '
         write (6,*) 'by mixed layer routine! WARNING WARNING WARNING '
         write (6,*) ' '
       endif
#endif
        write (6,*) 'Advection algorithm is ',advection
        write (6,*) ' '
        write (6,*) 'Diagnostic data for the grid points specified'
        if (print_points) then
          write (6,*) 'in ice_diagnostics.H will be printed'
        else
          write (6,*) 'in ice_diagnostics.H will NOT be printed'
        endif
        write (6,*) ' '
        if (grid_type .eq. 'displaced_pole') then
          write (6,*) 'Displaced pole grid'
          write (6,*) 'The grid file is ',grid_file
          write (6,*) 'The land mask file is ', kmt_file
        elseif (grid_type .eq. 'column') then
          write (6,*) 'Column model grid'
        else
          write (6,*) 'Rectangular grid'
        endif

        write (6,*) 'The dump file prefix is ', dump_file
        if (restart) then
          write (6,*) 'The restart file pointer is ', pointer_file
        else
          write (6,*) 'Starting from default initialization '
          write (6,*) 'rather than restart file'
        endif
        write (6,*) 'The history file prefix is ', history_file
        if (hist_avg) then
          write (6,*) 'History data will be averaged over 1 ',histfreq
        else
          write (6,*) 'History data will be snapshots'
        endif
      endif

 900  format ('  dt=',f7.1,'  ndte=',i3,'  npt=',i6, '  istep0=',i8)
 901  format ('  diagfreq=',i3,'  histfreq=',a2,'  dumpfreq=',a2)
 902  format ('  kitd=',i2,'  kdyn=',i2,'  kstrength=',i2,
     &    '  kcolumn=',i2)

      end subroutine input_data
!
!BOP
!
! !IROUTINE: init_state - initialize state for itd 
!
! !INTERFACE:
!
      subroutine init_state
!
! !DESCRIPTION:
!
! Initialize state for the itd model
!
! !REVISION HISTORY:
! 
! author C. M. Bitz
!
! !USES:
!
      use ice_model_size
      use ice_constants
      use ice_flux
      use ice_vthermo
      use ice_grid
      use ice_state
      use ice_itd
      use ice_dh
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) :: layer,nc,i,j,k
      real (kind=dbl_kind) ::
     &   slope, Ti
     &,  ainit(ncat)

      do nc=1,ncat
        do j=1,jmt_local
        do i=1,imt_local
          aicen(i,j,nc) = c0
          vicen(i,j,nc) = c0
          vsnon(i,j,nc) = c0
        enddo
        enddo
      enddo

      do nc=1,ncat
        Tsfcn(ilo:ihi,jlo:jhi,nc) = Tf(ilo:ihi,jlo:jhi)
        call bound(tsfcn(:,:,nc))
      enddo

      do layer=1,ntilay
        do j=1,jmt_local
        do i=1,imt_local
            eicen(i,j,layer) = c0
        enddo
        enddo
      enddo

      if (ncat.eq.1) then
      ainit(1)=.8_dbl_kind
      else
      ainit(1)=.2_dbl_kind
      ainit(2)=.3_dbl_kind
      ainit(3)=.3_dbl_kind
      do nc=4,ncat
         ainit(nc)=0.1_dbl_kind
      enddo
      endif

      do j = jlo,jhi
      do i = ilo,ihi
        if (tmask(i,j)) then
        ! place ice where ocean sfc is cold
        if ( (sst (i,j).le.Tf(i,j)+p2) .and.
     &       (ULAT(i,j).lt.-40.0_dbl_kind/rad_to_deg .or.
     &        ULAT(i,j).gt. 40.0_dbl_kind/rad_to_deg)    )    then

          do nc = 1,ncat
            aicen(i,j,nc) = ainit(nc)
            if (nc.lt.ncat) then
              vicen(i,j,nc) = p5*(hin_max(nc-1)
     $           + hin_max(nc))*aicen(i,j,nc)   ! m
            else ! nc=ncat
              vicen(i,j,nc) = (hin_max(nc-1)+0.9_dbl_kind)*aicen(i,j,nc) ! m
            endif
            vsnon(i,j,nc) = p2*aicen(i,j,nc)   ! m
            
            Tsfcn(i,j,nc) = min(Tsmelt,Tair(i,j) - Tffresh)    ! deg C       
            ! make linear temp profile and compute enthalpy
            slope = Tf(i,j) - Tsfcn(i,j,nc)
            k=1
            do layer=layer1(nc),layern(nc)
              Ti=Tsfcn(i,j,nc)+slope*(real(k)-p5)/nilay(nc)
         eicen(i,j,layer)=energ(Ti,saltz(k,nc))*vicen(i,j,nc)/nilay(nc)
              k=k+1
            enddo
          enddo
        endif
        endif
      enddo
      enddo

      ! compute aggregate ice state and open water area
      call aggregate 

      end subroutine init_state
!
!BOP
!
! !IROUTINE: init_flux - initialize fluxes exchanged with coupler
!
! !INTERFACE:
!
      subroutine init_flux
!
! !DESCRIPTION:
!
! Initialize all fluxes exchanged with flux coupler
! and some data derived fields
!
! !REVISION HISTORY:
! 
! author Elizabeth C. Hunke, LANL
!
! !USES:
!
      use ice_constants
      use ice_flux
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer i,j

c$OMP PARALLEL DO PRIVATE(i,j)
      do j=jlo,jhi
      do i=ilo,ihi
      !-----------------------------------------------------------------
      ! fluxes received
      !-----------------------------------------------------------------
        zlvl(i,j) = c10    ! atm level height (m)
        uatm(i,j) = c5     ! wind velocity    (m/s)
        vatm(i,j) = c5
        potT(i,j) = 260.0_dbl_kind   ! air potential temperature  (K)
        rhoa(i,j) = 1.3_dbl_kind    ! air density (kg/m^3)
        Fsnow(i,j) = 3.3e-6_dbl_kind ! snowfall rate (kg/m2/s)
        Frain(i,j) = c0     ! rainfall rate (kg/m2/s)
        Fsw(i,j) = c0      ! shortwave radiation (W/m^2)
        Tair(i,j) = 260.0_dbl_kind   ! air temperature  (K)
        Qa(i,j) = 0.0014_dbl_kind   ! specific humidity (kg/kg)
        Flw(i,j) = 140.0_dbl_kind    ! incoming longwave radiation (W/m^2)
        sst(i,j) = -1.9_dbl_kind    ! sea surface temperature (C) 
        sss(i,j) = 34.0_dbl_kind     ! sea surface salinity (o/oo)
        uocn(i,j) = c0    ! surface ocean currents (m/s)
        vocn(i,j) = c0
        frzmlt(i,j)=-c2    ! freezing/melting potential (W/m^2)

      !-----------------------------------------------------------------
      ! derived or computed fields
      !-----------------------------------------------------------------

        Tf(i,j) = -depressT*sss(i,j)                      ! freezing temp (C)
        wind (i,j) = sqrt(uatm(i,j)**2 + vatm(i,j)**2) ! wind speed, (m/s)

        strocnx(i,j) = c0        ! ice-ocean stress, x-direction (U-cell)
        strocny(i,j) = c0        ! ice-ocean stress, y-direction (U-cell)
        strocnxT(i,j) = c0        ! ice-ocean stress, x-direction (T-cell)
        strocnyT(i,j) = c0        ! ice-ocean stress, y-direction (T-cell)

        Fhocn(i,j) = c0  

      call init_flux_atm(i,j)
      call init_flux_ocn(i,j)

      enddo
      enddo

      end subroutine init_flux
!
!BOP
!
! !IROUTINE: setup_mpi - initialize mpi
!
! !INTERFACE:
!
      subroutine setup_mpi
!
! !DESCRIPTION:
!
! This routine initializes mpi for either internal parallel
! processing or for message passing with the coupler
!
! !REVISION HISTORY:
! 
! author Elizabeth C. Hunke, LANL
! code originally based on POP routine
!
! !USES:
!
      use ice_mpi_internal
      use ice_coupling
      use shr_sys_mod, only : shr_sys_abort
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) ::
     &  ice_task           ! master task for ice
     &, coords1, coords2, n, ilen, jlen
     &, interior_i, interior_j  ! dummies for interior blocks

      master_task = 0

#ifdef coupled
      !  if running in coupled mode
      call MPI_INIT(ierr)
      call mpi_coupled ('ice', cpl_task, ice_task, MPI_COMM_ICE)
#else
      !  if running in stand-alone MPI mode
#ifdef _MPI
#if fcd_coupled
#else
      call MPI_INIT(ierr)
#endif
      call MPI_COMM_DUP(MPI_COMM_WORLD, MPI_COMM_ICE, ierr)
#endif
#endif

#ifdef _MPI
      call MPI_COMM_SIZE (MPI_COMM_ICE, nb_tasks, ierr)
      call MPI_COMM_RANK (MPI_COMM_ICE, my_task, ierr)

      if (nb_tasks.ne.nproc_s) then
        write (6,*) 'nb_tasks must equal nproc_s',nb_tasks,nproc_s
        call shr_sys_abort ('(setup_mpi) ERROR in nb_tasks')
      endif

      if (real(imt_global)/real(nproc_x) /= int(imt_global/nproc_x))
     &  then
        write (6,*) 'number of pes in x must evenly divide imt_global',
     &  'nproc_x=',nproc_x,'imt_global=',imt_global
        call shr_sys_abort ('(setup_mpi) ERROR in x grid decomp')
      endif
      if (real(jmt_global)/real(nproc_y) /= int(jmt_global/nproc_y))
     &  then
        write (6,*) 'number of pes in y must evenly divide jmt_global',
     &  'nproc_y=',nproc_y,'jmt_global=',jmt_global
        call shr_sys_abort ('(setup_mpi) ERROR in y grid decomp')
      endif

      if ( ierr /= MPI_SUCCESS ) then 
         call shr_sys_abort ('(setup_mpi) ERROR after MPI_COMM_xxx')
      endif

      coords1=mod(my_task,nproc_x)
      coords2=my_task/nproc_x
      nbr_east = coords2*nproc_x+mod(my_task+1,nproc_x)
      nbr_west = coords2*nproc_x+mod(my_task-1+nproc_x,nproc_x)
      nbr_north = my_task+nproc_x
      nbr_south = my_task-nproc_x
      if (nbr_south.lt.0) nbr_south = -1
      if (nbr_north.gt.nproc_s-1) nbr_north=-1

      ilen=ihi-ilo+1
      jlen=jhi-jlo+1

      do n=1,nproc_s

      local_start(1,n)=((imt_global-1)/nproc_x+1)*mod((n-1),nproc_x)+1
      local_start(2,n)=((jmt_global-1)/nproc_y+1)*((n-1)/nproc_x)+1

      call MPI_TYPE_VECTOR(jlen, ilen, ilen, 
     &     mpi_integer, mpi_interior_int(n), ierr)
      call MPI_TYPE_COMMIT(mpi_interior_int(n), ierr)

      call MPI_TYPE_VECTOR(jlen, ilen, ilen, 
     &     mpi_real8, mpi_interior_real(n), ierr)
      call MPI_TYPE_COMMIT(mpi_interior_real(n), ierr)

      call MPI_TYPE_VECTOR(jlen, ilen, imt_global, 
     &     mpi_integer, mpi_interior_int_global(n), ierr)
      call MPI_TYPE_COMMIT(mpi_interior_int_global(n), ierr)

      call MPI_TYPE_VECTOR(jlen, ilen, imt_global, 
     &     mpi_real8, mpi_interior_real_global(n), ierr)
      call MPI_TYPE_COMMIT(mpi_interior_real_global(n), ierr)

      enddo

#else
      my_task = master_task
      nb_tasks = 1 
#endif

      end subroutine setup_mpi
!
      end module ice_init
!

