c CVS: $Id: ice_prnpnt.F,v 1.5 2002/04/15 14:24:08 schramm Exp $
c CVS: $Source: /fs/cgd/csm/models/CVS.REPOS/ice/csim4_src/source/ice_prnpnt.F,v $
c CVS: $Name: ccsm2_0_beta46 $
!
!BOP
!
! !MODULE: ice_prnpnt - diagnostic grid point info during execution
!
! !DESCRIPTION:
!
! This module defines the grid points desired for printout of
! atmospheric and ocean inputs and ice model outputs. Data for
! each point is written to a separate file as ascii tables. The
! specific points are set by lat/lons (-90 to 90 lat and -180 to
! +180 lon) in a data statement. A descriptive label is useful 
! to distinguish points. The code chooses the nearest model grid 
! point to the specified lat/lon. Points for task 0 are written
! in the ice model execution directory; all other task points 
! are written one directory up (on IBM).
!
! !REVISION HISTORY:
!
! author  Bruce P. Briegleb, NCAR
!       
! !INTERFACE:

      module ice_prnpnt
!
! !USES:
!
      use ice_domain
      use ice_constants
      use ice_calendar
      use shr_sys_mod, only : shr_sys_flush
!
!EOP
!
      implicit none

      ! point print data
      logical (kind=log_kind) ::
     &    print_points        ! if true, print point data
      integer (kind=int_kind), parameter ::
     &          npnt = 7      ! total number of points to be printed

      integer (kind=int_kind) ::
     $         iunit          ! temporary for print

      real (kind=dbl_kind), dimension(npnt) :: 
     &     latpnt,  ! latitude  of desired points
     $     lonpnt   ! longitude of desired points
      integer (kind=int_kind) ::
     &     untout(npnt)  ! unit of output points
     &,     iindx(npnt)  ! i index for points
     &,     jindx(npnt)  ! j index for points
     &,    nmbtsk(npnt)  ! task number for points

      data latpnt /   75.,  78.,   80., 70.,
     $               -65., -74.,  -55./
      data lonpnt / -145.,-167., -165., -15.,
     $               -40.,-170.,   90./
      data untout /    91,    92,    93,    94,
     $                 81,    82,    83/

!
      contains
!
!BOP 
!
! !IROUTINE: intpnt - initializes printout of grid point data
!
! !INTERFACE:
!
      subroutine intpnt
!
! !DESCRIPTION:
!
! Initialize printout of points by finding tasks for
! requested points, and defining file names for each
! point. Prints out description of points and tasks
! to log file.
!
! !REVISION HISTORY:
!
! author  Bruce P. Briegleb, NCAR
!
! !USES:
!
      use ice_grid
      use ice_mpi_internal
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      real (kind=dbl_kind) ::
     &     minbth,     ! sum of distances in lat and lon
     $     latdis,     ! latitude distance
     $     londis,     ! longitude distance
     $     totdis      ! total distance
      integer (kind=int_kind) ::
     $             m,  ! task index for search
     $             n,  ! point index for search
     $             i,  ! longitude index
     $             j,  ! latitude  index
     $          ilen,  ! number of longs in local domain
     $          jlen   ! number of lats  in local domain

      character (char_len) :: filename
      character (4) :: charnyr
      character (2) :: charmonth,charmday
      character (char_len) :: label(npnt)
      logical (kind=log_kind) :: prnt     ! for prints
      data prnt / .true. /

      ! initialize labels
      if (my_task.eq.master_task) 
     &   write(6,*) ' initialize diagnostic prints '

      label(1)(1:40)  = 'SHEBA Camp (Oct 97)                     '
      label(2)(1:40)  = 'SHEBA Camp (July 98)                    '
      label(3)(1:40)  = 'SHEBA Camp (Oct 98)                     '
      label(4)(1:40)  = 'Icelandic Sea                           '
      label(5)(1:40)  = 'Weddell Sea near Antarctic Circle       '
      label(6)(1:40)  = 'Ross Sea                                '
      label(7)(1:40)  = 'East Antarctica                         '

      ! initialize units. master task first calculates global indices
      ! for all points, then maps these to subdomains and broadcasts
      ! info to all tasks

      if( my_task .eq. master_task ) then
        write(6,*) ' find indices of points '
        do n=1,npnt

          iindx(n)   = 0
          jindx(n)   = 0
          nmbtsk(n)  = -1
      
          write(6,50) n,latpnt(n),lonpnt(n)
   50     format(' .... looking for indices for point = ',i2,
     $           ' lat lon =',2(f7.1,2x))

          ! select closest point less than minbth:
          minbth     = 3.0_dbl_kind    !  within this total distance
          do j=1,jmt_global
            do i=1,imt_global
              latdis = abs(latpnt(n)-TLAT_G(i,j))
              londis = abs(lonpnt(n)-TLON_G(i,j))
              totdis = latdis + londis
              if( totdis .lt. minbth ) then
                jindx(n)   = j
                iindx(n)   = i
              endif
            enddo
          enddo

          if( jindx(n) .eq. 0 .or. iindx(n) .eq. 0 ) then
            write(6,100) latpnt(n),lonpnt(n),iindx(n),jindx(n)
  100       format(/
     $      ' .... failure to find point ....'/
     $      ' .... lat lon iindx(n) jindx(n) = ',
     $        2(f7.1,1x),1x,2(i3,1x))
          else
            write(6,200) n,latpnt(n),lonpnt(n),
     $                TLAT_G(iindx(n),jindx(n)),
     $                TLON_G(iindx(n),jindx(n)),
     $                iindx(n),jindx(n)
  200       format(' .... point with global indices ....'/
     $' .... number  lat     lon   TLAT_G  TLON_G  iindx(n) jindx(n) '/
     $   6x,i4,1x,2(f7.1,1x),2(f7.1,1x),2(i3,8x))

          endif
        enddo

        ! find local indices and tasks, then broadcast

        ilen = ihi - ilo + 1
        jlen = jhi - jlo + 1

        do n=1,npnt
          do m=1,nproc_s
            if(local_start(2,m) .le. jindx(n) .and.
     $        jindx(n) .le. local_start(2,m)+jlen-1 ) then
              if(local_start(1,m) .le. iindx(n) .and.
     $          iindx(n) .le. local_start(1,m)+ilen-1 ) then
                  iindx(n)  = iindx(n) - local_start(1,m) + 1
                  jindx(n)  = jindx(n) - local_start(2,m) + 1
                  nmbtsk(n) = m - 1
              endif
            endif
          enddo
          write(6,300) n,iindx(n),jindx(n),nmbtsk(n)
  300     format(' .... point  local i,j  task = ',4(i3,1x))
        enddo

      endif
                     
      !-----------------------------------------------------------------
      ! Scatter scalars 
      !-----------------------------------------------------------------

      do n=1,npnt
        call ice_bcast_iscalar(iindx(n))
        call ice_bcast_iscalar(jindx(n))
        call ice_bcast_iscalar(nmbtsk(n))
      enddo
                     
      !-----------------------------------------------------------------
      ! Initialize units for specific tasks
      !-----------------------------------------------------------------

      ! make character strings for named files of ascii grid point data

      if( nyr .le. 9 ) then
        write(charnyr,71) nyr
 71     format('000',i1)
      else if( 10 .le. nyr .and. nyr .le. 99 ) then
        write(charnyr,72) nyr
 72     format('00',i2)
      else if( 100 .le. nyr .and. nyr .le. 999 ) then
        write(charnyr,73) nyr
 73     format('0',i3)
      else if( 1000 .le. nyr .and. nyr .le. 9999 ) then
        write(charnyr,74) nyr
 74     format(i4)
      endif 

      if( month .le. 9 ) then
        write(charmonth,81) month
 81     format('0',i1)
      else
        write(charmonth,82) month
 82     format(i2)
      endif

      if( mday .le. 9 ) then
        write(charmday,91) mday
 91     format('0',i1)
      else
        write(charmday,92) mday
 92     format(i2)
      endif

      do n=1,npnt
        if( my_task .eq. nmbtsk(n) ) then
          iunit = untout(n)
          if( n .le. 9 ) then
            write(filename,5) charnyr,charmonth,charmday,n 
    5       format('pnt.',a4,a2,a2,'.0',i1)   
          else
            write(filename,8) charnyr,charmonth,charmday,n 
    8       format('pnt.',a4,a2,a2,'.',i2)   
          endif 
          open(iunit,file=filename)
          write(iunit,10) label(n)(1:40)
   10     format('.* ',a40)
          write(iunit,20) latpnt(n),lonpnt(n)
   20     format('.* lat = ',f7.2,'  lon = ',f7.2)
          write(iunit,30) 
   30     format(
     $' atm  in idate     z      u      v      T       q     SW    ',
     $'LW   rain  snw    SST       Qfrzmlt'/
     $' ice out idate   netsw   lwup   sens   lat   aice    hi    ',
     $' hs    Tsfc    alb   melth   meltw'/
     $' ocn  in  idate  aice  hbl   sst    sss   uocn   vocn    qdp  ',
     $'   dhdx        dhdy'/
     $' ocn out  idate   hbl  sst    fswdn fswabs  flwdn  flwup  ',
     $'  flh   fsh    qdp    frzmlt')
        endif
      enddo

      end subroutine intpnt
!
!BOP 
!
! !IROUTINE: diags_cpl_in - grid point printout of input info from coupler
!
! !INTERFACE:
!
      subroutine diags_cpl_in
!
! !DESCRIPTION:
!
! Printout atmospheric forcing data for selected
! grid points as received from the coupler.
!
! !REVISION HISTORY:
!
! author  Bruce P. Briegleb, NCAR
!
! !USES:
!
      use ice_flux
      use ice_calendar
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) ::
     &   i, j, n                     ! generic indices
     &,  imx, jmx                  ! for diagnostic prints
      real (kind=dbl_kind) ::  
     &   rn,snw           ! for diagnostic prints

      if (print_points) then
       do n=1,npnt
        if( my_task .eq. nmbtsk(n) ) then
         do j=jlo,jhi
          do i=ilo,ihi
           if( j .eq. jindx(n) ) then
             if( i .eq. iindx(n) ) then
               iunit = untout(n)
               rn    = (Frain(i,j)/rhow)*mps_to_cmpyr  ! cm/yr
               snw   = (Fsnow(i,j)/rhos)*mps_to_cmpyr  ! cm/yr
               write(iunit,100) idate,zlvl(i,j),uatm(i,j),vatm(i,j),
     $                Tair(i,j)-Tffresh,Qa(i,j)*c1000,
     $                Fsw(i,j),Flw(i,j),rn,snw,sst(i,j),frzmlt(i,j)
 100           format(' atm  in',1x,i6,1x,3(f6.1,1x),f7.2,1x,f6.1,1x,
     $                4(f5.1,1x),f7.3,1x,f9.1)
               call shr_sys_flush(iunit)
             endif 
           endif
          enddo 
         enddo
        endif
       enddo 
      endif

      end subroutine diags_cpl_in
!
!BOP 
!
! !IROUTINE: diags_cpl_out - grid point printout of info sent to coupler
!
! !INTERFACE:
!
      subroutine diags_cpl_out
!
! !DESCRIPTION:
!
! Printout grid point info sent to coupler
!
! !USES:
!
      use ice_flux
      use ice_albedo
      use ice_calendar
      use ice_state
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) ::
     &   i, j, n                      ! generic indices
      real (kind=dbl_kind) ::  
     &   hi,hs,bbalb,frsh    ! for diagnostic prints 

      if(print_points) then
       do n=1,npnt
        if( my_task .eq. nmbtsk(n) ) then
         do j=jlo,jhi
          do i=ilo,ihi
           if( j .eq. jindx(n) ) then
             if( i .eq. iindx(n) ) then
               iunit = untout(n)
               hi = c0
               hs = c0
               if( aice(i,j) .gt. 0. ) then
                 hi = vice(i,j)/aice(i,j)
                 hs = vsno(i,j)/aice(i,j)
               endif
               bbalb = alvdf(i,j)*(awtvdr+awtvdf) 
     &               + alidf(i,j)*(awtidr+awtidf)
               frsh  = (Fresh(i,j)/rhow)*mps_to_cmpyr/c100 ! m/yr
               write(iunit,100) idate,Fswthru(i,j),Flwout(i,j),
     $               Fsensible(i,j),Flatent(i,j),aice(i,j),hi,hs,
     $               Tsfc(i,j),bbalb,Fhnet(i,j),frsh
 100           format(' ice out',1x,i6,1x,4(f6.1,1x),f5.3,1x,2(f7.3,1x),
     $                f6.2,1x,f5.3,1x,f7.2,1x,f6.2)
               call shr_sys_flush(iunit)
             endif
           endif
          enddo
         enddo
        endif
       enddo
      endif

      end subroutine diags_cpl_out
!
      end module ice_prnpnt
!
