c CVS: $Id: ice_read_write.F,v 1.6 2002/04/15 14:24:08 schramm Exp $
c CVS: $Source: /fs/cgd/csm/models/CVS.REPOS/ice/csim4_src/source/ice_read_write.F,v $
c CVS: $Name: ccsm2_0_beta46 $
!
!BOP
!
! !MODULE: ice_read_write - routines for opening, reading and writing external files
!
! !DESCRIPTION:
!
! Routines for opening, reading and writing external files
!
! !REVISION HISTORY:
!
! author: Tony Craig, NCAR
!
! !INTERFACE:
!
      module ice_read_write
!
! !USES:
!
      use ice_model_size
      use ice_domain 
      use ice_mpi_internal
!
!EOP
!
      implicit none
!
      contains
!
!BOP
!
! !IROUTINE: ice_open - opens an unformatted file for reading
!
! !INTERFACE:
!
      subroutine ice_open(nu,filename,nbits)
!
! !DESCRIPTION:
!
! Opens an unformatted file for reading \\
! nbits indicates whether the file is sequential or direct access
!
! !REVISION HISTORY:
!
! author: Tony Craig, NCAR
!
! !USES:
!
! !INPUT/OUTPUT PARAMETERS:
!
      integer (kind=int_kind), intent(in) :: nu,nbits
      character (*) :: filename
!
!EOP
!
      if (my_task.eq.master_task) then
      if (nbits.eq.0) then
        open(nu,file=filename,form='unformatted')
      else
! tcmod
!#ifdef OSF1
!        open(nu,file=filename,recl=imt_global*jmt_global*nbits/32,
!     &       form='unformatted',access='direct')
!#else
!        open(nu,file=filename,recl=imt_global*jmt_global*nbits/8,
!     &       form='unformatted',access='direct')
!#endif
        open(nu,file=filename,recl=imt_global*jmt_global*nbits/8,
     &       form='unformatted',access='direct')
      endif
      endif

      end subroutine ice_open
!
!BOP
!
! !IROUTINE: ice_read - reads an unformatted file
!
! !INTERFACE:
!
      subroutine ice_read(nu,nrec,work,atype,scatter)
!
! !DESCRIPTION:
!
! Reads an unformatted file \\
! work is a real array, atype indicates the format of the data
!
! !REVISION HISTORY:
!
! author: Tony Craig, NCAR
!
! !USES:
!
! !INPUT/OUTPUT PARAMETERS:
!
      integer (kind=int_kind), intent(in) :: nu,nrec
      real (kind=dbl_kind) ::  work(ilo:ihi,jlo:jhi)
      character (len=4) atype  ! real,int
      logical (kind=log_kind) :: scatter
!
!EOP
!
      integer (kind=int_kind) :: n,i,j
      real (selected_real_kind(12)) :: workg (imt_global,jmt_global) ! 8 byte
      real (selected_real_kind(6)) ::  workg4(imt_global,jmt_global) ! 4 byte
      integer (selected_int_kind(6)) :: workgi(imt_global,jmt_global)! 4 byte
      real (kind=dbl_kind) :: amin,amax

      if (my_task.eq.master_task) then
        if (atype.eq.'ida4') then
          read(nu,rec=nrec) workgi
          workg=real(workgi)
        elseif (atype.eq.'rda8') then
          read(nu,rec=nrec) workg
        elseif (atype.eq.'rda4') then
          read(nu,rec=nrec) workg4
          workg = workg4
        elseif (atype.eq.'ruf8') then
          read(nu) ((workg(i,j),i=1,imt_global),j=1,jmt_global)
        else
          write (6,*) ' ERROR: reading unknown atype ',atype
        endif
        amin=workg(1,1)
        amax=workg(1,1)
        do j=1,jmt_global
        do i=1,imt_global
          amin=min(amin,workg(i,j))
          amax=max(amax,workg(i,j))
        enddo
        enddo
        write (6,*) ' read_global ',nu,nrec,amin,amax
      endif

      if (scatter) call global_scatter(workg,work)

      end subroutine ice_read
!
!BOP
!
! !IROUTINE: ice_write - writes an unformatted file
!
! !INTERFACE:
!
      subroutine ice_write(nu,nrec,work,atype,gather)
!
! !DESCRIPTION:
!
! Writes an unformatted file \\
! work is a real array, atype indicates the format of the data
!
! !REVISION HISTORY:
!
! author: Tony Craig, NCAR
!
! !USES:
!
! !INPUT/OUTPUT PARAMETERS:
!
      integer (kind=int_kind), intent(in) :: nu,nrec
      real (kind=dbl_kind) ::  work(ilo:ihi,jlo:jhi)
      character (len=4) atype  ! real,int
      logical (kind=log_kind) :: gather 
!
!EOP
!
      integer (kind=int_kind) :: n,i,j
      real (selected_real_kind(12)) :: workg (imt_global,jmt_global) ! 8 byte
      integer (selected_int_kind(6)) :: workgi(imt_global,jmt_global)! 4 byte
      real (kind=dbl_kind) :: amin,amax

      if (gather) call global_gather(workg,work)

      if (my_task.eq.master_task) then
        if (atype.eq.'ida4') then
          workgi=nint(workg)
          write(nu,rec=nrec) workgi
        elseif (atype.eq.'rda8') then
          write(nu,rec=nrec) workg
        elseif (atype.eq.'ruf8') then
          write(nu) ((workg(i,j),i=1,imt_global),j=1,jmt_global)
        else
          write (6,*) ' ERROR: writing unknown atype ',atype
        endif
        amin=workg(1,1)
        amax=workg(1,1)
        do j=1,jmt_global
        do i=1,imt_global
          amin=min(amin,workg(i,j))
          amax=max(amax,workg(i,j))
        enddo
        enddo
        write (6,*) ' writ_global ',nu,nrec,amin,amax
      endif

      end subroutine ice_write
!
      end module ice_read_write
!
