!===============================================================================
! CVS $Id: advance.F90,v 1.6 2002/01/23 22:03:37 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ice/dice5/advance.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE advance()

      use calendar_mod  ! calendar routines
      use shr_timer_mod ! shared code: timing routines
      use const_mod     ! physical constants

      implicit none

      !----- global -----
#include <dims.h>       
#include <data_msg.h>   
#include <data_raw.h>   
#include <control.h>    

      !----- local constants ------
      real,parameter :: snwfrac  = 0.286    ! snow cover fraction ~ [0,1]
      real,parameter :: as_nidf  = 0.95     ! albedo: snow,near-infr,diffuse
      real,parameter :: as_vsdf  = 0.70     ! albedo: snow,visible  ,diffuse
      real,parameter :: as_nidr  = 0.96     ! albedo: snow,near-infr,direct
      real,parameter :: as_vsdr  = 0.80     ! albedo: snow,visible  ,direct
      real,parameter :: ai_nidf  = 0.7      ! albedo: ice, near-infr,diffuse
      real,parameter :: ai_vsdf  = 0.5      ! albedo: ice, visible  ,diffuse
      real,parameter :: ai_nidr  = 0.7      ! albedo: ice, near-infr,direct
      real,parameter :: ai_vsdr  = 0.5      ! albedo: ice, visible  ,direct

      !----- local ------
      integer  i,j          ! generic loop indicies
      real     f1,f2        ! two fractions used in time interpolation
      integer  m1,m2        ! two months    used in time interpolation
      integer  dpm          ! number of days per month
      integer  eday1,eday2  ! number of elapsed days at start of months
      real     rmonth       ! month as a real number, Jan 1 => rmonth=0
      real     Tice         ! temperature for solid ice
      real     dt           ! seconds per coupler communication
      real     Qacc         ! heat flux st: Qacc*dt = -latice*water
      real     old_ifrac    ! old/previous value for ice fraction
      real     new_ifrac    ! new/updated  value for ice fraction
      integer  timer_n      ! timer number

      logical  first_call   ! flags 1st call of this routine
      data     first_call  /.true./
      save

!-------------------------------------------------------------------------------
! PURPOSE:
!   advance the model output states & fluxes to current date & time
!   (ie. time interplate raw data fields onto current date & time)
!
! NOTE:
!   o here we assume the raw SST data is a 12 month climatology
!-------------------------------------------------------------------------------

      if (first_call) call shr_timer_get(timer_n,'advance')
      call shr_timer_start(timer_n)

      !-----------------------------------------------------------------
      ! read in the raw data (if necessary)
      !-----------------------------------------------------------------
      call data_in()

      !-----------------------------------------------------------------
      ! compute time interpolation factors (assumes monthly mean data)
      !-----------------------------------------------------------------
      if (month.lt.12) then
        call ymd2eday(year,month  ,1,eday1)
        call ymd2eday(year,month+1,1,eday2)
      else
        call ymd2eday(year  ,month,1,eday1)
        call ymd2eday(year+1,    1,1,eday2)
      end if
      dpm = eday2 - eday1
      if (day.le.15) then
          !---  m1 = month-1, m2=month
          !---  day= 1  +     0sec => f1=0.5, f2=0.5 
          !---  day= 15 + 86400sec => f1=0.0, f2=1.0 
          m1 = month - 1
          m2 = month
          if (m1.eq.0) m1=12
          f2 = 0.5 + 0.5*(float(day-1) + sec/86400.0)/15.0
          f1 = 1.0 - f2
          rmonth = float(month-1) + f2 - 0.5
      else
          !---  m1 = month, m2=m1+1
          !---  day= 16  +     0sec => f1=1.0, f2=0.0 
          !---  day= dpm + 86400sec => f1=0.0, f2=0.5 
          m1 = month
          m2 = month + 1
          if (m2.eq.13) m2=1
          f2 =       0.5*(float(day-16) + sec/86400.0)/float(dpm-15)
          f1 = 1.0 - f2
          rmonth = float(month-1) + f2 + 0.5
      end if

      if (info_dbug.ge.3) write(6,9003) 'm1,m2,f1,f2=',m1,m2,f1,f2

      !-----------------------------------------------------------------
      ! time interpolate raw SST data -- this is *not* ice temperature
      !-----------------------------------------------------------------
      do j=1,ny
      do i=1,nx
         T(i,j) = f1*r_t(i,j,m1) + f2*r_t(i,j,m2)
      end do
      end do

      !-----------------------------------------------------------------
      ! advance the model state (time interpolate) & compute swnet diag
      !-----------------------------------------------------------------
!$OMP PARALLEL DO PRIVATE(j,i,Tice,new_ifrac,old_ifrac)
      DO j=1,ny
      DO i=1,nx

          !--- update ice fraction ---------------------
	  old_ifrac  = ifrac(i,j)
          new_ifrac  = max( 0.0, min(1.0, Tfrz+1.0 - T(i,j)) )
          ifrac(i,j) = new_ifrac

          !--- update ice temperature ------------------
          if ( yc(i,j) > 0.0) then !-- peak n-hemi T on Sep 1st --
             Tice = 260.0 + 10.0*cos(2.0*pi*(rmonth - 8.0)/12.0)
          else
             Tice = 260.0 - 10.0*cos(2.0*pi*(rmonth - 8.0)/12.0)
          end if
          if (new_ifrac > 0.0) T(i,j) = Tfrz + new_ifrac*(Tice-Tfrz)

          !--- update albedos (currently time invariant) ---
          avsdr(i,j) = ai_vsdr*(1.0-snwfrac) + as_vsdr*snwfrac
          anidr(i,j) = ai_nidr*(1.0-snwfrac) + as_nidr*snwfrac
          avsdf(i,j) = ai_vsdf*(1.0-snwfrac) + as_vsdf*snwfrac
          anidf(i,j) = ai_nidf*(1.0-snwfrac) + as_nidf*snwfrac

          !--- swnet is sent to cpl as a diagnostic quantity only ---
          !--- newly recv'd swdn goes with previously sent albedo ---
          !--- NOTE: albedos are (currently) time invariant ---
          swnet(i,j) = (1.0 - avsdr(i,j))*svsdr(i,j) &
          &          + (1.0 - anidr(i,j))*snidr(i,j) &
          &          + (1.0 - avsdf(i,j))*svsdf(i,j) &
          &          + (1.0 - anidf(i,j))*snidf(i,j)

          !--- adjust swnet to work with new ice fraction ---
          !--- note: this fails when ice fraction vanishes ---
          if (new_ifrac > 0.0) swnet(i,j) = swnet(i,j)*old_ifrac/new_ifrac

      END DO
      END DO

      !-----------------------------------------------------------------
      ! compute atm/ice fluxes: sen,lat,lwup,evap,taux,tauy,tref
      !-----------------------------------------------------------------
!$OMP PARALLEL DO PRIVATE(j)
      DO j=1,ny
         call flux_ai(                                                  &
         &  nx         ,mask (1,j) ,z_a   (1,j) ,u_a  (1,j) ,v_a  (1,j) &
         & ,pt_a (1,j) ,sh_a (1,j) ,den_a (1,j) ,t_a  (1,j) ,T    (1,j) &
         & ,sen  (1,j) ,lat  (1,j) ,lwup  (1,j) ,evap (1,j) ,tauxa(1,j) &
         & ,tauya(1,j) ,tref (1,j) )
      END DO

      !-----------------------------------------------------------------
      ! compute ice/ocn fluxes & update accumlated water (if any)
      !-----------------------------------------------------------------
!$OMP PARALLEL DO PRIVATE(j,i,Qacc,swnet,dt)
      DO j=1,ny
      DO i=1,nx
         if (mask(i,j) .eq. 0) then
            pensw(i,j) = 1.0e30
            melth(i,j) = 1.0e30
            meltw(i,j) = 1.0e30
            sflx (i,j) = 1.0e30
            tauxo(i,j) = 1.0e30
            tauyo(i,j) = 1.0e30
         else
            !--- penetrating short wave ---
            if (flux_swpf .le. 0.0) then
              pensw(i,j) = 0.0
            else
              pensw(i,j) = flux_swpf*swnet(i,j)
            end if

            !--- i/o stress is exactly a/i stress ---
            tauxo(i,j) = tauxa(i,j)
            tauyo(i,j) = tauya(i,j)

            !--- no salt flux ---
            sflx (i,j) = 0.0

            !--- do melt/freeze water balance ---
            if ( flux_Qacc.eq.0 ) then
              if ( Q(i,j).lt.0 ) then ! Q<0 => melt
                melth(i,j) =  max(Q(i,j),flux_Qmin)
                meltw(i,j) = -melth(i,j)/latice
              else                   ! Q>0 => freeze
                melth(i,j) = 0.0
                meltw(i,j) = 0.0
              end if
            else
              !--- add or subract from mass/heat budget ---
              dt = 86400.0/float(ncpl)
              if ( Q(i,j) .lt. 0.0 ) then ! Q<0 => melt
                Qacc       = -water(i,j)*latice/dt
                melth(i,j) = max( Q(i,j),Qacc,flux_Qmin )
                meltw(i,j) = -melth(i,j)/latice
                water(i,j) =  water(i,j) - meltw(i,j)*dt
              else                       ! Q>0 => freeze
                melth(i,j) = 0.0
                meltw(i,j) = 0.0
                water(i,j) = water(i,j) + dt*Q(i,j)/latice
              end if
              !--- expand ice extent? ---
              if (ifrac(i,j).le.0.0 .and. water(i,j).gt.0.0) then
                ifrac(i,j) = 0.01
                ! T  (i,j) = Tfrz ! T can be above freezing?!?
              end if
              !--- kludge to assure fixed quantity flux ---
              if (ifrac(i,j).gt.0.0) then
                ifrac(i,j) = max( 0.01, ifrac(i,j))
                melth(i,j) = melth(i,j)/ifrac(i,j)
                meltw(i,j) = meltw(i,j)/ifrac(i,j)
              else
                melth(i,j) = 0.0
                meltw(i,j) = 0.0
              end if
            end if

         end if
      END DO
      END DO

      !-----------------------------------------------------------------
      ! wipe-out out-of-domain values (?)
      !-----------------------------------------------------------------
      if (.false.) then
!$OMP PARALLEL DO PRIVATE(j,i)
        do j=1,ny
        do i=1,nx
          if (mask(i,j).eq.0) then
            swnet(i,j) = 1.0e30
            pensw(i,j) = 1.0e30
            melth(i,j) = 1.0e30
            meltw(i,j) = 1.0e30
            sflx (i,j) = 1.0e30
            tauxo(i,j) = 1.0e30
            tauyo(i,j) = 1.0e30
          end if
        end do
        end do
      end if

      if (info_sleep > 0.0) call shr_timer_sleep(info_sleep)
      call shr_timer_stop(timer_n)
      first_call = .false.

      RETURN
 9003 format('(advance) ',a,2(1x,i3),2(1x,f5.3))
      END
!===============================================================================
