!===============================================================================
! CVS $Id: data_in.F90,v 1.3 2001/07/06 18:11:55 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ice/dice5/data_in.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE data_in()

      use const_mod  ! physical constants

      implicit none

      !----- global -----
#include <dims.h>       
#include <data_msg.h>   
#include <data_raw.h>   
#include <control.h>    
#include <netcdf.inc>   

      !----- netCDF ------
      character*80 fn         ! file name
      integer      fid        ! nc file ID
      integer      vid        ! nc variable ID
      integer      rcode      ! nc return code

      !----- STR data ------
      integer      mx,my      ! STR i & j dimensions
      parameter(mx=181,my=91)
      real         x(mx)      ! STR x-coordinates
      real         y(my)      ! STR y-coordinates
      real     SST(mx,my,12)  ! STR SST data on STR grid

      !----- local ------
      real          tmp       ! temporary variable
      integer       i,j,k,n   ! generic indicies
      character*240 str       ! data file ID string
      integer       datayear  ! data year corresponding to model year
      logical       firstcall ! flags 1-time initialization

      data          firstcall /.true./
      save          firstcall

!-------------------------------------------------------------------------------
! PURPOSE:
!   acquire raw data fields
!
! ASSUMPTIONS
!   o netCDF input data is a 12 month STR SST climatology on an 2x2 grid
!     with domain = [0,360]x[-90,90]
!-------------------------------------------------------------------------------

      if (.not. firstcall) return

      if (data_dir(1:4) .eq. 'null') then
        !---------------------------------------------------------------
        ! no input data file => fabricate bogus data
        !---------------------------------------------------------------
        if (firstcall) then
          write(6,9000) 'null data file => make bogus raw input data'
          do k=1,12
          do j=1,ny
          do i=1,nx
            tmp = Tzro + 30.0 - 40.0*abs(j-(ny/2))/(ny/2)
            tmp = max(Tfrz,tmp) ! T .ge. freezing temperature
            r_t    (i,j,k) =   tmp ! temperature               ~ Kelvin
          end do
          end do
          end do
        end if

      else if (data_dir(1:5).eq.'local') then
        !---------------------------------------------------------------
        ! read in SST data from an existing local file
        !---------------------------------------------------------------
        fn='data.sst.nc'

        write(6,9000) 'open netCDF file ',fn
        rcode = nf_open(fn,NF_NOWRITE,fid)
        if (rcode.ne.NF_NOERR) write(6,9000) nf_strerror(rcode)

        !--- print title info from nc file ---
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'title', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'title'  , str)
        else
          str = '<too long to print>   '
          n = index(str,'   ')
        end if
        write(6,9000) 'o title   = ',str(1:n)
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'history', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'history', str)
        else
          str = '<too long to print>   '
          n = index(str,'   ')
        end if
        write(6,9000) 'o history = ',str(1:n)
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'Source', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'Source' , str)
        else
          str = '<too long to print>   '
          n = index(str,'   ')
        end if
        write(6,9000) 'o source  = ',str(1:n)

        write(6,9000) 'read domain data...'
        rcode = nf_inq_varid(fid,'xc' ,vid)
        rcode = nf_get_var_double(fid,vid,x )
        rcode = nf_inq_varid(fid,'yc' ,vid)
        rcode = nf_get_var_double(fid,vid,y )

        if (x( 1) .ne.   0.0) stop 1
        if (x(mx) .ne. 360.0) stop 2
        if (y( 1) .ne. -90.0) stop 3
        if (y(my) .ne.  90.0) stop 4

        write(6,9000) 'read SST data...'
        rcode = nf_inq_varid(fid,'T',vid)
        rcode = nf_get_var_double(fid,vid,SST)

        rcode = nf_close(fid)

        !---------------------------------------------------------------
        write(6,9000) 'convert SST climatology to Kelvin'
        do k=1,12
        do j=1,my
        do i=1,mx
           SST(i,j,k) = SST(i,j,k) + Tzro
        end do
        end do
        end do

        write(6,9000) 'map SST from STR grid to model grid...'
        do k=1,12
          call Interp2D(x ,y ,SST(1,1,k),mx,my,          &
          &             xc,yc,r_t(1,1,k),nx,ny,1.0e30,0)
        end do

      else 
        !---------------------------------------------------------------
        ! invalid option
        !---------------------------------------------------------------
        write(6,9000) 'ERROR: can''t deal with data_dir = ',data_dir
        stop
      end if

      firstcall = .false.

      RETURN
 9000 format('(data_in) ',3a)
 9001 format('(data_in) ',a,i4,2a)
      END

!===============================================================================
