!===============================================================================
! CVS $Id: data_msg.h,v 1.4 2001/08/01 22:32:08 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ice/dice5/data_msg.h,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================
!  Sign convention:
!    positive value <=> downward flux
!
!  Unit convention:
!    T             ~ Kelvin
!    u,v           ~ m/s
!    pressure      ~ N/m^2 = Pa
!    heat flux     ~ W/m^2
!    momentum flux ~ N/m^2
!    water flux    ~ (kg/s)/m^2
!    salinity      ~ ppt
!    density       ~ kg/m^3
!    coordinates   ~ degrees north or degrees east
!-------------------------------------------------------------------------------

      integer,parameter :: nibuff=100

      !-----------------------------------------------------------------
      ! output: data sent to coupler  
      !-----------------------------------------------------------------

      !----- flags -----
      integer   ibuffs(nibuff) ! control flags sent to coupler

      !----- domain -----
      real      xc(  nx,ny)    ! x-grid center coordinates ~ deg east
      real      yc(  nx,ny)    ! y-grid center coordinates ~ deg north
      real      xv(4,nx,ny)    ! x-grid vertex coordinates ~ deg east
      real      yv(4,nx,ny)    ! y-grid vertex coordinates ~ deg north
      integer mask(  nx,ny)    ! domain mask: 0 <=> cell NOT in domain
      real    area(  nx,ny)    ! grid cell area

      !----- data fields -----
      real,target :: ifrac(nx,ny)  ! state, ice: frac ice coverage   ~ [0,1]
      real,target :: T    (nx,ny)  ! state, ice: temperature         ~ Kelvin
      real,target :: avsdr(nx,ny)  ! state, ice: albedo, visible/dir ~ [0,1]
      real,target :: anidr(nx,ny)  ! state, ice: albedo, near IR/dir ~ [0,1]
      real,target :: avsdf(nx,ny)  ! state, ice: albedo, visible/dif ~ [0,1]
      real,target :: anidf(nx,ny)  ! state, ice: albedo, near IR/dif ~ [0,1]

      real,target :: tauxa(nx,ny)  ! stress    : a/i zonal           ~ N/m^2
      real,target :: tauya(nx,ny)  ! stress    : a/i meridianal      ~ N/m^2
      real,target :: lat  (nx,ny)  ! heat  flux: latent              ~ W/m^2
      real,target :: sen  (nx,ny)  ! heat  flux: sensible            ~ W/m^2
      real,target :: lwup (nx,ny)  ! heat  flux: long wave up        ~ W/m^2
      real,target :: swnet(nx,ny)  ! heat  flux: sw-net absorbed     ~ W/m^2
      real,target :: evap (nx,ny)  ! water flux: evaporation         ~ kg/s/m^2
      real,target :: tref (nx,ny)  ! diagnostic: 2m ref air T        ~ Kelvin
      real,target :: pensw(nx,ny)  ! heat  flux: solar to ocn        ~ W/m^2
      real,target :: melth(nx,ny)  ! heat  flux: from melting        ~ W/m^2
      real,target :: meltw(nx,ny)  ! water flux: from melting        ~ kg/s/m^2
      real,target :: sflx (nx,ny)  ! salt  flux: ?? sources ??       ~ ppt/s/m^2
      real,target :: tauxo(nx,ny)  ! stress    : i/o zonal           ~ N/m^2
      real,target :: tauyo(nx,ny)  ! stress    : i/o meridianal      ~ N/m^2

      !-----------------------------------------------------------------
      ! input: data received from coupler 
      !-----------------------------------------------------------------

      !----- control flags -----
      integer   ibuffr(nibuff) ! control flags received from coupler

      !----- data fields -----
      real,target :: T_o  (nx,ny)  ! state, ocn: temperature           ~ Kelvin
      real,target :: S_o  (nx,ny)  ! state, ocn: salinity              ~ ppt
      real,target :: u_o  (nx,ny)  ! state, ocn: velocity, zonal       ~ m/s
      real,target :: v_o  (nx,ny)  ! state, ocn: velocity, meridional  ~ m/s
      real,target :: z_a  (nx,ny)  ! state, atm: bottom layer height   ~ m
      real,target :: u_a  (nx,ny)  ! state, atm: velocity, zonal       ~ m/s
      real,target :: v_a  (nx,ny)  ! state, atm: velocity, meridional  ~ m/s
      real,target :: pt_a (nx,ny)  ! state, atm: potential temperature ~ Kelvin
      real,target :: t_a  (nx,ny)  ! state, atm: temperature           ~ Kelvin
      real,target :: sh_a (nx,ny)  ! state, atm: specific humidity     ~ kg/kg
      real,target :: den_a(nx,ny)  ! state, atm: air density           ~ kg/m^3
      real,target :: dhdx (nx,ny)  ! state, ocn: surf slope, dh/dx     ~ m/m
      real,target :: dhdy (nx,ny)  ! state, ocn: surf slope, dh/dy     ~ m/m

      real,target :: q    (nx,ny)  ! heat  flux: ice-formed/melt-pot   ~ W/m^2
      real,target :: snidr(nx,ny)  ! heat  flux: sw dn, near-ir, dir   ~ W/m^2
      real,target :: svsdr(nx,ny)  ! heat  flux: sw dn, visible, dir   ~ W/m^2
      real,target :: snidf(nx,ny)  ! heat  flux: sw dn, near-ir, dif   ~ W/m^2
      real,target :: svsdf(nx,ny)  ! heat  flux: sw dn, visible, dif   ~ W/m^2
      real,target :: lwdn (nx,ny)  ! heat  flux: lw dn                 ~ W/m^2
      real,target :: rain (nx,ny)  ! water flux: precip, liquid     ~ kg/s/m^2
      real,target :: snow (nx,ny)  ! water flux: precip, frozen     ~ kg/s/m^2

      !-----------------------------------------------------------------
      ! miscellaneous (I didn't know where else to put it)
      !-----------------------------------------------------------------
      real  water(nx,ny)  ! water accumulated from ocn formed ice

      !-----------------------------------------------------------------
      ! make data global
      !-----------------------------------------------------------------
      COMMON /data_msg/                                   &
     &    xc     , yc    , xv    , yv    ,  mask  , area  &
     &  , ibuffs                                          & 
     &  , ifrac  , T     , avsdr , anidr ,  avsdf         &
     &  , anidf  , tauxa , tauya , lat   ,  sen           &
     &  , lwup   , swnet , evap  , tref  ,  pensw,  melth &
     &  , meltw  , sflx  , tauxo , tauyo                  &
     &  , ibuffr                                          &
     &  , T_o    , S_o   , u_o   , v_o   ,  z_a           &
     &  , u_a    , v_a   , pt_a  , t_a   ,  sh_a          &
     &  , den_a  , dhdx  , dhdy  , q     ,  snidr         &
     &  , svsdr  , snidf , svsdf , lwdn  ,  rain          &
     &  , snow   , water   

      SAVE   /data_msg/
