!===============================================================================
! CVS $Id: domain.F90,v 1.2 2001/01/09 22:59:55 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ice/dice5/domain.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE domain()

      implicit none

      !----- global -----
#include <dims.h>           ! model resolution
#include <data_msg.h>       ! data sent/received from coupler
#include <data_raw.h>       ! data read from data file
#include <control.h>        ! control info, date/time
#include <netcdf.inc>       ! netCDF defs


      !----- local  ------
      integer       i,j,n   ! generic loop indicies
      character*240 str     ! data file ID string
      character*80  fn      ! file name
      integer       fid     ! nc file ID
      integer       vid     ! nc variable ID
      integer       rcode   ! nc return code

!-------------------------------------------------------------------------------
! PURPOSE:
!   get and/or set the model's domain info
!-------------------------------------------------------------------------------


      if (data_dir(1:4).eq.'null') then
        !---------------------------------------------------------------
        ! fabricate grid data suitable for a code test
        !---------------------------------------------------------------
        write(6,9000) 'null data file => make bogus domain data'
        do j=1,ny
        do i=1,nx
          xv(1,i,j) = float(i-1)*360.0/float(nx)
          xv(2,i,j) = float(i  )*360.0/float(nx)
          xv(3,i,j) = xv(2,i,j)
          xv(4,i,j) = xv(1,i,j)
          xc(  i,j)=  (xv(1,i,j)+xv(2,i,j))*0.5

          yv(1,i,j) = -90.0 + float(j-1)*180.0/float(ny)
          yv(2,i,j) = yv(1,i,j)
          yv(3,i,j) = -90.0 + float(j  )*180.0/float(ny)
          yv(4,i,j) = yv(3,i,j)
          yc(  i,j)=  (yv(1,i,j)+yv(3,i,j))*0.5

          mask(i,j) = 1
          if (xv(2,i,j).le.120.AND.yv(4,i,j).le.30.AND.yv(1,i,j).ge.-30) &
          &  mask(i,j) = 0
        end do
        end do

      else if (data_dir(1:5).eq.'local') then
        !---------------------------------------------------------------
        ! read in a domain data from an existing local file
        !---------------------------------------------------------------
        fn='data.domain.nc'
        write(6,9000) 'open netCDF file ',fn
        rcode = nf_open(fn,NF_NOWRITE,fid)
        if (rcode.ne.NF_NOERR) write(6,9000) nf_strerror(rcode)

        !--- print title info from nc file ---
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'title', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'title'  , str)
        else
          str = '<too long to print>   '
          n = index(str,'   ')
        end if
        write(6,9000) 'o title   = ',str(1:n)
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'history', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'history', str)
        else
          str = '<too long to print>   '
          n = index(str,'   ')
        end if
        write(6,9000) 'o history = ',str(1:n)
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'source', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'source' , str)
        else
          str = '<too long to print>   '
          n = index(str,'   ')
        end if
        write(6,9000) 'o source  = ',str(1:n)

        write(6,9000) 'read domain data...'

        rcode = nf_inq_varid(fid,'xc' ,vid)
        rcode = nf_get_var_double(fid,vid,xc)
        rcode = nf_inq_varid(fid,'xv' ,vid)
        rcode = nf_get_var_double(fid,vid,xv)

        rcode = nf_inq_varid(fid,'yc' ,vid)
        rcode = nf_get_var_double(fid,vid,yc)
        rcode = nf_inq_varid(fid,'yv' ,vid)
        rcode = nf_get_var_double(fid,vid,yv)

        rcode = nf_inq_varid(fid,'mask',vid)
        rcode = nf_get_var_int(fid,vid,mask)

        rcode = nf_inq_varid(fid,'area',vid)
        rcode = nf_get_var_double(fid,vid,area)

        rcode = nf_close(fid)

      else 
        !---------------------------------------------------------------
        ! acquire a data file and read in domain data
        !---------------------------------------------------------------
        write(6,9000) 'ERROR: can''t deal with data_dir = ',data_dir
        stop
      end if

      write(6,9001) 'nx,ny = ',nx,ny
      write(6,9002) 'xc: ', (xc(  i,1),i=1,4), (xc(  i,1),i=nx-2,nx)
      write(6,9002) 'xv: ', (xv(1,i,1),i=1,4), (xv(2,i,1),i=nx-3,nx)
      write(6,9002) 'yc: ', (yc(  1,j),j=1,4), (yc(  1,j),j=ny-2,ny)
      write(6,9002) 'yv: ', (yv(1,1,j),j=1,4), (yv(4,1,j),j=ny-3,ny)

      RETURN
 9000 format('(domain) ',3a)
 9001 format('(domain) ',a,2i4)
 9002 format('(domain) ',a,4f9.3,' ...',4f9.3)
      END

!===============================================================================

      SUBROUTINE UnifGrid(x1,xn,n,x)

      implicit  none

      !----- arguments -----
      integer   n
      real      x1,xn,x(n)

      !----- local -----
      integer   i
      real      dx

!-------------------------------------------------------------------------------
!  PURPOSE: make a uniform grid st. x(1)=x1 & x(n)=xn
!-------------------------------------------------------------------------------

      dx   = (xn-x1)/float(n-1)
      x(1) = x1
      do i=2,n-1
        x(i) = x1 + float(i-1)*dx
      end do
      x(n) = xn

      return
      end

!===============================================================================
