!===============================================================================
! CVS $Id: initial.F90,v 1.4 2001/07/06 19:05:00 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ice/dice5/initial.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE initial()

      use shr_sys_mod   ! shared system routines
      use calendar_mod  ! calendar routines

      implicit none

      !----- global -----
#include <dims.h>       
#include <control.h>    

      !----- local -----

      NAMELIST / inparm /   &
     &  rest_type   ,  &
     &  rest_pfile  ,  &
     &  rest_bfile  ,  &
     &  rest_freq   ,  &
     &  rest_date   ,  &
     &  rest_nday   ,  &
     &  rest_odate  ,  &
     &  case_desc   ,  &
     &  case_name   ,  &
     &  data_dir    ,  &
     &  data_miss   ,  &
     &  data_rmlf   ,  &
     &  data_year0  ,  &
     &  data_nyear  ,  &
     &  data_oyear  ,  &
     &  flux_swpf   ,  &
     &  flux_uv     ,  &
     &  flux_Qmin   ,  &
     &  flux_Qacc   ,  &
     &  flux_Qacc0  ,  &
     &  mss_dir     ,  &
     &  mss_pass    ,  &
     &  mss_opts    ,  &
     &  mss_rtpd    ,  &
     &  mss_rmlf    ,  &
     &  info_dbug   ,  &
     &  info_sleep  ,  &
     &  info_time   ,  &
     &  ncpl        

      !----- local -----
      integer      i         ! generic loop indicies
      integer      rcode     ! return code

      !----- formats -----
      character(len=*),parameter :: F00 = "('(initial) ',2a)"
      character(len=*),parameter :: F01 = "('(initial) ',a,i7)"

!-------------------------------------------------------------------------------
! PURPOSE:
!  o set default control variable values
!  o read ice input parameter namelist file
!  o read ice coordinate grids & domain mask
!  o read sst climatology
!  o convert sst climatology from celsius to kelvin
!-------------------------------------------------------------------------------

      !-----------------------------------------------
      ! set initial namelist values
      !-----------------------------------------------
      write(6,'(a)') '(initial) setting namelist defaults.'
      rest_type   = 'continue'  ! run type: initial, cont, regen, branch
      rest_pfile  = 'null'      ! IC data pointer file
      rest_bfile  = 'null'      ! IC data file for branch runs
      rest_freq   = 'coupler'   ! restart file frequency
      rest_date   = 10101       ! start date for initial or branch runs
      rest_nday   = 365         ! restart file interval    (nday option
      rest_odate  = -1          ! restart file offset date (nday option)
      case_desc   = ' '         ! case description
      case_name   = ' '         ! case name
      data_dir    = 'null'      ! data: directory
      data_miss   = 'null'      ! data: inidicates missing data fields
      data_rmlf   = 0           ! data: remove file after reading
      data_year0  = 1           ! data: 1st year in sequence
      data_nyear  = 1           ! data: number of years in sequence
      data_oyear  = 1           ! data: cpl year matched to year0
      flux_swpf   = 0.0         ! flux: sw penetration factor
      flux_uv     = 'zero'      ! flux: (u,v) velocity option
      flux_Qmin   = -1000.0     ! flux: maximum ice melt rate
      flux_Qacc   = 0           ! flux: T => accumulate ice wrt Q
      flux_Qacc0  = 0.0         ! flux: initial ice accumulation
      mss_dir     = 'null'      ! MSS data file directory
      mss_pass    = ' '         ! MSS file password
      mss_opts = 'nowait,nomail'! MSS file mswrite options
      mss_rtpd    = 180         ! MSS file retention period
      mss_rmlf    = 1           ! MSS T => rm local file after mswrite
      info_dbug   = 1           ! dbug level: 0=lowest, 3=highest
      info_sleep  = 0.0         ! simulate time to advance an active model
      info_time   = 0           ! T => print extra timing info
      ncpl        = 24          ! number of msg pairs per day

      !-----------------------------------------------
      ! read input namelist
      !-----------------------------------------------
      write(6,F00) 'reading input namelist.'
      read (5,inparm,iostat=rcode)
      if (rcode > 0) then
         write(6,F01) 'ERROR: reading input namelist, iostat=',rcode
	 call shr_sys_abort("dice namelist read error")
      end if

      !-----------------------------------------------
      ! re-set some initial namelist values
      !-----------------------------------------------

      if ( rest_pfile(1:4).eq.'null') then
        if ( case_name(1:4).eq.'null') then
          rest_pfile = 'rpointer'
        else
          i = index(case_name,' ')-1
          rest_pfile = '$HOME/ice.'//case_name(1:i)//'.rpointer'
        endif
      else if ( rest_pfile(1:11).eq.'./rpointer ') then
        rest_pfile = 'rpointer'
      endif

      !--- mss_dir must end with a slash ---
      i = index(mss_dir,' ')-1
      if (mss_dir(i:i).ne.'/') mss_dir(i+1:i+2)='/ '

      !-----------------------------------------------
      ! record initial namelist values
      !-----------------------------------------------
      write(6,'(a)') '(initial) namelist values:'
      write(6,inparm)
      write(6,*) 

      !-----------------------------------------------
      ! set initial values for date & time
      !-----------------------------------------------
      call date2eday(rest_date,eday)
      call date2eday(rest_date,eday0)
      call date2ymd (rest_date,year,month,day)
      sec = 0

      !-----------------------------------------------
      ! inititial values for control flags
      !-----------------------------------------------
      ierr     = 0  ! error code
      stop_now = 0  ! T => stop now
      stop_eod = 0  ! T => stop at end-of-day
      rest_now = 0  ! T => create restart file now
      rest_eod = 0  ! T => create restart file at end-of-day
      hist_now = 0  ! T => create history file now
      hist_eod = 0  ! T => create history file at end-of-day
      hist_tav = 0  ! T => create monthly avg data
      diag_now = 0  ! T => create diagnostic data now
      diag_eod = 0  ! T => create diagnostic data at end-of-day

      RETURN
      END

!===============================================================================
