!===============================================================================
! CVS $Id: interp.F90,v 1.1 2000/05/22 23:53:32 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ice/dice5/interp.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE Interp2D(Xin ,Yin ,Ain ,mx,my,  &
      &                   Xout,Yout,Aout,nx,ny,spv,mFLAG)

      implicit none

      !----- input -----
      integer mx,my              ! input array dimensions
      real    Xin (mx),Yin(my)   ! input coordinate arrays
      real    Ain (mx,my)        ! data on input grid
      real    spv                ! denotes invalid data
      integer nx,ny              ! output array dimensions
      real    Xout(nx,ny)        ! output X coordinate array
      real    Yout(nx,ny)        ! output Y coordinate array
      integer mFLAG              ! selects extrapolation option

      !----- output -----
      real    Aout(nx,ny)        ! data on output grid

      !----- local -----
      integer i,j,ii,jj       ! loop indicies
      real    a,b,c,d         ! ... as defined in algorithm below
      real    dx,dy           ! ... as defined in algorithm below
      real    alpha,beta      ! ... as defined in algorithm below
      real    Ca,Cb,Cc,Cd     ! bilinear mapping weights 
      real    XL,XM,XH        ! XL=Xin(ii) < XM=Xout(i,j) < XH=Xin(ii+1)
      real    YL,YM,YH        ! YL=Yin(ii) < YM=Yout(i,j) < YH=Yin(ii+1)

!-------------------------------------------------------------------------------
!     PURPOSE:
!       map: Ain -> Aout
!       where Ain (i,j) has real world coordinates (Xin (i),Yin (j))
!       and   Aout(i,j) has real world coordinates (Xout(i,j),Yout(i,j))
!
!     NOTE:
!        mFLAG = 0 => map only if there are 4 valid values
!                     adjacent to the the output grid point
!        mFLAG = 1 => extrapolate if there is at least one valid value
!                     adjacent to the the output grid point
!        mFLAG = other => assume there are no special values
!
!     ASSUMPTION: 
!        Xin(1) =< Xout(i,j) =< Xin(mx), for all i,j
!        Yin(1) =< Yout(i,j) =< Yin(my), for all i,j
!
!     ALGORITHM:
!       for each i & j, 
!         find ii s.t. Xin(ii) < Xout(i,j) <= Xin(ii) and
!         find jj s.t. Yin(jj) < Yout(i,j) <= Yin(jj)
!       then 
!         a = F(Xin(ii  ),Yin(jj  ))
!         b = F(Xin(ii+1),Yin(jj  ))
!         c = F(Xin(ii+1),Yin(jj+1))
!         d = F(Xin(ii  ),Yin(jj+1))
!         dx = Xin(ii+1)-Xin(ii)
!         dy = Yin(jj+1)-Yin(jj)
!         alpha = (Xout(i,j)-Xin(ii))/dx
!         beta  = (Yout(i,j)-Yin(jj))/dy
!       with this info, interpolate to get the value
!         Aout(i,j) = F(Xout(i,j),Yout(i,j))
!
!-------------------------------------------------------------------------------
!           ^  d       c
!       b   |    
!       e  dy      f                interpolate f fromm a,b,c,d
!       t   |                       (avoid spval's)
!       a   v  a       b
!              <--dx--->
!               alpha
!-------------------------------------------------------------------------------

      IF (mFLAG.eq.0) THEN
        DO j=1,ny
        DO i=1,nx
            jj=1
            DO WHILE (Yin(jj+1) .lt. Yout(i,j))
              jj=jj+1
            END DO
            ii=1
            DO WHILE (Xin(ii+1) .lt. Xout(i,j))
              ii=ii+1
            END DO
            a = Ain(ii  ,jj  )
            b = Ain(ii+1,jj  )
            c = Ain(ii+1,jj+1)
            d = Ain(ii  ,jj+1)
            IF (a.eq.spv .or. b.eq.spv .or. c.eq.spv .or. d.eq.spv) THEN
              Aout(i,j) = spv
            ELSE
              alpha = ( Xout(i,j)-Xin(ii) )/( Xin(ii+1)-Xin(ii) )
              beta  = ( Yout(i,j)-Yin(jj) )/( Yin(jj+1)-Yin(jj) )
              Ca = (1.-alpha)*(1.-beta)
              Cb = (   alpha)*(1.-beta)
              Cc = (   alpha)*(   beta)
              Cd = (1.-alpha)*(   beta)
              Aout(i,j) = Ca*a + Cb*b + Cc*c + Cd*d
            END IF
        END DO
        END DO
      ELSE IF (mFLAG.eq.1) THEN
        DO j=1,ny
        DO i=1,nx
            jj=1
            DO WHILE (Yin(jj+1) .lt. Yout(i,j))
              jj=jj+1
            END DO
            ii=1
            DO WHILE (Xin(ii+1) .lt. Xout(i,j))
              ii=ii+1
            END DO
            a = Ain(ii  ,jj  )
            b = Ain(ii+1,jj  )
            c = Ain(ii+1,jj+1)
            d = Ain(ii  ,jj+1)
            XL = Xin (ii  )
            XM = Xout(i,j )
            XH = Xin (ii+1)
            YL = Yin (jj  )
            YM = Yout(i,j )
            YH = Yin (jj+1)
            call IntrpBox(a,b,c,d, XL,XM,XH,YL,YM,YH, spv, Ca,Cb,Cc,Cd)
            Aout(i,j) = Ca*a + Cb*b + Cc*c + Cd*d
        END DO
        END DO
      ELSE
        DO j=1,ny
        DO i=1,nx
            jj=1
            DO WHILE (Yin(jj+1) .lt. Yout(i,j))
              jj=jj+1
            END DO
            ii=1
            DO WHILE (Xin(ii+1) .lt. Xout(i,j))
              ii=ii+1
            END DO
            a = Ain(ii  ,jj  )
            b = Ain(ii+1,jj  )
            c = Ain(ii+1,jj+1)
            d = Ain(ii  ,jj+1)
            alpha = ( Xout(i,j)-Xin(ii) )/( Xin(ii+1)-Xin(ii) )
            beta  = ( Yout(i,j)-Yin(jj) )/( Yin(jj+1)-Yin(jj) )
            Ca = (1.-alpha)*(1.-beta)
            Cb = (   alpha)*(1.-beta)
            Cc = (   alpha)*(   beta)
            Cd = (1.-alpha)*(   beta)
            Aout(i,j) = Ca*a + Cb*b + Cc*c + Cd*d
        END DO
        END DO
      END IF

      RETURN
      END

!===============================================================================

      SUBROUTINE IntrpBox(a,b,c,d, XL,XM,XH, YL,YM,YH, spv, Ca,Cb,Cc,Cd)

      implicit none

      !----- input -----  
      real    a,b,c,d
      real    XL,XM,XH
      real    YL,YM,YH
      real    spv

      !----- output -----  
      real    Ca,Cb,Cc,Cd

      !----- local -----  
      integer ID
      real    dy,dx
      real    alpha ,beta
      real    alpha2,beta2

!-------------------------------------------------------------------------------
!           ^  d       c
!       b   |    
!       e  dy      f                interpolate f from a,b,c,d
!       t   |                       (avoiding special values)
!       a   v  a       b
!              <--dx--->
!               alpha
!-------------------------------------------------------------------------------
!     PURPOSE:  
!        given XL<=XM<=XH
!              YL<=YM<=YH
!              a=F(XL,YL)
!              b=F(XH,YL)
!              c=F(XH,YH)
!              d=F(XL,YH)
!        find Ca,Cb,Cc,Cd to get an interpolated value for F(XM,YM)
!             F(XM,YM) = Ca*a + Cb*b + Cc*c + Cd*d
!     INPUT:  
!        a,b,c,d,
!        XL,XM,XH
!        YL,YM,YH
!        spv
!     OUTPUT: 
!        Ca,Cb,Cc,Cd
!-------------------------------------------------------------------------------

      ID = 0
      IF (a .ne. spv) ID=ID+1
      IF (b .ne. spv) ID=ID+2
      IF (c .ne. spv) ID=ID+4
      IF (d .ne. spv) ID=ID+8

      Ca=0.0
      Cb=0.0
      Cc=0.0
      Cd=0.0

!     IF (ID .gt. 2) THEN
        dx    =  XH-XL
        dy    =  YH-YL
        alpha = (XM-XL)/dx
        beta  = (YM-YL)/dy
!     END IF

      !--- interpolate with four points --------------------------------
      IF (ID .eq. 15) THEN
         Ca = (1.-alpha)*(1.-beta)
         Cb = (   alpha)*(1.-beta)
         Cc = (   alpha)*(   beta)
         Cd = (1.-alpha)*(   beta)
      !--- interpolate with no points ----------------------------------
      ELSE IF (ID .eq. 0) THEN
         Ca=1.0
      !--- interpolate with one point ----------------------------------
      ELSE IF (ID .eq. 1) THEN
         Ca=1.0
      ELSE IF (ID .eq. 2) THEN
         Cb=1.0
      ELSE IF (ID .eq. 4) THEN
         Cc=1.0
      ELSE IF (ID .eq. 8) THEN
         Cd=1.0
      !--- interpolate with two adjacent points ------------------------
      ELSE IF (ID .eq.  3) THEN
         Ca=1.-alpha
         Cb=alpha
      ELSE IF (ID .eq. 12) THEN
         Cd=1.-alpha
         Cc=alpha
      ELSE IF (ID .eq.  6) THEN
         Cb = 1.-beta
         Cc = beta
      ELSE IF (ID .eq.  9) THEN
         Ca = 1.-beta 
         Cd = beta
      !--- interpolate with two opposite points ------------------------
      ELSE IF (ID .eq. 5) THEN
         alpha2 = sqrt((alpha*dx)**2+(beta*dy)**2)/sqrt(dx**2+dy**2)
         Ca = 1.-alpha2
         Cc = alpha2
      ELSE IF (ID .eq. 10) THEN
         alpha2= sqrt(((1.-alpha)*dx)**2+(beta*dy)**2)/sqrt(dx**2+dy**2)
         Cb = 1.-alpha2 
         Cd = alpha2
      !--- interpolate with three points -------------------------------
      ELSE IF (ID .eq. 11) THEN
         Ca = 1. - alpha - beta
         Cb = alpha
         Cd = beta
      ELSE IF (ID .eq. 7) THEN
         Ca = 1. - alpha
         Cb = alpha -beta
         Cc = beta
      ELSE IF (ID .eq. 13) THEN
         Ca = 1. - beta
         Cc = alpha
         Cd = -alpha + beta
      ELSE IF (ID .eq. 14) THEN
         Cc = alpha + beta -1.
         Cb = 1. - beta
         Cd = 1. - alpha
      ELSE
         write(6,*) '(InterpBox) FATAL ERROR'
         STOP 'IntrpBx'
      END IF

      RETURN
      END

!===============================================================================
