!===============================================================================
! CVS $Id: msg.F90,v 1.7 2001/08/01 22:32:09 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ice/dice5/msg.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE msg_pass(cOption)

      use shr_msg_mod   ! shared code: msg passing
      use shr_sys_mod   ! shared code: system calls
      use shr_timer_mod ! shared code: timing routines

      implicit none

      !----- arguments -----
      character*(*) cOption

      !----- global -----
#include <dims.h>                  
#include <data_msg.h>              
#include <control.h>               

      !----- local -----
      integer,parameter :: nsnd=21     ! number of 2d fields to send
      integer,parameter :: nrcv=21     ! number of 2d fields to recv
!!!   real        :: buffs(nx,ny,nsnd) ! contiguous array for sending data
!!!   real        :: buffr(nx,ny,nrcv) ! contiguous array for recving data
      real        :: buffs(nsnd,nx,ny) ! contiguous array for sending data
      real        :: buffr(nx*ny*nrcv) ! contiguous array for recving data
      real,pointer:: field(:,:)        ! generic pointer to 2d field
      integer     :: nc,ns,nn          ! indicies wrt buffr compression scheme
      integer     :: i,j,k,m,n         ! generic loop indicies

      integer,save:: timer_n           ! timer number
      integer,save:: timer_m           ! timer number

      integer*8   :: irtc_w            ! rtc tics when waiting for msg
      integer*8   :: irtc_r            ! rtc tics when msg sent or recved
      integer*8   :: irtc_u            ! rtc tics when msg unpacked
      integer*8   :: irtc_p            ! rtc tics when packing msg
      integer*8   :: irtc_s            ! rtc tics when sending msg

!-------------------------------------------------------------------------------
! PURPOSE:
!   implement various message passing tasks
!
! FUCTIONALITY:
!   cOption=
!     'connect'    => connect to the msg passing system
!     'init'       => recv/send initial info
!     'send'       => send model state & output fluxes
!     'recv'       => receive input fluxes & stop/continue flags
!     'disconnect' => disconnect from the msg passing system
!-------------------------------------------------------------------------------

      IF (cOption .eq. 'connect') THEN 
        !-----------------------------------------------------------------------
        ! connect to msg passing system
        !-----------------------------------------------------------------------
        write(6,9000) '(connect) connecting to cpl'
        call shr_msg_init('ice')
        call shr_msg_groups('ice')
        call shr_sys_flush(6)
         
        call shr_timer_get(timer_n,'msg: ice  -> cpl')
        call shr_timer_get(timer_m,'msg: ice <-  cpl')

      ELSE IF (cOption .eq. 'init') THEN 
        !-----------------------------------------------------------------------
        ! recv/send initial data
        !-----------------------------------------------------------------------
        write(6,9000) '(init) recv initial msg'

        call shr_msg_recv_i(ibuffr,nibuff ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2II)
        
        ierr      = ibuffr( 1) ! error code
        cdate     = ibuffr( 4) ! current date
        sec       = ibuffr( 5) ! elapsed sec on current date
        info_time = ibuffr(11) ! T => turn on msg-passing timing
        !nfo_dbug = ibuffr(12) ! cpl debug level

        !--- send grid & domain mask ---
        write(6,9000) '(init) send initial msg'

        ibuffs    =0      ! unused
        ibuffs( 1)=ierr   ! error code
        ibuffs( 7)=nx     ! number of grid points in x-direction
        ibuffs( 8)=ny     ! number of grid points in y-direction
        ibuffs( 9)=ncpl   ! number of msg-pairs per day
        
        call shr_msg_send_i(ibuffs,nibuff ,SHR_MSG_TID_CPL,SHR_MSG_TAG_I2CI)
        call shr_msg_send_r(xc    ,nx*ny  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_I2CI)
        call shr_msg_send_r(yc    ,nx*ny  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_I2CI)
        call shr_msg_send_r(xv    ,nx*ny*4,SHR_MSG_TID_CPL,SHR_MSG_TAG_I2CI)
        call shr_msg_send_r(yv    ,nx*ny*4,SHR_MSG_TID_CPL,SHR_MSG_TAG_I2CI)
        call shr_msg_send_r(area  ,nx*ny  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_I2CI)
        call shr_msg_send_i(mask  ,nx*ny  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_I2CI)

      ELSE IF (cOption .eq. 'send') THEN 
        !-----------------------------------------------------------------------
        ! send output field msg
        !-----------------------------------------------------------------------
        if (info_dbug.ge.3) write(6,9000) '(send) send data to cpl'
        call shr_timer_start(timer_n)

        !--- set info buffer flags ---
        irtc_p = shr_sys_irtc()   ! packing msg
        ibuffs     = 0     ! unused
        ibuffs( 1) = ierr  ! error code
        ibuffs( 3) = 0     ! stop flag: 0 <=> able to continue
        ibuffs( 4) = cdate ! model date, coded: yyyymmdd
        ibuffs( 5) = sec   ! elapsed seconds on model date
        ibuffs( 7) = nx    ! number of grid points in x-direction
        ibuffs( 8) = ny    ! number of grid points in y-direction
        ibuffs( 9) = ncpl  ! number of msg-pairs per day
        ibuffs(10) = nx*ny

        !--- pack & send msg buffer ---
        do j=1,ny
        do i=1,nx
           buffs(1 ,i,j) = ifrac(i,j) ! state  ice: frac ice coverage
           buffs(2 ,i,j) = T    (i,j) ! state  ice: temperature
           buffs(3 ,i,j) = avsdr(i,j) ! state  ice: albedo, visible/dir
           buffs(4 ,i,j) = anidr(i,j) ! state  ice: albedo, near-ir/dir
           buffs(5 ,i,j) = avsdf(i,j) ! state  ice: albedo, visible/dif
           buffs(6 ,i,j) = anidf(i,j) ! state  ice: albedo, near-ir/dif

           buffs(7 ,i,j) = tauxa(i,j) ! stress    : a/i zonal
           buffs(8 ,i,j) = tauya(i,j) ! stress    : a/i meridianal
           buffs(9 ,i,j) = lat  (i,j) ! heat  flux: latent
           buffs(10,i,j) = sen  (i,j) ! heat  flux: sensible
           buffs(11,i,j) = lwup (i,j) ! heat  flux: long wave up
           buffs(12,i,j) = evap (i,j) ! water flux: evaporation
           buffs(13,i,j) = tref (i,j) ! diagnostic: 2m ref temperature
           buffs(14,i,j) = pensw(i,j) ! heat  flux: solar thru ice to ocn
           buffs(15,i,j) = melth(i,j) ! heat  flux: from melting
           buffs(16,i,j) = meltw(i,j) ! water flux: from melting
           buffs(17,i,j) = sflx (i,j) ! salt  flux: from ???
           buffs(18,i,j) = tauxo(i,j) ! stress    : i/o zonal
           buffs(19,i,j) = tauyo(i,j) ! stress    : i/o meridianal
           buffs(20,i,j) = swnet(i,j) ! heat flux: short wave net absorbed
           buffs(21,i,j) = float(j-1)*nx+i  ! global index
        end do
        end do

        irtc_s = shr_sys_irtc()   ! sending msg
        call shr_msg_send_i(ibuffs,nibuff    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_I2C)
        call shr_msg_send_r( buffs,nx*ny*nsnd,SHR_MSG_TID_CPL,SHR_MSG_TAG_I2C)
        call shr_timer_stop(timer_n)

        if (info_time.ne.0) then
           write(6,9099) irtc_p,'i->c packing'
           write(6,9099) irtc_s,'i->c sending'
        end if

        
      ELSE IF (cOption .eq. 'recv') THEN
        !-----------------------------------------------------------------------
        ! recv input field msg
        !-----------------------------------------------------------------------
        if (info_dbug.ge.3) write(6,9000) '(recv) recv data to cpl'
        call shr_timer_start(timer_m)
        
        irtc_w = shr_sys_irtc()   ! waiting for msg
        call shr_msg_recv_i(ibuffr,nibuff    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2I)

        nc = ibuffr(13) ! number of points excluding equitorial "zero region"
        ns = ibuffr(14) ! ns+1 = southern bound of "zero region" (start point)
        nn = ibuffr(15) ! nn-1 = northern bound of "zero region" (end   point)

        irtc_r = shr_sys_irtc()   ! received msg
        call shr_msg_recv_r( buffr,   nc*nrcv,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2I)
        
        do k=1,nrcv
          if (k== 1) field => T_o   ! state  ocn: temperature
          if (k== 2) field => S_o   ! state  ocn: salinity
          if (k== 3) field => u_o   ! state  ocn: velocity, zonal
          if (k== 4) field => v_o   ! state  ocn: velocity, meridional
          if (k== 5) field => z_a   ! state  atm: bottom layer height
          if (k== 6) field => u_a   ! state  atm: velocity, zonal
          if (k== 7) field => v_a   ! state  atm: velocity, meridional
          if (k== 8) field => pt_a  ! state  atm: pot temperature
          if (k== 9) field => t_a   ! state  atm: temperature
          if (k==10) field => sh_a  ! state  atm: specific humidity
          if (k==11) field => den_a ! state  atm: air density
          if (k==12) field => dhdx  ! state  ocn: surface slope, dh/dx
          if (k==13) field => dhdy  ! state  ocn: surface slope, dh/dy
          if (k==14) field => q     ! heat  flux: ice-formed/melt-pot
          if (k==15) field => snidr ! heat  flux: swdn, near-ir, dir
          if (k==16) field => svsdr ! heat  flux: swdn, visible, dir
          if (k==17) field => snidf ! heat  flux: swdn, near-ir, dif
          if (k==18) field => svsdf ! heat  flux: swdn, visible, dif
          if (k==19) field => lwdn  ! heat  flux: lw down
          if (k==20) field => rain  ! water flux: precip, liquid
          if (k==21) field => snow  ! water flux: precip, frozen
          if (k >21) call shr_sys_abort("(msg.recv) dice: invalid field index")

          do j=1,ny
          do i=1,nx
            n=(j-1)*nx+i                 ! uncompressed grid index 
            if ( ns<n .and. n<nn) then
              field(i,j) = 0.0           ! zero region (equatorial region)
            else
                         m = n           ! compressed grid index (S hemisphere)
              if (n> ns) m = ns + n-nn+1 ! compressed grid index (N hemisphere)
              field(i,j) = buffr( m + (k-1)*nc )
            end if
          end do
          end do
        end do

        irtc_u = shr_sys_irtc()   ! unpacked msg

        ierr      = ibuffr( 1) ! cpl error code (0 => no error)
        stop_eod  = ibuffr( 2) ! T => stop at end-of-day
        stop_now  = ibuffr( 3) ! T => stop now
        !           ibuffr( 4) ! current date
        !           ibuffr( 5) ! elapsed sec on current date
        !           ibuffr(11) ! T => turn on msg-passing timing
        !           ibuffr(12) ! cpl debug level
        rest_eod  = ibuffr(21) ! T => create restart data at end-of-day
        rest_now  = ibuffr(22) ! T => create restart data now
        hist_eod  = ibuffr(23) ! T => create history data at end-of-day
        hist_now  = ibuffr(24) ! T => create history data now
        hist_tav  = ibuffr(25) ! T => create create monthly avg data
        diag_eod  = ibuffr(26) ! T => create diagnotic data at eod
        diag_now  = ibuffr(27) ! T => create diagnotic data now
        !           ibuffr(32) ! albedo calculation time shift

        call shr_timer_stop(timer_m)
        if (info_time.ne.0 .and. stop_now.eq.0) then
           write(6,9099) irtc_w,'c->i waiting'
           write(6,9099) irtc_r,'c->i received'
           write(6,9099) irtc_u,'c->i unpacked'
        end if
      ELSE IF (cOption .eq. 'disconnect') THEN 
        !-----------------------------------------------------------------------
        ! disconnect from msg passing
        !-----------------------------------------------------------------------
        write(6,9000) '(disconnect) disconnect from cpl'
        call shr_msg_finalize

      ELSE 
        !-----------------------------------------------------------------------
        ! this should never happen
        !-----------------------------------------------------------------------
        write(6,9000) 'ERROR: invalid cOption: ',cOption
        stop 
      END IF

      RETURN
 9000 format('(msg_pass) ',2a)
 9099 format('[mp timing]  irtc = ',i20,' ',a)
      END

!=======================================================================
