!===============================================================================
! CVS $Id: restart.F90,v 1.5 2002/01/23 22:03:38 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ice/dice5/restart.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE restart (cOption)

      use shr_sys_mod   ! shared code: system calls
      use shr_file_mod  ! shared code: file archive
      use calendar_mod  ! calendar routines
      use const_mod     ! physical constants
      use kind_mod      ! defines kinds

      implicit none

      !----- global -----
#include <dims.h>
#include <data_msg.h>
#include <control.h>

      character*(*),intent(in) :: cOption     ! flags read or write

      !--- local ---
      character(80)    :: str                 ! restart file ID string
      character(8 )    :: dstr                ! date string
      character(10)    :: tstr                ! time string
      character(CLONG) :: loc_fn              ! local file name (current)
      character(CLONG) :: loc_fn_old = 'null' ! local file name (previous)
      character(CLONG) :: rem_fn              ! remote/archive file name
      integer          :: i,j                 ! character string index
      integer          :: rcode               ! routine error code
      integer          :: LstIndex            ! last index function

      SAVE

!-------------------------------------------------------------------------------
! PURPOSE:
!   IF (cOption='read')
!     initialize the model state
!     1) set initial values
!     2) replace initial values with those from a restart file,
!        if a restart file exists
!   IF (cOption='write')
!     create new ice restart data & pointer files and send them to a 
!     file archive location
!
! OPTIONS:
!   o for selecting IC data:
!     IF (rest_type = 'continue' )
!        IC's are in the data file pointed to by the pointer file
!     IF (rest_type = 'initial' )
!        IC's are fabricated within the ice model code
!     IF (rest_type = 'branch' )
!        IC's are in the file specified by "rest_bfile"
!
!   o for selecting initial model date & time-step:
!     IF (rest_type = 'continue' )
!        initial date is specified by the IC data file
!     ELSE
!        initial date is specified by input parameter "rest_date"
!
! NOTES:
!   o on restart, the coupler does not use the first set of fields sent to it
!     by this model, rather it uses data from it's own restart file, this,
!     in conjuction with a subsequent receive from the coupler and a call to
!     routine advance, means that only a small number of fields need to be on 
!     the restart file.
!-------------------------------------------------------------------------------

      IF (cOption .eq. 'read') THEN
         !--------------------------------------------------------------
         ! first: fabricate all IC data
         !--------------------------------------------------------------
         write(6,9000) 'fabricating IC data, may be replaced by file data'

         T_o   = Tfrz       ! state  ocn: temperature
         S_o   = 36.0       ! state  ocn: salinity
         u_o   =  0.0       ! state  ocn: velocity, zonal
         v_o   =  0.0       ! state  ocn: velocity, meridional
         z_a   = 10.0       ! state  atm: bottom layer height
         u_a   =  5.0       ! state  atm: velocity, zonal
         v_a   =  5.0       ! state  atm: velocity, meridional
         pt_a  = 260.0      ! state  atm: pot temperature
         t_a   = 260.0      ! state  atm: temperature
         sh_a  = 0.0014     ! state  atm: specific humidity
         den_a = 1.3        ! state  atm: air density

         dhdx  = 0.0        ! state  ocn: surface slope, dh/dx
         dhdy  = 0.0        ! state  ocn: surface slope, dh/dy
         q     = 0.0        ! heat  flux: ice-formed/melt-potential
         snidr = 0.0        ! heat  flux: swdn, near-ir, dir
         svsdr = 0.0        ! heat  flux: swdn, visible, dir
         snidf = 0.0        ! heat  flux: swdn, near-ir, dif
         svsdf = 0.0        ! heat  flux: swdn, visible, dif
         lwdn  = 140.0      ! heat  flux: lw dn
         rain  = 0.0        ! water flux: precip, liquid
         snow  = 0.0        ! water flux: precip, frozen

         water = 0.0        ! initial accumulated water

         ifrac = 0.0        ! state  ice: frac ice coverage
         T     = spval      ! state  ice: temperature
         avsdr = 1.0        ! state  ice: albedo, visible/dir
         anidr = 1.0        ! state  ice: albedo, near-ir/dir
         avsdf = 1.0        ! state  ice: albedo, visible/dif
         anidf = 1.0        ! state  ice: albedo, near-ir/dif

         tauxa = spval      ! stress    : a/i zonal
         tauya = spval      ! stress    : a/i meridianal
         lat   = spval      ! heat  flux: latent
         sen   = spval      ! heat  flux: sensible
         lwup  = spval      ! heat  flux: long wave up
         evap  = spval      ! water flux: evaporation
         tref  = spval      ! diagnostic: 2m ref temperature
         pensw = spval      ! heat  flux: solar thru ice to ocn
         melth = spval      ! heat  flux: from melting
         meltw = spval      ! water flux: from melting
         sflx  = spval      ! salt  flux: from ???
         tauxo = spval      ! stress    : i/o zonal
         tauyo = spval      ! stress    : i/o meridianal
         swnet = spval      ! heat flux: short wave net absorbed

         !--------------------------------------------------------------
         ! second: read data off the restart file, if one exists
         !--------------------------------------------------------------

         !--- determine what restart file to read, if any ---
         IF ( rest_freq(1:5) == 'never') THEN
            write(6,9000) '*not* reading a restart data file'
            rem_fn='<null>'
         ELSE IF (rest_type(1:4) == 'init' ) THEN
            write(6,9000) '*not* reading a restart data file'
            rem_fn='<null>'
         ELSE IF (rest_type(1:4) == 'cont') THEN
            write(6,9000) 'IC data specified by ',trim(rest_pfile)
            call shr_file_get(rcode, 'rpointer', trim(rest_pfile))
            open (tmpunit,file='rpointer',form='formatted')
            read (tmpunit,'(a)') rem_fn
            close(tmpunit)
         ELSE IF (rest_type(1:4) == 'bran') THEN
            write(6,9000) 'IC data specified by input parameter'
            rem_fn=rest_bfile
         ELSE
            write(6,9000) 'invalid rest_type = ',rest_type
            STOP 'rest_type'
         END IF

         !--- read a restart file or fabricate data ---
         IF  ( rem_fn /= '<null>' ) THEN

            !--- get the file ---
            write(6,9000) 'IC data file: ',trim(rem_fn)
            i      = LstIndex(rem_fn,'/')+1
            j      = len_trim(rem_fn)
            loc_fn = rem_fn(i:j)//' '
            call shr_file_get(rcode, trim(loc_fn), trim(rem_fn))

            !--- read the file ---
            open (tmpunit,file=loc_fn,form='unformatted')
            read (tmpunit) str 
            read (tmpunit) cdate

            read (tmpunit) water ! accumulated water

            read (tmpunit) ifrac ! state  ice: frac ice coverage
          ! read (tmpunit) T     ! state  ice: temperature
          ! read (tmpunit) avsdr ! state  ice: albedo, visible/dir
          ! read (tmpunit) anidr ! state  ice: albedo, near-ir/dir
          ! read (tmpunit) avsdf ! state  ice: albedo, visible/dif
          ! read (tmpunit) anidf ! state  ice: albedo, near-ir/dif

          ! read (tmpunit) tauxa ! stress    : a/i zonal
          ! read (tmpunit) tauya ! stress    : a/i meridianal
          ! read (tmpunit) lat   ! heat  flux: latent
          ! read (tmpunit) sen   ! heat  flux: sensible
          ! read (tmpunit) lwup  ! heat  flux: long wave up
          ! read (tmpunit) evap  ! water flux: evaporation
          ! read (tmpunit) tref  ! diagnostic: 2m ref temperature
          ! read (tmpunit) pensw ! heat  flux: solar thru ice to ocn
          ! read (tmpunit) melth ! heat  flux: from melting
          ! read (tmpunit) meltw ! water flux: from melting
          ! read (tmpunit) sflx  ! salt  flux: from ???
          ! read (tmpunit) tauxo ! stress    : i/o zonal
          ! read (tmpunit) tauyo ! stress    : i/o meridianal
          ! read (tmpunit) swnet ! heat flux: short wave net absorbed
            close(tmpunit)
            
            !--- set the starting date ---
            write(6,9000) 'IC file ID string : ',str
            write(6,9001) 'IC file date      : ',cdate
            if (rest_type(1:4).eq.'bran' .and. rest_date.gt.1) then 
               cdate=rest_date
            end if
            call date2ymd (cdate,year,month,day)
            call date2eday(cdate,eday)
            sec   =0
            eday0 =eday

         END IF
         write(6,9001) 'using IC date       : ',cdate

      ELSE IF (cOption .eq. 'write') THEN
         !--------------------------------------------------------------
         ! write a new restart file?
         !--------------------------------------------------------------
         IF (rest_now.ne.0) THEN

            !--- create the file name ---
            !oc_fn='123456789+123456 ''
            loc_fn='yyyy-mm-dd-00000'
            write(loc_fn( 1: 4),'(i4.4)') year
            write(loc_fn( 6: 7),'(i2.2)') month
            write(loc_fn( 9:10),'(i2.2)') day
            write(loc_fn(12:16),'(i5.5)') sec
	    loc_fn = trim(case_name) // '.dice5.r.' // trim(loc_fn) // ' '

            !--- create a time stamp ---
            call date_and_time(dstr,tstr)
            write(str,9004) dstr(1:4),dstr(5:6),dstr(7:8) & 
            &              ,tstr(1:2),tstr(3:4),tstr(5:6)
            str = trim(str)//' '//trim(case_desc)
            write(6 ,9000) 'create restart file : ',trim(loc_fn)
            write(6 ,9000) 'restart file ID str : ',trim(str)
            write(6 ,9001) 'restart file date   : ',cdate

            !--- create the local file ---
            open (tmpunit,file=loc_fn,form='unformatted')
            write(tmpunit) str 
            write(tmpunit) cdate

            write(tmpunit) water ! accumulated water

            write(tmpunit) ifrac ! state  ice: frac ice coverage
          ! write(tmpunit) T     ! state  ice: temperature
          ! write(tmpunit) avsdr ! state  ice: albedo, visible/dir
          ! write(tmpunit) anidr ! state  ice: albedo, near-ir/dir
          ! write(tmpunit) avsdf ! state  ice: albedo, visible/dif
          ! write(tmpunit) anidf ! state  ice: albedo, near-ir/dif

          ! write(tmpunit) tauxa ! stress    : a/i zonal
          ! write(tmpunit) tauya ! stress    : a/i meridianal
          ! write(tmpunit) lat   ! heat  flux: latent
          ! write(tmpunit) sen   ! heat  flux: sensible
          ! write(tmpunit) lwup  ! heat  flux: long wave up
          ! write(tmpunit) evap  ! water flux: evaporation
          ! write(tmpunit) tref  ! diagnostic: 2m ref temperature
          ! write(tmpunit) pensw ! heat  flux: solar thru ice to ocn
          ! write(tmpunit) melth ! heat  flux: from melting
          ! write(tmpunit) meltw ! water flux: from melting
          ! write(tmpunit) sflx  ! salt  flux: from ???
          ! write(tmpunit) tauxo ! stress    : i/o zonal
          ! write(tmpunit) tauyo ! stress    : i/o meridianal
          ! write(tmpunit) swnet ! heat flux: short wave net absorbed
            close(tmpunit)

            !--- archive the restart file ---
            rem_fn = trim(mss_dir) // 'rest/' // loc_fn // ' '
            call shr_file_put( rcode, loc_fn, rem_fn, mss_pass, mss_rtpd )
            write(6,9002) rcode,'archive',trim(loc_fn),trim(rem_fn)

            !--- update the pointer file ---
            IF (rcode.eq.0) THEN
              open (tmpunit,file='rpointer',form='formatted')
              write(tmpunit,'(2a)') trim(rem_fn)
              write(tmpunit,'(2a)')'restart pointer for case ',trim(case_name)
              write(tmpunit,'(2a)') trim(str)
              close(tmpunit)
              call shr_file_put(rcode,'rpointer',rest_pfile,mss_pass,mss_rtpd)
              write(6,9002) rcode,'archive ','rpointer',trim(rest_pfile)
            END IF

            !--- remove older history file ---
            if (mss_rmlf /= 0 .and. loc_fn_old(1:4) /= "null" ) then
              call shr_sys_system('rm '//loc_fn_old,rcode)
            end if
            loc_fn_old = loc_fn

         END IF

      ELSE
         !--------------------------------------------------------------
         ! this should never happen
         !--------------------------------------------------------------
         write(6,9000) 'ERROR: invalid cOption = ',cOption
      END IF

      RETURN
 9000 format('(restart) ',4a )
 9001 format('(restart) ',a,i8.8)
 9002 format('(restart) return code=',i4,' for  ',a,' ',a,' ',a)
 9004 format('(',a4,'-',a2,'-',a2,1x,a2,':',a2,':',a2,')')
 9005 format(4a)
      END

!===============================================================================
