!===============================================================================
! CVS $Id: advance.F90,v 1.8 2001/08/01 22:48:17 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/lnd/dlnd5/advance.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE advance()

      use calendar_mod  ! calendar routines
      use shr_timer_mod ! shared code: timing routines

      implicit none

      !----- global -----
#include <dims.h>                
#include <data_raw.h>            
#include <data_msg.h>            
#include <control.h>             

      !----- local  ------
      integer   i,j,k        ! generic loop indicies
      integer   k1,k2        ! raw data time-slice indicies
      real      eday1        ! elapsed days for 1st data sample
      real      eday2        ! elapsed days for 2nd data sample
      real      edaym        ! elapsed days for model date
      integer   edayn        ! elapsed days for model date
      real      f1,f2        ! raw data fraction
      integer   timer_n      ! timer number
      logical   first_call   ! flags 1st envocation of this routine

      data      first_call  /.true./

      save

!-------------------------------------------------------------------------------
! PURPOSE: 
!   advance the model state to the current date
!   
! METHOD:
!  o get raw data time slices that bracket current model date & time 
!  o time interpolate raw data into model data fields to make model 
!    output data that is valid for the current model date/time
!-------------------------------------------------------------------------------

      if (first_call) call shr_timer_get(timer_n,'advance')
      call shr_timer_start(timer_n)

      !-----------------------------------------------------------------
      ! get raw data time slices that bracket current model date & time 
      !-----------------------------------------------------------------
      call data_in()

      !-----------------------------------------------------------------
      ! determine time interpolation factors
      !-----------------------------------------------------------------
      k1 = r_kold  ! k index of older time slice (must be 1 or 2)
      k2 = 3 - k1  ! k index of newer time slice
      if (info_dbug.gt.2) write(6,9001) 'advancing to cdate = ',cdate
      call date2eday(mod(r_cdate(k1),10000) ,edayn) ! ignore year
      eday1 = float(edayn) + r_sec(k1)/86400.0
      call date2eday(mod(r_cdate(k2),10000) ,edayn) ! ignore year
      eday2 = float(edayn) + r_sec(k2)/86400.0
      call date2eday(mod(cdate,10000)       ,edayn) ! ignore year
      edaym = float(edayn) + sec/86400.0

      !--- want eday1 < edaym < eday2 ---
      if (eday1 .gt. edaym) eday1 = eday1 - 365.
      if (eday2 .lt. edaym) eday2 = eday2 + 365.
      f1 = (eday2-edaym)/(eday2-eday1)
      f2 = 1.0 - f1

      !-----------------------------------------------------------------
      ! time interpolate raw data into model data fields
      !-----------------------------------------------------------------
      if (first_call) then
         swnet(:,:) = -999.0
      else
         DO j=1,ny
         DO i=1,nx 
	    !--- swnet is sent to cpl for diagnostic purposes only ---
	    !--- note: compute swnet before updating albedos because ---
	    !--- newly recv'd swdn goes with previously sent albedo ---
            swnet(i,j) = (1.0 - alvdr(i,j))*swvdr(i,j) &
            &          + (1.0 - alndr(i,j))*swndr(i,j) &
            &          + (1.0 - alvdf(i,j))*swvdf(i,j) &
            &          + (1.0 - alndf(i,j))*swndf(i,j)  
         END DO
         END DO
      endif
      DO j=1,ny
      DO i=1,nx
         t    (i,j) = f1*r_t    (i,j,k1) + f2*r_t    (i,j,k2)
         alvdr(i,j) = f1*r_alvdr(i,j,k1) + f2*r_alvdr(i,j,k2)
         alndr(i,j) = f1*r_alndr(i,j,k1) + f2*r_alndr(i,j,k2)
         alvdf(i,j) = f1*r_alvdf(i,j,k1) + f2*r_alvdf(i,j,k2)
         alndf(i,j) = f1*r_alndf(i,j,k1) + f2*r_alndf(i,j,k2)
         snowh(i,j) = f1*r_snowh(i,j,k1) + f2*r_snowh(i,j,k2)
         taux (i,j) = f1*r_taux (i,j,k1) + f2*r_taux (i,j,k2)
         tauy (i,j) = f1*r_tauy (i,j,k1) + f2*r_tauy (i,j,k2)
         lat  (i,j) = f1*r_lat  (i,j,k1) + f2*r_lat  (i,j,k2)
         sen  (i,j) = f1*r_sen  (i,j,k1) + f2*r_sen  (i,j,k2)
         lwup (i,j) = f1*r_lwup (i,j,k1) + f2*r_lwup (i,j,k2)
         evap (i,j) = f1*r_evap (i,j,k1) + f2*r_evap (i,j,k2)
         tref (i,j) = f1*r_tref (i,j,k1) + f2*r_tref (i,j,k2)
      END DO
      END DO

      DO j=1,ny_r
      DO i=1,nx_r
         roff_r(i,j) = f1*r_roff_r(i,j,k1) + f2*r_roff_r(i,j,k2)
      END DO
      END DO

      !--- DEBUG test pattern for 1/2 degree data ---
!!!   if (.false.) then
!!!      roff_r(:,:)       = 2.0e-5  ! background
!!!      roff_r(:,170:190) = 2.0e-4  ! 0 east
!!!      roff_r(350:370,:) = 1.0e-4  ! 0 north
!!!      roff_rc(:)        = 1.0e36
!!!   endif

      !--- put uncompressed runoff data into compressed vector ---
      do k=1,n_rc
        roff_rc(k) = roff_r(i_rc(k),j_rc(k))
      end do

      !--- debug info wrt time interpolation ---
      if ( (info_dbug.ge.2 .and. sec.eq.0) .or. info_dbug.ge.3) then
        write(6,9008) 't1: ',r_t(5,5,k1),r_cdate(k1)+r_sec(k1)/86400.,f1
        write(6,9008) 't : ',  t(5,5)   ,  cdate    +  sec    /86400.
        write(6,9008) 't2: ',r_t(5,5,k2),r_cdate(k2)+r_sec(k2)/86400.,f2
      end if

      if (info_sleep > 0.0) call shr_timer_sleep(info_sleep)
      call shr_timer_stop(timer_n)
      first_call = .false.

      RETURN
 9000 format('(advance) ',a,a)
 9001 format('(advance) ',a,i8.8)
 9008 format('(advance) ',a,f8.3,1x,f12.2,f6.3)
      END

!===============================================================================
