!===============================================================================
! CVS $Id: data_in.F90,v 1.2 2001/07/24 22:13:11 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/lnd/dlnd5/data_in.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE data_in()

      use calendar_mod ! calendar routines

      implicit none

      !----- global -----
#include <dims.h>            
#include <data_raw.h>        
#include <control.h>         

      !----- local  ------
      integer      i,j,k     ! generic indicies
      integer      year0     ! 1st year in raw data sequence
      integer      nyear     ! number of years in sequence
      integer      oyear     ! model year matched to raw data year0

      integer      dyear     ! data  year  of input data
      integer      dmonth    ! data  month of input data

      integer      r_knew    ! k-index for newer data slice

      integer      ryear     ! model year  assigned to input data
      integer      rmonth    ! model month assigned to input data
      integer      rday      ! model day   assigned to input data

      logical      firstcall ! flags one-time startup operations
 
      data         firstcall /.true./

      save

!-------------------------------------------------------------------------------
! PURPOSE:
!   update raw data fields
! ASSUMPTIONS:
!   o input data is in the form of monthly mean fields
!   o input data is assigned to the 15th day (+ 0sec) of any given month
!-------------------------------------------------------------------------------

      if (data_dir(1:4) .eq. 'null') then
        !-----------------------------------------------------------------------
        ! make bogus data that brackets model date & time 
        !-----------------------------------------------------------------------
        r_kold     = 1
        r_cdate(1) = cdate
        r_sec  (1) = sec
        r_cdate(2) = cdate
        r_sec  (2) = sec + 1
        if (firstcall) then
          write(6,9000) 'null data dir => fields constant in (x,y,t)'
          DO k=1,2
            DO j=1,ny
            DO i=1,nx
              r_t    (i,j,k) =  300.0 ! temperature        ~ Kelvin
              r_alvdr(i,j,k) =     .5 ! albedo: vis-dir    ~ [0,1]
              r_alndr(i,j,k) =     .5 ! albedo: nir-dir    ~ [0,1]
              r_alvdf(i,j,k) =     .5 ! albedo: vis-diff   ~ [0,1]
              r_alndf(i,j,k) =     .5 ! albedo: nir-diff   ~ [0,1]
              r_snowh(i,j,k) =    1.0 ! snow height        ~ m
              r_taux (i,j,k) =   -0.5 ! wind stress, zonal ~ N/m^2
              r_tauy (i,j,k) =    0.0 ! wind stress, merid ~ N/m^2
              r_lat  (i,j,k) =  -10.0 ! latent             ~ W/m^2
              r_sen  (i,j,k) = -100.0 ! sensible           ~ W/m^2
              r_lwup (i,j,k) = -400.0 ! longwave, upward   ~ W/m^2
              r_evap (i,j,k) = -1.e-6 ! evaporation        ~ kg/s/m^2
              r_tref (i,j,k) =  300.0 ! 2m air temperature ~ Kelvin
            END DO
            END DO
            DO i=1,100
              r_roff_r = 0.0          !basin runoff       ~ kg/s/basin
            END DO
          END DO
        end if
      else
        !-----------------------------------------------------------------------
        ! get real data that brackets model date & time 
        !-----------------------------------------------------------------------

        if (firstcall) then
          !------------------------------------------
          ! document the data cycle
          !------------------------------------------
          year0 = data_year0 ! 1st year in data sequence
          nyear = data_nyear ! # of years in data sequence
          oyear = data_oyear ! coupler year corresponding to year0
          dyear = year0 + modulo(year-oyear,nyear) ! current data year

          write(6,9001) 'first year in data sequence = '  ,year0
          write(6,9001) 'last  year in data sequence = '  ,year0+nyear-1
          write(6,9001) 'model year ',oyear,' => data year ',year0
          write(6,9001) 'model year ', year,' => data year ',dyear

          !-----------------------------------------------------
          ! set new/old dates to get two data slices on startup
          !-----------------------------------------------------
          r_kold = 1
          r_cdate(r_kold) = -999 
          r_sec(r_kold)   = 0
         
          !-----------------------------------------------
          ! compute date of data that brackets from below
          !-----------------------------------------------
          r_knew = 3-r_kold ! put data in this k-index

          ryear  = year  ! we want data corresponding to 
          rmonth = month ! this year, month, & day
          rday   = 15    

          if (day .lt. 15) then ! previous month
            rmonth = rmonth - 1 
            if (rmonth .lt. 1) then ! previous year
              rmonth = 12
              ryear  = ryear - 1
            end if
          end if
          if (ryear .lt. 0) then
            !--- can't handle negative years ---
            ryear  = 0
            rmonth = 1
            rday   = 1    
          end if
          call ymd2date(ryear,rmonth,rday,r_cdate(r_knew))
          r_sec(r_knew) = 0

          !----------------------------------------------
          ! get the data that brakets from below
          !----------------------------------------------
          dyear  = year0 + modulo(ryear-oyear,nyear)
          dmonth = rmonth
          call data_in_nc(dyear,dmonth,r_knew)
          write(6,9002) 'model date assigned to data = ',r_cdate(r_knew)

          ! initialize with 0 to prevent from being used unset
          r_roff_r(:,:,:) = 0.0

        endif

        r_knew = 3-r_kold ! k index of newer data

        if ( cdate.ge.r_cdate(r_knew) ) then
          !-----------------------------------------------
          ! advance raw data time slices
          !-----------------------------------------------
          r_kold = r_knew    ! toggle new/old k-index
          r_knew = 3-r_kold  ! toggle new/old k-index

          !-----------------------------------------------
          ! compute date of data that brackets from above
          !-----------------------------------------------
          ryear  = year  ! we want data corresponding to 
          rmonth = month ! this year, month, & day
          rday   = 15    

          if (day .ge. 15) then ! next month
            rmonth = rmonth + 1 
            if (rmonth .gt. 12) then ! next year
              rmonth = 1
              ryear  = ryear + 1
            end if
          end if
          call ymd2date(ryear,rmonth,rday,r_cdate(r_knew))
          r_sec(r_knew) = 0

          !----------------------------------------------
          ! get the data that brakets from above
          !----------------------------------------------
          dyear  = year0 + modulo(ryear-oyear,nyear)
          dmonth = rmonth
          k = 3 - r_kold ! k-index to be replaced
          call data_in_nc(dyear,dmonth,r_knew)
          write(6,9002) 'model date assigned to data = ',r_cdate(r_knew)

        else
          if (info_dbug.ge.3) write(6,9000) 'don''t need new data'
        endif

      endif

      firstcall = .false.

      RETURN
 9000 format('(data_in) ',a,a)
 9001 format('(data_in) ',a,i4,a,i4)
 9002 format('(data_in) ',a,i8.8)
      END
 
!===============================================================================
