!===============================================================================
! CVS $Id: data_in_nc.F90,v 1.5 2001/07/24 22:13:11 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/lnd/dlnd5/data_in_nc.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      subroutine data_in_nc(dyear,dmonth,k)

      use shr_sys_mod  ! shared code: system calls
      use shr_file_mod ! shared code: file archive
      use const_mod    ! physical constants

      implicit none

      !----- global ------
#include <dims.h>               
#include <data_raw.h>           
#include <control.h>            
#include <netcdf.inc>           

      !----- arguments ------
      integer       dyear       ! (in) data year to read
      integer       dmonth      ! (in) data month to read
      integer       k           ! (in) raw data k index to be replaced

      !----- local ------
      integer       i,j,n       ! generic indicies
      character*800 str         ! data file ID string
      character*120 mss_fn      ! MSS file name
      character*80  loc_fn      ! local file name
      integer       fid         ! nc file ID
      integer       vid         ! nc variable ID
      integer       did         ! nc dimension ID
      integer       beg(3)      ! nc hyperslab begin vector
      integer       cnt(3)      ! nc hyperslab count vector
      integer       fdate       ! time coordinate (date) from data file
      integer       rcode       ! return code
      integer       last_dyear  ! year of last nc data read in
      logical       first_call  ! flags 1st envocation of this routine

      data          last_dyear  / -999 /
      data          first_call  /.true./

      save    

!-------------------------------------------------------------------------------
! PURPOSE:
!   acquire a data file & read data corresponding to given year & month
!
! ASSUMPTIONS
!  o new data overwrites data in location data(*,*,k)
!  o each file has 12 monthly mean fields
!  o files have LSM-specific file name: "lsmh_yyyy.nc"
!  o assuming 2m reference T data is missing from data file
!-------------------------------------------------------------------------------

      if (dyear .ne. last_dyear) then
        !---------------------------------------------------------------
        ! acquire & open a new data file
        !---------------------------------------------------------------

        !----- close & remove an old file -----
        if (.not. first_call) then
           rcode = nf_close(fid)
           if (data_rmlf.ne.0) then
             call shr_sys_system('rm '//trim(loc_fn),rcode)
             write(6,9000) 'rm ',trim(loc_fn)
           end if
        end if
        last_dyear = dyear

        write(6,9001) 'acquire file for data year = ',dyear

        !----- msread & open a new file -----
        loc_fn = "lsmh_yyyy.nc "
        write(loc_fn(6:9),'(i4.4)') dyear
        j=index(loc_fn,' ')-1

        i = index(data_dir,' ')-1
        if (data_dir(i:i) .eq. '/') i = i-1
        mss_fn = data_dir(1:i) // '/' // trim(loc_fn)
        i=len_trim(mss_fn)
 
        call shr_file_get (rcode,loc_fn,mss_fn)
        write(6,9005) rcode,'acquire',loc_fn(1:j),mss_fn(1:i)

        write(6,9000) 'open netCDF file ',loc_fn
        rcode = nf_open(loc_fn,NF_NOWRITE,fid)
        if (rcode.ne.NF_NOERR) write(6,9000) nf_strerror(rcode)

        !----- print title info from nc file -----
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'case_title', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'case_title'  , str)
        else
          str = '<too long to print>   '
          n = index(str,'   ')
        end if
        write(6,9000) 'o title   = ',str(1:n)
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'history', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'history', str)
        else
          str = '<too long to print>   '
          n = index(str,'   ')
        end if
        write(6,9000) 'o history = ',str(1:n)
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'Source', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'Source' , str)
        else
          str = '<too long to print>   '
          n = index(str,'   ')
        end if
        write(6,9000) 'o source  = ',str(1:n)

      endif
      
      !-----------------------------------------------------------------
      ! read data from an open file
      !-----------------------------------------------------------------
      write(6,9002) 'desired data date = ',dyear*10000+dmonth*100+15

      beg(1) = dmonth
      cnt(1) = 1
      rcode = nf_inq_varid(fid,'date',vid)
      rcode = nf_get_vara_int(fid,vid,beg,cnt,fdate)
      write(6,9002) 'file''s  data date = ',fdate

      beg(1) = 1
      beg(2) = 1
      beg(3) = dmonth
      cnt(1) = nx
      cnt(2) = ny
      cnt(3) = 1

      rcode = nf_inq_varid(fid,'TG'    ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_t    (1,1,k))
      rcode = nf_inq_varid(fid,'ASDIR' ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_alvdr(1,1,k))
      rcode = nf_inq_varid(fid,'ALDIR' ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_alndr(1,1,k))
      rcode = nf_inq_varid(fid,'ASDIF' ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_alvdf(1,1,k))
      rcode = nf_inq_varid(fid,'ALDIF' ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_alndf(1,1,k))
      rcode = nf_inq_varid(fid,'H2OSNO',vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_snowh(1,1,k))
      rcode = nf_inq_varid(fid,'TAUX'  ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_taux (1,1,k))
      rcode = nf_inq_varid(fid,'TAUY'  ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_tauy (1,1,k))
      rcode = nf_inq_varid(fid,'FLH'   ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_lat  (1,1,k))
      rcode = nf_inq_varid(fid,'FSH'   ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_sen  (1,1,k))
      rcode = nf_inq_varid(fid,'FIRE'  ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_lwup (1,1,k))
      rcode = nf_inq_varid(fid,'EVAP'  ,vid)
      rcode = nf_get_vara_double(fid,vid,beg,cnt,r_evap (1,1,k))
      !!!!! = nf_inq_varid(fid,'TREF'  ,vid)
      !!!!! = nf_get_vara_double(fid,vid,beg,cnt,r_tref (1,1,k))

      !--- replace runoff data with auxiliary data, or with zero? ---
      if (data_aroff(1:4) == 'null') then
	 !--- default scheme: 19 basins from data file ---
         rcode = nf_inq_dimid(fid, 'basins', did)
         rcode = nf_inq_dimlen(fid, did, n)
         !--- check for coordinated array dimensions ---
         if (n /= nx_r .or. ny_r /= 1 ) then
            write(6,9003) 'ERROR: runoff dims don''t match nc data size'
            write(6,9003) 'ERROR: nx_r, ny_r = ',nx_r, ny_r
            write(6,9003) 'ERROR: nc data    = ',n   , 1
            stop 'roff dims'
         end if
         beg(1) = 1
         beg(2) = dmonth
         cnt(1) = n
         cnt(2) = 1 
         rcode = nf_inq_varid(fid,'BASIN_RUNOFF',vid)
         rcode = nf_get_vara_double(fid,vid,beg,cnt,r_roff_r(:,1,k))
         !--- KLUDGE: convert runoff units to kg/s/m^2
         !--- KLUDGE: runoff data needs units of kg/s/m^2 & associated area
         !--- KLUDGE: netCDF data files need to change to this convention
         r_roff_r(:,1,k) = r_roff_r(:,1,k) / (4.0*PI*rEarth**2)
      else if (data_aroff(1:4) == 'zero') then
	 !--- set runoff to zero ---
         if (first_call) write(6,9000) 'NOTE: setting runoff to zero'
         r_roff_r = 0.0
      else 
	 !--- get alternate runoff data out of the specified file ---
         call runoff_aux()
      endif

      !---------------------------------------------------
      ! NOTE: assuming 2m reference T data is unavailable
      !---------------------------------------------------
      if (first_call) then
         write(6,9000) &
         & 'WARNING: assuming tref data is unavailable, using tref(i,j) = 300.0'
      endif
      r_tref(:,:,k) = 300.0

      !---------------------------------------------------
      ! NOTE: sign is wrong on some input data files (?)
      !---------------------------------------------------
      if ( index(data_fix,'sign').ne.0 ) then
        if (first_call) write(6,9000) & 
        &     'FYI: data_fix ~ sign => flip sign of evap,lwup,sen,lat data'
        do j=1,ny
        do i=1,nx
           r_evap(i,j,k) = - r_evap(i,j,k)
           r_lwup(i,j,k) = - r_lwup(i,j,k)
           r_sen (i,j,k) = - r_sen (i,j,k)
           r_lat (i,j,k) = - r_lat (i,j,k)
        enddo
        enddo
      endif

      !---------------------------------------------------
      ! NOTE: sign is wrong on some input data files
      !---------------------------------------------------
      if ( data_mask(1:3) .eq. 'cpl' ) then
        if (first_call) write(6,9000) 'FYI: data_mask = cpl => ', &
        &  'fabricate lnd data to accommodate cpl supplied domain mask'
        call data_in_nc_fill(k)
      endif

      !---------------------------------------------------
      ! debug sanity check
      !---------------------------------------------------
      if (info_dbug.ge.3) then
        write(6,9010) '<dbug> t    :',r_t    (5,5,k)
        write(6,9010) '<dbug> alvdr:',r_alvdr(5,5,k)
        write(6,9010) '<dbug> alndr:',r_alndr(5,5,k)
        write(6,9010) '<dbug> alvdf:',r_alvdf(5,5,k)
        write(6,9010) '<dbug> alndf:',r_alndf(5,5,k)
        write(6,9010) '<dbug> taux :',r_taux (5,5,k)
        write(6,9010) '<dbug> tauy :',r_tauy (5,5,k)
        write(6,9010) '<dbug> lat  :',r_lat  (5,5,k)
        write(6,9010) '<dbug> sen  :',r_sen  (5,5,k)
        write(6,9010) '<dbug> lwup :',r_lwup (5,5,k)
        write(6,9010) '<dbug> evap :',r_evap (5,5,k)
        write(6,9010) '<dbug> tref :',r_tref (5,5,k)
        write(6,9010) '<dbug> roff :',r_roff_r(5,1,k)
      endif

      first_call = .false.

      RETURN
 9000 format('(data_in_nc) ',4a)
 9001 format('(data_in_nc) ',a,i4.4)
 9002 format('(data_in_nc) ',a,i8.8)
 9003 format('(data_in_nc) ',a,2i5)
 9005 format('(data_in_nc) return code ',i3, ' on ',4(a,1x))
 9010 format('(data_in_nc) ',a,e12.3)

      END

!===============================================================================

      subroutine data_in_nc_fill(k)

      implicit none

      !----- global ------
#include <dims.h>               
#include <data_raw.h>           
#include <data_msg.h>           
#include <control.h>            

      !----- arguments ------
      integer   k              ! (in) raw data k index to be replaced

      !----- local ------
      integer   i,j            ! generic indicies
      integer   i1,j1          ! generic indicies
      integer   m,n            ! generic indicies
      integer   n0,n1,n2,n3,n4 ! generic counters
      integer   i_valid(nx,ny) ! i-index of valid data for cell (i,j)
      integer   j_valid(nx,ny) ! j-index of valid data for cell (i,j)

      logical   first_call  ! flags 1st envocation of this routine

      data      first_call  /.true./

      save    

!-------------------------------------------------------------------------------
! PURPOSE:
!   Fabricate "reasonable" data where it is needed but isn't available.
!
!   r_mask(i,j) is the domain mask associated with the "raw data" 
!               ("raw data" is the data read in from a file)
!   mask(i,j)   is the domain mask associated with data exchanged with the cpl.
!
!   if mask(i,j).ne.0  and  r_mask(i,j).eq.0 
!   then data is needed for this grid cell, yet it isn't available.  
!   This routine fills such cells with "reasonable" data (from a nearby cell)
!
! ASSUMPTIONS
!   o there is at least one cell such that r_mask(i,j) .ne. 0
!-------------------------------------------------------------------------------

      if (first_call) then
        !---------------------------------------------------------------
        ! locate valid data for cells that need it
        !---------------------------------------------------------------

        !--- how different is the raw vs the cpl data mask? ---
        n0=nx*ny
        n1=0
        n2=0
        n3=0
        n4=0
        do j=1,ny
        do i=1,nx
           if (r_mask(i,j) /= 0) n1=n1+1
           if (mask  (i,j) /= 0) n2=n2+1
           if (r_mask(i,j) /= 0 .and. mask(i,j) == 0) n3=n3+1
           if (r_mask(i,j) == 0 .and. mask(i,j) /= 0) n4=n4+1
        enddo
        enddo
        write(6,9001) 'domain mask info:'
        write(6,9001) '  number of grid cells                    :',n0
        write(6,9001) '  number of active cells in raw data  mask:',n1
        write(6,9001) '  number of active cells in cpl data  mask:',n2
        write(6,9001) '  number of active cells in raw - cpl mask:',n3
        write(6,9001) '  number of active cells in cpl - raw mask:',n4

        !--- locate valid data for cells that need it ---
        i_valid = -99999
        j_valid = -99999
        n1=0
        DO j=1,ny
        DO i=1,nx
            if ( mask(i,j) /= 0 .and. r_mask(i,j) == 0 ) then 
              n1=n1+1
              do m=0,ny-1
                j1=max(1,j-m)    ! search south
                do n=0,nx-1      ! search east & west
                   i1=max(1,i-n) 
                   if ( r_mask(i1,j1) /= 0 ) goto 999
                   i1=min(nx,i+n)
                   if ( r_mask(i1,j1) /= 0 ) goto 999
                end do
                j1=min(1,j+m)  ! search north
                do n=0,nx-1    ! search east & west
                   i1=max(1,i-n)
                   if ( r_mask(i1,j1) /= 0 ) goto 999
                   i1=min(nx,i+n)
                   if ( r_mask(i1,j1) /= 0 ) goto 999
                end do
                write(6,9000) 'ERROR: no r_mask=0 everywhere !'
                stop 'r_mask'
              end do
  999         continue
              i_valid(i,j) = i1
              j_valid(i,j) = j1
            endif
        END DO
        END DO
        write(6,9001) 'WARNING: fabricating data for',n1,' cells'
      endif

      !-----------------------------------------------------------------
      ! fill in data cells will reasonable data as necessary
      !-----------------------------------------------------------------
      DO j=1,ny
      DO i=1,nx
         if ( mask(i,j) /= 0 .and. r_mask(i,j) == 0 ) then 
            i1 = i_valid(i,j)
            j1 = j_valid(i,j)
            r_t    (i,j,k) = r_t    (i1,j1,k)
            r_alvdr(i,j,k) = r_alvdr(i1,j1,k)
            r_alndr(i,j,k) = r_alndr(i1,j1,k)
            r_alvdf(i,j,k) = r_alvdf(i1,j1,k)
            r_alndf(i,j,k) = r_alndf(i1,j1,k)
            r_snowh(i,j,k) = r_snowh(i1,j1,k)
            r_taux (i,j,k) = r_taux (i1,j1,k)
            r_tauy (i,j,k) = r_tauy (i1,j1,k)
            r_lat  (i,j,k) = r_lat  (i1,j1,k)
            r_sen  (i,j,k) = r_sen  (i1,j1,k)
            r_lwup (i,j,k) = r_lwup (i1,j1,k)
            r_evap (i,j,k) = r_evap (i1,j1,k)
            r_tref (i,j,k) = r_tref (i1,j1,k)
         endif
      END DO
      END DO

      first_call = .false.

      RETURN
 9000 format('(data_in_nc_fill) ',4a)
 9001 format('(data_in_nc_fill) ',a,i6,a,i6,a)
      END

!===============================================================================
