!===============================================================================
! CVS $Id: data_msg.h,v 1.5 2001/08/01 22:48:17 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/lnd/dlnd5/data_msg.h,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================
!  Sign convention:
!    positive value <=> downward flux
!
!  Unit convention:
!    T             ~ Kelvin
!    u,v           ~ m/s
!    pressure      ~ N/m^2 = Pa
!    heat flux     ~ W/m^2
!    momentum flux ~ N/m^2
!    water flux    ~ (kg/s)/m^2
!    coordinates   ~ degrees north or degrees east
!-------------------------------------------------------------------------------

      integer,parameter :: nibuff=100

      !-------------------------------------------------------------------------
      ! output: data sent to coupler  
      !-------------------------------------------------------------------------

      !----- control flags -----
      integer   ibuffs(nibuff) ! control flags sent to coupler

      !----- domain - lnd -----
      real    xc  (nx,ny)     ! x-grid center coordinates ~ deg east
      real    yc  (nx,ny)     ! y-grid center coordinates ~ deg north
      real    xv(4,nx,ny)     ! x-grid vertex coordinates ~ deg east
      real    yv(4,nx,ny)     ! y-grid vertex coordinates ~ deg north
      real    area(nx,ny)     ! area of grid cells        ~ radians squared
      integer mask(nx,ny)     ! mask: 0 <=> not a active grid cell

      !----- domain - runoff -----
      real    xc_r  (nx_r,ny_r) ! x-grid center coordinates ~ deg east
      real    yc_r  (nx_r,ny_r) ! y-grid center coordinates ~ deg north
      real    xv_r(4,nx_r,ny_r) ! x-grid vertex coordinates ~ deg east
      real    yv_r(4,nx_r,ny_r) ! y-grid vertex coordinates ~ deg north
      real    area_r(nx_r,ny_r) ! area of grid cells        ~ radians squared
      integer mask_r(nx_r,ny_r) ! mask: 0 <=> not a active grid cell

      integer key_r (nx_r,ny_r) ! key : 0 <=> data excluded in compressed vector
      integer i_rc (n_r)        ! uncompressed i-index given compressed index
      integer j_rc (n_r)        ! uncompressed j-index given compressed index
      integer i_r  (nx_r,ny_r)  ! compressed index given uncompressed (i,j)

      !----- states -----
      real   t    (nx,ny)      ! temperature               ~ Kelvin
      real   alvdr(nx,ny)      ! albedo: vis-dir           ~ [0,1]
      real   alndr(nx,ny)      ! albedo: nir-dir           ~ [0,1]
      real   alvdf(nx,ny)      ! albedo: vis-diff          ~ [0,1]
      real   alndf(nx,ny)      ! albedo: nir-diff          ~ [0,1]
      real   snowh(nx,ny)      ! snow height               ~ m

      !----- fluxes -----
      real   taux (nx,ny)      ! wind stress, zonal        ~    N/m^2
      real   tauy (nx,ny)      ! wind stress, meridional   ~    N/m^2
      real   lat  (nx,ny)      ! latent                    ~    W/m^2
      real   sen  (nx,ny)      ! sensible                  ~    W/m^2
      real   lwup (nx,ny)      ! longwave, upward          ~    W/m^2
      real   swnet(nx,ny)      ! short-wave net            ~    W/m^2
      real   evap (nx,ny)      ! evaporation               ~ kg/s/m^2

      !----- diagnostics -----
      real   tref (nx,ny)      ! 2m atm temperature        ~ Kelvin

      !----- fluxes - runoff -----
      real  roff_r (nx_r,ny_r) ! runoff, uncompressed      ~ kg/s/m^2
      real  roff_rc(n_r)       ! runoff, compressed        ~ kg/s/m^2

      !-------------------------------------------------------------------------
      ! input: data received from coupler 
      !-------------------------------------------------------------------------

      !----- control flags -----
      integer   ibuffr(nibuff) ! control flags received from coupler
      real      rbuffr(nibuff) ! control flags received from coupler

      !----- fluxes -----
      real   lwdn (nx,ny)      ! longwave, downward        ~    W/m^2
      real   precc(nx,ny)      ! precip, liquid, convective  ~ kg/s/m^2
      real   precl(nx,ny)      ! precip, liquid, large-scale ~ kg/s/m^2
      real   snowc(nx,ny)      ! precip, frozen, convective  ~ kg/s/m^2
      real   snowl(nx,ny)      ! precip, frozen, large-scale ~ kg/s/m^2
      real   swvdr(nx,ny)      ! shortwave, down, vis-dir  ~    W/m^2
      real   swndr(nx,ny)      ! shortwave, down, nir-dir  ~    W/m^2
      real   swvdf(nx,ny)      ! shortwave, down, vis-diff ~    W/m^2
      real   swndf(nx,ny)      ! shortwave, down, nir-diff ~    W/m^2

      !----- states -----
      real   z    (nx,ny)      ! atm bottom level height   ~ m
      real   u    (nx,ny)      ! atm wind, zonal           ~ m/s
      real   v    (nx,ny)      ! atm wind, meridional      ~ m/s
      real   ptem (nx,ny)      ! atm potential temperature ~ Kelvin
      real   shum (nx,ny)      ! atm specific humidity     ~ kg/kg
      real   pres (nx,ny)      ! atm pressure              ~ Pa
      real   tair (nx,ny)      ! atm temperature           ~ Kelvin
      real   fl   (nx,ny)      ! fraction of lnd           ~ [0,1]

      !-------------------------------------------------------------------------
      ! make data global
      !-------------------------------------------------------------------------
      COMMON /data_msg/                                                 &
     &    ibuffs                                                        &
     &  , xc    , yc    , xv    , yv    , area  , mask                  &
     &  , xc_r  , yc_r  , xv_r  , yv_r  , area_r, mask_r                &
     &  , key_r , i_rc  , j_rc  ,  i_r                                  &
     &  , t     , alvdr , alndr , alvdf , alndf , snowh                 &
     &  , taux  , tauy  , lat   , sen   , lwup  , swnet , evap  , tref  &
     &  , roff_r, roff_rc                                               &
     &  , ibuffr, rbuffr                                                &
     &  , lwdn  , precc , precl , snowc , snowl                         &
     &  , swvdr , swndr , swvdf , swndf                                 &
     &  , z     , u     , v     , ptem  , shum  , pres  , tair  , fl

      SAVE   /data_msg/
