!===============================================================================
! CVS $Id: domain.F90,v 1.6 2001/07/24 22:13:12 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/lnd/dlnd5/domain.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE domain()

      use shr_file_mod ! shared code: file archive
      use const_mod    ! physical constants

      implicit none

      !----- global -----
#include <dims.h>             
#include <data_raw.h>         
#include <data_msg.h>         
#include <control.h>          
#include <netcdf.inc>         

      !----- local  ------
      real          xt(nx)   ! cell center x-coords
      real          yt(ny)   ! cell center y-coords
      real          xe(nx+1) ! cell edge   x-coords
      real          ye(ny+1) ! cell edge   y-coords
      real          dth      ! d(theta) for grid cell (radians)
      real          dph      ! d(phi)   for grid cell (radians)

      !----- local  ------
      integer       i,j,n     ! generic indicies
      character*240 str       ! data file ID string
      character*120 mss_fn    ! MSS   file name
      character*80  loc_fn    ! local file name
      integer       fid       ! nc file ID
      integer       vid       ! nc variable ID
      integer       did       ! nc dimension ID
      integer       beg(3)    ! begin wrt nc hyperslab indicies
      integer       cnt(3)    ! count wrt nc hyperslab indicies
      integer       rcode     ! return code
      real          tg(nx,ny) ! lnd surface T (used to derive mask)
      
!-------------------------------------------------------------------------------
! PURPOSE:
!   get and/or set the model domain info
!
! ASSUMPTIONS:
!   o the domain data is lacking in that there is no explicit domain 
!     mask, a mask is infered by assuming that 
!     T(i,j) < 1000   <=>   cell (i,j) is in the lnd model domain
!   o the runoff domain data is read in from a file, it is assumed that the
!     model's runoff array sizes match the sizes found in the data file
!-------------------------------------------------------------------------------

        !-----------------------------------------------------------------------
        ! read (time-invariant) domain data from a data file
        !-----------------------------------------------------------------------
        write(6,9000) 'acquire domain data file from MSS'

        loc_fn = "lsmh_yyyy.nc "
        write(loc_fn(6:9),'(i4.4)') data_year0
        i = len_trim(data_dir)
        j = len_trim(loc_fn  )
        if (data_dir(i:i) .eq. '/') i = i-1
        mss_fn = data_dir(1:i) // '/' // loc_fn(1:j) // ' '

        call shr_file_get (rcode,loc_fn,mss_fn)
        write(6,9010) rcode,'acquire',trim(loc_fn),trim(mss_fn)

        write(6,9000) 'open netCDF file ',trim(loc_fn)
        rcode = nf_open(loc_fn,NF_NOWRITE,fid)
        if (rcode.ne.NF_NOERR) write(6,9000) nf_strerror(rcode)

        !--- print title info from nc file ---
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'case_title', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'case_title'  , str)
        else
          str = '<too long to print>   '
          n = index(str,'   ')
        end if
        write(6,9000) 'o title   = ',str(1:n)
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'history', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'history', str)
        else
          str = '<too long to print>   '
          n = index(str,'   ')
        end if
        write(6,9000) 'o history = ',str(1:n)
        rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'Source', n)
        if (n.le.240) then
          rcode = nf_get_att_text(fid, NF_GLOBAL, 'Source' , str)
        else
          str = '<too long to print>   '
          n = index(str,'   ')
        end if
        write(6,9000) 'o source  = ',str(1:n)

        rcode = nf_inq_varid(fid,'longitude',vid)
        rcode = nf_get_var_double(fid,vid,xt)
        rcode = nf_inq_varid(fid,'latitude' ,vid)
        rcode = nf_get_var_double(fid,vid,yt)
        beg(1)=1
        beg(2)=1
        beg(3)=1
        cnt(1)=nx
        cnt(2)=ny
        cnt(3)=1
        rcode = nf_inq_varid(fid,'TG',vid)  ! ground T
        rcode = nf_get_vara_double(fid,vid,beg,cnt,tg)

        !--- construct (infer) domain mask from surface T data ---
        !--- KLUDGE: data file should provide this mask        ---
        r_mask = 0
        do j=1,ny
        do i=1,nx
           if (tg(i,j) < 1000.0) r_mask(i,j)=1  !? spval=1.0e30 ??
        enddo
        enddo
        mask = r_mask ! NOTE: desired mask may differ from raw data mask
        n=0
        do j=1,ny
        do i=1,nx
           if (mask(i,j).ne.0) n=n+1
        enddo
        enddo
        write(6,9003) 'domain data:'
        write(6,9003) 'o mask:'
        write(6,9003) '  number of        grid cells = ',nx*ny
        write(6,9003) '  number of active grid cells = ',n

        !--- construct edge grids from center grids ---
        write(6,9001) 'o FYI: edge grids not available in domain file'
        xe(   1) = xt( 1) - (xt(2) - xt(1))/2.
        xe(nx+1) = xt(nx) + (xt(2) - xt(1))/2.
        do i=2,nx
           xe(i) = (xt(i-1)+xt(i))/2.
        enddo

        ye(   1) = -90.
        ye(ny+1) = +90.
        do j=2,ny
           ye(j) = (yt(j-1)+yt(j))/2.
        enddo

        rcode = nf_close(fid)

      write(6,9001) 'o coordinate arrays:'
      write(6,9001) '  nx,ny = ',nx,ny
      write(6,9002) '  xc: ',(xt(i),i=1,4),' ...',(xt(i),i=nx-2,nx)
      write(6,9002) '  xe: ',(xe(i),i=1,4),' ...',(xe(i),i=nx-2,nx+1)
      write(6,9002) '  yc: ',(yt(i),i=1,4),' ...',(yt(i),i=ny-2,ny)
      write(6,9002) '  ye: ',(ye(i),i=1,4),' ...',(ye(i),i=ny-2,ny+1)

      !--- reformat domain data to format required by cpl ---
      do j=1,ny
      do i=1,nx
        xc(  i,j)=xt(i)
        yc(  i,j)=yt(j)
        xv(1,i,j)=xe(i  )
        xv(2,i,j)=xe(i+1)
        xv(3,i,j)=xe(i+1)
        xv(4,i,j)=xe(i  )
        yv(1,i,j)=ye(j  )
        yv(2,i,j)=ye(j  )
        yv(3,i,j)=ye(j+1)
        yv(4,i,j)=ye(j+1)
        dph = ( xe(i+1) - xe(i) )*DEGtoRAD
        dth = sin(ye(j+1)*DEGtoRAD)-sin(ye(j)*DEGtoRAD)
        area(i,j) = dth*dph
      end do
      end do

      !-------------------------------------------------------------------------
      ! read in runoff domain data from an existing local file
      !-------------------------------------------------------------------------
      loc_fn='domain.runoff.nc'  ! hard-coded input file name

      write(6,9000) 'open netCDF file ',loc_fn
      rcode = nf_open(loc_fn,NF_NOWRITE,fid)
      if (rcode.ne.NF_NOERR) write(6,9000) nf_strerror(rcode)

      !--- print title info from nc file ---
      rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'title', n)
      if (n.le.240) then
        rcode = nf_get_att_text(fid, NF_GLOBAL, 'title'  , str)
      else
        str = '<too long to print>   '
        n = index(str,'   ')
      end if
      write(6,9000) 'o title   = ',str(1:n)
      rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'history', n)
      if (n.le.240) then
        rcode = nf_get_att_text(fid, NF_GLOBAL, 'history', str)
      else
        str = '<too long to print>   '
        n = index(str,'   ')
      end if
      write(6,9000) 'o history = ',str(1:n)
      rcode = nf_inq_attlen  (fid, NF_GLOBAL, 'source', n)
      if (n.le.240) then
        rcode = nf_get_att_text(fid, NF_GLOBAL, 'source' , str)
      else
        str = '<too long to print>   '
        n = index(str,'   ')
      end if
      write(6,9000) 'o source  = ',str(1:n)

      write(6,9000) 'read domain data...'

      rcode = nf_inq_varid(fid,'xc' ,vid)
      rcode = nf_get_var_double(fid,vid,xc_r)
      rcode = nf_inq_varid(fid,'xv' ,vid)
      rcode = nf_get_var_double(fid,vid,xv_r)

      rcode = nf_inq_varid(fid,'yc' ,vid)
      rcode = nf_get_var_double(fid,vid,yc_r)
      rcode = nf_inq_varid(fid,'yv' ,vid)
      rcode = nf_get_var_double(fid,vid,yv_r)

      rcode = nf_inq_varid(fid,'mask',vid)
      rcode = nf_get_var_int(fid,vid,mask_r)

      rcode = nf_inq_varid(fid,'area',vid)
      rcode = nf_get_var_double(fid,vid,area_r)

      rcode = nf_close(fid)

      !------------------------------------------------------ 
      ! set compression key to domain mask (may not be ideal)
      !------------------------------------------------------ 
!     key_r = 1.0    !--- the "null" compression scheme 
      key_r = mask_r !--- compression key matches domain mask
      n=0
      do j=1,ny_r
      do i=1,nx_r
	if (key_r(i,j) /= 0) then
          n=n+1
          i_rc(n)  = i ! uncompressed i-index given compressed index
          j_rc(n)  = j ! uncompressed j-index given compressed index
          i_r(i,j) = n !  ncompressed   index given uncompressed (i,j)
	else
          i_r(i,j) = 0
	end if
      end do
      end do
      n_rc = n

      RETURN
 9000 format('(domain) ',4a)
 9001 format('(domain) ',a,2i4)
 9002 format('(domain) ',2(a,4f9.3))
 9003 format('(domain) ',a,i6)
 9010 format('(domain) return code ',i3, ' on ',4(a,1x))
      END
 
!=======================================================================
