! CVS $Id: initial.F90,v 1.5 2001/07/24 22:13:12 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/lnd/dlnd5/initial.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE initial()

      use calendar_mod ! calendar routines
      use shr_sys_mod  ! shared system routines

      implicit none

      !----- global -----
#include <dims.h>               
#include <control.h>            

      !----- local -----
      integer      i           ! generic loop indicies
      integer      rcode       ! return code

      NAMELIST / inparm /  &
     &  rest_type   ,      &
     &  rest_pfile  ,      &
     &  rest_bfile  ,      &
     &  rest_freq   ,      &
     &  rest_date   ,      &
     &  rest_nday   ,      &
     &  rest_odate  ,      &
     &  case_name   ,      &
     &  case_desc   ,      &
     &  data_dir    ,      &
     &  data_year0  ,      &
     &  data_nyear  ,      &
     &  data_oyear  ,      &
     &  data_rmlf   ,      &
     &  data_aroff  ,      &
     &  data_fix    ,      &
     &  data_mask   ,      &
     &  mss_dir     ,      &
     &  mss_pass    ,      &
     &  mss_opts    ,      &
     &  mss_rtpd    ,      &
     &  mss_rmlf    ,      &
     &  info_dbug   ,      &
     &  info_time   ,      &
     &  info_sleep  ,      &
     &  ncpl        

      !----- formats -----
      character(len=*),parameter :: F00 = "('(initial) ',2a)"
      character(len=*),parameter :: F01 = "('(initial) ',a,i7)"

!-------------------------------------------------------------------------------
! PURPOSE:
!  o set default control variable values
!  o read ice input parameter namelist file
!  o read ice coordinate grids & domain mask
!  o read sst climatology
!  o convert sst climatology from celsius to kelvin
!-------------------------------------------------------------------------------

      !-----------------------------------------------
      ! set initial namelist values
      !-----------------------------------------------
      write(6,'(a)') '(initial) setting namelist defaults.'
      rest_type   = 'continue'  ! run type: initial, cont, regen, branch
      rest_pfile  = 'null'      ! IC data pointer file
      rest_bfile  = 'null'      ! IC data file for branch runs
      rest_freq   = 'never'     ! restart file frequency
      rest_date   = 10101       ! start date for initial or branch runs
      rest_nday   = 365         ! restart file interval    (nday option)
      rest_odate  = 10101       ! restart file offset date (nday option)
      case_name   = ' '         ! case name
      case_desc   = ' '         ! case description
      data_dir    = 'null'      ! data file with domain data
      data_year0  = 1           ! 1st  year  in data sequence
      data_nyear  = 1           ! # of years in data sequence
      data_oyear  = 1           ! cpl year corresponding to data_year0
      data_rmlf   = 0           ! T => remove local file after input
      data_aroff  = 'zero'      ! 'null' => use roff from standard data files
      data_fix    = 'null'      ! flags to fix input data
      data_mask   = 'lnd'       ! flags source of domain mask
      mss_dir     = 'null'      ! MSS file directory
      mss_pass    = ' '         ! MSS file password
      mss_opts = 'nowait,nomail'! MSS file mswrite options
      mss_rtpd    = 180         ! MSS file MS retention period
      mss_rmlf    = 1           ! T => remove local file after mswrite
      info_dbug   = 1           ! dbug level: 0=lowest, 3=highest
      info_time   = 0           ! T => print extra timing info
      info_sleep  = 0.0         ! simulate time to advance an active model
      ncpl        = 24          ! number of msg pairs per day

      !-----------------------------------------------
      ! read input namelist
      !-----------------------------------------------
      write(6,F00) 'reading input namelist.'
      read (5,inparm,iostat=rcode)
      if (rcode > 0) then
         write(6,F01) 'ERROR: reading input namelist, iostat=',rcode
	 call shr_sys_abort("dlnd namelist read error")
      end if

      !-----------------------------------------------
      ! re-set some initial namelist values
      !-----------------------------------------------

      !--- null data works with any mask: get it from cpl ---
      if (data_dir(1:4).eq.'null') data_mask = 'cpl'

      !--- set flag wrt using cpl supplied domain mask ---
      init_mask = 0
      if (data_mask(1:3).eq.'cpl') init_mask = 1

      !-----------------------------------------------
      ! record initial namelist values
      !-----------------------------------------------
      write(6,F00) 'namelist values:'
      write(6,inparm)
      write(6,*) 

      !-----------------------------------------------
      ! set initial values for date & time
      !-----------------------------------------------
      call date2eday(rest_date,eday)
      call date2eday(rest_date,eday0)
      call date2ymd (rest_date,year,month,day)
      sec = 0

      !-----------------------------------------------
      ! inititial values for control flags
      !-----------------------------------------------
      ierr     = 0  ! error code
      stop_now = 0  ! T => stop now
      stop_eod = 0  ! T => stop at end-of-day
      rest_now = 0  ! T => create restart file now
      rest_eod = 0  ! T => create restart file at end-of-day
      hist_now = 0  ! T => create history file now
      hist_eod = 0  ! T => create history file at end-of-day
      hist_tav = 0  ! T => create monthly avg data
      diag_now = 0  ! T => create diagnostic data now
      diag_eod = 0  ! T => create diagnostic data at end-of-day


      RETURN
      END

!===============================================================================
