!===============================================================================
! CVS $Id: main.F90,v 1.2 2000/06/23 23:17:53 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/lnd/dlnd5/main.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      PROGRAM main

      use calendar_mod  ! calendar routines
      use shr_msg_mod   ! shared code: msg passing
      use shr_sys_mod   ! shared code: system calls
      use shr_timer_mod ! shared code: timing routines

      implicit none

      !----- global -----
#include <control.h>            

      !----- local -----
      integer      n        ! inner loop index
      character*8  dstr     ! real world date string
      character*10 tstr     ! real world time string
      integer      timer_n  ! timer number

!-------------------------------------------------------------------------------
! PURPOSE:
!   this is a climatological data lnd model
!   for use as an lnd component of a CSM coupled model system
!-------------------------------------------------------------------------------

      call shr_msg_stdio('lnd')
      call shr_timer_init
      call shr_timer_get(timer_n,'main ~ startup overhead')
      call shr_timer_start(timer_n)

      !-------------------------------------------------------------------------
      ! initialize
      !-------------------------------------------------------------------------
      call date_and_time(dstr,tstr)
      write(6,9001) 'CSM climatology data lnd model'
      write(6,9004) dstr(1:4),dstr(5:6),dstr(7:8),tstr(1:2),tstr(3:4),tstr(5:6)
      write(6,9001) 'CVS Tag: $Name: ccsm2_0_beta46 $'
      write(6,9002)

      call initial()              !--- read input namelist ---
      call domain ()              !--- read in domain data ---

      call msg_pass('connect')    !--- enable msg passing ---
      call msg_pass('init')       !--- send domain, get IC t ---
      call control()              !--- set control flags & IC t---

      if ( init_mask.ne.0 ) then  !--- auxiliary IC msgs ---
        write(6,9000) 'cpl providing lnd mask (requested by lnd)'
        call msg_pass('mask')     !--- recv domain mask  ---
      end if

      call advance()              !--- advance the data ---

      call shr_timer_stop (timer_n)
      call shr_timer_print(timer_n)
      call shr_timer_free (timer_n)
      call shr_timer_get  (timer_n,'main ~ model integration')
      call shr_timer_zero_all
      call shr_timer_start(timer_n)

      !-------------------------------------------------------------------------
      ! advance the model: outer loop corresponds to 1 day
      !-------------------------------------------------------------------------
      DO WHILE ( stop_now .eq. 0 )
         DO n=1,ncpl
           if (sec.le.0 .or. info_dbug.ge.2) then
              write(6,9003) 'date: ',year,month,day,sec
              call shr_sys_flush(6)
           end if

           call msg_pass('send')     !--- send output fields ---
           call msg_pass('recv')     !--- recv input fields ---

           call control()            !--- set control flags ---

           call diagnos()            !--- write diagnostics? ---
           call restart()            !--- make restart file? ---
           call history()            !--- make history file? ---

           if (stop_now .ne. 0) GOTO 900

           if (n.lt.ncpl) then       !--- advance the date ---
             sec  = sec + 86400/ncpl
           else
             eday = eday + 1
             call eday2ymd (eday,year,month,day)
             call eday2date(eday,cdate)
             sec  = 0
           end if
           call advance()            !--- advance the data ---

         END DO
      END DO
  900 CONTINUE
      call shr_timer_stop(timer_n)

      !-------------------------------------------------------------------------
      ! shut-down
      !-------------------------------------------------------------------------
      call msg_pass('disconnect')    !--- disable msg passing ---
      call shr_timer_print_all

      call date_and_time(dstr,tstr)
      write(6,9002)
      write(6,9001) 'Terminated model'
      write(6,9004) dstr(1:4),dstr(5:6),dstr(7:8),tstr(1:2),tstr(3:4),tstr(5:6)
      write(6,9002)

      STOP
 9000 format('(main) ',4a)
 9001 format(4a)
 9002 format(80('='))
 9003 format('(main) ',a,i4.4,'-',i2.2,'-',i2.2,' ',i5,'s')
 9004 format('date & time:',1x,a4,'-',a2,'-',a2,1x,a2,':',a2,':',a2)
 9999 format(                                                &
     & ' NCAR CSM Data Lnd Model,  Copyright (C) 2000'/,     &
     & ' University Corporation for Atmospheric Research, ', &
     & ' All Rights Reserved')

      END

!===============================================================================
