!===============================================================================
! CVS $Id: msg.F90,v 1.10 2001/08/01 22:48:18 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/lnd/dlnd5/msg.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      SUBROUTINE msg_pass(cOption)

      use shr_msg_mod   ! shared code: msg passing
      use shr_sys_mod   ! shared code: system calls
      use shr_timer_mod ! shared code: timing routines

      implicit none

      !----- arguments -----
      character*(*) cOption

      !----- global -----
#include <dims.h>                 
#include <data_raw.h>             
#include <data_msg.h>             
#include <control.h>              

      !----- local -----
      integer,parameter :: nsnd=14     ! number of l->c 2d fields
      integer,parameter :: nrcv=16     ! number of c->l 2d fields
      real        :: buffs(nx,ny,nsnd) ! contiguous array for sending data
      real        :: buffr(nx,ny,nrcv) ! contiguous array for recving data
      integer     :: maska(nx,ny)      ! atm domain mask
      integer     :: timer_n           ! timer number
      integer     :: timer_m           ! timer number
      integer     :: i,j               ! generic loop indicies
      integer*8   :: irtc_w            ! rtc tics when waiting for msg
      integer*8   :: irtc_r            ! rtc tics when msg sent or recved
      integer*8   :: irtc_u            ! rtc tics when msg unpacked
      integer*8   :: irtc_p            ! rtc tics when packing msg
      integer*8   :: irtc_s            ! rtc tics when sending msg

      save

!-------------------------------------------------------------------------------
! PURPOSE:
!   implement various message passing tasks
!
! FUCTIONALITY:
!   cOption=
!     'connect'    => connect to the msg passing system
!     'init'       => recv/send initial info
!     'send'       => send model state & output fluxes
!     'recv'       => receive input fluxes & stop/continue flags
!     'disconnect' => disconnect from the msg passing system
!-------------------------------------------------------------------------------

      IF (cOption .eq. 'connect') THEN 
        !-----------------------------------------------------------------------
        ! connect to msg passing system
        !-----------------------------------------------------------------------
        write(6,9000) '(connect) connecting to cpl'
        call shr_msg_init('lnd')
        call shr_msg_groups('lnd')
        call shr_sys_flush(6)

        call shr_timer_get(timer_n,'msg: lnd  -> cpl')
        call shr_timer_get(timer_m,'msg: lnd <-  cpl')
         
      ELSE IF (cOption .eq. 'init') THEN 
        !-----------------------------------------------------------------------
        ! recv/send initial data
        !-----------------------------------------------------------------------
        write(6,9000) '(init) recv initial msg'

        call shr_msg_recv_i(ibuffr, nibuff  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2LI)
        call shr_msg_recv_r(rbuffr, nibuff  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2LI)
        
        ierr      = ibuffr( 1) ! error code
        cdate     = ibuffr( 4) ! current date
        sec       = ibuffr( 5) ! elapsed sec on current date
        info_time = ibuffr(11) ! T => turn on msg-passing timing
        !nfo_dbug = ibuffr(12) ! cpl debug level

        !--- send grid & domain mask ---
        write(6,9000) '(init) send initial msg'

        ibuffs    =0         ! 0 => unused/ignore/invalid/false
        ibuffs( 1)=ierr      ! error code
        ibuffs( 7)=nx        ! number of grid points in x-direction
        ibuffs( 8)=ny        ! number of grid points in y-direction
        ibuffs( 9)=ncpl      ! number of msg-pairs per day
        ibuffs(34)=init_mask ! flag cpl to send back domain spec
        ibuffs(36)=n_rc      ! size of runoff vector in compressed 1d form
        if (n_rc == nx_r*ny_r) ibuffs(36)=0 ! runoff is not compressed
        ibuffs(37)=nx_r      ! nx of runoff data in uncompressed 2d array form
        ibuffs(38)=ny_r      ! ny of runoff data in uncompressed 2d array form
        
        call shr_msg_send_i(ibuffs, nibuff  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)

        call shr_msg_send_r(xc    , nx*ny   ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
        call shr_msg_send_r(yc    , nx*ny   ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
        call shr_msg_send_r(xv    , nx*ny*4 ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
        call shr_msg_send_r(yv    , nx*ny*4 ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
        call shr_msg_send_r(area  , nx*ny   ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
        call shr_msg_send_i(mask  , nx*ny   ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)

        call shr_msg_send_r(xc_r  ,nx_r*ny_r  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
        call shr_msg_send_r(yc_r  ,nx_r*ny_r  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
        call shr_msg_send_r(xv_r  ,nx_r*ny_r*4,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
        call shr_msg_send_r(yv_r  ,nx_r*ny_r*4,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
        call shr_msg_send_r(area_r,nx_r*ny_r  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
        call shr_msg_send_i(mask_r,nx_r*ny_r  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)

        if (n_rc < nx_r*ny_r) then ! runoff is compressed
          call shr_msg_send_i(i_rc  , n_rc    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
          call shr_msg_send_i(j_rc  , n_rc    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
        endif
        
      ELSE IF (cOption .eq. 'mask') THEN 
        !-----------------------------------------------------------------------
        ! recv domain spec msg
        !-----------------------------------------------------------------------
        write(6,9000) '(mask) recv domain spec'
        call shr_sys_flush(6)

        call shr_msg_recv_i(ibuffr,nibuff   ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2LI)
        call shr_msg_recv_r(  xc , nx*ny    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2LI)
        call shr_msg_recv_r(  yc , nx*ny    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2LI)
        call shr_msg_recv_r(  xv , nx*ny*4  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2LI)
        call shr_msg_recv_r(  yv , nx*ny*4  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2LI)
        call shr_msg_recv_r(area , nx*ny    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2LI)
        call shr_msg_recv_r(  fl , nx*ny    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2LI)
        call shr_msg_recv_i(mask , nx*ny    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2LI)
        call shr_msg_recv_i(maska, nx*ny    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2LI)
        
        write(6,9000) '(mask) send updated runoff domain spec'
        !--- note: a real model could update compression scheme here ---

        ibuffs    =0         ! 0 => unused/ignore/invalid/false
        ibuffs( 1)=ierr      ! error code
        ibuffs( 7)=nx        ! number of grid points in x-direction
        ibuffs( 8)=ny        ! number of grid points in y-direction
        ibuffs( 9)=ncpl      ! number of msg-pairs per day
        ibuffs(36)=n_rc      ! size of runoff vector in compressed 1d form
        if (n_rc == nx_r*ny_r) ibuffs(36)=0 ! runoff is not compressed
        ibuffs(37)=nx_r      ! nx of runoff data in uncompressed 2d array form
        ibuffs(38)=ny_r      ! ny of runoff data in uncompressed 2d array form
        
        call shr_msg_send_i(ibuffs, nibuff  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
        if (n_rc < nx_r*ny_r) then ! runoff is compressed
          call shr_msg_send_i(i_rc  , n_rc    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
          call shr_msg_send_i(j_rc  , n_rc    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2CI)
        end if

      ELSE IF (cOption .eq. 'send') THEN 
        !-----------------------------------------------------------------------
        ! send output field msg
        !-----------------------------------------------------------------------
        if (info_dbug.ge.3) write(6,9000) '(send) send data to cpl'
        call shr_timer_start(timer_n)

        !--- set info buffer flags ---
        irtc_p = shr_sys_irtc()   ! packing msg
        ibuffs     = 0     ! 0 => unused/ignore/invalid/false
        ibuffs( 1) = ierr  ! error code
        ibuffs( 3) = 0     ! stop flag: 0 <=> able to continue
        ibuffs( 4) = cdate ! model date, coded: yyyymmdd
        ibuffs( 5) = sec   ! elapsed seconds on model date
        ibuffs( 7) = nx    ! number of grid points in x-direction
        ibuffs( 8) = ny    ! number of grid points in y-direction
        ibuffs( 9) = ncpl  ! number of msg-pairs per day
        ibuffs(31) = 0     ! precip adjustment factor (* 1.0e+6)

        !--- pack & send msg buffer ---
        do j=1,ny
        do i=1,nx
          !--- lnd states ---
          buffs(i,j, 1) = t    (i,j)  ! surface T
          buffs(i,j, 2) = alvdr(i,j)  ! albedo, visible   , direct
          buffs(i,j, 3) = alndr(i,j)  ! albedo, near-infra, direct
          buffs(i,j, 4) = alvdf(i,j)  ! albedo, visible   , diffuse
          buffs(i,j, 5) = alndf(i,j)  ! albedo, near-infra, diffuse
          buffs(i,j, 6) = snowh(i,j)  ! snow depth
          !--- lnd output fluxes ---
          buffs(i,j, 7) = taux (i,j)  ! momentum flux - zonal
          buffs(i,j, 8) = tauy (i,j)  ! momentum flux - meridional
          buffs(i,j, 9) = lat  (i,j)  ! heat flux - latent
          buffs(i,j,10) = sen  (i,j)  ! heat flux - sensible
          buffs(i,j,11) = lwup (i,j)  ! heat flux - longwave, upward
          buffs(i,j,12) = evap (i,j)  ! water flux - evaporation
          buffs(i,j,14) = swnet(i,j)  ! heat flux - sw-net
          !--- lnd diagnostics ---
          buffs(i,j,13) = tref (i,j)  ! 2m reference T
        end do
        end do
 
        irtc_s = shr_sys_irtc()   ! sending msg
        call shr_msg_send_i(ibuffs, nibuff   ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2C)
        call shr_msg_send_r( buffs,nx*ny*nsnd,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2C)
        call shr_msg_send_r(roff_rc, n_rc    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_L2C)
        call shr_timer_stop(timer_n)

        if (info_time.ne.0) then
           write(6,9099) irtc_p,'l->c packing'
           write(6,9099) irtc_s,'l->c sending'
        end if

 
      ELSE IF (cOption .eq. 'recv') THEN
        !-----------------------------------------------------------------------
        ! recv input field msg
        !-----------------------------------------------------------------------
        if (info_dbug.ge.3) write(6,9000) '(recv) recv data to cpl'
        call shr_timer_start(timer_m)
        
        irtc_w = shr_sys_irtc()   ! waiting for msg
        call shr_msg_recv_i(ibuffr, nibuff   ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2L)
        irtc_r = shr_sys_irtc()   ! received msg
        call shr_msg_recv_r( buffr,nx*ny*nrcv,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2L)

        do j=1,ny
        do i=1,nx
           !--- atm states ---
           z     (i,j) = buffr(i,j, 1)  ! layer height
           u     (i,j) = buffr(i,j, 2)  ! velocity - zonal
           v     (i,j) = buffr(i,j, 3)  ! velocity - merid
           ptem  (i,j) = buffr(i,j, 4)  ! potential T
           shum  (i,j) = buffr(i,j, 5)  ! specific humidity
           pres  (i,j) = buffr(i,j, 6)  ! pressure
           t     (i,j) = buffr(i,j, 7)  ! T
           !--- atm/srf fluxes ---
           lwdn  (i,j) = buffr(i,j, 8)  ! longwave, down
           precc (i,j) = buffr(i,j, 9)  ! precip, liquid, convective
           precl (i,j) = buffr(i,j,10)  ! precip, liquid, large scale
           snowc (i,j) = buffr(i,j,11)  ! precip, frozen, convective
           snowl (i,j) = buffr(i,j,12)  ! precip, frozen, large scale
           swndr (i,j) = buffr(i,j,13)  ! sw down, near-infra, direct
           swvdr (i,j) = buffr(i,j,14)  ! sw down, visible     direct
           swndf (i,j) = buffr(i,j,15)  ! sw down, near-infra, diffuse
           swvdf (i,j) = buffr(i,j,16)  ! sw down, visible     duffuse
        end do
        end do

        irtc_u = shr_sys_irtc()   ! unpacked msg

        ierr      = ibuffr( 1) ! cpl error code (0 => no error)
        stop_eod  = ibuffr( 2) ! T => stop at end-of-day
        stop_now  = ibuffr( 3) ! T => stop now
        !           ibuffr( 4) ! current date
        !           ibuffr( 5) ! elapsed sec on current date
        !           ibuffr(11) ! T => turn on msg-passing timing
        !           ibuffr(12) ! cpl debug level
        rest_eod  = ibuffr(21) ! T => create restart data at end-of-day
        rest_now  = ibuffr(22) ! T => create restart data now
        hist_eod  = ibuffr(23) ! T => create history data at end-of-day
        hist_now  = ibuffr(24) ! T => create history data now
        hist_tav  = ibuffr(25) ! T => create create monthly avg data
        diag_eod  = ibuffr(26) ! T => create diagnotic data at eod
        diag_now  = ibuffr(27) ! T => create diagnotic data now
        !           ibuffr(32) ! albedo time shift
        !           ibuffr(33) ! number of active runoff basins

        call shr_timer_stop(timer_m)
        if (info_time.ne.0 .and. stop_now.eq.0) then
           write(6,9099) irtc_w,'c->l waiting'
           write(6,9099) irtc_r,'c->l received'
           write(6,9099) irtc_u,'c->l unpacked'
        end if

      ELSE IF (cOption .eq. 'disconnect') THEN 
        !-----------------------------------------------------------------------
        ! disconnect from msg passing
        !-----------------------------------------------------------------------
        write(6,9000) '(disconnect) disconnect from cpl'
        call shr_msg_finalize

      ELSE 
        !-----------------------------------------------------------------------
        ! this should never happen
        !-----------------------------------------------------------------------
        write(6,9000) 'ERROR: invalid cOption: ',cOption
        stop
      END IF

      RETURN
 9000 format('(msg_pass) ',2a)
 9099 format('[mp timing]  irtc = ',i20,' ',a)
      END

!===============================================================================
