!===============================================================================
! CVS $Id: runoff_aux.F90,v 1.4 2001/07/24 22:13:13 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/lnd/dlnd5/runoff_aux.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

SUBROUTINE runoff_aux()

   use const_mod  ! physical constants

   implicit none

   !----- global -----
#include <dims.h>            
#include <data_raw.h>        
#include <control.h>         
#include <netcdf.inc>         

   !----- local ------
   integer               :: i,n        ! generic loop indicies
   integer               :: ni,nj      ! size of data in data file
   integer               :: fid        ! nc file ID
   integer               :: vid        ! nc variable ID
   integer               :: did        ! nc dimension ID
   integer               :: rcode      ! return code
   character*80          :: str        ! input data text string
   real,allocatable,save :: data(:,:)  ! input data
   logical,save  :: firstcall = .true. ! flags one-time initializations

   !----- formats ------
   character(len=*),parameter :: F00 = "('(runoff_aux) ',3a)"
   character(len=*),parameter :: F01 = "('(runoff_aux) ',a,i2,a,e13.3,a)"
   character(len=*),parameter :: F02 = "('(runoff_aux) ',a,2i3)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   get auxiliary runoff data (as per Bill Large NCAR/OCE)
! NOTE:
! o this auxiliary data is time constant -- has no annual cycle
! o cpl expects units to be kg/s/m^2
!-------------------------------------------------------------------------------

   if (firstcall) then
     !-------------------------------------------------------------
     ! initialize/read-in data
     !-------------------------------------------------------------
     write(6,F00) "using auxiliary runoff data file ",data_aroff

     !--- open file ---
     write(6,F00) 'open netCDF file ',trim(data_aroff)
     rcode = nf_open(data_aroff,NF_NOWRITE,fid)
     if (rcode.ne.NF_NOERR) write(6,F00) nf_strerror(rcode)

     rcode = nf_inq_dimid(fid,'ni',did)
     rcode = nf_inq_dimlen(fid, did, ni)
     rcode = nf_inq_dimid(fid,'nj',did)
     rcode = nf_inq_dimlen(fid, did, nj)
     allocate(data(ni,nj))
     rcode = nf_inq_varid(fid,'runoff',vid)
     rcode = nf_get_var_double(fid,vid,data)

     rcode = nf_close(fid)

     !--- check for coordinated array dimensions ---
     if (ni /= nx_r .or. nj /= ny_r .or. nj /= 1) then
       write(6,F02) 'ERROR: model dims don''t match data file dims'
       write(6,F02) 'ERROR: model nx_r, ny_r = ',nx_r, ny_r
       write(6,F02) 'ERROR: file  ni  , nj   = ',ni  , nj
       stop 'roff dims'
     end if

     !-------------------------------------------------------------
     ! document data values 
     !-------------------------------------------------------------
     if (info_dbug.ge.1) then
       do i=1,ni
         write(6,F01) "basin ",i," ~ ",data(i,1)," kg/s/m^2"
       end do
     end if
   end if

   !-------------------------------------------------------------
   ! load appropriate data structure
   !-------------------------------------------------------------
   r_roff_r(:,1,1) = data(:,1)
   r_roff_r(:,1,2) = data(:,1)

   firstcall = .false.

RETURN
END

!===============================================================================
