!=======================================================================
! CVS $Id: advance.F90,v 1.4 2001/05/14 05:26:48 mvertens Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ocn/docn5/advance.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!=======================================================================

SUBROUTINE advance()

  !----modules -----
  use kind_mod              ! precision
  use control_mod           ! control info, date/time
  use calendar_mod          ! calendar routines
  use shr_msg_mod           ! data sent/received from coupler
  use shr_timer_mod         ! shared code: timing routines
  use buf_mod               ! allocatable arrays
  use data_in_mod, only : r_t, r_s, r_u, r_v, &
                          r_dhdx, r_dhdy, r_q, nm, np, eday_sst, &
                          data_in_init, data_in_multiyr

  implicit none

  !----- local ------
  integer :: i,j,n               ! generic loop indicies
  integer :: m1,m2               ! indices for time interpolation
  real(r8):: f1,f2               ! fractions for time interpolation
  real(r8):: deltat              ! SST input time difference         
  integer :: edoy                ! elapsed days within annually cycled year
  integer, save :: timer_n       ! timer number
  logical :: first_call = .true. ! flags 1-time initialization

!-----------------------------------------------------------------------
! PURPOSE:
! advance the model output states & fluxes to current date & time
! (ie. time interplate sst data fields onto current date & time)
!
! ASSUMPTIONS:
! the sst data is either a 12 month annual or multiyear dataset
!-----------------------------------------------------------------------

  if (first_call) call shr_timer_get(timer_n,'advance')
  call shr_timer_start(timer_n)

  !-----------------------------------------------------------------
  ! Read in raw data and time interpolate
  !-----------------------------------------------------------------
  if (first_call) then
     call data_in_init()
     first_call = .false.
  endif

  !-----------------------------------------------------------------
  ! compute time interpolation factors 
  !-----------------------------------------------------------------
  if (data_form(1:6) == 'annual') then

     m1=0
     m2=0
     edoy = mod(eday,365)
     if (edoy<=eday_sst(1) .OR. edoy>eday_sst(12)) then
        m1 = 12
        m2 = 1
        deltat = eday_sst(m2) + 365. - eday_sst(m1)
        if (edoy > eday_sst(m2)) then ! We're in December
           f1 = (eday_sst(m2) + 365. - edoy)/deltat
           f2 = (edoy - eday_sst(m1))/deltat
        else                          ! We're in January
           f1 = (eday_sst(m2) - edoy)/deltat
           f2 = (edoy + 365. - eday_sst(m1))/deltat
        end if
     else
        do n=1,11
           if (edoy>eday_sst(n) .AND. edoy<=eday_sst(n+1)) then
              m1 = n
              m2 = n+1
              deltat = eday_sst(m2) - eday_sst(m1)
              f1 = (eday_sst(m2) - edoy)/deltat
              f2 = (edoy - eday_sst(m1))/deltat
           endif
        end do
     endif
     if (m1==0 .OR. m2==0) then
        write(6,9000)'Failed to find dates bracketing edoy',edoy
        call shr_sys_abort
     endif

  elseif (data_form(1:9) == 'multiyear') then

     call data_in_multiyr()

     m1 = nm
     m2 = np
     deltat = eday_sst(m2) - eday_sst(m1)
     f1 = (eday_sst(m2) - eday )/deltat
     f2 = (eday  - eday_sst(m1))/deltat

  endif

  ! Check sanity of time interpolation calculation 
  ! to within 32-bit roundoff

  if (abs(f1+f2-1.)>1.e-6        .or. &
      f1>1.000001 .or. f1<-1.e-6 .or. &
      f2>1.000001 .or. f2<-1.e-6) then
     write(6,9001)'Bad f1 and/or f2=',f1,f2
     call shr_sys_abort()
  end if

  if (info_dbug>=3) write(6,9003) 'm1,m2,f1,f2=',m1,m2,f1,f2

  !-----------------------------------------------------------------
  ! advance the model state (ie. do time interpolation)
  ! SST's need to have a lower bound of -1.8
  !-----------------------------------------------------------------
  write(6,*)'(advance): f1 = ',f1,' f2= ',f2
  T   (:,:) = max((f1*r_t   (:,:,m1) + f2*r_t   (:,:,m2)),-1.8)
  s   (:,:) =      f1*r_s   (:,:,m1) + f2*r_s   (:,:,m2)
  u   (:,:) =      f1*r_u   (:,:,m1) + f2*r_u   (:,:,m2)
  v   (:,:) =      f1*r_v   (:,:,m1) + f2*r_v   (:,:,m2)
  dhdx(:,:) =      f1*r_dhdx(:,:,m1) + f2*r_dhdx(:,:,m2)
  dhdy(:,:) =      f1*r_dhdy(:,:,m1) + f2*r_dhdy(:,:,m2)
  Q   (:,:) =      f1*r_Q   (:,:,m1) + f2*r_Q   (:,:,m2)

  !-----------------------------------------------------------------
  ! wipe-out out-of-domain values (?)
  !-----------------------------------------------------------------
  if (.false.) then
     where (mask(:,:) == 0) 
        T   (:,:) = 1.0e30
        s   (:,:) = 1.0e30
        u   (:,:) = 1.0e30
        v   (:,:) = 1.0e30
        dhdx(:,:) = 1.0e30
        dhdy(:,:) = 1.0e30
        Q   (:,:) = 1.0e30
     end where
  end if

  if (info_sleep > 0.0) call shr_timer_sleep(info_sleep)
  call shr_timer_stop(timer_n)
  first_call=.false.

  RETURN
9000 format('(advance) ',a,2(1x,i3),i5)
9001 format('(advance) ',a,2(1x,f5.3))
9003 format('(advance) ',a,2(1x,i3),2(1x,f5.3))

END SUBROUTINE advance
!=======================================================================





