!=======================================================================
! CVS $Id: buf_mod.F90,v 1.1 2001/05/08 16:25:38 mvertens Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ocn/docn5/buf_mod.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!=======================================================================

module buf_mod

  use kind_mod
  implicit none

  !----- message send/receive parameters--------
  integer,parameter :: nibuff = 100  ! size of control buffers
  integer,parameter :: nsnd   =  7   ! number of 2d fields to send
  integer,parameter :: nrcv   = 14   ! number of 2d fields to receive

  !----- flags -----
  integer   ibuffs(nibuff) ! control flags sent to coupler
  integer   ibuffr(nibuff) ! control flags received from coupler

  !----- domain information -----
  integer               ::   nx        ! size of x grid
  integer               ::   ny        ! size of y grid
  integer               ::   nv        ! corners
  real(r8), allocatable ::   xc(:,:)   ! x-grid center coordinates ~ deg east
  real(r8), allocatable ::   yc(:,:)   ! y-grid center coordinates ~ deg north
  real(r8), allocatable ::   xv(:,:,:) ! x-grid vertex coordinates ~ deg east 
  real(r8), allocatable ::   yv(:,:,:) ! y-grid vertex coordinates ~ deg north 
  real(r8), allocatable ::   area(:,:) ! grid cell area            ~ rad^2
  integer , allocatable ::   mask(:,:) ! domain mask: 0 <=> cell NOT in domain

  !----- states sent to coupler -----
  real(r8), allocatable ::   T    (:,:)   ! temperature               ~ Kelvin
  real(r8), allocatable ::   s    (:,:)   ! state: salinity               ~ ppt   
  real(r8), allocatable ::   u    (:,:)   ! state: velocity, zonal        ~ m/s
  real(r8), allocatable ::   v    (:,:)   ! state: velocity, meridional   ~ m/s
      
  !----- fluxes sent to coupler -----
  real(r8), allocatable ::   dhdx (:,:)   ! dh/dx (surface slope)     ~ m/m
  real(r8), allocatable ::   dhdy (:,:)   ! dh/dy (surface slope)     ~ m/m
  real(r8), allocatable ::   Q    (:,:)   ! heat of fusion    (q > 0) ~ W/m^2

  !----- fluxes received from coupler -----
  real(r8), allocatable ::   taux (:,:)   ! wind stress, zonal        ~     N/m^2
  real(r8), allocatable ::   tauy (:,:)   ! wind stress, meridional   ~     N/m^2
  real(r8), allocatable ::   netsw(:,:)   ! heat flux: net short wave ~     W/m^2
  real(r8), allocatable ::   lat  (:,:)   ! heat flux: latent         ~     W/m^2
  real(r8), allocatable ::   sen  (:,:)   ! heat flux: sensible       ~     W/m^2
  real(r8), allocatable ::   lwup (:,:)   ! heat flux: long-wave up   ~     W/m^2
  real(r8), allocatable ::   lwdn (:,:)   ! heat flux: long-wave down ~     W/m^2
  real(r8), allocatable ::   melth(:,:)   ! heat flux: melt           ~     W/m^2
  real(r8), allocatable ::   salt (:,:)   ! salt flux                 ~ ppt/s/m^2
  real(r8), allocatable ::   prec (:,:)   ! water flux: precip        ~  kg/s/m^2
  real(r8), allocatable ::   evap (:,:)   ! water flux: evaporation   ~  kg/s/m^2
  real(r8), allocatable ::   meltw(:,:)   ! water flux: melt          ~  kg/s/m^2
  real(r8), allocatable ::   roff (:,:)   ! water flux: runoff        ~  kg/s/m^2
  
  !----- states received from coupler -----
  real(r8), allocatable ::   ifrac(:,:)   ! ice fraction              ~ in [0,1]
  real(r8), allocatable ::   patm (:,:)   ! atm sea level pressure    ~ Pa

  !----- send/receive buffers -------------
  real(r8), allocatable ::   buffs(:,:,:) ! contiguous array for sending data
  real(r8), allocatable ::   buffr(:,:,:) ! contiguous array for recving data

end module buf_mod
