!=======================================================================
! CVS $Id: control_mod.F90,v 1.2 2001/05/18 20:19:42 mvertens Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ocn/docn5/control_mod.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!=======================================================================

module control_mod

  use kind_mod     ! precision
  use shr_sys_mod	
  use calendar_mod ! calendar routines
  implicit none

  !----- input namelist parms --------------------------------------
  character(len=16) :: rest_type    ! run type: initial, cont, regen, branch
  character(len=64) :: rest_pfile   ! IC data pointer file
  character(len=64) :: rest_bfile   ! IC data file for branch runs
  character(len=16) :: rest_freq    ! restart file frequency
  integer           :: rest_date    ! start date for initial or branch runs
  integer           :: rest_nday    ! restart file interval    (nday option
  integer           :: rest_odate   ! restart file offset date (nday option)
  integer           :: rest_lag     ! restart date: lag by 1 sync interval?
  character(len=16) :: case_name    ! case name
  character(len=64) :: case_desc    ! case description
  character(len=256):: data_dir     ! data: directory for input files
  character(len=256):: domain_file  ! domain data file
  integer           :: domain_nx    ! nx for domain fabrication option
  integer           :: domain_ny    ! nx for domain fabrication option
  character(len=256):: data_file    ! sst data file
  character(len=16) :: data_form    ! either "annual" or "multiyear"
  character(len=16) :: data_sstname ! name of sst field on input data file
  character(len=16) :: data_lonname ! name of longitude coordinate on input data file
  character(len=16) :: data_latname ! name of latitude  coordinate on input data file
  character(len=64) :: mss_dir      ! MSS file directory
  character(len=16) :: mss_pass     ! MSS file password
  character(len=16) :: mss_opts     ! MSS file mswrite options
  integer           :: mss_rtpd     ! MSS file MS retention period
  integer           :: mss_rmlf     ! T => remove local file after mswrite
  integer           :: info_dbug    ! dbug level: 0=lowest, 3=highest
  integer           :: info_time    ! T => print extra timing info
  real(r8)          :: info_sleep   ! simulate time to advance an active model 
  integer           :: ncpl         ! number of msg pairs per day

  !----- timing info -----------------------------------------------
  integer       :: eday         ! elapsed days, 0 => 1 Jan year 0
  integer       :: eday0        ! value of eday on startup
  integer       :: cdate        ! coded model date (yyyymmdd)
  integer       :: year         ! model year
  integer       :: month        ! model month
  integer       :: day          ! model day
  integer       :: sec          ! model elapsed seconds during model day

  !----- control flags ---------------------------------------------
  integer       :: stop_now     ! T => stop model now
  integer       :: stop_eod     ! T => stop model at end of day
  integer       :: rest_now     ! T => create restart data now
  integer       :: rest_eod     ! T => create restart data at end of day
  integer       :: hist_now     ! T => create history data now
  integer       :: hist_eod     ! T => create history data at end of day
  integer       :: hist_tav     ! T => create monthly avg data
  integer       :: diag_now     ! T => create diagnostic data now
  integer       :: diag_eod     ! T => create diagnostic data now

  integer, private :: ierr         ! error code

  SAVE

!===========================================================================
CONTAINS
!===========================================================================

  SUBROUTINE control_init()

!-----------------------------------------------------------------------
! PURPOSE:
!   o select model's IC date
!
! NOTE:
!   currently this routine has no logic wrt setting control flags --
!   all rest, hist, diag, & stop flags are set by the cpl
!-----------------------------------------------------------------------

    !-----------------------------------
    ! set IC date to coupler's IC date
    !-----------------------------------
    call date2eday(cdate,eday)
    call date2eday(cdate,eday0)
    call date2ymd (cdate,year,month,day)
    sec = 0

    !-----------------------------------
    ! record value of some flags
    !-----------------------------------
    if (info_time/=0) write(6,9000) 'msg timing flag is on'
    if (hist_tav /=0) write(6,9000) 'monthly avg flag is on'

    !-------------------------------------
    ! lag IC date by 1 sync interval?     
    !-------------------------------------
    if (rest_lag /= 0) then
       if (rest_type(1:4) == 'init' ) then
          write(6,9000) 'IC date is lagged'
          write(6,9003) 'IC date was    ',year,month,day,sec
          if ( ncpl > 1) then
             !--lag seconds --
             rest_lag = 1
             sec = sec + 86400/ncpl
          else
             !-- lag date --
             eday  = eday + 1
             eday0 = eday
             call eday2date(eday,cdate)
             call eday2ymd (eday,year,month,day)
          end if
          write(6,9003) 'IC date is now ',year,month,day,sec
       end if
    end if

    return
9000 format('(control) ',a,i6,i6,a)
9003 format('(control) ',a,i4.4,'-',i2.2,'-',i2.2,' ',i5,'s')
  end SUBROUTINE control_init

!===========================================================================

  SUBROUTINE control

!-----------------------------------------------------------------------
! PURPOSE:
!   o set, reset, and or record the values of restart, history, 
!     diagnostic, stopping, etc., control flags
!
! NOTE:
!   currently this routine has no logic wrt setting control flags --
!   all rest, hist, diag, & stop flags are set by the cpl
!   
! ASSUMPTION:
!   This routine is called *every* time a message is received from the
!   cpl, immediately *after* the message is received, this allows the
!   model to learn & use the coupler's IC date and control flags
!-----------------------------------------------------------------------

    !-----------------------------------------------------------------
    ! error handling
    !-----------------------------------------------------------------
    if ( ierr /= 0 ) then
       write(6,9000) 'ierr     = ',ierr
       stop_now = 1
       call shr_sys_abort
    end if

    !-----------------------------------------------------------------
    ! print out control flag values
    !-----------------------------------------------------------------
    if ( stop_now /= 0) write(6,9000) 'stop_now = true'
    if ( info_dbug==1 .and. sec==0) then
       if (stop_eod /= 0) write(6,9000) 'stop_eod = true'
       if (rest_now /= 0) write(6,9000) 'rest_now = true'
       if (rest_eod /= 0) write(6,9000) 'rest_eod = true'
       if (hist_now /= 0) write(6,9000) 'hist_now = true'
       if (hist_eod /= 0) write(6,9000) 'hist_eod = true'
       if (diag_now /= 0) write(6,9000) 'diag_now = true'
       if (diag_eod /= 0) write(6,9000) 'diag_eod = true'
    else if ((info_dbug==2 .and. sec==0) .or. info_dbug.ge.3) then
       if (stop_now == 0) write(6,9000) 'stop_now = false'
       if (stop_eod /= 0) write(6,9000) 'stop_eod = true'
       if (stop_eod == 0) write(6,9000) 'stop_eod = false'
       if (rest_now /= 0) write(6,9000) 'rest_now = true'
       if (rest_now == 0) write(6,9000) 'rest_now = false'
       if (rest_eod /= 0) write(6,9000) 'rest_eod = true'
       if (rest_eod == 0) write(6,9000) 'rest_eod = false'
       if (hist_now /= 0) write(6,9000) 'hist_now = true'
       if (hist_now == 0) write(6,9000) 'hist_now = false'
       if (hist_eod /= 0) write(6,9000) 'hist_eod = true'
       if (hist_eod == 0) write(6,9000) 'hist_eod = false'
       if (diag_now /= 0) write(6,9000) 'diag_now = true'
       if (diag_now == 0) write(6,9000) 'diag_now = false'
       if (diag_eod /= 0) write(6,9000) 'diag_eod = true'
       if (diag_eod == 0) write(6,9000) 'diag_eod = false'
    end if

    RETURN
9000 format('(control) ',a,i6,i6,a)
  end SUBROUTINE control

!=======================================================================

end MODULE control_mod






