!===============================================================================
! CVS $Id: domain.F90,v 1.3 2001/05/08 16:25:43 mvertens Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ocn/docn5/domain.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

SUBROUTINE domain()

  use control_mod           ! control info, date/time
  use shr_sys_mod           ! share system code
  use buf_mod               ! allocatable arrays and parameters
  use data_in_mod, only : r_t, r_s, r_u, r_v, r_dhdx, r_dhdy, r_q

  implicit none

#include <netcdf.inc>             ! netCDF defs

  !----- local  ------
  character(len=80)  :: fn     ! file name
  integer            :: i,j,n  ! generic loop indicies        
  integer            :: fid    ! nc domain file ID            
  integer            :: dimid  ! nc dimension id              
  integer            :: vid    ! nc variable ID               
  integer            :: rcode  ! nc return code               
  integer            :: ntim   ! temporary 

  character(len=*),parameter :: F01 = "('(domain) ',a20,a3,60(a1))"

!-----------------------------------------------------------------------
! PURPOSE:
!  get and/or set the model's domain info
!-----------------------------------------------------------------------

  if (domain_file(1:9) == 'fabricate') then

     !---------------------------------------------------------------
     ! fabricate grid data suitable for a code test
     !---------------------------------------------------------------
     write(6,9000) 'make bogus domain data'
     nx = domain_nx
     ny = domain_ny
     if (domain_nx == 0) call shr_sys_abort('fabricate data- must specify domain_nx')
     if (domain_ny == 0) call shr_sys_abort('fabricate data- must specify domain_ny')
     allocate(xc(nx,ny))
     allocate(yc(nx,ny))
     allocate(xv(4,nx,ny)) 
     allocate(yv(4,nx,ny)) 
     allocate(mask(nx,ny))
     allocate(area(nx,ny)) 

     write(6,9000) 'null data file => make bogus domain data'
     do j=1,ny
     do i=1,nx
        xv(1,i,j) = float(i-1)*360.0/float(nx)
        xv(2,i,j) = float(i  )*360.0/float(nx)
        xv(3,i,j) = xv(2,i,j)
        xv(4,i,j) = xv(1,i,j)
        xc(  i,j)=  (xv(1,i,j)+xv(2,i,j))*0.5
        
        yv(1,i,j) = -90.0 + float(j-1)*180.0/float(ny)
        yv(2,i,j) = yv(1,i,j)
        yv(3,i,j) = -90.0 + float(j  )*180.0/float(ny)
        yv(4,i,j) = yv(3,i,j)
        yc(  i,j)=  (yv(1,i,j)+yv(3,i,j))*0.5
        
        mask(i,j) = 1
        if (xv(2,i,j)<=120.AND.yv(4,i,j)<=30.AND.yv(1,i,j)>=-30) mask(i,j) = 0
     end do
     end do

  else 

     !---------------------------------------------------------------
     ! read in a domain data from an existing local file
     !---------------------------------------------------------------
     call getfil (trim(domain_file), data_dir, fn)
     write(6,9000) ' open netCDF file ',fn
     call wrap_open (fn, NF_NOWRITE, fid)

     write(6,9000) 'read domain data...'

     ! obtain dimensions
     call wrap_inq_dimid  (fid, 'ni' , dimid)
     call wrap_inq_dimlen (fid, dimid, nx   ) 
     call wrap_inq_dimid  (fid, 'nj' , dimid)
     call wrap_inq_dimlen (fid, dimid, ny   ) 
     call wrap_inq_dimid  (fid, 'nv' , dimid)
     call wrap_inq_dimlen (fid, dimid, nv   ) 

     ! obtain grid variables
     allocate(xc(nx,ny))
     allocate(yc(nx,ny))
     allocate(xv(nv,nx,ny))
     allocate(yv(nv,nx,ny))
     allocate(mask(nx,ny))
     allocate(area(nx,ny))

     call wrap_inq_varid(fid, 'xc' ,vid)
     call wrap_get_var_realx(fid, vid, xc)

     call wrap_inq_varid(fid, 'yc' ,vid)
     call wrap_get_var_realx(fid, vid, yc)

     call wrap_inq_varid(fid, 'xv' ,vid)
     call wrap_get_var_realx(fid, vid, xv)

     call wrap_inq_varid(fid, 'yv' ,vid)
     call wrap_get_var_realx(fid, vid, yv)

     call wrap_inq_varid(fid, 'mask', vid)
     call wrap_get_var_int(fid,vid,mask)
     
     call wrap_inq_varid(fid, 'area', vid)
     call wrap_get_var_realx(fid,vid,area)

     call wrap_close(fid)

  end if

  ! ---------------------------------------------------------------
  ! Allocate space for variables dependent on domain grid
  ! ---------------------------------------------------------------
  allocate (T   (nx,ny))
  allocate (s   (nx,ny))
  allocate (u   (nx,ny))
  allocate (v   (nx,ny))
  allocate (dhdx(nx,ny))
  allocate (dhdy(nx,ny))
  allocate (Q   (nx,ny))

  allocate (taux (nx,ny))
  allocate (tauy (nx,ny))
  allocate (netsw(nx,ny))
  allocate (lat  (nx,ny))
  allocate (sen  (nx,ny))
  allocate (lwup (nx,ny))
  allocate (lwdn (nx,ny))
  allocate (melth(nx,ny))
  allocate (salt (nx,ny))
  allocate (prec (nx,ny))
  allocate (evap (nx,ny))
  allocate (meltw(nx,ny))
  allocate (roff (nx,ny))
  allocate (ifrac(nx,ny)) 
  allocate (patm (nx,ny)) 

  allocate (buffs(nx,ny,nsnd))
  allocate (buffr(nx,ny,nrcv))

  if (data_form(1:6) == 'annual') then
     ntim = 12
  else if (data_form(1:9) == 'multiyear') then
     ntim = 2
  else
     call shr_sys_abort('data_form must be set to annual or multiyear')
  end if

  allocate (r_t   (nx,ny,ntim)) 
  allocate (r_s   (nx,ny,ntim)) 
  allocate (r_u   (nx,ny,ntim)) 
  allocate (r_v   (nx,ny,ntim)) 
  allocate (r_dhdx(nx,ny,ntim)) 
  allocate (r_dhdy(nx,ny,ntim)) 
  allocate (r_q   (nx,ny,ntim)) 

  ! ---------------------------------------------------------------
  ! Write diagnostic output
  ! ---------------------------------------------------------------
  write(6,9001) 'nx,ny = ',nx,ny
  write(6,9002) 'xc: ', (xc(  i,1),i=1,4), (xc(  i,1),i=nx-2,nx)
  write(6,9002) 'xv: ', (xv(1,i,1),i=1,4), (xv(2,i,1),i=nx-3,nx)
  write(6,9002) 'yc: ', (yc(  1,j),j=1,4), (yc(  1,j),j=ny-2,ny)
  write(6,9002) 'yv: ', (yv(1,1,j),j=1,4), (yv(4,1,j),j=ny-3,ny)
  
  RETURN
9000 format('(domain) ',3a)
9001 format('(domain) ',a,2i4)
9002 format('(domain) ',a,4f9.3,' ...',4f9.3)

END SUBROUTINE domain

!=======================================================================
