!===============================================================================
! CVS $Id: initial.F90,v 1.4 2001/05/18 20:19:43 mvertens Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ocn/docn5/initial.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

SUBROUTINE initial()

  use control_mod       ! control info, date/time
  use calendar_mod      ! calendar routines
  implicit none

  NAMELIST / inparm / & 
       rest_type   ,  &
       rest_pfile  ,  &
       rest_bfile  ,  &
       rest_freq   ,  &
       rest_date   ,  &
       rest_nday   ,  &
       rest_odate  ,  &
       rest_lag    ,  &
       case_name   ,  &
       case_desc   ,  &
       domain_file ,  &    
       domain_nx   ,  &
       domain_ny   ,  &   
       data_dir    ,  &
       data_file   ,  &    
       data_form   ,  &
       data_sstname,  &
       data_lonname,  &
       data_latname,  &
       mss_dir     ,  &
       mss_pass    ,  &
       mss_opts    ,  &
       mss_rtpd    ,  &
       mss_rmlf    ,  &
       info_dbug   ,  &
       info_time   ,  &
       info_sleep  ,  &
       ncpl        	  

  !----- local -----
  integer  :: i            ! generic loop indicies
  integer  :: ierr         ! error code	

  !-----------------------------------------------------------------------
  ! PURPOSE:
  !  o set default control variable values
  !  o read input parameter namelist file
  !-----------------------------------------------------------------------

  !-----------------------------------------------
  ! set initial namelist values
  !-----------------------------------------------
  write(6,'(a)') '(initial) setting namelist defaults.'
  rest_type     = 'continue'     ! run type: initial, cont, regen, branch
  rest_pfile    = 'null'         ! IC data pointer file
  rest_bfile    = 'null'         ! IC data file for branch runs
  rest_freq     = 'null'         ! restart file frequency
  rest_date     = 10101          ! start date for initial or branch runs
  rest_nday     = 365            ! restart file interval    (nday option
  rest_odate    = -1             ! restart file offset date (nday option)
  rest_lag      = 1              ! restart date: lag by 1 sync interval?
  case_desc     = ' '            ! case description 
  case_name     = ' '            ! case name
  domain_file   = 'null'         ! input domain file     
  domain_nx     = 0              ! assume nx will be read in from domain data
  domain_ny     = 0              ! assume ny will be read in from domain data
  data_dir      = 'null'         ! data: directory for input files
  data_file     = 'null'         ! input sst data file 
  data_form     = 'annual'       ! either "annual" or "multiyear"
  data_sstname  = 'T'            ! name of sst field on input data file
  data_lonname  = 'xc'           ! name of x dimension on input data file
  data_latname  = 'yc'           ! name of y dimension on input data file
  mss_dir       = 'null'         ! MSS file directory
  mss_pass      = ' '            ! MSS file password
  mss_opts      = 'nowait,nomail'! MSS file mswrite options
  mss_rtpd      = 180            ! MSS file MS retention period
  mss_rmlf      = 1              ! true    => remove local file after mswrite
  info_dbug     = 1              ! dbug level: 0   =lowest, 3  =highest
  info_time     = 0              ! true => print extra timing info
  info_sleep    = 0.0            ! simulate time to advance an active model
  ncpl          = 4              ! number of msg pairs per day

  !-----------------------------------------------
  ! read input namelist
  !-----------------------------------------------
  write(6,9000) 'reading input namelist.'
  read(5, inparm, iostat=ierr)
  if (ierr /= 0) then
     write(6,*)'namelist input resulted in error code ',ierr
     call shr_sys_abort()
  endif

  !-----------------------------------------------
  ! check that domain_file and data_file are set
  !-----------------------------------------------
  if (domain_file(1:4)=='null') then
     call shr_sys_abort ('must set input domain file name in namelist')
  endif
  if (data_file(1:4)=='null') then
     call shr_sys_abort ('must set input SST data file name in namelist')
  endif

  !-----------------------------------------------
  ! re-set some initial namelist values
  !-----------------------------------------------
  if (rest_freq(1:4) == 'null') rest_freq='never'
  if (rest_freq(1:5) /= 'never') then
     !--- must input a restart directory ---
     if (mss_dir(1:4)=='null') then
        call shr_sys_abort ('must input mss_dir in namelist')
        i = index(mss_dir,' ')-1
        if (mss_dir(i:i) /= '/') mss_dir(i+1:i+2)='/ '
     end if
     !--- must have a restart pointer file ---
     if (rest_pfile(1:4)=='null') then
        if (case_name(1:4)=='null') then
           call shr_sys_abort ('must input case in namelist')
        else
           i = index(case_name,' ')-1
           rest_pfile = '~/.ocn.'//case_name(1:i)//'.rpointer'
        end if
     end if
  end if

  !-----------------------------------------------
  ! record initial namelist values
  !-----------------------------------------------
  write(6,'(a)') '(initial) namelist values:'
  write(6,inparm)
  write(6,*) 

  !-----------------------------------------------
  ! set initial values for date & time
  !-----------------------------------------------
  call date2eday(rest_date,eday)
  call date2eday(rest_date,eday0)
  call date2ymd (rest_date,year,month,day)
  sec = 0

  !-----------------------------------------------
  ! inititial values for control flags
  !-----------------------------------------------
  ierr     = 0  ! error code
  stop_now = 0  ! T => stop now
  stop_eod = 0  ! T => stop at end-of-day
  rest_now = 0  ! T => create restart file now
  rest_eod = 0  ! T => create restart file at end-of-day
  hist_now = 0  ! T => create history file now
  hist_eod = 0  ! T => create history file at end-of-day
  hist_tav = 0  ! T => create monthly avg data
  diag_now = 0  ! T => create diagnostic data now
  diag_eod = 0  ! T => create diagnostic data at end-of-day

  RETURN
9000 format('(initial) ',a,a)
END SUBROUTINE initial

!=======================================================================

