!=======================================================================
! CVS $Id: interp2D.F90,v 1.1 2001/05/08 16:25:44 mvertens Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ocn/docn5/interp2D.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!=======================================================================

SUBROUTINE Interp2D(Xin ,Yin ,Ain ,mx,my,  &
                    Xout,Yout,Aout,nx,ny,spv,mFLAG)

  use kind_mod
  use shr_sys_mod

  implicit none

  !----- input -----
  integer  mx,my            ! input array dimensions
  real(r8) Xin (mx),Yin(my) ! input coordinate arrays
  real(r8) Ain (mx,my)      ! data on input grid
  real(r8) spv              ! denotes invalid data
  integer  nx,ny            ! output array dimensions
  real(r8) Xout(nx,ny)      ! output X coordinate array
  real(r8) Yout(nx,ny)      ! output Y coordinate array
  integer  mFLAG            ! selects extrapolation option

  !----- output -----
  real(r8) Aout(nx,ny)      ! data on output grid

  !----- local -----
  integer  i,j,ii,jj        ! loop indicies
  real(r8) a,b,c,d          ! ... as defined in algorithm below
  real(r8) dx,dy            ! ... as defined in algorithm below
  real(r8) alpha,beta       ! ... as defined in algorithm below
  real(r8) Ca,Cb,Cc,Cd      ! bilinear mapping weights 
  real(r8) XL,XM,XH         ! XL=Xin(ii) < XM=Xout(i,j) < XH=Xin(ii+1)
  real(r8) YL,YM,YH         ! YL=Yin(ii) < YM=Yout(i,j) < YH=Yin(ii+1)

!-------------------------------------------------------------------------------
!     PURPOSE:
!       map: Ain -> Aout
!       where Ain (i,j) has real world coordinates (Xin (i),Yin (j))
!       and   Aout(i,j) has real world coordinates (Xout(i,j),Yout(i,j))
!
!     NOTE:
!        mFLAG = 0 => map only if there are 4 valid values
!                     adjacent to the the output grid point
!        mFLAG = 1 => extrapolate if there is at least one valid value
!                     adjacent to the the output grid point
!        mFLAG = other => assume there are no special values
!
!     ASSUMPTION: 
!        Xin(1) =< Xout(i,j) =< Xin(mx), for all i,j
!        Yin(1) =< Yout(i,j) =< Yin(my), for all i,j
!
!     ALGORITHM:
!       for each i & j, 
!         find ii s.t. Xin(ii) < Xout(i,j) <= Xin(ii) and
!         find jj s.t. Yin(jj) < Yout(i,j) <= Yin(jj)
!       then 
!         a = F(Xin(ii  ),Yin(jj  ))
!         b = F(Xin(ii+1),Yin(jj  ))
!         c = F(Xin(ii+1),Yin(jj+1))
!         d = F(Xin(ii  ),Yin(jj+1))
!         dx = Xin(ii+1)-Xin(ii)
!         dy = Yin(jj+1)-Yin(jj)
!         alpha = (Xout(i,j)-Xin(ii))/dx
!         beta  = (Yout(i,j)-Yin(jj))/dy
!       with this info, interpolate to get the value
!         Aout(i,j) = F(Xout(i,j),Yout(i,j))
!
!-------------------------------------------------------------------------------
!           ^  d       c
!       b   |    
!       e  dy      f                interpolate f fromm a,b,c,d
!       t   |                       (avoid spval's)
!       a   v  a       b
!              <--dx--->
!               alpha
!-------------------------------------------------------------------------------

  IF (mFLAG==0) THEN
     DO j=1,ny
        DO i=1,nx
           jj=1
           DO WHILE (Yin(jj+1) < Yout(i,j))
              jj=jj+1
           END DO
           ii=1
           DO WHILE (Xin(ii+1) < Xout(i,j))
              ii=ii+1
           END DO
           a = Ain(ii  ,jj  )
           b = Ain(ii+1,jj  )
           c = Ain(ii+1,jj+1)
           d = Ain(ii  ,jj+1)
           IF (a==spv .or. b==spv .or. c==spv .or. d==spv) THEN
              Aout(i,j) = spv
           ELSE
              alpha = ( Xout(i,j)-Xin(ii) )/( Xin(ii+1)-Xin(ii) )
              beta  = ( Yout(i,j)-Yin(jj) )/( Yin(jj+1)-Yin(jj) )
              Ca = (1.-alpha)*(1.-beta)
              Cb = (   alpha)*(1.-beta)
              Cc = (   alpha)*(   beta)
              Cd = (1.-alpha)*(   beta)
              Aout(i,j) = Ca*a + Cb*b + Cc*c + Cd*d
           END IF
        END DO
     END DO
  ELSE IF (mFLAG==1) THEN
     DO j=1,ny
        DO i=1,nx
           jj=1
           DO WHILE (Yin(jj+1) < Yout(i,j))
              jj=jj+1
           END DO
           ii=1
           DO WHILE (Xin(ii+1) < Xout(i,j))
              ii=ii+1
           END DO
           a = Ain(ii  ,jj  )
           b = Ain(ii+1,jj  )
           c = Ain(ii+1,jj+1)
           d = Ain(ii  ,jj+1)
           XL = Xin (ii  )
           XM = Xout(i,j )
           XH = Xin (ii+1)
           YL = Yin (jj  )
           YM = Yout(i,j )
           YH = Yin (jj+1)
           call IntrpBox(a,b,c,d, XL,XM,XH,YL,YM,YH, spv, Ca,Cb,Cc,Cd)
           Aout(i,j) = Ca*a + Cb*b + Cc*c + Cd*d
        END DO
     END DO
  ELSE
     DO j=1,ny
        DO i=1,nx
           jj=1
           DO WHILE (Yin(jj+1) < Yout(i,j))
              jj=jj+1
           END DO
           ii=1
           DO WHILE (Xin(ii+1) < Xout(i,j))
              ii=ii+1
           END DO
           a = Ain(ii  ,jj  )
           b = Ain(ii+1,jj  )
           c = Ain(ii+1,jj+1)
           d = Ain(ii  ,jj+1)
           alpha = ( Xout(i,j)-Xin(ii) )/( Xin(ii+1)-Xin(ii) )
           beta  = ( Yout(i,j)-Yin(jj) )/( Yin(jj+1)-Yin(jj) )
           Ca = (1.-alpha)*(1.-beta)
           Cb = (   alpha)*(1.-beta)
           Cc = (   alpha)*(   beta)
           Cd = (1.-alpha)*(   beta)
           Aout(i,j) = Ca*a + Cb*b + Cc*c + Cd*d
        END DO
     END DO
  END IF

  RETURN
END SUBROUTINE Interp2D

!===============================================================================

SUBROUTINE IntrpBox(a,b,c,d, XL,XM,XH, YL,YM,YH, spv, Ca,Cb,Cc,Cd)

  use kind_mod
  use shr_sys_mod

  implicit none

  !----- input -----  
  real(r8) a,b,c,d
  real(r8) XL,XM,XH
  real(r8) YL,YM,YH
  real(r8) spv

  !----- output -----  
  real(r8) Ca,Cb,Cc,Cd

  !----- local -----  
  integer  ID
  real(r8) dy,dx
  real(r8) alpha ,beta
  real(r8) alpha2,beta2

!-------------------------------------------------------------------------------
!           ^  d       c
!       b   |    
!       e  dy      f                interpolate f from a,b,c,d
!       t   |                       (avoiding special values)
!       a   v  a       b
!              <--dx--->
!               alpha
!-------------------------------------------------------------------------------
!     PURPOSE:  
!        given XL<=XM<=XH
!              YL<=YM<=YH
!              a=F(XL,YL)
!              b=F(XH,YL)
!              c=F(XH,YH)
!              d=F(XL,YH)
!        find Ca,Cb,Cc,Cd to get an interpolated value for F(XM,YM)
!             F(XM,YM) = Ca*a + Cb*b + Cc*c + Cd*d
!     INPUT:  
!        a,b,c,d,
!        XL,XM,XH
!        YL,YM,YH
!        spv
!     OUTPUT: 
!        Ca,Cb,Cc,Cd
!-------------------------------------------------------------------------------

  ID = 0
  IF (a /= spv) ID=ID+1
  IF (b /= spv) ID=ID+2
  IF (c /= spv) ID=ID+4
  IF (d /= spv) ID=ID+8

  Ca=0.0
  Cb=0.0
  Cc=0.0
  Cd=0.0

  !     IF (ID > 2) THEN
  dx    =  XH-XL
  dy    =  YH-YL
  alpha = (XM-XL)/dx
  beta  = (YM-YL)/dy
  !     END IF

  !--- interpolate with four points --------------------------------
  IF (ID == 15) THEN
     Ca = (1.-alpha)*(1.-beta)
     Cb = (   alpha)*(1.-beta)
     Cc = (   alpha)*(   beta)
     Cd = (1.-alpha)*(   beta)
     !--- interpolate with no points ----------------------------------
  ELSE IF (ID == 0) THEN
     Ca=1.0
     !--- interpolate with one point ----------------------------------
  ELSE IF (ID == 1) THEN
     Ca=1.0
  ELSE IF (ID == 2) THEN
     Cb=1.0
  ELSE IF (ID == 4) THEN
     Cc=1.0
  ELSE IF (ID == 8) THEN
     Cd=1.0
     !--- interpolate with two adjacent points ------------------------
  ELSE IF (ID ==  3) THEN
     Ca=1.-alpha
     Cb=alpha
  ELSE IF (ID == 12) THEN
     Cd=1.-alpha
     Cc=alpha
  ELSE IF (ID ==  6) THEN
     Cb = 1.-beta
     Cc = beta
  ELSE IF (ID ==  9) THEN
     Ca = 1.-beta 
     Cd = beta
     !--- interpolate with two opposite points ------------------------
  ELSE IF (ID == 5) THEN
     alpha2 = sqrt((alpha*dx)**2+(beta*dy)**2)/sqrt(dx**2+dy**2)
     Ca = 1.-alpha2
     Cc = alpha2
  ELSE IF (ID == 10) THEN
     alpha2= sqrt(((1.-alpha)*dx)**2+(beta*dy)**2)/sqrt(dx**2+dy**2)
     Cb = 1.-alpha2 
     Cd = alpha2
     !--- interpolate with three points -------------------------------
  ELSE IF (ID == 11) THEN
     Ca = 1. - alpha - beta
     Cb = alpha
     Cd = beta
  ELSE IF (ID == 7) THEN
     Ca = 1. - alpha
     Cb = alpha -beta
     Cc = beta
  ELSE IF (ID == 13) THEN
     Ca = 1. - beta
     Cc = alpha
     Cd = -alpha + beta
  ELSE IF (ID == 14) THEN
     Cc = alpha + beta -1.
     Cb = 1. - beta
     Cd = 1. - alpha
  ELSE
     write(6,*) '(InterpBox) FATAL ERROR'
     STOP 'IntrpBx'
  END IF

  RETURN
END SUBROUTINE IntrpBox

!===============================================================================
