!===============================================================================
! CVS $Id: msg.F90,v 1.5 2001/05/08 16:25:45 mvertens Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ocn/docn5/msg.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

SUBROUTINE msg_pass(cOption)

  use shr_msg_mod   ! shared code: msg passing
  use shr_sys_mod   ! shared code: system calls
  use shr_timer_mod ! shared code: timing routines
  use buf_mod       ! allocatables and parameters
  use control_mod   ! control info
  use constant_mod  ! constants

  implicit none

  !----- arguments -----
  character*(*) cOption

  !----- local -----
  integer,save:: timer_n           ! timer number
  integer,save:: timer_m           ! timer number
  integer     :: i,j               ! generic loop indicies
  integer(8)  :: irtc_w            ! rtc tics when waiting for msg
  integer(8)  :: irtc_r            ! rtc tics when msg received
  integer(8)  :: irtc_u            ! rtc tics when msg unpacked
  integer(8)  :: irtc_p            ! rtc tics when packing msg
  integer(8)  :: irtc_s            ! rtc tics when sending msg 
  integer     :: ierr              ! error code	

!-------------------------------------------------------------------------------
! PURPOSE:
!   implement various message passing tasks using the MPI library
!
! FUCTIONALITY:
!   cOption=
!     'connect'    => connect to the msg passing system
!     'init'       => recv/send initial info
!     'send'       => send model state & output fluxes
!     'recv'       => receive input fluxes & stop/continue flags
!     'disconnect' => disconnect from the msg passing system
!-------------------------------------------------------------------------------

  IF (cOption == 'connect') THEN 
     !-----------------------------------------------------------------------
     ! connect to msg passing system
     !-----------------------------------------------------------------------
     write(6,9000) '(connect) connecting to cpl'
     call shr_msg_init('ocn')
     call shr_msg_groups('ocn')
     call shr_sys_flush(6)

     call shr_timer_get(timer_n,'msg: ocn  -> cpl')
     call shr_timer_get(timer_m,'msg: ocn <-  cpl')

  ELSE IF (cOption == 'init') THEN 
     !-----------------------------------------------------------------------
     ! recv/send initial data
     !-----------------------------------------------------------------------
     write(6,9000) '(initial) recv msg'

     call shr_msg_recv_i(ibuffr,nibuff   ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2OI)

     ierr      = ibuffr( 1) ! error code
     cdate     = ibuffr( 4) ! current date
     sec       = ibuffr( 5) ! elapsed sec on current date
     info_time = ibuffr(11) ! T => turn on msg-passing timing
     !nfo_dbug = ibuffr(12) ! cpl debug level

     !--- send grid & domain mask ---
     write(6,9000) '(initial) send msg'

     ibuffs    =0       ! unused
     ibuffs( 1)=ierr    ! error code
     ibuffs( 7)=nx      ! number of grid points in x-direction
     ibuffs( 8)=ny      ! number of grid points in y-direction
     ibuffs( 9)=ncpl    ! number of msg-pairs per day

     call shr_msg_send_i(ibuffs,nibuff   ,SHR_MSG_TID_CPL,SHR_MSG_TAG_O2CI)
     call shr_msg_send_r(xc    ,nx*ny    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_O2CI)
     call shr_msg_send_r(yc    ,nx*ny    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_O2CI)
     call shr_msg_send_r(xv    ,nx*ny*4  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_O2CI)
     call shr_msg_send_r(yv    ,nx*ny*4  ,SHR_MSG_TID_CPL,SHR_MSG_TAG_O2CI)
     call shr_msg_send_r(area  ,nx*ny    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_O2CI)
     call shr_msg_send_i(mask  ,nx*ny    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_O2CI)

  ELSE IF (cOption == 'send') THEN 
     !-----------------------------------------------------------------------
     ! send output field msg
     !-----------------------------------------------------------------------
     if (info_dbug>=3) write(6,9000) '(send) send data to cpl'
     call shr_timer_start(timer_n)

     irtc_p = shr_sys_irtc()   ! packing msg

     !--- set info buffer flags ---
     ibuffs     = 0     ! unused
     ibuffs( 1) = ierr  ! error code
     ibuffs( 3) = 0     ! stop flag: 0 <=> able to continue
     ibuffs( 4) = cdate ! model date, coded: yyyymmdd
     ibuffs( 5) = sec   ! elapsed seconds on model date
     ibuffs( 7) = nx    ! number of grid points in x-direction
     ibuffs( 8) = ny    ! number of grid points in y-direction
     ibuffs( 9) = ncpl  ! number of msg-pairs per day
     ibuffs(31) = 0     ! precip adjustment factor (* 1.0e+6)

     do j=1,ny
        do i=1,nx
           buffs(i,j, 1) = T    (i,j)  ! temperature
           buffs(i,j, 2) = S    (i,j)  ! salinity
           buffs(i,j, 3) = U    (i,j)  ! velocity, zonal
           buffs(i,j, 4) = V    (i,j)  ! velocity, meridional
           buffs(i,j, 5) = dhdx (i,j)  ! surface slope, zonal
           buffs(i,j, 6) = dhdy (i,j)  ! surface slope, meridional
           buffs(i,j, 7) = q    (i,j)  ! heat of fusion xor melt pot
        end do
     end do

     irtc_s = shr_sys_irtc()   ! sending msg

     call shr_msg_send_i(ibuffs,nibuff    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_O2C)
     call shr_msg_send_r( buffs,nx*ny*nsnd,SHR_MSG_TID_CPL,SHR_MSG_TAG_O2C)
     call shr_timer_stop(timer_n)

     if (info_time/=0) then
        write(6,9099) irtc_p,'o->c packing'
        write(6,9099) irtc_s,'o->c sending'
     end if

  ELSE IF (cOption == 'recv') THEN
     !-----------------------------------------------------------------------
     ! recv input field msg
     !-----------------------------------------------------------------------
     if (info_dbug>=3) write(6,9000) '(recv) recv data to cpl'
     call shr_timer_start(timer_m)

     irtc_w = shr_sys_irtc()   ! waiting for msg
     call shr_msg_recv_i(ibuffr,nibuff    ,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2O)
     irtc_r = shr_sys_irtc()   ! received msg
     call shr_msg_recv_r( buffr,nx*ny*nrcv,SHR_MSG_TID_CPL,SHR_MSG_TAG_C2O)

     do j=1,ny
        do i=1,nx
           !--- states ---
           ifrac(i,j) = buffr(i,j, 1) ! ice fraction
           patm (i,j) = buffr(i,j, 2) ! atm sea level pressure
           !--- fluxes ---
           taux (i,j) = buffr(i,j, 3) ! surface stress, zonal
           tauy (i,j) = buffr(i,j, 4) ! surface stress, merid
           netsw(i,j) = buffr(i,j, 5) ! net sw rad
           ! lat  (i,j) = buffr(i,j, -) ! latent (derive from evap)
           sen  (i,j) = buffr(i,j, 6) ! sensible
           lwup (i,j) = buffr(i,j, 7) ! long-wave up
           lwdn (i,j) = buffr(i,j, 8) ! long-wave down
           melth(i,j) = buffr(i,j, 9) ! melt heat
           salt (i,j) = buffr(i,j,10) ! salinity flux
           prec (i,j) = buffr(i,j,11) ! precip
           evap (i,j) = buffr(i,j,12) ! evaporation
           meltw(i,j) = buffr(i,j,13) ! melt water
           roff (i,j) = buffr(i,j,14) ! runoff
        end do
     end do
     lat = LATVAP*evap

     irtc_u = shr_sys_irtc()   ! unpacked msg

     ierr      = ibuffr( 1) ! cpl error code (0 => no error)
     stop_eod  = ibuffr( 2) ! T => stop at end-of-day
     stop_now  = ibuffr( 3) ! T => stop now
     !           ibuffr( 4) ! current date
     !           ibuffr( 5) ! elapsed sec on current date
     !           ibuffr(11) ! T => turn on msg-passing timing
     !           ibuffr(12) ! cpl debug level
     rest_eod  = ibuffr(21) ! T => create restart data at end-of-day
     rest_now  = ibuffr(22) ! T => create restart data now
     hist_eod  = ibuffr(23) ! T => create history data at end-of-day
     hist_now  = ibuffr(24) ! T => create history data now
     hist_tav  = ibuffr(25) ! T => create create monthly avg data
     diag_eod  = ibuffr(26) ! T => create diagnotic data at eod
     diag_now  = ibuffr(27) ! T => create diagnotic data now

     call shr_timer_stop(timer_m)
     if (info_time/=0 .and. stop_now==0) then
        write(6,9099) irtc_w,'c->o waiting'
        write(6,9099) irtc_r,'c->o received'
        write(6,9099) irtc_u,'c->o unpacked'
     end if

  ELSE IF (cOption == 'disconnect') THEN 
     !-----------------------------------------------------------------------
     ! disconnect from msg passing
     !-----------------------------------------------------------------------
     write(6,9000) '(disconnect) disconnect from cpl'
     call shr_msg_finalize

  ELSE 
     !-----------------------------------------------------------------------
     ! this should never happen
     !-----------------------------------------------------------------------
     write(6,9000) 'ERROR: invalid cOption: ',cOption
     stop
  END IF

  RETURN
9000 format('(msg_pass) ',3a)
9099 format('[mp timing]  irtc = ',i20,' ',a)
END SUBROUTINE msg_pass

!===============================================================================





