!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module boundary

!***********************************************************************
!
!     this module contains routines for updating ghost cell boundaries
!     using MPI calls
!
!     CVS:$Id: boundary.F,v 1.2.2.1 2001/03/13 22:54:24 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use domain
      use communicate

      implicit none

      interface boundary_2d
        module procedure boundary_2d_real,
     &                   boundary_2d_int
      end interface

!***********************************************************************

      contains

!***********************************************************************

      subroutine boundary_2d_real(ARRAY)

      implicit none

!-----------------------------------------------------------------------
!
!     updates ghost cells for a 2-d array
!
!-----------------------------------------------------------------------

      include 'mpif.h'   ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     intent(inout)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(inout) :: 
     &   ARRAY              ! 2-d array to update

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &   istrt, jstrt,              ! starting address for messages
     &   ierr,                      ! MPI error flag
     &   request(4),                ! MPI request and status ids
     &   status(MPI_STATUS_SIZE,4)

!-----------------------------------------------------------------------
!
!     post receives for east-west boundary update
!
!-----------------------------------------------------------------------

      call MPI_IRECV(ARRAY(1,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_wshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(ARRAY(iphys_e+1,1), 1, mpi_ew_type, nbr_east,
     &               mpitag_eshift, MPI_COMM_OCN, request(4), ierr)

!-----------------------------------------------------------------------
!
!     send east-west boundary info
!
!-----------------------------------------------------------------------

      istrt = iphys_e +1 - num_ghost_cells
      call MPI_ISEND(ARRAY(istrt  ,1), 1, mpi_ew_type, nbr_east,
     &               mpitag_wshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(ARRAY(iphys_b,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_eshift, MPI_COMM_OCN, request(2), ierr)

!-----------------------------------------------------------------------
!
!     wait for completion before next stage
!
!-----------------------------------------------------------------------

      call MPI_WAITALL(4, request, status, ierr)

!-----------------------------------------------------------------------
!
!     exchange north-south boundary info 
!
!-----------------------------------------------------------------------

      call MPI_IRECV(ARRAY(1,jphys_e+1), 1, mpi_ns_type, nbr_north,
     &               mpitag_nshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(ARRAY(1,1), 1, mpi_ns_type, nbr_south,
     &               mpitag_sshift, MPI_COMM_OCN, request(4), ierr)

      jstrt = jphys_e + 1 - num_ghost_cells
      call MPI_ISEND(ARRAY(1,jphys_b), 1, mpi_ns_type, nbr_south,
     &               mpitag_nshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(ARRAY(1,jstrt  ), 1, mpi_ns_type, nbr_north,
     &               mpitag_sshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_WAITALL(4, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine boundary_2d_real

!***********************************************************************

      subroutine boundary_2d_int(ARRAY)

      implicit none

!-----------------------------------------------------------------------
!
!     updates ghost cells for a 2-d array
!
!-----------------------------------------------------------------------

      include 'mpif.h'   ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     intent(inout)
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(inout) :: 
     &   ARRAY                 ! 2-d array to update

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &   istrt, jstrt,              ! starting address for messages
     &   ierr,                      ! MPI error flag
     &   request(4),                ! MPI request and status ids
     &   status(MPI_STATUS_SIZE,4)

!-----------------------------------------------------------------------
!
!     post receives for east-west boundary update
!
!-----------------------------------------------------------------------

      call MPI_IRECV(ARRAY(1,1), 1, mpi_ew_type_int, nbr_west,
     &               mpitag_wshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(ARRAY(iphys_e+1,1), 1, mpi_ew_type_int, nbr_east,
     &               mpitag_eshift, MPI_COMM_OCN, request(4), ierr)

!-----------------------------------------------------------------------
!
!     send east-west boundary info
!
!-----------------------------------------------------------------------

      istrt = iphys_e +1 - num_ghost_cells
      call MPI_ISEND(ARRAY(istrt  ,1), 1, mpi_ew_type_int, nbr_east,
     &               mpitag_wshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(ARRAY(iphys_b,1), 1, mpi_ew_type_int, nbr_west,
     &               mpitag_eshift, MPI_COMM_OCN, request(2), ierr)

!-----------------------------------------------------------------------
!
!     wait for completion before next stage
!
!-----------------------------------------------------------------------

      call MPI_WAITALL(4, request, status, ierr)

!-----------------------------------------------------------------------
!
!     exchange north-south boundary info 
!
!-----------------------------------------------------------------------

      call MPI_IRECV(ARRAY(1,jphys_e+1), 1, mpi_ns_type_int, nbr_north,
     &               mpitag_nshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(ARRAY(1,1), 1, mpi_ns_type_int, nbr_south,
     &               mpitag_sshift, MPI_COMM_OCN, request(4), ierr)

      jstrt = jphys_e + 1 - num_ghost_cells
      call MPI_ISEND(ARRAY(1,jphys_b), 1, mpi_ns_type_int, nbr_south,
     &               mpitag_nshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(ARRAY(1,jstrt  ), 1, mpi_ns_type_int, nbr_north,
     &               mpitag_sshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_WAITALL(4, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine boundary_2d_int

!***********************************************************************

      subroutine boundary_baroclinic(ZX, ZY)

      implicit none

!-----------------------------------------------------------------------
!
!     updates ghost cells for baroclinic variables
!     included in special routine to buffer messages
!
!-----------------------------------------------------------------------

      include 'mpif.h'   ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     intent(inout)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(inout) :: 
     &  ZX, ZY   ! barotropic forcing terms

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i, j, n,                   ! dummy counters 
     &   ierr,                      ! MPI error flag
     &   request(4),                ! MPI request ids
     &   status(MPI_STATUS_SIZE,4)  ! MPI status flags

      integer (kind=int_kind), parameter ::  ! buffer sizes 
     &   buf_len_ew = (jmt-2*num_ghost_cells)*2*num_ghost_cells,
     &   buf_len_ns = imt*2*num_ghost_cells

      real (kind=dbl_kind), dimension(buf_len_ns) :: 
     &   buffer_north_snd,      ! message buffers for north/south
     &   buffer_north_rcv,      ! boundary updates
     &   buffer_south_snd,
     &   buffer_south_rcv

      real (kind=dbl_kind), dimension(buf_len_ew) :: 
     &   buffer_east_snd,       ! message buffers for east/west
     &   buffer_east_rcv,       ! boundary updates
     &   buffer_west_snd,
     &   buffer_west_rcv

!-----------------------------------------------------------------------
!
!     post receives to update east-west boundaries
!
!-----------------------------------------------------------------------

      call MPI_IRECV(buffer_west_rcv, 
     &               buf_len_ew, MPI_DOUBLE_PRECISION, nbr_west,
     &               mpitag_wshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(buffer_east_rcv, 
     &               buf_len_ew, MPI_DOUBLE_PRECISION, nbr_east,
     &               mpitag_eshift, MPI_COMM_OCN, request(4), ierr)

!-----------------------------------------------------------------------
!
!     fill buffers and send east-west boundary info
!
!-----------------------------------------------------------------------

      i = 1
      do j=jphys_b,jphys_e
        do n=1,num_ghost_cells
          buffer_east_snd(i  ) = ZX(iphys_e + n - num_ghost_cells, j)
          buffer_east_snd(i+1) = ZY(iphys_e + n - num_ghost_cells, j)

          buffer_west_snd(i  ) = ZX(iphys_b+n-1,j)
          buffer_west_snd(i+1) = ZY(iphys_b+n-1,j)

          i = i+2
        enddo
      enddo

      call MPI_ISEND(buffer_east_snd, 
     &               buf_len_ew, MPI_DOUBLE_PRECISION, nbr_east,
     &               mpitag_wshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(buffer_west_snd, 
     &               buf_len_ew, MPI_DOUBLE_PRECISION, nbr_west,
     &               mpitag_eshift, MPI_COMM_OCN, request(2), ierr)

!-----------------------------------------------------------------------
!
!     wait for message completion and copy buffers into ghost cells
!
!-----------------------------------------------------------------------

      call MPI_WAITALL(4, request, status, ierr)

      i = 1
      do j=jphys_b,jphys_e
        do n=1,num_ghost_cells
          ZX(n,j)   = buffer_west_rcv(i  )
          ZY(n,j)   = buffer_west_rcv(i+1)

          ZX(iphys_e+n,j) = buffer_east_rcv(i  )
          ZY(iphys_e+n,j) = buffer_east_rcv(i+1)

          i = i+2
        enddo
      enddo

!-----------------------------------------------------------------------
!
!     post receives to update north-south boundaries
!
!-----------------------------------------------------------------------

      call MPI_IRECV(buffer_north_rcv, buf_len_ns,
     &               MPI_DOUBLE_PRECISION, nbr_north, 
     &               mpitag_nshift, MPI_COMM_OCN, request(3), ierr)

      call MPI_IRECV(buffer_south_rcv, buf_len_ns,
     &               MPI_DOUBLE_PRECISION, nbr_south, 
     &               mpitag_sshift, MPI_COMM_OCN, request(4), ierr)

!-----------------------------------------------------------------------
!
!     fill buffers and send north-south boundary info
!
!-----------------------------------------------------------------------

      j = 1
      do n=1,num_ghost_cells
        do i=1,imt
          buffer_north_snd(j  ) = ZX(i,jphys_e + n - num_ghost_cells)
          buffer_north_snd(j+1) = ZY(i,jphys_e + n - num_ghost_cells)

          buffer_south_snd(j  ) = ZX(i,jphys_b + n - 1)
          buffer_south_snd(j+1) = ZY(i,jphys_b + n - 1)

          j = j+2
        enddo
      enddo

      call MPI_ISEND(buffer_north_snd, buf_len_ns,
     &               MPI_DOUBLE_PRECISION, nbr_north, 
     &               mpitag_sshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_ISEND(buffer_south_snd, buf_len_ns,
     &               MPI_DOUBLE_PRECISION, nbr_south, 
     &               mpitag_nshift, MPI_COMM_OCN, request(2), ierr)

!-----------------------------------------------------------------------
!
!     wait for message completion and copy buffers into ghost cells
!
!-----------------------------------------------------------------------

      call MPI_WAITALL(4, request, status, ierr)

      j = 1
      do n=1,num_ghost_cells
        do i=1,imt
          ZX(i,n)   = buffer_south_rcv(j  )
          ZY(i,n)   = buffer_south_rcv(j+1)

          ZX(i,jphys_e + n) = buffer_north_rcv(j  ) 
          ZY(i,jphys_e + n) = buffer_north_rcv(j+1) 

          j = j+2
        enddo
      enddo

!-----------------------------------------------------------------------

      end subroutine boundary_baroclinic

!***********************************************************************

      subroutine boundary_barotropic(SUA, SVA, UA, VA, RHOA, TA)

      implicit none

!-----------------------------------------------------------------------
!
!     exchanges barotropic boundary data
!     included in special routine to buffer messages
!
!-----------------------------------------------------------------------

      include 'mpif.h'   ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     intent(inout)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(inout) :: 
     &   SUA, SVA           ! barotropic velocities at new time

      real (kind=dbl_kind), dimension(imt,jmt,km), intent(inout) :: 
     &   UA, VA,            ! total velocities at new time
     &   RHOA               ! density at new time

      real (kind=dbl_kind), dimension(imt,jmt,km,nt), intent(inout) :: 
     &   TA                 ! tracers at new time

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter ::  ! msg buffer sizes 
     &   bufsize_ns = ((nt+3)*km+2)*imt*num_ghost_cells,
     &   bufsize_ew = ((nt+3)*km+2)*(jmt-2*num_ghost_cells)*
     &                                    num_ghost_cells

      real (kind=dbl_kind), dimension(bufsize_ns) ::
     &   buffer_north_snd,       ! buffers for north/south updates
     &   buffer_north_rcv,
     &   buffer_south_snd,
     &   buffer_south_rcv

      real (kind=dbl_kind), dimension(bufsize_ew) ::
     &   buffer_east_snd,        ! buffers for east/west updates
     &   buffer_east_rcv,
     &   buffer_west_snd,
     &   buffer_west_rcv

      integer (kind=int_kind) :: 
     &   i, j, k, n,              ! dummy counters
     &   istrt, jstrt,            ! addresses for ghost cell location
     &   index,                   ! current buffer location
     &   ierr,                    ! MPI error flag
     &   request(4),              ! MPI request ids
     &   status(MPI_STATUS_SIZE), ! MPI status flags
     &   status_wait(MPI_STATUS_SIZE,4)

!-----------------------------------------------------------------------
!
!     post the receives for exchanging east-west boundary info
!
!-----------------------------------------------------------------------

      call MPI_IRECV(buffer_west_rcv, bufsize_ew, 
     &               MPI_DOUBLE_PRECISION, nbr_west , 
     &               mpitag_wshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_IRECV(buffer_east_rcv, bufsize_ew,
     &               MPI_DOUBLE_PRECISION, nbr_east, 
     &               mpitag_eshift, MPI_COMM_OCN, request(4), ierr)

!-----------------------------------------------------------------------
!
!     pack the east-west boundary buffers
!
!-----------------------------------------------------------------------

      istrt = iphys_e - num_ghost_cells
      index = 1
      do j=jphys_b,jphys_e
        do i=1,num_ghost_cells
          buffer_east_snd(index  ) = SUA(istrt+i,j)
          buffer_east_snd(index+1) = SVA(istrt+i,j)

          buffer_west_snd(index  ) = SUA(iphys_b+i-1,j)
          buffer_west_snd(index+1) = SVA(iphys_b+i-1,j)

          index = index + 2
        enddo
      enddo

      do k=1,km
        do j=jphys_b,jphys_e
          do i=1,num_ghost_cells
            buffer_east_snd(index  ) = UA  (istrt+i,j,k)
            buffer_east_snd(index+1) = VA  (istrt+i,j,k)
            buffer_east_snd(index+2) = RHOA(istrt+i,j,k)

            buffer_west_snd(index  ) = UA  (iphys_b+i-1,j,k)
            buffer_west_snd(index+1) = VA  (iphys_b+i-1,j,k)
            buffer_west_snd(index+2) = RHOA(iphys_b+i-1,j,k)

            index = index + 3
          enddo
        enddo
      enddo

      do n=1,nt
        do k=1,km
          do j=jphys_b,jphys_e
            do i=1,num_ghost_cells
              buffer_east_snd(index) = TA(istrt+i,j,k,n)
 
              buffer_west_snd(index) = TA(iphys_b+i-1,j,k,n)

              index = index + 1
            enddo
          enddo
        enddo
      enddo

!-----------------------------------------------------------------------
!
!     send the east-west boundary buffers and wait for completion
!     before unpacking buffers.
!
!-----------------------------------------------------------------------

      call MPI_ISEND(buffer_east_snd, bufsize_ew, 
     &               MPI_DOUBLE_PRECISION, nbr_east, 
     &               mpitag_wshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_ISEND(buffer_west_snd , bufsize_ew,
     &               MPI_DOUBLE_PRECISION, nbr_west , 
     &               mpitag_eshift, MPI_COMM_OCN, request(3),ierr)

      call MPI_WAITALL(4, request, status_wait, ierr)

      index = 1
      do j=jphys_b,jphys_e
        do i=1,num_ghost_cells
          SUA(i,j)   = buffer_west_rcv(index)
          SVA(i,j)   = buffer_west_rcv(index+1)

          SUA(iphys_e+i,j) = buffer_east_rcv(index)
          SVA(iphys_e+i,j) = buffer_east_rcv(index+1)

          index = index + 2
        enddo
      enddo

      do k=1,km
        do j=jphys_b,jphys_e
          do i=1,num_ghost_cells
            UA  (i,j,k)   = buffer_west_rcv(index)
            VA  (i,j,k)   = buffer_west_rcv(index+1)
            RHOA(i,j,k)   = buffer_west_rcv(index+2)

            UA  (iphys_e+i,j,k) = buffer_east_rcv(index)
            VA  (iphys_e+i,j,k) = buffer_east_rcv(index+1)
            RHOA(iphys_e+i,j,k) = buffer_east_rcv(index+2)

            index = index + 3
          enddo
        enddo
      enddo

      do n=1,nt
        do k=1,km
          do j=jphys_b,jphys_e
            do i=1,num_ghost_cells
              TA(i  ,j,k,n) = buffer_west_rcv(index)

              TA(iphys_e+i,j,k,n) = buffer_east_rcv(index)

              index = index + 1
            enddo
          enddo
        enddo
      enddo

!-----------------------------------------------------------------------
!
!     post the receives for exchanging north-south boundary info
!
!-----------------------------------------------------------------------

      call MPI_IRECV(buffer_south_rcv, bufsize_ns,
     &               MPI_DOUBLE_PRECISION, nbr_south, 
     &               mpitag_sshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_IRECV(buffer_north_rcv, bufsize_ns,
     &               MPI_DOUBLE_PRECISION, nbr_north, 
     &               mpitag_nshift, MPI_COMM_OCN, request(4), ierr)

!-----------------------------------------------------------------------
!
!     pack the north-south boundary buffers
!
!-----------------------------------------------------------------------

      jstrt = jphys_e - num_ghost_cells
      index = 1
      do j=1,num_ghost_cells
        do i=1,imt
          buffer_north_snd(index  ) = SUA(i,jstrt+j)
          buffer_north_snd(index+1) = SVA(i,jstrt+j)

          buffer_south_snd(index  ) = SUA(i,jphys_b-1+j)
          buffer_south_snd(index+1) = SVA(i,jphys_b-1+j)

          index = index + 2
        enddo
      enddo

      do k=1,km
        do j=1,num_ghost_cells
          do i=1,imt
            buffer_north_snd(index  ) = UA  (i,jstrt+j,k)
            buffer_north_snd(index+1) = VA  (i,jstrt+j,k)
            buffer_north_snd(index+2) = RHOA(i,jstrt+j,k)

            buffer_south_snd(index  ) = UA  (i,jphys_b-1+j,k)
            buffer_south_snd(index+1) = VA  (i,jphys_b-1+j,k)
            buffer_south_snd(index+2) = RHOA(i,jphys_b-1+j,k)

            index = index + 3
          enddo
        enddo
      enddo

      do n=1,nt
        do k=1,km
          do j=1,num_ghost_cells
            do i=1,imt
              buffer_north_snd(index) = TA(i,jstrt+j,k,n)
 
              buffer_south_snd(index) = TA(i,jphys_b-1+j,k,n)

              index = index + 1
            enddo
          enddo
        enddo
      enddo

!-----------------------------------------------------------------------
!
!     send the north-south boundary buffers and wait for completion
!     before unpacking buffers.
!
!-----------------------------------------------------------------------

      call MPI_ISEND(buffer_north_snd, bufsize_ns,
     &               MPI_DOUBLE_PRECISION, nbr_north, 
     &               mpitag_sshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_ISEND(buffer_south_snd, bufsize_ns,
     &               MPI_DOUBLE_PRECISION, nbr_south, 
     &               mpitag_nshift, MPI_COMM_OCN, request(3), ierr)

      call MPI_WAITALL(4, request, status_wait, ierr)

      index = 1
      do j=1,num_ghost_cells
        do i=1,imt
          SUA(i,j)   = buffer_south_rcv(index)
          SVA(i,j)   = buffer_south_rcv(index+1)

          SUA(i,jphys_e+j) = buffer_north_rcv(index)
          SVA(i,jphys_e+j) = buffer_north_rcv(index+1)

          index = index + 2
        enddo
      enddo

      do k=1,km
        do j=1,num_ghost_cells
          do i=1,imt
            UA  (i,j,k) = buffer_south_rcv(index)
            VA  (i,j,k) = buffer_south_rcv(index+1)
            RHOA(i,j,k) = buffer_south_rcv(index+2)

            UA  (i,jphys_e+j,k) = buffer_north_rcv(index)
            VA  (i,jphys_e+j,k) = buffer_north_rcv(index+1)
            RHOA(i,jphys_e+j,k) = buffer_north_rcv(index+2)

            index = index + 3
          enddo
        enddo
      enddo

      do n=1,nt
        do k=1,km
          do j=1,num_ghost_cells
            do i=1,imt
              TA(i,j,k,n) = buffer_south_rcv(index)

              TA(i,jphys_e+j,k,n) = buffer_north_rcv(index)

              index = index + 1
            enddo
          enddo
        enddo
      enddo

!-----------------------------------------------------------------------

      end subroutine boundary_barotropic

!***********************************************************************

      subroutine boundary_1d_ew(ARRAY, imin, imax)

      implicit none

!-----------------------------------------------------------------------
!
!     this routine updates boundary info for a 1-d array in x direction
!
!-----------------------------------------------------------------------

      include 'mpif.h'   ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     intent(in)
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   imin, imax ! array extents

!-----------------------------------------------------------------------
!
!     intent(inout)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imin:imax), intent(inout) :: 
     &   ARRAY      ! array to be updated

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   ierr,                      ! MPI error flag
     &   request(4),                ! MPI request ids
     &   status(MPI_STATUS_SIZE,4)  ! MPI status flags

!-----------------------------------------------------------------------
!
!     swap messages to update boundaries
!
!-----------------------------------------------------------------------

      call MPI_IRECV(ARRAY(1), num_ghost_cells, 
     &               MPI_DOUBLE_PRECISION, nbr_west,
     &               mpitag_wshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(ARRAY(iphys_e+1), num_ghost_cells, 
     &               MPI_DOUBLE_PRECISION, nbr_east,
     &               mpitag_eshift, MPI_COMM_OCN, request(4), ierr)

      call MPI_ISEND(ARRAY(iphys_e+1-num_ghost_cells), num_ghost_cells, 
     &               MPI_DOUBLE_PRECISION, nbr_east,
     &               mpitag_wshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(ARRAY(iphys_b), num_ghost_cells, 
     &               MPI_DOUBLE_PRECISION, nbr_west,
     &               mpitag_eshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_WAITALL(4, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine boundary_1d_ew

!***********************************************************************

      subroutine boundary_1d_ns(ARRAY, jmin, jmax)

!-----------------------------------------------------------------------
!
!     this routine updates boundary info for a 1-d array in y direction
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use communicate

      implicit none

      include 'mpif.h'   ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     intent(in)
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   jmin, jmax                 ! array extents

!-----------------------------------------------------------------------
!
!     intent(inout)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(jmin:jmax), intent(inout) :: 
     &   ARRAY                      ! array to be updated

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   ierr,                      ! MPI error flag
     &   request(4),                ! MPI request ids
     &   status(MPI_STATUS_SIZE,4)  ! MPI status flags

!-----------------------------------------------------------------------
!
!     swap one-word messages to update boundaries
!
!-----------------------------------------------------------------------

      call MPI_IRECV(ARRAY(1), num_ghost_cells, 
     &               MPI_DOUBLE_PRECISION, nbr_south,
     &               mpitag_sshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(ARRAY(jphys_e+1), num_ghost_cells, 
     &               MPI_DOUBLE_PRECISION, nbr_north,
     &               mpitag_nshift, MPI_COMM_OCN, request(4), ierr)

      call MPI_ISEND(ARRAY(jphys_e+1-num_ghost_cells), num_ghost_cells, 
     &               MPI_DOUBLE_PRECISION, nbr_north,
     &               mpitag_sshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(ARRAY(jphys_b), num_ghost_cells, 
     &               MPI_DOUBLE_PRECISION, nbr_south,
     &               mpitag_nshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_WAITALL(4, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine boundary_1d_ns

!***********************************************************************

      end module boundary

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
