!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module stencils

!***********************************************************************
!
!     this module contains stencil communication routines used in POP
!
!     CVS:$Id: stencils.F,v 1.2.2.1 2001/03/13 22:54:26 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use communicate

      implicit none

!-----------------------------------------------------------------------
!
!     generic interfaces for shift routines
!
!-----------------------------------------------------------------------

      interface n_shift
        module procedure n_ishift, n_rshift
      end interface

      interface s_shift
        module procedure s_ishift, s_rshift
      end interface

      interface e_shift
        module procedure e_ishift, e_rshift
      end interface

      interface w_shift
        module procedure w_ishift, w_rshift
      end interface

      interface n_eoshift
        module procedure n_eoishift, n_eorshift
      end interface

      interface s_eoshift
        module procedure s_eoishift, s_eorshift
      end interface

      interface e_eoshift
        module procedure e_eoishift, e_eorshift
      end interface

      interface w_eoshift
        module procedure w_eoishift, w_eorshift
      end interface

!***********************************************************************

      contains

!***********************************************************************

      subroutine n_rshift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from north (real arrays)
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(2),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,2)   ! MPI status flags

!-----------------------------------------------------------------------

      call MPI_IRECV(XOUT(1,jphys_e+1), 1, mpi_ns_type, nbr_north,
     &               mpitag_nshift, MPI_COMM_OCN, request(2), ierr)

      do j=1,jphys_e
        do i=1,imt
          XOUT(i,j) = X(i,j+1)
        end do
      end do

      call MPI_ISEND(XOUT(1,jphys_b), 1, mpi_ns_type, nbr_south,
     &               mpitag_nshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_WAITALL(2, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine n_rshift

!***********************************************************************

      subroutine s_rshift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from south (real arrays)
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(2),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,2)   ! MPI status flags

!-----------------------------------------------------------------------

      call MPI_IRECV(XOUT(1,1), 1, mpi_ns_type, nbr_south,
     &               mpitag_sshift, MPI_COMM_OCN, request(2), ierr)

      do j=jphys_b,jmt
        do i=1,imt
          XOUT(i,j) = X(i,j-1)
        end do
      end do

      call MPI_ISEND(XOUT(1,jphys_e+1-num_ghost_cells), 1, 
     &               mpi_ns_type, nbr_north,
     &               mpitag_sshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_WAITALL(2, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine s_rshift

!***********************************************************************

      subroutine e_rshift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from east (real arrays)
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(2),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,2)   ! MPI status flags

!-----------------------------------------------------------------------

      call MPI_IRECV(XOUT(iphys_e+1,1), 1, mpi_ew_type, nbr_east,
     &               mpitag_eshift, MPI_COMM_OCN, request(2), ierr)

      do j=1,jmt
        do i=1,iphys_e
          XOUT(i,j) = X(i+1,j)
        end do
      end do

      call MPI_ISEND(XOUT(iphys_b,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_eshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_WAITALL(2, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine e_rshift

!***********************************************************************

      subroutine w_rshift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from west (real arrays)
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(2),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,2)   ! MPI status flags

!-----------------------------------------------------------------------

      call MPI_IRECV(XOUT(1,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_wshift, MPI_COMM_OCN, request(2), ierr)

      do j=1,jmt
        do i=iphys_b,imt
          XOUT(i,j) = X(i-1,j)
        end do
      end do

      call MPI_ISEND(XOUT(iphys_e+1-num_ghost_cells,1), 1, 
     &               mpi_ew_type, nbr_east,
     &               mpitag_wshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_WAITALL(2, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine w_rshift

!***********************************************************************

      subroutine n_ishift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from north (integer arrays)
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(2),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,2)   ! MPI status flags

!-----------------------------------------------------------------------

      call MPI_IRECV(XOUT(1,jphys_e+1), 1, mpi_ns_type_int, nbr_north,
     &               mpitag_nshift, MPI_COMM_OCN, request(2), ierr)

      do j=1,jphys_e
        do i=1,imt
          XOUT(i,j) = X(i,j+1)
        end do
      end do

      call MPI_ISEND(XOUT(1,jphys_b), 1, mpi_ns_type_int, nbr_south,
     &               mpitag_nshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_WAITALL(2, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine n_ishift

!***********************************************************************

      subroutine s_ishift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from south (integer arrays)
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(2),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,2)   ! MPI status flags

!-----------------------------------------------------------------------

      call MPI_IRECV(XOUT(1,1), 1, mpi_ns_type_int, nbr_south,
     &               mpitag_sshift, MPI_COMM_OCN, request(2), ierr)

      do j=jphys_b,jmt
        do i=1,imt
          XOUT(i,j) = X(i,j-1)
        end do
      end do

      call MPI_ISEND(XOUT(1,jphys_e+1-num_ghost_cells), 1, 
     &               mpi_ns_type_int, nbr_north,
     &               mpitag_sshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_WAITALL(2, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine s_ishift

!***********************************************************************

      subroutine e_ishift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from east (integer arrays)
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(2),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,2)   ! MPI status flags

!-----------------------------------------------------------------------

      call MPI_IRECV(XOUT(iphys_e+1,1), 1, mpi_ew_type_int, nbr_east,
     &               mpitag_eshift, MPI_COMM_OCN, request(2), ierr)

      do j=1,jmt
        do i=1,iphys_e
          XOUT(i,j) = X(i+1,j)
        end do
      end do

      call MPI_ISEND(XOUT(iphys_b,1), 1, mpi_ew_type_int, nbr_west,
     &               mpitag_eshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_WAITALL(2, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine e_ishift

!***********************************************************************

      subroutine w_ishift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from west (integer arrays)
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(2),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,2)   ! MPI status flags

!-----------------------------------------------------------------------

      call MPI_IRECV(XOUT(1,1), 1, mpi_ew_type_int, nbr_west,
     &               mpitag_wshift, MPI_COMM_OCN, request(2), ierr)

      do j=1,jmt
        do i=iphys_b,imt
          XOUT(i,j) = X(i-1,j)
        end do
      end do

      call MPI_ISEND(XOUT(iphys_e+1-num_ghost_cells,1), 1, 
     &               mpi_ew_type_int, nbr_east,
     &               mpitag_wshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_WAITALL(2, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine w_ishift

!***********************************************************************

      subroutine n_eorshift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from north (real arrays) with no ghost cell update
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j                         ! dummy counters

!-----------------------------------------------------------------------

      do j=1,jmt-1
        do i=1,imt
          XOUT(i,j) = X(i,j+1)
        end do
      end do

      XOUT(:,jmt) = c0

!-----------------------------------------------------------------------

      end subroutine n_eorshift

!***********************************************************************

      subroutine s_eorshift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from south (real arrays) without boundary update
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j                         ! dummy counters

!-----------------------------------------------------------------------

      do j=2,jmt
        do i=1,imt
          XOUT(i,j) = X(i,j-1)
        end do
      end do

      XOUT(:,1) = c0

!-----------------------------------------------------------------------

      end subroutine s_eorshift

!***********************************************************************

      subroutine e_eorshift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from east (real arrays) without ghost cell updates
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j                         ! dummy counters

!-----------------------------------------------------------------------

      do j=1,jmt
        do i=1,imt-1
          XOUT(i,j) = X(i+1,j)
        end do
      end do

      XOUT(imt,:) = 0

!-----------------------------------------------------------------------

      end subroutine e_eorshift

!***********************************************************************

      subroutine w_eorshift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from west (real arrays) without ghost cell updates
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j                         ! dummy counters

!-----------------------------------------------------------------------

      do j=1,jmt
        do i=2,imt
          XOUT(i,j) = X(i-1,j)
        end do
      end do

      XOUT(1,:) = c0

!-----------------------------------------------------------------------

      end subroutine w_eorshift

!***********************************************************************

      subroutine n_eoishift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from north (integer arrays) without ghost cell updates
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j                         ! dummy counters

!-----------------------------------------------------------------------

      do j=1,jmt-1
        do i=1,imt
          XOUT(i,j) = X(i,j+1)
        end do
      end do

      XOUT(:,jmt) = c0

!-----------------------------------------------------------------------

      end subroutine n_eoishift

!***********************************************************************

      subroutine s_eoishift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from south (integer arrays) without ghost cell updates
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j                         ! dummy counters

!-----------------------------------------------------------------------

      do j=2,jmt
        do i=1,imt
          XOUT(i,j) = X(i,j-1)
        end do
      end do

      XOUT(:,1) = c0

!-----------------------------------------------------------------------

      end subroutine s_eoishift

!***********************************************************************

      subroutine e_eoishift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from east (integer arrays) without ghost cell updates
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j                         ! dummy counters

!-----------------------------------------------------------------------

      do j=1,jmt
        do i=1,imt-1
          XOUT(i,j) = X(i+1,j)
        end do
      end do

      XOUT(imt,:) = c0

!-----------------------------------------------------------------------

      end subroutine e_eoishift

!***********************************************************************

      subroutine w_eoishift(XOUT,X)

!-----------------------------------------------------------------------
!
!     shift from west (integer arrays) without ghost cell updates
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &   X                  ! array to be shifted

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j                         ! dummy counters

!-----------------------------------------------------------------------

      do j=1,jmt
        do i=2,imt
          XOUT(i,j) = X(i-1,j)
        end do
      end do

      XOUT(1,:) = c0

!-----------------------------------------------------------------------

      end subroutine w_eoishift

!***********************************************************************

      subroutine ninept_9(XOUT,CC,CN,CS,CE,CW,CNE,CSE,CNW,CSW,X)

!-----------------------------------------------------------------------
!
!     9-point stencil  (2d, 3x3) with all 9 weights
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X,                 ! array to be shifted
     &   CC,CN,CS,CE,CW,    ! weights in each of the nine directions
     &   CNE,CSE,CNW,CSW

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(4),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,4)   ! MPI status flags

!-----------------------------------------------------------------------

      do j=jphys_b,jphys_e
        do i=iphys_b,iphys_e
          XOUT(i,j) = CNE(i,j)*X(i+1,j+1) + CNW(i,j)*X(i-1,j+1) +
     &                CSE(i,j)*X(i+1,j-1) + CSW(i,j)*X(i-1,j-1) +
     &                CN (i,j)*X(i  ,j+1) + CS (i,j)*X(i  ,j-1) +
     &                CE (i,j)*X(i+1,j  ) + CW (i,j)*X(i-1,j  ) +
     &                CC(i,j)*X(i,j)
        end do
      end do

!-----------------------------------------------------------------------
!
!     take care of boundaries
!
!-----------------------------------------------------------------------

      call MPI_IRECV(XOUT(1,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_wshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(XOUT(iphys_e+1,1), 1, mpi_ew_type, nbr_east,
     &               mpitag_eshift, MPI_COMM_OCN, request(4), ierr)

      call MPI_ISEND(XOUT(iphys_e+1-num_ghost_cells,1), 1, 
     &               mpi_ew_type, nbr_east,
     &               mpitag_wshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(XOUT(iphys_b,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_eshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_WAITALL(4, request, status, ierr)

      call MPI_IRECV(XOUT(1,jphys_e+1), 1, mpi_ns_type, nbr_north,
     &               mpitag_nshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(XOUT(1,1), 1, mpi_ns_type, nbr_south,
     &               mpitag_sshift, MPI_COMM_OCN, request(4), ierr)

      call MPI_ISEND(XOUT(1,jphys_b), 1, mpi_ns_type, nbr_south,
     &               mpitag_nshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(XOUT(1,jphys_e+1-num_ghost_cells), 1,
     &               mpi_ns_type, nbr_north,
     &               mpitag_sshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_WAITALL(4, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine ninept_9

!***********************************************************************

      subroutine ninept_4(XOUT,CC,CN,CE,CNE,X)

!-----------------------------------------------------------------------
!
!     9-point stencil - with minimum (4) weights
!     takes advantage of some 9pt weights being shifted versions of
!     others
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X,                 ! array to be operated on 
     &   CC,CN,CE,CNE       ! weights for center, north, east and ne

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! nine point operator result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(4),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,4)   ! MPI status flags

!-----------------------------------------------------------------------

      do j=jphys_b,jphys_e
        do i=iphys_b,iphys_e
          XOUT(i,j) = CC(i,j)*X(i  ,j  ) +
     &                CN(i,j)*X(i  ,j+1) +  CN(i,j-1)*X(i  ,j-1) +
     &                CE(i,j)*X(i+1,j  ) +  CE(i-1,j)*X(i-1,j  ) +
     &               CNE(i,j)*X(i+1,j+1) + CNE(i,j-1)*X(i+1,j-1) +
     &               CNE(i-1,j)*X(i-1,j+1) + CNE(i-1,j-1)*X(i-1,j-1)
        end do
      end do

!-----------------------------------------------------------------------
!
!     update ghost cell boundaries.
!
!-----------------------------------------------------------------------

      call MPI_IRECV(XOUT(1,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_wshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(XOUT(iphys_e+1,1), 1, mpi_ew_type, nbr_east,
     &               mpitag_eshift, MPI_COMM_OCN, request(4), ierr)

      call MPI_ISEND(XOUT(iphys_e+1-num_ghost_cells,1), 1, 
     &               mpi_ew_type, nbr_east,
     &               mpitag_wshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(XOUT(iphys_b,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_eshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_WAITALL(4, request, status, ierr)

      call MPI_IRECV(XOUT(1,jphys_e+1), 1, mpi_ns_type, nbr_north,
     &               mpitag_nshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(XOUT(1,1), 1, mpi_ns_type, nbr_south,
     &               mpitag_sshift, MPI_COMM_OCN, request(4), ierr)

      call MPI_ISEND(XOUT(1,jphys_b), 1, mpi_ns_type, nbr_south,
     &               mpitag_nshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(XOUT(1,jphys_e+1-num_ghost_cells), 1,
     &               mpi_ns_type, nbr_north,
     &               mpitag_sshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_WAITALL(4, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine ninept_4

!***********************************************************************

      subroutine fivept(XOUT,CC,CN,CS,CE,CW,X)

!-----------------------------------------------------------------------
!
!     5-point stencil  (2d)
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X,                 ! array to operate on with 5pt operator
     &   CC,CN,CS,CE,CW     ! stencil weight coefficients

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! result of 5pt stencil operator

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(4),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,4)   ! MPI status flags

!-----------------------------------------------------------------------
!
!     interior points
!
!-----------------------------------------------------------------------

      do j=jphys_b,jphys_e
        do i=iphys_b,iphys_e
          XOUT(i,j)   = CC(i,j)*X(i,j) +
     &                CN(i,j)*X(i  ,j+1) + CS(i,j)*X(i  ,j-1) +
     &                CE(i,j)*X(i+1,j  ) + CW(i,j)*X(i-1,j  )
        end do
      end do

!-----------------------------------------------------------------------
!
!     update boundaries
!
!-----------------------------------------------------------------------

      call MPI_IRECV(XOUT(1,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_wshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(XOUT(iphys_e+1,1), 1, mpi_ew_type, nbr_east,
     &               mpitag_eshift, MPI_COMM_OCN, request(4), ierr)

      call MPI_ISEND(XOUT(iphys_e+1-num_ghost_cells,1), 1, 
     &               mpi_ew_type, nbr_east,
     &               mpitag_wshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(XOUT(iphys_b,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_eshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_WAITALL(4, request, status, ierr)

      call MPI_IRECV(XOUT(1,jphys_e+1), 1, mpi_ns_type, nbr_north,
     &               mpitag_nshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(XOUT(1,1), 1, mpi_ns_type, nbr_south,
     &               mpitag_sshift, MPI_COMM_OCN, request(4), ierr)

      call MPI_ISEND(XOUT(1,jphys_b), 1, mpi_ns_type, nbr_south,
     &               mpitag_nshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(XOUT(1,jphys_e+1-num_ghost_cells), 1,
     &               mpi_ns_type, nbr_north,
     &               mpitag_sshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_WAITALL(4, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine fivept

!***********************************************************************

      subroutine fivept_nobndy(XOUT,CC,CN,CS,CE,CW,X)

!-----------------------------------------------------------------------
!
!     5-point stencil  (2d)
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X,                 ! array to operate on with 5pt operator
     &   CC,CN,CS,CE,CW     ! stencil weight coefficients

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! result of 5pt stencil operator

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j                         ! dummy counters

!-----------------------------------------------------------------------
!
!     interior points
!
!-----------------------------------------------------------------------

      do j=2,jmt-1
        do i=2,imt-1
          XOUT(i,j) = CC(i,j)*X(i,j) +
     &                CN(i,j)*X(i  ,j+1) + CS(i,j)*X(i  ,j-1) +
     &                CE(i,j)*X(i+1,j  ) + CW(i,j)*X(i-1,j  )
        end do
      end do

!-----------------------------------------------------------------------
!
!     boundary updates if required
!
!-----------------------------------------------------------------------

      XOUT(:,1  ) = c0
      XOUT(:,jmt) = c0
      XOUT(1  ,:) = c0
      XOUT(imt,:) = c0

!-----------------------------------------------------------------------

      end subroutine fivept_nobndy

!***********************************************************************

      subroutine advu_fivept(UOUT,VOUT,UUW,VUS,UAREA_R,UUU,VVV)

!-----------------------------------------------------------------------
!
!     5-point stencil specific for advu (2d)
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   UUU,VVV,           ! u,v velocity to be operated on
     &   UUW,VUS,           ! advective interface velocities and
     &   UAREA_R            ! 1/(u-cell area) for computing stencil wgts

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   UOUT,VOUT          ! result of 5pt operator on UUU,VVV

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j                         ! dummy counters

      real (kind=dbl_kind) ::
     &   cc                          ! scalar central weight

!-----------------------------------------------------------------------

      do j=jphys_b,jphys_e
        do i=iphys_b,iphys_e
          cc = VUS(i,j+1) - VUS(i,j) + UUW(i+1,j) - UUW(i,j)
          UOUT(i,j) = p5*(cc*UUU(i,j) +
     &                VUS(i,j+1)*UUU(i,j+1) - VUS(i,j)*UUU(i,j-1) +
     &                UUW(i+1,j)*UUU(i+1,j) - UUW(i,j)*UUU(i-1,j))*
     &                UAREA_R(i,j)
          VOUT(i,j) = p5*(cc*VVV(i,j) +
     &                VUS(i,j+1)*VVV(i,j+1) - VUS(i,j)*VVV(i,j-1) +
     &                UUW(i+1,j)*VVV(i+1,j) - UUW(i,j)*VVV(i-1,j))*
     &                UAREA_R(i,j)
        end do
      end do

      UOUT(:,1:num_ghost_cells)   = c0
      VOUT(:,1:num_ghost_cells)   = c0
      UOUT(:,jphys_e+1:jmt) = c0
      VOUT(:,jphys_e+1:jmt) = c0
      UOUT(1:num_ghost_cells,:)   = c0
      VOUT(1:num_ghost_cells,:)   = c0
      UOUT(iphys_e+1:imt,:) = c0
      VOUT(iphys_e+1:imt,:) = c0

!-----------------------------------------------------------------------

      end subroutine advu_fivept

!***********************************************************************

      subroutine advt_fivept(k,LTK,CN,CE,TAREA_R,TRCR)

!-----------------------------------------------------------------------
!
!     5-point stencil specifically for advt (2d)
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &   k                  ! vertical level to operate on

      real (kind=dbl_kind), dimension(imt,jmt,km,nt), intent(in) :: 
     &   TRCR               ! array of tracers to be operated on

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   CN,CE,             ! stencil weights
     &   TAREA_R            ! 1/(tracer cell area) for weights

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(out) ::
     &   LTK                ! result of operator for each tracer

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,n                     ! dummy counters

!-----------------------------------------------------------------------

      do n=1,nt
        do j=jphys_b,jphys_e
          do i=iphys_b,iphys_e
            LTK(i,j,n) = p5*(
     &   (CN(i,j)-CN(i,j-1)+CE(i,j)-CE(i-1,j))*TRCR(i,j,k,n) +
     &                       CN(i  ,j  )*TRCR(i  ,j+1,k,n) - 
     &                       CN(i  ,j-1)*TRCR(i  ,j-1,k,n) +
     &                       CE(i  ,j  )*TRCR(i+1,j  ,k,n) - 
     &                       CE(i-1,j  )*TRCR(i-1,j  ,k,n))*TAREA_R(i,j)
          end do
        end do
      end do

      LTK(:,1:num_ghost_cells,:) = c0
      LTK(:,jphys_e+1:jmt    ,:) = c0
      LTK(1:num_ghost_cells,:,:) = c0
      LTK(iphys_e+1:imt    ,:,:) = c0

!-----------------------------------------------------------------------

      end subroutine advt_fivept

!***********************************************************************

      subroutine ne_4pt(XOUT,CC,CN,CE,CNE,X)

!-----------------------------------------------------------------------
!
!     northeast 4-point 
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X,                 ! array to be shifted
     &   CC,CN,CE,CNE       ! stencil weight coefficients

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(2),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,2)   ! MPI status flags

!-----------------------------------------------------------------------

      do j=1,jphys_e
        do i=1,iphys_e
          XOUT(i,j) = CC (i,j)*X(i,j) +
     &                CN (i,j)*X(i  ,j+1) + CE(i,j)*X(i+1,j) +
     &                CNE(i,j)*X(i+1,j+1)

        end do
      end do


      call MPI_IRECV(XOUT(iphys_e+1,1), 1, mpi_ew_type, nbr_east,
     &               mpitag_eshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_ISEND(XOUT(iphys_b,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_eshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_WAITALL(2, request, status, ierr)

      call MPI_IRECV(XOUT(1,jphys_e+1), 1, mpi_ns_type, nbr_north,
     &               mpitag_nshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_ISEND(XOUT(1,jphys_b), 1, mpi_ns_type, nbr_south,
     &               mpitag_nshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_WAITALL(2, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine ne_4pt 

!***********************************************************************

      subroutine ne_4pt_nobndy(XOUT,CC,CN,CE,CNE,X)

!-----------------------------------------------------------------------
!
!     northeast 4-point without ghost cell updates 
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X,                 ! array to be shifted
     &   CC,CN,CE,CNE       ! stencil weight coefficients

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j                         ! dummy counters

!-----------------------------------------------------------------------

      do j=1,jmt-1
        do i=1,imt-1
          XOUT(i,j) = CC (i,j)*X(i,j) +
     &                CN (i,j)*X(i  ,j+1) + CE(i,j)*X(i+1,j) +
     &                CNE(i,j)*X(i+1,j+1)

        end do
      end do

      XOUT(:,jmt) = c0
      XOUT(imt,:) = c0

!-----------------------------------------------------------------------

      end subroutine ne_4pt_nobndy 

!***********************************************************************

      subroutine sw_4pt(XOUT,CC,CS,CW,CSW,X)

!-----------------------------------------------------------------------
!
!     southwest 4-point
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X,                 ! array to be shifted
     &   CC,CS,CW,CSW       ! stencil weight coefficients

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(2),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,2)   ! MPI status flags

!-----------------------------------------------------------------------

      do j=jphys_b,jmt
        do i=iphys_b,imt
          XOUT(i,j) = CC (i,j)*X(i,j) +
     &                CS (i,j)*X(i  ,j-1) + CW(i,j)*X(i-1,j) +
     &                CSW(i,j)*X(i-1,j-1)
        end do
      end do

      call MPI_IRECV(XOUT(1,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_wshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_ISEND(XOUT(iphys_e+1-num_ghost_cells,1), 1, 
     &               mpi_ew_type, nbr_east,
     &               mpitag_wshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_WAITALL(2, request, status, ierr)

      call MPI_IRECV(XOUT(1,1), 1, mpi_ns_type, nbr_south,
     &               mpitag_sshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_ISEND(XOUT(1,jphys_e+1-num_ghost_cells), 1,
     &               mpi_ns_type, nbr_north,
     &               mpitag_sshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_WAITALL(2, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine sw_4pt 

!***********************************************************************

      subroutine sw_4pt_nobndy(XOUT,CC,CS,CW,CSW,X)

!-----------------------------------------------------------------------
!
!     southwest 4-point without ghost cell updates
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X,                 ! array to be shifted
     &   CC,CS,CW,CSW       ! stencil weight coefficients

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j                         ! dummy counters

!-----------------------------------------------------------------------

      do j=2,jmt
        do i=2,imt
          XOUT(i,j) = CC (i,j)*X(i,j) +
     &                CS (i,j)*X(i  ,j-1) + CW(i,j)*X(i-1,j) +
     &                CSW(i,j)*X(i-1,j-1)
        end do
      end do

      XOUT(1,:) = c0
      XOUT(:,1) = c0

!-----------------------------------------------------------------------

      end subroutine sw_4pt_nobndy 

!***********************************************************************

      subroutine nw_4pt(XOUT,CC,CN,CW,CNW,X)

!-----------------------------------------------------------------------
!
!     northwest 4-point
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X,                 ! array to be shifted
     &   CC,CN,CW,CNW       ! stencil weight coefficients

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(2),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,2)   ! MPI status flags

!-----------------------------------------------------------------------

      do j=1,jphys_e
        do i=iphys_b,imt
          XOUT(i,j) = CC (i,j)*X(i,j) +
     &                CN (i,j)*X(i  ,j+1) + CW(i,j)*X(i-1,j) +
     &                CNW(i,j)*X(i-1,j+1)
        end do
      end do

      call MPI_IRECV(XOUT(1,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_wshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_ISEND(XOUT(iphys_e+1-num_ghost_cells,1), 1, 
     &               mpi_ew_type, nbr_east,
     &               mpitag_wshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_WAITALL(2, request, status, ierr)

      call MPI_IRECV(XOUT(1,jphys_e+1), 1, mpi_ns_type, nbr_north,
     &               mpitag_nshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_ISEND(XOUT(1,jphys_b), 1, mpi_ns_type, nbr_south,
     &               mpitag_nshift, MPI_COMM_OCN, request(1), ierr)

      call MPI_WAITALL(2, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine nw_4pt

!***********************************************************************

      subroutine nw_4pt_nobndy(XOUT,CC,CN,CW,CNW,X)

!-----------------------------------------------------------------------
!
!     northwest 4-point without ghost cell updates
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X,                 ! array to be shifted
     &   CC,CN,CW,CNW       ! stencil weight coefficients

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   XOUT               ! shifted result

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j                         ! dummy counters

!-----------------------------------------------------------------------

      do j=1,jmt-1
        do i=2,imt
          XOUT(i,j) = CC (i,j)*X(i,j) +
     &                CN (i,j)*X(i  ,j+1) + CW(i,j)*X(i-1,j) +
     &                CNW(i,j)*X(i-1,j+1)
        end do
      end do

      XOUT(:,jmt) = c0
      XOUT(1,:)   = c0

!-----------------------------------------------------------------------

      end subroutine nw_4pt_nobndy

!***********************************************************************

      subroutine grad_4pt(GX,GY,F)

!-----------------------------------------------------------------------
!
!     gradient 4-point stencil
!
!     GX,GY are proportional to the finite-difference
!     gradient of the scalar field F.
!     NOTE: this 'stencil' has 1 source      array  (F)
!                          and 2 destination arrays (GX,GY)
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   F                  ! field for which gradient required

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   GX, GY             ! components of gradient in i,j direction

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(4),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,4)   ! MPI status flags

!-----------------------------------------------------------------------

      do j=1,jphys_e
        do i=1,iphys_e
          GX(i,j) = F(i+1,j+1) - F(i,j) - F(i,j+1) + F(i+1,j)
          GY(i,j) = F(i+1,j+1) - F(i,j) + F(i,j+1) - F(i+1,j)
        end do
      end do

      !***
      !*** note using mpitag_wshift as new tag (NOT a wshift)
      !***

      call MPI_IRECV(GX(iphys_e+1,1), 1, mpi_ew_type, nbr_east,
     &               mpitag_eshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(GY(iphys_e+1,1), 1, mpi_ew_type, nbr_east,
     &               mpitag_wshift, MPI_COMM_OCN, request(4), ierr)

      call MPI_ISEND(GX(iphys_b,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_eshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(GY(iphys_b,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_wshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_WAITALL(4, request, status, ierr)

      !***
      !*** note using mpitag_sshift as new tag (NOT a sshift)
      !***

      call MPI_IRECV(GX(1,jphys_e+1), 1, mpi_ns_type, nbr_north,
     &               mpitag_nshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(GY(1,jphys_e+1), 1, mpi_ns_type, nbr_north,
     &               mpitag_sshift, MPI_COMM_OCN, request(4), ierr)

      call MPI_ISEND(GX(1,jphys_b), 1, mpi_ns_type, nbr_south,
     &               mpitag_nshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(GY(1,jphys_b), 1, mpi_ns_type, nbr_south,
     &               mpitag_sshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_WAITALL(4, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine grad_4pt

!***********************************************************************

      subroutine grad_4pt_nobndy(GX,GY,F)

!-----------------------------------------------------------------------
!
!     gradient 4-point stencil with no bounday updates
!
!     GX,GY are proportional to the finite-difference
!     gradient of the scalar field F.
!     NOTE: this 'stencil' has 1 source      array  (F)
!                          and 2 destination arrays (GX,GY)
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   F                  ! field for which gradient required

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   GX, GY             ! components of gradient in i,j direction

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j                         ! dummy counters

!-----------------------------------------------------------------------

      do j=1,jmt-1
        do i=1,imt-1
          GX(i,j) = F(i+1,j+1) - F(i,j) - F(i,j+1) + F(i+1,j)
          GY(i,j) = F(i+1,j+1) - F(i,j) + F(i,j+1) - F(i+1,j)
        end do
      end do

      GX(:,jmt) = c0
      GY(:,jmt) = c0
      GX(imt,:) = c0
      GY(imt,:) = c0

!-----------------------------------------------------------------------

      end subroutine grad_4pt_nobndy

!***********************************************************************

      subroutine div_4pt(D,UX,UY)

!-----------------------------------------------------------------------
!
!     divergence 4-point stencil
!
!     D is proportional to the finite-difference
!     divergence of the vector field (UX,UY).
!     NOTE: this 'stencil' has 2 source      arrays (UX,UY)
!                         and 1 destination array  (D)
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   UX,UY              ! components of vector field for divergence

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   D                  ! divergence of input vector field

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,                        ! dummy counters
     &   ierr,                       ! MPI error flag
     &   request(4),                 ! MPI request ids
     &   status(MPI_STATUS_SIZE,4)   ! MPI status flags

!-----------------------------------------------------------------------

      do j=jphys_b,jmt
        do i=iphys_b,imt
          D(i,j) =  UX(i,j) + UX(i,j-1) - UX(i-1,j) - UX(i-1,j-1) +
     &              UY(i,j) + UY(i-1,j) - UY(i,j-1) - UY(i-1,j-1)
        end do
      end do

      call MPI_IRECV(D(1,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_wshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(D(iphys_e+1,1), 1, mpi_ew_type, nbr_east,
     &               mpitag_eshift, MPI_COMM_OCN, request(4), ierr)

      call MPI_ISEND(D(iphys_e+1-num_ghost_cells,1), 1, 
     &               mpi_ew_type, nbr_east,
     &               mpitag_wshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(D(iphys_b,1), 1, mpi_ew_type, nbr_west,
     &               mpitag_eshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_WAITALL(4, request, status, ierr)

      call MPI_IRECV(D(1,jphys_e+1), 1,  mpi_ns_type, nbr_north,
     &               mpitag_nshift, MPI_COMM_OCN, request(3), ierr)
      call MPI_IRECV(D(1,1), 1, mpi_ns_type, nbr_south,
     &               mpitag_sshift, MPI_COMM_OCN, request(4), ierr)

      call MPI_ISEND(D(1,jphys_b), 1, mpi_ns_type, nbr_south,
     &               mpitag_nshift, MPI_COMM_OCN, request(1), ierr)
      call MPI_ISEND(D(1,jphys_e+1-num_ghost_cells), 1, 
     &               mpi_ns_type, nbr_north,
     &               mpitag_sshift, MPI_COMM_OCN, request(2), ierr)

      call MPI_WAITALL(4, request, status, ierr)

!-----------------------------------------------------------------------

      end subroutine div_4pt

!***********************************************************************

      subroutine hupw3(k,XOUT,X,CN,CS,CE,CW,
     &                 KMTN,KMTS,KMTE,KMTW,KMTNN,KMTEE,
     &                 TALFXP,TBETXP,TGAMXP,TALFYP,TBETYP,TGAMYP,
     &                 TALFXM,TBETXM,TDELXM,TALFYM,TBETYM,TDELYM,
     &                 talfzp,tbetzp,tgamzp,talfzm,tbetzm,tdelzm)

!-----------------------------------------------------------------------
!
!     upwind3 stencil  (2d)
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! vertical level index

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &  KMTN,               ! KMT field at north neighbor
     &  KMTS,               ! KMT field at south neighbor
     &  KMTE,               ! KMT field at east neighbor
     &  KMTW,               ! KMT field at west neighbor
     &  KMTNN,              ! KMT field two points to north
     &  KMTEE               ! KMT field two points to east

      real (kind=dbl_kind), dimension(imt,jmt,km,nt), intent(in) :: 
     &  X                   ! input tracer array

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &  CN,CS,CE,CW,          ! stencil weights based on flux velocities
     &  TALFXP,TBETXP,TGAMXP, ! geometric factors for horizontal
     &  TALFYP,TBETYP,TGAMYP, !   directions
     &  TALFXM,TBETXM,TDELXM,
     &  TALFYM,TBETYM,TDELYM

      real (kind=dbl_kind), dimension(km), intent(in) :: 
     &  talfzp, tbetzp, tgamzp,  ! geometric factors for vertical 
     &  talfzm, tbetzm, tdelzm   ! directions

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt)  :: 
     &  XOUT                ! output tracer advection

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  i,j,n                 ! dummy loop indices

      real (kind=dbl_kind) ::
     &  tplus, tminus, tplusm1, tminusm1,
     &  am, bm, dm, ap, bp, gp, work, work1,
     &  amm1, bmm1, dmm1, apm1, bpm1, gpm1

!-----------------------------------------------------------------------
!
!     3rd order upwinding stencil
!
!-----------------------------------------------------------------------

      do j=jphys_b,jphys_e
        do i=iphys_b,iphys_e

          if (k <= KMTE(i,j)) then
            work  = TBETXP(i,j)
            ap    = TALFXP(i,j)
          else
            work  = TBETXP(i,j) + TALFXP(i,j)
            ap    = c0
          endif
          if (k <= KMTE(i-1,j)) then
            work1 = TBETXP(i-1,j)
            apm1  = TALFXP(i-1,j)
          else
            work1 = TBETXP(i-1,j) + TALFXP(i-1,j)
            apm1  = c0
          endif


          if (k <= KMTW(i,j)) then
            bp   = work
            gp   = TGAMXP(i,j)
          else
            bp   = work + TGAMXP(i,j)
            gp   = c0
          endif
          if (k <= KMTW(i-1,j)) then
            bpm1 = work1
            gpm1 = TGAMXP(i-1,j)
          else
            bpm1 = work1 + TGAMXP(i-1,j)
            gpm1 = c0
          endif

          if (k <= KMTEE(i,j)) then
            am   = TALFXM(i,j)
            dm   = TDELXM(i,j)
          else
            am   = TALFXM(i,j) + TDELXM(i,j)
            dm   = c0
          endif
          if (k <= KMTEE(i-1,j)) then
            amm1 = TALFXM(i-1,j)
            dmm1 = TDELXM(i-1,j)
          else
            amm1 = TALFXM(i-1,j) + TDELXM(i-1,j)
            dmm1 = c0
          endif
          bm   = TBETXM(i,j)
          bmm1 = TBETXM(i-1,j)

          do n=1,nt
            tplus    = ap  *X(i+1,j,k,n) + bp  *X(i,j,k,n) + 
     &                 gp  *X(i-1,j,k,n)
            tplusm1  = apm1*X(i  ,j,k,n) + bpm1*X(i-1,j,k,n) + 
     &                 gpm1*X(i-2,j,k,n)

            tminus   = am  *X(i+1,j,k,n) + bm  *X(i,j,k,n) + 
     &                 dm  *X(i+2,j,k,n)
            tminusm1 = amm1*X(i  ,j,k,n) + bmm1*X(i-1,j,k,n) + 
     &                 dmm1*X(i+1,j,k,n)

            XOUT(i,j,n) = (CE(i,j)+abs(CE(i,j)))*tplus    +
     &                    (CE(i,j)-abs(CE(i,j)))*tminus   +
     &                    (CW(i,j)+abs(CW(i,j)))*tminusm1 +
     &                    (CW(i,j)-abs(CW(i,j)))*tplusm1
          end do
        end do
      end do


      do j=jphys_b,jphys_e
        do i=iphys_b,iphys_e

          if (k <= KMTN(i,j)) then
            work  = TBETYP(i,j)
            ap    = TALFYP(i,j)
          else
            work  = TBETYP(i,j) + TALFYP(i,j)
            ap    = c0
          endif
          if (k <= KMTN(i,j-1)) then
            work1 = TBETYP(i,j-1)
            apm1  = TALFYP(i,j-1)
          else
            work1 = TBETYP(i,j-1) + TALFYP(i,j-1)
            apm1  = c0
          endif

          if (k <= KMTS(i,j)) then
            bp   = work
            gp   = TGAMYP(i,j)
          else
            bp   = work + TGAMYP(i,j)
            gp   = c0
          endif
          if (k <= KMTS(i,j-1)) then
            bpm1 = work1
            gpm1 = TGAMYP(i,j-1)
          else
            bpm1 = work1 + TGAMYP(i,j-1)
            gpm1 = c0
          endif


          if (k <= KMTNN(i,j)) then
            am   = TALFYM(i,j)
            dm   = TDELYM(i,j)
          else
            am   = TALFYM(i,j) + TDELYM(i,j)
            dm   = c0
          endif
          if (k <= KMTNN(i,j-1)) then
            amm1 = TALFYM(i,j-1)
            dmm1 = TDELYM(i,j-1)
          else
            amm1 = TALFYM(i,j-1) + TDELYM(i,j-1)
            dmm1 = c0
          endif
          bm   = TBETYM(i,j)
          bmm1 = TBETYM(i,j-1)

          do n=1,nt
            tplus    = ap  *X(i,j+1,k,n) + bp  *X(i,j,k,n) + 
     &                 gp  *X(i,j-1,k,n)
            tplusm1  = apm1*X(i,j  ,k,n) + bpm1*X(i,j-1,k,n) + 
     &                 gpm1*X(i,j-2,k,n)

            tminus   = am  *X(i,j+1,k,n) + bm  *X(i,j,k,n) + 
     &                 dm  *X(i,j+2,k,n)
            tminusm1 = amm1*X(i,j  ,k,n) + bmm1*X(i,j-1,k,n) + 
     &                 dmm1*X(i,j+1,k,n)

            XOUT(i,j,n) = XOUT(i,j,n) +
     &                  (CN(i,j)+abs(CN(i,j)))*tplus    +
     &                  (CN(i,j)-abs(CN(i,j)))*tminus   +
     &                  (CS(i,j)+abs(CS(i,j)))*tminusm1 +
     &                  (CS(i,j)-abs(CS(i,j)))*tplusm1
          end do
        end do
      end do

!-----------------------------------------------------------------------

      end subroutine hupw3 

!***********************************************************************

      end module stencils

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
