!===============================================================================
! 
!    The Community Climate System Model (CCSM) Copyright and Release Notice
! 
!              +------------------------------------------+
!              | DISTRIBUTION TERMS AND CONDITIONS NOTICE |
!              +------------------------------------------+
! 
! 
! 
! CCSM was developed in cooperation with the National Science Foundation,
! the Department of Energy Los Alamos National Laboratory (LANL), the
! National Aeronautics and Space Administration Data Assimilation Office,
! and the University Corporation for Atmospheric Research's National
! Center for Atmospheric Research.*  Except for POP, SCRIP, and CICE,
! which are segregable components of the model, CCSM is public domain
! software.  
! 
! POP, SCRIP, and CICE were developed at LANL and are protected by the
! following copyright notice. 
! 
! Copyright (c)2002 The Regents of the University of California. All Rights
! Reserved. 
! 
! Permission to use, copy, modify, and distribute this software and its
! documentation for educational, research and non-profit purposes, without
! fee, and without a written agreement is hereby granted, provided that
! the above copyright notice, this paragraph and the following three
! paragraphs appear in all copies. 
! 
! Permission to incorporate this software into commercial products may be
! obtained by contacting the University of California, Charles Rzeszutko,
! Campus Liaison Officer Office of Technology Transfer, 1111 Franklin
! Street, 5th Floor, Oakland, CA 94607, (510) 587-6063,
! charles.rzeszutko@ucop.edu.
! 
! THIS SOFTWARE PROGRAM AND DOCUMENTATION ARE COPYRIGHTED BY 
! THE REGENTS OF THE UNIVERSITY OF CALIFORNIA. THE SOFTWARE 
! PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS", WITHOUT ANY 
! ACCOMPANYING SERVICES FROM THE REGENTS. THE REGENTS DOES NOT 
! WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE 
! UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS THAT 
! THE PROGRAM WAS DEVELOPED FOR RESEARCH PURPOSES AND IS 
! ADVISED NOT TO RELY EXCLUSIVELY ON THE PROGRAM FOR ANY 
! REASON. 
! 
! IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY 
! PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
! CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF 
! THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE 
! UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF 
! SUCH DAMAGE.  THE UNIVERSITY OF CALIFORNIA SPECIFICALLY 
! DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
! IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
! PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN 
! "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO 
! OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, 
! ENHANCEMENTS, OR MODIFICATIONS. 
! 
! *The National Center for Atmospheric Research is sponsored by the
! National Science Foundation.  Any opinions, findings and conclusions or
! recommendations expressed in this publication are those of the author(s)
! and do not necessarily reflect the views of the National Science
! Foundation.
!===============================================================================


!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      program POP

!***********************************************************************
!
!     This is the main driver for the Parallel Ocean Program (POP).
!
!     CVS:$Id: POP.F,v 1.2.2.6 2002/04/25 22:45:00 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use domain         ! Required because some compilers are buggy
      use kinds_mod
      use communicate
      use constants
      use timers         ! Timers module already uses domain
      use time_management
      use step_mod
      use initial
      use xdisplay
      use diagnostics
      use output
      use exit_mod
      use forcing_coupled
      use shr_sys_mod

      ! the following should not be needed once cleanup is finished
      use solvers
      use io

      implicit none

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  timer_total,           ! timer number for total time
     &  timer_step,            ! timer number for step
     &  ierr,                  ! error flag
     &  fstop_now,             ! flag id for stop_now flag
     &  nscan


!-----------------------------------------------------------------------
!
!     call system routine to catch kill signals and route them to the
!     routine set_term to be processed 
!     signal 15 is terminate
!     signal 24 is almost out of cpu time
!
!-----------------------------------------------------------------------


!-----------------------------------------------------------------------
!
!     initialize the model run
!
!-----------------------------------------------------------------------

      call initialize_POP

      if ( my_task == master_task )  call shr_sys_flush(stdout)
 
      fstop_now = init_time_flag('stop_now',default=.false.)
      nscan = 0

!-----------------------------------------------------------------------
!
!     start up the main timer
!
!-----------------------------------------------------------------------

      call get_timer(timer_total,'TOTAL')
      call timer_start(timer_total)

      call get_timer(timer_step,'STEP')

!-----------------------------------------------------------------------
!
!     advance the model in time
!
!-----------------------------------------------------------------------

      advance: do while (.not. check_time_flag(fstop_now))

        call timer_start(timer_step)
        call step
        call timer_stop(timer_step)
 
        if (lcoupled .and. check_time_flag(fstop_now))exit advance

        nscan = nscan + solv_sum_iters

        !***
        !*** exit if energy is blowing
        !***

        if (diag_ke < c0 .or. diag_ke > 100.0_dbl_kind) then
          call set_time_flag(fstop_now,.true.)
          call output_driver
          call exit_POP(' k.e. > 100 ', normal_exit = .false.)
        endif

!-----------------------------------------------------------------------
!
!       write restart dumps and archiving
!
!-----------------------------------------------------------------------

        call output_driver
 
        if ( my_task == master_task )  call shr_sys_flush(stdout)

      enddo advance

!-----------------------------------------------------------------------
!
!     write an end restart if we are through the stepping loop 
!     without an error
!
!-----------------------------------------------------------------------

      nscan = nscan/nsteps_run
      if(my_task.eq.master_task) 
     &   write(stdout,*) ' average # scans =', nscan

!-----------------------------------------------------------------------
!
!     print timing information and clean up various environments if 
!     they have been used
!
!-----------------------------------------------------------------------

      call timer_stop(timer_total)
      call timer_print_all

      if (lxdisplay) call clear_display
     
      call exit_POP('Successful completion of POP run'
     &,              normal_exit = .true. )

 
!-----------------------------------------------------------------------

      end program POP


!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
