!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module budget_diagnostics

!***********************************************************************
!
!     this module contains routines for tracers budget diagnostics. 
!
!     CVS:$Id: budget_diagnostics.F,v 1.1.2.6 2002/02/22 22:54:04 njn01 Exp $ 
!     CVS:$Name: ccsm2_0_beta46 $ 
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use diagnostics
      use domain
      use constants
      use prognostic
      use io
      use global_reductions
      use grid 
      use exit_mod
      use ice
      use qflux_mod
      use tavg
      use time_management

      implicit none
      save

      logical (kind=log_kind) ::
     &  ldiag_global_tracer_budgets  ! global budget diagnostics for tracers
     &, budget_warning_1st_step      ! warning flag for budget diagnostics
 
!***********************************************************************

      contains

!***********************************************************************

      subroutine init_budget_diagnostics

!-----------------------------------------------------------------------
!
!     this subroutine checks if the necessary TAVG_2D fields are
!     specified in the tavg_contents file if the volume/tracer
!     budget diagnostics are on.
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  budget_error_flag   ! error flag for the missing TAVG_2D fields, etc. 
     &, nml_error           ! namelist i/o error flag
 
      namelist /budget_diagnostics_nml/ 
     &                   ldiag_global_tracer_budgets

!-----------------------------------------------------------------------
!
!     set budget warning flag if this is the first step of a new run
!
!-----------------------------------------------------------------------
      
      if (first_step) then
          budget_warning_1st_step = .true.
      else
          budget_warning_1st_step = .false.
      endif

 
!-----------------------------------------------------------------------
!
!     read budget_diagnostics namelist
!
!-----------------------------------------------------------------------

      ldiag_global_tracer_budgets = .false.
 
      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read (nml_in, nml=budget_diagnostics_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR: reading budget_diagnostics_nml')
      endif
 
      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, budget_diagnostics_nml)
          write(stdout,*) ' '
      endif


      call broadcast_scalar(ldiag_global_tracer_budgets, master_task)
 
      if (my_task == master_task) then
        write(stdout,*) ' '
        write(stdout,"(72('-'))")
        write(stdout,*) ' '
        write(stdout,*) 'Budget Diagnostic Options'
        write(stdout,*) ' '
        write(stdout,"(72('-'))")
 
        if ( ldiag_global_tracer_budgets ) then
          write (stdout,*) 'Global tracer budgets diagnostics computed'
        else
          write (stdout,*) 'Global tracer budgets diagnostics ',
     &                     ' not computed'
        endif
      endif
 
      budget_error_flag = 0

 
      if (ldiag_global_tracer_budgets ) then
       if (tavg_freq_iopt == freq_opt_never) then
                                              budget_error_flag = -1000
       else
        if (.not. ltavg_flags(ntavg_SHF)    ) budget_error_flag = -2000
        if (.not. ltavg_flags(ntavg_SFWF)   ) budget_error_flag = -2000
        if (.not. ltavg_flags(ntavg_RESID_T)) budget_error_flag = -2000
        if (.not. ltavg_flags(ntavg_RESID_S)) budget_error_flag = -2000
        if (.not. ltavg_flags(ntavg_FW)     ) budget_error_flag = -2000
        if (.not. ltavg_flags(ntavg_TFW_T)  ) budget_error_flag = -2000
        if (.not. ltavg_flags(ntavg_TFW_S)  ) budget_error_flag = -2000
       endif
      endif

      if     ( budget_error_flag == -1000 ) then
        call exit_POP ('ERROR: ' //
     &    'init_budget_diag: you cannot select both '
     & // 'tavg_freq_iopt == freq_opt_never and '
     & // 'ldiag_global_tracer_budgets = .true. '                     )
      elseif ( budget_error_flag == -2000 ) then
        call exit_POP ( 'ERROR: ' //
     &    'init_budget_diag: SHF, SFWF, RESID_T, RESID_S, FW, TFW_T,'
     &  //' and TFW_S, should be included in the tavg_contents file.' )
      endif

      budget_error_flag = 0

      if (ldiag_global_tracer_budgets ) then
        if ( my_task == master_task ) then
          select case (tavg_freq_iopt)
 
           case ( freq_opt_never ) 
             write(stdout,1100) 'no budget interval specified'
             budget_error_flag = -1000
 
           case ( freq_opt_nyear ) 
             write(stdout,1101) tavg_freq,' years'
 
           case ( freq_opt_nmonth ) 
             write(stdout,1101) tavg_freq,' months'
 
           case ( freq_opt_nday ) 
             write(stdout,1101) tavg_freq,' days'

           case ( freq_opt_nhour ) 
             write(stdout,1101) tavg_freq,' hours'

           case ( freq_opt_nsecond )
             write(stdout,1101) tavg_freq,' seconds'
 
           case ( freq_opt_nstep )
             write(stdout,1101) tavg_freq,' steps'
 
          end select
        endif
      endif

      call broadcast_scalar (budget_error_flag, master_task)

      if ( budget_error_flag == -1000 ) then
        call exit_POP ('ERROR: ' //
     &    'init_budget_diag: no budget interval is specified.' ) 
      endif

1100  format ('(init_budget_diag): ',a)
1101  format ('(init_budget_diag): tracer budgets are for every ',i4,a)
 
      end subroutine init_budget_diagnostics 

!***********************************************************************

      subroutine diag_for_tracer_budgets ( tracer_mean, 
     &                                     modified_volume_t)

!-----------------------------------------------------------------------
!
!     this subroutine computes the global sum of volume * tracer
!     (returned in tracer_mean) and the global ocean volume at T points
!     at curtime (returned in modified_volume_t).
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! 
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(nt), intent(out) :: tracer_mean
      real (kind=dbl_kind), intent(out) :: modified_volume_t

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  WORK1, WORK2, WORK3, WORK4                ! work arrays

      real (kind=dbl_kind) :: sum_tmp, srf_volume

      integer (kind=int_kind) :: k, n

!-----------------------------------------------------------------------
!
!     compute the ocean volume based on oldtime and curtime
!
!----------------------------------------------------------------------- 

      k = 1

      if ( sfc_layer_type == sfc_layer_varthick ) then
        WORK2 = TAREA * (dz(k) + PSURF(:,:,curtime) / grav )
        WORK4 = TAREA * (dz(k) + PSURF(:,:,oldtime) / grav )
      else
        WORK2 = TAREA * dz(k)
        WORK4 = TAREA * dz(k)
      endif

      WORK2 = merge(WORK2, c0, CALCT)
      WORK4 = merge(WORK4, c0, CALCT)

      srf_volume = global_sum(WORK2, ONE)

      modified_volume_t = volume_t

      if ( sfc_layer_type == sfc_layer_varthick ) then
        modified_volume_t = volume_t - tarea_k(k) * dz(k) 
     &                     + srf_volume
      endif

!-----------------------------------------------------------------------
!
!     compute the sum of volume * tracers 
!
!-----------------------------------------------------------------------

      do n=1,nt

        do k=1,km

          if ( tmix_iopt == tmix_matsuno ) then
            if ( k == 1 ) then
              WORK1 = TRACER(:,:,k,n,curtime) * WORK2
            else
              WORK1 = TRACER(:,:,k,n,curtime)
            endif
          else
            if ( k == 1 ) then
              WORK1 = p5 * (TRACER(:,:,k,n,oldtime) * WORK4 
     &                    + TRACER(:,:,k,n,curtime) * WORK2)
            else
              WORK1 = p5 * (TRACER(:,:,k,n,oldtime)
     &                    + TRACER(:,:,k,n,curtime))
            endif
          endif

          if ( k == 1 ) then
            sum_tmp    = global_sum(WORK1, ONE)
            tracer_mean(n) = sum_tmp
          else
            WORK3 = merge (WORK1*TAREA, c0, KMT >= k)
            sum_tmp = global_sum(WORK3, ONE)
            tracer_mean(n) = tracer_mean(n) + sum_tmp * dz(k)
          endif

        enddo

      enddo

      end subroutine diag_for_tracer_budgets

!***********************************************************************

      subroutine tracer_budgets

!-----------------------------------------------------------------------
!
!     computes and prints global volume and tracer budgets for a given
!     time-averaging interval. because most of the analysis is based
!     on some time-averaged variables, the budgets are correct
!     if time-averaging starts from the first time step of an intended
!     budget interval. The following variables must be requested in 
!     the tavg_contents file:
!
!          SHF, SFWF, RESID_T, RESID_S, FW, TFW_T, TFW_S
!
!     the unit conversions are such that:
!
!            volume budget            ---->  cm / s
!            heat budget              ---->  W / m^2
!            virtual salt flux budget ---->  kg of fw   / m^2 / s
!            salt flux budget         ---->  kg of salt / m^2 / s
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) ::
     &  shf_mean,     sfwf_mean,
     &  resid_t_mean, resid_s_mean,
     &  qflux_t_mean, qflux_s_mean,
     &  tfw_t_mean,   tfw_s_mean,
     &  T_change,     S_change,
     &  fw_mean,      volume_change,
     &  tavg_norm,    sum

      character*132 :: explanation
 
!-----------------------------------------------------------------------

      tavg_norm = c1 / (tavg_sum * area_t)


!     compute mean SHF

      shf_mean = global_sum(TAVG_2D(:,:,tavg_bufloc(ntavg_SHF))
     &                      *TAREA, RCALCT) * tavg_norm


!     compute mean SFWF

      sfwf_mean = global_sum(TAVG_2D(:,:,tavg_bufloc(ntavg_SFWF))
     &                       *TAREA, RCALCT) * tavg_norm
      if ( sfc_layer_type == sfc_layer_varthick .and.
     &     .not. lfw_as_salt_flx ) then
        sfwf_mean = sfwf_mean
     &             * rho_sw * c10
      endif


!     compute mean RESID_T

      resid_t_mean = 
     &  global_sum(TAVG_2D(:,:,tavg_bufloc(ntavg_RESID_T))*TAREA,
     &             RCALCT) * tavg_norm
      resid_t_mean = - resid_t_mean


!     compute mean RESID_S

      resid_s_mean = 
     &  global_sum(TAVG_2D(:,:,tavg_bufloc(ntavg_RESID_S))*TAREA,
     &             RCALCT) * tavg_norm
      resid_s_mean = - resid_s_mean


!     compute mean heat and virtual salt fluxes due to ice formation

      qflux_t_mean = c0
      if ( tavg_sum_qflux /= c0 )
     &  qflux_t_mean = global_sum(TAVG_QFLUX*TAREA, RCALCT)
     &                / (tavg_sum_qflux * area_t)

      if ( sfc_layer_type == sfc_layer_varthick .and.
     &     .not. lfw_as_salt_flx ) then
        qflux_s_mean = c0
      else
        qflux_s_mean = - c10000*qflux_t_mean
     &             *  (ocn_ref_salinity - sea_ice_salinity)
     &             /   ocn_ref_salinity / latent_heat_fusion 
     &             /   rho_fw
      endif


!     compute mean FW

      fw_mean = global_sum(TAVG_2D(:,:,tavg_bufloc(ntavg_FW))
     &                     *TAREA, RCALCT) * tavg_norm


!     compute mean heat flux content of FW

      tfw_t_mean = global_sum(TAVG_2D(:,:,tavg_bufloc(ntavg_TFW_T))
     &                        *TAREA, RCALCT) * tavg_norm


!     compute mean salt flux content of FW

      tfw_s_mean = global_sum(TAVG_2D(:,:,tavg_bufloc(ntavg_TFW_S))
     &                        *TAREA, RCALCT) * tavg_norm


!     start computing time change of volume and tracers

      call diag_for_tracer_budgets (tracer_mean_final, volume_t_final)

      if ( my_task == master_task ) then
        write (stdout,999) volume_t_final,
     &                     tracer_mean_final(1),
     &                     salt_to_ppt * tracer_mean_final(2)
      endif

      volume_change = ( volume_t_final - volume_t_initial )
     &               * tavg_norm

      T_change = (  tracer_mean_final(1) - tracer_mean_initial(1) ) 
     &          * tavg_norm / hflux_factor

      S_change = (  tracer_mean_final(2) - tracer_mean_initial(2) ) 
     &          * tavg_norm
      if ( sfc_layer_type == sfc_layer_varthick .and.
     &     .not. lfw_as_salt_flx ) then
        S_change = S_change * rho_sw * c10
      else
        S_change = S_change/ salinity_factor
      endif

!-----------------------------------------------------------------------
!
!     print out the budget diagnostics
!
!-----------------------------------------------------------------------

      if ( my_task == master_task ) then

        write (stdout,1000) 
 
        if (budget_warning_1st_step) then
            write (stdout,10001)
            budget_warning_1st_step = .false.
        endif
 
        if (tmix_iopt == tmix_matsuno) write (stdout,1001 )

        write (stdout,1002)

        sum = - volume_change + fw_mean
        explanation = ' Imbalance = -tendency + FW flux '
        write (stdout,1003) volume_change, fw_mean, sum, explanation

        write (stdout,1004)
        sum = - T_change+ shf_mean + qflux_t_mean + tfw_t_mean 
     &        + resid_t_mean
 
        explanation = ' Imbalance           = -tendency + SHF flux'
     &  //' + ice formation + FW heat content + free-srf non-consv. '
 
        write (stdout, 1005) T_change, shf_mean, qflux_t_mean,
     &                       tfw_t_mean,  resid_t_mean, sum
     &,                      explanation

        if ( sfc_layer_type == sfc_layer_varthick .and.
     &       .not. lfw_as_salt_flx ) then
          write (stdout,1006)
        else
          write (stdout,1007)
        endif
        sum = - S_change + sfwf_mean + qflux_s_mean + tfw_s_mean 
     &        + resid_s_mean
        explanation = ' Imbalance = -tendency + SFWF flux'//
     &           ' + ice formation + FW salt content'//
     &           ' + free-srf non-consv. '
        write (stdout, 1008) S_change, sfwf_mean, qflux_s_mean,
     &                       tfw_s_mean,  resid_s_mean, sum
     &,                      explanation

      endif

      volume_t_initial    = volume_t_final
      tracer_mean_initial = tracer_mean_final

!-----------------------------------------------------------------------
!
!     format statements
!
!-----------------------------------------------------------------------

  999 format (/, 10x, 'AFTER CALL IN tracer_budgets (final):',
     &        /, 10x, '=====================================',
     &        /,  5x, ' volume_t (cm^3)           = ', e18.12,
     &        /,  5x, ' SUM [volume*T] (C   cm^3) = ', e18.12,
     &        /,  5x, ' SUM [volume*S] (ppt cm^3) = ', e18.12 )

 1000 format ( /, 10x, 'VOLUME AND TRACER BUDGETS:',
     &         /, 10x, '==========================',
     &     //,  5x, 'Please note the following about the budgets:     '     
     &,                      //, 5x, '*', 3x
     &,             'These budgets are meaningful when time-averaging '
     &,     /,  9x, 'starts at the first time step of an intended     '     
     &,     /,  9x, 'budget interval.                                 '     
     &,                      //, 5x, '*', 3x
     &,             'Positive-signed quantities ==> ocean gain        '     
     &,     /,  9x, 'Negative-signed quantities ==> ocean loss        '     
     &,                      //, 5x, '*', 3x
     &,             'The imbalance should be small in comparison      '
     &,     /,  9x, 'to the tendency term, except for the volume      '
     &,     /,  9x, 'budget when a fixed-volume ocean is used         '
     &,     /,  9x, '(e.g., when lfw_as_salt_flx = .false.)           '
     &,                      //, 5x, '*', 3x
     &,   'These budgets are computed from single-precision TAVG terms,' 
     &,     /,  9x, 'which contributes to inaccuracy in the imbalance '
     &,     /,  9x, 'term.')
 
10001 format(/, 5x, 'WARNING: Because the very first time step of a   ',
     &      /,  4x, '  new integration is an Euler-forward time step, ',
     &      /,  4x, '  any budget including this first step will be',
     &      /,  4x, '  slightly off balance.' )

 1001 format(  /,  5x,'WARNING: These budget diagnostics were designed',
     &      /,  4x, '  for use with the "avg" or "avgfit" mixing      ',
     &      /,  4x, '  options. You have selected the Matsuno mixing  ',
     &      /,  4x, '  option, for which these diagnostics are only   ',
     &      /,  4x, '  approximations.                                ')
 
 1002 format ( /, 5x, 'VOLUME BUDGET FOR THE GIVEN TAVG INTERVAL',
     &                ' (cm/s):',
     &         /, 5x, '-----------------------------------------',
     &                '--------' )

 1003 format ( /, 7x, ' tendency  = ', e18.12,
     &         /, 7x, ' FW flux   = ', e18.12,
     &         /, 7x, ' Imbalance = ', e18.12,
     &         /, 7x,   a                    )

 1004 format ( /, 5x, 'HEAT BUDGET FOR THE GIVEN TAVG INTERVAL',
     &                ' (W/m^2):',
     &         /, 5x, '---------------------------------------',
     &                '---------' )

 1005 format ( /, 7x, ' tendency            = ', e18.12,
     &         /, 7x, ' SHF flux            = ', e18.12,
     &         /, 7x, ' ice formation       = ', e18.12,
     &         /, 7x, ' FW heat content     = ', e18.12,
     &         /, 7x, ' free-srf non-consv. = ', e18.12,
     &         /, 7x, ' Imbalance           = ', e18.12,
     &         /, 7x,   a                              )

 1006 format ( /, 5x, 'SALT BUDGET FOR THE GIVEN TAVG INTERVAL',
     &                ' (kg of salt/m^2/s):',
     &         /, 5x, '---------------------------------------',
     &                '--------------------' )
 1007 format ( /, 5x, 'IMPLIED FW BUDGET FOR THE GIVEN TAVG INTERVAL',
     &                ' (kg of freshwater/m^2/s):',
     &         /, 5x, '---------------------------------------------',
     &                '--------' )

 1008 format ( /, 7x, ' tendency            = ', e18.12,
     &         /, 7x, ' SFWF flux           = ', e18.12,
     &         /, 7x, ' ice formation       = ', e18.12,
     &         /, 7x, ' FW salt content     = ', e18.12,
     &         /, 7x, ' free-srf non-consv. = ', e18.12,
     &         /, 7x, ' Imbalance           = ', e18.12,
     &         /, 7x,   a                              )

      end subroutine tracer_budgets

!***********************************************************************

      end module budget_diagnostics

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
