!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module constants

!***********************************************************************
!
!     This module defines a variety of physical and numerical constants
!     used throughout POP.
!
!     This version of the constants module references values of constants
!     that are defined in the csm_share/shr_const_mod module.  The original
!     LANL constant definitions are kept in this module as comments.
!
!     CVS:$Id: constants.F,v 1.3.2.8 2002/03/04 23:33:34 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use domain
      use shr_const_mod

      implicit none
      save

!-----------------------------------------------------------------------
!
!     numbers
!
!-----------------------------------------------------------------------

      character (char_len) ::
     &  char_blank          ! empty character string

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  ONE                 ! array of ones which is often useful

      real (kind=dbl_kind), parameter ::
     &  c0    = 0.0_dbl_kind,
     &  c1    = 1.0_dbl_kind,
     &  c2    = 2.0_dbl_kind,
     &  c3    = 3.0_dbl_kind,
     &  c4    = 4.0_dbl_kind,
     &  c5    = 5.0_dbl_kind,
     &  c8    = 8.0_dbl_kind,
     &  c10   = 10.0_dbl_kind,
     &  c16   = 16.0_dbl_kind,
     &  c1000 = 1000.0_dbl_kind,
     &  c10000= 10000.0_dbl_kind,
     &  c1p5  = 1.5_dbl_kind,
     &  p33   = c1/c3,
     &  p5    = 0.5_dbl_kind,
     &  p25   = 0.25_dbl_kind,
     &  p125  = 0.125_dbl_kind,
     &  p001  = 0.001_dbl_kind,
     &  eps   = 1.0e-10_dbl_kind,
     &  eps2  = 1.0e-20_dbl_kind

!-----------------------------------------------------------------------
!
!     conversion factors
!
!-----------------------------------------------------------------------

!     original LANL
!     real (kind=dbl_kind), parameter :: 
!    &  T0_Kelvin     = 273.16_dbl_kind      ! zero point for Celcius
 
 
!     ccsm
      real (kind=dbl_kind), parameter :: 
     &  T0_Kelvin     = SHR_CONST_TKFRZ
 
      real (kind=dbl_kind), parameter :: 
     &  mpercm        = .01_dbl_kind         ! meters per cm
     &, cmperm        = 100._dbl_kind        ! cm per meter
     &, salt_to_ppt   = 1000._dbl_kind       ! salt (g/g) to ppt
     &, ppt_to_salt   = 1.e-3_dbl_kind       ! salt ppt to g/g
     &, mass_to_Sv    = 1.0e-12_dbl_kind     ! mass flux to Sverdrups   
     &, heat_to_PW    = 4.186e-15_dbl_kind   ! heat flux to Petawatts 
     &, salt_to_Svppt = 1.0e-9_dbl_kind      ! salt flux to Sv*ppt
     &, salt_to_mmday = 3.1536e+5_dbl_kind   ! salt to water (mm/day)

      real (kind=dbl_kind) :: 
     &  radian               ! degree-radian conversion
 
 
!-----------------------------------------------------------------------
!
!     physical constants (all in cgs units except for those in MKS)
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!       original LANL definitions:
!     real (kind=dbl_kind), parameter ::
!    &  grav      = 980.6_dbl_kind,          ! gravit. accel. (cm/s**2)
!    &  omega     = 7.292123625e-5_dbl_kind, ! angular vel. of Earth 1/s
!    &  radius    = 6370.0e5_dbl_kind,       ! radius of Earth (cm)
!    &  cp_sw     = 3.996e7_dbl_kind,        ! specific heat salt water
!    &  cp_air    = 1005.0_dbl_kind,         ! heat capacity of air (J/kg/K)
!    &  rho_air   = 1.2_dbl_kind,            ! ambient air density (kg/m^3)
!    &  rho_sw = 4.1_dbl_kind/3.996_dbl_kind,! (g/cm^3)  (cgs)
!    &  sound     = 1.5e5_dbl_kind,          ! speed of sound (cm/s)
!    &  vonkar    = 0.4_dbl_kind,            ! von Karman constant
!    &  emissivity         = 1.0_dbl_kind,
!    &  stefan_boltzmann   = 567.0e-10_dbl_kind, !  W/m^2/K^4
!    &  latent_heat_vapor  = 2.5e6_dbl_kind, ! latent heat of vaporization (J/kg)
!    &  latent_heat_fusion = 3.34e9_dbl_kind,! latent heat of fusion (erg/g)
!    &  ocn_ref_salinity   = 34.7_dbl_kind   ! (psu)
 
 

!       ccsm definitions:
      real (kind=dbl_kind), parameter ::
     &  grav               = SHR_CONST_G*cmperm      ! cm/s^2   
     &, omega              = SHR_CONST_OMEGA         ! rad/s
     &, radius             = SHR_CONST_REARTH*cmperm ! cm
     &, cp_sw              = SHR_CONST_CPSW*c10000   ! erg/g/K
     &, cp_air             = SHR_CONST_CPDAIR        ! J/kg/K
     &, rho_air            = SHR_CONST_RHODAIR       ! kg/m^3
     &, rho_sw             = SHR_CONST_RHOSW*p001    ! g/cm^3
     &, rho_fw             = SHR_CONST_RHOFW*p001    ! g/cm^3 
     &, sound              = 1.5e5_dbl_kind          ! cm/s 
     &, vonkar             = SHR_CONST_KARMAN      
     &, emissivity         = 1.0_dbl_kind 
     &, stefan_boltzmann   = SHR_CONST_STEBOL        ! W/m^2/K^4
     &, latent_heat_vapor  = SHR_CONST_LATVAP        ! J/kg
     &, latent_heat_fusion = SHR_CONST_LATICE*c10000 ! erg/g
     &, ocn_ref_salinity   = SHR_CONST_OCN_REF_SAL   ! psu
     &, sea_ice_salinity   = SHR_CONST_ICE_REF_SAL   ! psu

      real (kind=dbl_kind) :: 
     &  pi, pih, pi2         ! pi, pi/2 and 2pi


!-----------------------------------------------------------------------
!
!   Flux conversions:
!
!    velocity       flux = windstress / rho
!    temperature    flux = surface heat flux / rho / cp
!    salinity       flux = surface freshwater flux
!                          * ocean reference salinity
!                          / rho(freshwater)
!
!   Units:
!
!    windstress           (N/m^2)
!    velocity       flux  (cm^2/s^2)
!
!    heat           flux  (W/m^2)
!    temperature    flux  (C*cm/s)
!
!    freshwater     flux  (Kg/m^2/s)
!    salinity       flux  (msu*cm/s)
!
!
!      convert windstress to velocity flux:
!      ---------------------------------------
!      windstress in (N/m^2) = (kg/s^2/m) = 10(g/s^2/cm) = 10(dyn/cm^2)
!      assume here that density of seawater rho = 1 (g/cm^3)
!      (because momentum equations assume rho = 1)
!
!      vel_flux   = windstress / rho
!
!      vel_flux (cm^2/s^2)
!                 = windstress (N/m^2)
!                 * 10 (g/s^2/cm)/(N/m^2)
!                 / [1 (g/cm^3)]
!
!                 = windstress (N/m^2)
!                 * momentum_factor (cm^2/s^2)/N/m^2)
!
!      ==>  momentum_factor = 10
!
!
!      convert heat and solar flux to temperature flux:
!      -----------------------------------------------
!      heat_flux in (W/m^2) = (J/s/m^2) = 1000(g/s^3)
!      density of seawater rho_sw in (g/cm^3)
!      specific heat of seawater cp_sw in (erg/g/C) = (cm^2/s^2/C)
!
!      temp_flux = heat_flux / (rho_sw*cp_sw)
!
!      temp_flux (C*cm/s)
!                 = heat_flux (W/m^2)
!                 * 1000 (g/s^3)/(W/m^2)
!                 / [(rho_sw*cp_sw) (g/cm/s^2/C)]
!
!                 = heat_flux (W/m^2)
!                 * hflux_factor (C*cm/s)/(W/m^2)
!
!      ==>  hflux_factor = 1000/(rho_sw*cp_sw)
!
!
!      convert net fresh water flux to salt flux (in model units):
!      ----------------------------------------------------------
!      ocean reference salinity in (o/oo=psu)
!      density of freshwater rho_fw = 1.0 (g/cm^3)
!      h2o_flux in (kg/m^2/s) = 0.1 (g/cm^2/s)
!
!      salt_flux  = - h2o_flux * ocn_ref_salinity / rho_fw
!
!      salt_flux (msu*cm/s)
!                 = - h2o_flux (kg/m^2/s)
!                 * ocn_ref_salinity (psu)
!                 * 1.e-3 (msu/psu)
!                 * 0.1 (g/cm^2/s)/(kg/m^2/s)
!                 / 1.0 (g/cm^3)
!
!                 = - h2o_flux (kg/m^2/s)
!                 * ocn_ref_salinity (psu)
!                 * fwflux_factor (cm/s)(msu/psu)/(kg/m^2/s)
!
!      ==>  fwflux_factor = 1.e-4
!
!      salt_flux(msu*cm/s) = h2oflux(kg/m^2/s) * salinity_factor
!
!      ==> salinity_factor = - ocn_ref_salinity(psu) * fwflux_factor
!
!      convert salt flux to salt flux (in model units):
!      ----------------------------------------------------------
!      density of freshwater rho_fw = 1.0 (g/cm^3)
!      salt_flux_kg in (kg/m^2/s) = 0.1 (g/cm^2/s)
!
!      salt_flux  = - h2o_flux * ocn_ref_salinity / rho_fw
!
!      salt_flux (msu*cm/s)
!                 = salt_flux_kg (kg/m^2/s)
!                 * 0.1 (g/cm^2/s)/(kg/m^2/s)
!                 / 1.0 (g/cm^3)
!
!                 = salt_flux_kg (kg/m^2/s)
!                 * sflux_factor (msu*cm/s)/(kg/m^2/s)
!
!      ==>  sflux_factor = 0.1
!
!      convert fresh water mass flux to fresh water flux (in cm/s):
!      ------------------------------------------------------------
!      density of freshwater rho_fw = 1.0 (g/cm^3)
!      h2o_flux in (kg/m^2/s) = 0.1 (g/cm^2/s)
!
!      fw_flux  = h2o_flux / rho_fw
!
!      fw_flux (cm/s)
!                 = h2o_flux (kg/m^2/s)
!                 * 0.1 (g/cm^2/s)/(kg/m^2/s)
!                 / 1.0 (g/cm^3)
!
!                 = h2o_flux (kg/m^2/s)
!                 * fwmass_to_fwflux (cm/s)/(kg/m^2/s)
!
!      ==>  fwmass_to_fwflux = 0.1
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), parameter :: 
     &  momentum_factor  = 10.0_dbl_kind
     &, hflux_factor = 1000.0_dbl_kind / (rho_sw * cp_sw)
     &, fwflux_factor =  1.e-4_dbl_kind
     &, salinity_factor =  - ocn_ref_salinity * fwflux_factor
     &, sflux_factor = 0.1_dbl_kind
     &, fwmass_to_fwflux = 0.1_dbl_kind

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_constants

      implicit none

!-----------------------------------------------------------------------
!
!     This subroutine initializes constants that are best defined
!     at run time (e.g. pi).
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n

!-----------------------------------------------------------------------

      ONE = c1

!     original LANL
!     pi  = c4*atan(c1)
 
!     csm
      pi  = SHR_CONST_PI
      pi2 = c2*pi
      pih = p5*pi

      radian = 180.0_dbl_kind/pi

      do n=1,char_len
        char_blank(n:n) = ' '
      end do

!-----------------------------------------------------------------------

      end subroutine init_constants


!***********************************************************************

      end module constants

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
