!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module diagnostics

!***********************************************************************
!
!     This module contains routines and data for producing diagnostic 
!     output, including global diagnostics, cfl checks and transport
!     diagnostics.
!
!     CVS:$Id: diagnostics.F,v 1.3.2.9 2002/03/04 23:33:34 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use time_management
      use io
      use global_reductions
      use grid 
      use solvers
      use forcing
      use timers
      use exit_mod
      use shr_sys_mod

      implicit none
      save

!-----------------------------------------------------------------------
!
!     variables for controlling frequency and output of diagnostics
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  diag_unit,            ! i/o unit for output diagnostic file
     &  trans_unit            ! i/o unit for output transport file

      integer (kind=int_kind) ::
     &  diag_global_flag       ! time flag id for global diags
     &, diag_cfl_flag          ! time flag id for cfl    diags
     &, diag_transp_flag       ! time flag id for transp diags

      logical (kind=log_kind) ::
     &  ldiag_global           ! time to compute global diagnostics
     &, ldiag_cfl              ! time to compute cvl diagnostics
     &, ldiag_transport        ! time to compute transport diagnostics
     &, cfl_all_levels
     &, diag_all_levels        ! writes some diags for all vert levels


      character (char_len_long) ::
     &  diag_outfile                ! filename for diagnostic output
     &, diag_outfile_old            ! previous diag_outfile name
     &, diag_outfile_root           ! root filename for diagnostic output
     &, diag_transport_outfile      ! filename for transport output
     &, diag_transport_outfile_old  ! previous diag_transport_outfile name
     &, diag_transport_outfile_root ! root filename for transport output

      character (char_len) ::
     &  ccsm_diag_date        ! character date

!-----------------------------------------------------------------------
!
!     global diagnostics
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) :: 
     &  diag_ke,         ! mean KE at new time
     &  diag_ke_adv,     ! KE change due to advection
     &  diag_ke_free,    ! advective KE change due to free surface
     &  diag_ke_hmix,    ! KE change due to horizontal diffusion
     &  diag_ke_vmix,    ! KE change due to vertical   diffusion
     &  diag_ke_press,   ! KE change due to pressure gradient
     &  diag_pe,         ! change in potential energy
     &  diag_press_free, ! pressure work contribution from free surface
     &  diag_ke_psfc,    ! KE change due to surface pressure
     &  diag_ws,         ! KE change due to wind stress
     &  diag_sealevel    ! global mean seal level

      real (kind=dbl_kind), dimension(nt) :: 
     &  dtracer_avg,        ! avg tracer change for this time step
     &  dtracer_abs,        ! abs tracer change for this time step
     &  diag_tracer_adv,    ! tracer change due to advection
     &  diag_tracer_hdiff,  ! tracer change due to horiz diffusion
     &  diag_tracer_vdiff,  ! tracer change due to vert  diffusion
     &  diag_tracer_source, ! tracer change due to source terms 
     &  avg_tracer,         ! global average tracer at new time
     &  sfc_tracer_flux     ! sfc tracer flux (Fw*tracer) or
                            !   resid sfc flux (w*tracer) 

      real (kind=dbl_kind), dimension(km,nt) :: 
     &  avg_tracer_k        ! mean tracer at every level

!-----------------------------------------------------------------------
!
!     global budget diagnostics
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(nt) :: 
     &  tracer_mean_initial,    ! SUM [volume*tracer] at the beginning  
                                ! of a time-averaging interval
     &  tracer_mean_final       ! SUM [volume*tracer] at the end of a
                                ! time-averaging interval

      real (kind=dbl_kind) ::
     &  volume_t_initial,       ! T-point volume at the beginning of a
                                ! time-averaging interval
     &  volume_t_final          ! T-point volume at the end of a
                                ! time-averaging interval

!-----------------------------------------------------------------------
!
!     2d arrays for incrementing vertical sums of global balances
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  DIAG_KE_ADV_2D,
     &  DIAG_KE_HMIX_2D,
     &  DIAG_KE_VMIX_2D,
     &  DIAG_KE_PRESS_2D,
     &  DIAG_PE_2D 

      real (kind=dbl_kind), dimension(imt,jmt,nt) :: 
     &  DIAG_TRACER_ADV_2D,
     &  DIAG_TRACER_HDIFF_2D,
     &  DIAG_TRACER_VDIFF_2D,
     &  DIAG_TRACER_SOURCE_2D

!-----------------------------------------------------------------------
!
!     cfl diagnostics
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(km) ::
     &  cfl_advuk,          ! zonal advective cfl number for level k
     &  cfl_advvk,          ! merid advective cfl number for level k
     &  cfl_advwk,          ! vert  advective cfl number for level k
     &  cfl_advtk,          ! total advective cfl number for level k
     &  cfl_vdifftk,        ! tracer vert diff cfl num for level k
     &  cfl_vdiffuk,        ! momentum vert diff cfl num for level k
     &  cfl_hdifftk,        ! tracer horiz diff cfl num for level k
     &  cfl_hdiffuk         ! momentum horiz diff cfl num for level k

      integer (kind=int_kind), dimension(2,km) ::
     &  cfladd_advuk,       ! horiz addresses for max cfl numbers at
     &  cfladd_advvk,       !   level k
     &  cfladd_advwk,
     &  cfladd_advtk,
     &  cfladd_vdifftk,
     &  cfladd_vdiffuk,
     &  cfladd_hdifftk,
     &  cfladd_hdiffuk 

!-----------------------------------------------------------------------
!
!     transport diagnostics
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter ::
     &  transport_type_zonal = 1,  ! zonal or meridional transport type
     &  transport_type_merid = 2

      integer (kind=int_kind) ::
     &  num_transports      ! number of transport diagnostics computed

      integer (kind=int_kind), dimension(:), allocatable ::
     &  transport_type      ! type (merid or zonal) of each transport

      integer (kind=int_kind), dimension(:,:,:), allocatable ::
     &  transport_add       ! local logical addresses for each transport

      character (char_len), dimension(:), allocatable ::
     &  transport_name      ! name for each transport

      real (kind=dbl_kind), dimension(:,:), allocatable ::
     &  transports          ! mass, heat and salt transport sums

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_diagnostics

!-----------------------------------------------------------------------
!
!     initializes diagnostic info
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  nu,                 ! unit for contents input file
     &  nfield,             ! dummy for field number
     &  i,j,n,              ! dummy loop indices
     &  nml_error           ! namelist i/o error flag
     &, cindx               ! character index
     &, len_date            ! length of date character variable

      integer (kind=int_kind), dimension(6) ::
     &  tmp_add             ! transport global addresses 
                            ! (ibeg,iend,jbeg,jend,kbeg,kend)

      character (char_len) ::
     &  diag_global_freq_opt, ! choice for freq of global diagnostics
     &  diag_cfl_freq_opt,    ! choice for freq of cfl diagnostics
     &  diag_transp_freq_opt, ! choice for freq of transport diagnostics
     &  diag_transport_file,  ! filename for choosing fields for output
     &  transport_ctype,      ! type of transport (zonal,merid)
     &  outfile_tmp           ! temp for appending to outfile name

      integer (kind=int_kind) ::
     &  diag_global_freq_iopt, ! freq option for computing global diags
     &  diag_global_freq,      ! freq for computing global diagnostics
     &  diag_cfl_freq_iopt,    ! freq option for computing cfl diags
     &  diag_cfl_freq,         ! freq for computing cfl diagnostics
     &  diag_transp_freq_iopt, ! freq option for comp transport diags
     &  diag_transp_freq       ! freq for computing transp diagnostics

      namelist /diagnostics_nml/ 
     &                   diag_global_freq_opt, diag_global_freq,
     &                   diag_cfl_freq_opt,    diag_cfl_freq,
     &                   diag_transp_freq_opt, diag_transp_freq,
     &                   diag_transport_file, 
     &                   diag_all_levels, cfl_all_levels,
     &                   diag_outfile, diag_transport_outfile 

!-----------------------------------------------------------------------
!
!     read diagnostic file output frequency and filenames from namelist
!
!-----------------------------------------------------------------------

      diag_global_freq_opt   = 'never'
      diag_cfl_freq_opt      = 'never'
      diag_transp_freq_opt   = 'never'
      diag_transport_file    = 'unknown_transport_file'
      diag_outfile           = 'unknown_diag_outfile'
      diag_transport_outfile = 'unknown_transport_outfile'
      diag_all_levels        = .false.
      cfl_all_levels         = .false.

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read (nml_in, nml=diagnostics_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR: reading diagnostics_nml')
      endif

      call broadcast_scalar(diag_outfile          , master_task)
      call broadcast_scalar(diag_transport_outfile, master_task)
 
      diag_outfile_root           = diag_outfile
      diag_transport_outfile_root = diag_transport_outfile

      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, diagnostics_nml)
          write(stdout,*) ' '
      endif



!-----------------------------------------------------------------------
!
!     set, broadcast and print options to log file
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        write(stdout,*) ' '
        write(stdout,"(72('-'))")
        write(stdout,*) ' '
        write(stdout,*) 'Diagnostic options'
        write(stdout,*) ' '
        write(stdout,"(72('-'))")

        select case (diag_global_freq_opt)
        case ('never')
          diag_global_freq_iopt = freq_opt_never
          write(stdout,*) 'Global diagnostics not computed'
        case ('nyear')
          diag_global_freq_iopt = freq_opt_nyear
          write(stdout,*) 'Global diagnostics computed every ',
     &                    diag_global_freq, ' years'
        case ('nmonth')
          diag_global_freq_iopt = freq_opt_nmonth
          write(stdout,*) 'Global diagnostics computed every ',
     &                    diag_global_freq, ' months'
        case ('nday')
          diag_global_freq_iopt = freq_opt_nday
          write(stdout,*) 'Global diagnostics computed every ',
     &                    diag_global_freq, ' days'
        case ('nhour')
          diag_global_freq_iopt = freq_opt_nhour
          write(stdout,*) 'Global diagnostics computed every ',
     &                    diag_global_freq, ' hours'
        case ('nsecond')
          diag_global_freq_iopt = freq_opt_nsecond
          write(stdout,*) 'Global diagnostics computed every ',
     &                    diag_global_freq, ' seconds'
        case ('nstep')
          diag_global_freq_iopt = freq_opt_nstep
          write(stdout,*) 'Global diagnostics computed every ',
     &                    diag_global_freq, ' steps'
        case default
          diag_global_freq_iopt = -1000
        end select

!---------------------------------------------------------------------
!       ccsm date string will be appended after _last_ record
!         is written, so actual file name is not known at this time
!---------------------------------------------------------------------
 
        if (diag_global_freq_iopt /= freq_opt_never) then
          write(stdout,*) 'Global diagnostics written to file:',
     &      trim(diag_outfile) // '_yyyy-mm-dd-sssss'
        endif

        select case (diag_cfl_freq_opt)
        case ('never')
          diag_cfl_freq_iopt = freq_opt_never
          write(stdout,*) 'CFL diagnostics not computed'
        case ('nyear')
          diag_cfl_freq_iopt = freq_opt_nyear
          write(stdout,*) 'CFL diagnostics computed every ',
     &                    diag_cfl_freq, ' years'
        case ('nmonth')
          diag_cfl_freq_iopt = freq_opt_nmonth
          write(stdout,*) 'CFL diagnostics computed every ',
     &                    diag_cfl_freq, ' months'
        case ('nday')
          diag_cfl_freq_iopt = freq_opt_nday
          write(stdout,*) 'CFL diagnostics computed every ',
     &                    diag_cfl_freq, ' days'
        case ('nhour')
          diag_cfl_freq_iopt = freq_opt_nhour
          write(stdout,*) 'CFL diagnostics computed every ',
     &                    diag_cfl_freq, ' hours'
        case ('nsecond')
          diag_cfl_freq_iopt = freq_opt_nsecond
          write(stdout,*) 'CFL diagnostics computed every ',
     &                    diag_cfl_freq, ' seconds'
        case ('nstep')
          diag_cfl_freq_iopt = freq_opt_nstep
          write(stdout,*) 'CFL diagnostics computed every ',
     &                    diag_cfl_freq, ' steps'
        case default
          diag_cfl_freq_iopt = -1000
        end select

        if (diag_cfl_freq_iopt /= freq_opt_never) then
!---------------------------------------------------------------------
!       ccsm date string will be appended after _last_ record
!         is written, so actual file name is not known at this time
!---------------------------------------------------------------------
          write(stdout,*) 'CFL diagnostics written to file:',
     &      trim(diag_outfile) // '_yyyy-mm-dd-sssss'
          if (cfl_all_levels) then
            write(stdout,*) 
     &        'CFL diagnostics output for all vertical levels'
          endif
        endif

        if (diag_all_levels .and. 
     &      diag_global_freq_iopt /= freq_opt_never) then
          write(stdout,*) 
     &        'Diagnostics output for all vertical levels'
        endif

        select case (diag_transp_freq_opt)
        case ('never')
          diag_transp_freq_iopt = freq_opt_never
          write(stdout,*) 'Transport diagnostics not computed'
        case ('nyear')
          diag_transp_freq_iopt = freq_opt_nyear
          write(stdout,*) 'Transport diagnostics computed every ',
     &                    diag_transp_freq, ' years'
        case ('nmonth')
          diag_transp_freq_iopt = freq_opt_nmonth
          write(stdout,*) 'Transport diagnostics computed every ',
     &                    diag_transp_freq, ' months'
        case ('nday')
          diag_transp_freq_iopt = freq_opt_nday
          write(stdout,*) 'Transport diagnostics computed every ',
     &                    diag_transp_freq, ' days'
        case ('nhour')
          diag_transp_freq_iopt = freq_opt_nhour
          write(stdout,*) 'Transport diagnostics computed every ',
     &                    diag_transp_freq, ' hours'
        case ('nsecond')
          diag_transp_freq_iopt = freq_opt_nsecond
          write(stdout,*) 'Transport diagnostics computed every ',
     &                    diag_transp_freq, ' seconds'
        case ('nstep')
          diag_transp_freq_iopt = freq_opt_nstep
          write(stdout,*) 'Transport diagnostics computed every ',
     &                    diag_transp_freq, ' steps'
        case default
          diag_transp_freq_iopt = -1000
        end select


      endif

      call broadcast_scalar(diag_global_freq_iopt, master_task)
      call broadcast_scalar(diag_global_freq,      master_task)
      call broadcast_scalar(diag_cfl_freq_iopt,    master_task)
      call broadcast_scalar(diag_cfl_freq,         master_task)
      call broadcast_scalar(diag_transp_freq_iopt, master_task)
      call broadcast_scalar(diag_transp_freq,      master_task)
      call broadcast_scalar(diag_all_levels,       master_task)
      call broadcast_scalar(cfl_all_levels,        master_task)

      if (diag_global_freq_iopt == -1000) then
        call exit_POP('ERROR: unknown global diag frequency option')
      endif
      if (diag_cfl_freq_iopt == -1000) then
        call exit_POP('ERROR: unknown cfl diagnostic frequency option')
      endif
      if (diag_transp_freq_iopt == -1000) then
        call exit_POP('ERROR: unknown transport diag frequency option')
      endif

      diag_global_flag = init_time_flag('diag_global', default=.false.,
     &                                freq_opt = diag_global_freq_iopt,
     &                                freq     = diag_global_freq)

      diag_cfl_flag    = init_time_flag('diag_cfl', default=.false.,
     &                                freq_opt = diag_cfl_freq_iopt,
     &                                freq     = diag_cfl_freq)

      diag_transp_flag = init_time_flag('diag_transp', default=.false.,
     &                                freq_opt = diag_transp_freq_iopt,
     &                                freq     = diag_transp_freq)

!-----------------------------------------------------------------------
!
!     read transports file to determine which transports to compute
!
!-----------------------------------------------------------------------

      if (diag_transp_freq_iopt /= freq_opt_never) then

        call get_unit(nu)
        if (my_task == master_task) then
!---------------------------------------------------------------------
!       ccsm date string will be appended after _last_ record
!         is written, so actual file name is not known at this time
!---------------------------------------------------------------------
          write(stdout,*) 'Transport diagnostics written to file:',
     &              trim(diag_transport_outfile)// '_yyyy-mm-dd-sssss'
          write(stdout,*) ' '

          open(nu, file=diag_transport_file, status='old')
          read(nu,*) num_transports

          write(stdout,*) 'The following ',num_transports,
     &                    'transports will be computed'
        endif

        call broadcast_scalar(num_transports, master_task)

        allocate( transport_add (3,2,num_transports),
     &            transport_name    (num_transports),
     &            transport_type    (num_transports),
     &            transports      (3,num_transports) )

        do n=1,num_transports
          if (my_task == master_task) then
            read(nu,1000) tmp_add, transport_ctype, transport_name(n)
 1000       format (6(i4,1x),a5,2x,a80)
            write(stdout,*) '  ',trim(transport_name(n))

            select case (transport_ctype(1:5)) 
            case ('zonal')
              transport_type(n) = transport_type_zonal
            case ('merid')
              transport_type(n) = transport_type_merid
            case default
              transport_type(n) = -1000
            end select
          endif

          call broadcast_scalar(transport_type(n),master_task)
          call broadcast_vector(tmp_add,master_task,6)
          if (transport_type(n) == -1000) then
            call exit_POP('ERROR: unknown transport type')
          endif

          !***
          !*** compute local addresses for transport
          !***

          transport_add(1,1,n) = imt
          transport_add(1,2,n) = 1
          transport_add(2,1,n) = jmt
          transport_add(2,2,n) = 1
          transport_add(3,1,n) = tmp_add(5)  ! beginning k index
          transport_add(3,2,n) = tmp_add(6)  ! ending    k index

          if (tmp_add(1) <= i_global(iphys_e) .and.
     &        tmp_add(2) >= i_global(iphys_b) ) then
            if (tmp_add(1) >= i_global(iphys_b)) then
              transport_add(1,1,n) = 
     &          tmp_add(1) - i_global(iphys_b) + iphys_b
            else
              transport_add(1,1,n) = iphys_b
            endif
            if (tmp_add(2) <= i_global(iphys_e)) then
              transport_add(1,2,n) =
     &          tmp_add(2) - i_global(iphys_b) + iphys_b
            else
              transport_add(1,2,n) = iphys_e
            endif
          endif

          if (tmp_add(3) <= j_global(jphys_e) .and.
     &        tmp_add(4) >= j_global(jphys_b) ) then
            if (tmp_add(3) >= j_global(jphys_b)) then
              transport_add(2,1,n) =
     &          tmp_add(3) - j_global(jphys_b) + jphys_b
            else
              transport_add(2,1,n) = jphys_b
            endif
            if (tmp_add(4) <= j_global(jphys_e)) then
              transport_add(2,2,n) =
     &          tmp_add(4) - j_global(jphys_b) + jphys_b
            else
              transport_add(2,2,n) = jphys_e
            endif
          endif

        end do

        if (my_task == master_task) close(nu)
        call release_unit(nu)
      endif

!-----------------------------------------------------------------------
!
!     open output files if required, then write a blank and close.
!
!-----------------------------------------------------------------------

      if (diag_global_freq_iopt /= freq_opt_never .or.
     &    diag_cfl_freq_iopt    /= freq_opt_never) then
 
        call get_unit(diag_unit)
        if (my_task == master_task) then
          open(diag_unit, file=diag_outfile, status='unknown')
          write(diag_unit,*)' '
          close(diag_unit)
        endif
      endif
 
      if (diag_transp_freq_iopt /= freq_opt_never) then
        call get_unit(trans_unit)
        if (my_task == master_task) then
          open(trans_unit, file=diag_transport_outfile, 
     &         status='unknown')
          write(trans_unit,*)' '
          close(trans_unit)
        endif
      endif

!-----------------------------------------------------------------------
!
!     initialize cfl arrays if required
!
!-----------------------------------------------------------------------

      if (diag_cfl_freq_iopt /= freq_opt_never) then
        cfl_advuk       = c0
        cfl_advvk       = c0
        cfl_advwk       = c0
        cfl_advtk       = c0
        cfl_vdifftk     = c0
        cfl_vdiffuk     = c0
        cfl_hdifftk     = c0
        cfl_hdiffuk     = c0
        cfladd_advuk    = 0
        cfladd_advvk    = 0
        cfladd_advwk    = 0
        cfladd_advtk    = 0
        cfladd_vdifftk  = 0
        cfladd_vdiffuk  = 0
        cfladd_hdifftk  = 0
        cfladd_hdiffuk  = 0
      endif


!-----------------------------------------------------------------------

      end subroutine init_diagnostics

!***********************************************************************

      subroutine diag_init_sums

!-----------------------------------------------------------------------
!
!     sets diagnostic flags and initializes diagnostic sums
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     set logical switches for diagnostic timesteps
!     do not compute on first pass of Matsuno
!
!-----------------------------------------------------------------------

      ldiag_global    = check_time_flag(diag_global_flag)
      ldiag_cfl       = check_time_flag(diag_cfl_flag)
      ldiag_transport = check_time_flag(diag_transp_flag)

      if (mix_pass == 1) then
        ldiag_global    = .false.
        ldiag_cfl       = .false.
        ldiag_transport = .false.
      endif

!-----------------------------------------------------------------------
!
!     initialize arrays for global diagnostic sums.
!
!-----------------------------------------------------------------------

      if (ldiag_global) then
        DIAG_KE_ADV_2D  = c0
        DIAG_KE_HMIX_2D = c0
        DIAG_KE_VMIX_2D = c0
        DIAG_KE_PRESS_2D = c0
        DIAG_PE_2D = c0
        DIAG_TRACER_ADV_2D = c0
        DIAG_TRACER_HDIFF_2D = c0
        DIAG_TRACER_VDIFF_2D = c0
        DIAG_TRACER_SOURCE_2D = c0
      endif

!-----------------------------------------------------------------------
!
!     write some stuff to log file (stdout)
!
!-----------------------------------------------------------------------

      if ((ldiag_global .or. ldiag_cfl) .and. 
     &    my_task == master_task) then
        write(stdout,*) ' '
        write(stdout,*) 'Step number: ',nsteps_total
        write(stdout,*) 'Date       : ',cday,' ',cmonth3,' ',cyear
        write(stdout,*) 'Hour       : ',ihour
        write(stdout,*) 'Minute     : ',iminute
        write(stdout,*) 'Second     : ',isecond

        if (tsecond < seconds_in_hour) then
         write(stdout,*)'Time       :', tsecond00
     &,   ' in seconds since 0000-01-01'
        elseif (tsecond < seconds_in_day) then
         write(stdout,*)'Time       :', tsecond/seconds_in_hour
     &,   ' in hours since 0000-01-01'
        elseif (tsecond < 31536000._dbl_kind) then
         write(stdout,*)'Time       :', tday00
     &,   ' in days since 0000-01-01'
        else
          write(stdout,*) 'Time(years)  :', tyear00
     &,   ' in years since 0000-01-01'
        endif
      endif

!-----------------------------------------------------------------------

      end subroutine diag_init_sums

!***********************************************************************

      subroutine diag_global_preupdate

!-----------------------------------------------------------------------
!
!     computes diagnostic sums for those diagnostics computed from
!     current time prognostic variables (before updating to new time)
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  k  ! vertical level index
     &, n  ! tracer index

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  WORK1              ! local work space

!-----------------------------------------------------------------------
!
!     compute these diagnostics only if it is time
!
!-----------------------------------------------------------------------

      if (ldiag_global) then

!-----------------------------------------------------------------------
!
!     finish integration of vertical sums accumulated during timestep
!
!-----------------------------------------------------------------------

      WORK1 = (UVEL(:,:,1,curtime)*SMF(:,:,1)
     &          +VVEL(:,:,1,curtime)*SMF(:,:,2))*UAREA
      diag_ws = global_sum(WORK1,RCALCU)/volume_t

      WORK1 = DIAG_KE_ADV_2D*UAREA
      diag_ke_adv = global_sum(WORK1,RCALCU)/volume_t

      WORK1 = DIAG_KE_HMIX_2D*UAREA
      diag_ke_hmix = global_sum(WORK1,RCALCU)/volume_t

      WORK1 = DIAG_KE_VMIX_2D*UAREA
      diag_ke_vmix = global_sum(WORK1,RCALCU)/volume_t

      WORK1 = DIAG_KE_PRESS_2D*UAREA
      diag_ke_press = global_sum(WORK1,RCALCU)/volume_t

      WORK1 = DIAG_PE_2D*TAREA
      diag_pe = global_sum(WORK1,RCALCT)/volume_t

      do n=1,nt
        WORK1 = DIAG_TRACER_ADV_2D(:,:,n)*TAREA
        diag_tracer_adv(n) = global_sum(WORK1,RCALCT)/volume_t

        WORK1 = DIAG_TRACER_HDIFF_2D(:,:,n)*TAREA
        diag_tracer_hdiff(n) = global_sum(WORK1,RCALCT)/volume_t

        WORK1 = DIAG_TRACER_VDIFF_2D(:,:,n)*TAREA
        diag_tracer_vdiff(n) = global_sum(WORK1,RCALCT)/volume_t

        WORK1 = DIAG_TRACER_SOURCE_2D(:,:,n)*TAREA
        diag_tracer_source(n) = global_sum(WORK1,RCALCT)/volume_t
      end do

!-----------------------------------------------------------------------
!
!     compute global changes in temp and salt for this time step
!
!-----------------------------------------------------------------------

      dtracer_avg = c0
      dtracer_abs = c0

      do n=1,nt
        do k=1,km
          if (sfc_layer_type == sfc_layer_varthick .and. k == 1) then
            WORK1 = merge(TAREA*(
     &        (dz(1)+PSURF(:,:,newtime)/grav)*TRACER(:,:,k,n,newtime)
     &       -(dz(1)+PSURF(:,:,mixtime)/grav)*TRACER(:,:,k,n,oldtime)),
     &                    c0, k <= KMT)
          else
            WORK1 = merge(dz(k)*TAREA*(TRACER(:,:,k,n,newtime) -
     &                                 TRACER(:,:,k,n,oldtime)),
     &                    c0, k <= KMT)
          endif

          dtracer_avg(n) = dtracer_avg(n) + 
     &                     global_sum(WORK1,ONE)/c2dtt(k)
          WORK1 = abs(WORK1)
          dtracer_abs(n) = dtracer_abs(n) + 
     &                     global_sum(WORK1,ONE)/c2dtt(k)
        end do
      end do

      dtracer_avg = dtracer_avg/volume_t
      dtracer_abs = dtracer_abs/volume_t

!-----------------------------------------------------------------------
!
!     compute some additional diagnostics
!
!-----------------------------------------------------------------------

      WORK1 = -(UBTROP(:,:,curtime)*GRADPX(:,:,curtime) +
     &          VBTROP(:,:,curtime)*GRADPY(:,:,curtime))*HU*UAREA
      diag_press_free = global_sum(WORK1,RCALCU)/volume_t

      WORK1 = p5*(UVEL(:,:,1,curtime)**2 +
     &            VVEL(:,:,1,curtime)**2)*DHU*UAREA
      diag_ke_free = global_sum(WORK1,RCALCU)/volume_t

      WORK1 = PSURF(:,:,curtime)*DH*TAREA
      diag_ke_psfc = global_sum(WORK1,RCALCT)/volume_t

!-----------------------------------------------------------------------

      endif ! ldiag_global

!-----------------------------------------------------------------------

      end subroutine diag_global_preupdate

!***********************************************************************

      subroutine diag_global_afterupdate

!-----------------------------------------------------------------------
!
!     compute global diagnotics from new time prognostic variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: k,n   ! vertical level index

      real (kind=dbl_kind)  ::
     &  sum_tmp

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  WORK1               ! local work space

!-----------------------------------------------------------------------
!
!     compute these diagnostics only if it is time
!
!-----------------------------------------------------------------------

      if (ldiag_global) then

!-----------------------------------------------------------------------
!
!     compute global mean sea level
!
!-----------------------------------------------------------------------

      diag_sealevel = global_sum(TAREA*PSURF(:,:,curtime)/grav
     &                           ,RCALCT)/area_t

!-----------------------------------------------------------------------
!
!     compute average temp, salinity and horiz KE at new time
!
!-----------------------------------------------------------------------

      diag_ke  = c0
      do k = 1,km
        WORK1 = merge(p5*(UVEL(:,:,k,curtime)**2 +
     &                    VVEL(:,:,k,curtime)**2)*UAREA*dz(k)
     &               ,c0,KMU >= k)

        diag_ke = diag_ke + global_sum(WORK1,ONE)
      enddo

      avg_tracer = c0
      do n=1,nt
        do k = 1,km
          if (sfc_layer_type == sfc_layer_varthick .and. k == 1) then
            WORK1 = merge(TRACER(:,:,k,n,curtime)*TAREA
     &        *(c1 + PSURF(:,:,curtime)/(dz(1)*grav)),c0,KMT >= k)
          else
            WORK1 = merge(TRACER(:,:,k,n,curtime)*TAREA,c0,KMT >= k)
          endif
          sum_tmp = global_sum(WORK1,ONE)
          avg_tracer(n) = avg_tracer(n) + sum_tmp*dz(k)
          if (diag_all_levels) then
            avg_tracer_k(k,n) = sum_tmp/tarea_k(k)
          endif
        end do
      end do

      diag_ke = diag_ke/volume_u
      avg_tracer = avg_tracer/volume_t
      sfc_tracer_flux = sfc_tracer_flux/area_t

!-----------------------------------------------------------------------
!
!     convert some tracer diagnostics to different units
!
!-----------------------------------------------------------------------

      avg_tracer(2) = avg_tracer(2)*salt_to_ppt ! salinity to ppt
      if (diag_all_levels) then
        do k=1,km
          avg_tracer_k(k,2) = avg_tracer_k(k,2)*salt_to_ppt
        end do
      endif

      sfc_tracer_flux(1) = sfc_tracer_flux(1)/hflux_factor !flux in W/m2
      sfc_tracer_flux(2) = sfc_tracer_flux(2)/salinity_factor ! FW flux

!-----------------------------------------------------------------------

      endif ! ldiag_global

!-----------------------------------------------------------------------

      end subroutine diag_global_afterupdate

!***********************************************************************

      subroutine diag_print

!-----------------------------------------------------------------------
!
!     local variable
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: k,n ! dummy indices
     &,                          ier ! error code

      character (25) :: diag_name  ! name of output diagnostic
      character (char_len_long) :: string 

!-----------------------------------------------------------------------
!
!     prints diagnostic stuff
!
!-----------------------------------------------------------------------

      if (ldiag_global) then
        if (my_task == master_task) then

        open(diag_unit, file=diag_outfile,
     &         status='old', position='append')

!-----------------------------------------------------------------------
!
!         print global diagnostics to stdout
!
!-----------------------------------------------------------------------

          write(stdout,*) ' '
          write(stdout,*) 'K.E. diagnostics:'
          write(stdout,*) ' mean K.E.       :', diag_ke
          write(stdout,*)
     &    ' K.E. change from horiz. visc. :', diag_ke_hmix
          write(stdout,*)
     &    ' K.E. change from vert.  visc. :', diag_ke_vmix
          write(stdout,*)
     &    ' K.E. change from wind stress  :', diag_ws
          write(stdout,*) ' '
          write(stdout,*)
     &    ' advective K.E. balance        :', diag_ke_adv
          write(stdout,*)
     &    '                               :', diag_ke_free
          write(stdout,*)
     &    ' P.E. verses pressure work     :', diag_pe
          write(stdout,*)
     &    '                               :', diag_ke_press
          write(stdout,*)
     &    ' surface-pressure contribution :', diag_ke_psfc
          write(stdout,*)
     &    '                               :', diag_press_free

          write(stdout,*) ' '
          write(stdout,*) 'Tracer diagnostics:'
          do n=1,nt
            write(stdout,*) ' '
            write(stdout,*)
     &      ' mean value  of tracer ',n,'                  :', 
     &                                            avg_tracer(n)
            write(stdout,*)
     &      ' abs  change in tracer ',n,'                  :', 
     &                                            dtracer_abs(n)
            write(stdout,*)
     &      ' mean change in tracer ',n,'                  :', 
     &                                            dtracer_avg(n)
            write(stdout,*)
     &      ' mean change in tracer ',n,' from advection   :', 
     &                                            diag_tracer_adv(n)
            write(stdout,*)
     &      ' mean change in tracer ',n,' from horiz. diff.:', 
     &                                            diag_tracer_hdiff(n)
            write(stdout,*)
     &      ' mean change in tracer ',n,' from vert.  diff.:', 
     &                                            diag_tracer_vdiff(n)
            write(stdout,*)
     &      ' mean change in tracer ',n,' from sources     :', 
     &                                            diag_tracer_source(n)
            write(stdout,*)
     &      ' surface flux of tracer ',n,' :', sfc_tracer_flux(n)

            if (diag_all_levels) then
              do k=1,km
                write(stdout,*)
     &          ' mean value  of tracer ',n,' at level ',k,' :', 
     &                                            avg_tracer_k(k,n)
              end do
            endif
          end do

          write(stdout,*) ' '
          write(stdout,*) 'Other diagnostics:'
          write(stdout,*) ' global mean sea level:', diag_sealevel
          write(stdout,*) ' residual, scans :', rms_residual,
     &                                          solv_sum_iters

!-----------------------------------------------------------------------
!
!         print global diagnostics to output file
!
!-----------------------------------------------------------------------

          diag_name = 'mean KE'
          write(diag_unit,1000) tday00, diag_ke, diag_name
          do n=1,nt
            write(diag_name,'(a12,i2)') 'mean tracer ',n
            write(diag_unit,1000) tday00, avg_tracer(n), diag_name
            if (diag_all_levels) then
              do k=1,km
              write(diag_name,'(a12,i2,a8,i2)') 
     &          'mean tracer ',n,' at lvl ',k
              write(diag_unit,1000) tday00,avg_tracer_k(k,n),diag_name
              enddo
            endif
          end do

          diag_name = 'mean sea level'
          write(diag_unit,1000) tday00, diag_sealevel, diag_name
          diag_name = 'solver residual'
          write(diag_unit,1000) tday00, rms_residual, diag_name
          diag_name = 'solver iterations'
          write(diag_unit,1000) tday00,real(solv_sum_iters),diag_name

          do n=1,nt
            write(diag_name,'(a7,i3,a11)') 'Tracer ',n,' change avg'
            write(diag_unit,1000) tday00, dtracer_avg(n), diag_name
            write(diag_name,'(a7,i3,a11)') 'Tracer ',n,' change abs'
            write(diag_unit,1000) tday00, dtracer_abs(n), diag_name

            write(diag_name,'(a7,i3,a14)') 'Tracer ',n,' change advect'
            write(diag_unit,1000) tday00,diag_tracer_adv(n),diag_name
            write(diag_name,'(a7,i3,a13)') 'Tracer ',n,' change hdiff'
            write(diag_unit,1000) tday00,diag_tracer_hdiff(n),diag_name
            write(diag_name,'(a7,i3,a13)') 'Tracer ',n,' change vdiff'
            write(diag_unit,1000) tday00,diag_tracer_vdiff(n),diag_name
            write(diag_name,'(a7,i3,a14)') 'Tracer ',n,' change source'
            write(diag_unit,1000) tday00,diag_tracer_source(n),diag_name

            write(diag_name,'(a7,i3,a14)') 'Tracer ',n,' surface flux'
            write(diag_unit,1000) tday00,sfc_tracer_flux(n),diag_name
          end do

          diag_name = 'KE change horiz visc'
          write(diag_unit,1000) tday00, diag_ke_hmix, diag_name
          diag_name = 'KE change vert  visc'
          write(diag_unit,1000) tday00, diag_ke_vmix, diag_name
          diag_name = 'KE change wind stress'
          write(diag_unit,1000) tday00, diag_ws, diag_name

          diag_name = 'advect KE balance'
          write(diag_unit,1000) tday00, diag_ke_adv, diag_name
          diag_name = 'advect KE bal free sfc'
          write(diag_unit,1000) tday00, diag_ke_free, diag_name
          diag_name = 'PE change'
          write(diag_unit,1000) tday00, diag_pe, diag_name
          diag_name = 'pressure work'
          write(diag_unit,1000) tday00, diag_ke_press, diag_name
          diag_name = 'KE change sfc-pressure'
          write(diag_unit,1000) tday00, diag_ke_psfc, diag_name
          diag_name = 'press work free sfc'
          write(diag_unit,1000) tday00, diag_press_free, diag_name

!-----------------------------------------------------------------------

        close(diag_unit)
 
        call ccsm_date_stamp (ccsm_diag_date, 'ymds')
 
        diag_outfile_old = trim(diag_outfile)
        diag_outfile = trim(diag_outfile_root)//'.'//ccsm_diag_date
 
        string = 'mv '//trim(diag_outfile_old)//' '
     &                //trim(diag_outfile)
        call shr_sys_system (trim(string), ier)

        endif  ! master_task

        call timer_print_all
 
        call document ('diag_print', 'file written: '
     &                   //trim(diag_outfile))

      endif ! ldiag_global


!-----------------------------------------------------------------------

 1000 format(1pe22.15,1x,1pe22.15,2x,a25)

!-----------------------------------------------------------------------

      end subroutine diag_print

!***********************************************************************

      subroutine diag_transport

!-----------------------------------------------------------------------
!
!     calculates and prints various user-defined transports
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  i,j,k,n               ! dummy loop indices
     &, ier

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  MASS_Z,               ! zonal transport of mass
     &  HEAT_Z,               ! zonal transport of heat
     &  SALT_Z,               ! zonal transport of salt
     &  MASS_M,               ! merid transport of mass
     &  HEAT_M,               ! merid transport of heat
     &  SALT_M,               ! merid transport of salt
     &  WORK1, WORK2          ! local temp space

      real (kind=dbl_kind) ::
     &  mass_tran,            ! local sum of mass transport
     &  heat_tran,            ! local sum of heat transport
     &  salt_tran             ! local sum of salt transport

      character (char_len_long) :: string 
 
!-----------------------------------------------------------------------
!
!     only compute if it is time
!     initialize transport sums
!
!-----------------------------------------------------------------------

      if (ldiag_transport) then

      if (my_task == master_task) then
        open(trans_unit, file=diag_transport_outfile,
     &         status='old', position='append')
      endif

      transports = c0

!-----------------------------------------------------------------------
!
!     calculate transports at each vertical level
!
!-----------------------------------------------------------------------

      do k = 1,km
           
        WORK1 = UVEL(:,:,k,curtime)*DYU
        call s_eoshift(WORK2,WORK1)
        MASS_M = p5*dz(k)*(WORK2+WORK1)

        call e_eoshift(WORK1,TRACER(:,:,k,1,curtime))
        HEAT_M = p5*(WORK1+TRACER(:,:,k,1,curtime))*MASS_M

        call e_eoshift(WORK1,TRACER(:,:,k,2,curtime))
        SALT_M = p5*(WORK1+TRACER(:,:,k,2,curtime))*MASS_M
           
        WORK1 = VVEL(:,:,k,curtime)*DXU
        call w_eoshift(WORK2,WORK1)
        MASS_Z = p5*dz(k)*(WORK2+WORK1)

        call n_eoshift(WORK1,TRACER(:,:,k,1,curtime))
        HEAT_Z = p5*(WORK1+TRACER(:,:,k,1,curtime))*MASS_Z

        call n_eoshift(WORK1,TRACER(:,:,k,2,curtime))
        SALT_Z = p5*(WORK1+TRACER(:,:,k,2,curtime))*MASS_Z
           
!-----------------------------------------------------------------------
!
!       compute transport sums for each defined transport
!
!-----------------------------------------------------------------------

        do n=1,num_transports
          if (k >= transport_add(3,1,n) .and.
     &        k <= transport_add(3,2,n) ) then

            mass_tran = c0
            heat_tran = c0
            salt_tran = c0

            !*** local sum

            select case (transport_type(n))

            case (transport_type_zonal)

              do j=transport_add(2,1,n),transport_add(2,2,n)
                do i=transport_add(1,1,n),transport_add(1,2,n)
                  mass_tran = mass_tran + MASS_Z(i,j)
                  heat_tran = heat_tran + HEAT_Z(i,j)
                  salt_tran = salt_tran + SALT_Z(i,j)
                end do
              end do

            case (transport_type_merid)

              do j=transport_add(2,1,n),transport_add(2,2,n)
                do i=transport_add(1,1,n),transport_add(1,2,n)
                  mass_tran = mass_tran + MASS_M(i,j)
                  heat_tran = heat_tran + HEAT_M(i,j)
                  salt_tran = salt_tran + SALT_M(i,j)
                end do
              end do

            end select

            !*** global sum

            transports(1,n) = transports(1,n) +
     &                        global_real_scalar_sum (mass_tran)
            transports(2,n) = transports(2,n) +
     &                        global_real_scalar_sum (heat_tran)
            transports(3,n) = transports(3,n) +
     &                        global_real_scalar_sum (salt_tran)
          endif
        end do
      enddo

!-----------------------------------------------------------------------
!
!     write transports to transport output file
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        do n=1,num_transports

          transports(1,n) = transports(1,n)*mass_to_Sv
          transports(2,n) = transports(2,n)*heat_to_PW
          transports(3,n) = transports(3,n)*salt_to_Svppt

          write(trans_unit,1000)tday00,transports(:,n),transport_name(n)
        end do
      endif

 1000 format(4(1pe13.6,2x), a80)

      if (my_task == master_task) then
        close(trans_unit)
        call ccsm_date_stamp (ccsm_diag_date, 'ymds')
        diag_transport_outfile_old = trim(diag_transport_outfile)
        diag_transport_outfile = trim(diag_transport_outfile_root)
     &              //'.'//ccsm_diag_date
 
        string = 'mv '//trim(diag_transport_outfile_old)//' '
     &                //trim(diag_transport_outfile)
        call shr_sys_system (trim(string), ier)
      endif ! master_task

      call document ('diag_transport', 'file written: '
     &               //trim(diag_transport_outfile))

      endif ! ldiag_transport
!-----------------------------------------------------------------------

      end subroutine diag_transport

!***********************************************************************

      subroutine cfl_advect(k,UTE,UTW,VTN,VTS,WTK,WTKB)

!-----------------------------------------------------------------------
!
!     calculate maximum advective cfl numbers and their locations
!     in i,j,k directions
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! vertical level index

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) ::
     &  UTE, UTW,           ! advective fluxes through east,west faces
     &  VTN, VTS,           ! advective fluxes through nrth,south faces
     &  WTK, WTKB           ! advective fluxes through top,bottom faces

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  cfladd1, cfladd2      ! temporary addresses for max cfl number

      real (kind=dbl_kind) ::
     &  cfltemp               ! temporary cfl number

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  WORK0, WORK1, WORK2, WORK4

!-----------------------------------------------------------------------
!
!     only compute cfl numbers if it is time
!
!-----------------------------------------------------------------------

      if (ldiag_cfl) then

!-----------------------------------------------------------------------
!
!       zonal advective CFL number
!
!-----------------------------------------------------------------------

        WORK1 =  UTE*TAREA_R  ! east flux
        WORK2 =  UTW*TAREA_R  ! west flux, note sign change

        WORK4 = abs(WORK1) + abs(WORK2)

        WORK0 = abs(WORK1)
        call global_real_maxloc(WORK0,cfladd_advuk(1,k),
     &                                cfladd_advuk(2,k),cfl_advuk(k))
        cfl_advuk(k) = cfl_advuk(k)*dt(k)
 
        WORK0 = abs(WORK2)
        call global_real_maxloc(WORK0,cfladd1,cfladd2,cfltemp)
        cfltemp = cfltemp*dt(k)

        if (cfltemp > cfl_advuk(k)) then
          cfl_advuk(k) = cfltemp
          cfladd_advuk(1,k) = cfladd1
          cfladd_advuk(2,k) = cfladd2
        endif

!-----------------------------------------------------------------------
!
!       meridional advective CFL number
!
!-----------------------------------------------------------------------

        WORK1 =  VTN*TAREA_R  ! north flux
        WORK2 =  VTS*TAREA_R  ! south flux, note sign change

        WORK4 = WORK4 + abs(WORK1) + abs(WORK2)

        WORK0 = abs(WORK1)
        call global_real_maxloc(WORK0,cfladd_advvk(1,k),
     &                                cfladd_advvk(2,k),cfl_advvk(k))
        cfl_advvk(k) = cfl_advvk(k)*dt(k)
 
        WORK0 = abs(WORK2)
        call global_real_maxloc(WORK0,cfladd1,cfladd2,cfltemp)
        cfltemp = cfltemp*dt(k)

        if (cfltemp > cfl_advvk(k)) then
          cfl_advvk(k) = cfltemp
          cfladd_advvk(1,k) = cfladd1
          cfladd_advvk(2,k) = cfladd2
        endif

!-----------------------------------------------------------------------
!
!       vertical advective CFL number
!
!-----------------------------------------------------------------------

        WORK1 = WTK /dz(k)  ! top flux
        WORK2 = WTKB/dz(k)  ! bottom flux

        WORK4 = WORK4 + abs(WORK1) + abs(WORK2)

        WORK0 = abs(WORK1)
        call global_real_maxloc(WORK0,cfladd_advwk(1,k),
     &                                cfladd_advwk(2,k),cfl_advwk(k))
        cfl_advwk(k) = cfl_advwk(k)*dt(k)
 
        WORK0 = abs(WORK2)
        call global_real_maxloc(WORK0,cfladd1,cfladd2,cfltemp)
        cfltemp = cfltemp*dt(k)

        if (cfltemp > cfl_advwk(k)) then
          cfl_advwk(k) = cfltemp
          cfladd_advwk(1,k) = cfladd1
          cfladd_advwk(2,k) = cfladd2
        endif

!-----------------------------------------------------------------------
!
!       total advective CFL number.
!       note that it mimics individual component computations using
!       absolute values.
!
!-----------------------------------------------------------------------

        WORK0 = p5*WORK4
        call global_real_maxloc(WORK0,cfladd_advtk(1,k),
     &                                cfladd_advtk(2,k),cfl_advtk(k))
        cfl_advtk(k) = cfl_advtk(k)*dt(k)
 
!-----------------------------------------------------------------------
!
!     finished computing advective cfl numbers
!
!-----------------------------------------------------------------------

      endif ! ldiag_cfl

!-----------------------------------------------------------------------

      end subroutine cfl_advect

!***********************************************************************

      subroutine cfl_vdiff(k,VDC,VVC)

!-----------------------------------------------------------------------
!
!     calculate maximum vertical diffusion cfl numbers and their 
!     locations in i,j,k directions
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! vertical level index

      real (kind=dbl_kind), dimension(:,:,:,:), intent(in) ::
     &  VDC                 ! tracer diffusivity

      real (kind=dbl_kind), dimension(:,:,:), intent(in) ::
     &  VVC                 ! viscosity

#ifndef impvmix
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  kk, kmax,             ! vertical level indices
     &  n                     ! dummy tracer index

      real (kind=dbl_kind), dimension(:,:), allocatable, save ::
     &  VDCM1, VVCM1        ! save k-1 values for next k level

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  WORK1, WORK2        ! local work space

!-----------------------------------------------------------------------
!
!     only compute cfl numbers if it is time
!
!-----------------------------------------------------------------------

      if (ldiag_cfl) then

!-----------------------------------------------------------------------
!
!       determine size of VDC, VVC arrays 
!
!-----------------------------------------------------------------------

        kmax = size(VVC, DIM=3)

!-----------------------------------------------------------------------
!
!       if VDC, VVC 2-d arrays, compute cfl number for this level only
!
!-----------------------------------------------------------------------

        if (kmax == 1) then

          if (k == 1) then
            if ( .not. allocated(VVCM1) ) then
              allocate (VVCM1(imt,jmt), VDCM1(imt,jmt))
            endif
            VVCM1 = c0
            VDCM1 = c0
          endif

          WORK1 = abs(c2*(VDC(:,:,1,1)*dzwr(k  ) + 
     &                    VDCM1       *dzwr(k-1))*dzr(k))
          WORK2 = abs(c2*(VVC(:,:,1)  *dzwr(k  ) + 
     &                    VVCM1       *dzwr(k-1))*dzr(k))

          VVCM1 = VVC(:,:,1)    ! swap to prepare for next k level
          VDCM1 = VDC(:,:,1,1)

          call global_real_maxloc(WORK1, cfladd_vdifftk(1,k),
     &                                   cfladd_vdifftk(2,k),
     &                                   cfl_vdifftk(k))
          cfl_vdifftk(k) = cfl_vdifftk(k)*dt(k)
 
          call global_real_maxloc(WORK2, cfladd_vdiffuk(1,k),
     &                                   cfladd_vdiffuk(2,k),
     &                                   cfl_vdiffuk(k))
          cfl_vdiffuk(k) = cfl_vdiffuk(k)*dt(k)
 
!-----------------------------------------------------------------------
!
!       if VDC, VVC 3-d arrays, compute cfl number for all levels
!
!-----------------------------------------------------------------------

        else

          !***
          !*** k=1 case (k-1 values = 0)
          !***

          WORK1 = c0
          do n=1,size(VDC,DIM=4)
            WORK1 = max(WORK1,
     &                  abs(c2*VDC(:,:,1,n)*dzwr(1)*dzr(1)))
          end do

          WORK2 = abs(c2*VVC(:,:,1)*dzwr(1)*dzr(1))

          call global_real_maxloc(WORK1, cfladd_vdifftk(1,1),
     &                                   cfladd_vdifftk(2,1),
     &                                   cfl_vdifftk(1))
          cfl_vdifftk(1) = cfl_vdifftk(1)*dt(1)
 
          call global_real_maxloc(WORK2, cfladd_vdiffuk(1,1),
     &                                   cfladd_vdiffuk(2,1),
     &                                   cfl_vdiffuk(1))
          cfl_vdiffuk(1) = cfl_vdiffuk(1)*dt(1)
 
          !***
          !*** all the rest
          !***

          do kk=2,kmax

            WORK1 = c0
            do n=1,size(VDC,DIM=4)
              WORK1 = max(WORK1,
     &                    abs(c2*(VDC(:,:,kk  ,n)*dzwr(kk  ) + 
     &                             VDC(:,:,kk-1,n)*dzwr(kk-1))*dzr(kk)))
            end do

            WORK2 = abs(c2*(VVC(:,:,kk  )*dzwr(kk  ) + 
     &                       VVC(:,:,kk-1)*dzwr(kk-1))*dzr(kk))

            call global_real_maxloc(WORK1,
     &                              cfladd_vdifftk(1,kk),
     &                              cfladd_vdifftk(2,kk),
     &                              cfl_vdifftk(kk))
            cfl_vdifftk(kk) = cfl_vdifftk(kk)*dt(kk)
 
            call global_real_maxloc(WORK2,
     &                              cfladd_vdiffuk(1,kk),
     &                              cfladd_vdiffuk(2,kk),
     &                              cfl_vdiffuk(kk))
            cfl_vdiffuk(kk) = cfl_vdiffuk(kk)*dt(kk)
 
          end do

        endif

!-----------------------------------------------------------------------
!
!     finished computing vertical diffusion cfl numbers
!
!-----------------------------------------------------------------------

      endif ! ldiag_cfl

!-----------------------------------------------------------------------
#endif

      end subroutine cfl_vdiff

!***********************************************************************

      subroutine cfl_hdiff(k,HDIFFCFL,t_or_u)

!-----------------------------------------------------------------------
!
!     calculate maximum horizontal diffusion cfl numbers and their 
!     locations in i,j,k directions
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &  k,                     ! vertical level index
     &  t_or_u                 ! called from hdifft(1) or hdiffu(2)

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) ::
     &  HDIFFCFL               ! hdiff cfl number at all grid points

!-----------------------------------------------------------------------
!
!     compute global max cfl number for this level
!
!-----------------------------------------------------------------------

      select case (t_or_u)

      case(1)
        call global_real_maxloc(HDIFFCFL, cfladd_hdifftk(1,k),
     &                                    cfladd_hdifftk(2,k),
     &                                    cfl_hdifftk(k))
        cfl_hdifftk(k) = cfl_hdifftk(k)*dt(k)
 
      case(2)
        call global_real_maxloc(HDIFFCFL, cfladd_hdiffuk(1,k),
     &                                    cfladd_hdiffuk(2,k),
     &                                    cfl_hdiffuk(k))
        cfl_hdiffuk(k) = cfl_hdiffuk(k)*dtu
 
      end select

!-----------------------------------------------------------------------

      end subroutine cfl_hdiff

!***********************************************************************

      subroutine cfl_check

!-----------------------------------------------------------------------
!
!     finish computing cfl limits for each term
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: k  ! vertical level index

      real (kind=dbl_kind) ::
     &  cfl_advu,          ! max total cfl number for zonal advection
     &  cfl_advv,          ! max total cfl number for merid advection
     &  cfl_advw,          ! max total cfl number for vert  advection
     &  cfl_advt,          ! max total cfl number for total advection
     &  cfl_vdifft,        ! max cfl num for vert tracer diffusion
     &  cfl_vdiffu,        ! max cfl num for vert momentum diffusion
     &  cfl_hdifft,        ! max cfl num for horiz tracer diffusion
     &  cfl_hdiffu         ! max cfl num for horiz momentum diffusion

      integer (kind=int_kind), dimension(3) ::
     &  cfladd_advu,       ! address of max zonal advection cfl number
     &  cfladd_advv,       ! address of max merid advection cfl number
     &  cfladd_advw,       ! address of max vert  advection cfl number
     &  cfladd_advt,       ! address of max total advection cfl number
     &  cfladd_vdifft,     ! address of max vertical tracer diff cfl num
     &  cfladd_vdiffu,     ! address of max vertical moment diff cfl num
     &  cfladd_hdifft,     ! address of max horiz tracer diff cfl num
     &  cfladd_hdiffu      ! address of max horiz momentum diff cfl num

      character (25) :: name_temp1, name_temp2

!-----------------------------------------------------------------------
!
!     compute cfl diagnostics only if it is time
!
!-----------------------------------------------------------------------

      if (ldiag_cfl) then

!-----------------------------------------------------------------------
!
!     initialize max cfl numbers
!
!-----------------------------------------------------------------------

      cfl_advu   = -c1
      cfl_advv   = -c1
      cfl_advw   = -c1
      cfl_advt   = -c1
      cfl_vdifft = -c1
      cfl_vdiffu = -c1
      cfl_hdifft = -c1
      cfl_hdiffu = -c1

!-----------------------------------------------------------------------
!
!     compute global maximum cfl numbers from level maxima
!
!-----------------------------------------------------------------------

      do k = 1,km

!-----------------------------------------------------------------------
!
!       advective cfl numbers
!
!-----------------------------------------------------------------------

        if (cfl_advuk(k) > cfl_advu) then
          cfl_advu = cfl_advuk(k)
          cfladd_advu(1) = cfladd_advuk(1,k)
          cfladd_advu(2) = cfladd_advuk(2,k)
          cfladd_advu(3) = k
        endif

        if (cfl_advvk(k) > cfl_advv) then
          cfl_advv = cfl_advvk(k)
          cfladd_advv(1) = cfladd_advvk(1,k)
          cfladd_advv(2) = cfladd_advvk(2,k)
          cfladd_advv(3) = k
        endif

        if (cfl_advwk(k) > cfl_advw) then
          cfl_advw = cfl_advwk(k)
          cfladd_advw(1) = cfladd_advwk(1,k)
          cfladd_advw(2) = cfladd_advwk(2,k)
          cfladd_advw(3) = k
        endif

        if (cfl_advtk(k) > cfl_advt) then
          cfl_advt = cfl_advtk(k)
          cfladd_advt(1) = cfladd_advtk(1,k)
          cfladd_advt(2) = cfladd_advtk(2,k)
          cfladd_advt(3) = k
        endif

#ifndef impvmix
!-----------------------------------------------------------------------
!
!       vertical diffusion cfl numbers
!
!-----------------------------------------------------------------------

        if (cfl_vdifftk(k) > cfl_vdifft) then
          cfl_vdifft = cfl_vdifftk(k)
          cfladd_vdifft(1) = cfladd_vdifftk(1,k)
          cfladd_vdifft(2) = cfladd_vdifftk(2,k)
          cfladd_vdifft(3) = k
        endif

        if (cfl_vdiffuk(k) > cfl_vdiffu) then
          cfl_vdiffu = cfl_vdiffuk(k)
          cfladd_vdiffu(1) = cfladd_vdiffuk(1,k)
          cfladd_vdiffu(2) = cfladd_vdiffuk(2,k)
          cfladd_vdiffu(3) = k
        endif
#endif

!-----------------------------------------------------------------------
!
!       horizontal diffusion cfl numbers
!
!-----------------------------------------------------------------------

        if (cfl_hdifftk(k) > cfl_hdifft) then
          cfl_hdifft = cfl_hdifftk(k)
          cfladd_hdifft(1) = cfladd_hdifftk(1,k)
          cfladd_hdifft(2) = cfladd_hdifftk(2,k)
          cfladd_hdifft(3) = k
        endif

        if (cfl_hdiffuk(k) > cfl_hdiffu) then
          cfl_hdiffu = cfl_hdiffuk(k)
          cfladd_hdiffu(1) = cfladd_hdiffuk(1,k)
          cfladd_hdiffu(2) = cfladd_hdiffuk(2,k)
          cfladd_hdiffu(3) = k
        endif

      end do

!-----------------------------------------------------------------------
!
!     write results to stdout
!
!-----------------------------------------------------------------------
      
      if (my_task == master_task) then
        write(stdout,*) ' '
        write(stdout,1000) cfl_advu, cfladd_advu(1), cfladd_advu(2),
     &                               cfladd_advu(3)
        write(stdout,1010) cfl_advv, cfladd_advv(1), cfladd_advv(2),
     &                               cfladd_advv(3)
        write(stdout,1020) cfl_advw, cfladd_advw(1), cfladd_advw(2),
     &                               cfladd_advw(3)
        write(stdout,1030) cfl_advt, cfladd_advt(1), cfladd_advt(2),
     &                               cfladd_advt(3)

#ifndef impvmix
        write(stdout,1050) cfl_vdifft, cfladd_vdifft(1), 
     &                                 cfladd_vdifft(2),
     &                                 cfladd_vdifft(3)
        write(stdout,1070) cfl_vdiffu, cfladd_vdiffu(1), 
     &                                 cfladd_vdiffu(2),
     &                                 cfladd_vdiffu(3)
#endif
        write(stdout,1040) cfl_hdifft, cfladd_hdifft(1), 
     &                                 cfladd_hdifft(2),
     &                                 cfladd_hdifft(3)
        write(stdout,1060) cfl_hdiffu, cfladd_hdiffu(1), 
     &                                 cfladd_hdiffu(2),
     &                                 cfladd_hdiffu(3)

        if (cfl_all_levels) then
          write(stdout,*) 'CFL numbers at all levels: '
          do k=1,km
            write(stdout,1030) cfl_advtk(k),   cfladd_advtk  (1,k),
     &                                         cfladd_advtk  (2,k), k
            write(stdout,1040) cfl_hdifftk(k), cfladd_hdifftk(1,k), 
     &                                         cfladd_hdifftk(2,k), k
          end do
        endif

!-----------------------------------------------------------------------
!
!       write results to output file
!
!-----------------------------------------------------------------------
      
        open(diag_unit, file=diag_outfile,
     &       status='old', position='append')

        name_temp1 = 'cfl_advu'
        name_temp2 = 'cfl_advv'
        write(diag_unit,1080) tday00, cfl_advu, name_temp1
        write(diag_unit,1080) tday00, cfl_advv, name_temp2
        name_temp1 = 'cfl_advw'
        name_temp2 = 'cfl_advt'
        write(diag_unit,1080) tday00, cfl_advw, name_temp1
        write(diag_unit,1080) tday00, cfl_advt, name_temp2

#ifndef impvmix
        name_temp1 = 'cfl_vdifft'
        name_temp2 = 'cfl_vdiffu'
        write(diag_unit,1080) tday00, cfl_vdifft, name_temp1
        write(diag_unit,1080) tday00, cfl_vdiffu, name_temp2
#endif
        name_temp1 = 'cfl_hdifft'
        name_temp2 = 'cfl_hdiffu'
        write(diag_unit,1080) tday00, cfl_hdifft, name_temp1
        write(diag_unit,1080) tday00, cfl_hdiffu, name_temp2

        if (cfl_all_levels) then
          do k=1,km
            write(name_temp1,"('cfl_advtk ',a3)") k
            write(name_temp2,"('cfl_advuk ',a3)") k
            write(diag_unit,1080) tday00, cfl_advtk(k),   name_temp1
            write(diag_unit,1080) tday00, cfl_hdifftk(k), name_temp2
          end do
        endif

!-----------------------------------------------------------------------
!
!       finished writing max cfl numbers
!
!-----------------------------------------------------------------------

        close(diag_unit)
      endif ! master task

      endif ! ldiag_cfl

!-----------------------------------------------------------------------
      
 1000 format(' max. zonal advection cfl number:       ',1pe12.4,4x,3i5)
 1010 format(' max. meridional advection cfl number:  ',1pe12.4,4x,3i5)
 1020 format(' max. vertical advection cfl number:    ',1pe12.4,4x,3i5)
 1030 format(' max. total advection cfl number:       ',1pe12.4,4x,3i5)
 1040 format(' max. horiz. tracer mixing cfl number:  ',1pe12.4,4x,3i5)
 1050 format(' max. vert. tracer mixing cfl number:   ',1pe12.4,4x,3i5)
 1060 format(' max. horiz. moment. mixing cfl number: ',1pe12.4,4x,3i5)
 1070 format(' max. vert. moment. mixing cfl number:  ',1pe12.4,4x,3i5)
 1080 format(1pe22.15,1x,1pe22.15,2x,a25)

!-----------------------------------------------------------------------

      end subroutine cfl_check

!***********************************************************************

      end module diagnostics

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
