!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module exit_mod

!***********************************************************************
!
!     this module provides a means for a graceful exit from POP when
!     encountering an error.  it contains only the routine exit_POP
!
!     CVS:$Id: exit_mod.F,v 1.2.2.6 2002/04/25 22:45:01 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------


      use kinds_mod
      use communicate
      use domain
      use shr_sys_mod

      implicit none

!***********************************************************************

      contains

!***********************************************************************

      subroutine exit_POP(exit_message, normal_exit)

!-----------------------------------------------------------------------
!
!     this routine prints a message, exits any message environment
!     and cleans up before stopping
!
!     if this is NOT a normal exit (eg, anything other than successful 
!      completion at the end of program pop), then call mpi_abort
!      if running in a coupled environment.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      character (*) :: exit_message ! message to print before stopping
      logical (kind=log_kind), optional, intent(in) :: normal_exit

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: ierr  ! error flag
      logical (kind=log_kind) :: call_mpi_abort

!-----------------------------------------------------------------------
!
!     print message
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        write (6,1000)
        write (6,*)
        write (6,*) 'POP exiting...'
        if (present (normal_exit) .and. .not. normal_exit) then
           write (6,*) 'FATAL ERROR: ', exit_message
        else
           write (6,*) exit_message
        endif
        write (6,*)
        write (6,1000)
        call shr_sys_flush(6)
 1000   format(72('-'))
      endif

!-----------------------------------------------------------------------
!
!     exit the message-passing environment if required
!     
!-----------------------------------------------------------------------


#ifdef coupled
      if (present (normal_exit) ) then
         call_mpi_abort = .not. normal_exit
      else
         call_mpi_abort = .true.
      endif
 
      if (call_mpi_abort) then
         call mpi_barrier(MPI_COMM_OCN)
         ierr = 13
         call mpi_abort(0,ierr)
      endif
#endif

      call exit_message_environment(ierr)
 

!-----------------------------------------------------------------------
!
!     now we can stop
!
!-----------------------------------------------------------------------

      stop

!-----------------------------------------------------------------------

      end subroutine exit_POP


      subroutine check_allocate(status, exit_message)

!-----------------------------------------------------------------------
!
!     this routine tests the value of allocate_error_status from
!     the "allocate" command, and exits pop if the allocation 
!     failed.
!
!-----------------------------------------------------------------------
 
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: status
      character (*), optional :: exit_message ! message to print before stopping


      if (status /= 0) then
 
        if (my_task == master_task) then
           write (6,1000)exit_message
           call shr_sys_flush(6)
        endif
 
        call exit_POP ('FATAL ERROR -- allocating an array'
     &,                 normal_exit = .false.)
 
      endif

 


 1000  format(/,5x, 'Allocation failed: ', a)
!-----------------------------------------------------------------------

      end subroutine check_allocate


!***********************************************************************

      end module exit_mod

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
