!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module forcing_coupled

!***********************************************************************
!
!  CVS:$Id: forcing_coupled.F,v 1.2.2.20 2002/03/04 23:33:35 njn01 Exp $
!  CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------


      use kinds_mod
      use model_size
      use domain
      use communicate
      use constants
      use io
      use stencils
      use time_management
      use grid
      use prognostic
      use exit_mod
      use ice
      use forcing_shf
      use forcing_sfwf
      use qflux_mod
      use ms_balance
      use timers
      use shr_sys_mod
      
      implicit none
      save
      
!-----------------------------------------------------------------------
!
!     module variables
!
!-----------------------------------------------------------------------

      logical (kind=log_kind) ::
     &  lcoupled              ! flag for coupled forcing
     &, ldiag_cpl = .false.


      integer (kind=int_kind) ::
     &  coupled_freq_iopt,    ! coupler frequency option
     &  coupled_freq          ! frequency of coupling

#if coupled
      integer (kind=int_kind) :: 
     &  cpl_task              ! task id for coupler

      integer (kind=int_kind) ::
     &  timer_send_to_cpl  
     &, timer_recv_from_cpl
     &, timer_recv_to_send 
     &, timer_send_to_recv 
 
      integer (kind=int_kind), private :: 
     &  cpl_stop_now          ! flag id for stop_now flag
     &, cpl_ts                ! flag id for coupled_ts flag
     &, cpl_write_restart     ! flag id for write restart
     &, cpl_write_history     ! flag id for write history
     &, cpl_write_tavg        ! flag id for write tavg      
     &, cpl_diag_global       ! flag id for computing diagnostics
     &, cpl_diag_transp       ! flag id for computing diagnostics

      integer (kind=int_kind), parameter :: 
     &  nibuff       = 100,   ! size of integer control buffer
     &  nsfields     = 7,     ! number of fields sent  to   coupler
     &  nrfields     = 14,    ! number of fields recvd from coupler
     &  msgtype_d2oi = 51,    ! message type for cpl->ocn init
     &  msgtype_o2di = 60,    ! message type for ocn->cpl init
     &  msgtype_d2o  = 50,    ! message type for cpl->ocn
     &  msgtype_o2d  = 61     ! message type for ocn->cpl

      integer (kind=int_kind), parameter :: 
     &  krbuff_ifrac =  1,    ! loc in recv buff of ice fraction
     &  krbuff_mslp  =  2,    ! loc in recv buff of mean sea-level pressure
     &  krbuff_taux  =  3,    ! loc in recv buff of taux field 
     &  krbuff_tauy  =  4,    ! loc in recv buff of tauy field
     &  krbuff_netsw =  5,    ! loc in recv buff of shortwave heat flux
     &  krbuff_senhf =  6,    ! loc in recv buff of sensible  heat flux
     &  krbuff_lwup  =  7,    ! loc in recv buff of longwave  heat flux (up)
     &  krbuff_lwdwn =  8,    ! loc in recv buff of longwave  heat flux (dwn)
     &  krbuff_melthf=  9,    ! loc in recv buff of heat flux from snow&ice melt
     &  krbuff_salt  = 10,    ! loc in recv buff of salt flux (kg of salt)
     &  krbuff_prec  = 11,    ! loc in recv buff of precipitation flux
     &  krbuff_evap  = 12,    ! loc in recv buff of evaporation   flux 
     &  krbuff_melt  = 13,    ! loc in recv buff of snow melt     flux 
     &  krbuff_roff  = 14     ! loc in recv buff of river runoff  flux 


      integer (kind=int_kind), parameter :: 
     &  ksbuff_sst   = 1,     ! loc in send buff of sst
     &  ksbuff_sss   = 2,     ! loc in send buff of sst
     &  ksbuff_uvel  = 3,     ! loc in send buff of u vel
     &  ksbuff_vvel  = 4,     ! loc in send buff of v vel
     &  ksbuff_dhdx  = 5,     ! loc in send buff of e-w surface slope
     &  ksbuff_dhdy  = 6,     ! loc in send buff of n-s surface slope
     &  ksbuff_q     = 7      ! loc in send buff of freeze/melt heat flx


      real (kind=dbl_kind) ::
     &  tlast_coupled

      real (kind=dbl_kind), dimension(imt,jmt,nsfields) ::
     &  SBUFF_SUM           ! accumulated sum of send buffer quantities
                            ! for averaging before being sent

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  EVAP_F = c0         ! evaporation   flux    -- from cpl (kg/m2/s)
     &, PREC_F = c0         ! precipitation flux    -- from cpl (kg/m2/s)
     &, MELT_F = c0         ! melt          flux    -- from cpl (kg/m2/s)
     &, ROFF_F = c0         ! river runoff  flux    -- from cpl (kg/m2/s)
     &, SALT_F = c0         ! salt          flux    -- from cpl (kg(salt)/m2/s)
     &, SENH_F = c0         ! sensible heat flux    -- from cpl (W/m2   )
     &, LWUP_F = c0         ! longwave heat flux up -- from cpl (W/m2   )
     &, LWDN_F = c0         ! longwave heat flux dn -- from cpl (W/m2   )
     &, MELTH_F= c0         ! melt     heat flux    -- from cpl (W/m2   )
 
#endif
!***********************************************************************

      contains

!***********************************************************************

      subroutine init_coupled(SMF, SMFT, STF, SHF_QSW, lsmft_avail)

!-----------------------------------------------------------------------
!
!     this routine sets up everything necessary for coupling with
!     the NCAR flux coupler.
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,2), intent(out) ::
     &   SMF                !  surface momentum fluxes (wind stress)
     &,  SMFT               !  surface momentum fluxes at T points

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(out) ::
     &   STF                !  surface tracer fluxes

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   SHF_QSW            !  penetrative solar heat flux

      logical (kind=log_kind), intent(out) ::
     &   lsmft_avail        ! true if SMFT is an available field

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      character (char_len) ::
     &  coupled_freq_opt

      namelist /coupled_nml/ coupled_freq_opt, coupled_freq 

      integer (kind=int_kind) :: k,
     &  ncouple_per_day,      ! num of coupler comms per day
     &  nml_error             ! namelist i/o error flag

#if coupled

      include 'mpif.h'

      integer (kind=int_kind) ::
     &   ier                           ! error flag for messages

      integer (kind=int_kind), dimension(nibuff) ::
     &   ibuff                         ! integer control buffer

      integer (kind=int_kind), dimension(MPI_STATUS_SIZE) ::
     &   mpistatus

      integer (kind=int_kind), dimension(imt_global,jmt_global) ::
     &   IMASK_G                       ! global ocean mask

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   LONSW,                        ! longitude of SW corner
     &   LONSE,                        ! longitude of SE corner
     &   LONNW,                        ! longitude of NW corner
     &   LONNE                         ! longitude of NE corner

      real (kind=dbl_kind), dimension(imt_global,jmt_global) ::
     &   TLAT_G,                       ! latitude  of cell center
     &   TLON_G                        ! longitude of cell center

      real (kind=dbl_kind), dimension(4,imt_global,jmt_global) ::
     &   TLON_V                        ! coord of cell vertices

#endif
!-----------------------------------------------------------------------
!
!     read coupled_nml namelist to start coupling and determine
!     coupling frequency
!
!-----------------------------------------------------------------------
      
      lcoupled          = .false.
      coupled_freq_opt  = 'never'
      coupled_freq_iopt = freq_opt_never
      coupled_freq      = 100000
      
      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=coupled_nml, err=10, end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR: reading coupled_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, coupled_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        select case (coupled_freq_opt)

        case ('nyear')
          coupled_freq_iopt = -1000

        case ('nmonth')
          coupled_freq_iopt = -1000

        case ('nday')
          if (coupled_freq == 1) then
            lcoupled = .true.
            coupled_freq_iopt = freq_opt_nday
            ncouple_per_day = 1
          else
            coupled_freq_iopt = -1000
          endif

        case ('nhour')
          if (coupled_freq <= 24) then
            lcoupled = .true.
            coupled_freq_iopt = freq_opt_nhour
            ncouple_per_day = 24/coupled_freq
          else
            coupled_freq_iopt = -1000
          endif

        case ('nsecond')
          if (coupled_freq <= seconds_in_day) then
            lcoupled = .true.
            coupled_freq_iopt = freq_opt_nsecond
            ncouple_per_day = seconds_in_day/coupled_freq
          else
            coupled_freq_iopt = -1000
          endif

        case ('nstep')
          if (coupled_freq <= nsteps_per_day) then
            lcoupled = .true.
            coupled_freq_iopt = freq_opt_nstep
            ncouple_per_day = nsteps_per_day/coupled_freq
          else
            coupled_freq_iopt = -1000
          endif

        case ('never')
          lcoupled = .false.

        case default
          coupled_freq_iopt = -2000
        end select

      endif
            
      call broadcast_scalar(lcoupled,          master_task)
      call broadcast_scalar(coupled_freq_iopt, master_task)
      call broadcast_scalar(coupled_freq     , master_task)

      if (coupled_freq_iopt == -1000) then
        call exit_POP(
     &       'ERROR: Coupling frequency must be at least once per day')
      else if (coupled_freq_iopt == -2000) then
        call exit_POP('ERROR: Unknown option for coupling frequency')
      endif

#if coupled
      if (.not. lcoupled) then
        call exit_POP(
     &       'ERROR: Coupled ifdef option enabled but lcoupled=false')
      endif

!-----------------------------------------------------------------------
!
!     Initialize flags and shortwave absorption profile
!     Note that the cpl_write_xxx flags have _no_ default value;
!     therefore, they must be explicitly set .true. and .false.
!     at the appropriate times
!
!-----------------------------------------------------------------------

      cpl_stop_now      = init_time_flag('stop_now',default=.false.)
      cpl_ts            = init_time_flag('coupled_ts',
     &                                   freq_opt = coupled_freq_iopt,
     &                                   freq     = coupled_freq)
      cpl_write_restart = init_time_flag('cpl_write_restart')
      cpl_write_history = init_time_flag('cpl_write_history')
      cpl_write_tavg    = init_time_flag('cpl_write_tavg'   )
      cpl_diag_global   = init_time_flag('cpl_diag_global')
      cpl_diag_transp   = init_time_flag('cpl_diag_transp')

      lsmft_avail = .true.
      tlast_coupled = c0

      sw_absorb(0) = c1
      sw_absorb(km) = c0
      do k = 1, km-1
         call sw_absorb_frac(zw(k),jerlov_water_type,sw_absorb(k))
      enddo

!-----------------------------------------------------------------------
!
!     receive initial buffer from coupler and ignore content
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        call MPI_RECV(ibuff, nibuff, MPI_INTEGER, cpl_task,
     &                msgtype_d2oi, MPI_COMM_WORLD, mpistatus, ier)
      endif

      call broadcast_scalar(ier, master_task)
      if (ier < MPI_SUCCESS ) then
        call exit_POP('ERROR in coupler after initial recv')
      endif

!-----------------------------------------------------------------------
!
!       initialize and send buffer
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        ibuff = 0

        ibuff(4) = iyear*10000 + imonth*100 + iday
        ibuff(5) = ihour*seconds_in_hour + iminute*seconds_in_minute
     &           + isecond
        ibuff(7) = imt_global
        ibuff(8) = jmt_global
        ibuff(9) = ncouple_per_day

        call MPI_SEND(ibuff, nibuff, MPI_INTEGER, cpl_task,
     &                       msgtype_o2di, MPI_COMM_WORLD, ier)

      endif

      call broadcast_scalar(ier, master_task)
      if (ier < MPI_SUCCESS ) then
        call exit_POP('ERROR in coupler after initial ibuff send')
      endif

!-----------------------------------------------------------------------
!
!     initialize and send grid center coordinates in degrees 
!
!-----------------------------------------------------------------------

      !***
      !*** use LONSE, LONSW as temps to hold TLAT, TLON
      !***

      LONSW = radian*TLONG
      LONSE = radian*TLAT

      call gather_global(TLON_G, LONSW, master_task)
      call gather_global(TLAT_G, LONSE, master_task)

      if (my_task == master_task) then
        call MPI_SEND(TLON_G, imt_global*jmt_global, 
     &                        MPI_DOUBLE_PRECISION,
     &                cpl_task, msgtype_o2di, MPI_COMM_WORLD, ier)

        call MPI_SEND(TLAT_G, imt_global*jmt_global,
     &                        MPI_DOUBLE_PRECISION,
     &                cpl_task, msgtype_o2di, MPI_COMM_WORLD, ier)

      endif

      call broadcast_scalar(ier, master_task)
      if (ier < MPI_SUCCESS ) then
        call exit_POP(
     &       'ERROR in coupler after initial send of center coords')
      endif

!-----------------------------------------------------------------------
!
!     initialize and send grid vertex coordinates in degrees 
!
!-----------------------------------------------------------------------

      call s_shift(LONSE, ULONG)
      call w_shift(LONNW, ULONG)
      call s_shift(LONSW, LONNW)

      where (LONSE - ULONG >  c4) LONSE = LONSE - pi2
      where (LONSE - ULONG < -c4) LONSE = LONSE + pi2
      where (LONNW - ULONG >  c4) LONNW = LONNW - pi2
      where (LONNW - ULONG < -c4) LONNW = LONNW + pi2
      where (LONSW - ULONG >  c4) LONSW = LONSW - pi2
      where (LONSW - ULONG < -c4) LONSW = LONSW + pi2

      LONNE = radian*ULONG
      LONNW = radian*LONNW
      LONSE = radian*LONSE
      LONSW = radian*LONSW

      call gather_global(TLON_V(1,:,:),LONSW,master_task)
      call gather_global(TLON_V(2,:,:),LONSE,master_task)
      call gather_global(TLON_V(3,:,:),LONNE,master_task)
      call gather_global(TLON_V(4,:,:),LONNW,master_task)

      if (my_task == master_task) then
        call MPI_SEND(TLON_V, 4*imt_global*jmt_global,
     &                        MPI_DOUBLE_PRECISION,
     &              cpl_task, msgtype_o2di, MPI_COMM_WORLD, ier)
      endif

!-----------------------------------------------------------------------
!
!     do latitudes now using LON arrays as temps
!
!-----------------------------------------------------------------------

      call s_shift(LONSE, ULAT)
      call w_shift(LONNW, ULAT)
      call s_shift(LONSW, LONNW)

      LONNE = radian*ULAT
      LONNW = radian*LONNW
      LONSE = radian*LONSE
      LONSW = radian*LONSW

      call gather_global(TLON_V(1,:,:),LONSW,master_task)
      call gather_global(TLON_V(2,:,:),LONSE,master_task)
      call gather_global(TLON_V(3,:,:),LONNE,master_task)
      call gather_global(TLON_V(4,:,:),LONNW,master_task)

      if (my_task == master_task) then
        call MPI_SEND(TLON_V, 4*imt_global*jmt_global,
     &                        MPI_DOUBLE_PRECISION,
     &              cpl_task, msgtype_o2di, MPI_COMM_WORLD, ier)
      endif

      call broadcast_scalar(ier, master_task)
      if (ier < MPI_SUCCESS ) then
        call exit_POP(
     &       'ERROR in coupler after initial send of vertex coords')
      endif

!-----------------------------------------------------------------------
!
!     initialize and send T-grid areas (TAREA) in square radians
!
!-----------------------------------------------------------------------

      !***
      !*** use LONSW and TLON_G as temporaries
      !***

      LONSW = TAREA / (radius * radius)
 
      call gather_global(TLON_G, LONSW, master_task)
 
      if (my_task == master_task) then
        call MPI_SEND(TLON_G, imt_global*jmt_global,
     &                        MPI_DOUBLE_PRECISION,
     &                cpl_task, msgtype_o2di, MPI_COMM_WORLD, ier)
      endif
 
      call broadcast_scalar(ier, master_task)
      if (ier < MPI_SUCCESS ) then
        call exit_POP(
     &       'ERROR in coupler after initial send of T-grid areas')
      endif

!-----------------------------------------------------------------------
!
!     create and send imask 
!
!-----------------------------------------------------------------------

      call gather_global(IMASK_G,REGION_MASK,master_task)

      if (my_task == master_task) then

        call MPI_SEND(IMASK_G, imt_global*jmt_global, MPI_INTEGER,
     &                cpl_task, msgtype_o2di, MPI_COMM_WORLD, ier)

        if (ier == MPI_SUCCESS ) then
          write(stdout,*) '(ocn) Sent grid info'
        endif

      endif

      call broadcast_scalar(ier, master_task)
      if (ier < MPI_SUCCESS ) then
        call exit_POP(
     &       'ERROR in coupler after initial send of vertex coords')
      endif

!-----------------------------------------------------------------------
!
!     send initial state info to coupler
!
!-----------------------------------------------------------------------

      call sum_buffer

      call send_to_coupler

!-----------------------------------------------------------------------
!
!     initialize timers for coupled model
!
!-----------------------------------------------------------------------
      
      call get_timer (timer_send_to_cpl  , 'SEND'        )
      call get_timer (timer_recv_from_cpl, 'RECV'        )
      call get_timer (timer_recv_to_send , 'RECV to SEND')
      call get_timer (timer_send_to_recv , 'SEND to RECV')
#endif
!-----------------------------------------------------------------------

      end subroutine init_coupled

!***********************************************************************

      subroutine set_coupled_forcing(SMF,SMFT,STF,SHF_QSW)

!-----------------------------------------------------------------------
!
!     this routine call coupler communication routines to set
!     surface forcing data
!
!-----------------------------------------------------------------------
      
      real (kind=dbl_kind), dimension(imt,jmt,2), intent(out) ::
     &   SMF                !  surface momentum fluxes (wind stress)
     &,  SMFT               !  surface momentum fluxes at T points

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(out) ::
     &   STF                !  surface tracer fluxes

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   SHF_QSW            !  penetrative solar heat flux
     
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   IFRAC              !  fractional ice coverage
      
      integer (kind=int_kind) :: n

!-----------------------------------------------------------------------
!
!     exit now if coupling not enabled
!
!-----------------------------------------------------------------------
 
      if (.not. lcoupled) return
           
#if coupled
!-----------------------------------------------------------------------
!
!     if it is time to couple, exchange data with flux coupler
!     be sure to trigger communication on very first time step
!
!-----------------------------------------------------------------------

      if (nsteps_run /= 0) call sum_buffer

      if (check_time_flag(cpl_ts) .or. nsteps_run == 0) then

        !*** send state variables at end of coupling interval 
 

        if (nsteps_run /= 0) then
           call timer_stop  (timer_recv_to_send)
 
           call timer_start (timer_send_to_cpl)
           call send_to_coupler
           call timer_stop  (timer_send_to_cpl)
        endif

        call timer_start (timer_send_to_recv)
        call timer_stop  (timer_send_to_recv) 
 
        !*** recv data to advance next time step

        call timer_start (timer_recv_from_cpl)
        call recv_from_coupler(SMF,SMFT,STF,SHF_QSW,IFRAC)
        call timer_stop  (timer_recv_from_cpl)

        call timer_start (timer_recv_to_send)
 

        if ( shf_formulation == 'partially-coupled' ) then
          SHF_COMP(:,:,shf_comp_cpl) = STF(:,:,1) 
          if ( .not. lms_balance ) then
            SHF_COMP(:,:,shf_comp_cpl) = 
     &              SHF_COMP(:,:,shf_comp_cpl) * MASK_SR
            SHF_QSW = SHF_QSW * MASK_SR
          endif
        endif
 
        if ( sfwf_formulation == 'partially-coupled' ) then

          if (sfc_layer_type == sfc_layer_varthick .and.
     &        .not. lfw_as_salt_flx) then
            SFWF_COMP(:,:,sfwf_comp_cpl) =  FW * MASK_SR
            do n=1,nt
              TFW_COMP(:,:,n,tfw_comp_cpl) = TFW(:,:,n) * MASK_SR
            enddo
          else
            SFWF_COMP(:,:,sfwf_comp_cpl) = STF(:,:,2) * MASK_SR
          endif

        else

          if ( sfc_layer_type == sfc_layer_varthick .and.
     &         .not. lfw_as_salt_flx .and. liceform ) then
            SFWF_COMP(:,:,sfwf_comp_cpl) = FW
            TFW_COMP(:,:,:,tfw_comp_cpl) = TFW
          endif

        endif
 
        if ( luse_cpl_ifrac ) then
          OCN_WGT = (c1-IFRAC) * RCALCT
        endif
 
      endif

#endif
!-----------------------------------------------------------------------

      end subroutine set_coupled_forcing

!***********************************************************************

      subroutine set_combined_forcing (STF)

!-----------------------------------------------------------------------
!
!     this routine combines terms when the "partially-coupled"
!     has been selected
!
!-----------------------------------------------------------------------
 
      real (kind=dbl_kind), dimension(imt,jmt,nt) ::
     &   STF    !  surface tracer fluxes
 
      real (kind=dbl_kind), dimension(imt,jmt) :: WORK1, WORK2
 
      
#if coupled

      if ( shf_formulation == 'partially-coupled' ) then
        STF(:,:,1) =  SHF_COMP(:,:,shf_comp_wrest)
     &              + SHF_COMP(:,:,shf_comp_srest)
     &              + SHF_COMP(:,:,shf_comp_cpl)
      endif

      if ( sfwf_formulation == 'partially-coupled' ) then
        if (sfc_layer_type == sfc_layer_varthick .and.
     &      .not. lfw_as_salt_flx) then
           STF(:,:,2) =  SFWF_COMP(:,:,sfwf_comp_wrest)
     &                 + SFWF_COMP(:,:,sfwf_comp_srest)
           FW         =  SFWF_COMP(:,:,sfwf_comp_cpl)
     &                 + SFWF_COMP(:,:,sfwf_comp_flxio)
           TFW        =  TFW_COMP(:,:,:,tfw_comp_cpl)
     &                 + TFW_COMP(:,:,:,tfw_comp_flxio)
        else
          if ( lms_balance ) then

            WORK1 = SFWF_COMP(:,:,sfwf_comp_flxio) / salinity_factor
            WORK2 = SFWF_COMP(:,:,sfwf_comp_cpl)

            call ms_balancing ( WORK2, EVAP_F, PREC_F, MELT_F,
     &                          ROFF_F, SALT_F, 'salt',
     &                          ICEOCN_F=WORK1 )

            STF(:,:,2) =  SFWF_COMP(:,:,sfwf_comp_wrest)
     &                  + SFWF_COMP(:,:,sfwf_comp_srest)
     &                  + WORK2 
     &                  + SFWF_COMP(:,:,sfwf_comp_flxio) * MASK_SR
          else     

            STF(:,:,2) =  SFWF_COMP(:,:,sfwf_comp_wrest)
     &                  + SFWF_COMP(:,:,sfwf_comp_srest)
     &                  + SFWF_COMP(:,:,sfwf_comp_cpl)
     &                  + SFWF_COMP(:,:,sfwf_comp_flxio) 
 
          endif
        endif
      endif

#endif
!-----------------------------------------------------------------------

      end subroutine set_combined_forcing

#if coupled
!***********************************************************************

      subroutine recv_from_coupler(SMF,SMFT,STF,SHF_QSW,IFRAC)

!-----------------------------------------------------------------------
!
!     this routine receives message from coupler with surface flux
!     data
!
!-----------------------------------------------------------------------

      include 'mpif.h'

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,2), intent(out) ::
     &   SMF                !  surface momentum fluxes (wind stress)
     &,  SMFT               !  surface momentum fluxes at T points

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(out) ::
     &   STF                !  surface tracer fluxes

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &   SHF_QSW            !  penetrative solar heat flux
     &,  IFRAC              !  fractional ice coverage
     
!-----------------------------------------------------------------------
!
!     local variables
!
!     contents of ibuff values received from cpl5:
!
!        ibuff( 1) = ierr         error code, 0 => no error
!        ibuff( 2) = stop_eod     flag: stop at end of day
!        ibuff( 3) = stop_now     flag: stop now
!        ibuff( 4) = cdate        date (encoded: yyyymmdd)
!        ibuff( 5) = sec          seconds
!        ibuff(11) = info_time    flag: mp timing
!        ibuff(12) = info_dbug    dbug diagnostic level
!        ibuff(21) = rest_eod     flag: make restart data at eod
!        ibuff(22) = rest_now     flag: make restart data now
!        ibuff(23) = hist_eod     flag: make history data at eod
!        ibuff(24) = hist_now     flag: make history data now
!        ibuff(25) = hist_tavg    flag: make monthly average data
!        ibuff(26) = diag_eod     flag: make diagnostic at eod
!        ibuff(27) = diag_now     flag: make diagnostic data now
!        ibuff(32) = ashift       albedo time shift
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(nibuff) :: 
     &  ibuff                  ! integer control buffer

      integer (kind=int_kind), parameter ::  ! words in received msg
     &  nrwords = nrfields*imt_global*jmt_global

      integer (kind=int_kind), dimension(MPI_STATUS_SIZE) ::
     &   mpistatus

      integer (kind=int_kind) ::
     &  ier                 ! error flag for message calls
     &, itmp, jtmp
     &, n
 

      real (kind=dbl_kind) :: rval

      real (kind=dbl_kind), dimension(imt_global,jmt_global,nrfields) ::
     &  RBUFF               ! received message buffer

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  WORK1, WORK2        ! local work space
     &, WORK3, WORK4        ! local work space


!-----------------------------------------------------------------------
!
!     receive message from coupler and check for terminate signal
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        call MPI_RECV(ibuff, nibuff, MPI_INTEGER, cpl_task,
     &                msgtype_d2o, MPI_COMM_WORLD, mpistatus, ier)

        call MPI_RECV(RBUFF, nrwords, MPI_DOUBLE_PRECISION, cpl_task,
     &                msgtype_d2o, MPI_COMM_WORLD, mpistatus, ier)

      endif

      call broadcast_scalar(ier, master_task)
      if (ier /= MPI_SUCCESS) then
        call exit_POP('ERROR recieving message from coupler')
      end if

!-----------------------------------------------------------------------
!
!     broadcast all coupler flags and respond appropriately
!
!-----------------------------------------------------------------------

      call broadcast_vector(ibuff, master_task, nibuff)

      !***
      !*** received terminate signal, set stop_now flag and exit
      !***

      if (ibuff(3) == 1) then
        call set_time_flag(cpl_stop_now,.true.)
        if (my_task == master_task) then
          call int_to_char (4,iyear   , cyear )
          call int_to_char (2,imonth  ,cmonth )
          call int_to_char (2,iday    ,cday   )
          call int_to_char (2,ihour   ,chour  )
          call int_to_char (2,iminute ,cminute)
          call int_to_char (2,isecond ,csecond)
          write(stdout,*) ' cpl requests termination now:'             
     &,                     ' ', cyear,'/',cmonth,'/',cday
     &,                     ' ', chour,':',cminute,':',csecond
        endif
       !call exit_POP('Received terminate signal from coupler')
        RETURN
      endif


      if (ibuff(12) >= 2) then
         ldiag_cpl = .true. 
      else
         ldiag_cpl = .false.
      endif

       if (ibuff(21) == 1) then
         call set_time_flag(cpl_write_restart,.true.)
         if (my_task == master_task) then
           write(stdout,*) ' cpl requests restart file at eod '
     &,                     ' ', cyear,'/',cmonth,'/',cday, '  '
         endif
       endif
  
!      if (ibuff(23) == 1) then
!       ignore for now
!        call set_time_flag(cpl_write_history,.true.)
!        call int_to_char (4,iyear   , cyear )
!        call int_to_char (2,imonth  ,cmonth )
!        call int_to_char (2,iday    ,cday   )
!        call int_to_char (2,ihour   ,chour  )
!        call int_to_char (2,iminute ,cminute)
!        call int_to_char (2,isecond ,csecond)
!        if (my_task == master_task) then
!        write(stdout,*) ' cpl requests history file at eod '
!    &,                     ' ', cyear,'/',cmonth,'/',cday, '  '
!        endif
!      endif
  
  
!      if (ibuff(25) == 1) then
!       ignore for now
!        call set_time_flag(cpl_write_tavg, .true.)
!        call int_to_char (4,iyear   , cyear )
!        call int_to_char (2,imonth  ,cmonth )
!        call int_to_char (2,iday    ,cday   )
!        call int_to_char (2,ihour   ,chour  )
!        call int_to_char (2,iminute ,cminute)
!        call int_to_char (2,isecond ,csecond)
!        if (my_task == master_task) then
!        write(stdout,*) ' cpl requests tavg file at eod '
!    &,                     ' ', cyear,'/',cmonth,'/',cday, '  '
!        endif
!      endif
  
       if (ibuff(26) == 1) then
         call set_time_flag(cpl_diag_global,.true.)
         call set_time_flag(cpl_diag_transp,.true.)
 
         call int_to_char (4,iyear   ,cyear  )
         call int_to_char (2,imonth  ,cmonth )
         call int_to_char (2,iday    ,cday   )
         call int_to_char (2,ihour   ,chour  )
         call int_to_char (2,iminute ,cminute)
         call int_to_char (2,isecond ,csecond)
 
         if (my_task == master_task) then
         write(stdout,*) ' cpl requests diagnostics at eod '
     &,                     ' ', cyear,'/',cmonth,'/',cday, '  '
         endif
       endif

!-----------------------------------------------------------------------
!
!     unpack and distribute wind stress, then convert to correct units
!     if necessary, rotate components to logical coordinates
!
!-----------------------------------------------------------------------

      call scatter_global(WORK1, RBUFF(:,:,krbuff_taux), master_task)
      call scatter_global(WORK2, RBUFF(:,:,krbuff_tauy), master_task)

      !***
      !*** Rotate true zonal/meridional wind stress into local
      !*** coordinates and convert to dyne/cm**2
      !***

      SMFT(:,:,1) = (WORK1*cos(ANGLET) + WORK2*sin(ANGLET))*
     &              RCALCT*momentum_factor
      SMFT(:,:,2) = (WORK2*cos(ANGLET) - WORK1*sin(ANGLET))*
     &              RCALCT*momentum_factor
 
      !***
      !*** Shift SMFT to U grid
      !***

      call ne_4pt_nobndy(SMF (:,:,1),AU0,AUN,AUE,AUNE,SMFT(:,:,1))
      call ne_4pt_nobndy(SMF (:,:,2),AU0,AUN,AUE,AUNE,SMFT(:,:,2))


!-----------------------------------------------------------------------
!
!     unpack and distribute fresh water flux and salt flux
!
!-----------------------------------------------------------------------

      call scatter_global(PREC_F, RBUFF(:,:,krbuff_prec), master_task)
      call scatter_global(EVAP_F, RBUFF(:,:,krbuff_evap), master_task)
      call scatter_global(MELT_F, RBUFF(:,:,krbuff_melt), master_task)
      call scatter_global(ROFF_F, RBUFF(:,:,krbuff_roff), master_task)
      call scatter_global(SALT_F, RBUFF(:,:,krbuff_salt), master_task)
 
!-----------------------------------------------------------------------
!
!     for variable thickness surface layer, compute fresh water and
!     salt fluxes
!
!-----------------------------------------------------------------------

      if (sfc_layer_type == sfc_layer_varthick .and. 
     &    .not. lfw_as_salt_flx) then

        !*** compute fresh water flux (cm/s)

        FW = RCALCT*(PREC_F+EVAP_F+ROFF_F)*fwmass_to_fwflux

        WORK1 = RCALCT * MELT_F * fwmass_to_fwflux

        !*** compute tracer concentration in fresh water
        !*** in principle, temperature of each water flux
        !*** could be different. e.g.
        !TFW(:,:,1) = RCALCT*(PREC_F*TEMP_PREC +
        !                     EVAP_F*TEMP_EVAP +
        !                     MELT_F*TEMP_MELT +
        !                     ROFF_F*TEMP_ROFF)*fwmass_to_fwflux
        !*** currently assume water comes in at sea surface temp

        TFW(:,:,1) = FW*TRACER(:,:,1,1,curtime)
     &              + WORK1 * TMLT(TRACER(:,:,1,2,curtime))

        FW = FW + WORK1

        !*** compute salt flux
        !*** again, salinity could be different for each
        !***   component of water flux
        !TFW(:,:,2) = RCALCT*(PREC_F*SALT_PREC +
        !                     EVAP_F*SALT_EVAP +
        !                     MELT_F*SALT_MELT +
        !                     ROFF_F*SALT_ROFF)*fwmass_to_fwflux
        !*** currently assume prec, evap and roff are fresh
        !*** and all salt come from ice melt

        where (MELT_F /= c0)
          WORK1 = SALT_F/MELT_F ! salinity (msu) of melt water
        elsewhere
          WORK1 = c0
        end where

        TFW(:,:,2) =  RCALCT*MELT_F*fwmass_to_fwflux*WORK1
                      ! + PREC_F*c0 + EVAP_F*c0 + ROFF_F*c0

        do n=3,nt
          TFW(:,:,n) = c0  ! no additional tracers in fresh water
        end do

!-----------------------------------------------------------------------
!
!     if not a variable thickness surface layer or if fw_as_salt_flx
!     flag is on, convert fresh and salt inputs to a virtual salinity
!     flux
!
!-----------------------------------------------------------------------

      else  ! convert fresh water to virtual salinity flux

        STF(:,:,2) = RCALCT*(
     &               (PREC_F+EVAP_F+MELT_F+ROFF_F)*salinity_factor 
     &              + SALT_F*sflux_factor)  
 
!-----------------------------------------------------------------------
!
!     balance salt/freshwater in marginal seas
!
!-----------------------------------------------------------------------
 
      if  (lms_balance .and. sfwf_formulation /= 'partially-coupled' )
     & call ms_balancing (STF(:,:,2),EVAP_F, PREC_F, MELT_F,ROFF_F
     &,                   SALT_F, 'salt')
 
      endif
 
!-----------------------------------------------------------------------
!
!     unpack and distribute net shortwave and convert units
!
!-----------------------------------------------------------------------

      call scatter_global(WORK1, RBUFF(:,:,krbuff_netsw), master_task)
      SHF_QSW = WORK1*RCALCT*hflux_factor  !  convert from W/m**2

!-----------------------------------------------------------------------
!
!     unpack and distribute non-sw heat flux (lat+sen+lw+melt)
!
!-----------------------------------------------------------------------

      call scatter_global(WORK1, RBUFF(:,:,krbuff_senhf ), master_task)
      call scatter_global(WORK2, RBUFF(:,:,krbuff_lwup  ), master_task)
      call scatter_global(WORK3, RBUFF(:,:,krbuff_lwdwn ), master_task)
      call scatter_global(WORK4, RBUFF(:,:,krbuff_melthf), master_task)
 
      SENH_F  = WORK1
      LWUP_F  = WORK2
      LWDN_F  = WORK3
      MELTH_F = WORK4

!-----------------------------------------------------------------------
!
!     combine and convert from W/m**2
!           (note: latent heat flux = evaporation*latent_heat_vapor)
!
!-----------------------------------------------------------------------

      STF(:,:,1) = (EVAP_F*latent_heat_vapor+WORK1+WORK2+WORK3+WORK4)
     &             *RCALCT*hflux_factor 

!-----------------------------------------------------------------------
!
!     unpack and distribute fractional ice coverage
!
!-----------------------------------------------------------------------

      call scatter_global(WORK1, RBUFF(:,:,krbuff_ifrac), master_task)

      IFRAC = WORK1 * RCALCT

      if (ldiag_cpl) call coupled_diagnostics_recv (RBUFF)

!-----------------------------------------------------------------------

      end subroutine recv_from_coupler

!***********************************************************************

      subroutine send_to_coupler

!-----------------------------------------------------------------------
!
!     this routine packs fields into a message buffer and sends the
!     message to the flux coupler
!
!-----------------------------------------------------------------------

      include 'mpif.h'

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(nibuff) :: 
     &  ibuff                  ! integer control buffer

      integer (kind=int_kind), parameter ::  ! words in received msg
     &  nswords = nsfields*imt_global*jmt_global

      integer (kind=int_kind), dimension(MPI_STATUS_SIZE) ::
     &   mpistatus

      integer (kind=int_kind) ::
     &  ier                 ! error flag for message calls
     &, itmp, jtmp

      real (kind=dbl_kind) :: rval

      real (kind=dbl_kind), dimension(imt_global,jmt_global,nsfields) ::
     &  SBUFF               ! sent message buffer

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  WORK1, WORK2        ! local work space
     &, WORK3, WORK4

!-----------------------------------------------------------------------
!
!     initialize control buffer
!
!-----------------------------------------------------------------------

      ibuff = 0

      if (check_time_flag(cpl_stop_now)) then
        ibuff(3) = 1
      endif


      ibuff(4) = iyear*10000 + imonth*100 + iday
      ibuff(5) = ihour*seconds_in_hour + iminute*seconds_in_minute
     &           + isecond

      if ( lsend_precip_fact )
     &  ibuff(31) = precip_fact * 1.0e6_dbl_kind   ! send real as integer
 
!-----------------------------------------------------------------------
!
!     interpolate onto T-grid points and rotate on T grid
!
!-----------------------------------------------------------------------

      call sw_4pt_nobndy(WORK3,AT0,ATS,ATW,ATSW,
     &                   SBUFF_SUM(:,:,ksbuff_uvel))
      call sw_4pt_nobndy(WORK4,AT0,ATS,ATW,ATSW,
     &                   SBUFF_SUM(:,:,ksbuff_vvel))

      WORK1 = (WORK3*cos(ANGLET)+WORK4*sin(-ANGLET))
     &       * mpercm/tlast_coupled
      WORK2 = (WORK4*cos(ANGLET)-WORK3*sin(-ANGLET))
     &       * mpercm/tlast_coupled

      call gather_global(SBUFF(:,:,ksbuff_uvel), WORK1, master_task)
      call gather_global(SBUFF(:,:,ksbuff_vvel), WORK2, master_task)

!-----------------------------------------------------------------------
!
!     convert and pack surface temperature
!
!-----------------------------------------------------------------------

      WORK1 = SBUFF_SUM(:,:,ksbuff_sst)/tlast_coupled + T0_Kelvin

      call gather_global(SBUFF(:,:,ksbuff_sst), WORK1, master_task)

!-----------------------------------------------------------------------
!
!     convert and pack salinity
!
!-----------------------------------------------------------------------

      WORK1 = SBUFF_SUM(:,:,ksbuff_sss)*salt_to_ppt/tlast_coupled

      call gather_global(SBUFF(:,:,ksbuff_sss), WORK1, master_task)

!-----------------------------------------------------------------------
!
!     interpolate onto T-grid points, then rotate on T grid
!
!-----------------------------------------------------------------------

      call sw_4pt_nobndy(WORK3,AT0,ATS,ATW,ATSW,
     &                   SBUFF_SUM(:,:,ksbuff_dhdx))
 
      call sw_4pt_nobndy(WORK4,AT0,ATS,ATW,ATSW,
     &                   SBUFF_SUM(:,:,ksbuff_dhdy))
 
      WORK1 = (WORK3*cos(ANGLET) + WORK4*sin(-ANGLET))
     &        /grav/tlast_coupled
      WORK2 = (WORK4*cos(ANGLET) - WORK3*sin(-ANGLET))
     &        /grav/tlast_coupled


      call gather_global(SBUFF(:,:,ksbuff_dhdx), WORK1, master_task)
      call gather_global(SBUFF(:,:,ksbuff_dhdy), WORK2, master_task)

!-----------------------------------------------------------------------
!
!     pack heat flux due to freezing/melting (W/m^2)
!     QFLUX computation and units conversion occurs in ice.F
!
!-----------------------------------------------------------------------

      call gather_global(SBUFF(:,:,ksbuff_q), QFLUX, master_task)
 
      tlast_ice = c0
      AQICE     = c0
      QICE      = c0

!-----------------------------------------------------------------------
!
!     send fields to coupler
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        call MPI_SEND(ibuff, nibuff, MPI_INTEGER, cpl_task,
     &                       msgtype_o2d, MPI_COMM_WORLD, ier)

        call MPI_SEND(SBUFF, nswords, MPI_DOUBLE_PRECISION, cpl_task,
     &                       msgtype_o2d, MPI_COMM_WORLD, ier)

      endif

      call broadcast_scalar(ier, master_task)
      if (ier /= MPI_SUCCESS) then
        call exit_POP('ERROR after send to coupler')
      end if
 
      if (ldiag_cpl) call coupled_diagnostics_send (SBUFF)

      tlast_coupled = c0

!-----------------------------------------------------------------------

      end subroutine send_to_coupler

!***********************************************************************

      subroutine sum_buffer

!-----------------------------------------------------------------------
!
!     this routine accumulates sums for averaging fields to
!     be sent to the coupler
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) :: 
     &  delt                ! time interval since last step

!-----------------------------------------------------------------------
!
!     zero buffer if this is the first time after a coupling interval
!
!-----------------------------------------------------------------------

      if (tlast_coupled == c0) SBUFF_SUM = c0

!-----------------------------------------------------------------------
!
!     update time since last coupling
!
!-----------------------------------------------------------------------

      if (avg_ts .or. back_to_back) then
        delt = p5*dtt
      else
        delt =    dtt
      endif
      tlast_coupled = tlast_coupled + delt

!-----------------------------------------------------------------------
!
!     accumulate sums of U,V,T,S and GRADP
!     ice formation flux is handled separately in ice routine
!
!-----------------------------------------------------------------------

      SBUFF_SUM(:,:,ksbuff_uvel) = SBUFF_SUM(:,:,ksbuff_uvel) + delt*
     &                             UVEL(:,:,1,curtime)

      SBUFF_SUM(:,:,ksbuff_vvel) = SBUFF_SUM(:,:,ksbuff_vvel) + delt*
     &                             VVEL(:,:,1,curtime)

      SBUFF_SUM(:,:,ksbuff_sst ) = SBUFF_SUM(:,:,ksbuff_sst ) + delt*
     &                             TRACER(:,:,1,1,curtime)

      SBUFF_SUM(:,:,ksbuff_sss ) = SBUFF_SUM(:,:,ksbuff_sss ) + delt*
     &                             TRACER(:,:,1,2,curtime)

      SBUFF_SUM(:,:,ksbuff_dhdx) = SBUFF_SUM(:,:,ksbuff_dhdx) + delt*
     &                             GRADPX(:,:,curtime)

      SBUFF_SUM(:,:,ksbuff_dhdy) = SBUFF_SUM(:,:,ksbuff_dhdy) + delt*
     &                             GRADPY(:,:,curtime)

!-----------------------------------------------------------------------

      end subroutine sum_buffer

!***********************************************************************

      subroutine mpi_coupled 

!-----------------------------------------------------------------------
!
!     this routine queries all the components of the full coupled
!     system and sets up proper communicators and task ids for each
!     component of the coupled system
!
!     this routine should be called after mpi_init, but before 
!     setting up any internal mpi setups (since these will require
!     the internal communicators returned by this routine)
!
!-----------------------------------------------------------------------

      implicit none

!-----------------------------------------------------------------------
!
!     MPI stuff
!
!-----------------------------------------------------------------------

      include 'mpif.h'

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      character (3) :: cmodel   ! model name temporary

      integer, dimension(3) ::  ! array for creating groups for
     &  range                   !  each coupled model

      integer ::
     &  MPI_GROUP_WORLD,        ! group id for MPI_COMM_WORLD
     &  MPI_GROUP_CPL,          ! group of processors assigned to cpl
     &  MPI_GROUP_ATM,          ! group of processors assigned to atm
     &  MPI_GROUP_OCN,          ! group of processors assigned to ocn
     &  MPI_GROUP_ICE,          ! group of processors assigned to ice
     &  MPI_GROUP_LND           ! group of processors assigned to lnd

      integer ::
     &  n,                           ! dummy loop counter
     &  ierr,                        ! error flag for MPI comms
     &  nprocs_cpl,                  ! total processor count
     &  my_task_coupled,             ! rank of process in coupled domain
     &  cpl_rank_min, cpl_rank_max,  ! processor range for each
     &  ocn_rank_min, ocn_rank_max,  !  component model
     &  atm_rank_min, atm_rank_max,
     &  ice_rank_min, ice_rank_max,
     &  lnd_rank_min, lnd_rank_max,
     &  atm_tid,          ! task id for the process in each component
     &  ice_tid,          !   which communicates w coupler - these are
     &  lnd_tid,          !   defined as the lowest rank process
     &  ocn_tid,          !   in the processor group
     &  MPI_COMM_ATM,     ! MPI communicator to be used for 
     &  MPI_COMM_CPL,     !   messages internal to the component model
     &  MPI_COMM_ICE,     !   as opposed to between models
     &  MPI_COMM_LND      !   these must be defined by all models even
                          !   though only used internally by each model


!-----------------------------------------------------------------------
!
!     determine processor rank in coupled domain
!
!-----------------------------------------------------------------------

      call MPI_COMM_RANK  (MPI_COMM_WORLD, my_task_coupled, ierr)

!-----------------------------------------------------------------------
!
!     determine which group of processes assigned to each model
!
!-----------------------------------------------------------------------

      call MPI_COMM_SIZE (MPI_COMM_WORLD, nprocs_cpl, ierr)

      atm_rank_min = nprocs_cpl
      atm_rank_max = 0
      ocn_rank_min = nprocs_cpl
      ocn_rank_max = 0
      ice_rank_min = nprocs_cpl
      ice_rank_max = 0
      lnd_rank_min = nprocs_cpl
      lnd_rank_max = 0
      cpl_rank_min = nprocs_cpl
      cpl_rank_max = 0

      !***
      !*** each processor broadcasts its model to all the processors
      !*** in the coupled domain
      !***

      do n=0,nprocs_cpl-1
        if (n == my_task_coupled) then
          cmodel = 'ocn'
        else
          cmodel = 'unk'
        endif

        call MPI_BCAST(cmodel, 3, MPI_CHARACTER, n,
     &                 MPI_COMM_WORLD, ierr)

        select case(cmodel)
        case ('atm')
          atm_rank_min = min(atm_rank_min, n)
          atm_rank_max = max(atm_rank_max, n)
        case ('ocn')
          ocn_rank_min = min(ocn_rank_min, n)
          ocn_rank_max = max(ocn_rank_max, n)
        case ('ice')
          ice_rank_min = min(ice_rank_min, n)
          ice_rank_max = max(ice_rank_max, n)
        case ('lnd')
          lnd_rank_min = min(lnd_rank_min, n)
          lnd_rank_max = max(lnd_rank_max, n)
        case ('cpl')
          cpl_rank_min = min(cpl_rank_min, n)
          cpl_rank_max = max(cpl_rank_max, n)
        case ('dud')
        case default
          write(stdout,*) 'Unknown model '
     &,                    cmodel,' in coupled model domain'
          write(stdout,*) 'Model must be atm, cpl, ice, lnd, ocn'
          stop
        end select

      end do

!-----------------------------------------------------------------------
!
!     assume the first processor in each domain is the task that will
!     communicate coupled model messages
!
!-----------------------------------------------------------------------

      atm_tid = atm_rank_min
      cpl_task = cpl_rank_min
      ice_tid = ice_rank_min
      lnd_tid = lnd_rank_min
      ocn_tid = ocn_rank_min

!-----------------------------------------------------------------------
!
!     create subroup and communicators for each models internal 
!     communciations, note that MPI_COMM_CREATE must be called by
!     all processes in MPI_COMM_WORLD so this must be done by all
!     models consistently and in the same order.
!
!-----------------------------------------------------------------------

      call MPI_COMM_GROUP(MPI_COMM_WORLD, MPI_GROUP_WORLD, ierr)

      range(3) = 1

      range(1) = atm_rank_min
      range(2) = atm_rank_max
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, 
     &                          MPI_GROUP_ATM, ierr)

      range(1) = ocn_rank_min
      range(2) = ocn_rank_max
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, 
     &                          MPI_GROUP_OCN, ierr)

      range(1) = ice_rank_min
      range(2) = ice_rank_max
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, 
     &                          MPI_GROUP_ICE, ierr)

      range(1) = lnd_rank_min
      range(2) = lnd_rank_max
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, 
     &                          MPI_GROUP_LND, ierr)

      range(1) = cpl_rank_min
      range(2) = cpl_rank_max
      call MPI_GROUP_RANGE_INCL(MPI_GROUP_WORLD, 1, range, 
     &                          MPI_GROUP_CPL, ierr)

      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_ATM, 
     &                      MPI_COMM_ATM, ierr)
 
      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_OCN, 
     &                      MPI_COMM_OCN, ierr)
 
      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_ICE, 
     &                      MPI_COMM_ICE, ierr)
 
      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_LND, 
     &                      MPI_COMM_LND, ierr)
 
      call MPI_COMM_CREATE (MPI_COMM_WORLD, MPI_GROUP_CPL, 
     &                      MPI_COMM_CPL, ierr)
 
!-----------------------------------------------------------------------

      end subroutine mpi_coupled

#endif
!***********************************************************************

      subroutine coupled_diagnostics_recv (RBUFF)
#ifdef coupled
 
      real (kind=dbl_kind) :: 
     &  gsum_ifrac  
     &, gsum_mslp
     &, gsum_taux    
     &, gsum_tauy    
     &, gsum_netsw  
     &, gsum_senhf
     &, gsum_lwup 
     &, gsum_lwdn 
     &, gsum_melthf 
     &, gsum_salt    
     &, gsum_prec    
     &, gsum_evap    
     &, gsum_melt    
     &, gsum_roff    
     &, m2percm2   
 
      real (kind=dbl_kind), dimension(imt_global,jmt_global,nrfields) ::
     &  RBUFF               ! received message buffer
 
      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  WORK1               ! local work space
 
      m2percm2  = mpercm*mpercm
 
      call scatter_global(WORK1, RBUFF(:,:,krbuff_ifrac), master_task)
      WORK1 = WORK1*TAREA
      gsum_ifrac   = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, RBUFF(:,:,krbuff_mslp), master_task)
      WORK1 = WORK1*TAREA
      gsum_mslp     = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, RBUFF(:,:,krbuff_taux), master_task)
      WORK1 = WORK1*TAREA
      gsum_taux = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, RBUFF(:,:,krbuff_tauy), master_task)
      WORK1 = WORK1*TAREA
      gsum_tauy = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, RBUFF(:,:,krbuff_netsw), master_task)
      WORK1 = WORK1*TAREA
      gsum_netsw = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, RBUFF(:,:,krbuff_senhf ), master_task)
      WORK1 = WORK1*TAREA
      gsum_senhf = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, RBUFF(:,:,krbuff_lwup  ), master_task)
      WORK1 = WORK1*TAREA
      gsum_lwup  = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, RBUFF(:,:,krbuff_lwdwn ), master_task)
      WORK1 = WORK1*TAREA
      gsum_lwdn  = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, RBUFF(:,:,krbuff_melthf), master_task)
      WORK1 = WORK1*TAREA
      gsum_melthf= global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, RBUFF(:,:,krbuff_salt), master_task)
      WORK1 = WORK1*TAREA
      gsum_salt = global_sum(WORK1,RCALCT)*m2percm2
  
      call scatter_global(WORK1, RBUFF(:,:,krbuff_prec), master_task)
      WORK1 = WORK1*TAREA
      gsum_prec = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, RBUFF(:,:,krbuff_evap), master_task)
      WORK1 = WORK1*TAREA
      gsum_evap = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, RBUFF(:,:,krbuff_melt), master_task)
      WORK1 = WORK1*TAREA
      gsum_melt = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, RBUFF(:,:,krbuff_roff), master_task)
      WORK1 = WORK1*TAREA
      gsum_roff = global_sum(WORK1,RCALCT)*m2percm2
 
      if  (my_task == master_task) then
        call int_to_char (4,iyear   ,cyear  )
        call int_to_char (2,imonth  ,cmonth )
        call int_to_char (2,iday    ,cday   )
        call int_to_char (2,ihour   ,chour  )
        call int_to_char (2,iminute ,cminute)
        call int_to_char (2,isecond ,csecond)
        write(stdout,*) ' report global averages of fluxes from cpl at '
     &,                    ' ', cyear,'/',cmonth,'/',cday, '  '
     &,                    ' ', chour,':',cminute,':',csecond
 
        write(stdout,1100)'ocn','recv', krbuff_ifrac ,gsum_ifrac , 
     &                                                    'ifrac' 
        write(stdout,1100)'ocn','recv', krbuff_mslp  ,gsum_mslp  ,'mslp'
        write(stdout,1120)'ocn','recv', krbuff_taux  ,gsum_taux  ,'taux'
        write(stdout,1120)'ocn','recv', krbuff_tauy  ,gsum_tauy  ,'tauy'
        write(stdout,1130)'ocn','recv', krbuff_netsw ,gsum_netsw ,
     &                                                    'netsw' 
        write(stdout,1130)'ocn','recv', krbuff_senhf ,gsum_senhf ,
     &                                                    'senhf'  
        write(stdout,1130)'ocn','recv', krbuff_lwup  ,gsum_lwup  ,'lwup'
        write(stdout,1130)'ocn','recv', krbuff_lwdwn ,gsum_lwdn , 'lwdn'
        write(stdout,1130)'ocn','recv', krbuff_melthf,gsum_melthf,
     &                                                    'melthf' 
        write(stdout,1140)'ocn','recv', krbuff_salt  ,gsum_salt  ,'salt'
        write(stdout,1140)'ocn','recv', krbuff_prec  ,gsum_prec  ,'prec'
        write(stdout,1140)'ocn','recv', krbuff_evap  ,gsum_evap  ,'evap'
        write(stdout,1140)'ocn','recv', krbuff_melt  ,gsum_melt  ,'melt'
        write(stdout,1140)'ocn','recv', krbuff_roff  ,gsum_roff  ,'roff'
        call shr_sys_flush(stdout)
      endif
 
1100  format ('comm_diag ', a3, 1x, a4, 1x, i3, 1x, es26.19, 1x, a6)
1120  format ('comm_diag ', a3, 1x, a4, 1x, i3, 1x, es26.19, 1x, a6
     &,       1x, '(Newtons)'    )
1130  format ('comm_diag ', a3, 1x, a4, 1x, i3, 1x, es26.19, 1x, a6
     &,       1x, '(Watts)')
1140  format ('comm_diag ', a3, 1x, a4, 1x, i3, 1x, es26.19, 1x, a6
     &,       1x, '(kg/s)' )
 
#else
      real (kind=dbl_kind) :: RBUFF
#endif
      end subroutine coupled_diagnostics_recv
 
 
      subroutine coupled_diagnostics_send (SBUFF)
#ifdef coupled
 
      real (kind=dbl_kind) :: 
     &  gsum_sst
     &, gsum_sss     
     &, gsum_uvel    
     &, gsum_vvel    
     &, gsum_dhdx    
     &, gsum_dhdy    
     &, gsum_q       
     &, m2percm2   
 
 
      real (kind=dbl_kind), dimension(imt_global,jmt_global,nsfields) ::
     &  SBUFF               ! sent message buffer
 
      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  WORK1               ! local work space
 
 
      m2percm2  = mpercm*mpercm
 
 
      call scatter_global(WORK1, SBUFF(:,:,ksbuff_sst), master_task)
      WORK1 = WORK1*TAREA
      gsum_sst  = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, SBUFF(:,:,ksbuff_sss), master_task)
      WORK1 = WORK1*TAREA
      gsum_sss  = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, SBUFF(:,:,ksbuff_uvel), master_task)
      WORK1 = WORK1*TAREA
      gsum_uvel = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, SBUFF(:,:,ksbuff_vvel), master_task)
      WORK1 = WORK1*TAREA
      gsum_vvel = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, SBUFF(:,:,ksbuff_dhdx), master_task)
      WORK1 = WORK1*TAREA
      gsum_dhdx = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, SBUFF(:,:,ksbuff_dhdy), master_task)
      WORK1 = WORK1*TAREA
      gsum_dhdy = global_sum(WORK1,RCALCT)*m2percm2
 
      call scatter_global(WORK1, SBUFF(:,:,ksbuff_q), master_task)
      WORK1 = WORK1*TAREA
      gsum_q = global_sum(WORK1,RCALCT)*m2percm2
 
      if  (my_task == master_task) then
        call int_to_char (4,iyear   ,cyear  )
        call int_to_char (2,imonth  ,cmonth )
        call int_to_char (2,iday    ,cday   )
        call int_to_char (2,ihour   ,chour  )
        call int_to_char (2,iminute ,cminute)
        call int_to_char (2,isecond ,csecond)
        write(stdout,*) ' report global averages of fluxes to cpl at '
     &,                    ' ', cyear,'/',cmonth,'/',cday, '  '
     &,                    ' ', chour,':',cminute,':',csecond
 
        write(stdout,1100)'ocn','send', ksbuff_sst  ,gsum_sst  ,'sst'
        write(stdout,1100)'ocn','send', ksbuff_sss  ,gsum_sss  ,'sss'
        write(stdout,1100)'ocn','send', ksbuff_uvel ,gsum_uvel ,'uvel'
        write(stdout,1100)'ocn','send', ksbuff_vvel ,gsum_vvel ,'vvel'
        write(stdout,1100)'ocn','send', ksbuff_dhdx ,gsum_dhdx ,'dhdx'
        write(stdout,1100)'ocn','send', ksbuff_dhdy ,gsum_dhdy ,'dhdy'
        write(stdout,1150)'ocn','send', ksbuff_q    ,gsum_q    ,'q   '
        call shr_sys_flush(stdout)
      endif
 
1100  format ('comm_diag ', a3, 1x, a4, 1x, i3, 1x, es26.19, 1x, a6)
1150  format ('comm_diag ', a3, 1x, a4, 1x, i3, 1x, es26.19, 1x, a6
     &,        1x, '(Watts)')
 
#else
      real (kind=dbl_kind) :: SBUFF
#endif
      end subroutine coupled_diagnostics_send
 
 

!***********************************************************************

      end module forcing_coupled

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
