!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module forcing_sfwf

!***********************************************************************

!-----------------------------------------------------------------------
!
!     contains routines and variables used for determining the
!       surface fresh water flux.
!
!     CVS:$Id: forcing_sfwf.F,v 1.3.2.7 2002/01/25 23:24:59 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use io
      use grid
      use forcing_tools
      use forcing_shf
      use time_management
      use prognostic
      use exit_mod
      use ice

      implicit none
      save

      real (kind=dbl_kind), allocatable, dimension(:,:,:,:) ::
     &   SFWF_DATA    ! surface salinity at multiple times to be restored 
                      ! to and may be interpolated in time to get SFWF

      real (kind=dbl_kind), allocatable, dimension(:,:,:) ::
     &   SFWF_COMP

      real (kind=dbl_kind), allocatable, dimension(:,:,:,:) ::
     &   TFW_COMP

      real (kind=dbl_kind), dimension(12) :: 
     &   sfwf_data_time  ! time (in hours) corresponding to surface fresh 
                         ! water fluxes

      real (kind=dbl_kind), dimension(20) :: sfwf_data_renorm

      real (kind=dbl_kind) :: 
     &    sfwf_data_inc    ! time increment between values of forcing data
     &  , sfwf_data_next   ! time that will be used for the next value of 
                           ! forcing data that is needed
     &  , sfwf_data_update ! time when the a new forcing value needs to be 
                           ! added to interpolation set
     &  , sfwf_interp_inc  ! time increment between interpolation
     &  , sfwf_interp_next ! time when next interpolation will be done
     &  , sfwf_interp_last ! time when last interpolation was done
     &  , sfwf_restore_tau
     &  , sfwf_restore_rtau
     &  , sfwf_weak_restore
     &  , sfwf_strong_restore
     &  , sfwf_strong_restore_ms

      integer (kind=int_kind) :: 
     &    sfwf_interp_order  !  order of temporal interpolation
     &  , sfwf_data_time_min_loc  ! index of the third dimension of
                                  ! SFWF_DATA, sfwf_data_time containing 
                                  ! the minimum forcing time
     &  , sfwf_data_num_fields
     &  , sfwf_num_comps
     
      logical (kind=log_kind) ::
     &  lfw_as_salt_flx           ! treat fw flux as virtual salt flux
                                  ! even with var.thickness sfc layer

!-----------------------------------------------------------------------
!  the following is necessary for sss restoring and for partially-coupled
!  option
!-----------------------------------------------------------------------
      integer (kind=int_kind) :: 
     &    sfwf_data_sss

!-----------------------------------------------------------------------
!  the following is necessary for bulk-NCEP
!  some are also used for the partially coupled option
!-----------------------------------------------------------------------
      integer (kind=int_kind) :: 
     &    sfwf_data_precip
     &  , sfwf_comp_precip
     &  , sfwf_comp_evap
     &  , sfwf_comp_wrest
     &  , sfwf_comp_srest

      real (kind=dbl_kind) ::
     &    sum_precip
     &  , ann_avg_precip
     &  , fres_hor_ave
     &  , fres_hor_ave_area
!    &  , sum_fw
!    &  , ann_avg_fw
     &  , ssh_initial, ssh_final

      real (kind=dbl_kind) :: precip_fact = c1
      real (kind=dbl_kind), dimension (km) :: sal_initial, sal_final
      logical (kind=log_kind) :: ladjust_precip
 
!-----------------------------------------------------------------------
!  the following are used with the partially-coupled option
!
!     lsend_precip_fact = .T.   precip_fact is sent to coupler so that
!                               it will use this value for its freshwater
!                               balance
!                         .F.   precip_fact is not sent to coupler
!-----------------------------------------------------------------------
 
      real (kind=dbl_kind), parameter :: precip_mean = 3.4e-5_dbl_kind
      logical (kind=log_kind) :: lsend_precip_fact
      integer (kind=int_kind) :: 
     &    sfwf_comp_cpl
     &,   sfwf_data_flxio
     &,   sfwf_comp_flxio
     &,   tfw_num_comps
     &,   tfw_comp_cpl
     &,   tfw_comp_flxio

      character (char_len) :: 
     &    sfwf_data_type   ! period keyword for the period of the 
                           ! forcing data
     &  , sfwf_filename    ! name of file conainting forcing data
     &  , sfwf_interp_freq ! period keyword for the period of the
                           ! temporal interpolation
     &  , sfwf_interp_type
     &  , sfwf_formulation
     &  , sfwf_data_label

!-----------------------------------------------------------------------
!  the following is used to enable the balancing of precip, evaporation,
!  melt, runoff, and salt in marginal seas
!  presently, this is an option only with the sfc_layer_oldfree option
!-----------------------------------------------------------------------
      logical (kind=log_kind) ::
     &    lms_balance           
!***********************************************************************

      contains

!***********************************************************************

!***********************************************************************

      subroutine init_sfwf(SFWF)
      
!-----------------------------------------------------------------------
!
!     initialize surface fresh water flux forcing by either calculating 
!       or reading in the surface fresh water flux.  also do initial 
!       book-keeping concerning when new data is needed for the temporal 
!       interpolation and when the forcing will need to be updated.
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   SFWF    !  surface fresh water flux at current timestep

      integer(kind=int_kind) :: k, n, nu
     &, nml_error 
     &, allocate_error_status

      character (char_len) :: forcing_filename

      namelist /forcing_sfwf_nml/ sfwf_data_type, sfwf_data_inc
     &         ,sfwf_interp_type, sfwf_interp_freq
     &         ,sfwf_interp_inc, sfwf_restore_tau
     &         ,sfwf_filename, sfwf_data_renorm, sfwf_formulation
     &         ,ladjust_precip, sfwf_weak_restore, sfwf_strong_restore
     &         ,lfw_as_salt_flx, lsend_precip_fact
     &         ,sfwf_strong_restore_ms
     &         ,lms_balance

!-----------------------------------------------------------------------
!
!     read surface fresh water flux namelist input after setting 
!       default values.
!
!-----------------------------------------------------------------------

      sfwf_formulation = 'restoring'
      sfwf_data_type   = 'analytic'
      sfwf_data_inc    = 1.e20_dbl_kind
      sfwf_interp_type = 'nearest'
      sfwf_interp_freq = 'never'
      sfwf_interp_inc  = 1.e20_dbl_kind
      sfwf_restore_tau = 1.e20_dbl_kind
      sfwf_filename    = 'unknown-sfwf'
      sfwf_data_renorm = c1
!     sfwf_data_renorm = 1.e-3_dbl_kind  !  convert from psu to msu
      ladjust_precip   = .false.
      lms_balance      = .false.
      lfw_as_salt_flx  = .false.
      lsend_precip_fact = .false.
      sfwf_weak_restore   = 0.092_dbl_kind
      sfwf_strong_restore = 0.6648_dbl_kind
      sfwf_strong_restore_ms = 0.6648_dbl_kind

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in,nml=forcing_sfwf_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading forcing_sfwf_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, forcing_sfwf_nml)
          write(stdout,*) ' '
      endif

      call broadcast_scalar(sfwf_data_type,   master_task)
      call broadcast_scalar(sfwf_data_inc,    master_task)
      call broadcast_scalar(sfwf_interp_type, master_task)
      call broadcast_scalar(sfwf_interp_freq, master_task)
      call broadcast_scalar(sfwf_interp_inc,  master_task)
      call broadcast_scalar(sfwf_restore_tau, master_task)
      call broadcast_scalar(sfwf_filename,    master_task)
      call broadcast_scalar(sfwf_formulation, master_task)
      call broadcast_vector(sfwf_data_renorm, master_task,20)
      call broadcast_scalar(ladjust_precip,   master_task)
      call broadcast_scalar(sfwf_weak_restore,   master_task)
      call broadcast_scalar(sfwf_strong_restore, master_task)
      call broadcast_scalar(lfw_as_salt_flx,     master_task)
      call broadcast_scalar(lsend_precip_fact,   master_task)
      call broadcast_scalar(lms_balance,         master_task)
      call broadcast_scalar(sfwf_strong_restore_ms,master_task)

      if (my_task == master_task) then
        if ( sfwf_formulation == 'bulk-NCEP' )  write (stdout,1000) 
      endif 
      
!-----------------------------------------------------------------------
!     convert data_type to 'monthly-calendar' if input is 'monthly'
!-----------------------------------------------------------------------

      if (sfwf_data_type == 'monthly') 
     &    sfwf_data_type = 'monthly-calendar'

!-----------------------------------------------------------------------
!     set values based on sfwf_formulation
!-----------------------------------------------------------------------
      
      select case (sfwf_formulation)
      case ('restoring')
        sfwf_data_num_fields = 1
        sfwf_data_sss = 1
        
      case ('bulk-NCEP')
        sfwf_data_num_fields = 2
        sfwf_data_sss = 1
        sfwf_data_precip = 2

        sfwf_num_comps = 4
        sfwf_comp_precip = 1
        sfwf_comp_evap = 2
        sfwf_comp_wrest = 3
        sfwf_comp_srest = 4

      case ('partially-coupled')
        sfwf_data_num_fields = 2 
        sfwf_data_sss   = 1
        sfwf_data_flxio = 2

        sfwf_num_comps = 4 
        sfwf_comp_wrest = 1
        sfwf_comp_srest = 2
        sfwf_comp_cpl   = 3
        sfwf_comp_flxio = 4

        tfw_num_comps  = 2
        tfw_comp_cpl   = 1
        tfw_comp_flxio = 2

      case default
         call exit_POP('init_sfwf: Unknown value for sfwf_formulation')

      end select


      if ( sfwf_formulation == 'bulk-NCEP' .or.
     &     sfwf_formulation == 'partially-coupled' ) then
 
!-----------------------------------------------------------------------
!     calculate initial salinity profile for ocean points that are
!       not marginal seas.
!-----------------------------------------------------------------------
        !*** very first step of run
        if (ladjust_precip .and. nsteps_total == 0) then
          sum_precip = c0
          ssh_initial = c0
!         sum_fw = c0

          do k = 1,km
            SFWF =  merge(TRACER(:,:,k,2,curtime)*TAREA*dz(k), c0 
     &                    , k <= KMT .and. MASK_SR > 0)  ! use SFWF as temporary
            sal_initial(k) = global_sum(SFWF, ONE)/volgk_m_marg(k)
            sal_initial(k) = sal_initial(k)*1.0e-6_dbl_kind  ! convert to m^3
          enddo
        endif
      endif

      
!-----------------------------------------------------------------------
!     calculate inverse of restoring time scale and convert to seconds.
!-----------------------------------------------------------------------

      sfwf_restore_rtau = c1/(seconds_in_day*sfwf_restore_tau)
      
!-----------------------------------------------------------------------
!     convert interp_type to corresponding integer value.
!-----------------------------------------------------------------------

      select case (sfwf_interp_type)
      case ('nearest')
        sfwf_interp_order = 1
        
      case ('linear')
        sfwf_interp_order = 2
        
      case ('4point')
        sfwf_interp_order = 4
        
      case default
        call exit_POP('init_sfwf: Unknown value for sfwf_interp_type') 
        
      end select

!-----------------------------------------------------------------------
!
!     set values of the surface fresh water flux arrays (SFWF or 
!       SFWF_DATA) depending on type of the surface fresh water flux 
!       data.
!
!-----------------------------------------------------------------------

      select case (sfwf_data_type)
      
!-----------------------------------------------------------------------
!     no surface fresh water flux, therefore no interpolation in time 
!       is needed (sfwf_interp_freq = 'none'), nor are there any new 
!       values to be used (sfwf_data_next = sfwf_data_update = never).
!-----------------------------------------------------------------------
      case ('none')
      
        SFWF = c0
        sfwf_data_next = never
        sfwf_data_update = never
        sfwf_interp_freq = 'never'
        
!-----------------------------------------------------------------------
!     simple analytic surface salinity that is constant in time, 
!       therefore no new values will be needed (sfwf_data_next = 
!       sfwf_data_update = never).
!-----------------------------------------------------------------------
      case ('analytic')
      
        allocate( SFWF_DATA(imt,jmt,sfwf_data_num_fields,1)
     &          , STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,   exit_message = 'forcing_sfwf.F -- SFWF_DATA')
 
        SFWF_DATA = c0

        select case (sfwf_formulation)
        case ('restoring')
          SFWF_DATA = 0.035_dbl_kind
        
        end select
        
        sfwf_data_next = never
        sfwf_data_update = never
        sfwf_interp_freq = 'never'
        
!-----------------------------------------------------------------------
!     annual mean climatological surface salinity (read in from a file)
!       that is constant in time, therefore no new values will be needed
!       (sfwf_data_next = sfwf_data_update = never).
!-----------------------------------------------------------------------
      case ('annual')
      
        allocate( SFWF_DATA(imt,jmt,sfwf_data_num_fields,1)
     &          , STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,   exit_message = 'forcing_sfwf.F -- SFWF_DATA')
 
        SFWF_DATA = c0
      
        forcing_filename = sfwf_filename
        call open_parallel_file(nu,forcing_filename,recl_dbl)

        select case (sfwf_formulation)
        case ('restoring')
          call read_array(nu,SFWF_DATA(:,:,sfwf_data_sss,1))
          
        case ('bulk-NCEP')
          call read_array(nu,SFWF_DATA(:,:,sfwf_data_sss,1))
          call read_array(nu,SFWF_DATA(:,:,sfwf_data_precip,1))
          
          allocate( SFWF_COMP(imt,jmt,sfwf_num_comps)
     &          , STAT = allocate_error_status )
          call check_allocate (allocate_error_status
     &,     exit_message = 'forcing_sfwf.F -- SFWF_COMP')
 
          SFWF_COMP = c0  ! initialize

        case ('partially-coupled')
          call read_array(nu,SFWF_DATA(:,:,sfwf_data_sss,1))
          call read_array(nu,SFWF_DATA(:,:,sfwf_data_flxio,1))
 
          allocate( SFWF_COMP(imt,jmt,sfwf_num_comps)
     &          , STAT = allocate_error_status )
          call check_allocate (allocate_error_status
     &,     exit_message = 'forcing_sfwf.F -- SFWF_COMP')
 
          SFWF_COMP = c0  ! initialize
 
        end select
        
        call close_parallel_file(nu)
        
!-----------------------------------------------------------------------
!     renormalize values if necessary to compensate for different units.
!-----------------------------------------------------------------------
        do n = 1,sfwf_data_num_fields
          if(sfwf_data_renorm(n) /= c1) SFWF_DATA(:,:,n,:)
     &      = SFWF_DATA(:,:,n,:)*sfwf_data_renorm(n)
        enddo

        sfwf_data_next = never
        sfwf_data_update = never
        sfwf_interp_freq = 'never'
      
        if(my_task.eq.master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' SFWF Annual file read: ', forcing_filename
        endif
        
!-----------------------------------------------------------------------
!     monthly mean climatological surface fresh water flux. all 
!       12 months are read in from a file. interpolation order 
!       (sfwf_interp_order) may be specified with namelist input.
!-----------------------------------------------------------------------               
      case ('monthly-equal','monthly-calendar')
      
        allocate( SFWF_DATA(imt,jmt,sfwf_data_num_fields,0:12)
     &          , STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,   exit_message = 'forcing_sfwf.F -- SFWF_DATA')
        SFWF_DATA = c0

        call find_forcing_times(sfwf_data_time, sfwf_data_inc
     &       , sfwf_interp_type, sfwf_data_next
     &       , sfwf_data_time_min_loc, sfwf_data_update, sfwf_data_type)
        
        forcing_filename = sfwf_filename
        call open_parallel_file(nu,forcing_filename,recl_dbl)

        select case (sfwf_formulation)
        case ('restoring')
          call read_array(nu,SFWF_DATA(:,:,sfwf_data_sss,1:12))

        case ('bulk-NCEP')
          call read_array(nu,SFWF_DATA(:,:,sfwf_data_sss,1:12))
          call read_array(nu,SFWF_DATA(:,:,sfwf_data_precip,1:12))

          allocate( SFWF_COMP(imt,jmt,sfwf_num_comps)
     &          , STAT = allocate_error_status )
          call check_allocate (allocate_error_status
     &,     exit_message = 'forcing_sfwf.F -- SFWF_COMP')
          SFWF_COMP = c0  ! initialize
 
        case ('partially-coupled')
          call read_array(nu,SFWF_DATA(:,:,sfwf_data_sss,1:12))
          call read_array(nu,SFWF_DATA(:,:,sfwf_data_flxio,1:12))
 
          allocate( SFWF_COMP(imt,jmt,sfwf_num_comps)
     &          , STAT = allocate_error_status )
          call check_allocate (allocate_error_status
     &,     exit_message = 'forcing_sfwf.F -- SFWF_COMP')
          SFWF_COMP = c0  ! initialize

        end select
        
        call close_parallel_file(nu)

!-----------------------------------------------------------------------
!     renormalize values if necessary to compensate for different units.
!-----------------------------------------------------------------------
        do n = 1,sfwf_data_num_fields
          if(sfwf_data_renorm(n) /= c1) SFWF_DATA(:,:,n,:)
     &      = SFWF_DATA(:,:,n,:)*sfwf_data_renorm(n)
        enddo

        if(my_task.eq.master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' SFWF Monthly file read: ', forcing_filename
        endif        
        
!-----------------------------------------------------------------------
!     surface salinity specified every n-hours, where the n-hour 
!       increment should be specified with namelist input  
!       (sfwf_data_inc). only as many times as are necessary based on 
!       the order of the temporal interpolation scheme 
!       (sfwf_interp_order) reside in memory at any given time.
!-----------------------------------------------------------------------

      case ('n-hour')
      
        allocate( SFWF_DATA(imt,jmt,sfwf_data_num_fields
     &           ,0:sfwf_interp_order) ,STAT = allocate_error_status )
          call check_allocate (allocate_error_status
     &,     exit_message = 'forcing_sfwf.F -- SFWF_DATA')
        SFWF_DATA = c0

        call find_forcing_times(sfwf_data_time, sfwf_data_inc
     &      , sfwf_interp_type, sfwf_data_next
     &      , sfwf_data_time_min_loc, sfwf_data_update, sfwf_data_type)
        
        do n = 1, sfwf_interp_order
          call get_forcing_filename(forcing_filename,sfwf_filename
     &                             ,sfwf_data_time(n),sfwf_data_inc)

          call open_parallel_file(nu,forcing_filename,recl_dbl)

          select case (sfwf_formulation)
          case ('restoring')
            call read_array(nu,SFWF_DATA(:,:,:,n))
            
          case ('bulk-NCEP')
            call read_array(nu,SFWF_DATA(:,:,sfwf_data_sss,n))
            call read_array(nu,SFWF_DATA(:,:,sfwf_data_precip,n))

          case ('partially-coupled')
            call read_array(nu,SFWF_DATA(:,:,sfwf_data_sss,n))
            call read_array(nu,SFWF_DATA(:,:,sfwf_data_flxio,n))
 
          end select

          call close_parallel_file(nu)

          if(my_task.eq.master_task) then
            write(stdout,*) ' '
            write(stdout,*) ' SFWF n-hour file read: ', forcing_filename
          endif
        enddo
                
        if( sfwf_formulation == 'bulk-NCEP' .or.
     &      sfwf_formulation == 'partially-coupled' ) then
          allocate( SFWF_COMP(imt,jmt,sfwf_num_comps)
     &          , STAT = allocate_error_status )
          call check_allocate (allocate_error_status
     &,     exit_message = 'forcing_sfwf.F -- SFWF_COMP')
          SFWF_COMP = c0  ! initialize
        endif

!-----------------------------------------------------------------------
!     renormalize values if necessary to compensate for different units.
!-----------------------------------------------------------------------
        do n = 1,sfwf_data_num_fields
          if(sfwf_data_renorm(n) /= c1) SFWF_DATA(:,:,n,:)
     &      = SFWF_DATA(:,:,n,:)*sfwf_data_renorm(n)
        enddo

      case default

        call exit_POP('init_sfwf: Unknown value for sfwf_data_type') 
        
      end select

      if ( sfwf_formulation == 'partially-coupled' ) then
        allocate ( TFW_COMP(imt,jmt,nt,tfw_num_comps), 
     &             STAT = allocate_error_status )
          call check_allocate (allocate_error_status
     &,     exit_message = 'forcing_sfwf.F -- TFW_COMP')
        TFW_COMP = c0
      endif

!-----------------------------------------------------------------------
!
!  now check interpolation period (sfwf_interp_freq) to set the
!    time for the next temporal interpolation (sfwf_interp_next).
!
!  if no interpolation is to be done, set next interpolation time 
!    to a large number so the surface fresh water flux update test  
!    in routine set_surface_forcing will always be false.
!
!  if interpolation is to be done every n-hours, find the first 
!    interpolation time greater than the current time.
!
!  if interpolation is to be done every timestep, set next interpolation 
!    time to a large negative number so the surface fresh water flux 
!    update test in routine set_surface_forcing will always be true.
!
!-----------------------------------------------------------------------

      select case (sfwf_interp_freq)
      
      case ('never')
      
        sfwf_interp_next = never
        sfwf_interp_last = never
        sfwf_interp_inc  = c0
        
      case ('n-hour')
      
        call find_interp_time(sfwf_interp_inc, sfwf_interp_next)

      case ('every-timestep')
      
        sfwf_interp_next = always
        sfwf_interp_inc  = c0
        
      case default

        call exit_POP('init_sfwf: Unknown value for sfwf_interp_freq')

      end select

      if(nsteps_total == 0) sfwf_interp_last = thour00

!-----------------------------------------------------------------------
!  echo forcing options to stdout.
!-----------------------------------------------------------------------

      sfwf_data_label = 'Surface Fresh Water Flux'
      call echo_forcing_options(sfwf_data_type
     &   , sfwf_formulation, sfwf_data_inc
     &   , sfwf_interp_freq, sfwf_interp_type
     &   , sfwf_interp_inc, sfwf_data_label)

      if (my_task == master_task) then
        if (lfw_as_salt_flx .and. sfc_layer_type == sfc_layer_varthick)
     &    write(stdout,*) 
     &      '    Fresh water flux input as virtual salt flux'
      endif

!-----------------------------------------------------------------------

 1000 format (/, 4x, 'WARNING (forcing_sfwf): runoff and marginal sea',
     &    /, 4x, '   balance cannot be used with bulk-NCEP option.')

      end subroutine init_sfwf

!***********************************************************************

      subroutine set_sfwf(SFWF)
      
!-----------------------------------------------------------------------
!
!     update the current value of the surface fresh water flux array 
!       (SFWF) by interpolating to the current time.  it may also be 
!       necessary to use newer data values than were used in the 
!       previous interpolation.
!       
!-----------------------------------------------------------------------

      implicit none

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   SFWF    !  surface fresh water flux at current timestep
                      
      integer (kind=int_kind) :: nu, first, second, third, fourth
     &  , sfwf_data_time_max_loc
     
!-----------------------------------------------------------------------
!     check if new data is necessary for interpolation.  if yes, then
!       shuffle indices in SFWF_DATA and sfwf_data_time arrays  
!       and read in new data if necessary ('n-hour' case).  also 
!       increment values of sfwf_data_time_min_loc, sfwf_data_next 
!       and sfwf_data_update.then do the temporal interpolation. note 
!       that no new data is necessary for 'analytic' and 'annual' cases.
!-----------------------------------------------------------------------
      
      select case(sfwf_data_type)

      case('analytic')

        select case (sfwf_formulation)
        case ('restoring')      
          SFWF =  (SFWF_DATA(:,:,sfwf_data_sss,1) -
     &             TRACER(:,:,1,2,curtime))*sfwf_restore_rtau*dz(1)
     
        end select

      case('annual')
        select case (sfwf_formulation)
        case ('restoring')            
          SFWF =  (SFWF_DATA(:,:,sfwf_data_sss,1) - 
     &             TRACER(:,:,1,2,curtime))*sfwf_restore_rtau*dz(1)
     
        case ('bulk-NCEP')
          call calc_sfwf_bulk_ncep(1,SFWF)

        case ('partially-coupled')
          call calc_sfwf_partially_coupled (1)
 
        end select

      case ('monthly-equal','monthly-calendar')

        sfwf_data_label = 'SFWF Monthly'
        if(thour00 >= sfwf_data_update)               
     &    call update_forcing_data(sfwf_data_time
     &    , sfwf_data_time_min_loc, sfwf_interp_type, sfwf_data_next
     &    , sfwf_data_update, sfwf_data_type, sfwf_data_inc
     &    , sfwf_filename, sfwf_data_renorm, SFWF_DATA(:,:,:,1:12)
     &    , sfwf_data_num_fields, 12, sfwf_data_label)

        select case (sfwf_formulation)
        case ('restoring')            

          if(thour00 >= sfwf_interp_next .or. nsteps_run == 0) then
            call interpolate_forcing(SFWF_DATA(:,:,sfwf_data_sss,0)
     &      , SFWF_DATA(:,:,sfwf_data_sss,1:12), sfwf_data_time
     &      , sfwf_interp_type, sfwf_data_time_min_loc, 12
     &      , sfwf_interp_freq, sfwf_interp_inc
     &      , sfwf_interp_next, sfwf_interp_last, nsteps_run)
            if(nsteps_run /= 0) sfwf_interp_next = 
     &        sfwf_interp_next + sfwf_interp_inc
          endif
     
          SFWF = (SFWF_DATA(:,:,sfwf_data_sss,0) 
     &              - TRACER(:,:,1,2,curtime))*sfwf_restore_rtau*dz(1)
 
        case ('bulk-NCEP')
          call calc_sfwf_bulk_ncep(12,SFWF)

        case ('partially-coupled')
          call calc_sfwf_partially_coupled (12)
 
        end select
        
      case('n-hour')

        sfwf_data_label = 'SFWF n-hour'
        if(thour00 >= sfwf_data_update)
     &    call update_forcing_data(sfwf_data_time
     &    , sfwf_data_time_min_loc, sfwf_interp_type, sfwf_data_next
     &    , sfwf_data_update, sfwf_data_type, sfwf_data_inc
     &    , sfwf_filename, sfwf_data_renorm
     &    , SFWF_DATA(:,:,:,1:sfwf_interp_order)
     &    , sfwf_data_num_fields, sfwf_interp_order, sfwf_data_label)

        select case (sfwf_formulation)
        case ('restoring')            
      
          if(thour00 >= sfwf_interp_next .or. nsteps_run == 0) then
            call interpolate_forcing(SFWF_DATA(:,:,sfwf_data_sss,0)
     &      , SFWF_DATA(:,:,sfwf_data_sss,1:sfwf_interp_order)
     &      , sfwf_data_time ,sfwf_interp_type, sfwf_data_time_min_loc
     &      , sfwf_interp_order, sfwf_interp_freq, sfwf_interp_inc
     &      , sfwf_interp_next, sfwf_interp_last, nsteps_run)
            if(nsteps_run /= 0) sfwf_interp_next = 
     &        sfwf_interp_next + sfwf_interp_inc
          endif
     
          SFWF = (SFWF_DATA(:,:,sfwf_data_sss,0) 
     &              - TRACER(:,:,1,2,curtime))*sfwf_restore_rtau*dz(1)
     
        case ('bulk-NCEP')
          call calc_sfwf_bulk_ncep(sfwf_interp_order,SFWF)

        case ('partially-coupled')
          call calc_sfwf_partially_coupled (sfwf_interp_order)
 
        end select
                 
      end select
      

      end subroutine set_sfwf
!***********************************************************************


      subroutine calc_sfwf_bulk_ncep(time_dim,SFWF)
      
!-----------------------------------------------------------------------
!  Purpose:
!
!       Calculate surface freshwater flux from a combination of
!       air-sea fluxes (precipitation,  evaporation based on 
!       latent heat flux computed in calc_shf_bulk_ncep),
!       and restoring terms (due to restoring fields of SSS).
!
!  Notes:
!       the forcing data (on t-grid) are computed and 
!       stored in SFWF_DATA(:,:,sfwf_comp_*,now) where:
!          sfwf_data_sss    is   restoring SSS             (msu)
!          sfwf_data_precip is   precipitation             (m/y)
!-----------------------------------------------------------------------

      implicit none

      integer (kind=int_kind) :: time_dim, nearest_data, now, k, n, nu

      real (kind=dbl_kind) ::
     &  dttmp    ! temporary time step variable

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   SFWF    !  surface freshwater flux at current timestep
     &  ,WORK

!-----------------------------------------------------------------------
!  sfwf_weak_restore= weak(non-ice) restoring h2o flux per msu  (kg/s/m^2/msu)
!  sfwf_strong_restore= strong  (ice) ..      ..  ..   ..  ..      ..
!
!  to calculate restoring factors, use mixed layer of 50m,
!  and restoring time constant tau (days):
!
!                    F (kg/s/m^2/msu)
!      tau =   6  :     2.77
!      tau =  30  :     0.55
!      tau = 182.5:     0.092
!      tau = 365  :     0.046
!      tau = 730  :     0.023
!      tau = Inf  :     0.0
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!  if annual forcing, no interpolation to current time is necessary.
!-----------------------------------------------------------------------

      if (sfwf_data_type == 'annual') then

        now = 1
        nearest_data = 1

      else

!----------------------------------------------------------------------
!  interpolate all sfwf data to current time and
!  store in SFWF_DATA(:,:,:,now)
!----------------------------------------------------------------------

        now = 0

        if(thour00 >= sfwf_interp_next .or. nsteps_run == 0) then
          do n = 1, sfwf_data_num_fields
            call interpolate_forcing(SFWF_DATA(:,:,n,now)
     &      , SFWF_DATA(:,:,n,1:time_dim),sfwf_data_time
     &      , sfwf_interp_type, sfwf_data_time_min_loc, time_dim
     &      , sfwf_interp_freq, sfwf_interp_inc
     &      , sfwf_interp_next, sfwf_interp_last, nsteps_run)
          enddo
          if(nsteps_run /= 0) sfwf_interp_next =
     &      sfwf_interp_next + sfwf_interp_inc
        endif

      endif

!----------------------------------------------------------------------
!  compute evaporation 
!----------------------------------------------------------------------

      SFWF_COMP(:,:,sfwf_comp_evap) = 
     &       SHF_COMP(:,:,shf_comp_qlat)/latent_heat_vapor

!----------------------------------------------------------------------
!  precipitation (kg/m^2/s)
!----------------------------------------------------------------------

      SFWF_COMP(:,:,sfwf_comp_precip) = 
     &   SFWF_DATA(:,:,sfwf_data_precip,now) 
     &   * precip_fact
!    &   * c1000/seconds_in_year ! convert m/y to Kg/m^2/s if needed

!----------------------------------------------------------------------
!  weak salinity restoring term (note: OCN_WGT = 0. at land points)
!----------------------------------------------------------------------

      SFWF_COMP(:,:,sfwf_comp_wrest) = - sfwf_weak_restore * OCN_WGT
     &  * MASK_SR
     &  * (SFWF_DATA(:,:,sfwf_data_sss,now) - TRACER(:,:,1,2,curtime))

!----------------------------------------------------------------------
! subtract global mean from weak restoring term
!----------------------------------------------------------------------

      fres_hor_ave = 
     &   global_sum(TAREA*SFWF_COMP(:,:,sfwf_comp_wrest), ONE)
      fres_hor_ave_area = 
     &   global_sum(TAREA*OCN_WGT*MASK_SR, ONE)
       
      SFWF_COMP(:,:,sfwf_comp_wrest) = SFWF_COMP(:,:,sfwf_comp_wrest)
     &  -OCN_WGT*MASK_SR*fres_hor_ave/fres_hor_ave_area

!----------------------------------------------------------------------
!  strong salinity restoring term
!----------------------------------------------------------------------

      where(KMT.gt.0)
        SFWF_COMP(:,:,sfwf_comp_srest) = 
     &    - sfwf_strong_restore * (c1 - OCN_WGT)
     &    * (SFWF_DATA(:,:,sfwf_data_sss,now) - TRACER(:,:,1,2,curtime))
      endwhere

      where( KMT > 0 .and. MASK_SR == 0 ) 
        SFWF_COMP(:,:,sfwf_comp_srest) =
     &    - sfwf_strong_restore_ms
     &    * (SFWF_DATA(:,:,sfwf_data_sss,now) - TRACER(:,:,1,2,curtime))
      endwhere
  
!----------------------------------------------------------------------
!
!     if variable thickness surface layer, compute net surface
!     freshwater flux (kg/m^2/s) due to restoring terms only
!     then compute freshwater flux due to P-E and convert to (cm/s)
!     then set the tracer content in the freshwater flux
!       defaults are FW*SST for tracer 1 (temperature)
!                      0 for salinity (really fresh water)
!                      0 for all other tracers
!
!     IF DATA ARE AVAILABLE...
!     IMPLEMENT SUM OVER FRESHWATER TYPES (E,P,R,F,M) HERE:
!
!     TFW(:,:,n) = FW_EVAP*TW_EVAP(:,:,n) + FW_PRECIP*TW_PRECIP(:,:,n) 
!                + FW_ROFF*TW_ROFF(:,:,n) + FW_MELT*TW_MELT(:,:,n) 
!                + FW_FREEZE*TW_FREEZE(:,:,n)
!
!     where, for example  FW_ROFF is the water flux from rivers
!     and TW_ROFF(:,:,n) is the concentration of the nth tracer
!     in the river water;  similarly for water fluxes due to
!     evaporation, precipitation, ice freezing, and ice melting.
!
!----------------------------------------------------------------------

      if (sfc_layer_type == sfc_layer_varthick .and. 
     &    .not. lfw_as_salt_flx) then

        SFWF = SFWF_COMP(:,:,sfwf_comp_wrest)
     &       + SFWF_COMP(:,:,sfwf_comp_srest)

        FW = OCN_WGT * MASK_SR *
     &    ( SFWF_COMP(:,:,sfwf_comp_evap) 
     &    + SFWF_COMP(:,:,sfwf_comp_precip) )*fwmass_to_fwflux

        TFW(:,:,1) = FW*TRACER(:,:,1,1,curtime)
        TFW(:,:,2:nt) = c0

!----------------------------------------------------------------------
!
!     if rigid lid or old free surface form, compute
!     net surface freshwater flux (kg/m^2/s)
!
!----------------------------------------------------------------------

      else

         SFWF = OCN_WGT * MASK_SR *
     &    ( SFWF_COMP(:,:,sfwf_comp_evap) 
     &    + SFWF_COMP(:,:,sfwf_comp_precip) ) 
     &    + SFWF_COMP(:,:,sfwf_comp_wrest)  
     &    + SFWF_COMP(:,:,sfwf_comp_srest) 

      endif

!-----------------------------------------------------------------------
!
!     convert surface freshwater flux (kg/m^2/s) to 
!     salinity flux (msu*cm/s)
!
!-----------------------------------------------------------------------

      SFWF = SFWF*salinity_factor

!-----------------------------------------------------------------------
!  Compute the volume-averaged salinity at the end of the year; note
!  that it is evaluated at the current time level.
!-----------------------------------------------------------------------

      if (eoy) then
        if (sfc_layer_type == sfc_layer_varthick) then
          WORK = merge(TRACER(:,:,1,2,curtime)*TAREA*
     &                 (dz(1) + PSURF(:,:,curtime)/grav),
     &                 c0, KMT > 0 .and. MASK_SR > 0)
          sal_final(1) = (global_sum(WORK, ONE)/volgk_m_marg(1))*
     &                   1.0e-6_dbl_kind  ! convert to m^3
        else
          WORK = merge(TRACER(:,:,1,2,curtime)*TAREA*dz(1),
     &                 c0, KMT > 0 .and. MASK_SR > 0)
          sal_final(1) = (global_sum(WORK, ONE)/volgk_m_marg(1))*
     &                   1.0e-6_dbl_kind  ! convert to m^3
        endif

        do k = 2,km
          WORK =  merge(TRACER(:,:,k,2,curtime)*TAREA*dz(k), c0 
     &                    , k <= KMT .and. MASK_SR > 0)
          sal_final(k) = (global_sum(WORK, ONE)/volgk_m_marg(k))*
     &                   1.0e-6_dbl_kind  ! convert to m^3
        enddo

        if (sfc_layer_type == sfc_layer_varthick) then
          WORK = merge(PSURF(:,:,curtime)*TAREA/grav,
     &                 c0, KMT > 0 .and. MASK_SR > 0)
          ssh_final = global_sum(WORK, ONE)*
     &                1.0e-4_dbl_kind/areag_m_marg/seconds_in_year
          if (my_task == master_task) then
            write(stdout,*) ' '
            write(stdout,*) 'annual change in SSH: ', ssh_final
          endif
          ssh_final = ssh_final*10.0_dbl_kind ! convert (cm/s) -> kg/m^2/s
                                              ! (cm/s)x0.01(m/cm)x1000kg/m^3
        else
          ssh_final = c0
        endif

      endif !  (eoy)

!-----------------------------------------------------------------------
!  accumulate annual-mean precipitation over ocean points that are
!    not marginal seas.
!-----------------------------------------------------------------------

      if (avg_ts .or. back_to_back) then
        dttmp = p5*dtt
      else
        dttmp = dtt
      endif

      WORK = merge(SFWF_COMP(:,:,sfwf_comp_precip)*TAREA*OCN_WGT,
     &             c0, MASK_SR > 0)
      sum_precip = sum_precip + dttmp*
     &             global_sum(WORK,ONE)*1.0e-4_dbl_kind/areag_m_marg

!      WORK = merge(FW_OLD*TAREA, c0, MASK_SR > 0)
!      sum_fw = sum_fw + dttmp*
!     &         global_sum(WORK,ONE)*1.0e-4_dbl_kind/areag_m_marg

!-----------------------------------------------------------------------
!  at end of year find annual mean precip and reset annual counters
!-----------------------------------------------------------------------

      if (eoy) then
!        ann_avg_fw = sum_fw / seconds_in_year
!        if(my_task == master_task) then
!          write(stdout,*) ' '
!          write(stdout,*) 'annual average freshwater flux: ', ann_avg_fw
!        endif
!        sum_fw = c0

        ann_avg_precip = sum_precip / seconds_in_year
        sum_precip = c0
      endif

!?  call flux diagnostics here?

      if (eoy .and. ladjust_precip) then
          call precip_adjustment
      endif

!-----------------------------------------------------------------------
!  copy "sal_final" to "sal_initial"
!-----------------------------------------------------------------------

      if (eoy) then
        sal_initial = sal_final
        ssh_initial = ssh_final
      endif

      end subroutine calc_sfwf_bulk_ncep

!***********************************************************************

      subroutine calc_sfwf_partially_coupled (time_dim)

!-----------------------------------------------------------------------
!  calculate ice-ocean flux, weak restoring, and strong restoring 
!  components of surface freshwater flux for partially-coupled formulation.
!  these components will later be used in
!  set_surface_forcing (forcing.F) to form the total surface freshwater
!  (salt) flux.
!
!  the forcing data (on t-grid) sets needed are 
!     sfwf_data_sss,    restoring SSS                       (msu)
!     sfwf_data_flxio,  diagnosed ("climatological")        (kg/m^2/s)
!                       ice-ocean freshwater flux 
!-----------------------------------------------------------------------

      implicit none

      integer (kind=int_kind) :: time_dim, nearest_data, now, k, n

      real (kind=dbl_kind), dimension(imt,jmt) :: WORK

!-----------------------------------------------------------------------
!  if annual forcing, no interpolation to current time is necessary.
!-----------------------------------------------------------------------

      if (sfwf_data_type == 'annual') then

        now = 1
        nearest_data = 1

      else

!----------------------------------------------------------------------
!  interpolate all sfwf data to current time and
!  store in SFWF_DATA(:,:,:,now)
!----------------------------------------------------------------------

        now = 0

        if(thour00 >= sfwf_interp_next .or. nsteps_run == 0) then
          do n = 1, sfwf_data_num_fields
            call interpolate_forcing(SFWF_DATA(:,:,n,now)
     &      , SFWF_DATA(:,:,n,1:time_dim),sfwf_data_time
     &      , sfwf_interp_type, sfwf_data_time_min_loc, time_dim
     &      , sfwf_interp_freq, sfwf_interp_inc
     &      , sfwf_interp_next, sfwf_interp_last, nsteps_run)
          enddo
          if (nsteps_run /= 0) sfwf_interp_next =
     &                         sfwf_interp_next + sfwf_interp_inc
        endif

      endif

      if (.not. luse_cpl_ifrac) then
        WORK = OCN_WGT*MASK_SR
      else
        WORK = MASK_SR
      endif
 
!----------------------------------------------------------------------
!  weak salinity restoring term (note: MASK_SR = 0. at land and
!  marginal sea points)
!----------------------------------------------------------------------

      SFWF_COMP(:,:,sfwf_comp_wrest) = - sfwf_weak_restore * WORK
     &  * (SFWF_DATA(:,:,sfwf_data_sss,now) - TRACER(:,:,1,2,curtime))

!----------------------------------------------------------------------
! subtract global mean from weak restoring term
!----------------------------------------------------------------------

      fres_hor_ave =
     &   global_sum(TAREA*SFWF_COMP(:,:,sfwf_comp_wrest), ONE)
      fres_hor_ave_area = global_sum(TAREA*WORK, ONE)

      SFWF_COMP(:,:,sfwf_comp_wrest) = SFWF_COMP(:,:,sfwf_comp_wrest)
     &  -WORK*fres_hor_ave/fres_hor_ave_area

!----------------------------------------------------------------------
!  strong salinity restoring term
!----------------------------------------------------------------------

      where(KMT > 0)
        SFWF_COMP(:,:,sfwf_comp_srest) =
     &    - sfwf_strong_restore * (c1 - OCN_WGT)
     &    * (SFWF_DATA(:,:,sfwf_data_sss,now) - TRACER(:,:,1,2,curtime))
      endwhere

      where( KMT > 0 .and. MASK_SR == 0 )
        SFWF_COMP(:,:,sfwf_comp_srest) =
     &    - sfwf_strong_restore_ms
     &    * (SFWF_DATA(:,:,sfwf_data_sss,now) - TRACER(:,:,1,2,curtime))
      endwhere

!----------------------------------------------------------------------
!  ice-ocean climatological flux term
!----------------------------------------------------------------------

      if ( .not. lactive_ice ) then
        where(KMT > 0)
          SFWF_COMP(:,:,sfwf_comp_flxio) = (c1 - OCN_WGT) 
     &                          * SFWF_DATA(:,:,sfwf_data_flxio,now)
        endwhere
        if ( .not. lms_balance ) 
     &    SFWF_COMP(:,:,sfwf_comp_flxio) = 
     &                      SFWF_COMP(:,:,sfwf_comp_flxio) * MASK_SR
      endif

!----------------------------------------------------------------------
!  convert surface freshwater flux components (kg/m^2/s) to
!  salinity flux components (msu*cm/s)
!----------------------------------------------------------------------

      SFWF_COMP(:,:,sfwf_comp_wrest) =  SFWF_COMP(:,:,sfwf_comp_wrest)
     &                                * salinity_factor
      SFWF_COMP(:,:,sfwf_comp_srest) =  SFWF_COMP(:,:,sfwf_comp_srest)
     &                                * salinity_factor


      if ( sfc_layer_type == sfc_layer_varthick .and. 
     &     .not. lfw_as_salt_flx) then
        WORK = SFWF_COMP(:,:,sfwf_comp_flxio) * fwmass_to_fwflux
        SFWF_COMP(:,:,sfwf_comp_flxio) = WORK
        TFW_COMP(:,:,1,tfw_comp_flxio) = WORK 
     &                                  * TMLT(TRACER(:,:,1,2,curtime))
        TFW_COMP(:,:,2:nt,tfw_comp_flxio) = c0
      else
        SFWF_COMP(:,:,sfwf_comp_flxio) = SFWF_COMP(:,:,sfwf_comp_flxio)
     &                                 * salinity_factor
      endif

!-----------------------------------------------------------------------
!  Compute the volume-averaged salinity at the end of the year; note
!  that it is evaluated at the current time level.
!-----------------------------------------------------------------------

      if (eoy) then
        if (sfc_layer_type == sfc_layer_varthick) then
          WORK = merge(TRACER(:,:,1,2,curtime)*TAREA*
     &                 (dz(1) + PSURF(:,:,curtime)/grav),
     &                 c0, KMT > 0 .and. MASK_SR > 0)
          sal_final(1) = (global_sum(WORK, ONE)/volgk_m_marg(1))*
     &                   1.0e-6_dbl_kind  ! convert to m^3
        else
          WORK = merge(TRACER(:,:,1,2,curtime)*TAREA*dz(1),
     &                 c0, KMT > 0 .and. MASK_SR > 0)
          sal_final(1) = (global_sum(WORK, ONE)/volgk_m_marg(1))*
     &                   1.0e-6_dbl_kind  ! convert to m^3
        endif

        do k = 2,km
          WORK =  merge(TRACER(:,:,k,2,curtime)*TAREA*dz(k), c0 
     &                    , k <= KMT .and. MASK_SR > 0)
          sal_final(k) = (global_sum(WORK, ONE)/volgk_m_marg(k))*
     &                   1.0e-6_dbl_kind  ! convert to m^3
        enddo

        if (sfc_layer_type == sfc_layer_varthick) then
          WORK = merge(PSURF(:,:,curtime)*TAREA/grav,
     &                 c0, KMT > 0 .and. MASK_SR > 0)
          ssh_final = global_sum(WORK, ONE)*
     &                1.0e-4_dbl_kind/areag_m_marg/seconds_in_year
          if (my_task == master_task) then
            write(stdout,*) ' '
            write(stdout,*) 'annual change in SSH: ', ssh_final
          endif
          ssh_final = ssh_final*10.0_dbl_kind ! convert (cm/s) -> kg/m^2/s
                                              ! (cm/s)x0.01(m/cm)x1000kg/m^3
        else
          ssh_final = c0
        endif


      endif !  (eoy)

      if (eoy .and. ladjust_precip) then
        call precip_adjustment
      endif

!-----------------------------------------------------------------------
!  copy "sal_final" to "sal_initial"
!-----------------------------------------------------------------------

      if (eoy) then
        sal_initial = sal_final
        ssh_initial = ssh_final
      endif

      end subroutine calc_sfwf_partially_coupled


!***********************************************************************

      subroutine precip_adjustment

      real (kind=dbl_kind) :: sal_tendency, fw_tendency, precip_tav

      integer (kind=int_kind) :: k

!-----------------------------------------------------------------------
!  adjust precipitation factor "precip_fact"
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!  compute tendency of salinity for eack "k" layer, considering the
!  effects of depth acceleration
!-----------------------------------------------------------------------

      do k=1,km
        sal_initial(k) = (sal_final(k) - sal_initial(k))
     &                  /(dttxcel(k)*seconds_in_year)
      enddo

!-----------------------------------------------------------------------
!  form the global volume-averaged tendency to be used in "precip_fact"
!  computation
!-----------------------------------------------------------------------

      sal_tendency = c0
      do k=1,km
        sal_tendency = sal_tendency + volgk_m_marg(k)*sal_initial(k)
      enddo

      sal_tendency = sal_tendency /  volgt_m_marg

      if (my_task == master_task) then
        write (stdout,*)
     &  ' calc_sfwf_bulk_ncep: volume-averaged salinity tendency = ',
     &  sal_tendency
      endif

!-----------------------------------------------------------------------
!  convert "sal_tendency" from (msu/s) to -(kg/m^2/s). note that
!  "areag" and "volgt" are in m^2 and m^3, respectively.
!  assumes density of fresh water = 1000 kg/m**3
!-----------------------------------------------------------------------

      sal_tendency = - sal_tendency * volgt_m_marg
     &               * 1.0e6_dbl_kind / areag_m_marg
     &                 / ocn_ref_salinity

!-----------------------------------------------------------------------
!  compute annual change in mass due to freshwater flux (kg/m^2/s)
!-----------------------------------------------------------------------

      fw_tendency = ssh_final - ssh_initial

      if (my_task == master_task) then
        write (stdout,*) ' precip_adjustment: '
        write (stdout,*) '   sal_tendency (kg/m^2/s): ', sal_tendency
        write (stdout,*) '    fw_tendency (kg/m^2/s): ',  fw_tendency
      endif

!-----------------------------------------------------------------------
!  change "precip_fact" based on tendency of freshwater and previous
!  amount of precipitation
!-----------------------------------------------------------------------

      if ( sfwf_formulation == 'partially-coupled' ) then
        precip_tav = precip_mean
      else
        precip_tav = ann_avg_precip/precip_fact
      endif

      precip_fact = precip_fact 
     &            - (sal_tendency + fw_tendency)/precip_tav

      if (my_task == master_task) then
        write (stdout,4141) precip_fact
      endif
4141  format (1x,' changed precipitation factor to ',e14.8)


      end subroutine precip_adjustment

!***********************************************************************

      end module forcing_sfwf

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
