!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module forcing_shf

!***********************************************************************

!-----------------------------------------------------------------------
!
!     contains routines and variables used for determining the
!       surface heat flux.
!
!     CVS:$Id: forcing_shf.F,v 1.3.2.3 2002/01/25 23:25:00 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use io
      use grid
      use forcing_tools
      use time_management
      use prognostic
      use exit_mod

      implicit none
      save

      real (kind=dbl_kind), allocatable, dimension(:,:,:,:) ::
     &   SHF_DATA

      real (kind=dbl_kind), allocatable, dimension(:,:,:) ::
     &   SHF_COMP

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   OCN_WGT

      integer (kind=int_kind), allocatable, dimension(:,:) ::
     &   MASK_SR

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   SHF_QSW

      real (kind=dbl_kind), dimension(20) :: shf_data_renorm

      real (kind=dbl_kind), dimension(0:km) :: sw_absorb

      real (kind=dbl_kind), dimension(12) :: 
     &   shf_data_time  !  time (in hours) corresponding to surface heat fluxes

      real (kind=dbl_kind) :: 
     &    shf_data_inc    ! time increment between values of forcing data
     &  , shf_data_next   ! time that will be used for the next value of forcing data that is needed
     &  , shf_data_update ! time when the a new forcing value needs to be added to interpolation set
     &  , shf_interp_inc  ! time increment between interpolation
     &  , shf_interp_next ! time when next interpolation will be done
     &  , shf_interp_last ! time when last interpolation was done
     &  , shf_restore_tau
     &  , shf_restore_rtau
     &  , shf_weak_restore      ! heat flux weak restoring coefficient
     &  , shf_strong_restore    ! heat flux strong restoring coefficient
     &  , shf_strong_restore_ms ! heat flux strong restoring coefficient
                                ! for marginal seas

      integer (kind=int_kind) :: 
     &    shf_interp_order  !  order of temporal interpolation
     &  , shf_data_time_min_loc  !  index of the third dimension of shf_data_time containing the minimum forcing time
     &  , shf_data_num_fields
     &  , shf_num_comps

!  the following is necessary for restoring and partially-coupled
      integer (kind=int_kind) :: 
     &    shf_data_sst

!  the following are necessary for Barnier-restoring
      integer (kind=int_kind) :: 
     &    shf_data_tstar
     &  , shf_data_tau
     &  , shf_data_ice
     &  , shf_data_qsw

!  the following are necessary for bulk-NCEP. shf_comp_wrest and
!  shf_comp_srest are also used for partially-coupled.
      integer (kind=int_kind) :: 
     &    shf_data_tair
     &  , shf_data_qair
     &  , shf_data_cldfrac
     &  , shf_data_windspd
     &  , shf_comp_qsw
     &  , shf_comp_qlw
     &  , shf_comp_qlat
     &  , shf_comp_qsens
     &  , shf_comp_wrest
     &  , shf_comp_srest

      character (char_len) :: 
     &    shf_data_type  !  period keyword for the period of the forcing data
     &  , shf_interp_freq !  period keyword for the period of the temporal interpolation
     &  , shf_filename
     &  , shf_interp_type
     &  , shf_data_label
     &  , shf_formulation

!-----------------------------------------------------------------------
!  the following is necessary for penetrative shortwave used with
!  Barnier-restoring and bulk-NCEP
!-----------------------------------------------------------------------
      integer (kind=int_kind) ::
     &  jerlov_water_type

!-----------------------------------------------------------------------
!  the following are needed for long-wave heat flux
!  with bulk-NCEP forcing
!-----------------------------------------------------------------------

      real (kind=dbl_kind), allocatable, dimension (:,:) :: CCINT
      real (kind=dbl_kind), allocatable, dimension (:) :: DEL_LAT
      real (kind=dbl_kind), dimension(21) ::
     &  cc = ( / 0.88_dbl_kind,0.84_dbl_kind,0.80_dbl_kind
     &          ,0.76_dbl_kind,0.72_dbl_kind,0.68_dbl_kind
     &          ,0.63_dbl_kind,0.59_dbl_kind,0.52_dbl_kind
     &          ,0.50_dbl_kind,0.50_dbl_kind,0.50_dbl_kind
     &          ,0.52_dbl_kind,0.59_dbl_kind,0.63_dbl_kind
     &          ,0.68_dbl_kind,0.72_dbl_kind,0.76_dbl_kind
     &          ,0.80_dbl_kind,0.84_dbl_kind,0.88_dbl_kind / )
     & ,clat = ( / -90.0_dbl_kind,-80.0_dbl_kind,-70.0_dbl_kind
     &            ,-60.0_dbl_kind,-50.0_dbl_kind,-40.0_dbl_kind
     &            ,-30.0_dbl_kind,-20.0_dbl_kind,-10.0_dbl_kind
     &            , -5.0_dbl_kind, 0.00_dbl_kind,  5.0_dbl_kind
     &            , 10.0_dbl_kind, 20.0_dbl_kind, 30.0_dbl_kind
     &            , 40.0_dbl_kind, 50.0_dbl_kind, 60.0_dbl_kind
     &            , 70.0_dbl_kind, 80.0_dbl_kind, 90.0_dbl_kind / )
      integer (kind=int_kind) :: i, j

!-----------------------------------------------------------------------
!  the following are needed for weak and strong restoring components
!  of surface heat and freshwater fluxes.
!-----------------------------------------------------------------------

      real (kind=dbl_kind), parameter ::
     &  T_strong_restore_limit = -1.8_dbl_kind
     & ,T_weak_restore_limit   = -0.8_dbl_kind
     & ,dT_restore_limit = T_weak_restore_limit - T_strong_restore_limit

!-----------------------------------------------------------------------
!  the following is necessary for partially-coupled. 
!     luse_cpl_ifrac  = .T.     use fractional ice coverage
!                               sent by the coupler from the (dummy) ice,
!                       .F.     use fractional ice coverage based on the
!                               STR SST climatology.
!-----------------------------------------------------------------------

      logical (kind=log_kind) :: luse_cpl_ifrac 
      integer (kind=int_kind) :: shf_comp_cpl


!***********************************************************************

      contains

!***********************************************************************

!***********************************************************************

      subroutine init_shf(SHF)
      
!-----------------------------------------------------------------------
!
!     initialize surface heat flux forcing by either calculating 
!       or reading in the surface heat flux.  also do initial 
!       book-keeping concerning when new data is needed for the temporal 
!       interpolation and when the forcing will need to be updated.
!
!-----------------------------------------------------------------------

      implicit none

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   SHF    !  surface heat flux at current timestep

      integer (kind=int_kind) :: k, n, nu
     &, nml_error               ! namelist error flag
     &, allocate_error_status   ! allocate error flag

      character (char_len) :: forcing_filename
                      
      namelist /forcing_shf_nml/ shf_data_type, shf_data_inc
     &         ,shf_interp_type, shf_interp_freq
     &         ,shf_interp_inc, shf_restore_tau
     &         ,shf_filename, shf_data_renorm, shf_formulation
     &         ,jerlov_water_type
     &         ,shf_weak_restore, shf_strong_restore
     &         ,luse_cpl_ifrac, shf_strong_restore_ms

!-----------------------------------------------------------------------
!
!     read surface heat flux namelist input after setting default values.
!
!-----------------------------------------------------------------------

      shf_formulation      = 'restoring'
      shf_data_type        = 'analytic'
      shf_data_inc         = 1.e20_dbl_kind
      shf_interp_type      = 'nearest'
      shf_interp_freq      = 'never'
      shf_interp_inc       = 1.e20_dbl_kind
      shf_restore_tau      = 1.e20_dbl_kind
      shf_filename         = 'unknown-shf'
      shf_data_renorm      = c1
      jerlov_water_type    = 3
      shf_weak_restore     = c0
      shf_strong_restore   = 92.64_dbl_kind 
      luse_cpl_ifrac       = .false.
      shf_strong_restore_ms= 92.64_dbl_kind

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in,nml=forcing_shf_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading forcing_shf_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, forcing_shf_nml)
          write(stdout,*) ' '
      endif

      call broadcast_scalar(shf_formulation,       master_task)
      call broadcast_scalar(shf_data_type,         master_task)
      call broadcast_scalar(shf_data_inc,          master_task)
      call broadcast_scalar(shf_interp_type,       master_task)
      call broadcast_scalar(shf_interp_freq,       master_task)
      call broadcast_scalar(shf_interp_inc,        master_task)
      call broadcast_scalar(shf_restore_tau,       master_task)
      call broadcast_scalar(shf_filename,          master_task)
      call broadcast_scalar(jerlov_water_type,     master_task)
      call broadcast_vector(shf_data_renorm,       master_task,20)
      call broadcast_scalar(shf_weak_restore,      master_task)
      call broadcast_scalar(shf_strong_restore,    master_task)
      call broadcast_scalar(shf_strong_restore_ms, master_task)
      call broadcast_scalar(luse_cpl_ifrac ,       master_task)
      
!-----------------------------------------------------------------------
!     convert data_type to 'monthly-calendar' if input is 'monthly'
!-----------------------------------------------------------------------

      if (shf_data_type == 'monthly') shf_data_type = 'monthly-calendar'

!-----------------------------------------------------------------------
!     set values based on shf_formulation
!-----------------------------------------------------------------------
      
      select case (shf_formulation)
      case ('restoring')
        shf_data_num_fields = 1
        shf_data_sst = 1
        
      case ('Barnier-restoring')
        shf_data_num_fields = 4
        shf_data_tstar = 1
        shf_data_tau   = 2
        shf_data_ice   = 3
        shf_data_qsw   = 4
        
      case ('bulk-NCEP')
        shf_data_num_fields = 6
        shf_data_sst  = 1
        shf_data_tair  = 2
        shf_data_qair  = 3
        shf_data_qsw  = 4
        shf_data_cldfrac  = 5
        shf_data_windspd  = 6

        shf_num_comps = 6
        shf_comp_qsw  = 1
        shf_comp_qlw  = 2
        shf_comp_qlat  = 3
        shf_comp_qsens  = 4
        shf_comp_wrest  = 5
        shf_comp_srest  = 6
        
!---------------------------------------------------------------------
!  initialize CCINT (cloud factor used in long-wave heat flux
!  with bulk-NCEP forcing).
!---------------------------------------------------------------------

        allocate( CCINT(imt,jmt), DEL_LAT(imt)
     &            , STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,     exit_message = 'forcing_shf.F -- CCINT')
        do j=1,jmt
           do i=1,20
              where ( (TLAT(:,j)*radian > clat(i))
     &           .and. (TLAT(:,j)*radian <= clat(i+1)) )
                 DEL_LAT = 
     &             (TLAT(:,j)*radian-clat(i))/(clat(i+1)-clat(i))
                 CCINT(:,j) = cc(i)+DEL_LAT*(cc(i+1)-cc(i))
              endwhere
           enddo ! i
        enddo ! j

      case ('partially-coupled')
        shf_data_num_fields = 1
        shf_data_sst = 1

        shf_num_comps = 3 
        shf_comp_wrest = 1
        shf_comp_srest = 2  
        shf_comp_cpl   = 3
        
      case default
        call exit_POP('init_shf: Unknown value for shf_formulation')
        
      end select
 
!-----------------------------------------------------------------------
!     calculate inverse of restoring time scale and convert to seconds.
!-----------------------------------------------------------------------

      shf_restore_tau  = seconds_in_day*shf_restore_tau
      shf_restore_rtau = c1/shf_restore_tau

!-----------------------------------------------------------------------
!     define shortwave solar absorption model.
!     also, initialize SHF_QSW in case a value is needed but not
!       supplied by data:  for example, with KPP and restoring.
!-----------------------------------------------------------------------

      sw_absorb(0) = c1
      sw_absorb(km) = c0
      do k = 1, km-1
        call sw_absorb_frac(zw(k),jerlov_water_type,sw_absorb(k))
      enddo

      SHF_QSW = c0


!-----------------------------------------------------------------------
!    set strong restoring mask to 0 only at ocean points that are 
!      marginal seas and land
!-----------------------------------------------------------------------

      if (allocated(REGION_MASK)) then
        allocate( MASK_SR(imt,jmt))
        MASK_SR = merge(0, 1, REGION_MASK <= 0)
      endif

!-----------------------------------------------------------------------
!     convert interp_type to corresponding integer value.
!-----------------------------------------------------------------------

      select case (shf_interp_type)
      case ('nearest')
        shf_interp_order = 1
        
      case ('linear')
        shf_interp_order = 2
        
      case ('4point')
        shf_interp_order = 4
        
      case default
        call exit_POP('init_shf: Unknown value for shf_interp_type') 
        
      end select
             
!-----------------------------------------------------------------------
!
!     set values of the surface heat flux arrays (STF or SHF_DATA)
!       depending on the type of the surface heat flux data.
!
!-----------------------------------------------------------------------

      select case (shf_data_type)
      
!-----------------------------------------------------------------------
!     no surface heat flux, therefore no interpolation in time is needed 
!       (shf_interp_freq = 'none'), nor are there any new values 
!       to be used (shf_data_next = shf_data_update = never).
!-----------------------------------------------------------------------      
      case ('none')
      
        SHF = c0
        shf_data_next = never
        shf_data_update = never
        shf_interp_freq = 'never'

!-----------------------------------------------------------------------
!     simple analytic surface temperature that is constant in time, 
!       therefore no new values will be needed (shf_data_next = 
!       shf_data_update = never).
!-----------------------------------------------------------------------
      case ('analytic')
      
        allocate( SHF_DATA(imt,jmt,shf_data_num_fields,1)
     &          , STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,     exit_message = 'forcing_shf.F -- SHF_DATA')
        SHF_DATA = c0

        select case (shf_formulation)
        case ('restoring')
          SHF_DATA(:,:,shf_data_sst,1) = 28.0_dbl_kind*(c1 - sin(ULAT))
        
        end select
        
        shf_data_next = never
        shf_data_update = never
        shf_interp_freq = 'never'
        
!-----------------------------------------------------------------------
!     annual mean climatological surface temperature (read in from file)
!       that is constant in time, therefore no new values will be needed
!       (shf_data_next = shf_data_update = never).
!-----------------------------------------------------------------------
      case ('annual')
      
        allocate( SHF_DATA(imt,jmt,shf_data_num_fields,1)
     &          , STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,     exit_message = 'forcing_shf.F -- SHF_DATA')
        SHF_DATA = c0
      
        forcing_filename = shf_filename
        call open_parallel_file(nu,forcing_filename,recl_dbl)
        
        select case (shf_formulation)
        case ('restoring')
          call read_array(nu,SHF_DATA(:,:,shf_data_sst,1))
          
        case ('Barnier-restoring')
          call read_array(nu,SHF_DATA(:,:,shf_data_tstar,1))
          call read_array(nu,SHF_DATA(:,:,shf_data_tau  ,1))
          call read_array(nu,SHF_DATA(:,:,shf_data_ice  ,1))
          call read_array(nu,SHF_DATA(:,:,shf_data_qsw  ,1))
          SHF_DATA(:,:,shf_data_tau  ,1) =
     &       seconds_in_day*SHF_DATA(:,:,shf_data_tau  ,1)

        case ('bulk-NCEP')
          call read_array(nu,SHF_DATA(:,:,shf_data_sst,1))
          call read_array(nu,SHF_DATA(:,:,shf_data_tair,1))
          call read_array(nu,SHF_DATA(:,:,shf_data_qair,1))
          call read_array(nu,SHF_DATA(:,:,shf_data_qsw,1))
          call read_array(nu,SHF_DATA(:,:,shf_data_cldfrac,1))
          call read_array(nu,SHF_DATA(:,:,shf_data_windspd,1))

          allocate( SHF_COMP(imt,jmt,shf_num_comps)
     &       , STAT = allocate_error_status )
          call check_allocate (allocate_error_status
     &,       exit_message = 'forcing_shf.F -- SHF_COMP')
          SHF_COMP = c0   ! initialize

        case ('partially-coupled')
          call read_array(nu,SHF_DATA(:,:,shf_data_sst,1))

          allocate( SHF_COMP(imt,jmt,shf_num_comps)
     &       , STAT = allocate_error_status )
          call check_allocate (allocate_error_status
     &,       exit_message = 'forcing_shf.F -- SHF_COMP')
          SHF_COMP = c0   ! initialize

        end select

        call close_parallel_file(nu)

!-----------------------------------------------------------------------
!     renormalize values if necessary to compensate for different units.
!-----------------------------------------------------------------------
        do n = 1,shf_data_num_fields
          if(shf_data_renorm(n) /= c1) SHF_DATA(:,:,n,:) 
     &      = SHF_DATA(:,:,n,:)*shf_data_renorm(n)
        enddo
        
        shf_data_next = never
        shf_data_update = never
        shf_interp_freq = 'never'
      
        if(my_task.eq.master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' SHF Annual file read: ', forcing_filename
        endif
        
!-----------------------------------------------------------------------
!     monthly mean climatological surface heat flux. all 
!       12 months are read in from a file. interpolation order 
!       (shf_interp_order) may be specified with namelist input.
!-----------------------------------------------------------------------
      case ('monthly-equal','monthly-calendar')
      
        allocate( SHF_DATA(imt,jmt,shf_data_num_fields,0:12)
     &          , STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,     exit_message = 'forcing_shf.F -- SHF_DATA')
        SHF_DATA = c0

        call find_forcing_times(shf_data_time, shf_data_inc
     &          , shf_interp_type, shf_data_next
     &          , shf_data_time_min_loc, shf_data_update, shf_data_type)
        
        forcing_filename = shf_filename
        call open_parallel_file(nu,forcing_filename,recl_dbl)

        select case (shf_formulation)
        case ('restoring')
          call read_array(nu,SHF_DATA(:,:,shf_data_sst,1:12))
          
        case ('Barnier-restoring')
          call read_array(nu,SHF_DATA(:,:,shf_data_tstar,1:12))
          call read_array(nu,SHF_DATA(:,:,shf_data_tau  ,1:12))
          call read_array(nu,SHF_DATA(:,:,shf_data_ice  ,1:12))
          call read_array(nu,SHF_DATA(:,:,shf_data_qsw  ,1:12))
          SHF_DATA(:,:,shf_data_tau  ,1:12) =
     &       seconds_in_day*SHF_DATA(:,:,shf_data_tau  ,1:12)

        case ('bulk-NCEP')
          call read_array(nu,SHF_DATA(:,:,shf_data_sst,1:12))
          call read_array(nu,SHF_DATA(:,:,shf_data_tair,1:12))
          call read_array(nu,SHF_DATA(:,:,shf_data_qair,1:12))
          call read_array(nu,SHF_DATA(:,:,shf_data_qsw,1:12))
          call read_array(nu,SHF_DATA(:,:,shf_data_cldfrac,1:12))
          call read_array(nu,SHF_DATA(:,:,shf_data_windspd,1:12))

          allocate( SHF_COMP(imt,jmt,shf_num_comps)
     &         , STAT = allocate_error_status )
          call check_allocate (allocate_error_status
     &,       exit_message = 'forcing_shf.F -- SHF_COMP')
          SHF_COMP = c0   ! initialize

        case ('partially-coupled')
          call read_array(nu,SHF_DATA(:,:,shf_data_sst,1:12))

          allocate( SHF_COMP(imt,jmt,shf_num_comps)
     &         , STAT = allocate_error_status )
          call check_allocate (allocate_error_status
     &,       exit_message = 'forcing_shf.F -- SHF_COMP')
          SHF_COMP = c0   ! initialize

        end select

        call close_parallel_file(nu)

!-----------------------------------------------------------------------
!     renormalize values if necessary to compensate for different units.
!-----------------------------------------------------------------------
        do n = 1,shf_data_num_fields
          if(shf_data_renorm(n) /= c1) SHF_DATA(:,:,n,:) 
     &      = SHF_DATA(:,:,n,:)*shf_data_renorm(n)
        enddo
        
        if(my_task.eq.master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' SHF Monthly file read: ', forcing_filename
        endif 
               
!-----------------------------------------------------------------------
!     surface temperature specified every n-hours, where the n-hour 
!       increment should be specified with namelist input  
!       (shf_data_inc). only as many times as are necessary based on 
!       the order of the temporal interpolation scheme 
!       (shf_interp_order) reside in memory at any given time.
!-----------------------------------------------------------------------                   
      case ('n-hour')
              
        allocate( 
     &    SHF_DATA(imt,jmt,shf_data_num_fields,0:shf_interp_order)
     &   ,STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,    exit_message = 'forcing_shf.F -- SHF_DATA')
        SHF_DATA = c0

        call find_forcing_times(shf_data_time, shf_data_inc
     &      , shf_interp_type, shf_data_next
     &      , shf_data_time_min_loc, shf_data_update, shf_data_type)
        
        do n = 1, shf_interp_order
          call get_forcing_filename(forcing_filename,shf_filename
     &                             ,shf_data_time(n),shf_data_inc)

          call open_parallel_file(nu,forcing_filename,recl_dbl)

          select case (shf_formulation)
          case ('restoring')
            call read_array(nu,SHF_DATA(:,:,:,n))
          
          case ('Barnier-restoring')
            call read_array(nu,SHF_DATA(:,:,shf_data_tstar,n))
            call read_array(nu,SHF_DATA(:,:,shf_data_tau  ,n))
            call read_array(nu,SHF_DATA(:,:,shf_data_ice  ,n))
            call read_array(nu,SHF_DATA(:,:,shf_data_qsw  ,n))
            SHF_DATA(:,:,shf_data_tau  ,n) =
     &         seconds_in_day*SHF_DATA(:,:,shf_data_tau  ,n)
  
          case ('bulk-NCEP')
            call read_array(nu,SHF_DATA(:,:,shf_data_sst,n))
            call read_array(nu,SHF_DATA(:,:,shf_data_tair,n))
            call read_array(nu,SHF_DATA(:,:,shf_data_qair,n))
            call read_array(nu,SHF_DATA(:,:,shf_data_qsw,n))
            call read_array(nu,SHF_DATA(:,:,shf_data_cldfrac,n))
            call read_array(nu,SHF_DATA(:,:,shf_data_windspd,n))

          case ('partially-coupled')
            call read_array(nu,SHF_DATA(:,:,:,n))

          end select

          call close_parallel_file(nu)

          if(my_task.eq.master_task) then
            write(stdout,*) ' '
            write(stdout,*) ' SHF n-hour file read: ', forcing_filename
          endif
        enddo
               
        if( shf_formulation == 'bulk-NCEP' .or.
     &      shf_formulation == 'partially-coupled' ) then
            allocate( SHF_COMP(imt,jmt,shf_num_comps)
     &          , STAT = allocate_error_status )
            call check_allocate (allocate_error_status
     &,        exit_message = 'forcing_shf.F -- SHF_COMP')
          SHF_COMP = c0   ! initialize
        endif

!-----------------------------------------------------------------------
!     renormalize values if necessary to compensate for different units.
!-----------------------------------------------------------------------
        do n = 1,shf_data_num_fields
          if(shf_data_renorm(n) /= c1) SHF_DATA(:,:,n,:) 
     &      = SHF_DATA(:,:,n,:)*shf_data_renorm(n)
        enddo
        
      case default

        call exit_POP('init_shf: Unknown value for shf_data_type') 
        
      end select

!-----------------------------------------------------------------------
!
!  now check interpolation period (shf_interp_freq) to set the
!    time for the next temporal interpolation (shf_interp_next).
!
!  if no interpolation is to be done, set next interpolation time 
!    to a large number so the surface heat flux update test  
!    in routine set_surface_forcing will always be false.
!
!  if interpolation is to be done every n-hours, find the first 
!    interpolation time greater than the current time.
!
!  if interpolation is to be done every timestep, set next interpolation 
!    time to a large negative number so the surface heat flux 
!    update test in routine set_surface_forcing will always be true.
!
!-----------------------------------------------------------------------

      select case (shf_interp_freq)
      
      case ('never')      
        shf_interp_next = never
        shf_interp_last = never
        shf_interp_inc  = c0
        
      case ('n-hour')      
        call find_interp_time(shf_interp_inc, shf_interp_next)

      case ('every-timestep')      
        shf_interp_next = always
        shf_interp_inc  = c0
        
      case default
        call exit_POP('init_shf: Unknown value for shf_interp_freq')

      end select

      if(nsteps_total == 0) shf_interp_last = thour00

!-----------------------------------------------------------------------
!  echo forcing options to stdout.
!-----------------------------------------------------------------------

      shf_data_label = 'Surface Heat Flux'
      call echo_forcing_options(shf_data_type
     &   , shf_formulation, shf_data_inc, shf_interp_freq
     &   , shf_interp_type, shf_interp_inc, shf_data_label)

!-----------------------------------------------------------------------

      end subroutine init_shf

!***********************************************************************

      subroutine set_shf(SHF)

!-----------------------------------------------------------------------
!
!     update the current value of the surface heat flux array 
!       (shf) by interpolating to the current time.  it may also be 
!       necessary to use new data values than were used in the 
!       previous interpolation.
!       
!-----------------------------------------------------------------------

      implicit none

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   SHF    !  surface heat flux at current timestep
                      
      integer (kind=int_kind) :: nu, first, second, third, fourth
     &  , shf_data_time_max_loc
      
!-----------------------------------------------------------------------
!     check if new data is necessary for interpolation.  if yes, then
!       shuffle indices in SHF_DATA and shf_data_time arrays  
!       and read in new data if necessary ('n-hour' case).  also 
!       increment values of shf_data_time_min_loc, shf_data_next 
!       and shf_data_update.then do the temporal interpolation. note 
!       that no new data is necessary for 'analytic' and 'annual' cases.
!-----------------------------------------------------------------------

      select case(shf_data_type)
        
      case('analytic')
      
        select case (shf_formulation)
        case ('restoring')
          SHF =  (SHF_DATA(:,:,shf_data_sst,1) - 
     &            TRACER(:,:,1,1,curtime))*shf_restore_rtau*dz(1)
          
        end select
 
      case('annual')

        select case (shf_formulation)
        case ('restoring')
          SHF =  (SHF_DATA(:,:,shf_data_sst,1) - 
     &            TRACER(:,:,1,1,curtime))*shf_restore_rtau*dz(1)
          
        case ('Barnier-restoring')
          call calc_shf_barnier_restoring(1,SHF)

        case ('bulk-NCEP')
          call calc_shf_bulk_ncep(1,SHF)

        case ('partially-coupled')
          call calc_shf_partially_coupled (1)

        end select
 
      case('monthly-equal','monthly-calendar')
      
        shf_data_label = 'SHF Monthly'
        if(thour00 >= shf_data_update)              
     &    call update_forcing_data(shf_data_time, shf_data_time_min_loc
     &    , shf_interp_type, shf_data_next, shf_data_update
     &    , shf_data_type, shf_data_inc, shf_filename
     &    , shf_data_renorm, SHF_DATA(:,:,:,1:12)
     &    , shf_data_num_fields,12,shf_data_label)

        select case (shf_formulation)
        case ('restoring')

          if(thour00 >= shf_interp_next .or. nsteps_run == 0) then
            call interpolate_forcing(SHF_DATA(:,:,shf_data_sst,0)
     &      , SHF_DATA(:,:,shf_data_sst,1:12), shf_data_time 
     &      , shf_interp_type, shf_data_time_min_loc, 12
     &      , shf_interp_freq, shf_interp_inc
     &      , shf_interp_next, shf_interp_last, nsteps_run)
            if(nsteps_run /= 0) shf_interp_next = 
     &        shf_interp_next + shf_interp_inc
          endif

          SHF = (SHF_DATA(:,:,shf_data_sst,0) - TRACER(:,:,1,1,curtime))
     &            *shf_restore_rtau*dz(1)
     
        case ('Barnier-restoring')
          call calc_shf_barnier_restoring(12,SHF)

        case ('bulk-NCEP')
          call calc_shf_bulk_ncep(12,SHF)

        case ('partially-coupled')
          call calc_shf_partially_coupled (12)

        end select
            
      case('n-hour')
      
        shf_data_label = 'SHF n-hour'
        if(thour00 >= shf_data_update)               
     &    call update_forcing_data(shf_data_time, shf_data_time_min_loc
     &    , shf_interp_type, shf_data_next, shf_data_update
     &    , shf_data_type, shf_data_inc, shf_filename
     &    , shf_data_renorm, SHF_DATA(:,:,:,1:shf_interp_order)
     &    , shf_data_num_fields, shf_interp_order, shf_data_label)

        select case (shf_formulation)
        case ('restoring')

          if(thour00 >= shf_interp_next .or. nsteps_run == 0) then
            call interpolate_forcing(SHF_DATA(:,:,shf_data_sst,0)
     &      , SHF_DATA(:,:,shf_data_sst,1:shf_interp_order)
     &      , shf_data_time, shf_interp_type, shf_data_time_min_loc
     &      , shf_interp_order, shf_interp_freq, shf_interp_inc
     &      , shf_interp_next, shf_interp_last, nsteps_run)
            if(nsteps_run /= 0) shf_interp_next = 
     &        shf_interp_next + shf_interp_inc
          endif

          SHF = (SHF_DATA(:,:,shf_data_sst,0) - TRACER(:,:,1,1,curtime))
     &            *shf_restore_rtau*dz(1)
     
        case ('Barnier-restoring')
          call calc_shf_barnier_restoring(shf_interp_order,SHF)

        case ('bulk-NCEP')
          call calc_shf_bulk_ncep(shf_interp_order,SHF)

        case ('partially-coupled')
          call calc_shf_partially_coupled (shf_interp_order)

        end select
                
      end select    ! shf_data_type

      end subroutine set_shf
!***********************************************************************

      subroutine calc_shf_barnier_restoring(time_dim,SHF)
      
!-----------------------------------------------------------------------
!
!
!-----------------------------------------------------------------------

      implicit none

      integer (kind=int_kind) :: time_dim, n, nearest_data, now

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   SHF    !  surface heat flux at current timestep

      real (kind=dbl_kind) :: tcheck, ice_cutoff, ice_restore_temp

!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
      ice_cutoff = 0.9_dbl_kind
      ice_restore_temp = -2.0_dbl_kind

!-----------------------------------------------------------------------
!  if annual forcing, no interpolation to current time is necessary.
!-----------------------------------------------------------------------
      if (shf_data_type == 'annual') then
        now = 1
        nearest_data = 1
        
      else
!-----------------------------------------------------------------------
!  interpolate all shf data EXCEPT ice (n=3) to current time and 
!    store in SHF_DATA(:,:,:,0) if necessary.
!-----------------------------------------------------------------------

        now = 0

        if(thour00 >= shf_interp_next .or. nsteps_run == 0) then

          do n = 1, shf_data_num_fields
            if(n /= 3) call interpolate_forcing(SHF_DATA(:,:,n,now)
     &      , SHF_DATA(:,:,n,1:time_dim), shf_data_time, shf_interp_type
     &      , shf_data_time_min_loc, time_dim
     &      , shf_interp_freq, shf_interp_inc
     &      , shf_interp_next, shf_interp_last, nsteps_run)
          enddo
          if(nsteps_run /= 0) shf_interp_next = 
     &      shf_interp_next + shf_interp_inc

        endif

!-----------------------------------------------------------------------
!  find nearest data time and use it for determining the ice mask.
!    NOTE:  this is for backward compatibility.  perhaps interpolating
!    and using a cut-off of .45 would be acceptable (and easier).
!-----------------------------------------------------------------------

        tcheck = (shf_data_update - thour00)/shf_data_inc

        select case(shf_interp_type)

        case('nearest')
          nearest_data = shf_data_time_min_loc

        case('linear')
          if(tcheck > 0.5)  then
            nearest_data = shf_data_time_min_loc
          else
            nearest_data = shf_data_time_min_loc + 1
          endif

        case('4point')
          if(tcheck > 0.5)  then
            nearest_data = shf_data_time_min_loc + 1
          else
            nearest_data = shf_data_time_min_loc + 2
          endif

        end select

        if ((nearest_data - time_dim) > 0 ) 
     &       nearest_data = nearest_data - time_dim

      endif

!-----------------------------------------------------------------------
!  check for ice concentration >= ice_cutoff in the nearest month. 
!      if there is ice, set TAU to be constant and set TSTARR to 
!      ice_restore_temp.
!-----------------------------------------------------------------------

      where(SHF_DATA(:,:,shf_data_ice,nearest_data) >= ice_cutoff)
         SHF_DATA(:,:,shf_data_tau,now)   = shf_restore_tau
         SHF_DATA(:,:,shf_data_tstar,now) = ice_restore_temp
      endwhere

!-----------------------------------------------------------------------
!  apply restoring only where TAU is defined.
!-----------------------------------------------------------------------

      SHF = c0
      where (SHF_DATA(:,:,shf_data_tau,now) > c0)
        SHF = (SHF_DATA(:,:,shf_data_tstar,now)-TRACER(:,:,1,1,curtime))
     &            *dz(1)/SHF_DATA(:,:,shf_data_tau,now)
      endwhere

!-----------------------------------------------------------------------
!  copy penetrative shortwave into its own array (SHF_QSW) and convert
!    to T flux from W/m^2.
!-----------------------------------------------------------------------

      SHF_QSW = SHF_DATA(:,:,shf_data_qsw,now) * hflux_factor

      end subroutine calc_shf_barnier_restoring

!***********************************************************************

!***********************************************************************

      subroutine calc_shf_bulk_ncep(time_dim,SHF)
      
!-----------------------------------------------------------------------
!  Purpose:
!
!       Calculate surface heat flux from a combination of
!       air-sea fluxes (based on air temperature, specific humidity,
!       solar short wave flux, cloud fraction, and windspeed)
!       and restoring terms (due to restoring fields of SST).
!
!  Notes:
!       the forcing data (on t-grid)
!       are computed as SHF_DATA(:,:,shf_comp_*,now) where:
!
!       shf_data_sst,       restoring SST                       (C)
!       shf_data_tair,      surface air temp. at tair_height    (K)
!       shf_data_qair,      specific humidity at qair_height    (kg/kg)
!       shf_data_qsw,       surface short wave flux   (W/m2)
!       shf_data_cldfrac,   cloud fraction            (0.-1.)
!       shf_data_windspd ,  windspeed at height windspd_height (m/s)
!-----------------------------------------------------------------------

      implicit none

      integer (kind=int_kind) :: time_dim, n, nearest_data, now, nu, k

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   SHF    !  surface heat flux at current timestep
     &  ,RTEA   !  work array
     &  ,FRAC_CLOUD_COVER   !  fractional cloud cover

      real (kind=dbl_kind), parameter :: 
     &  windspd_height = 10.0_dbl_kind
     & ,tair_height    =  2.0_dbl_kind
     & ,qair_height    =  2.0_dbl_kind
     & ,qair_mod_fact  =  0.94_dbl_kind  ! factor to modify humidity
     & ,sw_mod_fact    =  0.875_dbl_kind  ! factor to modify short-wave flux
     & ,sw_mod_albedo  =  0.93_dbl_kind  ! factor to modify albedo

!-----------------------------------------------------------------------
!      shf_weak_restore= weak(non-ice) restoring heatflux per degree (W/m2/C)
!      shf_strong_restore= strong  (ice) ..        ..  ..   ..  ..      ..
!
!      to calculate restoring factors, use mixed layer of 50m,
!      and restoring time constant tau (days):
!
!                    Q (W/m2/C)
!      tau =   6  :  386.0
!      tau =  30  :   77.2
!      tau = 182.5:   12.0
!      tau = 365  :    6.0
!      tau = 730  :    3.0
!      tau = Inf  :    0.0
!---------------------------------------------------------------------

!-----------------------------------------------------------------------
!  if annual forcing, no interpolation to current time is necessary.
!-----------------------------------------------------------------------

      if (shf_data_type == 'annual') then

        now = 1
        nearest_data = 1

      else

!----------------------------------------------------------------------
!  interpolate all shf data to current time and
!  store in SHF_DATA(:,:,:,now)
!----------------------------------------------------------------------

        now = 0

        if(thour00 >= shf_interp_next .or. nsteps_run == 0) then
          do n = 1, shf_data_num_fields
            call interpolate_forcing(SHF_DATA(:,:,n,now)
     &      , SHF_DATA(:,:,n,1:time_dim), shf_data_time, shf_interp_type
     &      , shf_data_time_min_loc, time_dim
     &      , shf_interp_freq, shf_interp_inc
     &      , shf_interp_next, shf_interp_last, nsteps_run)
          enddo
          if(nsteps_run /= 0) shf_interp_next = 
     &      shf_interp_next + shf_interp_inc
        endif

      endif


!----------------------------------------------------------------------
!  compute ocean weights (fraction of ocean vs. ice) every timestep
!----------------------------------------------------------------------

      call ocean_weights (now)

!----------------------------------------------------------------------
!  compute sensible and latent heat fluxes
!----------------------------------------------------------------------

      call sen_lat_flux(SHF_DATA(:,:,shf_data_windspd,now)
     &,                 windspd_height
     &,                 TRACER(:,:,1,1,curtime)
     &,                 SHF_DATA(:,:,shf_data_tair,now)
     &,                 tair_height
     &,                 SHF_DATA(:,:,shf_data_qair,now)
     &,                 qair_height, T0_Kelvin
     &,                 SHF_COMP(:,:,shf_comp_qsens)
     &,                 SHF_COMP(:,:,shf_comp_qlat))

!----------------------------------------------------------------------
!  compute short wave and long wave fluxes
!----------------------------------------------------------------------

      SHF_COMP(:,:,shf_comp_qsw) = sw_mod_albedo * sw_mod_fact *
     &                  SHF_DATA(:,:,shf_data_qsw,now)

      FRAC_CLOUD_COVER = 
     &   c1 - CCINT*SHF_DATA(:,:,shf_data_cldfrac,now)**2

      RTEA = sqrt( c1000*SHF_DATA(:,:,shf_data_qair,now)
     &             /(0.622_dbl_kind + 0.378_dbl_kind
     &               *SHF_DATA(:,:,shf_data_qair,now)) + eps2 )

      SHF_COMP(:,:,shf_comp_qlw) = -emissivity*stefan_boltzmann
     &     * SHF_DATA(:,:,shf_data_tair,now)**3 
     &     * ( SHF_DATA(:,:,shf_data_tair,now)
     &         *(0.39_dbl_kind - 0.05_dbl_kind*RTEA)*FRAC_CLOUD_COVER
     &         + c4*(TRACER(:,:,1,1,curtime)+T0_Kelvin
     &               - SHF_DATA(:,:,shf_data_tair,now)) )

!----------------------------------------------------------------------
!  weak temperature restoring term (note: OCN_WGT = 0. at land points
!----------------------------------------------------------------------

      SHF_COMP(:,:,shf_comp_wrest) = shf_weak_restore * MASK_SR *OCN_WGT
     &    * (SHF_DATA(:,:,shf_data_sst,now) - TRACER(:,:,1,1,curtime))

!----------------------------------------------------------------------
!  strong temperature restoring term
!----------------------------------------------------------------------

      where(KMT > 0)
        SHF_COMP(:,:,shf_comp_srest) = shf_strong_restore * (c1-OCN_WGT)
     &    * (SHF_DATA(:,:,shf_data_sst,now) - TRACER(:,:,1,1,curtime))
      endwhere

      where( KMT > 0 .and. MASK_SR == 0 )
        SHF_COMP(:,:,shf_comp_srest) = shf_strong_restore_ms
     &    * (SHF_DATA(:,:,shf_data_sst,now) - TRACER(:,:,1,1,curtime))
      endwhere

!----------------------------------------------------------------------
!  net surface heat flux (W/m^2) (except penetrative shortwave flux)
!----------------------------------------------------------------------

      SHF = OCN_WGT * MASK_SR *
     &    ( SHF_COMP(:,:,shf_comp_qsens) 
     &    + SHF_COMP(:,:,shf_comp_qlat) 
     &    + SHF_COMP(:,:,shf_comp_qlw) )
     &    + SHF_COMP(:,:,shf_comp_wrest)  
     &    + SHF_COMP(:,:,shf_comp_srest)  

!----------------------------------------------------------------------
!  copy penetrative shortwave flux into its own array (SHF_QSW) and
!    convert it and SHF to model units.
!----------------------------------------------------------------------

      SHF     = SHF * hflux_factor
      SHF_QSW = SHF_COMP(:,:,shf_comp_qsw) * OCN_WGT * MASK_SR 
     &         * hflux_factor

      end subroutine calc_shf_bulk_ncep

!***********************************************************************


      subroutine calc_shf_partially_coupled (time_dim)

!-----------------------------------------------------------------------
!  calculate weak and strong restoring components of surface heat flux
!  for partially-coupled formulation. these components will later be
!  added to shf_comp_cpl component in set_coupled_forcing 
!  (forcing_coupled) to form the total surface heat flux. 
!
!  the only forcing data (on t-grid) set needed is
!     shf_data_sst,       restoring SST                       (C)
!-----------------------------------------------------------------------

      implicit none

      integer (kind=int_kind) :: time_dim, n, nearest_data, now
 
      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  WORK1

!-----------------------------------------------------------------------
!  if annual forcing, no interpolation to current time is necessary.
!-----------------------------------------------------------------------

      if (shf_data_type == 'annual') then

        now = 1
        nearest_data = 1

      else

!----------------------------------------------------------------------
!  interpolate all shf data to current time and
!  store in SHF_DATA(:,:,:,now)
!----------------------------------------------------------------------

        now = 0

        if(thour00 >= shf_interp_next .or. nsteps_run == 0) then
          do n = 1, shf_data_num_fields
            call interpolate_forcing(SHF_DATA(:,:,n,now)
     &      , SHF_DATA(:,:,n,1:time_dim), shf_data_time, shf_interp_type
     &      , shf_data_time_min_loc, time_dim
     &      , shf_interp_freq, shf_interp_inc
     &      , shf_interp_next, shf_interp_last, nsteps_run)
          enddo
          if (nsteps_run /= 0) shf_interp_next =
     &                         shf_interp_next + shf_interp_inc
        endif

      endif

!----------------------------------------------------------------------
!  compute ocean weights (fraction of ocean vs. ice) every timestep,
!  if needed
!----------------------------------------------------------------------

      if ( .not. luse_cpl_ifrac  ) then
        call ocean_weights (now)
        WORK1 = OCN_WGT*MASK_SR
      else
        WORK1 = MASK_SR
      endif

!----------------------------------------------------------------------
!  weak temperature restoring term (note: MASK_SR = 0. at land and
!  marginal sea points)
!  note that weak restoring may be applied to every non-marginal-sea
!  ocean point.
!----------------------------------------------------------------------

      SHF_COMP(:,:,shf_comp_wrest) = shf_weak_restore * WORK1    
     &    * (SHF_DATA(:,:,shf_data_sst,now) - TRACER(:,:,1,1,curtime))

!----------------------------------------------------------------------
!  strong temperature restoring term
!  note that strong restoring may be applied only in marginal seas.
!  in under-ice regions, the ice formation term may replace the
!  strong-restoring term.
!----------------------------------------------------------------------

      where (KMT > 0)
        SHF_COMP(:,:,shf_comp_srest) = shf_strong_restore * (c1-OCN_WGT)
     &    * (SHF_DATA(:,:,shf_data_sst,now) - TRACER(:,:,1,1,curtime))
      endwhere

      where ( KMT > 0 .and. MASK_SR == 0 )
        SHF_COMP(:,:,shf_comp_srest) = shf_strong_restore_ms
     &    * (SHF_DATA(:,:,shf_data_sst,now) - TRACER(:,:,1,1,curtime))
      endwhere     

!----------------------------------------------------------------------
!  convert to model units: (W/m^2) to (C*cm/s)
!----------------------------------------------------------------------

      SHF_COMP(:,:,shf_comp_wrest) =  SHF_COMP(:,:,shf_comp_wrest)
     &                              * hflux_factor
      SHF_COMP(:,:,shf_comp_srest) =  SHF_COMP(:,:,shf_comp_srest)
     &                              * hflux_factor

      end subroutine calc_shf_partially_coupled

!***********************************************************************

      subroutine sw_absorb_frac( depth, jerlov_water_type
     &                         , sw_absorb_fraction )

!-----------------------------------------------------------------------
!
!     compute fraction of solar short-wave flux penetrating to
!     specified depth, depth, due to exponential decay in
!     Jerlov water type jerlov_water_type.
!     reference : two band solar absorption model of simpson and
!     paulson (1977)
!     note: below 200m the solar penetration gets set to zero,
!     otherwise the limit for the exponent (+/- 5678) needs to be taken
!     care of.
!
!-----------------------------------------------------------------------

      use kinds_mod
      use constants

      implicit none

      integer (kind=int_kind), parameter ::
     &   num_water_types = 5  ! max number of different water types

      real (kind=dbl_kind) ::
     &   depth     ! vertical depth (cm, >0.) for desired sw fraction
     &  ,sw_absorb_fraction     ! short wave (radiation) fractional decay

      integer (kind=int_kind) ::
     &   jerlov_water_type ! index for jerlov water type

      real (kind=dbl_kind), parameter ::
     &   depth_cutoff = -200.0_dbl_kind

      real (kind=dbl_kind) ::
     &   depth_neg_meters

!-----------------------------------------------------------------------
!   define Jerlov water properties with rfac, depth1, depth2
!     Jerlov water type :  I       IA      IB      II      III
!     jerlov_water_type :  1       2       3       4       5
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(num_water_types) ::
     &  rfac = (/   0.58_dbl_kind, 0.62_dbl_kind, 0.67_dbl_kind
     &            , 0.77_dbl_kind, 0.78_dbl_kind / )
     & ,depth1 = (/ 0.35_dbl_kind,  0.6_dbl_kind,  1.0_dbl_kind
     &            ,  1.5_dbl_kind,  1.4_dbl_kind / )
     & ,depth2 = (/ 23.0_dbl_kind, 20.0_dbl_kind, 17.0_dbl_kind
     &            , 14.0_dbl_kind,  7.9_dbl_kind / )

!-----------------------------------------------------------------------
!-----------------------------------------------------------------------

      depth_neg_meters =  -depth*mpercm  !  convert from cm to m and change sign

      if (depth_neg_meters < depth_cutoff) then
         sw_absorb_fraction = c0
      else
         sw_absorb_fraction = rfac(jerlov_water_type)
     &       * exp(depth_neg_meters/depth1(jerlov_water_type))
     &                + (c1 - rfac(jerlov_water_type))
     &       * exp(depth_neg_meters/depth2(jerlov_water_type))
      endif

      end subroutine sw_absorb_frac

!***********************************************************************

      subroutine sen_lat_flux(US,hu,SST,TH,ht,QH,hq,tk0,HS,HL)

      implicit none

      real (kind=dbl_kind), dimension (imt,jmt) ::
     &    SH,T0,DELP,DELQ,STABLETMP,RDN,RHN,USTARR,TSTARR,QSTARR,TAU
     &,   HUOL,HTOL,HQOL,SSHUM,PSIMH,PSIXH,RD,UZN,RH,RE

      real (kind=dbl_kind)
     &    hu,ht,hq,tk0,ren
     &,   umin,zolmin,vonk,lapse_rate,gravity_mks,f1,refhgt,aln,czol
 
c--------------------------------------------------------------------------
c Following procedures and coefficients in Large and Pond (1981 ; 1982)
 
c output : bulk estimates of the turbulent surface fluxes
      real (kind=dbl_kind), dimension (imt,jmt) :: 
     &      HS, ! sensible heat flux  (w/m^2),          into ocean
     &      HL  ! latent   heat flux  (w/m^2),          into ocean
 
c  inputs:
      real (kind=dbl_kind), dimension (imt,jmt) ::
     &      US, ! mean wind speed (m/s)        at height hu (m)
     &      TH, ! mean air temperature (k)     at height ht (m)
     &      QH, ! mean air humidity (kg/kg)    at height hq (m)
     &     SST  ! sea surface temperature (k)
c
c assume 1) a neutral 10m drag coefficient = cdn =
c                    .0027/u10 + .000142 + .0000764 u10
c        2) a neutral 10m stanton number  ctn= .0327 sqrt(cdn), unstable
c                                         ctn= .0180 sqrt(cdn), stable
c        3) a neutral 10m dalton  number  cen= .0346 sqrt(cdn)
c        4) the saturation humidity of air at t(k)  = qsat(t)  (kg/m^3)
c
c  note  1) here, tstar = <wt>/u*, and qstar = <wq>/u*.
c        2) wind speedx should all be above a minimum speed say 0.5 m/s
c        3) with optional interation loop, niter=3, should suffice
c
c ***  this version is for analyses inputs with hu = 10m and ht = hq **
c ***  also, SST enters in Celsius  *************************** 
c
c-----------------------------------------------------------------------
 
c-----------------------------------------------------------------------
c  constants
c-----------------------------------------------------------------------
 
      umin         = 0.5_dbl_kind          ! minimum wind speed
      zolmin       = -100._dbl_kind        ! minimum stability parameter
      vonk         = 0.4_dbl_kind          ! Von Karman''s constant
      lapse_rate   = 0.01_dbl_kind         ! abiabatic lapse rate deg/m
      gravity_mks  = grav/100.0_dbl_kind   ! gravity m/s/s
      f1           = 0.606_dbl_kind    
      refhgt       = 10.0_dbl_kind         ! reference height
      aln          = log(ht/refhgt)
      czol         = hu*vonk*gravity_mks

      SH = max(US,umin)

c-----------------------------------------------------------------------
c  initial guess z/l=0.0; hu=ht=hq=z  
c-----------------------------------------------------------------------
 
      T0  = TH * (c1 + f1 * QH)        ! virtual temperature (k)
      SSHUM = 0.98_dbl_kind * QSAT(SST+tk0) /rho_air 
                                       ! sea surface humidity (kg/kg)
      DELP = TH + lapse_rate*ht - SST - tk0 ! pot temperature diff (k)
      DELQ = QH - SSHUM
      STABLETMP = 0.5_dbl_kind + sign(0.5_dbl_kind , DELP)
      RDN  = sqrt(CDN(SH))
      RHN  = (c1-STABLETMP)* 0.0327_dbl_kind + 
     &           STABLETMP * 0.0180_dbl_kind
      ren  = 0.0346_dbl_kind
      USTARR = RDN * SH
      TSTARR = RHN * DELP
      QSTARR = REN * DELQ

c-----------------------------------------------------------------------
c  first iteration loop
c-----------------------------------------------------------------------
 
      HUOL = czol * (TSTARR/T0 + QSTARR/(c1/f1+QH)) / USTARR**2
      HUOL = max(HUOL,zolmin)
      STABLETMP = 0.5_dbl_kind + sign(0.5_dbl_kind , HUOL)
      HTOL = HUOL * ht / hu
      HQOL = HUOL * hq / hu

c-----------------------------------------------------------------------
c  evaluate all stability functions assuming hq = ht
c-----------------------------------------------------------------------

      SSHUM   = max(sqrt(abs(c1 - 16._dbl_kind*HUOL)),c1)
      SSHUM   = sqrt(SSHUM)
      PSIMH = -5._dbl_kind * HUOL * STABLETMP + (c1-STABLETMP)
     &        * log((c1+SSHUM*(c2+SSHUM))*(c1+SSHUM*SSHUM)/8._dbl_kind) 
     &        - c2*atan(SSHUM)+1.571_dbl_kind
      SSHUM   = max(sqrt(abs(c1 - 16._dbl_kind*HTOL)),c1)
      PSIXH = -5._dbl_kind * HTOL * STABLETMP + (c1-STABLETMP)*
     &       c2*log((c1+SSHUM)/c2)

c-----------------------------------------------------------------------
c  shift wind speed using old coefficient
c-----------------------------------------------------------------------

      RD   = RDN / (c1-RDN/vonk*PSIMH )
      UZN  = max(SH * RD / RDN , umin)

c-----------------------------------------------------------------------
c  update the transfer coefficients at 10 meters and neutral stability
c-----------------------------------------------------------------------

      RDN = sqrt(CDN(UZN))
      ren = 0.0346_dbl_kind
      RHN = (c1-STABLETMP)*0.0327_dbl_kind + 
     &          STABLETMP *0.0180_dbl_kind

c-----------------------------------------------------------------------
c  shift all coefficients to the measurement height and stability
c-----------------------------------------------------------------------

      RD   = RDN / (c1-RDN/vonk*PSIMH )
      RH   = RHN / (c1+RHN/vonk*(  aln     -PSIXH) )
      RE   = ren / (c1+ren/vonk*(  aln     -PSIXH) )

c-----------------------------------------------------------------------
c  update USTARR, TSTARR, QSTARR using updated, shifted  coefficients 
c-----------------------------------------------------------------------

      USTARR = RD * SH  
      QSTARR = RE * DELQ
      TSTARR = RH * DELP

c-----------------------------------------------------------------------
c  second iteration to converge on z/l and hence the fluxes
c-----------------------------------------------------------------------
 
      HUOL= czol * (TSTARR/T0+QSTARR/(c1/f1+QH)) / USTARR**2
      HUOL= max(HUOL,zolmin)
      STABLETMP = 0.5_dbl_kind + sign(0.5_dbl_kind , HUOL)
      HTOL = HUOL * ht / hu
      HQOL = HUOL * hq / hu

c-----------------------------------------------------------------------
c  evaluate all stability functions assuming hq = ht
c-----------------------------------------------------------------------

      SSHUM   = max(sqrt(abs(c1 - 16.*HUOL)),c1)
      SSHUM   = sqrt(SSHUM)
      PSIMH = -5._dbl_kind * HUOL * STABLETMP + (c1-STABLETMP)
     &         * log((c1+SSHUM*(c2+SSHUM))*(c1+SSHUM*SSHUM)/8._dbl_kind) 
     &         - c2*atan(SSHUM)+1.571_dbl_kind
      SSHUM   = max(sqrt(abs(c1 - 16._dbl_kind*HTOL)),c1)
      PSIXH = -5._dbl_kind * HTOL * STABLETMP + (c1-STABLETMP)*
     &       c2*log((c1+SSHUM)/c2)

c-----------------------------------------------------------------------
c  shift wind speed using old coefficient
c-----------------------------------------------------------------------

      RD   = RDN / (c1-RDN/vonk*PSIMH )
      UZN  = max(SH * RD / RDN , umin)

c-----------------------------------------------------------------------
c  update the transfer coefficients at 10 meters and neutral stability
c-----------------------------------------------------------------------

      RDN = sqrt(CDN(UZN))
      ren = 0.0346_dbl_kind
      RHN = (c1-STABLETMP)*0.0327_dbl_kind + 
     &          STABLETMP *0.0180_dbl_kind

c-----------------------------------------------------------------------
c  shift all coefficients to the measurement height and stability
c-----------------------------------------------------------------------

      RD   = RDN / (c1-RDN/vonk*PSIMH )
      RH   = RHN / (c1+RHN/vonk*(  aln     -PSIXH) )
      RE   = ren / (c1+ren/vonk*(  aln     -PSIXH) )

c-----------------------------------------------------------------------
c  update USTARR, TSTARR, QSTARR using updated, shifted  coefficients 
c-----------------------------------------------------------------------

      USTARR = RD * SH
      QSTARR = RE * DELQ
      TSTARR = RH * DELP

c-----------------------------------------------------------------------
c  done >>>>  compute the fluxes
c-----------------------------------------------------------------------

      TAU     = rho_air * USTARR**2   
      TAU     = TAU  * US / SH
      HS      = cp_air* TAU * TSTARR / USTARR
      HL      = latent_heat_vapor * TAU * QSTARR / USTARR

 1000 continue

      end subroutine sen_lat_flux

c***********************************************************************

      function QSAT(TKELVIN) 
        real (kind=dbl_kind), dimension(imt,jmt) :: QSAT, TKELVIN
        QSAT = 640380._dbl_kind / exp(5107.4_dbl_kind/TKELVIN)
      end function

      function CDN(UMPS) 
        real (kind=dbl_kind), dimension(imt,jmt) :: CDN, UMPS
        CDN = 0.0027_dbl_kind / UMPS 
     &      + .000142_dbl_kind + .0000764_dbl_kind * UMPS
      end function


c***********************************************************************

      subroutine ocean_weights (now)

!----------------------------------------------------------------------
!  compute ocean weights (fraction of ocean vs. ice) every timestep.
!  these weights are used in weak and strong restoring components
!  of surface heat and freshwater fluxes for some formulations.
!----------------------------------------------------------------------

      implicit none

      integer (kind=int_kind) :: now


      where (SHF_DATA(:,:,shf_data_sst,now) <= T_strong_restore_limit)
        OCN_WGT = c0
      elsewhere
        OCN_WGT = (SHF_DATA(:,:,shf_data_sst,now)
     &           - T_strong_restore_limit)/dT_restore_limit
      endwhere

      where (SHF_DATA(:,:,shf_data_sst,now) >= T_weak_restore_limit)
     &  OCN_WGT = c1


      where (KMT == 0) OCN_WGT = c0     ! zero OCN_WGT at land points

      end subroutine ocean_weights

!***********************************************************************

      end module forcing_shf

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
