!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module history


!***********************************************************************

!-----------------------------------------------------------------------
!
!     this module contains information necessary for writing history
!     file output
!
!     CVS:$Id: history.F,v 1.2.2.6 2002/04/08 17:48:56 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use communicate
      use domain
      use constants
      use prognostic
      use grid
      use io
      use global_reductions
      use time_management
      use forcing
      use exit_mod
      use io_general
      use io_tools

      implicit none
      save     hist_att     ! redundant saves for compilation on IBM
     &,        history_ncdf
     &,        history_bin
     &,        history_hdr
      save

!-----------------------------------------------------------------------
!
!     module variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  history_flag          ! time flag for writing history files
     &, history_freq_iopt     ! frequency option for writing history
     &, history_freq          ! frequency of history output

      logical (kind=log_kind) ::
     &  lhistory_on           ! history file output wanted
     &, lhistory_use_ncdf = .false.
                              ! use netcdf format for history output file

      character (char_len_long) ::
     &  history_outfile       ! root filename for history output

      !***
      !*** available history fields
      !***

      integer (kind=int_kind) ::
     &  num_hist_fields

      integer (kind=int_kind), dimension(:), allocatable ::
     &  hist_fields


      integer (kind=int_kind), parameter ::
     &  hist_field_shgt   =  1,
     &  hist_field_suf    =  2,
     &  hist_field_svf    =  3,
     &  hist_field_shf    =  4,
     &  hist_field_sfwf   =  5,
     &  hist_field_solar  =  6,
     &  hist_field_uvel   =  7,
     &  hist_field_vvel   =  8,
     &  hist_field_temp   =  9,
     &  hist_field_salt   = 10,
     &  hist_field_ubtrop = 11,
     &  hist_field_vbtrop = 12

      integer, parameter          :: max_hist_att = 15
 
      type (var_attributes)
     &, dimension(max_hist_att)   :: hist_att
 
      type (output_file)          :: history_ncdf
     &,                              history_bin
     &,                              history_hdr
 
 
 
 
 
!***********************************************************************

      contains

!***********************************************************************

      subroutine write_history(file_suffix,ltavg_on)

!-----------------------------------------------------------------------
!
!     write history dump
!
!     fields written are:
!
!       PSURF/grav     ! surface height (surface-pressure formulation only)
!       UBTROP,VBTROP  ! barotropic velocities
!       UVEL,VVEL      ! 3d velocities
!       TRACER         ! 3d tracers
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      character (char_len), intent(in) ::
     &  file_suffix         ! suffix to append to root filename 

      logical (kind=log_kind),intent(in) :: ltavg_on
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  nu,                 ! i/o unit for output file
     &  nfield,             ! dummy field index
     &  n,                  ! dummy tracer index
     &  num_writes          ! number of time levels on one netcdf file
 
      character (char_len) ::
     &  date_string
 
      character (char_len_long) ::
     &  history_filename,   ! filename for history data
     &  header_filename     ! filename for header file

      real (kind=real_kind), dimension(imt,jmt) ::
     &  WORK2D              ! single precision array for 2d output

      real (kind=real_kind), dimension(imt,jmt,km) ::
     &  WORK3D              ! single precision array for 3d output

 
!-----------------------------------------------------------------------
!
!     write header file with contents
!
!-----------------------------------------------------------------------

 
      if (lhistory_use_ncdf) then
       call create_output_filename (history_outfile,'.',file_suffix
     &,                             '.nc',history_filename,history_ncdf)
      else
       call create_output_filename (history_outfile,'.',file_suffix
     &,                            '.hdr',header_filename,history_hdr)
      endif


      if (lhistory_use_ncdf) then
       call output_header (history_ncdf,ltavg_on)  
       call set_outvar_dim (num_hist_fields, hist_att)
       call define_var_att_ncdf  (history_ncdf,num_hist_fields,hist_att)
       num_writes = 1   
       call write_time (history_ncdf,tday00,num_writes)
      else
       call output_header (history_hdr, ltavg_on)
 
        nu = history_hdr%id
 
       if (my_task == master_task) then
        write(nu,1000)
     &       num_hist_fields, nsteps_total, tday00,cyear,cmonth, cday
        do nfield=1,num_hist_fields
            write(nu,'(a,1x,a,3(5x,a))')
     &                  trim(hist_att(nfield)%grid_code)
     &,                 trim(hist_att(nfield)%req%short_name)
     &,                 trim(hist_att(nfield)%req%units)
     &,                 trim(hist_att(nfield)%req%long_name)
        end do
       endif

       if (my_task == master_task) close(nu)
       call release_unit(nu)
 
      endif



 1000 format(i3,1x,i10,1x,f14.6,1x,a4,'-',a2,'-',a2)

!-----------------------------------------------------------------------
!
!     write data file
!
!-----------------------------------------------------------------------

      if (.not. lhistory_use_ncdf) then
        call create_output_filename (history_outfile,'.',file_suffix
     &,                              ' ',history_filename, history_bin)
 
        call open_parallel_file(nu,history_filename,recl_real)
      endif

!-----------------------------------------------------------------------
!
!     write arrays to parallel file
!
!-----------------------------------------------------------------------

        do nfield=1,num_hist_fields
 

          select case(hist_fields(nfield))

          case(hist_field_shgt)
    
            WORK2D = PSURF(:,:,curtime)/grav
 
            if (lhistory_use_ncdf) then
              call write_array_ncdf(history_ncdf,hist_att(nfield)
     &,                           WORK2D,1)         
            else
              call write_array(nu,WORK2D)
            endif

          case(hist_field_suf)

            WORK2D = SMF(:,:,1)
            if (lhistory_use_ncdf) then
              call write_array_ncdf(history_ncdf,hist_att(nfield)
     &,                           WORK2D,1)         
            else
              call write_array(nu,WORK2D)
            endif

          case(hist_field_svf)

            WORK2D = SMF(:,:,2)
            if (lhistory_use_ncdf) then
              call write_array_ncdf(history_ncdf,hist_att(nfield)
     &,                           WORK2D,1)         
            else
              call write_array(nu,WORK2D)
            endif
 
          case(hist_field_shf)

            if (shf_formulation == 'Barnier-restoring' .or.
     &          shf_formulation == 'bulk-NCEP') then  !  add shortwave
              WORK2D = (STF(:,:,1)+SHF_QSW)/hflux_factor
            else
              WORK2D = STF(:,:,1)/hflux_factor
            endif
            if (lhistory_use_ncdf) then
              call write_array_ncdf(history_ncdf,hist_att(nfield)
     &,                           WORK2D,1)         
            else
              call write_array(nu,WORK2D)
            endif

          case(hist_field_sfwf)

            WORK2D = STF(:,:,2)*seconds_in_year/c1000/salinity_factor
            if (lhistory_use_ncdf) then
              call write_array_ncdf(history_ncdf,hist_att(nfield)
     &,                           WORK2D,1)         
            else
              call write_array(nu,WORK2D)
            endif

          case(hist_field_solar)

            WORK2D = SHF_QSW
            if (lhistory_use_ncdf) then
              call write_array_ncdf(history_ncdf,hist_att(nfield)
     &,                           WORK2D,1)         
            else
              call write_array(nu,WORK2D)
            endif

          case(hist_field_uvel)

            WORK3D = UVEL(:,:,:,curtime) 
            if (lhistory_use_ncdf) then
              call write_array_ncdf(history_ncdf,hist_att(nfield)
     &,                           WORK3D,km)        
            else
              call write_array(nu,WORK3D)
            endif

          case(hist_field_vvel)

            WORK3D = VVEL(:,:,:,curtime) 
 
            if (lhistory_use_ncdf) then
             call write_array_ncdf(history_ncdf,hist_att(nfield)
     &,                           WORK3D,km)        
            else
              call write_array(nu,WORK3D)
            endif

          case(hist_field_temp)

            WORK3D = TRACER(:,:,:,1,curtime) 
            if (lhistory_use_ncdf) then
             call write_array_ncdf(history_ncdf,hist_att(nfield)
     &,                           WORK3D,km)        
            else
              call write_array(nu,WORK3D)
            endif

          case(hist_field_salt)
     
            WORK3D = TRACER(:,:,:,2,curtime) 
            if (lhistory_use_ncdf) then
             call write_array_ncdf(history_ncdf,hist_att(nfield)
     &,                           WORK3D,km)        
            else
              call write_array(nu,WORK3D)
            endif

          case(hist_field_ubtrop)

            WORK2D = UBTROP(:,:,curtime)
            if (lhistory_use_ncdf) then
              call write_array_ncdf(history_ncdf,hist_att(nfield)
     &,                           WORK2D,1)         
            else
              call write_array(nu,WORK2D)
            endif

          case(hist_field_vbtrop)

            WORK2D = VBTROP(:,:,curtime)
            if (lhistory_use_ncdf) then
              call write_array_ncdf(history_ncdf,hist_att(nfield)
     &,                           WORK2D,1)         
            else
              call write_array(nu,WORK2D)
            endif

          end select
        end do

        if (lhistory_use_ncdf) then
          call close_file (history_ncdf)
        else
          call close_parallel_file(nu)
          call document ('write_history', 'binary file written: '
     &                   //trim(history_filename))
        endif
 

 
 
      end subroutine write_history

 
 
!***********************************************************************

      subroutine init_history

!-----------------------------------------------------------------------
!
!     initializes history output choices from input files
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input namelist
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  nu,                 ! unit for contents input file
     &  nfield,             ! dummy for field number
     &  n,                  ! dummy for position in character string
     &  nml_error           ! error flag for namelist i/o

      character (char_len) ::
     &  history_freq_opt,   ! choice for frequency of history output
     &  history_contents    ! filename for choosing fields for output

      namelist /history_nml/ history_freq_opt, history_freq,
     &                       history_outfile, history_contents
     &,                      lhistory_use_ncdf

 
!-----------------------------------------------------------------------
!
!     read history file output frequency and filenames from namelist
!
!-----------------------------------------------------------------------

      history_freq_iopt = freq_opt_never
      history_freq      = 100000
      history_outfile   = 'h'
      history_contents  = 'unknown_history_contents'

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=history_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading history_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, history_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        select case (history_freq_opt)
        case ('never')
          history_freq_iopt = freq_opt_never
        case ('nyear')
          history_freq_iopt = freq_opt_nyear
        case ('nmonth')
          history_freq_iopt = freq_opt_nmonth
        case ('nday')
          history_freq_iopt = freq_opt_nday
        case ('nhour')
          history_freq_iopt = freq_opt_nhour
        case ('nsecond')
          history_freq_iopt = freq_opt_nsecond
        case ('nstep')
          history_freq_iopt = freq_opt_nstep
        case default
          history_freq_iopt = -1000
        end select

      endif

      call broadcast_scalar(history_freq_iopt, master_task)
      call broadcast_scalar(history_freq,      master_task)

      if (history_freq_iopt == -1000) then
        call exit_POP('ERROR: unknown option for history file freq')
      else if (history_freq_iopt == freq_opt_never) then
        lhistory_on = .false.
      else
        lhistory_on = .true.
      endif

      if (lhistory_on) then
        call broadcast_scalar(history_outfile,  master_task)
        call broadcast_scalar(history_contents, master_task)
        call broadcast_scalar(lhistory_use_ncdf,master_task)
      endif

      history_flag = init_time_flag('history',default=.false.,
     &                              freq_opt = history_freq_iopt,
     &                              freq     = history_freq)
 
!-----------------------------------------------------------------------
!     set netcdf variables required by the new io routines
!-----------------------------------------------------------------------
      if (lhistory_use_ncdf) call init_io_ncdf
 
 
!-----------------------------------------------------------------------
!
!     read contents file to determine which fields to dump
!
!-----------------------------------------------------------------------

      if (lhistory_on) then
        call get_unit(nu)
        call read_contents_header(nu,num_hist_fields
     &,                           history_contents,newfile=.true.)
        call set_var_defaults(lhistory_use_ncdf,num_hist_fields
     &,                       hist_att)
 
        call read_contents (nu,num_hist_fields,hist_att
     &,                     close_contents=.true.)
 
 
        allocate ( hist_fields(num_hist_fields))


 
        do nfield=1,num_hist_fields
 
          select case(trim(hist_att(nfield)%req%short_name))
          case('SHGT')
            hist_fields(nfield) = hist_field_shgt
          case('SUF')
            hist_fields(nfield) = hist_field_suf
          case('SVF')
            hist_fields(nfield) = hist_field_svf
          case('SHF')
            hist_fields(nfield) = hist_field_shf
          case('SFWF')
            hist_fields(nfield) = hist_field_sfwf
          case('SOLAR')
            hist_fields(nfield) = hist_field_solar
          case('UVEL')
            hist_fields(nfield) = hist_field_uvel
          case('VVEL')
            hist_fields(nfield) = hist_field_vvel
          case('TEMP')
            hist_fields(nfield) = hist_field_temp
          case('SALT')
            hist_fields(nfield) = hist_field_salt
          case('UBTROP')
            hist_fields(nfield) = hist_field_ubtrop
          case('VBTROP')
            hist_fields(nfield) = hist_field_vbtrop
          case default
            hist_fields(nfield) = -1000
          end select
        end do
 
        if (my_task == master_task) close(nu)

        call release_unit(nu)
        call broadcast_vector(hist_fields, master_task, num_hist_fields)

        if (any(hist_fields == -1000)) then
          call exit_POP('unknown field requested for history file')
        endif
      endif


!-----------------------------------------------------------------------
!       omit cell_methods for all history fields.  By setting
!       cell_methods = 'unknown_cell_methods',
!       the netcdf-writing code will omit this attribute
!       (a missing cell_methods implies an instantaneous field)
!-----------------------------------------------------------------------
      hist_att(:)%cell_methods = 'unknown_cell_methods'
 
 
!-----------------------------------------------------------------------

      end subroutine init_history

!***********************************************************************

      end module history

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
