!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module hmix_del2

!***********************************************************************
!
!     this module contains routine for computing Laplacian horizontal
!     diffusion of momentum and tracers
!
!     CVS:$Id: hmix_del2.F,v 1.2.2.2 2001/07/05 22:43:25 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use operators
      use topostress
      use diagnostics
      use grid

 
      implicit none

!***********************************************************************

      contains

!***********************************************************************

      subroutine hdiffu_del2(k,HDUK,HDVK,am,lvariable_hmix,AMF)

!-----------------------------------------------------------------------
!
!     horizontial diffusion of momentum
!
!     HDiff(Ub) = am*Del**2(Ub)  (laplacian mixing)
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! depth level index

      real (kind=dbl_kind), intent(in) ::
     &  am                  ! horizontal momentum mixing coefficient

      logical (kind=log_kind), intent(in) ::
     &  lvariable_hmix      ! flag for spatial variation of mix coeffs

      real (kind=dbl_kind), dimension(:,:), intent(in) ::
     &  AMF                 ! spatial variation of mixing coeff

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &  HDUK,               ! Hdiff(Ub) at level k
     &  HDVK                ! Hdiff(Vb) at level k

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  UTMP, VTMP,         ! modified velocities to use with topostress
     &  HDIFFCFL            ! for cfl number diagnostics
      
!-----------------------------------------------------------------------
!
!     laplacian mixing
!
!-----------------------------------------------------------------------

      if (ltopostress) then
        UTMP = merge(UVEL(:,:,k,mixtime) - TSU,
     &               UVEL(:,:,k,mixtime), k <= KMU)
        VTMP = merge(VVEL(:,:,k,mixtime) - TSV,
     &               VVEL(:,:,k,mixtime), k <= KMU)
        call del2u(k,HDUK,HDVK,UTMP,VTMP)
      else
        call del2u(k,HDUK,HDVK,UVEL(:,:,k,mixtime),
     &                         VVEL(:,:,k,mixtime))
      endif

!-----------------------------------------------------------------------
!
!     multiply by viscosity
!
!-----------------------------------------------------------------------

      HDUK = am*HDUK
      HDVK = am*HDVK

!-----------------------------------------------------------------------
!
!     compute horiz diffusion cfl diagnostics if required
!
!-----------------------------------------------------------------------

      if (ldiag_cfl) then
 
        if (lvariable_hmix) then
          HDIFFCFL = merge(c4*am*AMF*(DXUR**2 + DYUR**2), c0, KMU > k)
        else
          HDIFFCFL = merge(c4*am*    (DXUR**2 + DYUR**2), c0, KMU > k)
        endif
        HDIFFCFL = abs(HDIFFCFL)
        call cfl_hdiff(k,HDIFFCFL,2)
            
      endif

!-----------------------------------------------------------------------

      end subroutine hdiffu_del2

!***********************************************************************

      subroutine hdifft_del2(k,HDTK,ah,lvariable_hmix,AHF)

!-----------------------------------------------------------------------
!
!     horizontial diffusion of tracers
!
!     HDiff(Tb) = ah*Del**2(Tb)  (laplacian mixing)
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! depth level index

      real (kind=dbl_kind), intent(in) ::
     &  ah                  ! horizontal tracer mixing coefficient

      logical (kind=log_kind), intent(in) ::
     &  lvariable_hmix      ! flag for spatial variation of mix coeffs

      real (kind=dbl_kind), dimension(:,:), intent(in) ::
     &  AHF                 ! spatial variation of mixing coeff

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt) :: 
     &  HDTK                ! HDIFF(T) for tracer n at level k

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  HDIFFCFL            ! for cfl number diagnostics

!-----------------------------------------------------------------------
!
!     laplacian mixing
!
!-----------------------------------------------------------------------

      call del2t(k,HDTK)

!-----------------------------------------------------------------------
!
!     multiply by diffusivity
!
!-----------------------------------------------------------------------

      HDTK = ah*HDTK

!-----------------------------------------------------------------------
!
!     compute horiz diffusion cfl diagnostics if required
!
!-----------------------------------------------------------------------

      if (ldiag_cfl) then
 
        if (lvariable_hmix) then
          HDIFFCFL = merge(c4*ah*AHF*(DXTR**2 + DYTR**2), c0, KMT > k)
        else
          HDIFFCFL = merge(c4*ah*    (DXTR**2 + DYTR**2), c0, KMT > k)
        endif
        HDIFFCFL = abs(HDIFFCFL)
        call cfl_hdiff(k,HDIFFCFL,1)
            
      endif

!-----------------------------------------------------------------------

      end subroutine hdifft_del2

!***********************************************************************

      end module hmix_del2

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
