!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module horizontal_mix

!***********************************************************************
!
!     CVS:$Id: horizontal_mix.F,v 1.2.2.5 2002/01/04 22:00:29 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use io
      use operators
      use global_reductions
      use hmix_del2
      use hmix_del4
      use hmix_gm
      use hmix_aniso
      use hmix_smag
      use exit_mod
      use tavg
      use timers

      implicit none
 
      integer (kind=int_kind), private ::
     &   timer_hmix_tracer
     &,  timer_hmix_momentum
 
      save

!-----------------------------------------------------------------------
!
!     horizontal mixing variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter :: ! choices for mixing type
     &  hmix_momentum_type_del2 = 1,
     &  hmix_momentum_type_del4 = 2,
     &  hmix_momentum_type_smag = 3,
     &  hmix_momentum_type_anis = 4,
     &  hmix_tracer_type_del2 = 1,
     &  hmix_tracer_type_del4 = 2,
     &  hmix_tracer_type_smag = 3,
     &  hmix_tracer_type_gm   = 4

      integer (kind=int_kind) ::  ! integer choice for type of mixing
     &  hmix_momentum_itype,
     &  hmix_tracer_itype

      real (kind=dbl_kind) ::
     &  ah,              ! horizontal tracer   mixing coefficient
     &  am               ! horizontal momentum mixing coefficient

      logical (kind=log_kind) ::
     &  lauto_hmix,      ! automatically computing mixing coeffs
     &  lvariable_hmix   ! spatially varying mixing coeffs

      real (kind=dbl_kind), dimension(:,:), allocatable :: 
     &  AHF,             ! variable mixing factor for tracer   mixing
     &  AMF              ! variable mixing factor for momentum mixing

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_horizontal_mix

!-----------------------------------------------------------------------
!
!     initialize various mixing parameters based on namelist input
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: nml_error  ! error flag for namelist

      character (char_len) ::  ! character choice for type of mixing
     &  hmix_momentum_choice,
     &  hmix_tracer_choice

      namelist /hmix_nml/ hmix_momentum_choice, hmix_tracer_choice,
     &                    lauto_hmix, lvariable_hmix, ah, am

      real (kind=dbl_kind) ::
     &  ahfmin, ahfmax, amfmin, amfmax

!-----------------------------------------------------------------------
!
!     read namelist input
!
!-----------------------------------------------------------------------

      hmix_momentum_choice = 'unknown_hmix_momentum_choice'
      hmix_tracer_choice = 'unknown_hmix_tracer_choice'
      lauto_hmix = .false.
      lvariable_hmix = .false.
      ah = c0
      am = c0

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=hmix_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading hmix_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, hmix_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then

        write(stdout,1100)
        write(stdout,*)
        write(stdout,*) 'Horizontal mixing options'
        write(stdout,*)
        write(stdout,1100)
 1100   format(72('-')) 

        select case (hmix_momentum_choice(1:4))
        case ('del2')
          hmix_momentum_itype = hmix_momentum_type_del2
          write(stdout,*) 'Laplacian horizontal momentum mixing used.'
        case ('del4')
          hmix_momentum_itype = hmix_momentum_type_del4
          write(stdout,*) 'Biharmonic horizontal momentum mixing used.'
        case ('smag')
          hmix_momentum_itype = hmix_momentum_type_smag
          write(stdout,*) 'Smagorinsky horizontal momentum mixing used.'
        case ('anis')
          hmix_momentum_itype = hmix_momentum_type_anis
          write(stdout,*) 'Anisotropic horizontal momentum mixing used.'
        case ('gent')
          hmix_momentum_itype = -1000
        case default
          hmix_momentum_itype = -2000
        end select

        select case (hmix_tracer_choice(1:4))
        case ('del2')
          hmix_tracer_itype = hmix_tracer_type_del2
          write(stdout,*) 'Laplacian horizontal tracer   mixing chosen.'
        case ('del4')
          hmix_tracer_itype = hmix_tracer_type_del4
          write(stdout,*) 'Biharmonic horizontal tracer   mixing used.'
        case ('smag')
          hmix_tracer_itype = hmix_tracer_type_smag
          write(stdout,*) 'Smagorinsky horizontal tracer   mixing used.'
        case ('gent')
          hmix_tracer_itype = hmix_tracer_type_gm
          write(stdout,*) 'Gent-McWilliams tracer mixing used.'
        case default
          hmix_tracer_itype = -1000
        end select

        if (.not. lauto_hmix .and.
     &      (hmix_momentum_itype == hmix_momentum_type_del2 .or.
     &       hmix_momentum_itype == hmix_momentum_type_del4)) then
          write(stdout,*) 'Using input horizontal viscosity:'
          write(stdout,1000) '  am =',am
 1000     format(a7,2x,1pe12.5)
        endif

        if (.not. lauto_hmix .and.
     &      (hmix_tracer_itype == hmix_tracer_type_del2 .or.
     &       hmix_tracer_itype == hmix_tracer_type_del4 .or.
     &       hmix_tracer_itype == hmix_tracer_type_gm)) then
          write(stdout,*) 'Using input horizontal diffusivity:'
          write(stdout,1000) '  ah =',ah
        endif

        if (.not. lvariable_hmix .and.
     &      (hmix_momentum_itype == hmix_momentum_type_del2 .or.
     &       hmix_momentum_itype == hmix_momentum_type_del4 .or.
     &       hmix_tracer_itype == hmix_tracer_type_del2 .or.
     &       hmix_tracer_itype == hmix_tracer_type_del4 .or.
     &       hmix_tracer_itype == hmix_tracer_type_gm)) then
          write(stdout,*) 'Variable horizontal mixing disabled'
        endif

      endif

      call broadcast_scalar(lauto_hmix,          master_task)
      call broadcast_scalar(lvariable_hmix,      master_task)
      call broadcast_scalar(hmix_momentum_itype, master_task)
      call broadcast_scalar(hmix_tracer_itype,   master_task)
      call broadcast_scalar(ah,                  master_task)
      call broadcast_scalar(am,                  master_task)

      if (hmix_momentum_itype == -1000) then
        call exit_POP(
     &          'Gent-McWilliams can only be used for tracer mixing')
      else if (hmix_momentum_itype == -2000) then
        call exit_POP( 'Unknown type for horizontal momentum mixing')
      endif

      if (hmix_tracer_itype == -1000) then
        call exit_POP( 'Unknown type for horizontal tracer mixing')
      endif

      !***
      !*** check compatibility of some of these choices
      !***

      if ((hmix_momentum_itype == hmix_momentum_type_smag .and.
     &     hmix_tracer_itype   /= hmix_tracer_type_smag) .or. 
     &    (hmix_momentum_itype /= hmix_momentum_type_smag .and.
     &     hmix_tracer_itype   == hmix_tracer_type_smag)) then
        call exit_POP(
     &    'init_hmix: Both horizontal mixing types must be Smagorinsky')
      endif

!-----------------------------------------------------------------------
!
!     automatically set horizontal mixing coefficients if requested 
!
!-----------------------------------------------------------------------

      if (lauto_hmix) then

        select case (hmix_momentum_itype)
        case (hmix_momentum_type_del2)

          ! scale to 1e7 at 1/2 degree
          am = 1.0e7_dbl_kind*(720.0_dbl_kind/float(imt_global))
          if (my_task == master_task) then
            write(stdout,*)
     &        'Horizontal viscosity computed automatically:'
            write(stdout,1000) '  am =',am
          endif

        case (hmix_momentum_type_del4)

          am = -0.6e20_dbl_kind*(1280.0_dbl_kind/float(imt_global))
          if (my_task == master_task) then
            write(stdout,*)
     &        'Horizontal viscosity computed automatically:'
            write(stdout,1000) '  am =',am
          endif

        end select

        select case (hmix_tracer_itype)
        case (hmix_tracer_type_del2)

          ! scale to 1e7 at 1/2 degree
          ah = 1.0e7_dbl_kind*(720.0_dbl_kind/float(imt_global))
          if (my_task == master_task) then
            write(stdout,*)
     &        'Horizontal diffusivity computed automatically:'
            write(stdout,1000) '  ah =',ah
          endif

        case (hmix_tracer_type_del4)

          ah = -0.2e20_dbl_kind*(1280.0_dbl_kind/float(imt_global))
          if (my_task == master_task) then
            write(stdout,*)
     &        'Horizontal diffusivity computed automatically:'
            write(stdout,1000) '  ah =',ah
          endif

        end select

      endif

!-----------------------------------------------------------------------
!
!     set spatially variable mixing arrays if requested
!
!     this applies only to momentum or tracer mixing
!     with del2 or del4 options
!
!     functions describing spatial dependence of horizontal mixing
!     coefficients:   ah -> ah*AHF, am -> am*AHM
!
!     for standard laplacian  mixing they scale like (cell area)**0.5
!     for standard biharmonic mixing they scale like (cell area)**1.5
!     (note: these forms assume a global mesh with a grid line along
!     the equator, and the mixing coefficients are the set relative
!     to the average grid spacing at the equator - for cells of
!     this size AMF = AHF = 1.0).
!
!-----------------------------------------------------------------------

      if (lvariable_hmix) then

        amfmin = c1
        amfmax = c1

        select case (hmix_momentum_itype)
        case (hmix_momentum_type_del2)
          allocate(AMF(imt,jmt))
          AMF = sqrt(UAREA/(c2*pi*radius/imt_global)**2)

          !RSx3 *** set variable viscosity for x3-prime run
          !AMF = c4*am*(DXUR**2+DYUR**2)*dtu
          !where(AMF.gt.p5)
          !  AMF = p5/AMF
          !elsewhere
          !  AMF = c1
          !endwhere
          !RSx3

          amfmin = global_minval(AMF)
          amfmax = global_maxval(AMF)
        case (hmix_momentum_type_del4)
          allocate(AMF(imt,jmt))
          AMF = (UAREA/uarea_equator)**1.5
          amfmin = global_minval(AMF)
          amfmax = global_maxval(AMF)
        end select

        if (my_task == master_task) then
          write(stdout,*) 'Variable horizontal viscosity enabled'
          write(stdout,1020) amfmin, amfmax
 1020     format('  Min AMF =',1x,1pe12.5,3x,'Max AMF =',1x,1pe12.5)
        endif

        ahfmin = c1
        ahfmax = c1

        select case (hmix_tracer_itype)
        case (hmix_tracer_type_del2)
          allocate(AHF(imt,jmt))
          AHF = sqrt(TAREA/(c2*pi*radius/imt_global)**2)
          ahfmin = global_minval(AHF)
          ahfmax = global_maxval(AHF)
        case (hmix_tracer_type_del4)
          allocate(AHF(imt,jmt))
          AHF = (TAREA/uarea_equator)**1.5
          ahfmin = global_minval(AHF)
          ahfmax = global_maxval(AHF)
        end select

        if (my_task == master_task) then
          if (hmix_tracer_itype == hmix_tracer_type_gm) then
            write(stdout,*) 'WARNING: variable horizontal mixing is'
            write(stdout,*) 'enabled but does not effect GM mixing;'
            write(stdout,*) 'spatially varying kappa must be computed'
            write(stdout,*) 'in the GM routine.'
          endif
          write(stdout,*) 'Variable horizontal diffusivity enabled'
          write(stdout,1010) ahfmin, ahfmax
 1010     format('  Min AHF =',1x,1pe12.5,3x,'Max AHF =',1x,1pe12.5)
        endif

      endif

!-----------------------------------------------------------------------
!
!     calculate additional coefficients if necessary
!
!-----------------------------------------------------------------------
 
      if (hmix_momentum_itype == hmix_momentum_type_anis) then
        call init_aniso ! variables used with anisotropic viscosity 
      endif

      if (hmix_momentum_itype == hmix_momentum_type_smag) then
        call init_smag  ! coefficients used by Smagorinsky viscosity
      endif

      if (hmix_tracer_itype == hmix_tracer_type_gm) then
        call init_gm  ! variables used by GM parameterization
      endif

      if (.not. allocated(AHF)) allocate(AHF(1,1))
      if (.not. allocated(AMF)) allocate(AMF(1,1))

!-----------------------------------------------------------------------
!
!     initialize timers
!
!-----------------------------------------------------------------------
 
      select case (hmix_momentum_itype)
        case (hmix_momentum_type_del2)
          call get_timer (timer_hmix_momentum,'HMIX_DEL2_MOMENTUM')
        case (hmix_momentum_type_del4)
          call get_timer (timer_hmix_momentum,'HMIX_DEL4_MOMENTUM')
        case (hmix_momentum_type_smag)
          call get_timer (timer_hmix_momentum,'HMIX_SMAG_MOMENTUM')
        case (hmix_momentum_type_anis)
          call get_timer (timer_hmix_momentum,'HMIX_ANISO_MOMENTUM')
      end select
 
      select case (hmix_tracer_itype)
        case (hmix_tracer_type_del2)
          call get_timer (timer_hmix_tracer,'HMIX_DEL2_TRACER')
        case (hmix_tracer_type_del4)
          call get_timer (timer_hmix_tracer,'HMIX_DEL4_TRACER')
        case (hmix_tracer_type_smag)
          call get_timer (timer_hmix_tracer,'HMIX_SMAG_TRACER')
        case (hmix_tracer_type_gm)
          call get_timer (timer_hmix_tracer,'HMIX_GM_TRACER')
      end select
!-----------------------------------------------------------------------


      end subroutine init_horizontal_mix

!***********************************************************************

      subroutine hdiffu(k,HDUK,HDVK)

!-----------------------------------------------------------------------
!
!     horizontial diffusion of momentum
!
!     this is only a wrapper which calls a specific routine for the
!     type of horizontal mixing chosen in init_horizontal_mix
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k   ! depth level index

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &  HDUK,               ! returned as Hdiff(U) at level k
     &  HDVK                ! returned as Hdiff(V) at level k

!-----------------------------------------------------------------------

      call timer_start(timer_hmix_momentum)
 
      select case (hmix_momentum_itype)

      case (hmix_momentum_type_del2)
        call hdiffu_del2(k,HDUK,HDVK,am,lvariable_hmix,AMF)
      case (hmix_momentum_type_del4)
        call hdiffu_del4(k,HDUK,HDVK,am,lvariable_hmix,AMF)
      case (hmix_momentum_type_anis)
        call hdiffu_aniso(k,HDUK,HDVK)
      case (hmix_momentum_type_smag)
        call hdiffu_smag(k,HDUK,HDVK)
      end select

      call timer_stop (timer_hmix_momentum)
 
!-----------------------------------------------------------------------

      end subroutine hdiffu

!***********************************************************************

      subroutine hdifft(k,HDTK)

!-----------------------------------------------------------------------
!
!     horizontial diffusion of tracers
!
!     this is only a wrapper which calls a specific routine for the
!     type of horizontal mixing chosen in init_horizontal_mix
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k   ! depth level index

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(out) :: 
     &  HDTK                ! Hdiff(T) for nth tracer at level k

!-----------------------------------------------------------------------

      call timer_start(timer_hmix_tracer)
 
      select case (hmix_tracer_itype)
      case (hmix_tracer_type_del2)
        call hdifft_del2(k,HDTK,ah,lvariable_hmix,AHF)
      case (hmix_tracer_type_del4)
        call hdifft_del4(k,HDTK,ah,lvariable_hmix,AHF)
      case (hmix_tracer_type_smag)
        call hdifft_smag(k,HDTK)
      case (hmix_tracer_type_gm)
        call hdifft_gm(k,HDTK,ah)
      end select
 
      call timer_stop (timer_hmix_tracer)

      if (ltavg_on .and. mix_pass /= 1) then
        call tavg_hdifft(k,HDTK)
      endif

!-----------------------------------------------------------------------

      end subroutine hdifft

!***********************************************************************

      end module horizontal_mix

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
