!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module interp

!***********************************************************************

!-----------------------------------------------------------------------
!
!     module containing various interpolation routines used by the
!       forcing modules to temporally interpolate forcing data
!       to the current timestep.
!
!     CVS:$Id: interp.F,v 1.2.2.1 2001/03/13 22:54:56 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants

!***********************************************************************

      contains

!***********************************************************************

      subroutine interp_4pt(INTERP,DATA1,DATA2,DATA3,DATA4,
     &                      tdata1,tdata2,tdata3,tdata4,time) 

!-----------------------------------------------------------------------
!    3rd degree polynomial fit to 4 consecutive in time data points 
!      using Nevilles algorithm described in Numerical Recipes.
!-----------------------------------------------------------------------

      implicit none

      real (kind=dbl_kind), dimension(:,:) :: 
     &   DATA1,DATA2,DATA3,DATA4
      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   WORK1,WORK2,WORK3,WORK4,INTERP

      real (kind=dbl_kind) tdata1,tdata2,tdata3,tdata4,time

c***********************************************************************

      WORK1 = c0
      WORK2 = c0
      WORK3 = c0
      WORK4 = c0
      call det(WORK1,DATA1,DATA2,tdata1,tdata2,time) ! P12
      call det(WORK2,DATA2,DATA3,tdata2,tdata3,time) ! P23
      call det(WORK3,DATA3,DATA4,tdata3,tdata4,time) ! P34

      call det(WORK4,WORK1,WORK2,tdata1,tdata3,time) ! P123
      call det(WORK1,WORK2,WORK3,tdata2,tdata4,time) ! P234
      
      call det(INTERP,WORK4,WORK1,tdata1,tdata4,time) ! P1234

      return
      end subroutine interp_4pt

c|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      subroutine det(C,A,B,y,z,x)

      implicit none

      real (kind=dbl_kind) y,z,x
      real (kind=dbl_kind), dimension(imt,jmt) :: A,B,C

c***********************************************************************

      C = ( A*(z-x) - B*(y-x) )/(z-y)

      end subroutine det

c|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      subroutine interp_2pt(INTERP,DATA1,DATA2,tdata1,tdata2,time)

!-----------------------------------------------------------------------
!    linear interpolation in time.
!-----------------------------------------------------------------------

      implicit none

      real (kind=dbl_kind), dimension(:,:) :: DATA1,DATA2,INTERP

      real (kind=dbl_kind) tdata1,tdata2,weight,time

c***********************************************************************

      weight = (tdata2 - time)/(tdata2 - tdata1)

      INTERP = weight*DATA1 + (c1 - weight)*DATA2

      end subroutine interp_2pt

!***********************************************************************

      end module interp

!***********************************************************************

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
