      module io_general
!     =================
 
      use grid
      use global_reductions
      use ice
      use io_ncdf 
      use io
      use time_management
      
      implicit none
 
!-----------------------------------------------------------------------
!
!
!     CVS:$Id: io_general.F,v 1.1.4.10 2002/04/25 22:45:04 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

 
      save
 
      contains
 
 
 
      subroutine create_output_filename (root,separator, suffix
!     =================================
     &,                                  type, output_filename,file)
 
!---------------------------------------------------------------------
!     build an output filename from input information
!---------------------------------------------------------------------
 
 
      character (*)               :: root
     &,                              separator
     &,                              suffix
     &,                              type
      character (*)               :: output_filename
 
      integer (kind=int_kind)     :: cindex, cindex2
 
      type (output_file)          :: file
 
      intent (in)                 :: root, separator, suffix, type
      intent (out)                :: file, output_filename
 
        
      output_filename = ' '   
      
      write(output_filename,'(a,a,a,a)') trim(root)  ,trim(separator)
     &,                                  trim(suffix),trim(type)
 
      file%name   = trim (output_filename)
      file%root   = trim (root)
      file%suffix = trim (suffix)
      file%type   = trim (type)
 
      if (file%type == '.nc') then
          file%lncdf = .true.
      else
          file%lncdf = .false.
      endif
 
 
      end subroutine create_output_filename
 
 
 
      subroutine output_header (file,time_average)
!     ========================
 
!---------------------------------------------------------------------
!      Create the output file and write header information.
!      For netcdf files, this includes:
!        unlimited dimension (time), global attributes, dimensions,
!        coordinates, time-invariant arrays, and scalars
!      The netcdf-eqivalent for binary files has not been implemented
!---------------------------------------------------------------------

 
      type    (output_file)       :: file
      logical (kind=log_kind)     :: time_average
 
      intent (inout)              :: file
      intent (in)                 :: time_average
 
 
 
 
!---------------------------------------------------------------------
!     create a new netcdf file
!---------------------------------------------------------------------
      call create_file (file)
 
 
!---------------------------------------------------------------------
!     define unlimited dimension attributes (time)
!---------------------------------------------------------------------
        call unlimited (file)
 
 
!---------------------------------------------------------------------
!     define global attributes
!---------------------------------------------------------------------
        call global  (file)
 
 
!---------------------------------------------------------------------
!     define dimensions  (nlon,nlat,z_t,z_w,tracers)
!---------------------------------------------------------------------
      call dimensions  (file)
 
 
!---------------------------------------------------------------------
!     define coordinates (ULONG,ULAT,etc)
!---------------------------------------------------------------------
      call coordinates (file)
 
 
!---------------------------------------------------------------------
!     define time-invariant variables
!---------------------------------------------------------------------
      call time_invar (file)
                                                                           
 
!---------------------------------------------------------------------
!     define scalars
!---------------------------------------------------------------------
      call scalars (file)
 
 
!---------------------------------------------------------------------
!     define time-bounds, now that dimensions are available
!---------------------------------------------------------------------
      call time_bounds (file)
 
 
      end subroutine output_header
 
 
 
      subroutine create_file (file)
!     ======================
 
!---------------------------------------------------------------------
!     create new netcdf file or open a new binary file
!---------------------------------------------------------------------
 
      type    (output_file)       :: file
      integer (kind=int_kind)     :: nu
 
 
      if (file%lncdf) then
        call create_file_ncdf (file)
      else
!---------------------------------------------------------------------
!     open a binary file
!---------------------------------------------------------------------
        call get_unit(nu)
        file%id = nu
        if (my_task == master_task) then   
          open (nu, file=trim(file%name), status='unknown')
        endif
 
        if (debug_ncdf) then
          call document ('create_file'
     &,                  'opened the binary file: '//trim(file%name))
        endif
      endif
 
 
      end subroutine create_file    
 
 
 
      subroutine unlimited  (file)
!     ====================
 
!---------------------------------------------------------------------
!     define the required time attributes 
!---------------------------------------------------------------------
      type    (output_file)       :: file
 
      intent (in)                 :: file
 
 
      time_info%dim%name = 'time'
      time_info%dim%len  =  xtype_unlim
      time_info%dim%id   =  -1
      time_info%units    = 'days since 0000-01-01 00:00:00'
      time_info%bounds   = 'time_bound'
      if (allow_leapyear) then
      time_info%calendar = 'gregorian'
      else  
      time_info%calendar = 'noleap' 
      endif 
 
      time_info%type     =  xtype_double
 
 
      if (file%lncdf) then
          call def_dim_ncdf  (file,time_info%dim)
          call def_time_ncdf (file)
      else
          ! add binary equivalent here
      endif
        
 
      end subroutine unlimited
 
 
 
      subroutine global (file)
!     =================
 
!---------------------------------------------------------------------
!     define the required global attributes 
!---------------------------------------------------------------------
      use time_management
 
 
      type (output_file)            
     &,intent (in)                :: file
 
      integer (kind=int_kind)     :: NG
 
      character (char_len)        :: title       
      character (char_len)        :: contents
      character (char_len)        :: source      
      character (char_len)        :: revision
      character (char_len)        :: calendar  
      character (320     )        :: cell_methods
      character (char_len)        :: conventions
      character (char_len)        :: start_time 
      character (char_len)        :: current_date
     &,                              current_time
 
 
 
      title       =   runid
      contents    =  'Diagnostic and Prognostic Variables' 
      source      =  'POP, the NCAR/CSM Ocean Component'
      if (file%lncdf) then
         conventions = 'CF-1.0; ' //
     &       'http://www.cgd.ucar.edu/cms/eaton/netcdf/CF-current.htm'
      else
         conventions =  ' '
      endif

      revision    =  ' $Name: ccsm2_0_beta46 $ '
 
      if (allow_leapyear) then
         write(calendar,'(a,i5,a,i5,a)') 
     &         ' Leap years allowed. '
     &      // ' Normal years have' , days_in_norm_year, ' days.'
     &      // ' Leap years have'   , days_in_leap_year, ' days.'
      else
         write(calendar,'(a,i5,a)') 
     &        'All years have exactly', days_in_norm_year, ' days.'
      endif
 
      call date_and_time (date=current_date,time=current_time)
      write(start_time,1000)
     &              current_date(1:4), current_date(5:6)
     &,             current_date(7:8), current_time(1:2)
     &,             current_time(3:4), current_time(5:8)
           
      cell_methods = 
     &     'cell_methods = time: mean ==> the variable values '
     & //  'are averaged over the time interval between the previous '
     & //  'time coordinate and the current one.          '
     & //  'cell_methods  absent  ==> the variable values '
     & //  'are at the time given by the current time coordinate. '
 
 
 
      if (file%lncdf) then
        call redefine_ncdf   (file)
        NG = xtype_global
        call put_att_ncdf(file,NG,'title'       ,title      )
        call put_att_ncdf(file,NG,'contents'    ,contents   )
        call put_att_ncdf(file,NG,'source'      ,source     )
        call put_att_ncdf(file,NG,'revision'    ,revision   )
        call put_att_ncdf(file,NG,'calendar'    ,calendar   )
        call put_att_ncdf(file,NG,'conventions' ,conventions)
        call put_att_ncdf(file,NG,'start_time'  ,start_time )
        call put_att_ncdf(file,NG,'cell_methods',cell_methods)
        call end_define_ncdf (file, 'global')
      else
      endif

1000  format('This dataset was created on '
     &,       a,'-',a,'-',a,' at ',a,':',a,':',a)
 
 
      end subroutine global
 
 
 
      subroutine dimensions (file)
!     =====================
       
!---------------------------------------------------------------------
!     define dimensions (short_name, len, & id) and write values to 
!       history file
!---------------------------------------------------------------------
 
      integer (kind=int_kind)
     &, parameter                 ::
     &     max_dims    =  15                ! max number of dimensions
 
      type (output_file)       
     &, intent (in)               :: file
 
      integer (kind=int_kind)     :: fid
     &,                              ind
     &,                              vid 
     &,                              nn

      type (dim_attributes) 
     &, dimension(max_dims)       :: dims
 

      ind = 0

!---------------------------------------------------------------------
!     define information for the creation of the dimensions
!---------------------------------------------------------------------
      ind         = ind+1
      dims(ind)   = dim_attributes ('nlon',imt_global,-1)
      dim_nlon    = dims(ind)
 
      ind         = ind+1
      dims(ind)   = dim_attributes ('nlat',jmt_global,-1)
      dim_nlat    = dims(ind)
 
      ind         = ind+1
      dims(ind)   = dim_attributes ('z_t',km,-1)
      dim_z_t     = dims(ind)

      ind         = ind+1
      dims(ind)   = dim_attributes ('z_w',km,-1)
      dim_z_w     = dims(ind)
 
      ind         = ind+1
      dims(ind)   = dim_attributes ('tracers',nt,-1)
      dim_tracers = dims(ind)
 
      ind         = ind+1
      dims(ind)   = dim_attributes ('nchar',char_len,-1)
      dim_nchar   = dims(ind)
 
      ind         = ind+1
      dims(ind)   = dim_attributes ('d2',2,-1)
      dim_d2      = dims(ind)
 
 
!---------------------------------------------------------------------
!     do not allow dimensions to exceed maximum value
!---------------------------------------------------------------------
      if (ind > max_dims) then
        if (debug_ncdf) then
         call document ('dimensions'
     &,                 'Fatal error: '//' The number of dimensions'
     &                  //'defined exceeds max_dims.')
        endif
        call exit_POP ('dimension limit exceeded',normal_exit=.false.) 
      endif
 
 
 
!---------------------------------------------------------------------
!     create the dimensions, then set dim_xx%id, once the corresponding
!        dims(ind)%id have been set
!---------------------------------------------------------------------
      if (file%lncdf) then
 
         do nn=1,ind
            call def_dim_ncdf (file, dims(nn) ) 
         enddo
 
         call set_dim_ncdf (file)
      
      else
         ! add binary equivalent here
      endif
 
 
      end subroutine dimensions
 
 
 
      subroutine coordinates (file)
!     ======================
 
!---------------------------------------------------------------------
!     define coordinates (short_name, long_name, units, and id)
!       and write values to netcdf file
!---------------------------------------------------------------------
      integer (kind=int_kind)
     &, parameter                 ::
     &     max_coords  =  25                ! max no. of coordinates
 
      type (output_file)
     &, intent(in)                :: file
 
      integer (kind=int_kind)     :: ind 
     &,                              id 
     &,                              nn
 
      type (coord_attributes) 
     &, dimension(max_coords)     :: coord 
 
      real (kind=real_kind)
     &,  dimension(km)            :: WORK1D  ! single precision array
 
      real (kind=real_kind)
     &,  dimension(imt,jmt)       :: WORK2D  ! single precision array
                                             !    for 2d output
 
      real (kind=real_kind)
     &, dimension(imt_global,jmt_global) 
     &                            :: RTEMP2D_G
 
 
 
 
      ind = 0
 
 
!---------------------------------------------------------------------
!     define default values for coords
!---------------------------------------------------------------------
      do nn=1,max_coords
         coord(nn)%edges      = 'unknown_edges'
         coord(nn)%positive   = 'unknown_positive'
         coord(nn)%valid_min  = -1.0*flt_fill
         coord(nn)%valid_max  =  1.0*flt_fill
      enddo
 
 
!---------------------------------------------------------------------
!     define information for the creation of the coordinates
!---------------------------------------------------------------------
      ind = ind+1
      coord(ind)%req          = req_attributes (
     &                         'z_t'
     &,                        'depth from surface to midpoint of layer'
     &,                        'centimeters'
     &,                         1,-1,xtype_float)
      coord(ind)%len          = km
      coord(ind)%dim_ids(1)   = dim_z_t%id
      coord(ind)%positive     = 'down'
      coord(ind)%valid_min    = zt(1) 
      coord(ind)%valid_max    = zt(km)
 
      ind = ind+1
      coord(ind)%req          = req_attributes (
     &                          'z_w'
     &,                         'depth from surface to bottom of layer'
     &,                         'centimeters'
     &,                         1,-1,xtype_float)
      coord(ind)%len          = km
      coord(ind)%dim_ids(1)   = dim_z_w%id
      coord(ind)%positive     ='down'
      coord(ind)%valid_min    = c0         
      coord(ind)%valid_max    = zw(km-1)
 
 
      ind = ind+1
      coord(ind)%req          = req_attributes (
     &                          'ULAT'
     &,                         'array of u-grid latitudes'
     &,                         'degrees_north'
     &,                         2,-1,xtype_float)
      coord(ind)%len          = imt_global*jmt_global
      coord(ind)%dim_ids(1)   = dim_nlon%id
      coord(ind)%dim_ids(2)   = dim_nlat%id
      coord(ind)%valid_min    = flt_fill
      coord(ind)%valid_max    = flt_fill
 
      ind = ind+1
      coord(ind)%req          = req_attributes (
     &                          'ULONG'
     &,                         'array of u-grid longitudes'
     &,                         'degrees_east'  
     &,                         2,-1,xtype_float)
      coord(ind)%len          = imt_global*jmt_global
      coord(ind)%dim_ids(1)   = dim_nlon%id
      coord(ind)%dim_ids(2)   = dim_nlat%id
      coord(ind)%valid_min    = flt_fill
      coord(ind)%valid_max    = flt_fill
 
      ind = ind+1
      coord(ind)%req          = req_attributes (
     &                          'TLAT'
     &,                         'array of t-grid latitudes'
     &,                         'degrees_north'
     &,                         2,-1,xtype_float)
      coord(ind)%len          = imt_global*jmt_global
      coord(ind)%dim_ids(1)   = dim_nlon%id
      coord(ind)%dim_ids(2)   = dim_nlat%id
      coord(ind)%valid_min    = flt_fill
      coord(ind)%valid_max    = flt_fill
 
      ind = ind+1
      coord(ind)%req          = req_attributes (
     &                          'TLONG'
     &,                         'array of t-grid longitudes'
     &,                         'degrees_east'  
     &,                         2,-1,xtype_float)
      coord(ind)%len          = imt_global*jmt_global
      coord(ind)%dim_ids(1)   = dim_nlon%id
      coord(ind)%dim_ids(2)   = dim_nlat%id
      coord(ind)%valid_min    = flt_fill
      coord(ind)%valid_max    = flt_fill
 
 
 
!---------------------------------------------------------------------
!     do not allow coordinates to exceed maximum value
!---------------------------------------------------------------------
      if (ind > max_coords) then
        if (debug_ncdf) then
         call document ('coordinates'
     &,                 'Fatal error: '//' The number of coordinates'
     &                  //'defined exceeds max_coords.')
        endif
         call exit_POP ('coordinate limit exceeded',normal_exit=.false.)
      endif
 
 

 
!---------------------------------------------------------------------
!     create the coordinates and their attributes
!---------------------------------------------------------------------
      if (file%lncdf) then
         do nn=1,ind
 
            call def_coord_ncdf   (file,  coord(nn)    )
 
            id = coord(nn)%req%id
 
            call put_req_att_ncdf (file,  coord(nn)%req)
 
           if (trim(coord(nn)%edges) /= 'unknown_edges')
     &        call put_att_ncdf (file, id,      'edges'
     &,                                coord(nn)%edges )
 
           if (trim(coord(nn)%positive) /= 'unknown_positive')
     &        call put_att_ncdf (file, id,         'positive'
     &,                                   coord(nn)%positive )
    
           if (coord(nn)%valid_min /= flt_fill)
     &        call put_att_ncdf (file, id,  'valid_min' 
     &,                            coord(nn)%valid_min)
 
           if (coord(nn)%valid_max /= flt_fill)
     &        call put_att_ncdf (file, id,  'valid_max' 
     &,                            coord(nn)%valid_max)
 
    
             
         enddo
      else
      endif
 
!---------------------------------------------------------------------
!     write the coordinates to the output file
!---------------------------------------------------------------------
      if (file%lncdf) then
         do nn=1,ind
            select case (coord(nn)%req%short_name)
              case ('z_t')
                WORK1D = zt
                call put_var_ncdf(file,coord(nn)%req,WORK1D)
              case ('z_w')
                WORK1D(1)    = c0
                WORK1D(2:km) = zw(1:km-1)
                call put_var_ncdf(file,coord(nn)%req,WORK1D)
              case ('ULAT')
                WORK2D = ULAT*radian
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf(file,coord(nn)%req,RTEMP2D_G)
              case ('ULONG')
                WORK2D = ULONG*radian
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf(file,coord(nn)%req,RTEMP2D_G)
              case ('TLAT')
                WORK2D = TLAT*radian
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf(file,coord(nn)%req,RTEMP2D_G)
              case ('TLONG')
                WORK2D = TLONG*radian
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf(file,coord(nn)%req,RTEMP2D_G)
            end select
         enddo
      else
      endif

 
      end subroutine coordinates
 
 
 
      subroutine scalars (file)
!     ==================
 
!---------------------------------------------------------------------
!     define scalar attributes  (short_name, long_name, units, and id)
!       and write values to a file , for all scalars identified in
!       the scalar_contents file
!---------------------------------------------------------------------
 
      use time_management
 
      integer (kind=int_kind)
     &, parameter                 ::
     &     max_scalar  = 75                 ! max no. of scalars
 
      type (output_file)        
     &, intent(in)                :: file
 
      integer (kind=int_kind)     :: ind
     &,                              num_scalars
     &,                              nn
     &,                              nu
     &,                              vdum(1)
     &,                              mt
 
      type (scalar_attributes)       
     &, dimension(max_scalar)     :: scalar_info
 
 
!---------------------------------------------------------------------
!    read namelist for scalar_contents filename
!---------------------------------------------------------------------
 
      character (char_len) ::
     &  scalar_contents     ! filename for choosing scalars for output
 
      integer (kind=int_kind) ::
     &  nml_error           ! error flag for namelist i/o
 
      namelist /scalar_nml/ scalar_contents
 
      scalar_contents  = 'unknown_scalar_contents'
 
      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=scalar_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif
 
      call broadcast_scalar(nml_error, master_task)
 
      if (nml_error /= 0) then
        call exit_POP('reading scalar_nml',normal_exit=.false.)
      endif
 
      call broadcast_scalar(scalar_contents, master_task)
 
!---------------------------------------------------------------------
!    read scalar_contents 
!---------------------------------------------------------------------
      call get_unit(nu)
 
      call read_contents_header (nu,num_scalars,scalar_contents,.true.)
 
      if (num_scalars > max_scalar) then
        if (debug_ncdf) call document (
     &                   'scalars','Fatal error: The number of ' //
     &                   'scalars selected exceeds max_scalar.')
         call exit_POP ('scalar limit exceeded',normal_exit=.false.)
      endif
 


      nn = 0      ! may need to modify nn if leap years not allowed
      do ind = 1,num_scalars
         nn = nn + 1
         if (my_task == master_task) then
            read(nu,*) scalar_info(nn)%req%short_name
     &,                scalar_info(nn)%req%long_name
     &,                scalar_info(nn)%req%units
     &,                scalar_info(nn)%req%ndims
     &,                scalar_info(nn)%req%id
         endif
 
         mt = master_task
         call broadcast_scalar(scalar_info(nn)%req%short_name,mt)
         call broadcast_scalar(scalar_info(nn)%req%long_name ,mt)
         call broadcast_scalar(scalar_info(nn)%req%units     ,mt)
         call broadcast_scalar(scalar_info(nn)%req%ndims     ,mt)
         call broadcast_scalar(scalar_info(nn)%req%id        ,mt)
 
         scalar_info(nn)%req%type = xtype_double
 
         select case(trim(scalar_info(nn)%req%short_name))
            case('days_in_norm_year')
              scalar_info(nn)%value = float (days_in_norm_year)
            case('days_in_leap_year')
              if (allow_leapyear) then
                 scalar_info(nn)%value = float(days_in_leap_year)
              else
                 num_scalars = num_scalars - 1
                 nn = nn - 1
              endif 
            case('grav')
              scalar_info(nn)%value = grav
            case('omega')
              scalar_info(nn)%value = omega
            case('radius')
              scalar_info(nn)%value = radius
            case('cp_sw')
              scalar_info(nn)%value = cp_sw
            case('sound')
              scalar_info(nn)%value = sound
            case('vonkar')
              scalar_info(nn)%value = vonkar
            case('cp_air')
              scalar_info(nn)%value = cp_air
            case('rho_air')
              scalar_info(nn)%value = rho_air
            case('rho_sw')
              scalar_info(nn)%value = rho_sw
            case('rho_fw')
              scalar_info(nn)%value = rho_fw
            case('stefan_boltzmann')
              scalar_info(nn)%value = stefan_boltzmann
            case('latent_heat_vapor')
              scalar_info(nn)%value = latent_heat_vapor
            case('latent_heat_fusion')
              scalar_info(nn)%value = latent_heat_fusion
            case('ocn_ref_salinity')
              scalar_info(nn)%value = ocn_ref_salinity
            case('sea_ice_salinity')
              scalar_info(nn)%value = sea_ice_salinity
            case('T0_Kelvin')
              scalar_info(nn)%value = T0_Kelvin
            case('salt_to_ppt')
              scalar_info(nn)%value = salt_to_ppt
            case('ppt_to_salt')
              scalar_info(nn)%value = ppt_to_salt
            case('mass_to_Sv')
              scalar_info(nn)%value = mass_to_Sv
            case('heat_to_PW')
              scalar_info(nn)%value = heat_to_PW
            case('salt_to_Svppt')
              scalar_info(nn)%value = salt_to_Svppt
            case('salt_to_mmday')
              scalar_info(nn)%value = salt_to_mmday
            case('momentum_factor')
              scalar_info(nn)%value = momentum_factor
            case('hflux_factor')
              scalar_info(nn)%value = hflux_factor
            case('fwflux_factor')
              scalar_info(nn)%value = fwflux_factor
            case('salinity_factor')
              scalar_info(nn)%value = salinity_factor
            case('sflux_factor')
              scalar_info(nn)%value = sflux_factor
            case('nsurface_t')
              scalar_info(nn)%value = float(nsurface_t)
            case('nsurface_u')
              scalar_info(nn)%value = float(nsurface_u)
            case default
              scalar_info(nn)%value   = -1000000
            end select
            if (scalar_info(nn)%value == -1000000 ) then
              call exit_POP('unknown field in scalar_contents'
     &,                      normal_exit=.false.)
            endif
          enddo
 
        if (my_task == master_task) close(nu)
        call release_unit(nu)

 
      if (file%lncdf) then
 
         do nn=1,num_scalars
            call def_var_ncdf     (file,scalar_info(nn)%req, vdum)
            call put_req_att_ncdf (file,scalar_info(nn)%req)
         enddo
 
         do nn=1,num_scalars
            call put_var_ncdf (file,scalar_info(nn)%req
     &,                            scalar_info(nn)%value)
         enddo
      else
      endif
 
 
      end subroutine scalars
 
 
 
      subroutine time_invar (file)
!     =====================
 
!---------------------------------------------------------------------
!     define time-invariant quantities (KMU,KMT, masks, ULAT,ULONG,etc)
!       and write them to the output file
!---------------------------------------------------------------------
 
      use model_size
      use forcing_shf
       
      integer (kind=int_kind)
     &, parameter                 ::
     &     max_vars    =  30                ! max no. of variables
 
 
      type (output_file)
     &, intent(in)                :: file
 
      integer (kind=int_kind)     :: ind 
     &,                              nn
 
      integer (kind=int_kind)
     &, dimension(imt_global,jmt_global) 
     &                            :: ITEMP2D_G
 
      real (kind=real_kind)
     &, dimension(imt_global,jmt_global) 
     &                            :: RTEMP2D_G
 
      real (kind=real_kind)
     &,  dimension(0:km)          :: WORK1D  ! single precision array
 
      real (kind=real_kind)
     &,  dimension(imt,jmt)       :: WORK2D  ! single precision array
                                             !    for 2d output
      type (var_attributes) 
     &, dimension(max_vars)       :: var    
 
 
      ind = 0
 
!---------------------------------------------------------------------
!     define default values for time-invariant variables
!---------------------------------------------------------------------
      call set_var_defaults (file%lncdf,max_vars,var)
 
 
!---------------------------------------------------------------------
!     define information for the creation of the time-invariant variables
!---------------------------------------------------------------------
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'dz'
     &,                      'thickness of layer k'
     &,                      'centimeters'
     &,                       1,-1,xtype_float)
      var(ind)%dim_ids(1)   = dim_z_t%id
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'dzw'
     &,                      'midpoint of k to midpoint of k+1'
     &,                      'centimeters'
     &,                       1,-1,xtype_float)
      var(ind)%dim_ids(1)   = dim_z_w%id
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'KMT'
     &,                      'k Index of Deepest Grid Cell on T Grid' 
     &,                      'unitless'
     &,                       2,-1,xtype_int)
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'TLONG TLAT'
     
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'KMU'
     &,                      'k Index of Deepest Grid Cell on U Grid' 
     &,                      'unitless'
     &,                       2,-1,xtype_int)
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'ULONG ULAT'
 
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'REGION_MASK'
     &,                      'basin index number (signed integers)'
     &,                      'unitless'
     &,                       2,-1,xtype_int)
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'TLONG TLAT'
 
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'UAREA'
     &,                      'area of U cells'
     &,                      'centimeter^2'
     &,                       2,-1,xtype_float)  
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'ULONG ULAT'
 
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'TAREA'
     &,                      'area of T cells'
     &,                      'centimeter^2'
     &,                       2,-1,xtype_float)  
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'TLONG TLAT'
 
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'HU'
     &,                      'ocean depth at U points'
     &,                      'centimeter'
     &,                       2,-1,xtype_float)  
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'ULONG ULAT'
 
 
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'HT'
     &,                      'ocean depth at T points'
     &,                      'centimeter'
     &,                       2,-1,xtype_float)  
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'TLONG TLAT'
 
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'DXU'
     &,                      'x-spacing centered at U points'
     &,                      'centimeters'  
     &,                       2,-1,xtype_float)
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'ULONG ULAT'
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'DYU'
     &,                      'y-spacing centered at U points'
     &,                      'centimeters'  
     &,                       2,-1,xtype_float)
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'ULONG ULAT'
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                        'DXT'
     &,                       'x-spacing centered at T points'
     &,                       'centimeters'  
     &,                       2,-1,xtype_float)
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'TLONG TLAT'
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'DYT'
     &,                      'y-spacing centered at T points'
     &,                      'centimeters'  
     &,                       2,-1,xtype_float)
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'TLONG TLAT'
 
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'HTN'
     &,                      'cell widths on North sides of T cell'
     &,                      'centimeters'  
     &,                       2,-1,xtype_float)
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'TLONG TLAT'
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'HTE'
     &,                      'cell widths on East sides of T cell'
     &,                      'centimeters'  
     &,                       2,-1,xtype_float)
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'TLONG TLAT'
 
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'HUS'
     &,                      'cell widths on South sides of U cell'
     &,                      'centimeters'  
     &,                       2,-1,xtype_float)
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'ULONG ULAT'
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'HUW'
     &,                      'cell widths on West sides of U cell'
     &,                      'centimeters'  
     &,                       2,-1,xtype_float)
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'ULONG ULAT'
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'ANGLE'
     &,                      'angle grid makes with latitude line'
     &,                      'radians'
     &,                       2,-1,xtype_float)
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'ULONG ULAT'
 
      ind = ind+1
      var(ind)%req          = req_attributes (
     &                       'ANGLET'
     &,                      'angle grid makes with latitude line'
     &                     //' on T grid'
     &,                      'radians'
     &,                       2,-1,xtype_float)
      var(ind)%dim_ids(1)   = dim_nlon%id
      var(ind)%dim_ids(2)   = dim_nlat%id
      var(ind)%coordinates  = 'TLONG TLAT'
 
!---------------------------------------------------------------------
!     do not allow variables to exceed maximum value
!---------------------------------------------------------------------
      if (ind > max_vars) then
        if (debug_ncdf) then
         call document ('time_invar'
     &,                 'Fatal error: '//' The number of variables' 
     &                  //'defined exceeds max_var.')
        endif
         call exit_POP ('variable limit exceeded',normal_exit=.false.)
      endif
 
 
!---------------------------------------------------------------------
!     create the variables and their attributes 
!---------------------------------------------------------------------
      if (file%lncdf) then
         call define_var_att_ncdf (file,ind,var)
      else
      endif

 
!---------------------------------------------------------------------
!     write the variables
!---------------------------------------------------------------------
      if (file%lncdf) then
         do nn=1,ind
            select case (var(nn)%req%short_name)
              case ('dz')
                WORK1D(1:) = dz
                call put_var_ncdf(file,var(nn)%req,WORK1D(1:))
              case ('dzw')
                WORK1D(0:) = dzw
                call put_var_ncdf(file,var(nn)%req,WORK1D(0:))
              case ('KMT')
                call gather_global(KMT_G, KMT, master_task)
                call put_var_ncdf     (file,var(nn)%req,KMT_G)
              case ('KMU')
                call gather_global(KMU_G, KMU, master_task)
                call put_var_ncdf     (file,var(nn)%req,KMU_G)
              case ('REGION_MASK')
                call gather_global(ITEMP2D_G, REGION_MASK, master_task)
                call put_var_ncdf     (file,var(nn)%req,ITEMP2D_G)
              case ('UAREA')
                WORK2D = UAREA
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf     (file,var(nn)%req,RTEMP2D_G)
              case ('TAREA')
                WORK2D = TAREA
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf     (file,var(nn)%req,RTEMP2D_G)
              case ('HU')
                WORK2D = HU
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf     (file,var(nn)%req,RTEMP2D_G)
              case ('HT')
                WORK2D = HT
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf     (file,var(nn)%req,RTEMP2D_G)
              case ('DXU')
                WORK2D = DXU
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf(file,var(nn)%req,RTEMP2D_G)
              case ('DYU')
                WORK2D = DYU
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf(file,var(nn)%req,RTEMP2D_G)
              case ('DXT')
                WORK2D = DXT
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf(file,var(nn)%req,RTEMP2D_G)
              case  ('DYT')
                WORK2D = DYT
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf(file,var(nn)%req,RTEMP2D_G)
              case ('HTN')
                WORK2D = HTN
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf(file,var(nn)%req,RTEMP2D_G)
              case ('HTE')
                WORK2D = HTE
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf(file,var(nn)%req,RTEMP2D_G)
              case ('HUS')
                WORK2D = HUS
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf(file,var(nn)%req,RTEMP2D_G)
              case ('HUW')
                WORK2D = HUW
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf(file,var(nn)%req,RTEMP2D_G)
              case ('ANGLE')
                WORK2D = ANGLE
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf(file,var(nn)%req,RTEMP2D_G)
              case ('ANGLET')
                WORK2D = ANGLET
                call gather_global(RTEMP2D_G, WORK2D, master_task)
                call put_var_ncdf(file,var(nn)%req,RTEMP2D_G)
            end select
         enddo
      else
      endif
 
 
      end subroutine time_invar
 
 
 
      subroutine time_bounds (file)
!     ======================
 
      type (output_file)             
     &, intent(in)                :: file

 
!---------------------------------------------------------------------
!     define the time boundaries of the time-averaging interval
!---------------------------------------------------------------------
 
      time_bound%req%short_name = 'time_bound'
      time_bound%req%long_name  = 'boundaries for time-averaging '
     &                             //'interval'
      time_bound%req%units      = 'days since 0000-01-01 00:00:00'
      time_bound%req%ndims      =  2
      time_bound%req%type       =  xtype_double
      time_bound%req%id         =  -1
      time_bound%dim_ids(1)     =  dim_d2%id
      time_bound%dim_ids(2)     =  time_info%dim%id
 
      if (file%lncdf) then
          call def_var_ncdf (file, time_bound%req,time_bound%dim_ids)
 
          call put_att_ncdf (file, time_bound%req%id, 'long_name'
     &,                            time_bound%req%long_name)
 
          call put_att_ncdf (file, time_bound%req%id, 'units'
     &,                            time_bound%req%units)     
      else
         ! add binary equivalent here
      endif
 
 
      end subroutine time_bounds
 
 
 
      subroutine write_time (file,time,num_netcdf_writes)
!     ===================== 
 
      type (output_file)          :: file
      real (kind=dbl_kind)        :: time
      integer (kind=int_kind)     :: num_netcdf_writes
 
      intent(in)                  :: file,time,num_netcdf_writes
 
      if (file%lncdf) then
        call put_time_ncdf (file, 'time',time_info%vid,num_netcdf_writes
     &,                     1,time)
      else
      endif
      
 
      end subroutine write_time
 
 
 
      subroutine write_time_bound (file,num_writes)
!     =========================== 
 
!----------------------------------------------------------------------
!       write lower and upper time-averaging limits
!----------------------------------------------------------------------
      type (output_file)          :: file
      integer (kind=int_kind)     :: num_writes
      intent (in)                 :: file, num_writes
 
      integer (kind=int_kind)
     &, dimension(2)              :: beg, cnt
 
 
 
      if (file%lncdf) then
 
        call end_define_ncdf (file, 'write_time_bound')
 
!----------------------------------------------------------------------
!       write lower time bound
!----------------------------------------------------------------------
        beg(1) = 1
        beg(2) = num_writes
         
        cnt(1) = 1
        cnt(2) = 1
 
        call put_var_ncdf (file,time_bound%req,beg,cnt,time_bound_lower)
 
!----------------------------------------------------------------------
!       write upper time bound
!----------------------------------------------------------------------
        beg(1) = 2
         
        call put_var_ncdf (file,time_bound%req,beg,cnt,time_bound_upper)
      else
      endif
      
 
      end subroutine write_time_bound
 
 
 
      subroutine set_outvar_dim (nvar,var_att)
!     ========================= 
 
!---------------------------------------------------------------------
!     now that the dimensions nlon, nlat, etc are defined, and the   
!       grid-codes are known for each variable, set the dimensions  
!       (lat,lon,z,time) of the 2D & 3D output variables
!---------------------------------------------------------------------
      integer (kind=int_kind)
     &,   intent(in)              ::  nvar
      integer (kind=int_kind)     ::  nn
     &,                               ndims
      type (var_attributes)       
     &,   dimension(nvar)      
     &,   intent(inout)           ::  var_att
 
 
      do nn = 1,nvar
 
        ndims = var_att(nn)%req%ndims
 
!---------------------------------------------------------------------
!     test for error in grid code
!---------------------------------------------------------------------
        if (ndims < 2) then 
          call document ('set_outvar_dim'
     &,                  'Fatal error: '//' The number of dimensions'
     &                   //'is less than two.')
         call exit_POP ('incorrect ndims value',normal_exit=.false.)
        endif
 
!---------------------------------------------------------------------
!     the first dimension is longitude
!---------------------------------------------------------------------
        if (ndims >= 2) then 
          var_att(nn)%dim_ids(1) = dim_nlon%id
          var_att(nn)%dim_ids(2) = time_info%vid
        endif
 
!---------------------------------------------------------------------
!     the second dimension is latitude
!---------------------------------------------------------------------
        if(ndims >= 3) then 
          var_att(nn)%dim_ids(2) = dim_nlat%id
          var_att(nn)%dim_ids(3) = time_info%vid
        endif
 
 
!---------------------------------------------------------------------
!     the third dimension is depth; determine if it indicates
!       the "T" grid or the "W" grid
!---------------------------------------------------------------------
        if (ndims == 4) then 
          select case( var_att(nn)%grid_code(4:4))   ! k-grid
            case('1' ) 
              var_att(nn)%dim_ids(3) = dim_z_t%id
            case('2' ) 
              var_att(nn)%dim_ids(3) = dim_z_w%id
             case default
              var_att(nn)%dim_ids(3) = -1000
           end select
          var_att(nn)%dim_ids(4) = time_info%vid
        endif
    
      enddo
 
 
      end subroutine set_outvar_dim
 
 
 
      subroutine read_contents_header (nu,num_fields,filename,newfile)
!     ===============================
 
        integer (kind=int_kind)   :: nu
        character (*)             :: filename
        logical (kind=log_kind)   :: newfile
 
        intent (in)               :: nu,filename,newfile
 
        integer (kind=int_kind)
     &,   intent(out)             :: num_fields
 
!---------------------------------------------------------------------
!       read the number of output fields from the 'xxx_contents' files
!---------------------------------------------------------------------
 
 
      if (my_task == master_task) then
         if (newfile) open(nu, file=filename, status='old')
         read(nu,*) num_fields
      endif
 
      call broadcast_scalar (num_fields, master_task)
 
 
      end subroutine read_contents_header
 
 
 
      subroutine read_contents (nu,num_fields,attributes,close_contents)
!     ========================
 
!---------------------------------------------------------------------
!       read the tavg_contents and history_contents files
!---------------------------------------------------------------------
 
 
      integer (kind=int_kind)     :: mt
     &,                              nn
     &,                              nu 
     &,                              num_fields
 
      type (var_attributes)      
     &,  dimension(num_fields)    :: attributes
 
      logical (kind=log_kind)     :: close_contents
 
      intent (in)                 :: nu,num_fields,close_contents
      intent (out)                :: attributes
 
      do nn = 1,num_fields
        if (my_task == master_task) then
           read (nu,*) attributes(nn)%grid_code
     &,                attributes(nn)%req%short_name
     &,                attributes(nn)%req%long_name
     &,                attributes(nn)%req%units
        endif
 
        mt = master_task
        call broadcast_scalar(attributes(nn)%grid_code     ,mt)
        call broadcast_scalar(attributes(nn)%req%short_name,mt)
        call broadcast_scalar(attributes(nn)%req%long_name ,mt)
        call broadcast_scalar(attributes(nn)%req%units     ,mt)
 
        attributes(nn)%req%type = xtype_float
 
        call decode_grid (attributes(nn))
 
        select case(trim(attributes(nn)%req%short_name))
           case('SALT')
              attributes(nn)%scale_factor = 1.0e3
              attributes(nn)%req%units    = 'gram/kilogram'
        end select

       enddo
 
      if (close_contents) then
        if (my_task == master_task) close(nu)
        call release_unit(nu)
      endif
 
 
      end subroutine read_contents
 
 
 
      subroutine test_contents (num_fields,attributes,contents_name)
!     ========================
 
!---------------------------------------------------------------------
!       Test the xxx_contents files for invalid choices.  The boundary-
!       layer variables cannot be tested until vmix_itype and 
!       vmix_type_kpp can be made available to this module...
!---------------------------------------------------------------------
 
 
      character (char_len)        :: error_msg
      character (*)               :: contents_name
      integer (kind=int_kind)     :: contents_error
     &,                              mt
     &,                              nn
     &,                              num_fields
     &,                              error
     &,                              no_error
 
      type (var_attributes)      
     &,  dimension(num_fields)    :: attributes
 
 
      intent (in)                 :: num_fields
     &,                              attributes
     &,                              contents_name
 
      error    = 0
      no_error = -1000
 
      do nn = 1,num_fields
        if (my_task == master_task) then
          select case(trim(attributes(nn)%req%short_name))
            case('HMXL','TMXL','XMXL','HBLT','TBLT','XBLT')
*****************----- temporary code ------******************
                  contents_error = no_error
*****************---------------------------******************
********      if (vmix_itype == vmix_type_kpp) then
*                 contents_error = no_error
*             else
*                 contents_error = error
*                 error_msg = 'vmix_itype must equal vmix_type_kpp '
*    &                      //'when selecting '
*    &                      //'HMXL,TMXL,XMXL,HBLT,TBLT,or XBLT'
********      endif
            case default
              contents_error = no_error
          end select
 
        endif

        call broadcast_scalar(contents_error,master_task)
 
        if (contents_error /= no_error ) then
          call document (contents_name
     &,     trim(attributes(nn)%req%short_name)
     &      // ' is inconsistent with selected model options')
          call exit_POP(contents_name//' selection',normal_exit=.false.)
        endif
      enddo
 
 
      end subroutine test_contents
 
 
 
      subroutine decode_grid (var_att)
!     ====================== 
 
!---------------------------------------------------------------------
!     translate grid information from input dataset to 'ULONG','TLONG',
!       etc, and add this information to the coordinates attribute
!---------------------------------------------------------------------
      type (var_attributes)   
     &, intent (inout)            :: var_att
 
!---------------------------------------------------------------------
!     determine if the variable is 2- or 3-dimensional
!       (then add one to to account for time as the unlimited dimension)
!---------------------------------------------------------------------
      select case( var_att%grid_code(1:1) ) ! no. of dimensions
 
        case('1' ) 
          var_att%req%ndims = 2    
        case('2' ) 
          var_att%req%ndims = 3
        case('3' ) 
          var_att%req%ndims = 4
        case default
          call exit_POP('(decode_grid): unexpected number of dimensions'
     &                   //'  -- '//var_att%grid_code(1:1) 
     &,                  normal_exit=.false.)
      end select
 
 
 
!---------------------------------------------------------------------
!     the first dimension is longitude; determine if the grid code
!       indicates the "T" grid or the "U" grid
!---------------------------------------------------------------------
 
      select case( var_att%grid_code(2:2))   ! i-grid
        case('1' ) 
          var_att%coordinates = trim('TLONG')
        case('2' ) 
          var_att%coordinates = trim('ULONG')
        case('3' ) 
          var_att%coordinates = trim('TLONG')
        case default
          call exit_POP('(decode_grid) unknown longitude grid'
     &                   //' value -- '//var_att%grid_code(2:2)
     &,                  normal_exit=.false.)
      end select
 
 
!---------------------------------------------------------------------
!     the second dimension is latitude; determine if it indicates
!       the "T" grid or the "U" grid
!---------------------------------------------------------------------
      select case( var_att%grid_code(3:3))   ! j-grid
         case('1' ) 
           var_att%coordinates = trim(var_att%coordinates)//' TLAT'
         case('2' ) 
           var_att%coordinates = trim(var_att%coordinates)//' ULAT'
         case('3' ) 
           var_att%coordinates = trim(var_att%coordinates)//' TLAT'
         case default
           call exit_POP('(decode_grid) unknown latitude grid'
     &                   //' value -- '//var_att%grid_code(3:3) 
     &,                  normal_exit=.false.)
      end select
 
 
!---------------------------------------------------------------------
!     the third dimension is depth; determine if it indicates
!       the "T" grid or the "W" grid
!---------------------------------------------------------------------
      if (var_att%req%ndims == 4) then 
         select case( var_att%grid_code(4:4))   ! k-grid
            case('1' ) 
               var_att%coordinates = trim(var_att%coordinates)//' z_t'
            case('2' ) 
               var_att%coordinates = trim(var_att%coordinates)//' z_w'
             case default
               call exit_POP('(decode_grid) unknown vertical grid '
     &                   //' value -- '//var_att%grid_code(4:4)  
     &,                  normal_exit=.false.)
             end select
      endif
 
 
!---------------------------------------------------------------------
!     the last dimension is always time 
!---------------------------------------------------------------------
      var_att%coordinates = trim(var_att%coordinates)//' time'

 
!---------------------------------------------------------------------
!     now determine the "mask code" which dictates how the output field
!       should be masked with a special value before being written
!       to a netcdf file.
!       presently, we only mask quantities on the standard tracer
!       and standard velocity grids.
!---------------------------------------------------------------------
 
      select case( var_att%grid_code(2:2))       ! i-grid
 
        case('1' ) 
           select case( var_att%grid_code(3:3))  ! j-grid
               case('1')
                  var_att%mask_code = '1-1'      ! standard tracer grid
               case default
                  var_att%mask_code = 'no_masking'
               end select
                
        case('2' ) 
           select case( var_att%grid_code(3:3))  ! j-grid
               case('2')
                  var_att%mask_code = '2-2'      ! standard velocity grid
               case default
                  var_att%mask_code = 'no_masking'
               end select
        case default
                  var_att%mask_code = 'no_masking'
                
      end select
 
      end subroutine decode_grid
 
 
 
      subroutine set_var_defaults (use_ncdf,num_vars,var)
!     ===========================
 
!---------------------------------------------------------------------
!     set some default values for variables of the type var_attributes.
!---------------------------------------------------------------------
 
      integer (kind=int_kind)     :: nn
     &,                              num_vars
 
      logical (kind=log_kind)     :: use_ncdf
 
      type (var_attributes) 
     &, dimension(num_vars)       :: var    
 
      intent (in)                 :: use_ncdf,num_vars
      intent (inout)              :: var
 
 
      do nn=1,num_vars
         var(nn)%flux_direction = 'unknown_flux_direction'
         var(nn)%coordinates    = 'unknown_coordinates'
         var(nn)%cell_methods   = 'unknown_cell_methods'
         var(nn)%add_offset     = flt_fill   
         var(nn)%scale_factor   = flt_fill   
         var(nn)%req%type       = xtype_unknown
      enddo
 
 
      end subroutine set_var_defaults 
 
 
 
 
 
 
      subroutine open_file (file)
!     ====================
 
!---------------------------------------------------------------------
!     open existing netcdf or binary file
!---------------------------------------------------------------------
      type (output_file)      
     &, intent (inout)            :: file
 
 
      if (file%lncdf) then
         call open_file_ncdf (file)
      else
         ! add binary equivalent here
      endif
 
 
      end subroutine open_file
 
 
 
      subroutine close_file (file)
!     =====================
 
!---------------------------------------------------------------------
!     close existing netcdf file
!---------------------------------------------------------------------
 
      type (output_file)      
     &, intent (in)               :: file
 
 
 
      if (file%lncdf) then
        call close_file_ncdf (file)
      else
        if (my_task == master_task) close(file%id)
      endif
 
 
      end subroutine close_file
 
 
 
!     =====================
      end module io_general
!     =====================

