      Module io_ncdf
!     ==============

!-----------------------------------------------------------------------
!  This module contains a collection of subroutines used to generate
!    netcdf history or tavg files
!
!   CVS:$Id: io_ncdf.F,v 1.1.4.9 2002/05/10 22:42:47 njn01 Exp $
!   CVS:$Name: ccsm2_0_beta46 $
!-----------------------------------------------------------------------

 
      use grid
      use global_reductions
      use types
      use io_tools
      use shr_sys_mod
      
      implicit none
 
 
      include 'netcdf.inc'
 

      integer (kind=int_kind)
     &, dimension(imt_global,jmt_global) 
     &                            :: KMT_G
     &,                              KMU_G
 
 
 
      interface put_att_ncdf
         module procedure  put_att_text_ncdf
     &,                    put_att_dbl_ncdf
     &,                    put_att_real_ncdf
     &,                    put_att_int_ncdf
      end interface 
 
 
      interface put_var_ncdf
         module procedure  put_all_var_dbl_0d_ncdf
     &,                    put_all_var_dbl_1d_ncdf
     &,                    put_all_var_dbl_2d_ncdf
     &,                    put_all_var_dbl_3d_ncdf
     &,                    put_all_var_real_0d_ncdf
     &,                    put_all_var_real_1d_ncdf
     &,                    put_all_var_real_2d_ncdf
     &,                    put_all_var_real_3d_ncdf
     &,                    put_all_var_int_0d_ncdf
     &,                    put_all_var_int_1d_ncdf
     &,                    put_all_var_int_2d_ncdf
     &,                    put_all_var_int_3d_ncdf
     &,                    put_part_var_dbls_ncdf
     &,                    put_part_var_dbl_ncdf
     &,                    put_part_var_reals_ncdf
     &,                    put_part_var_ints_ncdf
      end interface 
 
      interface mask_lev
         module procedure  mask_lev_real
     &,                    mask_lev_dbl
      end interface 
 
      save
 
      contains
 
 
 
      subroutine init_io_ncdf
!     =======================
      
 
!---------------------------------------------------------------------
!      initialize variables needed for netcdf
!---------------------------------------------------------------------
 
 
 
!---------------------------------------------------------------------
!     define fill values, nf_float/nf_double, etc.  
!---------------------------------------------------------------------
 
 
      dbl_fill      = NF_FILL_DOUBLE
      flt_fill      = NF_FILL_FLOAT
      int_fill      = NF_FILL_INT               
      xtype_unlim   = NF_UNLIMITED
      xtype_int     = NF_INT   
      xtype_float   = NF_FLOAT
      xtype_double  = NF_DOUBLE
      xtype_global  = NF_GLOBAL
      no_error_ncdf = NF_NOERR
 
 
      end subroutine init_io_ncdf
 
 
 
      subroutine create_file_ncdf (file)
!     ===========================
 
!---------------------------------------------------------------------
!     create new netcdf file 
!---------------------------------------------------------------------
 
      type    (output_file)       :: file
 
 
      if (my_task == master_task) then   
        ier_ncdf = nf_create (trim(file%name),NF_CLOBBER,file%id)
      endif
 
      call broadcast_scalar (ier_ncdf,master_task)
      call broadcast_scalar (file%id ,master_task)
 
      if (debug_ncdf) then
        call document('create_file', 'file%id = ', file%id)
      endif
 
      call test_ier_ncdf   (ier_ncdf, 'create_file'
     &,                    'Creating new netcdf file: ' //     
     &                      trim(file%name) )
      file%define_on = .true.
 
 
      end subroutine create_file_ncdf
 
 
 
      subroutine write_array_ncdf (file, var_info, ARRAY, klevs)
!     ===========================
 
!-----------------------------------------------------------------------
!     This routine is the netcdf equivalent of subroutine write_array,
!        which writes a variable to an output file
!-----------------------------------------------------------------------
      type (output_file)          :: file
      type (var_attributes)       :: var_info
      type (req_attributes)       :: req_info
      integer (kind=int_kind)     :: klevs
     &,                              klev
      real (kind=real_kind)
     &,  dimension(imt,jmt,klevs) :: ARRAY
 
      intent (in)                 :: var_info,ARRAY,klevs
 
      real (kind=real_kind)       ! global storage array for netcdf output
     &, dimension(imt_global,jmt_global,klevs) :: RTEMP_G
 
 
!--------------------------------------------------------------------
!     Collect local fields into one global array, then write
!      array to netcdf file  (most efficient for writing 3D netcdf fields)
!---------------------------------------------------------------------
 
      do klev=1,klevs
         call gather_global(RTEMP_G(:,:,klev),ARRAY(:,:,klev)
     &,                     master_task)
         call mask_lev     (RTEMP_G(:,:,klev), klev, var_info)
      enddo               
 
      req_info = var_info%req
 
      call put_var_ncdf(file,req_info,RTEMP_G)   
 
     
      end subroutine write_array_ncdf
 
 
      subroutine mask_lev_real (A_G, klev, var)
!     ======================== 
 
!---------------------------------------------------------------------
!     mask out land values in a global array before writing to 
!       output file
!     
!     note: presently, this routine only masks fields on the
!           pure tracer or pure velocity grids; fields on hybrid
!           grids are not yet masked
!---------------------------------------------------------------------
 
      type (var_attributes)       :: var
      real (kind=real_kind)    
     &, dimension(imt_global,jmt_global) 
     &                            :: A_G          ! global array for netcdf output
      real (kind=real_kind)       :: fill
      integer (kind=int_kind)     :: klev         ! vertical level index
     &,                              i,ip1,j      ! local indices

      intent (in)                 :: klev
 
!---------------------------------------------------------------------
!     determine if the variable is on a standard tracer or standard
!       velocity grid.  if so, mask accordingly.  if not, do not
!       mask.
!---------------------------------------------------------------------
 
!---------------------------------------------------------------------
!     because of a deficiency in the ncview program, any variable with
!       a scale_factor must have its _FillValue and missing_value 
!       divided by the scale_factor; otherwise, the ncview program 
!       fails to properly plot the variable. 
!       Also see subroutine define_var_att_ncdf.
!---------------------------------------------------------------------
 
      if (var%scale_factor  /=  flt_fill  .and.
     &    var%scale_factor  /=  c0       ) then 
         fill = flt_fill/var%scale_factor
      else
         fill = flt_fill
      endif
 
!---------------------------------------------------------------------
!     apply the mask to a global array, on the master_task only
!---------------------------------------------------------------------
      if (my_task == master_task) then
 
       select case( var%mask_code(1:3) ) 
 
        case('1-1' )       
 
!---------------------------------------------------------------------
!     mask variable on standard tracer grid
!---------------------------------------------------------------------
 
           A_G = merge (fill, A_G, klev > KMT_G)
          
        case('2-2' )       
 
!---------------------------------------------------------------------
!     mask variable on standard velocity grid. must treat boundaries
!       separately
!---------------------------------------------------------------------
 
           do j=1,jmt_global - 1
              do i=1,imt_global - 1
                 if (klev > KMU_G(i,j) ) then
                    if (klev > KMT_G(i  ,j  )     .and.
     &                  klev > KMT_G(i+1,j  )     .and.
     &                  klev > KMT_G(i  ,j+1)     .and.
     &                  klev > KMT_G(i+1,j+1) )    then
                        A_G(i,j) = fill
                     endif
                 endif
              enddo
           enddo
   
           A_G(:,jmt_global) = fill
 
           do j=1,jmt_global - 1
              i   = imt_global 
              ip1 = 1
                 if (klev > KMU_G(i,j) ) then
                    if (klev > KMT_G(i  ,j  )     .and.
     &                  klev > KMT_G(ip1,j  )     .and.
     &                  klev > KMT_G(i  ,j+1)     .and.
     &                  klev > KMT_G(ip1,j+1) )    then
                        A_G(i,j) = fill
                     endif
                 endif
           enddo
           
        case default
          if (debug_ncdf) then
          write(stdout,1000)'mask_lev_real','note: no masking for '
     &    //trim(var%req%long_name)//' in the netcdf dataset'
          call shr_sys_flush(stdout)
          endif
       end select
 
!---------------------------------------------------------------------
!     end master_task region
!---------------------------------------------------------------------
      endif
 
1000  format(5x,'(',a,')', a )
 
      end subroutine mask_lev_real
 
 
      subroutine mask_lev_dbl (A_G, klev, var)
!     ======================= 
 
!---------------------------------------------------------------------
!     mask out land values in a global array before writing to 
!       output file
!     
!     note: presently, this routine only masks fields on the
!           pure tracer or pure velocity grids; fields on hybrid
!           grids are not yet masked
!---------------------------------------------------------------------
 
      type (var_attributes)       :: var
      real (kind=dbl_kind)    
     &, dimension(imt_global,jmt_global) 
     &                            :: A_G          ! global array for netcdf output
      real (kind=dbl_kind)        :: fill
      integer (kind=int_kind)     :: klev         ! vertical level index
     &,                              i,ip1,j      ! local indices
 

      intent (in)                 :: klev

!---------------------------------------------------------------------
!     because of a deficiency in the ncview program, any variable with
!       a scale_factor must have its _FillValue and missing_value 
!       divided by the scale_factor; otherwise, the ncview program 
!       fails to properly plot the variable. 
!       Also see subroutine define_var_att_ncdf.
!---------------------------------------------------------------------
 
      if (var%scale_factor  /=  dbl_fill  .and.
     &    var%scale_factor  /=  c0            ) then 
         fill = dbl_fill/var%scale_factor
      else
         fill = dbl_fill
      endif
 
 
!---------------------------------------------------------------------
!     determine if the variable is on a standard tracer or standard
!       velocity grid.  if so, mask accordingly.  if not, do not
!       mask.
!---------------------------------------------------------------------
 
!---------------------------------------------------------------------
!     apply the mask to a global array, on the master_task only
!---------------------------------------------------------------------
      if (my_task == master_task) then
 
       select case( var%mask_code(1:3) ) 
 
        case('1-1' )       
 
!---------------------------------------------------------------------
!     mask variable on standard tracer grid
!---------------------------------------------------------------------
 
           A_G = merge (fill, A_G, klev > KMT_G)
          
        case('2-2' )       
 
!---------------------------------------------------------------------
!     mask variable on standard velocity grid. must treat boundaries
!       separately
!---------------------------------------------------------------------
 
           do j=1,jmt_global - 1
              do i=1,imt_global - 1
                 if (klev > KMU_G(i,j) ) then
                    if (klev > KMT_G(i  ,j  )     .and.
     &                  klev > KMT_G(i+1,j  )     .and.
     &                  klev > KMT_G(i  ,j+1)     .and.
     &                  klev > KMT_G(i+1,j+1) )    then
                        A_G(i,j) = fill
                     endif
                 endif
              enddo
           enddo
   
           A_G(:,jmt_global) = fill
 
           do j=1,jmt_global - 1
              i   = imt_global 
              ip1 = 1
                 if (klev > KMU_G(i,j) ) then
                    if (klev > KMT_G(i  ,j  )     .and.
     &                  klev > KMT_G(ip1,j  )     .and.
     &                  klev > KMT_G(i  ,j+1)     .and.
     &                  klev > KMT_G(ip1,j+1) )    then
                        A_G(i,j) = fill
                     endif
                 endif
           enddo
           
        case default
          if (debug_ncdf) then
          write(stdout,1000)'mask_lev_dbl','note: no masking for '
     &    //trim(var%req%long_name)//' in the netcdf dataset'
          call shr_sys_flush(stdout)
          endif
       end select
 
!---------------------------------------------------------------------
!     end master_task region
!---------------------------------------------------------------------
      endif
 
1000  format(5x,'(',a,')', a )
 
      end subroutine mask_lev_dbl
 
 
      subroutine define_var_att_ncdf (file, num_vars, var)
!     ==============================
 
!---------------------------------------------------------------------
!     define all attributes for variables of type (var_attributes)
!---------------------------------------------------------------------
 
      integer (kind=int_kind)     :: id
     &,                              num_vars
     &,                              nn
 
      type (output_file)          :: file
      type (var_attributes) 
     &, dimension(num_vars)       :: var    
 
      real (kind=dbl_kind)        :: dbl_FillValue
     &,                              dbl_missing
 
      real (kind=real_kind)       :: flt_FillValue
     &,                              flt_missing
 
      intent (in)                 :: num_vars
      intent (inout)              :: var
 
 
        do nn=1,num_vars
 
           call def_var_ncdf     (file, var(nn)%req,var(nn)%dim_ids)

           id = var(nn)%req%id
 
           call put_req_att_ncdf (file, var(nn)%req )
 

 
           if (trim(var(nn)%flux_direction) /= 'unknown_flux_direction')
     &        call put_att_ncdf (file, id, 'flux_direction' 
     &,                            var(nn)%flux_direction )
 
 
           if (trim(var(nn)%coordinates) /= 'unknown_coordinates' )
     &        call put_att_ncdf (file, id, 'coordinates' 
     &,                            var(nn)%coordinates )
 
 
           if (trim(var(nn)%cell_methods) /= 'unknown_cell_methods'.and.
     &         trim(var(nn)%cell_methods) /= ' ')  
     &        call put_att_ncdf (file, id, 'cell_methods'
     &,                            var(nn)%cell_methods)
 
 
           if (var(nn)%add_offset  /=  flt_fill ) 
     &        call put_att_ncdf (file, id, 'add_offset'
     &,                             var(nn)%add_offset )
      
 
           if (var(nn)%scale_factor  /=  flt_fill ) 
     &        call put_att_ncdf  (file, id, 'scale_factor'
     &,                             var(nn)%scale_factor) 

           dbl_FillValue = dbl_fill
           dbl_missing   = dbl_fill
           flt_FillValue = flt_fill
           flt_missing   = flt_fill
 
!--------------------------------------------------------------
!       scale the following to ensure that any (real) variable with
!       a scale_factor is properly plotted by ncview
!--------------------------------------------------------------
           if (var(nn)%scale_factor  /=  flt_fill   .and.
     &         var(nn)%scale_factor  /=  c0        ) then 
              dbl_FillValue = dbl_fill/var(nn)%scale_factor
              dbl_missing   = dbl_fill/var(nn)%scale_factor
              flt_FillValue = flt_fill/var(nn)%scale_factor
              flt_missing   = flt_fill/var(nn)%scale_factor
           endif
 
           if (var(nn)%req%type == xtype_float) then
              call put_att_ncdf(file, id, '_FillValue'   ,flt_FillValue)
              call put_att_ncdf(file, id, 'missing_value',flt_missing)
           elseif (var(nn)%req%type == xtype_double) then
              call put_att_ncdf(file, id, '_FillValue'   ,dbl_FillValue)
              call put_att_ncdf(file, id, 'missing_value',dbl_missing)
           elseif (var(nn)%req%type == xtype_int   ) then
              call put_att_ncdf(file, id, '_FillValue'   ,int_fill)
              call put_att_ncdf(file, id, 'missing_value',int_fill)
           endif

 
        enddo
 
 
      end subroutine define_var_att_ncdf
 
 
 
      subroutine def_dim_ncdf (file,dim_info)
!     =======================
 
!---------------------------------------------------------------------
!     create dimensions  (nlon, nlat, tracers, z_t, and z_w)
!---------------------------------------------------------------------
 
      type (output_file)          :: file
      type (dim_attributes)       :: dim_info
 
      intent (inout)              :: dim_info
 
 
      call redefine_ncdf (file)
 
      if (my_task == master_task) then
        ier_ncdf = nf_def_dim (file%id
     &,                        trim(dim_info%name)
     &,                        dim_info%len 
     &,                        dim_info%id)
      endif
 
      call broadcast_scalar (ier_ncdf   , master_task)
 
      call test_ier_ncdf   (ier_ncdf, 'def_dim_ncdf'
     &,                    'Defining dimension: '//trim(dim_info%name) )
 
      call broadcast_scalar (dim_info%id, master_task)
 
 
      end subroutine def_dim_ncdf
 
 
 
      subroutine def_var_ncdf (file,req_info,vdims)
!     =======================
 
!---------------------------------------------------------------------
!     create the variable req_info%short_name
!---------------------------------------------------------------------
 
      type (output_file)          :: file
      type (req_attributes)       :: req_info
 
      integer (kind=int_kind)     :: vdims(req_info%ndims)
 
      intent (in)                 :: vdims   
      intent (inout)              :: req_info
 
 
      call redefine_ncdf (file)
 
      if (my_task == master_task) then
        ier_ncdf = nf_def_var (file%id
     &,                        trim(req_info%short_name)
     &,                        req_info%type
     &,                        req_info%ndims
     &,                        vdims
     &,                        req_info%id)
      endif
 
      call broadcast_scalar (ier_ncdf   ,master_task)
 
      call test_ier_ncdf    (ier_ncdf, 'def_var_ncdf'
     &,                     'Defining variable: '
     &                    // trim(req_info%short_name) )
 
      call broadcast_scalar (req_info%id,master_task)
 
 
      end subroutine def_var_ncdf
 
 
 
      subroutine def_coord_ncdf (file,coord_info)
!     =========================
 
!---------------------------------------------------------------------
!     create coordinates: ULAT,ULON, etc
!---------------------------------------------------------------------
 
 
      type (output_file)          :: file
      type (coord_attributes)     :: coord_info
      integer (kind=int_kind)     :: varid
      
      intent (inout)              :: coord_info
 
 
      call redefine_ncdf (file)
 
      if (my_task == master_task) then
        ier_ncdf = nf_def_var (file%id
     &,                        trim(coord_info%req%short_name)
     &,                        coord_info%req%type
     &,                        coord_info%req%ndims
     &,                        coord_info%dim_ids 
     &,                        varid )
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf    (ier_ncdf, 'def_coord_ncdf'
     &,                    'Defining coordinate: '
     &                      //trim(coord_info%req%short_name)  )
 
 
      call broadcast_scalar (varid, master_task)
 
      coord_info%req%id = varid
 
 
      end subroutine def_coord_ncdf
 
 

      subroutine set_dim_ncdf (file)
!     =======================
 
      type (output_file)
     &, intent(in)                :: file
 
      integer (kind=int_kind)     :: vid
 
!---------------------------------------------------------------------
!     set dim_xx%id values via the netcdf nf_inq_dimid function
!---------------------------------------------------------------------
      
      if (my_task == master_task) then
          fid = file%id
          ier_ncdf         = nf_inq_dimid (fid, 'nlon'    , vid)
          dim_nlon%id      = vid
          ier_ncdf         = nf_inq_dimid (fid, 'nlat'    , vid) 
          dim_nlat%id      = vid
          ier_ncdf         = nf_inq_dimid (fid, 'z_t'     , vid) 
          dim_z_t%id       = vid
          ier_ncdf         = nf_inq_dimid (fid, 'z_w'     , vid) 
          dim_z_w%id       = vid
          ier_ncdf         = nf_inq_dimid (fid, 'tracers' , vid) 
          dim_tracers%id   = vid
          ier_ncdf         = nf_inq_dimid (fid, 'nchar'   , vid) 
          dim_nchar%id     = vid
          ier_ncdf         = nf_inq_dimid (fid, 'd2'      , vid) 
          dim_d2%id        = vid
      endif
 
      call broadcast_scalar (dim_nlon%id   ,master_task)
      call broadcast_scalar (dim_nlat%id   ,master_task)
      call broadcast_scalar (dim_z_t%id    ,master_task)
      call broadcast_scalar (dim_z_w%id    ,master_task)
      call broadcast_scalar (dim_tracers%id,master_task)
      call broadcast_scalar (dim_nchar%id  ,master_task)
      call broadcast_scalar (dim_d2%id     ,master_task)
 
 
      end subroutine set_dim_ncdf
 
 
 
      subroutine put_req_att_ncdf (file,req_info)
!     ===========================
 
!---------------------------------------------------------------------
!     write the required attributes long_name and units
!---------------------------------------------------------------------
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      integer (kind=int_kind)     :: id
      character (char_len)        :: lname
      
      intent (inout)              :: req_info
 
 
      id  = req_info%id 
 
      if (trim(req_info%long_name) /= 'unknown_longname' )
     &    call put_att_ncdf(file,id,'long_name' 
     &,                 trim( req_info%long_name )  )
 

      if (trim(req_info%units) /= 'unknown_units' .and. 
     &    trim(req_info%units) /= 'unitless') 
     &    call put_att_ncdf(file,id,'units'     ,req_info%units     )
 
 
      end subroutine put_req_att_ncdf
 
 
 
      subroutine put_att_text_ncdf (file,loc_varid,attribute,text)
!     ============================
 
      type (output_file)          :: file
      integer (kind=int_kind)     :: loc_varid
      character(*)                :: attribute
     &,                              text
 
      intent (in)                 :: loc_varid,attribute,text
 
 
      call redefine_ncdf (file)
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_att_text(file%id
     &,                            loc_varid
     &,                            trim(attribute) 
     &,                            len_trim(text)
     &,                            trim(text)        )
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf    (ier_ncdf, 'put_att_text_ncdf'
     &,                     'Writing attribute text values for '
     &                      //trim(attribute) )
 
 
      end subroutine put_att_text_ncdf
 
 
 
      subroutine put_att_dbl_ncdf (file,loc_varid,attribute,value)
!     ===========================
 
      type (output_file)          :: file
      integer (kind=int_kind)     :: loc_varid
     &,                              len
      character (*)                  attribute  
      real (kind=dbl_kind)        :: value
                                                  
      intent (in)                 :: attribute,loc_varid,value
 
 
      len = 1
 
      call redefine_ncdf (file)
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_att_double (file%id
     &,                               loc_varid
     &,                               trim(attribute) 
     &,                               NF_FLOAT 
     &,                               len
     &,                               value )
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf    (ier_ncdf, 'put_att_dbl_ncdf'
     &,                      'Writing attribute double values: for '
     &                      //trim(attribute) )
 
 
      end subroutine put_att_dbl_ncdf
 
 
 
      subroutine put_att_real_ncdf (file,loc_varid,attribute,value)
!     ============================
 
      type (output_file)          :: file
      integer (kind=int_kind)     :: loc_varid
      real (kind=real_kind)       :: value
 
      integer (kind=int_kind)     :: len
      character (*)                  attribute  
 
      intent (in)                  :: loc_varid,attribute,value
 
 
      len = 1
 
      call redefine_ncdf (file)
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_att_real   (file%id
     &,                               loc_varid
     &,                               trim(attribute) 
     &,                               NF_FLOAT 
     &,                               len
     &,                               value )
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf    (ier_ncdf, 'put_att_real_ncdf'
     &,                      'Writing attribute real values for: '
     &                       //trim(attribute) )
 
 
      end subroutine put_att_real_ncdf
 
 
 
      subroutine put_att_int_ncdf (file,loc_varid,attribute,value)
!     ============================
 
      type (output_file)          :: file
      integer (kind=int_kind)     :: loc_varid
      integer (kind=int_kind)     :: value
 
      integer (kind=int_kind)     :: len
      character (*)                  attribute  
 
      intent (in)                  :: loc_varid,attribute,value
 
 
      len = 1
 
      call redefine_ncdf (file)
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_att_int    (file%id
     &,                               loc_varid
     &,                               trim(attribute) 
     &,                               NF_INT    
     &,                               len
     &,                               value )
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf    (ier_ncdf, 'put_att_int_ncdf'
     &,                      'Writing attribute integer values for: '
     &                       //trim(attribute) )
 
 
      end subroutine put_att_int_ncdf
 
 
 
      subroutine put_all_var_dbl_0d_ncdf (file,req_info,value) 
!     ==================================
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put a scalar value of kind=dbl_kind into netcdf file
!---------------------------------------------------------------------
 
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      real (kind=dbl_kind)        :: value          
 
      intent (in)                 :: req_info,value
 
!---------------------------------------------------------------------
!     error testing -- ensure that the defined variable type matches
!                      the actual type
!---------------------------------------------------------------------
      call type_test ('put_all_var_dbl_0d_ncdf'
     &,                req_info%type, xtype_double)
 
 
      call end_define_ncdf (file, 'put_all_var_dbl_0d_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_var_double(file%id,req_info%id,value)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf,'put_all_var_dbl_0d_ncdf'
     &,                  'Writing single value: '
     &                 // trim(req_info%short_name) )

 
      end subroutine put_all_var_dbl_0d_ncdf    
 
 
 
      subroutine put_all_var_dbl_1d_ncdf (file,req_info,values)
!     ==================================
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put a 1-D array of values of kind=dbl_kind into netcdf file
!---------------------------------------------------------------------
 
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      real (kind=dbl_kind)        :: values(:)       
 
      intent (in)                 :: req_info,values
 
 
      call type_test ('put_all_var_dbl_1d_ncdf'
     &,                req_info%type, xtype_double)
 
 
      call end_define_ncdf (file, 'put_all_var_dbl_1d_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_var_double(file%id,req_info%id,values)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_all_var_dbl_1d_ncdf'
     &,                  'Writing array of values: '
     &                 // trim(req_info%short_name) )

 
      end subroutine put_all_var_dbl_1d_ncdf    
 
 
 
      subroutine put_all_var_dbl_2d_ncdf (file,req_info,values)
!     ================================== 
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put an entire 2D array of (kind=dbl_kind) vaules into netcdf file
!---------------------------------------------------------------------
      
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      real (kind=dbl_kind)        :: values(:,:)    
 
      intent (in)                 :: req_info,values
 
      call type_test ('put_all_var_dbl_2d_ncdf'
     &,                req_info%type, xtype_double)
     
 
 
      call end_define_ncdf (file, 'put_all_var_dbl_2d_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_var_double(file%id,req_info%id,values)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
 
      call test_ier_ncdf (ier_ncdf,  'put_all_var_dbl_2d_ncdf'
     &,                  'Writing 2D array :'
     &                 // trim(req_info%short_name) )

 
 
      end subroutine put_all_var_dbl_2d_ncdf
 
 
 
      subroutine put_all_var_dbl_3d_ncdf (file,req_info,values)
!     ================================== 
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put an entire 3D array of (kind=dbl_kind) vaules into netcdf file
!---------------------------------------------------------------------
 
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      real (kind=dbl_kind)        :: values(:,:,:)  
 
      intent (in)                 :: req_info,values
      
 
      call type_test ('put_all_var_dbl_3d_ncdf'
     &,                req_info%type, xtype_double)
 
      call end_define_ncdf (file, 'put_all_var_dbl_3d_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_var_double(file%id,req_info%id,values)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_all_var_dbl_2d_ncdf'
     &,                  'Writing 3D array: '
     &                 // trim(req_info%short_name) )

 
 
      end subroutine put_all_var_dbl_3d_ncdf
 
 
 
      subroutine put_all_var_real_0d_ncdf (file,req_info,value) 
!     ===================================
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put single scalar value of kind=real_kind into netcdf file
!---------------------------------------------------------------------
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      real (kind=real_kind)       :: value
 
      intent (in)                 :: req_info,value
 
 
      call type_test ('put_all_var_real_0d_ncdf'
     &,                req_info%type, xtype_float  )
     
 
      call end_define_ncdf (file, 'put_all_var_real_0d_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_var_real(file%id,req_info%id,value)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_all_var_real_0d_ncdf'
     &,                  'Writing single value: '
     &                 // trim(req_info%short_name) )
 
 
      end subroutine put_all_var_real_0d_ncdf   
 
 
 
      subroutine put_all_var_real_1d_ncdf (file,req_info,values)
!     ===================================
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put value of kind=real_kind into netcdf file
!---------------------------------------------------------------------
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      real (kind=real_kind)       :: values(:)
 
      intent (in)                 :: req_info,values
      
 
      call type_test ('put_all_var_real_1d_ncdf'
     &,                req_info%type, xtype_float  )
     
 
 
      call end_define_ncdf (file, 'put_all_var_real_1d_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_var_real(file%id,req_info%id,values)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_all_var_real_1d_ncdf'
     &,                  'Writing 1D array of values: '
     &                 // trim(req_info%short_name) )
 
 
      end subroutine put_all_var_real_1d_ncdf   
 
 
 
      subroutine put_all_var_real_2d_ncdf (file,req_info,values)
!     =================================== 
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put an entire 2D array of (kind=real_kind) vaules into netcdf file
!---------------------------------------------------------------------
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      real (kind=real_kind)       :: values(:,:)
 
      intent (in)                 :: req_info,values
      
 
     
      call type_test ('put_all_var_real_2d_ncdf'
     &,                req_info%type, xtype_float  )
 
 
      call end_define_ncdf (file, 'put_all_var_real_2d_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_var_real (file%id,req_info%id,values)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_all_var_real_2d_ncdf'
     &,                  'Writing 2D array: '
     &                 // trim(req_info%short_name) )
 
 
      end subroutine put_all_var_real_2d_ncdf
 
 
 
      subroutine put_all_var_real_3d_ncdf (file,req_info,values)
!     =================================== 
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put an entire 3D array of (kind=real_kind) vaules into netcdf file
!---------------------------------------------------------------------
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      real (kind=real_kind)       :: values(:,:,:)
 
      intent (in)                 :: req_info,values
      
     
      call type_test ('put_all_var_real_3d_ncdf'
     &,                req_info%type, xtype_float  )
 
 
      call end_define_ncdf (file, 'put_all_var_real_3d_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_var_real (file%id,req_info%id,values)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_all_var_real_3d_ncdf'
     &,                 'Writing 3D array: '//trim(req_info%short_name))
 
 
      end subroutine put_all_var_real_3d_ncdf
 
 
 
      subroutine put_all_var_int_0d_ncdf (file,req_info,value)
!     ================================== 
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put scalar (kind=int_kind) value into netcdf file
!---------------------------------------------------------------------
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      integer (kind=int_kind)     :: value
 
      intent (in)                 :: req_info,value
 
      call type_test ('put_all_var_int_0d_ncdf'
     &,                req_info%type, xtype_int )
 
 
      call end_define_ncdf (file, 'put_all_var_int_0d_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_var_int(file%id,req_info%id,value)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_all_var_int_0d_ncdf'
     &,                  'Writing scalar integer: '
     &                 // trim(req_info%short_name) )

 
      end subroutine put_all_var_int_0d_ncdf
 
 
 
      subroutine put_all_var_int_1d_ncdf (file,req_info,values)
!     ================================== 
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put (kind=int_kind) value into netcdf file
!---------------------------------------------------------------------
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      integer (kind=int_kind)     :: values(:)
 
      intent (in)                 :: req_info,values
 
      call type_test ('put_all_var_int_1d_ncdf'
     &,                req_info%type, xtype_int )
 
 
      call end_define_ncdf (file, 'put_all_var_int_1d_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_var_int(file%id,req_info%id,values)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_all_var_int_1d_ncdf'
     &,                  'Writing 1-D array '
     &                 // trim(req_info%short_name) )

 
 
      end subroutine put_all_var_int_1d_ncdf
 
 

      subroutine put_all_var_int_2d_ncdf (file,req_info,values)
!     ================================== 
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put an entire 2D array of (kind=int_kind) values into netcdf file
!---------------------------------------------------------------------
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      integer (kind=int_kind)     :: values(:,:)
 
      intent (in)                 :: req_info,values
 
 
      call type_test ('put_all_var_int_2d_ncdf'
     &,                req_info%type, xtype_int )
 
 
      call end_define_ncdf (file, 'put_all_var_int_2d_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_var_int(file%id,req_info%id,values)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_all_var_int_2d_ncdf'
     &,                  'Writing 2-D integer array: '
     &                 // trim(req_info%short_name) )

 
 
      end subroutine put_all_var_int_2d_ncdf
 
 
 
      subroutine put_all_var_int_3d_ncdf (file,req_info,values)
!     ================================== 
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put an entire 3D array of (kind=int_kind) values into netcdf file
!---------------------------------------------------------------------
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      integer (kind=int_kind)     :: values(:,:,:)
 
      intent (in)                 :: req_info,values
      
 
      call type_test ('put_all_var_int_3d_ncdf'
     &,                req_info%type, xtype_int )
 
 
      call end_define_ncdf (file, 'put_all_var_int_3d_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_var_int(file%id,req_info%id,values)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_all_var_int_3d_ncdf'
     &,                  'Writing 3-D integer array: '
     &                 // trim(req_info%short_name) )

 
      end subroutine put_all_var_int_3d_ncdf
 
 
 
      subroutine put_part_var_dbls_ncdf (file,req_info,beg,cnt,values)
!     ================================= 
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put a subregion of an array of (kind=dbl_kind) vaules into 
!       a netcdf file
!---------------------------------------------------------------------
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      integer   (kind=int_kind)   :: beg(:)   
     &,                              cnt(:)   
      real      (kind=dbl_kind)   :: values(:)
 
      intent (in)                 :: req_info,beg,cnt,values
 
 
      call type_test ('put_part_var_dbls_ncdf' 
     &,                req_info%type, xtype_double)
 
 
      call end_define_ncdf (file, 'put_part_var_dbls_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf=nf_put_vara_double(file%id,req_info%id,beg,cnt,values)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_part_var_dbls_ncdf'
     &,                  'Writing array: '
     &                 // trim(req_info%short_name) )
 
 
      end subroutine put_part_var_dbls_ncdf
 
 
 
      subroutine put_part_var_dbl_ncdf (file,req_info,beg,cnt,value)
!     ================================ 
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put a subregion of an array of (kind=dbl_kind) vaules into 
!       a netcdf file
!---------------------------------------------------------------------
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      integer   (kind=int_kind)   :: beg(:)
     &,                              cnt(:)
      real      (kind=dbl_kind)   :: value
 
      intent (in)                 :: req_info,beg,cnt,value 
      
 
      call type_test ('put_part_var_dbl_ncdf'  
     &,                req_info%type, xtype_double)
 
 
      call end_define_ncdf (file, 'put_part_var_dbl_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf=nf_put_vara_double(file%id,req_info%id,beg,cnt,value)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_part_var_dbl_ncdf'
     &,                  'Writing array: '
     &                 // trim(req_info%short_name) )

 
 
      end subroutine put_part_var_dbl_ncdf
 
 
 
      subroutine put_part_var_reals_ncdf (file,req_info,beg,cnt,values)
!     ================================== 
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put a subregion of an array of (kind=real_kind) vaules into 
!       a netcdf file
!---------------------------------------------------------------------
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      integer   (kind=int_kind)   :: beg(:)   
     &,                              cnt(:)   
      real      (kind=real_kind)  :: values(:)
 
      intent (in)                 :: req_info,beg,cnt,values
 
 
      call type_test ('put_part_var_reals_ncdf'
     &,                req_info%type, xtype_float) 
 
 
      call end_define_ncdf (file, 'put_part_var_reals_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_vara_real(file%id,req_info%id,beg,cnt,values)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_part_var_reals_ncdf'
     &,                  'Writing array: '
     &                 // trim(req_info%short_name) )

 
      end subroutine put_part_var_reals_ncdf
 
 

      subroutine put_part_var_ints_ncdf (file,req_info,beg,cnt,values)
!     ================================= 
 
!---------------------------------------------------------------------
!     interface: put_var_ncdf
!     put a subregion of an array of (kind=int_kind) vaules into 
!       a netcdf file
!---------------------------------------------------------------------
      type (output_file)          :: file
      type (req_attributes)       :: req_info
      integer   (kind=int_kind)   :: beg(:)   
     &,                              cnt(:)   
     &,                              values(:)
 
      intent (in)                 :: req_info,beg,cnt,values
 
      call type_test ('put_part_var_ints_ncdf'
     &,                req_info%type, xtype_int  ) 
 
 
      call end_define_ncdf (file, 'put_part_var_ints_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf = nf_put_vara_int(file%id,req_info%id,beg,cnt,values)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_part_var_ints_ncdf'
     &,                  'Writing array: '
     &                 // trim(req_info%short_name) )
 
 
      end subroutine put_part_var_ints_ncdf
 
 
 
      subroutine def_time_ncdf (file)
!     ========================
 
!---------------------------------------------------------------------
!     define the required time attributes 
!---------------------------------------------------------------------
      type    (output_file)   
     &, intent (in)               :: file
 
 
      if (my_task == master_task) then
          ier_ncdf = nf_def_var(file%id
     &,                         trim(time_info%dim%name)
     &,                         time_info%type
     &,                         1
     &,                         time_info%dim%id
     &,                         time_info%vid)
      endif
          
      call broadcast_scalar (ier_ncdf     ,master_task)
      call broadcast_scalar (time_info%vid,master_task)
 
      call test_ier_ncdf   (ier_ncdf, 'unlimited'
     &,                    'Defining time (unlimited) var: '
     &                     // trim(time_info%dim%name)  )
        
!---------------------------------------------------------------------
!     define attributes of the time coordinate
!---------------------------------------------------------------------
      call put_att_ncdf (file, time_info%dim%id, 'long_name'
     &,                  time_info%dim%name)
      call put_att_ncdf (file, time_info%dim%id, 'units'
     &,                  time_info%units)
      call put_att_ncdf (file, time_info%dim%id, 'bounds'
     &,                  time_info%bounds)
      call put_att_ncdf (file, time_info%dim%id, 'calendar'
     &,                  time_info%calendar)
 
 
 
      end subroutine def_time_ncdf
 
 
 
      subroutine put_time_ncdf (file,short_name,vid,beg,cnt,value)
!     ========================
 
!---------------------------------------------------------------------
!     no interface
!     put a subregion of an array of (kind=dbl_kind) vaules into 
!       a netcdf file, when the array is not a variable of type
!       var_attributes
!---------------------------------------------------------------------
      type (output_file)          :: file
      character(*)                :: short_name
      integer   (kind=int_kind)   :: beg
     &,                              cnt
     &,                              vid
      real      (kind=dbl_kind)   :: value
 
      intent (in)                 :: vid,short_name,beg,cnt,value 
      
 
      call end_define_ncdf (file, 'put_time_ncdf')
 
      if (my_task == master_task) then
        ier_ncdf=nf_put_vara_double(file%id,vid,beg,cnt,value)
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      call test_ier_ncdf (ier_ncdf, 'put_part_var_dbl_ncdf'
     &,                  'Writing array: '
     &                 // trim(short_name) )

 
      end subroutine put_time_ncdf
 
 
 
      subroutine redefine_ncdf (file)
!     ========================
!-----------------------------------------------------------------------
!     Enter or re-enter netCDF define mode
!-----------------------------------------------------------------------

      type (output_file)          :: file
 
 
      if (my_task == master_task) then
        if (.not. file%define_on) then
          ier_ncdf = nf_redef  (file%id)
        else
          ier_ncdf = no_error_ncdf
        endif
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      if (ier_ncdf == no_error_ncdf) then
         file%define_on = .true.
         if (debug_ncdf) call document ('redefine_ncdf'
     &,                                 'Re-entering define mode')
      else
         call error_stop_ncdf ('redefine_ncdf',ier_ncdf
     &,                       'Re-entering define mode')
      endif
 
 
      end subroutine redefine_ncdf
 
 
 
      subroutine end_define_ncdf (file, calling_routine_name)
!     ==========================
 
!-----------------------------------------------------------------------
!     Exit netCDF define mode
!-----------------------------------------------------------------------
      character (*)               :: calling_routine_name
      type (output_file)          :: file
 
 
      if (my_task == master_task) then
        if (file%define_on) then
          ier_ncdf = nf_enddef (file%id)
        else
          ier_ncdf = no_error_ncdf
        endif
      endif
 
      call broadcast_scalar (ier_ncdf, master_task)
 
      if (ier_ncdf == no_error_ncdf) then
         file%define_on = .false.
         if (debug_ncdf) call document ('end_define_ncdf'  
     &,                 'called from subroutine '//calling_routine_name)
         if (debug_ncdf) call document ('end_define_ncdf'  
     &,                  'leaving define mode & entering put mode')
      else
         call document ('end_define_ncdf'  
     &,                 'called from subroutine '//calling_routine_name)
         call error_stop_ncdf ('end_define_ncdf',ier_ncdf
     &,                       'Re-entering define mode')
      endif
 
 
      end subroutine end_define_ncdf
 
 
 
      subroutine open_file_ncdf (file)
!     =========================
 
!---------------------------------------------------------------------
!     open existing netcdf file
!---------------------------------------------------------------------
      type (output_file)      
     &, intent (inout)            :: file
 
 
      if (my_task == master_task) then
         ier_ncdf = nf_open  (trim(file%name),NF_WRITE,file%id)
      endif
 
      if (debug_ncdf) call document('close_file','file%id = ',file%id)
 
      call broadcast_scalar (ier_ncdf,master_task)
 
      call test_ier_ncdf  (ier_ncdf, 'open_file'
     &,                  ' Opening netcdf file '//trim(file%name))
 
      call broadcast_scalar (file%id ,master_task)
 
 
      end subroutine open_file_ncdf
 
 
 
      subroutine close_file_ncdf (file)
!     ==========================
 
!---------------------------------------------------------------------
!     close existing netcdf file
!---------------------------------------------------------------------
 
      type (output_file)      
     &, intent (in)               :: file
 
 
      if (my_task == master_task) ier_ncdf = nf_close (file%id)
 

      call broadcast_scalar (ier_ncdf,master_task)
 
      call test_ier_ncdf   (ier_ncdf, 'close_file'
     &,                    'Closing netcdf file '//trim(file%name))
 
 
      end subroutine close_file_ncdf
 
 
 
      subroutine test_ier_ncdf (ier, subname, operation_name)
!     ========================
 
      integer   (kind=int_kind)   :: ier
      character (*)               :: subname
     &,                              operation_name
 
      intent (in)                 :: subname,operation_name
 
 
      if (ier == no_error_ncdf) then
         if (debug_ncdf) call document (subname, operation_name)
      else
         call error_stop_ncdf (subname,ier,operation_name)
      endif
 
 
      end subroutine test_ier_ncdf
 
 
 
      subroutine error_stop_ncdf (subname,ier,message)
!     ==========================
 
      use exit_mod
      integer (kind=int_kind)     :: ier
      character (*)               :: subname, message
 
      intent (in)                 :: subname,ier,message
      
 
      call document (subname, 'Fatal error: '//nf_strerror(ier) )
      call exit_POP (message, normal_exit=.false.)
 
 
      end subroutine error_stop_ncdf 
 
 
      
 
      subroutine type_test (subname,specified_type, data_type)
 
      character (*)               :: subname
 
      integer (kind=int_kind)     :: specified_type
     &,                              data_type
 
!---------------------------------------------------------------------
!     error testing -- ensure that the defined variable type matches
!                      the actual type
!---------------------------------------------------------------------
     
      if (specified_type /= data_type  ) then
          call document (subname, 'type mismatch')
          call document (subname, 'specified type = ', specified_type)
          call document (subname, 'actual data type = ', data_type)
          call exit_POP (subname,  normal_exit=.false.)
      endif
 
      end subroutine type_test
 
 
!     ==================
      end module io_ncdf
!     ==================

