!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module output

!***********************************************************************
!
!     contains necessary routines, variables for large model output
!     files - restart, history, movies, drifter, time average files
!
!     CVS:$Id: output.F,v 1.2.2.6 2001/12/21 21:24:36 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use time_management
      use restart
      use history
      use movie
      use tavg

      implicit none
      save

!-----------------------------------------------------------------------
!
!     time flag ids for output flags
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  out_stop_now           ! flag id for stop_now flag

      integer (kind=int_kind),private ::
     &  cpl_write_restart      ! flag id for restart-file signal from cpl
     &, cpl_write_history      ! flag id for history-file signal from cpl
     &, cpl_write_tavg         ! flag id for    tavg-file signal from cpl

!***********************************************************************

      contains

!***********************************************************************

      subroutine output_driver

!-----------------------------------------------------------------------
!
!     Write restart dumps, data/avg/movie/history/drifter archives.
!
!-----------------------------------------------------------------------

      implicit none

      logical (kind=log_kind) ::
     &  lrestart_write,
     &  levenodd_write,
     &  lhistory_write,
     &  lmovie_write,
     &  ltavg_write

      character (char_len) ::
     &  file_suffix        ! suffix to append to root filename

!-----------------------------------------------------------------------
!
!     check to see whether it is time to write files
!
!-----------------------------------------------------------------------

      lrestart_write = .false.
      levenodd_write = .false.
      lhistory_write = .false.
      lmovie_write   = .false.
      ltavg_write    = .false.

      file_suffix = char_blank

!-----------------------------------------------------------------------
!
!     write restart files if code is stopping for any reason
!
!-----------------------------------------------------------------------

      if (check_time_flag(out_stop_now) .and. 
     &    (lrestart_on .or. leven_odd_on)) then
        file_suffix = '.end'
        call create_suffix(freq_opt_nsecond,file_suffix
     &,                                     in_suffix=file_suffix)
        call write_restart (file_suffix)
        if (ltavg_on) then
        ! create suffix with yyyy-mm-dd-sssss format for lreset_tavg .f.
          call create_suffix(freq_opt_nsecond,file_suffix)
          call write_tavg_dump(file_suffix,.false.)
        endif
      endif

!-----------------------------------------------------------------------
!
!     first check even/odd output
!
!-----------------------------------------------------------------------

      if (leven_odd_on .and. .not. check_time_flag(out_stop_now)) then
        levenodd_write = time_to_do(freq_opt_nstep, even_odd_freq)

        if (levenodd_write) then
          if (last_even_odd == even) then
            file_suffix(1:4) = '.odd'
            last_even_odd = odd
          else
            file_suffix(1:5) = '.even'
            last_even_odd = even
          endif
          call create_suffix(freq_opt_nsecond,file_suffix
     &,                             in_suffix=file_suffix)
          call write_restart (file_suffix)
          if (ltavg_on) then
           ! create suffix with yyyy-mm-dd-sssss format for lreset_tavg .f.
             call create_suffix(freq_opt_nsecond,file_suffix)
             call write_tavg_dump(file_suffix,.false.)
          endif
        endif
      endif

!-----------------------------------------------------------------------
!
!     check regular restart output
!
!-----------------------------------------------------------------------

      if (lrestart_on) then
        lrestart_write = check_time_flag(restart_flag)  .or.
     &                  (check_time_flag(cpl_write_restart) .and. eod)

        if (lrestart_write) then
          call create_suffix(freq_opt_nsecond,file_suffix
     &,                             in_suffix=file_suffix)
          call write_restart (file_suffix) 
          if (ltavg_on) then
           ! create suffix with yyyy-mm-dd-sssss format for lreset_tavg .f.
             call create_suffix(freq_opt_nsecond,file_suffix)
             call write_tavg_dump(file_suffix,.false.)
          endif
        endif
 
 
      endif
 
      if (check_time_flag(cpl_write_restart) .and. eod)
     & call set_time_flag(cpl_write_restart, .false.)

!-----------------------------------------------------------------------
!
!     write tavg output if it is time
!     ignore cpl instructions for now
!-----------------------------------------------------------------------

      if (ltavg_on) then
        ltavg_write = time_to_do(tavg_freq_iopt, tavg_freq) 
!    &         .or.  (check_time_flag(cpl_write_tavg) .and. eod)

        if (ltavg_write) then
        ! create suffix with tavg_freq_iopt format for lreset_tavg .t.
          call create_suffix(tavg_freq_iopt, file_suffix)
          call write_tavg_dump(file_suffix,.true.)
        endif
 
      endif

      if (check_time_flag (cpl_write_tavg) .and. eod)
     & call set_time_flag (cpl_write_tavg, .false.)
 
!-----------------------------------------------------------------------
!
!     write history files
!
!-----------------------------------------------------------------------

      if (lhistory_on) then
        lhistory_write = check_time_flag(history_flag)  
!    &           .or.    check_time_flag(cpl_write_history)

        if (lhistory_write) then
          call create_suffix(freq_opt_nsecond, file_suffix)
          call write_history(file_suffix,ltavg_on)
        endif
      endif

      if (check_time_flag (cpl_write_history) .and. eod)
     & call set_time_flag (cpl_write_history, .false.)
 
!-----------------------------------------------------------------------
!
!     write movie files
!
!-----------------------------------------------------------------------

      if (lmovie_on) then
        lmovie_write = time_to_do(movie_freq_iopt, movie_freq)

        if (lmovie_write) then
          call create_suffix(movie_freq_iopt, file_suffix)
          call write_movie(file_suffix)
        endif
      endif

!-----------------------------------------------------------------------

      end subroutine output_driver

!***********************************************************************


      subroutine create_suffix(in_freq_opt, out_suffix, in_suffix)

!-----------------------------------------------------------------------
!
!     determine suffix to append to files based on frequency option
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  in_freq_opt           ! frequency of output

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      character (char_len), intent(out) ::
     &  out_suffix           ! suffix to append to root filename

      character (char_len), intent(in), optional    ::
     &  in_suffix             ! existing suffix to be concatenated with
                              ! the suffix created here (out_suffix)
 
!-----------------------------------------------------------------------
!
!     local variable
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  cindx, cindx2         ! indices into character strings
     &, len_date

      character (char_len) ::
     &  char_temp          ! temp character space (for removing spaces)


!-----------------------------------------------------------------------
!
!     determine ccsm required date string from frequency option
!
!-----------------------------------------------------------------------


      out_suffix = char_blank

      select case (in_freq_opt)
      case (freq_opt_nyear)
        char_temp = 'y'

      case (freq_opt_nmonth)
        char_temp = 'ym'

      case (freq_opt_nday)
        char_temp = 'ymd'

      case (freq_opt_nhour)
        char_temp = 'ymds'

      case (freq_opt_nsecond)
        char_temp = 'ymds'

      case (freq_opt_nstep)
        char_temp = 'ymds'
 
      case default
        char_temp = 'ymds'
      end select
 
      call ccsm_date_stamp (ccsm_date_string, char_temp)
 
      out_suffix = trim(ccsm_date_string)
 
      if (present (in_suffix)) then
         if (in_suffix .ne. char_blank) then
           out_suffix = out_suffix // trim (in_suffix)
         endif
      endif
 
 
!-----------------------------------------------------------------------

      end subroutine create_suffix


!***********************************************************************

      subroutine init_output

!-----------------------------------------------------------------------
!
!     initializes frequency of output and filenames for
!     various files by calling individual initialization routines
!
!-----------------------------------------------------------------------

 
!-----------------------------------------------------------------------
!
!     get integer id's for stop flag, and history- and restart-writing 
!     signals from the coupler
!
!-----------------------------------------------------------------------
      out_stop_now      = init_time_flag('stop_now',default=.false.)
      cpl_write_restart = init_time_flag('cpl_write_restart' )
      cpl_write_history = init_time_flag('cpl_write_history' )
      cpl_write_tavg    = init_time_flag('cpl_write_tavg'    )

      call init_history
      call init_movie

!-----------------------------------------------------------------------

      end subroutine init_output

!***********************************************************************

      end module output

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
