!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module prognostic

!***********************************************************************
!
!     This module contains all the prognostic variables used in POP.
!
!     CVS:$Id: prognostic.F,v 1.2.2.2 2001/08/10 19:50:27 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants

      implicit none
      save

!-----------------------------------------------------------------------
!
!     3d prognostic variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,km,nt,3) ::
     &     TRACER         ! 3d tracer fields       at 3 time levels

      real (kind=dbl_kind), dimension(imt,jmt,km,3) ::
     &     UVEL,          ! 3d horizontal velocity at 3 time levels
     &     VVEL,          ! 3d horizontal velocity at 3 time levels
     &     RHO            ! 3d density fields,     at 3 time levels

!-----------------------------------------------------------------------
!
!     2d barotropic fields
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,3) ::
     &     UBTROP, VBTROP,     ! barotropic velocities, 3 time levels 
     &     PSURF,              ! surface pressure,      3 time levels
     &     GRADPX, GRADPY      ! surface-pressure gradient  "    "

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &     PGUESS,             ! next guess for surface pressure
     &     ZX,ZY,              ! vertically integrated forcing terms
     &     DH,DHU,             ! time change of surface height minus
                               ! freshwater flux an T, U points
     &     FW,FW_OLD,          ! freshwater flux at T points (cm/s)
                               ! FW_OLD is at time n-1
     &     FW_FREEZE           ! water flux at T points due to frazil
                               ! ice formation (cm/s)

      real (kind=dbl_kind), dimension(imt,jmt,nt) ::
     &     TFW                 ! tracer content in freshwater flux

!-----------------------------------------------------------------------
!
!     time indices for prognostic arrays
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &     curtime,            ! current time level  (n) 
     &     newtime,            ! next time level     (n+1)
     &     oldtime,            ! previous time level (n-1)
     &     mixtime             ! set to oldtime on leafrog steps
                               ! and to curtime on matsuno steps

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_prognostic

!-----------------------------------------------------------------------
!
!     This subroutine simply intializes prognostic arrays to zero.
!     In the future, this subroutine will allocate the prognostic
!     arrays as well.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     allocate space for prognostic arrays
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     initialize prognostic arrays to zero - they will be filled
!     later with real values from restart or initialization.
!     initialize time indices.
!
!-----------------------------------------------------------------------

      oldtime = 1
      curtime = 2
      newtime = 3

      TRACER  = c0
      UVEL    = c0
      VVEL    = c0
      RHO     = c0
      UBTROP  = c0
      VBTROP  = c0
      PSURF   = c0
      GRADPX  = c0
      GRADPY  = c0
      PGUESS  = c0
      ZX      = c0
      ZY      = c0
      DH      = c0
      DHU     = c0
      FW      = c0
      FW_OLD  = c0
      FW_FREEZE = c0
      TFW     = c0

!-----------------------------------------------------------------------

      end subroutine init_prognostic

!***********************************************************************

      end module prognostic

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
