!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||


      module qflux_mod

!***********************************************************************

!-----------------------------------------------------------------------
!
!     This module computes running time-averages of qflux
!    
!
!     CVS:$Id: qflux_mod.F,v 1.1.4.7 2002/02/22 22:54:07 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use grid
      use exit_mod
      use io_tools
      use types
      use time_management
      use shr_sys_mod

      implicit none
      save


 
      real (kind=real_kind), dimension(imt,jmt) ::
     &  TAVG_QFLUX                ! time-averaged qflux 
                                  !   merge with TAVG_2D when variable
                                  !   demominators are added 

      real (kind=dbl_kind)       :: tavg_sum_qflux
 
!-----------------------------------------------------------------------
!
!     derived type
!
!-----------------------------------------------------------------------

      type (var_attributes)
     &, dimension (1)            :: tavg_att_qflux
 


!***********************************************************************

      contains

!***********************************************************************

 
      subroutine tavg_qflux_compute (QFLUX, tlast_ice)

!-----------------------------------------------------------------------
!
!     increment time average of qflux field
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input fields
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) ::
     &  QFLUX                 ! ice formation / melt heat flux in  W/m^2 

      real (kind=dbl_kind), intent(in) :: 
     &  tlast_ice             ! time since last ice flux computed
 
      tavg_sum_qflux = tavg_sum_qflux + tlast_ice
      TAVG_QFLUX = TAVG_QFLUX + tlast_ice * max (c0,QFLUX)


      end subroutine tavg_qflux_compute


      subroutine tavg_global_qflux

!-----------------------------------------------------------------------
!
!     print global average of time-mean qflux field
!
!-----------------------------------------------------------------------
      real (kind=dbl_kind) :: 
     &  global_qflux_sum      ! sum of tavg qflux field
     &, qflux_norm_factor     ! normalization factor
 
      real (kind=dbl_kind), dimension (imt,jmt) :: 
     &  WORK                  ! temp for holding area_weighted field
 
 
 
      if (tavg_sum_qflux /= c0) then
 
        qflux_norm_factor = c1/(tavg_sum_qflux*area_t)
        WORK  = TAVG_QFLUX*TAREA
        global_qflux_sum = global_sum(WORK,RCALCT)*qflux_norm_factor
 
        
        if (my_task == master_task) then
          write (stdout,*) 'QFLUX',': ', global_qflux_sum
          call shr_sys_flush(stdout)
        endif
 
      endif
 


      end subroutine tavg_global_qflux   


      subroutine set_qflux_att 
!     ========================
 
      character (char_long) :: string
 
!---------------------------------------------------------------------
!     set attributes for the variable qflux -- temporary until
!       a more general averaging scheme is developed for POP
!---------------------------------------------------------------------
 
      string = 'Internal Ocean Heat Flux Due to Ice Formation; '
     &      // 'heat of fusion > 0 or ice-melting potential < 0 '
 
      tavg_att_qflux%req       = req_attributes ('QFLUX', string 
     &,                                          'Watts/meter^2'
     &,                                           3,-1,xtype_float)
 
      tavg_att_qflux%flux_direction = 'unknown_flux_direction'
      tavg_att_qflux%cell_methods   = 'unknown_cell_methods'
      tavg_att_qflux%coordinates    = 'TLONG TLAT time'
      tavg_att_qflux%mask_code      = '1-1'
      tavg_att_qflux%add_offset     = flt_fill   
      tavg_att_qflux%scale_factor   = flt_fill   
 
 
      end subroutine set_qflux_att
 
 
 
      subroutine set_qflux_dim 
!     ======================== 
 
!---------------------------------------------------------------------
!     now that the dimensions nlon, nlat, etc are defined, and the   
!       grid-codes are known for each variable, set the dimensions  
!       (lat,lon,time) for qflux  (eliminate once tavg.F is generalized
!---------------------------------------------------------------------
 
 
      tavg_att_qflux%dim_ids(1) = dim_nlon%id
      tavg_att_qflux%dim_ids(2) = dim_nlat%id
      tavg_att_qflux%dim_ids(3) = time_info%vid
 
 
 
      end subroutine set_qflux_dim
 
 
 
!***********************************************************************

      end module qflux_mod

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
