!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||


      module tavg

!***********************************************************************

!-----------------------------------------------------------------------
!
!     This module computes running time-averages of selected fields
!     and periodically writes this data to files.
!
!     CVS:$Id: tavg.F,v 1.3.2.16 2002/05/10 22:42:47 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use grid
      use time_management
      use global_reductions
      use io
      use stencils
      use operators
      use state_mod
      use forcing
      use exit_mod
      use vmix_kpp
      use io_tools
      use io_general    
      use qflux_mod
      use hmix_gm_uvwisop
      use timers
      use diag_bsf

      implicit none
      save  tavg_bin    ! redundant save for compilation on IBM
     &,     tavg_hdr
     &,     tavg_ncdf
      save

!-----------------------------------------------------------------------
!
!     variables for writing data
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  tavg_freq_iopt,       ! frequency option for writing tavg
     &  tavg_freq,            ! frequency of tavg output
     &  tavg_start_iopt,      ! start after option
     &  tavg_start            ! start tavg after tavg_start

      logical (kind=log_kind) ::
     &  ltavg_on      = .false.  ! tavg file output wanted
     &, ltavg_use_ncdf= .false.  ! use netcdf format for output tavg file
     &, ltavg_restart = .false.  ! run started from restart

      character (char_len_long) ::
     &  tavg_infile        ! filename for restart input
     &, tavg_outfile       ! root filename for tavg output

!-----------------------------------------------------------------------
!
!     available tavg fields
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter ::
     &  all_tavg_fields = 82 

      integer (kind=int_kind) ::
     &  num_tavg_fields_2d, num_tavg_fields_3d

      integer (kind=int_kind), dimension(:), allocatable ::
     &  tavg_fields


      real (kind=dbl_kind), parameter ::
     &  epsr = 1.0e+10_dbl_kind
 
!-----------------------------------------------------------------------
!
!     3d time averaged fields
!
!-----------------------------------------------------------------------

      real (kind=real_kind), dimension(:,:,:,:), allocatable ::
     &  TAVG_3D            ! buffer for all 3-d tavg fields

!-----------------------------------------------------------------------
!
!     2d time averaged fields
!
!-----------------------------------------------------------------------

      real (kind=real_kind), dimension(:,:,:), allocatable ::
     &  TAVG_2D            ! buffer for all 2-d tavg fields
 


!-----------------------------------------------------------------------
!
!     identifier for each tavg field
!     buffer location and logical flags for the available fields
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::  ! 3d fields 
     &  ntavg_TEMP  =  1,   ! potential temperature
     &  ntavg_SALT  =  2,   ! salinity
     &  ntavg_TEMP2 =  3,   ! potential temperature squared
     &  ntavg_SALT2 =  4,   ! salinity squared
     &  ntavg_UVEL  =  5,   ! zonal velocity
     &  ntavg_VVEL  =  6,   ! meridional velocity
     &  ntavg_KE    =  7,   ! horizontal kinetic energy
     &  ntavg_ST    =  8,   ! temperature * salinity
     &  ntavg_RHO   =  9,   ! in situ density
     &  ntavg_UET   = 10,   ! heat flux across east  face
     &  ntavg_VNT   = 11,   ! heat flux across north face
     &  ntavg_WTT   = 12,   ! heat flux across top   face
     &  ntavg_UES   = 13,   ! salt flux across east  face
     &  ntavg_VNS   = 14,   ! salt flux across north face
     &  ntavg_WTS   = 15,   ! salt flux across top   face
     &  ntavg_UEU   = 16,   ! zonal momentum flux across east  face
     &  ntavg_VNU   = 17,   ! zonal momentum flux across north face
     &  ntavg_WTU   = 18,   ! zonal momentum flux across top   face
     &  ntavg_UEV   = 19,   ! merid momentum flux across east  face
     &  ntavg_VNV   = 20,   ! merid momentum flux across north face
     &  ntavg_WTV   = 21,   ! merid momentum flux across top   face
     &  ntavg_PV    = 22,   ! potential vorticity
     &  ntavg_Q     = 23,   ! d(rho)/dz
     &  ntavg_PD    = 24,   ! potential density
     &  ntavg_UDP   = 25,   ! pressure gradient work (ugradp)
     &  ntavg_PEC   = 26,   ! potential energy release due to convection
     &  ntavg_NCNV  = 27,   ! number of convective events
     &  ntavg_VUF   = 28,   ! zonal viscous stress at bottom of U cells
     &  ntavg_VVF   = 29,   ! merid viscous stress at bottom of U cells
     &  ntavg_UV    = 30,   ! uvel*vvel
     &  ntavg_RHOU  = 31,   ! rho*uvel
     &  ntavg_RHOV  = 32,   ! rho*vvel
     &  ntavg_PVWM  = 33,   ! pot vorticity flux through top    face
     &  ntavg_PVWP  = 34,   ! pot vorticity flux through bottom face
     &  ntavg_UPV   = 35,   ! pot vorticity flux through east   face
     &  ntavg_VPV   = 36,   ! pot vorticity flux through north  face
     &  ntavg_URHO  = 37,   ! density flux across east  face
     &  ntavg_VRHO  = 38,   ! density flux across north face
     &  ntavg_WRHO  = 39,   ! density flux across top   face
     &  ntavg_UQ    = 40,   ! uvel*d(rho)/dz
     &  ntavg_VQ    = 41    ! vvel*d(rho)/dz

      integer (kind=int_kind) ::  ! 2d fields 
     &  ntavg_SHF   = 42,   ! surface heat flux  (total, including SW)
     &  ntavg_SHF_QSW=43,   ! solar short-wave heat flux
     &  ntavg_SFWF  = 44,   ! surface salt flux  (kg/m^2/s)
     &  ntavg_SSH   = 45,   ! surface height
     &  ntavg_H2    = 46,   ! surface height squared
     &  ntavg_H3    = 47,   ! Laplacian of surface height
     &  ntavg_TAUX  = 48,   ! zonal component of wind stress
     &  ntavg_TAUY  = 49,   ! merid component of wind stress
     &  ntavg_SU    = 50,   ! vertically integrated zonal velocity
     &  ntavg_SV    = 51,   ! vertically integrated merid velocity
     &  ntavg_T1_8  = 52,   ! temp in upper 8 levels
     &  ntavg_S1_8  = 53,   ! salt in upper 8 levels
     &  ntavg_U1_8  = 54,   ! zonal velocity in upper 8 levels
     &  ntavg_V1_8  = 55,   ! merid velocity in upper 8 levels
     &  ntavg_HDIFT = 56,   ! vertically-integrated horz diff T tend
     &  ntavg_ADVT  = 57,   ! vertically-integrated T advection tend
     &  ntavg_HMXL  = 58,   ! mix layer depth
     &  ntavg_HBLT  = 59,   ! boundary layer depth
     &  ntavg_XMXL  = 60,   ! maximum mix layer depth
     &  ntavg_XBLT  = 61,   ! maximum boundary layer depth
     &  ntavg_TMXL  = 62,   ! minimum mix layer depth
     &  ntavg_TBLT  = 63,   ! minimum boundary layer depth
     &  ntavg_RESID_T=64,   ! free-surface residual flux (T)
     &  ntavg_RESID_S=65,   ! free-surface residual flux (S)
     &  ntavg_PREC_F =66,   ! precipitation flux
     &  ntavg_EVAP_F =67,   ! evaporation   flux
     &  ntavg_MELT_F =68,   ! (snow) melt   flux
     &  ntavg_ROFF_F =69,   ! river runoff  flux
     &  ntavg_SALT_F =70,   ! salt          flux
     &  ntavg_SENH_F =71,   ! sensible heat flux
     &  ntavg_LWUP_F =72,   ! long wave heatflux (up)
     &  ntavg_LWDN_F =73,   ! long wave heatflux (down)
     &  ntavg_MELTH_F=74    ! melt heat     flux

      integer (kind=int_kind) ::  ! more new fields
     &  ntavg_UISOP = 75,   ! zonal isopycnal mixing velocity
     &  ntavg_VISOP = 76,   ! meridional isopycnal mixing velocity
     &  ntavg_WISOP = 77,   ! vertical isopyc. mixing vel. (T grid-box top center)
     &  ntavg_FW    = 78,   ! freshwater flux
     &  ntavg_TFW_T = 79,   ! T flux due to freshwater flux
     &  ntavg_TFW_S = 80,   ! S flux due to freshwater flux
     &  ntavg_WVEL  = 81,   ! vertical velocity (T grid-box top center)
     &  ntavg_BSF   = 82    ! diagnostic barotropic streamfunction


      character (char_len), dimension(all_tavg_fields) ::
     &  tavg_names_all = (/'TEMP    ',
     &                     'SALT    ',
     &                     'TEMP2   ',
     &                     'SALT2   ',
     &                     'UVEL    ',
     &                     'VVEL    ',
     &                     'KE      ',
     &                     'ST      ',
     &                     'RHO     ',
     &                     'UET     ',
     &                     'VNT     ',
     &                     'WTT     ',
     &                     'UES     ',
     &                     'VNS     ',
     &                     'WTS     ',
     &                     'UEU     ',
     &                     'VNU     ',
     &                     'WTU     ',
     &                     'UEV     ',
     &                     'VNV     ',
     &                     'WTV     ',
     &                     'PV      ',
     &                     'Q       ',
     &                     'PD      ',
     &                     'UDP     ',
     &                     'PEC     ',
     &                     'NCNV    ',
     &                     'VUF     ',
     &                     'VVF     ',
     &                     'UV      ',
     &                     'RHOU    ',
     &                     'RHOV    ',
     &                     'PVWM    ',
     &                     'PVWP    ',
     &                     'UPV     ',
     &                     'VPV     ',
     &                     'URHO    ',
     &                     'VRHO    ',
     &                     'WRHO    ',
     &                     'UQ      ',
     &                     'VQ      ',
     &                     'SHF     ',
     &                     'SHF_QSW ',
     &                     'SFWF    ',
     &                     'SSH     ',
     &                     'H2      ',
     &                     'H3      ',
     &                     'TAUX    ',
     &                     'TAUY    ',
     &                     'SU      ',
     &                     'SV      ',
     &                     'T1-8    ',
     &                     'S1-8    ',
     &                     'U1-8    ',
     &                     'V1-8    ',
     &                     'HDIFT   ',
     &                     'ADVT    ',
     &                     'HMXL    ',
     &                     'HBLT    ',
     &                     'XMXL    ',
     &                     'XBLT    ',
     &                     'TMXL    ',
     &                     'TBLT    ', 
     &                     'RESID_T ',
     &                     'RESID_S ',
     &                     'PREC_F  ',
     &                     'EVAP_F  ',
     &                     'MELT_F  ',
     &                     'ROFF_F  ',
     &                     'SALT_F  ',
     &                     'SENH_F  ',
     &                     'LWUP_F  ',
     &                     'LWDN_F  ',
     &                     'MELTH_F ',
     &                     'UISOP   ',
     &                     'VISOP   ',
     &                     'WISOP   ',
     &                     'FW      ',
     &                     'TFW_T   ',
     &                     'TFW_S   ',
     &                     'WVEL    ',
     &                     'BSF     ' /) 

      integer (kind=int_kind), dimension(all_tavg_fields) ::
     &  tavg_grid = (/ 1,             ! TEMP
     &                 1,             ! SALT
     &                 1,             ! TEMP2
     &                 1,             ! SALT2
     &                 2,             ! UVEL
     &                 2,             ! VVEL
     &                 2,             ! KE
     &                 1,             ! ST
     &                 1,             ! RHO
     &                 1,             ! UET
     &                 1,             ! VNT
     &                 1,             ! WTT
     &                 1,             ! UES
     &                 1,             ! VNS
     &                 1,             ! WTS
     &                 2,             ! UEU
     &                 2,             ! VNU
     &                 2,             ! WTU
     &                 2,             ! UEV
     &                 2,             ! VNV
     &                 2,             ! WTV
     &                 1,             ! PV
     &                 1,             ! Q
     &                 1,             ! PD
     &                 2,             ! UDP
     &                 1,             ! PEC
     &                 1,             ! NCNV
     &                 2,             ! VUF
     &                 2,             ! VVF
     &                 2,             ! UV
     &                 1,             ! RHOU
     &                 1,             ! RHOV
     &                 1,             ! PVWM
     &                 1,             ! PVWP
     &                 1,             ! UPV
     &                 1,             ! VPV
     &                 1,             ! URHO
     &                 1,             ! VRHO
     &                 1,             ! WRHO
     &                 1,             ! UQ
     &                 1,             ! VQ
     &                 1,             ! SHF
     &                 1,             ! SHF_QSW
     &                 1,             ! SFWF
     &                 1,             ! SSH
     &                 1,             ! H2
     &                 1,             ! H3
     &                 2,             ! TAUX
     &                 2,             ! TAUY
     &                 2,             ! SU
     &                 2,             ! SV
     &                 1,             ! T1-8
     &                 1,             ! S1-8
     &                 2,             ! U1-8
     &                 2,             ! V1-8
     &                 1,             ! HDIFT
     &                 1,             ! ADVT
     &                 1,             ! HMXL
     &                 1,             ! HBLT
     &                 1,             ! XMXL
     &                 1,             ! XBLT
     &                 1,             ! TMXL
     &                 1,             ! TBLT
     &                 1,             ! RESID_T
     &                 1,             ! RESID_S
     &                 1,             ! PREC_F
     &                 1,             ! EVAP_F
     &                 1,             ! MELT_F
     &                 1,             ! ROFF_F
     &                 1,             ! SALT_F
     &                 1,             ! SENH_F
     &                 1,             ! LWUP_F
     &                 1,             ! LWDN_F
     &                 1,             ! MELTH_F
     &                 2,             ! UISOP
     &                 2,             ! VISOP
     &                 1,             ! WISOP
     &                 1,             ! FW
     &                 1,             ! TFW_T
     &                 1,             ! TFW_S
     &                 1,             ! WVEL
     &                 2              ! BSF
     &                   /)

      integer (kind=int_kind), dimension(all_tavg_fields) ::
     &  tavg_bufloc

      logical (kind=log_kind), dimension(all_tavg_fields) ::
     &  ltavg_flags

!-----------------------------------------------------------------------
!
!     scalars
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) ::
     &  tavg_sum,         ! accumulated time (in seconds)
     &  dtavg             ! current time step
 
!-----------------------------------------------------------------------
!
!     derived types
!
!-----------------------------------------------------------------------

      type (var_attributes)
     &, dimension(all_tavg_fields)  :: tavg_att_2d
     &,                                tavg_att_3d
 
      type (output_file)               
     &                              :: tavg_bin
     &,                                tavg_hdr
     &,                                tavg_ncdf

!-----------------------------------------------------------------------
!
!     timers
!
!-----------------------------------------------------------------------
      
      integer(kind=int_kind), private ::
     &  timer_write_tavg_dump


!***********************************************************************

      contains

!***********************************************************************

      subroutine init_tavg

!-----------------------------------------------------------------------
!
!     initializes tavg field choices from input files
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input namelist
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  nu,                 ! unit for contents input file
     &  nfield,             ! dummy for field number
     &  isrch,              ! dummy for searching name array
     &  iii,                ! dummy for position in character string
     &  n,                  ! dummy for position in character string
     &  num_tavg_fields,    ! total number of tavg fields
     &  nml_error,          ! namelist i/o error flag
     &  contents_error      ! error flag for contents file read

      character (char_len) ::
     &  tavg_freq_opt,      ! choice for frequency of tavg output
     &  tavg_start_opt,     ! choice for starting averaging
     &  tavg_contents,      ! filename for choosing fields for output
     &  char_temp           ! temporary for manipulating fields
     &, max_string          ! local character string
     &, min_string          ! local character string

      namelist /tavg_nml/ tavg_freq_opt, tavg_freq, tavg_infile,
     &                    tavg_outfile, tavg_contents, tavg_start_opt,
     &                    tavg_start, ltavg_use_ncdf, debug_ncdf

 
!-----------------------------------------------------------------------
!
!     read tavg file output frequency and filenames from namelist
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,*)
        write(stdout,*) 'Tavg options'
        write(stdout,*)
        write(stdout,1000)
 1000   format(72('-'))
      endif

      tavg_freq_iopt    = freq_opt_never
      tavg_freq         = 100000
      tavg_start_iopt   = start_opt_nstep
      tavg_start        = 0
      tavg_infile       = 'unknown_tavg_infile'
      tavg_outfile      = 't'
      tavg_contents     = 'unknown_tavg_contents'
      debug_ncdf        = .false.

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=tavg_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('reading tavg_nml',normal_exit=.false.)
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, tavg_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        select case (tavg_freq_opt)
        case ('never')
          tavg_freq_iopt = freq_opt_never
          write(stdout,*) 'tavg diagnostics off'
        case ('nyear')
          tavg_freq_iopt = freq_opt_nyear
          write(stdout,*) 'tavg diagnostics every ',tavg_freq,' years'
        case ('nmonth')
          tavg_freq_iopt = freq_opt_nmonth
          write(stdout,*) 'tavg diagnostics every ',tavg_freq,' months'
        case ('nday')
          tavg_freq_iopt = freq_opt_nday
          write(stdout,*) 'tavg diagnostics every ',tavg_freq,' days'
        case ('nhour')
          tavg_freq_iopt = freq_opt_nhour
          write(stdout,*) 'tavg diagnostics every ',tavg_freq,' hours'
        case ('nsecond')
          tavg_freq_iopt = freq_opt_nsecond
          write(stdout,*) 'tavg diagnostics every ',tavg_freq,' seconds'
        case ('nstep')
          tavg_freq_iopt = freq_opt_nstep
          write(stdout,*) 'tavg diagnostics every ',tavg_freq,' steps'
        case default
          tavg_freq_iopt = -1000
        end select

        if (tavg_freq_iopt /= freq_opt_never) then
          select case (tavg_start_opt)
          case ('nstep')
            tavg_start_iopt = start_opt_nstep
            write(stdout,*) 'tavg sums accumulated starting at step ',
     &                      tavg_start
          case ('nday')
            tavg_start_iopt = start_opt_nday
            write(stdout,*) 'tavg sums accumulated starting at day ',
     &                      tavg_start
          case ('nyear')
            tavg_start_iopt = start_opt_nyear
            write(stdout,*) 'tavg sums accumulated starting at year ',
     &                      tavg_start
          case ('date')
            tavg_start_iopt = start_opt_date
            write(stdout,*) 'tavg sums accumulated starting at ',
     &                      tavg_start
          case default
            tavg_start_iopt = -1000
          end select
        endif

      endif

      if (my_task == master_task) then
          search_string: do iii=1,len_trim(tavg_outfile)
            if (tavg_outfile(iii:iii+3) == 'hist') then
                exit search_string
            endif
             
            if (iii == len_trim(tavg_outfile))
     &        call exit_POP ('(init_tavg) tavg output filename must '
     &          // 'contain the string "hist"', normal_exit = .false.)
          enddo search_string
      endif
 
      call broadcast_scalar(tavg_freq_iopt, master_task)

      if (tavg_freq_iopt == -1000) then
        call exit_POP('unknown option for tavg file frequency'
     &,                normal_exit=.false.)
      else if (tavg_freq_iopt /= freq_opt_never) then
        call broadcast_scalar(tavg_freq,         master_task)
        call broadcast_scalar(tavg_start_iopt,   master_task)
        call broadcast_scalar(tavg_start,        master_task)
        call broadcast_scalar(tavg_infile,       master_task)
        call broadcast_scalar(tavg_outfile,      master_task)
        call broadcast_scalar(tavg_contents,     master_task)
        call broadcast_scalar(ltavg_use_ncdf,    master_task)
        call broadcast_scalar(debug_ncdf,        master_task)

 
        if (tavg_start_iopt == -1000) then
          call exit_POP('unknown option for tavg start option' 
     &,                  normal_exit=.false.)
        endif

      endif
 
 
!-----------------------------------------------------------------------
!     initialize ltavg_flags and tavg_bufloc even if 
!     tavg_freq_iopt = freq_opt_never
!-----------------------------------------------------------------------
      ltavg_flags = .false.
      tavg_bufloc = 0

!-----------------------------------------------------------------------
!     set netcdf variables required by the new io routines
!-----------------------------------------------------------------------
      if (ltavg_use_ncdf) call init_io_ncdf
 
 
!-----------------------------------------------------------------------
!
!     read contents file to determine which fields to dump
!
!-----------------------------------------------------------------------

      if (tavg_freq_iopt /= freq_opt_never) then

        call get_unit(nu)

!-----------------------------------------------------------------------
!       read contents file for 2-D tavg information
!-----------------------------------------------------------------------
 
        call read_contents_header (nu,num_tavg_fields_2d
     &,                            tavg_contents,newfile=.true.)
 
        call set_var_defaults (ltavg_use_ncdf,num_tavg_fields_2d
     &,                       tavg_att_2d)
        
    !----------------------------------------
    ! temporary; rm when tavg is generalized
        call set_qflux_att 
    !
    !----------------------------------------
 
        call read_contents (nu,num_tavg_fields_2d, tavg_att_2d
     &,                     close_contents = .false.)
 
!-----------------------------------------------------------------------
!       test contents file: are requested fields available in this
!         model configuration?
!-----------------------------------------------------------------------
 
        call test_contents (num_tavg_fields_2d,tavg_att_2d
     &,   'tavg_contents')
 
        call document ('init_tavg', 'num_tavg_fields_2d = '
     &,                              num_tavg_fields_2d)
     
!-----------------------------------------------------------------------
!       read contents file for 3-D tavg information
!-----------------------------------------------------------------------
 
        call read_contents_header (nu,num_tavg_fields_3d
     &,                            tavg_contents,newfile=.false.)
 
        call document ('init_tavg', 'num_tavg_fields_3d = '
     &,                              num_tavg_fields_3d)
 
        call set_var_defaults (ltavg_use_ncdf,num_tavg_fields_3d
     &,                        tavg_att_3d)
        
        call read_contents (nu,num_tavg_fields_3d,tavg_att_3d
     &,                     close_contents = .true.)
 
        num_tavg_fields = num_tavg_fields_3d + num_tavg_fields_2d

        allocate (         tavg_fields(num_tavg_fields),
     &             TAVG_3D (imt,jmt,km,num_tavg_fields_3d),
     &             TAVG_2D (imt,jmt,   num_tavg_fields_2d) )

        tavg_sum       = c0
        TAVG_2D        = c0
        TAVG_3D        = c0
        TAVG_QFLUX     = c0
        tavg_sum_qflux = c0
        contents_error = 0

        if (my_task == master_task) then   
          write(stdout,*) 'tavg diagnostics requested for fields:'
        endif
 
 
        field_loop_2d: do nfield=1,num_tavg_fields_2d

          call document ('init_tavg'
     &,                   trim(tavg_att_2d(nfield)%req%short_name) )
 
          if (tavg_att_2d(nfield)%grid_code(1:1) == '3') then
            call document ('init_tavg', 'Error: tavg 2-D grid code '
     &                      // '= 3')
            contents_error = -1000
            exit field_loop_2d
          endif

        
          name_srch2d:do isrch=1,all_tavg_fields
            if (trim(tavg_att_2d(nfield)%req%short_name)
     &                         == tavg_names_all(isrch)) then
              tavg_bufloc(isrch) = nfield
              ltavg_flags(isrch) = .true.
              tavg_fields(nfield) = isrch
              exit name_srch2d
            endif
          end do name_srch2d
 
          if (isrch > all_tavg_fields) then
            call document (  'tavg 2d field '
     &,           trim(tavg_att_2d(nfield)%req%short_name) )
            contents_error = -2000
            exit field_loop_2d
          endif
        end do field_loop_2d

        call broadcast_scalar(contents_error, master_task)
 
        if (contents_error == -1000) then
         call exit_POP('tavg contents: 2d fields must precede '
     &      // 'all 3d fields', normal_exit=.false.)  
        else if (contents_error == -2000) then
         call exit_POP('unknown tavg 2d field name',normal_exit=.false.)  
        endif
 
 
        call document ('init_tavg', 'Begin testing 3-D names')
 
 
        field_loop_3d:do nfield=1,num_tavg_fields_3d
          call document ('init_tavg'
     &,                   trim(tavg_att_3d(nfield)%req%short_name) )

        if (tavg_att_3d(nfield)%grid_code(1:1) == '2') then
          call document ('init_tavg', 'Error: tavg 3-D grid code=2')
          contents_error = -1000
          exit field_loop_3d
        endif
        
        name_srch3d:do isrch=1,all_tavg_fields
          if (trim(tavg_att_3d(nfield)%req%short_name)
     &                       == tavg_names_all(isrch)) then
            tavg_bufloc(isrch) = nfield
            ltavg_flags(isrch) = .true.
            tavg_fields(num_tavg_fields_2d+nfield) = isrch
            exit name_srch3d
          endif
        end do name_srch3d
 
        if (isrch > all_tavg_fields) then
            call document (  'tavg 3d field '
     &,           trim(tavg_att_3d(nfield)%req%short_name) )
          contents_error = -2000
          exit field_loop_3d
        endif
      end do field_loop_3d
 
      if (my_task == master_task)   close(nu)

      call broadcast_scalar(contents_error, master_task)
 
      if (contents_error == -1000) then
        call exit_POP('tavg contents: 3d fields must follow all '
     &    // '2d fields',normal_exit=.false.)  
      else if (contents_error == -2000) then
        call exit_POP('unknown tavg 3d field name',normal_exit=.false.) 
      endif
 
      call release_unit(nu)
      call broadcast_vector(tavg_fields, master_task, num_tavg_fields)
      call broadcast_vector(tavg_bufloc, master_task, all_tavg_fields)
      call broadcast_vector(ltavg_flags, master_task, all_tavg_fields)

      if ( ltavg_flags(ntavg_BSF) .and. 
     &( .not. ltavg_flags(ntavg_SU) .or. .not. ltavg_flags(ntavg_SV) ) )
     &  then
        call exit_POP('init_tavg: SU and SV must be defined '
     &               //'if BSF is requested',normal_exit=.false.)
      endif

      endif

!-----------------------------------------------------------------------
!
!     set ltavg_on = .true. if tavg requested and meets start condition
!
!-----------------------------------------------------------------------

      call tavg_set_flag

      if (ltavg_flags(ntavg_BSF))  call init_diag_bsf 
 
      if (ltavg_flags(ntavg_TMXL)) then
        TAVG_2D(:,:,tavg_bufloc(ntavg_TMXL)) = epsr
      endif
 
      if (ltavg_flags(ntavg_TBLT)) then
        TAVG_2D(:,:,tavg_bufloc(ntavg_TBLT)) = epsr
      endif

      if (ltavg_on .and. ltavg_restart) then
        !*** do not read restart if last restart was at a tavg dump
        !*** interval (should start new tavg sums in this case)

        if (.not. time_to_do(tavg_freq_iopt, tavg_freq)) then
          call read_tavg_restart
        endif
      endif

!-----------------------------------------------------------------------
!       define cell_methods for all tavg fields
!-----------------------------------------------------------------------
      tavg_att_2d(:)%cell_methods = 'time: mean'
      tavg_att_3d(:)%cell_methods = 'time: mean'
 
 
!-----------------------------------------------------------------------
!       define cell_methods for the exceptions:
!-----------------------------------------------------------------------
       max_string = 'time: maximum'
 
      if (ltavg_flags(ntavg_XMXL)) then
        tavg_att_2d(tavg_bufloc(ntavg_XMXL))%cell_methods = max_string
      endif
 
      if (ltavg_flags(ntavg_XBLT)) then
        tavg_att_2d(tavg_bufloc(ntavg_XBLT))%cell_methods = max_string
      endif
 
      min_string = 'time: minimum'
 
      if (ltavg_flags(ntavg_TMXL)) then
        tavg_att_2d(tavg_bufloc(ntavg_TMXL))%cell_methods = min_string
      endif
 
      if (ltavg_flags(ntavg_TBLT)) then
        tavg_att_2d(tavg_bufloc(ntavg_TBLT))%cell_methods = min_string
      endif
 
      

      if (ltavg_use_ncdf) then
        call get_timer (timer_write_tavg_dump,'IO_WRITE_TAVG_DUMP_NCDF')
      else
        call get_timer (timer_write_tavg_dump,'IO_WRITE_TAVG_DUMP_BIN')
      endif

!-----------------------------------------------------------------------

      end subroutine init_tavg

!***********************************************************************

      subroutine tavg_set_flag

!-----------------------------------------------------------------------
!
!     this routine checks the time avg option and tavg start condition
!     to see whether tavg sums should be accumulated.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     if tavg requested and tavg not already turned on, check to see
!     if it is time to start time averaging
!
!-----------------------------------------------------------------------

      if (.not. ltavg_on .and. tavg_freq_iopt /= freq_opt_never) then
        ltavg_on = time_to_start(tavg_start_iopt, tavg_start)
        time_bound_lower = tday00
      endif

!-----------------------------------------------------------------------

      end subroutine tavg_set_flag

!***********************************************************************

      subroutine write_tavg_dump(file_suffix,lreset_tavg)

!-----------------------------------------------------------------------
!
!     write time average dump
!
!     fields written are chosen from input contents file
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      character (char_len), intent(in) ::
     &  file_suffix         ! suffix to append to root filename 


      logical (kind=log_kind), intent(in) ::
     &  lreset_tavg         ! true if regular tavg dump (not restart) 

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  nu                  ! i/o unit for output file
     &, nfield              ! dummy field index
     &, cindx, cindx2       ! indices into character string
     &,  k,n,iii 
     &,  i,j,ii,jj
     &, num_writes          ! number of time levels on one netcdf file

      character (char_len_long) ::
     &  tavg_filename         ! filename for tavg data
     &, header_filename       ! filename for header file
     &, tavg_outfile_temp     ! modified tavg_outfile
     &, tavg_pointer_file     ! filename for pointer file containing
                              !   location/name of last restart file

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  WORK1, WORK2, WORK3, PSI_T, PSI_U

!-----------------------------------------------------------------------
!
!     compute global averages of tavg fields
!     do this before normalization
!
!-----------------------------------------------------------------------

      call timer_start (timer_write_tavg_dump)
 
      call tavg_global

***********************************************
      call tavg_global_qflux  ! in qflux_mod.F
***********************************************

 

!-----------------------------------------------------------------------
!
!     normalize time averages
!
!-----------------------------------------------------------------------

      TAVG_2D = TAVG_2D/tavg_sum
      TAVG_3D = TAVG_3D/tavg_sum
      if (tavg_sum_qflux /= c0) TAVG_QFLUX = TAVG_QFLUX/tavg_sum_qflux
 
!-----------------------------------------------------------------------
!
!     unnormalize exceptions 
!
!-----------------------------------------------------------------------
      call tavg_exceptions ('multiply')
 
 
!-----------------------------------------------------------------------
!
!     compute diagnostic barotropic streamfunction based on normalized
!     SU and SV for efficiency. note that this computation is performed
!     if this is a regular tavg dump (as opposed to a restart dump).
!
!-----------------------------------------------------------------------

      if (lreset_tavg .and. ltavg_flags(ntavg_BSF)) then

        WORK2 = TAVG_2D(:,:,tavg_bufloc(ntavg_SU))
        WORK3 = TAVG_2D(:,:,tavg_bufloc(ntavg_SV))
        call zcurl ( 1, WORK1, WORK2, WORK3 )

!-----------------------------------------------------------------------
!
!     solve elliptic equation for streamfunction with cg routine.
!     initial guess for solver is always zero
!
!-----------------------------------------------------------------------

        WORK2 = c0 
        call pcg_diag_bsf ( WORK2, WORK1 )

        TAVG_2D(:,:,tavg_bufloc(ntavg_BSF)) = 
     &                             merge(c0, WORK2, .not. CALCT)

        TAVG_2D(:,:,tavg_bufloc(ntavg_BSF)) =
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_BSF)) * mass_to_Sv ! convert to Sv
 
         PSI_T = TAVG_2D(:,:,tavg_bufloc(ntavg_BSF))
 
         do j=2,jmt-1
          do i=2,imt-1
            if (KMT(i,j) == 0) then
              ii_loop: do ii=i-1,i+1
               jj_loop: do jj=j-1,j+1
                 if (KMT(ii,jj) /= 0) then
                     PSI_T(i,j) = PSI_T(ii,jj) 
                     exit ii_loop
                 endif
               enddo jj_loop
              enddo ii_loop
            endif
          enddo ! i
         enddo ! j
 
         call ne_4pt_nobndy (PSI_U,AU0,AUN,AUE,AUNE,PSI_T)

         TAVG_2D(:,:,tavg_bufloc(ntavg_BSF))= PSI_U
      endif


!-----------------------------------------------------------------------
!
!     write header file with contents
!
!-----------------------------------------------------------------------

      header_filename = char_blank

      if (lreset_tavg) then
        if (ltavg_use_ncdf) then
          call create_output_filename (tavg_outfile,'.',file_suffix
     &,                               '.nc',header_filename,tavg_ncdf)
        else
          call create_output_filename (tavg_outfile,'.',file_suffix
     &,                                '.hdr',header_filename,tavg_hdr)  
        endif
      else
          tavg_outfile_temp = trim(tavg_outfile)
          iii = len_trim(tavg_outfile_temp) 
          if (iii .lt. 1) 
     &        call exit_POP ('in write_tavg_dump: ' //
     &         ' error forming tavg_outfile_temp',normal_exit=.false.)
          tavg_outfile_temp(iii:iii+1) = 'rh'

 
          search_string: do iii=1,len_trim(tavg_outfile_temp)
            if (tavg_outfile_temp(iii:iii+3) == 'hist') then
                tavg_outfile_temp(iii:iii+3) = 'rest'
                exit search_string
            endif
             
          enddo search_string
 
          call create_output_filename (tavg_outfile_temp,'.'
     &,                                file_suffix, '.hdr'
     &,                                header_filename,tavg_hdr)

      endif

 
      if (ltavg_use_ncdf .and. lreset_tavg) then
        call output_header (tavg_ncdf,ltavg_on)
 
        call set_outvar_dim (num_tavg_fields_2d, tavg_att_2d)
        call define_var_att_ncdf(tavg_ncdf,num_tavg_fields_2d
     &,                          tavg_att_2d)
 
      !------------------------------------------------
      ! temporary; rm when tavg is generalized
          call set_qflux_dim 
          call define_var_att_ncdf (tavg_ncdf,1,tavg_att_qflux)
      !
      !------------------------------------------------
 
 
        call set_outvar_dim (num_tavg_fields_3d, tavg_att_3d)
        call define_var_att_ncdf(tavg_ncdf,num_tavg_fields_3d
     &,                          tavg_att_3d)
 
        num_writes = 1   
        call write_time (tavg_ncdf,tday00,num_writes)
 
        time_bound_upper = tday00
        call write_time_bound (tavg_ncdf,num_writes)
 
        time_bound_lower = tday00
 
      else
        call output_header (tavg_hdr,ltavg_on )
 
        nu = tavg_hdr%id
 
        if (my_task == master_task) then

          write(nu,*) num_tavg_fields_2d,num_tavg_fields_3d,
     &                tavg_sum,nsteps_total,tday,iyear,imonth,iday
     &,               tavg_sum_qflux

          do nfield=1,num_tavg_fields_2d
            write(nu,'(a,1x,a,3(5x,a))')
     &                  trim(tavg_att_2d(nfield)%grid_code)
     &,                 trim(tavg_att_2d(nfield)%req%short_name)
     &,                 trim(tavg_att_2d(nfield)%req%units)
     &,                 trim(tavg_att_2d(nfield)%req%long_name)
          end do
          do nfield=1,num_tavg_fields_3d
            write(nu,'(a,1x,a,3(5x,a))')
     &                  trim(tavg_att_3d(nfield)%grid_code)
     &,                 trim(tavg_att_3d(nfield)%req%short_name)
     &,                 trim(tavg_att_3d(nfield)%req%units)
     &,                 trim(tavg_att_3d(nfield)%req%long_name)
          end do
          close(nu)
        endif
        call release_unit(nu)
      endif


!-----------------------------------------------------------------------
!
!     write data file
!
!-----------------------------------------------------------------------

      tavg_filename = char_blank
 
      if (lreset_tavg) then
        if (ltavg_use_ncdf) then
          ! do nothing; data and header information are on one file
        else
          call create_output_filename (tavg_outfile,'.',file_suffix
     &,                                ' ',tavg_filename,tavg_bin)  
          nu = tavg_bin%id
          call open_parallel_file(nu,tavg_filename,recl_real)
        endif
      else 
          tavg_outfile_temp = trim(tavg_outfile)
          iii = len_trim(tavg_outfile_temp) 
          tavg_outfile_temp(iii:iii+1) = 'rh'

          search_string2: do iii=1,len_trim(tavg_outfile_temp)
            if (tavg_outfile_temp(iii:iii+3) == 'hist') then
                tavg_outfile_temp(iii:iii+3) = 'rest'
            endif
          enddo search_string2
          call create_output_filename (tavg_outfile_temp,'.'
     &,                                file_suffix
     &,                                ' ',tavg_filename,tavg_bin)
          nu = tavg_bin%id
          call open_parallel_file(nu,tavg_filename,recl_real)
      endif
 
 
 
!-----------------------------------------------------------------------
!
!     write arrays to parallel file
!
!-----------------------------------------------------------------------

        do nfield=1,num_tavg_fields_2d
          if (ltavg_use_ncdf .and. lreset_tavg) then
            call write_array_ncdf(tavg_ncdf,tavg_att_2d(nfield)
     &,                           TAVG_2D(:,:,nfield),1)         
          else
            call write_array(nu,TAVG_2D(:,:,nfield))
          endif
        end do
 
    !-----------------------------------------------------------------------
    !
    !  write TAVG_QFLUX to output file (temporary; rm when tavg is generalized)
    !
          if (ltavg_use_ncdf .and. lreset_tavg) then
            call write_array_ncdf(tavg_ncdf,tavg_att_qflux(1)
     &,                           TAVG_QFLUX,1)
          else
            call write_array(nu,TAVG_QFLUX)
          endif
    !
    !-----------------------------------------------------------------------

 
        do nfield=1,num_tavg_fields_3d
          if (ltavg_use_ncdf .and. lreset_tavg) then
            call write_array_ncdf(tavg_ncdf,tavg_att_3d(nfield)
     &,                           TAVG_3D(:,:,:,nfield),km)        
          else
            call write_array(nu,TAVG_3D(:,:,:,nfield))
          endif
        end do


      
      if (ltavg_use_ncdf .and. lreset_tavg) then
        call close_file (tavg_ncdf)
        call document ('write_tavg_dump'
     &,                ' file written: ' // tavg_ncdf%name)
      else
        call close_parallel_file(nu)
        call document ('write_tavg_dump'
     &,                ' file written: ' // tavg_hdr%name)
        call document ('write_tavg_dump'
     &,                ' file written: ' // tavg_bin%name)
      endif

!-----------------------------------------------------------------------
!
!     if pointer files are used, write tavg filenames to pointer file
!     do this only for tavg restarts - not tavg dumps
!
!-----------------------------------------------------------------------

      if (luse_pointer_files .and. .not. lreset_tavg) then
        call get_unit(nu)
        if (my_task == master_task) then
          tavg_pointer_file = pointer_filename
          cindx = len_trim(pointer_filename) + 1
          cindx2= cindx + 4
          tavg_pointer_file(cindx:cindx2) = '.tavg'

          open(nu, file=tavg_pointer_file, form='formatted', 
     &             status='unknown')
          write(nu,1000) header_filename
          write(nu,1000) tavg_filename
          close(nu)
        endif
        call release_unit(nu)
      endif

 1000 format(a)    

!-----------------------------------------------------------------------
!
!     reset time averages if this is a regular tavg dump (as opposed
!     to a restart dump)  if this is a restart dump, unnormalize
!     in case a normal restart dump is written on the same timestep
!
!-----------------------------------------------------------------------

      if (lreset_tavg) then
        tavg_sum       = c0
        TAVG_2D        = c0
        TAVG_3D        = c0
        TAVG_QFLUX     = c0
        tavg_sum_qflux = c0
        if (ltavg_flags(ntavg_TMXL)) then
          TAVG_2D(:,:,tavg_bufloc(ntavg_TMXL)) = epsr
        endif
        if (ltavg_flags(ntavg_TBLT)) then
          TAVG_2D(:,:,tavg_bufloc(ntavg_TBLT)) = epsr
        endif
      else
        TAVG_2D    = TAVG_2D*tavg_sum
        TAVG_3D    = TAVG_3D*tavg_sum
        TAVG_QFLUX = TAVG_QFLUX*tavg_sum_qflux
        call tavg_exceptions ('divide')
 
      endif

      call timer_stop  (timer_write_tavg_dump)
 
!-----------------------------------------------------------------------

      end subroutine write_tavg_dump

!***********************************************************************

      subroutine read_tavg_restart

!-----------------------------------------------------------------------
!
!     read time-average restart dump to keep running time averages
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  nu                    ! i/o unit
     &, n                     ! dummy for indexing character string
     &, in_fields_3d          ! num of 3d fields in restart file
     &, in_fields_2d          ! num of 2d fields in restart file
     &, nfield                ! dummy field counter
     &, hdr_error             ! error file for reading restart hdr
     &, in_nsteps_total       ! nsteps_total according to tavg file
     &, in_iyear              ! iyear according to tavg file
     &, in_imonth             ! imonth according to tavg file
     &, in_iday               ! iday according to tavg file
     &, cindx, cindx2         ! indices into character strings

      real (kind=dbl_kind) ::
     &  in_tday               ! tday according to tavg file

      character (char_len_long) ::
     &  header_filename       ! filename for restart contents
     &, tavg_pointer_file     ! filename for pointer file containing
                              !   location/name of last restart file
      character (char_len) ::
     &  char_temp             ! for string manipulation

!-----------------------------------------------------------------------
!
!     if pointer files are used, pointer file and must be read to get 
!     actual filenames
!     otherwise, build header filename
!
!-----------------------------------------------------------------------

      if (luse_pointer_files) then
        call get_unit(nu)
        if (my_task == master_task) then
          tavg_pointer_file = pointer_filename
          cindx = len_trim(pointer_filename) + 1
          cindx2= cindx + 4
          tavg_pointer_file(cindx:cindx2) = '.tavg'

          write(stdout,*) 'Reading pointer file: ',
     &                    trim(tavg_pointer_file)
   
          open(nu, file=tavg_pointer_file, form='formatted', 
     &             status='old')
          read(nu,1000) header_filename
          read(nu,1000) tavg_infile
          close(nu)
        endif
        call release_unit(nu)

        call broadcast_scalar(tavg_infile, master_task)

      else ! build header filename

        if (my_task == master_task) then
          header_filename = char_blank

          cindx  = 1
          cindx2 = len_trim(tavg_infile)
          header_filename(cindx:cindx2) = trim(tavg_infile)
          cindx  = cindx2 + 1
          cindx2 = cindx + 3
          header_filename(cindx:cindx2) = '.hdr'
        endif

      endif

 1000 format(a)   

!-----------------------------------------------------------------------
!
!     read header file to check contents/consistency with content file
!
!-----------------------------------------------------------------------

      hdr_error = 0
      call get_unit(nu)
      if (my_task == master_task) then

        write(stdout,*) 'Reading tavg header file: ',
     &                  trim(header_filename)
        open(nu,file=header_filename,status='old')
        read(nu,*) in_fields_2d,in_fields_3d,tavg_sum,
     &             in_nsteps_total,in_tday,in_iyear,in_imonth,in_iday
     &,            tavg_sum_qflux

        if (in_fields_3d /= num_tavg_fields_3d .or.
     &      in_fields_2d /= num_tavg_fields_2d) then
          hdr_error = -1000
        endif
      endif

 
 
      call broadcast_scalar(hdr_error, master_task)
      if (hdr_error == -1000) then
        if (my_task == master_task) then
            write(stdout,*) ' in_fields_3d = ', in_fields_3d
            write(stdout,*) ' num_tavg_fields_3d = ',num_tavg_fields_3d

            write(stdout,*) ' in_fields_2d = ', in_fields_2d
            write(stdout,*) ' num_tavg_fields_2d = ',num_tavg_fields_2d
        endif
        call exit_POP('number of tavg fields in restart inconsistent'
     &,                normal_exit=.false.) 
      endif

      !**** add check here with prognostic restart time?

      if (my_task == master_task) then
        field_loop: do nfield=1,num_tavg_fields_2d + num_tavg_fields_3d
          read(nu,'(a80)') char_temp
          char_temp = adjustl(char_temp)
          char_temp(1:4) = '    '
          char_temp = adjustl(char_temp)
          n = index(char_temp,' ') - 1
        
          if (char_temp(1:n) /= 
     &        tavg_names_all(tavg_fields(nfield))(1:n)) then
            hdr_error = -2000
            exit field_loop
          endif
        end do field_loop

        close(nu)
        write(stdout,*) ' file read: ', header_filename
      endif
      call release_unit(nu)

 
 
 
      call broadcast_scalar(hdr_error, master_task)
      if (hdr_error == -2000) then
        call exit_POP('tavg field name in restart file inconsistent'
     &,                normal_exit=.false.) 
      endif

!-----------------------------------------------------------------------
!
!     Broadcast information to other processors
!
!-----------------------------------------------------------------------

      call broadcast_scalar_real(tavg_sum      , master_task)
      call broadcast_scalar_real(tavg_sum_qflux, master_task)

!-----------------------------------------------------------------------
!
!     read parallel arrays from data restart file
!
!-----------------------------------------------------------------------

      call open_parallel_file(nu,tavg_infile,recl_real)

      do nfield=1,num_tavg_fields_2d
        call read_array(nu,TAVG_2D(:,:,nfield))
      end do

      call read_array(nu,TAVG_QFLUX(:,:) )
 
      do nfield=1,num_tavg_fields_3d
        call read_array(nu,TAVG_3D(:,:,:,nfield))
      end do

      call close_parallel_file(nu)

      if (my_task == master_task) then
        write(stdout,*) ' '
        write(stdout,*) ' file read: ', tavg_infile
      endif

!-----------------------------------------------------------------------
!
!     de-normalize sums
!
!-----------------------------------------------------------------------

      TAVG_2D    = TAVG_2D*tavg_sum
      TAVG_3D    = TAVG_3D*tavg_sum
      TAVG_QFLUX = TAVG_QFLUX*tavg_sum_qflux
      call tavg_exceptions ('divide')

!-----------------------------------------------------------------------

      call tavg_global   ! print global sums of time averages
***********************************************
      call tavg_global_qflux  ! in qflux_mod.F
***********************************************

!-----------------------------------------------------------------------

      end subroutine read_tavg_restart

!***********************************************************************

      subroutine tavg_global

!-----------------------------------------------------------------------
!
!     calculate and print global integrals of time average fields
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  k,                    ! vertical level index
     &  ifield,               ! field identifier
     &  nfield                ! dummy field index

      real (kind=dbl_kind) :: 
     &  tavg_field_sum,       ! sum of tavg field
     &  tavg_norm             ! normalization for average
 
      real (kind=dbl_kind), dimension (imt,jmt) :: 
     &  WORK,                 ! temp for holding area_weighted field
     &  RMASK                 ! topography mask for global sum

!-----------------------------------------------------------------------
!
!     calculate globally-integrated time average of each chosen 2d field
!
!-----------------------------------------------------------------------

      call time_stamp (time_string, date_string)
 
      if (my_task == master_task) then
        write (stdout,*) ' '
        write (stdout,*) 'Global Time Averages: ' // trim(time_string)  
     &                    // ' ' // trim(date_string)
      endif

      do nfield=1,num_tavg_fields_2d
        ifield = tavg_fields(nfield)

        if (tavg_grid(ifield) == 1) then
          tavg_norm = c1/(tavg_sum*area_t)
          RMASK = RCALCT
          WORK  = TAVG_2D(:,:,nfield)*TAREA
        else
          tavg_norm = c1/(tavg_sum*area_u)
          RMASK = RCALCU
          WORK  = TAVG_2D(:,:,nfield)*UAREA
        endif
 
        tavg_field_sum = global_sum(WORK,RMASK)*tavg_norm
 
!-----------------------------------------------------------------------
!
!     unnormalize exceptions 
!
!-----------------------------------------------------------------------
 
        if   (ifield == ntavg_XMXL .or. ifield == ntavg_XBLT
     &  .or.  ifield == ntavg_TMXL .or. ifield == ntavg_TBLT) then
          tavg_field_sum = tavg_field_sum*tavg_sum
        endif
 

        if (my_task == master_task) then
          write (stdout,*) trim(tavg_names_all(ifield)),': ',
     &                     tavg_field_sum
        endif
 
      end do

!-----------------------------------------------------------------------
!
!     calculate globally-integrated time average of each chosen 3d field
!
!-----------------------------------------------------------------------

      do nfield=1,num_tavg_fields_3d
        ifield = tavg_fields(num_tavg_fields_2d + nfield)

        tavg_field_sum = c0

        do k=1,km
          if (tavg_grid(ifield) == 1) then
            RMASK = merge(c1, c0, k <= KMT) 
            WORK  = TAVG_3D(:,:,k,nfield)*TAREA
          else
            RMASK = merge(c1, c0, k <= KMU) 
            WORK  = TAVG_3D(:,:,k,nfield)*UAREA
          endif
          tavg_field_sum = tavg_field_sum + dz(k)*global_sum(WORK,RMASK)

        end do

        if (tavg_grid(ifield) == 1) then
          tavg_norm = c1/(tavg_sum*volume_t)
        else
          tavg_norm = c1/(tavg_sum*volume_u)
        endif
        tavg_field_sum = tavg_field_sum*tavg_norm

        if (my_task == master_task) then
          write (stdout,*) trim(tavg_names_all(ifield)),': ',
     &                     tavg_field_sum
        endif
      end do

 
!-----------------------------------------------------------------------

      end subroutine tavg_global

!***********************************************************************

      subroutine tavg_free_sfc_resid_flx

      real (kind=dbl_kind)        :: factor
!-----------------------------------------------------------------------
!
!     increment time-average of the free-surface residual flux for
!       temperature and salt tracers
!       this subroutine is called only 
!       if (k==1 .and. ltavg_on .and. mix_pass /= 1)
!-----------------------------------------------------------------------


!-----------------------------------------------------------------------
!
!     accumulate the sum if these fields have been chosen
!
!-----------------------------------------------------------------------

 
      if (ltavg_flags(ntavg_RESID_T)) then                            
 
               ! factor/dtavg converts to W/m^2
        factor = dtavg/hflux_factor  
        where (CALCT) TAVG_2D(:,:,tavg_bufloc(ntavg_RESID_T)) = 
     &                TAVG_2D(:,:,tavg_bufloc(ntavg_RESID_T)) + 
     &                DH*TRACER(:,:,1,1,curtime)*factor
      endif

 
      if (ltavg_flags(ntavg_RESID_S)) then                            
     
       !factor = dtavg/salinity_factor converts to kg(freshwater)/m^2/s
        factor = dtavg/salinity_factor
        where (CALCT) TAVG_2D(:,:,tavg_bufloc(ntavg_RESID_S)) = 
     &                TAVG_2D(:,:,tavg_bufloc(ntavg_RESID_S)) + 
     &                DH*TRACER(:,:,1,2,curtime)*factor
      endif


!-----------------------------------------------------------------------

      end subroutine tavg_free_sfc_resid_flx

!***********************************************************************

      subroutine tavg_vdiffu(k,VUF,VVF)

!-----------------------------------------------------------------------
!
!     increment time average of viscous stresses
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  k                     ! vertical level index

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &  VUF,VVF

!-----------------------------------------------------------------------
!
!     accumulate the sum if these fields have been chosen
!
!-----------------------------------------------------------------------

 
      if (ltavg_on .and. mix_pass /= 1) then
 
        if (ltavg_flags(ntavg_VUF)) then
          TAVG_3D(:,:,k,tavg_bufloc(ntavg_VUF)) = 
     &    TAVG_3D(:,:,k,tavg_bufloc(ntavg_VUF)) + dtavg*VUF
        endif

        if (ltavg_flags(ntavg_VVF)) then
          TAVG_3D(:,:,k,tavg_bufloc(ntavg_VVF)) = 
     &    TAVG_3D(:,:,k,tavg_bufloc(ntavg_VVF)) + dtavg*VVF
        endif

      endif 
!-----------------------------------------------------------------------

      end subroutine tavg_vdiffu

!***********************************************************************

      subroutine tavg_hdifft(k,HDTK)

!-----------------------------------------------------------------------
!
!     increment time average of horizontal mixing tendencies
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  k                     ! vertical level index

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(in) :: 
     &  HDTK                  ! horiz mix tendencies

!-----------------------------------------------------------------------
!
!     accumulate the sum if these fields have been chosen
!
!-----------------------------------------------------------------------

      !*** vertically-integrated horizontal mix tendencies

      if (ltavg_flags(ntavg_HDIFT)) then
        where (k <= KMT) 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_HDIFT)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_HDIFT)) + 
     &    dtavg*dz(k)*HDTK(:,:,1)
      endif

!-----------------------------------------------------------------------

      end subroutine tavg_hdifft

!***********************************************************************
 


      subroutine tavg_gradp(k,WORK)

!-----------------------------------------------------------------------
!
!     increment time average of work done by pressure gradient
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  k                    ! vertical level index

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &  WORK                 ! pressure gradient work

!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_UDP)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_UDP)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_UDP)) + dtavg*WORK
      endif

!-----------------------------------------------------------------------

      end subroutine tavg_gradp

!***********************************************************************

      subroutine tavg_convad(k,RHODIFF)

!-----------------------------------------------------------------------
!
!     increment time average of various quantities associated
!     with convective adjustment
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  k                    ! vertical level index

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &  RHODIFF              ! difference between convectively adjusted
                             ! density and density before adjustment

!-----------------------------------------------------------------------
!
!     update potential energy release due to convection and
!     number of convective events 
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_PEC)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_PEC)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_PEC)) + dtavg*RHODIFF
      endif

      if (ltavg_flags(ntavg_NCNV)) then
        where (abs(RHODIFF) > 1.e-8_dbl_kind) 
     &    TAVG_3D(:,:,k,tavg_bufloc(ntavg_NCNV)) = 
     &    TAVG_3D(:,:,k,tavg_bufloc(ntavg_NCNV)) + c1
      endif

!-----------------------------------------------------------------------

      end subroutine tavg_convad

!***********************************************************************

      subroutine tavg_advu(k,FUE,FVN,WUK)

!-----------------------------------------------------------------------
!
!     increment time average of advective correlations
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  k                    ! vertical level index

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &  FUE,                ! advective velocity through east  face
     &  FVN,                ! advective velocity through north face
     &  WUK                 ! vert velocity through top face

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  WORK1               ! local temp space

!-----------------------------------------------------------------------
!
!     advective flux of zonal momentum through east, north faces
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_UEU)) then
        call e_eoshift(WORK1,UVEL(:,:,k,curtime))
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_UEU)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_UEU)) + 
     &               dtavg*FUE*(UVEL(:,:,k,curtime) + WORK1)
      endif

      if (ltavg_flags(ntavg_VNU)) then
        call n_eoshift(WORK1,UVEL(:,:,k,curtime))
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_VNU)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_VNU)) + 
     &               dtavg*FVN*(UVEL(:,:,k,curtime) + WORK1)
      endif

!-----------------------------------------------------------------------
!
!     advective flux of merid momentum through east, north faces
!
!-----------------------------------------------------------------------


      if (ltavg_flags(ntavg_UEV)) then
        call e_eoshift(WORK1,VVEL(:,:,k,curtime))
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_UEV)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_UEV)) + 
     &               dtavg*FUE*(VVEL(:,:,k,curtime) + WORK1)
      endif

      if (ltavg_flags(ntavg_VNV)) then
        call n_eoshift(WORK1,VVEL(:,:,k,curtime))
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_VNV)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_VNV)) + 
     &               dtavg*FVN*(VVEL(:,:,k,curtime) + WORK1)
      endif

!-----------------------------------------------------------------------
!
!     advective fluxes of zonal, merid momentum through top face
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_WTU)) then
        if (k == 1) then
          TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTU)) = 
     &    TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTU)) + 
     &                 dtavg*dzr(k)*WUK*UVEL(:,:,k,curtime)
        else
          TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTU)) = 
     &    TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTU)) + 
     &                 dtavg*dz2r(k)*WUK*(UVEL(:,:,k  ,curtime) 
     &                                  + UVEL(:,:,k-1,curtime))
        endif
      endif

      if (ltavg_flags(ntavg_WTV)) then
        if (k == 1) then
          TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTV)) = 
     &    TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTV)) + 
     &                 dtavg*dzr(k)*WUK*VVEL(:,:,k,curtime)
        else
          TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTV)) = 
     &    TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTV)) + 
     &                 dtavg*dz2r(k)*WUK*(VVEL(:,:,k  ,curtime) 
     &                                  + VVEL(:,:,k-1,curtime))
        endif
      endif

!-----------------------------------------------------------------------

      end subroutine tavg_advu

!***********************************************************************

      subroutine tavg_advt(k,FUE,FVN,WTK,WTKB,LTK)

!-----------------------------------------------------------------------
!
!     increment time average of tracer advective correlations
!     and many other quantities
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  k                      ! vertical level index

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &  FUE,                ! advective velocity across east   face
     &  FVN,                ! advective velocity across north  face
     &  WTK,                ! advective velocity across top    face
     &  WTKB                ! advective velocity across bottom face

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(in) :: 
     &  LTK                 ! total advective tendency
 
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) ::
     &  rho_ref1           ! reference density

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  RHOK1M,            ! pot density at k-1
     &  WORK1,WORK2,
     &  WORK3,WORK4        ! local temp space

      real (kind=dbl_kind), dimension(imt,jmt), save :: 
     &  RHOK1              ! pot density at k relative to k=1


!-----------------------------------------------------------------------
!
!     vertical average of advective tendency
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_ADVT)) then
        where (k <= KMT) 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_ADVT)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_ADVT)) - dtavg*dz(k)*LTK(:,:,1)
      endif

!-----------------------------------------------------------------------
!
!     increment time averages of 3d prognostic fields
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_UVEL)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_UVEL)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_UVEL)) + 
     &               dtavg*UVEL(:,:,k,curtime)
      endif

      if (ltavg_flags(ntavg_VVEL)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_VVEL)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_VVEL)) + 
     &               dtavg*VVEL(:,:,k,curtime)
      endif

      if (ltavg_flags(ntavg_WVEL)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_WVEL)) =
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_WVEL)) + dtavg * WTK 
      endif

      if (ltavg_flags(ntavg_KE)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_KE)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_KE)) + 
     &       dtavg*p5*(UVEL(:,:,k,curtime)**2 + VVEL(:,:,k,curtime)**2)
      endif

      if (ltavg_flags(ntavg_UV)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_UV)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_UV)) + 
     &               dtavg*UVEL(:,:,k,curtime)*VVEL(:,:,k,curtime)
      endif

      if (ltavg_flags(ntavg_TEMP)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_TEMP)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_TEMP)) + 
     &               dtavg*TRACER(:,:,k,1,curtime)
      endif

      if (ltavg_flags(ntavg_SALT)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_SALT)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_SALT)) + 
     &               dtavg*TRACER(:,:,k,2,curtime)
      endif

      if (ltavg_flags(ntavg_TEMP2)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_TEMP2)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_TEMP2)) + 
     &               dtavg*TRACER(:,:,k,1,curtime)**2
      endif

      if (ltavg_flags(ntavg_SALT2)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_SALT2)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_SALT2)) + 
     &               dtavg*TRACER(:,:,k,2,curtime)**2
      endif

      if (ltavg_flags(ntavg_ST)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_ST)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_ST)) + 
     &      dtavg*TRACER(:,:,k,1,curtime)*TRACER(:,:,k,2,curtime)
      endif

      if (ltavg_flags(ntavg_RHO)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_RHO)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_RHO)) + 
     &               dtavg*RHO(:,:,k,curtime)
      endif

!-----------------------------------------------------------------------
!
!     advection fluxes (per unit volume) of heat, salt
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_UET)) then
        call e_eoshift(WORK1,TRACER(:,:,k,1,curtime))
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_UET)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_UET)) + 
     &               dtavg*FUE*(TRACER(:,:,k,1,curtime) + WORK1)
      endif

      if (ltavg_flags(ntavg_VNT)) then
        call n_eoshift(WORK1,TRACER(:,:,k,1,curtime))
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_VNT)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_VNT)) + 
     &               dtavg*FVN*(TRACER(:,:,k,1,curtime) + WORK1)
      endif

      if (ltavg_flags(ntavg_UES)) then
        call e_eoshift(WORK1,TRACER(:,:,k,2,curtime))
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_UES)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_UES)) + 
     &               dtavg*FUE*(TRACER(:,:,k,2,curtime) + WORK1)
      endif

      if (ltavg_flags(ntavg_VNS)) then
        call n_eoshift(WORK1,TRACER(:,:,k,2,curtime))
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_VNS)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_VNS)) + 
     &               dtavg*FVN*(TRACER(:,:,k,2,curtime) + WORK1)
      endif

      if (ltavg_flags(ntavg_WTT)) then
        if (k == 1) then
          TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTT)) = 
     &    TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTT)) + 
     &                 dtavg*dzr(k)*WTK*TRACER(:,:,k,1,curtime)
        else
          TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTT)) = 
     &    TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTT)) + 
     &                 dtavg*dz2r(k)*WTK*(TRACER(:,:,k  ,1,curtime) 
     &                                  + TRACER(:,:,k-1,1,curtime))
        endif
      endif

      if (ltavg_flags(ntavg_WTS)) then
        if (k == 1) then
          TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTS)) = 
     &    TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTS)) + 
     &                 dtavg*dzr(k)*WTK*TRACER(:,:,k,2,curtime)
        else
          TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTS)) = 
     &    TAVG_3D(:,:,k,tavg_bufloc(ntavg_WTS)) + 
     &                 dtavg*dz2r(k)*WTK*(TRACER(:,:,k  ,2,curtime) 
     &                                  + TRACER(:,:,k-1,2,curtime))
        endif
      endif

!-----------------------------------------------------------------------
!
!     potential density referenced to k=1 needed for a variety of 
!     tavg fields
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_PD) .or.
     &    ltavg_flags(ntavg_RHOU) .or.
     &    ltavg_flags(ntavg_RHOV) .or.
     &    ltavg_flags(ntavg_URHO) .or.
     &    ltavg_flags(ntavg_VRHO) .or.
     &    ltavg_flags(ntavg_WRHO)) then

        if (k == 1) then
          RHOK1 = RHO(:,:,k,curtime)
          RHOK1M = RHOK1
        else
          RHOK1M = RHOK1
          call state(k,1,TRACER(:,:,k,1,curtime),
     &                   TRACER(:,:,k,2,curtime),RHOOUT=RHOK1)
        endif
        where (k > KMT) RHOK1 = c0  ! zero RHOK1 at land points
      endif

      if (ltavg_flags(ntavg_PD)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_PD)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_PD)) + 
     &               dtavg*RHOK1
      endif

      if (ltavg_flags(ntavg_URHO)) then
        call e_eoshift(WORK1,RHOK1)
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_URHO)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_URHO)) + 
     &               dtavg*FUE*(RHOK1 + WORK1)
      endif

      if (ltavg_flags(ntavg_VRHO)) then
        call n_eoshift(WORK1,RHOK1)
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_VRHO)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_VRHO)) + 
     &               dtavg*FVN*(RHOK1 + WORK1)
      endif

      if (ltavg_flags(ntavg_WRHO)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_WRHO)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_WRHO)) + 
     &               dtavg*dz2r(k)*WTK*(RHOK1 + RHOK1M)
      endif

!-----------------------------------------------------------------------
!
!     average U,V to T points for <rho*U>
!     remember RHO is the density anomoly wrt the reference density.
!
!-----------------------------------------------------------------------

      rho_ref1 =  1.0244262_dbl_kind  ! reference density for level 1 

      if (ltavg_flags(ntavg_RHOU)) then
        call sw_4pt_nobndy(WORK1,AT0,ATS,ATW,ATSW,UVEL(:,:,k,curtime))
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_RHOU)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_RHOU)) + 
     &               dtavg*(RHOK1 + rho_ref1)*WORK1
      endif

      if (ltavg_flags(ntavg_RHOV)) then
        call sw_4pt_nobndy(WORK1,AT0,ATS,ATW,ATSW,VVEL(:,:,k,curtime))
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_RHOV)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_RHOV)) + 
     &               dtavg*(RHOK1 + rho_ref1)*WORK1
      endif

!-----------------------------------------------------------------------
!
!     vertical density gradient and potential vorticity
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_Q)    .or.
     &    ltavg_flags(ntavg_PV)   .or.
     &    ltavg_flags(ntavg_PVWM) .or.
     &    ltavg_flags(ntavg_PVWP) .or.
     &    ltavg_flags(ntavg_UPV)  .or.
     &    ltavg_flags(ntavg_VPV)  .or.
     &    ltavg_flags(ntavg_UQ)   .or.
     &    ltavg_flags(ntavg_VQ) ) then

        if (k == 1 ) then
          WORK3 = RHO(:,:,1,curtime)
        else
          call state(k-1,k,TRACER(:,:,k-1,1,curtime)
     &                    ,TRACER(:,:,k-1,2,curtime),RHOOUT=WORK3)
          WORK3 = p5*(WORK3 + RHO(:,:,k,curtime))
        endif
        if (k == km) then
          WORK4 = RHO(:,:,km,curtime)
        else
          call state(k+1,k,TRACER(:,:,k+1,1,curtime)
     &                    ,TRACER(:,:,k+1,2,curtime),RHOOUT=WORK4)

          where (k /= KMT)
            WORK4 = p5*(WORK4 + RHO(:,:,k,curtime))
          elsewhere 
            WORK4 = RHO(:,:,k,curtime)
          end where
        endif

        WORK1 = merge((WORK3 - WORK4)*dzr(k),c0,k <= KMT)  ! drho/dz 

        call zcurl(k,WORK3,UVEL(:,:,k,curtime),VVEL(:,:,k,curtime))
        WORK2 = WORK1*(WORK3*TAREA_R + FCORT) ! PV = pot vorticity
      endif

      if (ltavg_flags(ntavg_Q)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_Q)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_Q)) + 
     &               dtavg*WORK1
      endif

      if (ltavg_flags(ntavg_PV)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_PV)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_PV)) + 
     &               dtavg*WORK2
      endif

!-----------------------------------------------------------------------
!
!     advection of potential vorticity, UQ, VQ
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_PVWM)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_PVWM)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_PVWM)) + 
     &               dtavg*WORK2*WTK
      endif

      if (ltavg_flags(ntavg_PVWP)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_PVWP)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_PVWP)) + 
     &               dtavg*WORK2*WTKB
      endif

      if (ltavg_flags(ntavg_UPV)) then
        call e_eoshift(WORK3,WORK2)
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_UPV)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_UPV)) + 
     &               dtavg*FUE*(WORK2 + WORK3)
      endif

      if (ltavg_flags(ntavg_VPV)) then
        call n_eoshift(WORK4,WORK2)
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_VPV)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_VPV)) + 
     &               dtavg*FVN*(WORK2 + WORK4)
      endif

      if (ltavg_flags(ntavg_UQ)) then
        call e_eoshift(WORK3,WORK1)
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_UQ)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_UQ)) + 
     &               dtavg*FUE*(WORK3 + WORK1)
      endif

      if (ltavg_flags(ntavg_VQ)) then
        call n_eoshift(WORK4,WORK1)
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_VQ)) = 
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_VQ)) + 
     &               dtavg*FVN*(WORK4 + WORK1)
      endif

!-----------------------------------------------------------------------
!
!     isopycnal velocities
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_UISOP)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_UISOP)) =
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_UISOP)) + dtavg * UISOP
      endif

      if (ltavg_flags(ntavg_VISOP)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_VISOP)) =
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_VISOP)) + dtavg * VISOP
      endif

      if (ltavg_flags(ntavg_WISOP)) then
        TAVG_3D(:,:,k,tavg_bufloc(ntavg_WISOP)) =
     &  TAVG_3D(:,:,k,tavg_bufloc(ntavg_WISOP)) + dtavg * WISOPT
      endif

!-----------------------------------------------------------------------

      end subroutine tavg_advt

!***********************************************************************

      subroutine tavg_2d_compute

!-----------------------------------------------------------------------
!
!     increment time average of 2d fields
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  k                     ! vertical level index
 
      real (kind=dbl_kind), dimension (imt,jmt) ::
     &  WORK

      real (kind=dbl_kind) :: global_min, global_max
 
      real (kind=dbl_kind) :: factor
!-----------------------------------------------------------------------
!
!     setup time step and total integrated time for time average
!     adjust for averaging timesteps: if this is an averaging timestep,
!      the past values only contribute for 1/4 of a step and the
!      values for the step just before an averaging timestep contribute
!      for 1 1/4 steps.
!
!-----------------------------------------------------------------------

 
      if (avg_ts .or. back_to_back) then
        dtavg = p5*dtt
      else
        dtavg = dtt
      endif


      tavg_sum = tavg_sum + dtavg
 

!-----------------------------------------------------------------------
!
!     2d surface heat (total and short-wave only) and freshwater flux
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_SHF)) then
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_SHF)) =
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_SHF)) +
     &             dtavg*(STF(:,:,1)+SHF_QSW)/hflux_factor ! W/m^2
      endif

      if (ltavg_flags(ntavg_SHF_QSW)) then
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_SHF_QSW)) =
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_SHF_QSW)) +
     &             dtavg*SHF_QSW/hflux_factor ! W/m^2
      endif

      if (ltavg_flags(ntavg_SFWF)) then
       !factor = dtavg/salinity_factor converts to kg(freshwater)/m^2/s
        factor = dtavg/salinity_factor
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_SFWF)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_SFWF)) + 
     &     STF(:,:,2)*factor
      endif

      if (ltavg_flags(ntavg_FW)) then
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_FW)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_FW)) + dtavg * FW     ! cm/s
      endif

      if (ltavg_flags(ntavg_TFW_T)) then
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_TFW_T)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_TFW_T)) 
     &     + dtavg * TFW(:,:,1) / hflux_factor                ! W/m^2
      endif

      if (ltavg_flags(ntavg_TFW_S)) then
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_TFW_S)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_TFW_S)) 
     &     + dtavg * TFW(:,:,2) * rho_sw * c10   ! kg of salt/m^2/s
      endif


#if coupled
      if (ltavg_flags(ntavg_PREC_F)) then
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_PREC_F)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_PREC_F)) + 
     &     dtavg*PREC_F                          ! kg/m2/s
      endif
 
      if (ltavg_flags(ntavg_EVAP_F)) then
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_EVAP_F)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_EVAP_F)) + 
     &     dtavg*EVAP_F                          ! kg/m2/s
      endif
 
      if (ltavg_flags(ntavg_MELT_F)) then
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_MELT_F)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_MELT_F)) + 
     &     dtavg*MELT_F                          ! kg/m2/s
      endif
 
      if (ltavg_flags(ntavg_ROFF_F)) then
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_ROFF_F)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_ROFF_F)) + 
     &     dtavg*ROFF_F                          ! kg/m2/s
      endif
 
      if (ltavg_flags(ntavg_SALT_F)) then
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_SALT_F)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_SALT_F)) + 
     &     dtavg*SALT_F                          ! kg of salt/m2/s
      endif
 
      if (ltavg_flags(ntavg_SENH_F)) then
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_SENH_F)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_SENH_F)) + 
     &     dtavg*SENH_F                          ! W/m2
      endif
 
      if (ltavg_flags(ntavg_LWUP_F)) then
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_LWUP_F)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_LWUP_F)) + 
     &     dtavg*LWUP_F                          ! W/m2
      endif
 
      if (ltavg_flags(ntavg_LWDN_F)) then
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_LWDN_F)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_LWDN_F)) + 
     &     dtavg*LWDN_F                          ! W/m2
      endif
 
      if (ltavg_flags(ntavg_MELTH_F)) then
        where (KMT > 0)
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_MELTH_F)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_MELTH_F)) + 
     &     dtavg*MELTH_F                          ! W/m2
      endif
 
 
#endif
!-----------------------------------------------------------------------
!
!     surface height and associated arrays
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_SSH)) then
        TAVG_2D(:,:,tavg_bufloc(ntavg_SSH)) = 
     &  TAVG_2D(:,:,tavg_bufloc(ntavg_SSH)) + 
     &             dtavg*(PSURF(:,:,curtime)/grav)
      endif

      if (ltavg_flags(ntavg_H2)) then
        TAVG_2D(:,:,tavg_bufloc(ntavg_H2)) = 
     &  TAVG_2D(:,:,tavg_bufloc(ntavg_H2)) + 
     &             dtavg*(PSURF(:,:,curtime)/grav)**2
      endif

      if (ltavg_flags(ntavg_H3)) then
        TAVG_2D(:,:,tavg_bufloc(ntavg_H3)) = 
     &  TAVG_2D(:,:,tavg_bufloc(ntavg_H3)) + 
     &             dtavg*((GRADPX(:,:,curtime)/grav)**2 + 
     &                    (GRADPY(:,:,curtime)/grav)**2)
      endif

!-----------------------------------------------------------------------
!
!     wind stress
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_TAUX)) then
        TAVG_2D(:,:,tavg_bufloc(ntavg_TAUX)) = 
     &  TAVG_2D(:,:,tavg_bufloc(ntavg_TAUX)) + 
     &             dtavg*SMF(:,:,1)
      endif

      if (ltavg_flags(ntavg_TAUY)) then
        TAVG_2D(:,:,tavg_bufloc(ntavg_TAUY)) = 
     &  TAVG_2D(:,:,tavg_bufloc(ntavg_TAUY)) + 
     &             dtavg*SMF(:,:,2)
      endif

!-----------------------------------------------------------------------
!
!     vertically integrated velocities
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_SU)) then
        TAVG_2D(:,:,tavg_bufloc(ntavg_SU)) = 
     &  TAVG_2D(:,:,tavg_bufloc(ntavg_SU)) + 
     &             dtavg*HU*UBTROP(:,:,curtime)
      endif

      if (ltavg_flags(ntavg_SV)) then
        TAVG_2D(:,:,tavg_bufloc(ntavg_SV)) = 
     &  TAVG_2D(:,:,tavg_bufloc(ntavg_SV)) + 
     &             dtavg*HU*VBTROP(:,:,curtime)
      endif

!-----------------------------------------------------------------------
!
!     T,S,U,V  in upper 8 levels
!
!-----------------------------------------------------------------------

      if (ltavg_flags(ntavg_T1_8)) then
        do k = 1,8
          TAVG_2D(:,:,tavg_bufloc(ntavg_T1_8)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_T1_8)) + 
     &               dtavg*TRACER(:,:,k,1,curtime)
        enddo
      endif

      if (ltavg_flags(ntavg_S1_8)) then
        do k = 1,8
          TAVG_2D(:,:,tavg_bufloc(ntavg_S1_8)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_S1_8)) + 
     &               dtavg*TRACER(:,:,k,2,curtime)
        enddo
      endif

      if (ltavg_flags(ntavg_U1_8)) then
        do k = 1,8
          TAVG_2D(:,:,tavg_bufloc(ntavg_U1_8)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_U1_8)) + 
     &               dtavg*UVEL(:,:,k,curtime)
        enddo
      endif

      if (ltavg_flags(ntavg_V1_8)) then
        do k = 1,8
          TAVG_2D(:,:,tavg_bufloc(ntavg_V1_8)) = 
     &    TAVG_2D(:,:,tavg_bufloc(ntavg_V1_8)) + 
     &               dtavg*VVEL(:,:,k,curtime)
        enddo
      endif

 
!-----------------------------------------------------------------------
!
!     mix layer depth and boundary layer depth
!
!-----------------------------------------------------------------------
 
      if (ltavg_flags(ntavg_HMXL)) then
        TAVG_2D(:,:,tavg_bufloc(ntavg_HMXL)) =
     &  TAVG_2D(:,:,tavg_bufloc(ntavg_HMXL)) +
     &             dtavg*HMXL
      endif
 
      if (ltavg_flags(ntavg_HBLT)) then
        TAVG_2D(:,:,tavg_bufloc(ntavg_HBLT)) =
     &  TAVG_2D(:,:,tavg_bufloc(ntavg_HBLT)) +
     &             dtavg*HBLT
      endif
 
      if (ltavg_flags(ntavg_XMXL)) then
        WORK = TAVG_2D(:,:,tavg_bufloc(ntavg_XMXL))
        TAVG_2D(:,:,tavg_bufloc(ntavg_XMXL)) = max(WORK,HMXL)
      endif
 
      if (ltavg_flags(ntavg_XBLT)) then
        WORK = TAVG_2D(:,:,tavg_bufloc(ntavg_XBLT))
        TAVG_2D(:,:,tavg_bufloc(ntavg_XBLT)) = max(WORK,HBLT)
      endif
 
       if (ltavg_flags(ntavg_TMXL)) then
        global_min = global_minval (HMXL)
        global_max = global_maxval (HMXL)
 
        if (global_min == c0 .and. global_max == c0 ) then
            ! do nothing -- HMXL hasn't yet been computed
        else
          WORK = TAVG_2D(:,:,tavg_bufloc(ntavg_TMXL))
          TAVG_2D(:,:,tavg_bufloc(ntavg_TMXL)) = min(WORK,HMXL)
        endif
      endif
 
      if (ltavg_flags(ntavg_TBLT)) then
        global_min = global_minval (HBLT)
        global_max = global_maxval (HBLT)
 
        if (global_min == c0 .and. global_max == c0 ) then
            ! do nothing -- HBLT hasn't yet been computed
        else
          WORK = TAVG_2D(:,:,tavg_bufloc(ntavg_TBLT))
          TAVG_2D(:,:,tavg_bufloc(ntavg_TBLT)) = min(WORK,HBLT)
        endif
      endif
 
!-----------------------------------------------------------------------

      end subroutine tavg_2d_compute

 
      subroutine tavg_exceptions (operation)
      
      character (*)               :: operation
      integer (kind=int_kind)     :: e_index
      real    (kind=dbl_kind)     :: factor

 
      select case (operation)
   
        case ('multiply')
           factor = tavg_sum
        case ('divide')
           if (tavg_sum .ne. c0) then
              factor = c1/tavg_sum
           else
               call exit_POP ('divide by zero in subroutine '
     &                      //'exceptions',normal_exit=.false.) 
           endif
        case  default 
           call document ('tavg_exceptions', 'operation failed; '
     &                    // 'unknown operation: '// operation )
 
      end select
 
 
      if ( ltavg_flags(ntavg_XMXL) ) then
        e_index = tavg_bufloc(ntavg_XMXL)
        TAVG_2D(:,:,e_index) = TAVG_2D(:,:,e_index)*factor
      endif
 
      if ( ltavg_flags(ntavg_XBLT) ) then
        e_index = tavg_bufloc(ntavg_XBLT)
        TAVG_2D(:,:,e_index) = TAVG_2D(:,:,e_index)*factor
      endif
 
      if ( ltavg_flags(ntavg_TMXL) ) then
        e_index = tavg_bufloc(ntavg_TMXL)
        TAVG_2D(:,:,e_index) = TAVG_2D(:,:,e_index)*factor
      endif
  
      if ( ltavg_flags(ntavg_TBLT) ) then
        e_index = tavg_bufloc(ntavg_TBLT)
        TAVG_2D(:,:,e_index) = TAVG_2D(:,:,e_index)*factor 
      endif
 
 
      end subroutine tavg_exceptions

!***********************************************************************

      end module tavg

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
