!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module time_management

!***********************************************************************
!
!     CVS:$Id: time_management.F,v 1.3.2.21 2002/04/25 22:45:09 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use global_reductions
      use grid
      use io
      use exit_mod
      use shr_sys_mod

      implicit none
      save

!-----------------------------------------------------------------------
!
!     variables for run control
!
!-----------------------------------------------------------------------

      character (char_len) ::
     &  stop_option         ! specify how to determine stopping time
     &, runid               ! an identifier for the run
     &, dt_option           ! method to determine tracer timestep size

      integer (kind=int_kind) ::
     &  stop_count          ! num of stop_option intervals before stop
                            !   OR date (yyyymmdd) at which model stops
     &, stop_iopt           ! integer value for stop_option
     &, nsteps_run          ! timesteps taken since beginning
                            !   of this run (full&half)
     &, nsteps_total        ! timesteps taken since beginning
                            !   of very first run (full&half)
     &, nsteps_per_day      ! integer number of steps per day
     &, len_runid           ! length of character runid

      integer (kind=int_kind) ::  ! variables used with avgfit
     &  fit_freq               ! num of intervals/day into which full &
                               !  half timesteps must exactly "fit"
     &, fullsteps_per_interval ! num of full timesteps per fitting interval
     &, halfsteps_per_interval ! num of half timesteps per fitting interval
     &, fullsteps_per_day      ! num of full timesteps per day
     &, halfsteps_per_day      ! num of half timesteps per day 
     &, nsteps_per_interval    ! number of steps taken in each 'fit' interval
     &, nsteps_this_interval   ! number of steps taken in current 'fit' interval
 

      integer (kind=int_kind), private ::
     &  stop_now            ! time_flag id for stopping
     &, coupled_ts          ! time_flag id for a coupled timestep

      logical (kind=log_kind)   :: ! this timestep is:
     &  adjust_year                !   step at which year values updated
     &, eod                        !   at the end of the day
     &, eom                        !   at the end of the month
     &, eoy                        !   at the end of the year
     &, avg_ts                     !   an averaging timestep
     &, back_to_back               !   the second of two avg timesteps in row
     &, f_euler_ts                 !   a forward Euler timestep (first ts)
     &, first_step                 !   first time step
     &, leapfrogts                 !   a leapfrog timestep
     &, matsuno_ts                 !   an Euler-backward timestep
     &, midnight                   !   at midnight
     &, ice_ts                     !   an ice-formation timestep
     &, sample_qflux_ts            !   time to sample qflux for time avg 

      logical (kind=log_kind)   :: ! the last timestep was:
     &  eod_last                   !   at the end of the day
     &, eom_last                   !   at the end of the month
     &, eoy_last                   !   at the end of the year
     &, midnight_last              !   at midnight

      logical (kind=log_kind)   :: ! the next timestep is:
     &  adjust_year_next           !   step at which year values updated
     &, eom_next                   !   at the end of the month
     &, midnight_next              !   at midnight
     &, avg_ts_next                !   an averaging ts?
     &, back_to_back_next          !   a second avg step in a row
     &, end_run_at_midnight        ! does model run end at midnight
     &, new_dtt_value              ! does restart have a new step size
 
      real (kind=dbl_kind)      ::
     &  steps_per_year          ! number of timesteps in one year
     &, steps_per_day           ! number of timesteps in one day
     &, dt_tol                  ! used to determine close enough
     &, dt_tol_year             ! used to determine if seconds_this_year
                                !  is close enough to seconds_in_year

!-----------------------------------------------------------------------
!
!     quantities related to date
!
!-----------------------------------------------------------------------

      character (1) ::
     &  date_separator      ! character to separate year-month-day
 
      character (char_len)    ::
     &  date_string           ! date in dd/mm/yyyy format
     &, time_string           ! time in hh:mm:ss format
     &, ccsm_date_string      ! date in format required by ccsm

      integer (kind=int_kind) ::
     &  iyear                 ! year    [0,inf)  for present timestep
     &, imonth                ! month   [1,12]          |
     &, iday                  ! day     [1,31]          |
     &, ihour                 ! hour    [0,23]          |
     &, iminute               ! minute  [0,59]          |
     &, isecond               ! second  [0,59]          |
     &, iday_of_year          ! day no. [1,365/6]       V

      integer (kind=int_kind) ::
     &  imonth_next           ! month   [1,12]    for next timestep
     &, iday_next             ! day     [1,31]          |
     &, ihour_next            ! hour    [0,23]          |
     &, iminute_next          ! minute  [0,59]          |
     &, isecond_next          ! second  [0,59]          |
     &, iday_of_year_next     ! day no. [1,365/6]       V

      integer (kind=int_kind) ::
     &  iyear_last            ! year    [0,inf)   from previous timestep
     &, imonth_last           ! month   [1,12]          |
     &, iday_last             ! day     [1,31]          |
     &, ihour_last            ! hour    [0,23]          |
     &, iday_of_year_last     ! day no. [1,365/6]       V

      integer (kind=int_kind) ::
     &  iyear0                ! initial start date and time
     &, imonth0               !   for complete run
     &, iday0                 !
     &, ihour0                !
     &, iminute0              !
     &, isecond0              !

 
      integer (kind=int_kind) ::
     &  iyear_start_run         ! initial start date and time
     &, imonth_start_run        !   for this run              
     &, iday_start_run          !
     &, ihour_start_run         !
     &, iminute_start_run       !
     &, isecond_start_run       !
     &, iday_of_year_start_run  !

      integer (kind=int_kind) ::
     &  iyear_end_run           ! final date for this run
     &, imonth_end_run          !  
     &, iday_end_run            !

      integer (kind=int_kind)   :: ! number of:
     &  days_in_year             ! days in present year
     &, days_in_prior_year       ! days in prior   year 
     &, elapsed_days             ! full days elapsed since    0000-01-01
     &, elapsed_days0            ! full days elapsed between  0000-01-01 
                                 !                   and day0 
     &, elapsed_days_jan1        ! full days elapsed prior to 01-01-iyear
     &, elapsed_days_this_year   ! full days elapsed since beginning of yr
     &, elapsed_days_this_run    ! full days elapsed since beginning of
                                 !                   this segment of run
     &, elapsed_days_init_date   ! full days elapsed since initial time
     &, elapsed_days_end_run     ! full days elapsed from 0000-01-01 to end
                                 !                   of this run
     &, elapsed_days_max         ! maximum number of full days allowed 
     &, elapsed_months           ! full months elapsed since 0000-01-01
     &, elapsed_months_this_run  ! full months elapsed since beginning of
                                 !                     this segment of run
     &, elapsed_months_init_date ! full months elapsed since initial time
     &, elapsed_years            ! full years  elapsed since 0000-01-01
     &, elapsed_years_this_run   ! full years  elapsed since beginning of
                                 !                     this segment of run
     &, elapsed_years_init_date  ! full years  elapsed since initial time
 
      integer (kind=int_kind), dimension(12) ::
     &  days_in_prior_months    ! cumulative num days in preceeding months
     &, days_in_month =         ! number of days in each calendar month
     &    (/31,28,31,  30,31,30, 31,31,30,   31,30,31/)
        !   J  F  M    A  M  J   J  A  S     O  N  D

      real (kind=dbl_kind)      ::
     &  seconds_this_year       ! seconds elapsed since beginning of year
     &, seconds_this_day        ! seconds elapsed this day    
     &, seconds_this_day_next   ! seconds elapsed this day  at next timestep
     &, seconds_this_year_next  ! seconds elapsed this year at next timestep
     &, seconds_in_year         ! seconds in one year -- this varies,
                                !         if leap years are allowed
     &, hours_in_year           ! hours   in one year
 
      real (kind=dbl_kind)      ::
     &  frac_day                ! fraction of the day elapsed today
     &, tyear
     &, tmonth
     &, tday
     &, thour
     &, tsecond                 ! elapsed time in seconds from initial date
     &, tsecond_old             ! tsecond from previous timestep
     &, tyear00   
     &, tsecond00
     &, tday00
     &, thour00
     &, thour00_begin_this_year

      real (kind=dbl_kind), dimension (12) ::
     &  thour00_midmonth_calendar ! num hours to middle of calendar month
     &, thour00_endmonth_calendar ! num hours to end of calendar month
     &, thour00_midmonth_equal    ! num hours to middle of equal-spaced month
     &, thour00_endmonth_equal    ! num hours to end of equal-spaced month

      logical (kind=log_kind)   ::
     &  newday
     &, newhour
     &, allow_leapyear          ! allow leap years?
     &, leapyear                ! is this a leapyear?

      character (4) ::
     &  cyear               ! character version of year

      character (2) ::
     &  cmonth              ! character version of month
     &, cday                ! character version of day
     &, chour               ! character version of hour
     &, cminute             ! character version of minute
     &, csecond             ! character version of second

      character (3) ::
     &  cmonth3             ! character month in 3-letter form

      character (3), dimension(12) ::
     &  month3_all = (/'jan','feb','mar','apr','may','jun','jul',
     &                 'aug','sep','oct','nov','dec'/)

      character (2), dimension(12) ::
     &  cmonths  = ( /'01','02','03','04','05','06',
     &                '07','08','09','10','11','12' /)
 
      character (2), dimension(31) ::
     &  cdays    = ( /'01','02','03','04','05','06','07','08','09','10'
     &,               '11','12','13','14','15','16','17','18','19','20'
     &,               '21','22','23','24','25','26','27','28','29','30'
     &,               '31' /)
 
      integer (kind=int_kind), parameter ::
     &  days_in_leap_year = 366 !   days in a leap year
     &, days_in_norm_year = 365 !   days in a non-leap year

      real (kind=dbl_kind), parameter ::
     &  seconds_in_minute =    60.0_dbl_kind
     &, seconds_in_hour   =  3600.0_dbl_kind
     &, seconds_in_day    = 86400.0_dbl_kind
     &, minutes_in_hour   =    60.0_dbl_kind

!-----------------------------------------------------------------------
!
!     parameters for time frequency and start options
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter ::
     &  freq_opt_never    = 0
     &, freq_opt_nyear    = 1
     &, freq_opt_nmonth   = 2
     &, freq_opt_nday     = 3
     &, freq_opt_nhour    = 4
     &, freq_opt_nsecond  = 5
     &, freq_opt_nstep    = 6

      integer (kind=int_kind), parameter ::
     &  start_opt_nstep   = 1
     &, start_opt_nday    = 2
     &, start_opt_nyear   = 3
     &, start_opt_date    = 4

      integer (kind=int_kind), parameter ::
     &  next_opt_day      = 1 
     &, next_opt_month    = 2 
     &, next_opt_year     = 3 
     &, stop_opt_never    = 0 
     &, stop_opt_sometime = 1 

!-----------------------------------------------------------------------
!
!     user defined time flags
!
!-----------------------------------------------------------------------

      type time_flag

        character (char_len) ::
     &    name                  ! name for flag

        logical (kind=log_kind) ::
     &    value                 ! logical state of flag
     &,   old_value             ! last state of flag
     &,   default               ! default state of flag
     &,   has_default           ! T if default defined, F if no default

        integer (kind=int_kind) ::
     &    freq_opt              ! frequency units for switching flag on
     &,   freq                  ! freq in above units for switching flag

      end type time_flag

      integer (kind=int_kind), parameter ::
     &  max_time_flags=99       ! max number of user-defined flags

      type (time_flag), dimension(max_time_flags) ::
     &  time_flags              ! array containing user-defined flags

      integer (kind=int_kind) ::
     &  num_time_flags = 0

!-----------------------------------------------------------------------
!
!     time-step related constants and variables
!
!-----------------------------------------------------------------------

      logical (kind=log_kind) ::
     &  laccel                ! flag for acceleration

      real (kind=dbl_kind)      ::
     &   dt_count               ! input count to determine dtt
     &,  dtt                    ! tracer timestep (sec)
     &,  dtt_input              ! tracer timestep (sec) as specified in namelist
                                !   input; may be different from restart value
     &,  dtu                    ! momentum timestep (sec)
     &,  dtp                    ! barotropic timestep (sec)
     &,  c2dtu
     &,  c2dtp
     &,  c2dtq
     &,  dtuxcel                ! factor to multiply MOMENTUM timestep
     &,  dttxcel(km)            ! array for depth-dependent acceleration
     &,  dt(km)                 ! time step at each level
     &,  c2dtt(km)
     &,  dztxcel(km)
     &,  dzwxcel(km)
     &,  stepsize               ! size of present timestep (sec)
     &,  stepsize_next          ! size of next timestep (sec)

!-----------------------------------------------------------------------
!
!     time-centering and mixing variables
!
!-----------------------------------------------------------------------

      logical (kind=log_kind) ::
     &  impcor                ! implicit treatment of Coriolis terms

      integer (kind=int_kind), parameter ::
     &  tmix_matsuno = 1,    ! use matsuno step for time mixing
     &  tmix_avg     = 2,    ! use averaging step for time mixing
     &  tmix_avgbb   = 3,    ! use averaging step for time mixing, with
                             !  back_to_back option to keep time boundaries
     &  tmix_avgfit  = 4     ! use averaging step for time mixing, 
                             !  selecting the timestep size in such
                             !  a way as to force the end of the day
                             !  (or interval) to coincide with the end of 
                             !  a timestep


      integer (kind=int_kind) ::
     &  tmix_iopt,           ! option for which time mixing to use
     &  time_mix_freq,       ! frequency of mixing
     &  mix_pass             ! number of passes to perform mixing

      real (kind=dbl_kind) ::
     &  beta              ! = {alpha,theta} on {leapfrog,Matsuno} steps

      real (kind=dbl_kind), parameter ::
     &  alpha = c1/c3,        ! leapfrog grap(ps) time-centering param
     &  theta = p5,           ! Matsuno grap(ps) time-centering param
     &  gamma = c1 - c2*alpha ! for geostrophic balance, otherwise
                              ! coriolis and  surface-pressure gradient
                              ! are not time centered

!***********************************************************************
 
      logical (kind=log_kind),private   :: 
     &    debug_time_management   = .false. 

      contains

!***********************************************************************

      subroutine init_time_manager1

!-----------------------------------------------------------------------
!
!     intializes some time manager variables from namelist inputs
!     and sets time step and
!     remaining time manager variables are initialized after restart
!     files are read
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  k                      ! vertical level index
     &, nu                     ! i/o unit number
     &, nm                     ! month index

!-----------------------------------------------------------------------
!
!     namelist input
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   nml_error            ! namelist i/o error flag

      character (char_len) ::
     &  time_mix_opt        ! option for time mixing (Matsuno,averaging)
     &, accel_file          ! file containing acceleration factors


      namelist /time_manager_nml/ 
     &         runid,          time_mix_opt,    time_mix_freq, 
     &         impcor,         laccel,          accel_file,
     &         dtuxcel,        iyear0,          imonth0,
     &         iday0,          ihour0,          iminute0,
     &         isecond0,       dt_option,       dt_count,
     &         stop_option,    stop_count,      date_separator,
     &         allow_leapyear, fit_freq 

!-----------------------------------------------------------------------
!
!     Set logical flags to default values.
!
!-----------------------------------------------------------------------

      stop_now    = init_time_flag('stop_now'   ,default=.false.)
      coupled_ts  = init_time_flag('coupled_ts' )

      call reset_switches

!-----------------------------------------------------------------------
!
!     set initial values  for namelist inputs
!
!-----------------------------------------------------------------------

      runid            = 'unknown_runid'
      impcor           = .true.
      time_mix_opt     = 'avgfit'
      fit_freq         =  1
      time_mix_freq    = 17
      dtuxcel          = c1
      laccel           = .false.
      accel_file       = 'unknown_accel_file'
      allow_leapyear   = .false.
      stop_option      = 'unknown_stop_option'
      stop_count       = -1
      dt_option        = 'auto_dt'
      dt_count         =  1

      dt_tol     = 1.0e-6
      dt_tol_year= 100.0*dt_tol

      iyear0         = 0
      imonth0        = 1
      iday0          = 1
      ihour0         = 0
      iminute0       = 0
      isecond0       = 0


      date_separator = ' '

 
!-----------------------------------------------------------------------
!
!     read options from namelist input file
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=time_manager_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('reading time_manager_nml',normal_exit=.false.)
      endif
 
      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, time_manager_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        select case (time_mix_opt)
        case ('matsuno')
          tmix_iopt = tmix_matsuno
        case ('avg')
          tmix_iopt = tmix_avg
        case ('avgbb')
          tmix_iopt = tmix_avgbb
        case ('avgfit')
          tmix_iopt = tmix_avgfit
        case default
          tmix_iopt = -1000
        end select
      endif

      call broadcast_scalar (runid           , master_task)
      call broadcast_scalar (tmix_iopt       , master_task)
      call broadcast_scalar (fit_freq        , master_task)
      call broadcast_scalar (time_mix_freq   , master_task)
      call broadcast_scalar (impcor          , master_task)
      call broadcast_scalar (laccel          , master_task)
      call broadcast_scalar (dtuxcel         , master_task)
      call broadcast_scalar (iyear0          , master_task)
      call broadcast_scalar (imonth0         , master_task)
      call broadcast_scalar (iday0           , master_task)
      call broadcast_scalar (ihour0          , master_task)
      call broadcast_scalar (iminute0        , master_task)
      call broadcast_scalar (isecond0        , master_task)
      call broadcast_scalar (dt_option       , master_task)
      call broadcast_scalar (dt_count        , master_task)
      call broadcast_scalar (stop_option     , master_task)
      call broadcast_scalar (stop_count      , master_task)
      call broadcast_scalar (allow_leapyear  , master_task)
      call broadcast_scalar (date_separator  , master_task)
 
      if (tmix_iopt == -1000) then
        call exit_POP('unknown option for time mixing'
     &,                normal_exit=.false.)
      endif

      len_runid = len_trim(runid)

!-----------------------------------------------------------------------
!
!     determine the value for dtt, based upon model input parameters
!
!-----------------------------------------------------------------------

      select case (dt_option)
 
      case('auto_dt')   
        !*** scale tracer timestep  dt = 1 hr at dx = 2 degrees
        dtt = seconds_in_hour*(180.0_dbl_kind/float(imt_global))
        steps_per_day  = seconds_in_day/dtt
        steps_per_year = steps_per_day*days_in_norm_year

      case('steps_per_year')
        steps_per_year = dt_count
        steps_per_day  = steps_per_year/days_in_norm_year
        dtt            = seconds_in_day/steps_per_day

      case('steps_per_day')
        steps_per_day  = dt_count
        steps_per_year = steps_per_day*days_in_norm_year
        dtt            = seconds_in_day/steps_per_day

      case('seconds')
        dtt            = dt_count
        steps_per_day  = seconds_in_day/dtt
        steps_per_year = steps_per_day *days_in_norm_year

      case('hours'  )
        dtt            = dt_count*seconds_in_hour
        steps_per_day  = seconds_in_day/dtt
        steps_per_year = steps_per_day*days_in_norm_year

      case default
        call exit_POP('unknown dt_option',normal_exit=.false.)
      end select
 

 
!-----------------------------------------------------------------------
!
!     modify dtt value, if using avgfit option
!
!-----------------------------------------------------------------------

      if (tmix_iopt  == tmix_avgfit) then
 
!-----------------------------------------------------------------------
!     determine the number of full, half, and total number of
!     steps in each interval. an interval is typically one day, unless
!     fit_freq is greater than one (eg, coupling frequency > 1x/day)
!-----------------------------------------------------------------------
        nsteps_per_day         = steps_per_day
        fullsteps_per_interval = nsteps_per_day/fit_freq
        if (fullsteps_per_interval < 1) fullsteps_per_interval = 1
        halfsteps_per_interval = 1 + 
     &                           nsteps_per_day/(fit_freq*time_mix_freq)
        nsteps_per_interval    = fullsteps_per_interval +
     &                           halfsteps_per_interval
 
!-----------------------------------------------------------------------
!     is an adjustment to the number of half and full steps in
!     each interval needed?
!-----------------------------------------------------------------------
 
        if ((fullsteps_per_interval/time_mix_freq) <  
     &       nsteps_per_interval   /time_mix_freq    ) then
            halfsteps_per_interval = halfsteps_per_interval + 1
            nsteps_per_interval    = nsteps_per_interval    + 1
        endif
 
!-----------------------------------------------------------------------
!     determine the number of half, full, and total steps in
!     each day
!-----------------------------------------------------------------------
        fullsteps_per_day      = fit_freq*fullsteps_per_interval
        halfsteps_per_day      = fit_freq*halfsteps_per_interval
        nsteps_per_day         = fullsteps_per_day + halfsteps_per_day
 
!-----------------------------------------------------------------------
!     compute modified dtt value
!-----------------------------------------------------------------------
        dtt            = seconds_in_day/
     &                   (fullsteps_per_day + 0.5*halfsteps_per_day)
        steps_per_day  = seconds_in_day/dtt
 
      else
        nsteps_per_interval = steps_per_day
      endif
 
 

      dtt_input = dtt
      dtp       = dtt
      dtu       = dtt


!-----------------------------------------------------------------------
!
!     multiply tracer timestep by acceleration factor(s)
!     for depth varying acceleration factors, create several arrays
!       to store the variable timestep (dt and c2dtt) and some
!       combination dz/dttxcel arrays for use in convective adjustment.
!
!-----------------------------------------------------------------------

      if (laccel) then

        call get_unit(nu)
        if (my_task == master_task) then

          open(nu, file=accel_file, status = 'old')
          do k = 1,km
            read(nu,*) dttxcel(k)
          end do

          if(dttxcel(1) /= c1) then   !  make sure no accel in top layer
            write(stdout,*) 'no acceleration allowed in top layer'
            write(stdout,*) 'resetting acceleration factor to 1.0'
            dttxcel(1) = c1
          endif

          close(nu)
        endif
        call release_unit(nu)

        call broadcast_vector (dttxcel, master_task, km)

      else

        dttxcel = c1

      endif

      do k = 1,km
        dt(k) = dtt*dttxcel(k)
        dztxcel(k) = dz(k)/dttxcel(k)
      enddo
      do k = 1,km-1
        dzwxcel(k) = c1/(dztxcel(k) + dztxcel(k+1))
      enddo
      dzwxcel(km) = c0

      dtu = dtu*dtuxcel
      dtp = dtp*dtuxcel
      if (dtuxcel /= c1 .and. my_task == master_task) then
        write(stdout,*)' '
        write(stdout,*)' ** MOMENTUM TIMESTEP ACCELERATION ACTIVE **'
      endif

!-----------------------------------------------------------------------
!
!     set initial values; some of these may be overwritten by
!     restart input
!
!-----------------------------------------------------------------------

      eom_next          = .false.  
      eom_last          = .false.  
      eod_last          = .false.  
      eoy_last          = .false.  
      midnight_last     = .false.  

      iyear        = iyear0
      imonth       = imonth0
      iday         = iday0
      ihour        = ihour0
      iminute      = iminute0
      isecond      = isecond0

      nsteps_total    = 0

      seconds_this_day        = ihour0  *seconds_in_hour
     &                        + iminute0*seconds_in_minute
     &                        + isecond0

!-----------------------------------------------------------------------
!
!     define days_in_prior_months; leap-year adjustments are made in
!       subroutine init_timemanager_2
!
!-----------------------------------------------------------------------

      call prior_days (days_in_prior_months, days_in_month)

!-----------------------------------------------------------------------

      end subroutine init_time_manager1

!***********************************************************************

      subroutine init_time_manager2

!-----------------------------------------------------------------------
!
!     complete initialization of time manager quantities now that
!     information from restart files is known
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  nm                       ! month index
     &, days_in_month_temp
     &, days_in_year_end_run
     &, ndays_temp

      logical (kind=log_kind) ::
     &  leapyear_test

      character (4) ::
     &  cyear_end_run            ! character version of ending year

      character (2) ::
     &  cmonth_end_run           ! character version of ending month
     &, cday_end_run             ! ! character version of ending day

!-----------------------------------------------------------------------
!
!     determine the number of days, months, and years elapsed since
!       0000-01-01  and number of days and months elapsed this year
!
!-----------------------------------------------------------------------

      call ymd2eday (iyear , imonth , iday , elapsed_days     )
      call ymd2eday (iyear , 1      , 1    , elapsed_days_jan1)
      call ymd2eday (iyear0, imonth0, iday0, elapsed_days0    )

      elapsed_days_this_year   = elapsed_days - elapsed_days_jan1

      elapsed_years            = iyear
      elapsed_months           = iyear*12 + imonth - 1
      elapsed_days_this_run    = 0
      elapsed_years_this_run   = 0
      elapsed_months_this_run  = 0
      elapsed_days_init_date   = elapsed_days - elapsed_days0
      elapsed_years_init_date  = iyear - iyear0
      elapsed_months_init_date = elapsed_years_init_date*12
     &                         - imonth0 + imonth

      seconds_this_year       = elapsed_days_this_year*seconds_in_day
     &                            +  seconds_this_day
      iday_of_year            = elapsed_days_this_year + 1

!-----------------------------------------------------------------------
!
!     determine tsecond, the number of seconds elapsed since beginning
!       of complete simulation.
!
!-----------------------------------------------------------------------

      tsecond = elapsed_days_init_date*seconds_in_day + seconds_this_day

!-----------------------------------------------------------------------
!
!     compare the value of dtt selected for this run via namelist input
!      with that from the previous run.  if different, time_manager
!      will execute as if it were not a restart.
!
!-----------------------------------------------------------------------

      if (dtt /= dtt_input ) then
        new_dtt_value = .true.
        dtt = dtt_input
      endif

!-----------------------------------------------------------------------
!
!     determine if this is a leap year; set days_in_year and 
!       days_in_prior_months, regardless of value of allow_leapyear
!
!-----------------------------------------------------------------------

      call leap_adjust

      if (iyear > 0 .and. is_leapyear(iyear-1) ) then
        days_in_prior_year = days_in_leap_year
      else
        days_in_prior_year = days_in_norm_year
      endif

!-----------------------------------------------------------------------
!
!     check initial iday, imonth, etc values for reasonableness
!
!-----------------------------------------------------------------------

      if ( .not. valid_ymd_hms () ) then
        call exit_POP('invalid ymd_hms',normal_exit=.false.)
      endif

!-----------------------------------------------------------------------
!
!     Compute decimal time in days, months, years, etc
!     NOTE:  newhour is not set initially
!
!-----------------------------------------------------------------------

      call get_tday

      call int_to_char (4,iyear,cyear)
      cday    = cdays     (iday)
      cmonth  = cmonths   (imonth)
      cmonth3 = month3_all(imonth)

      nsteps_this_interval = 0
      nsteps_run = 0

!-----------------------------------------------------------------------
!
!     thour00_begin_this_year, thour00_midmonth_{equal,calendar} and 
!       thour00_endmonth_{equal,calendar} are used in forcing routines 
!       with the 'monthly-equal' or  'monthly-calendar'  option for 
!       forcing_data_freq, where 'monthly-equal' designates 12 equally 
!       spaced months of length 365/12 days and 'monthly-calendar' uses 
!       the non-leapyear calendar.
!       
!     thour00_midmonth_{equal,calendar} and 
!     thour00_endmonth_{equal,calendar} 
!       are relative to the beginning of the year, so vary between 
!       0 and 365*24.
!
!-----------------------------------------------------------------------

      thour00_begin_this_year = thour00
     &   - (seconds_this_year/seconds_in_hour)

      thour00_midmonth_calendar(1) = 24.0_dbl_kind*p5*days_in_month(1)
      thour00_endmonth_calendar(1) = 24.0_dbl_kind*days_in_month(1)

      thour00_endmonth_equal(1) = hours_in_year/12.0_dbl_kind
      thour00_midmonth_equal(1) = p5*thour00_endmonth_equal(1)

      do nm = 2,12

        thour00_endmonth_calendar(nm) =
     &                        thour00_endmonth_calendar(nm - 1)
     &                        + 24.0_dbl_kind*days_in_month(nm)
        thour00_midmonth_calendar(nm) =
     &                        thour00_endmonth_calendar(nm - 1)
     &                        + 24.0_dbl_kind*p5*days_in_month(nm)

        thour00_endmonth_equal(nm) = thour00_endmonth_equal(1)*nm
        thour00_midmonth_equal(nm) = thour00_midmonth_equal(nm-1)
     &                             + thour00_endmonth_equal(1)

      enddo

!-----------------------------------------------------------------------
!
!     set midnight
!
!-----------------------------------------------------------------------

      if (ihour == 0 .and. iminute == 0 .and. isecond == 0) then
        midnight = .true.
      else
        midnight = .false.
      endif

!-----------------------------------------------------------------------
!
!     save iyear, imonth, etc from the beginning of this run
!
!-----------------------------------------------------------------------
      
      iyear_start_run        = iyear
      imonth_start_run       = imonth
      iday_start_run         = iday
      ihour_start_run        = ihour
      iminute_start_run      = iminute
      isecond_start_run      = isecond

      iday_of_year_start_run = iday_of_year

!-----------------------------------------------------------------------
!
!     error checking -- after restart file has been read
!
!-----------------------------------------------------------------------

      if (tmix_iopt == tmix_avgfit) then 
        if (.not. midnight) then
             call exit_POP ('the model run must start at a day boundary'
     &       // ' when using the avgfit option',normal_exit=.false.)
        endif
      endif
 
!-----------------------------------------------------------------------
!
!     will this run end exactly at midnight?
!       (this tests only obvious possibilities)
!
!-----------------------------------------------------------------------

      if ( is_near (mod (seconds_in_day, dtt),c0,dt_tol)  .and.
     &     is_near (seconds_this_day,         c0,dt_tol) ) then
        end_run_at_midnight = .true.
      else
        end_run_at_midnight = .false.
      endif

      if (tmix_iopt == tmix_avgfit) end_run_at_midnight = .true.
 
!-----------------------------------------------------------------------
!
!     determine iyear, imonth, and iday for the end of this run
!
!-----------------------------------------------------------------------

      stop_iopt = stop_opt_sometime
 
      select case (stop_option)        
 
      case ('never')   !*** coupler or signal catcher stops POP     
        stop_iopt = stop_opt_never     
        iyear_end_run  = 9999
        imonth_end_run = 1        
        iday_end_run   = 1              
        elapsed_days_max = 1e9  
 
 
      case ('eoy')     !*** stop at end of stop_count years

        if (end_run_at_midnight) then
          iyear_end_run  = iyear + stop_count
          imonth_end_run = 1        
          iday_end_run   = 1              
        else
          if (imonth  == 12 .and. iday == 31) then
            iyear_end_run = iyear + stop_count
          else
            iyear_end_run = iyear + stop_count - 1
          endif 
          imonth_end_run = 12
          iday_end_run   = 31
        endif

      case ('eom')     !*** stop at end of stop_count months
 
        iyear_end_run  = iyear
        imonth_end_run = imonth + stop_count 
 
        call reduce_months (imonth_end_run, iyear_end_run )
 
        if (end_run_at_midnight) then
          iday_end_run = 1
        else
          iday_end_run = days_in_month(imonth_end_run)
        endif
 
      case ('eod')     !*** stop at end of stop_count days

        if (end_run_at_midnight) then
          iyear_end_run  = iyear
          imonth_end_run = imonth 
          iday_end_run   = iday + stop_count
        else
          iyear_end_run  = iyear
          imonth_end_run = imonth 
          iday_end_run   = iday + stop_count - 1
        endif
 
      case ('nyear', 'nyears') !*** stop after stop_count years
                               !*** need not be end of year

        iyear_end_run  = iyear + stop_count
        imonth_end_run = imonth
        iday_end_run   = iday           
        if (allow_leapyear .and. is_leapyear(iyear_end_run)) then
          days_in_year_end_run = days_in_leap_year
        else
          days_in_year_end_run = days_in_norm_year
        endif
        if (is_near(mod(seconds_in_day*days_in_year_end_run, dtt)  
     &,             c0, dt_tol) ) then
          end_run_at_midnight = .true.
        else
          end_run_at_midnight = .false.
        endif
 
      case ('nmonth', 'nmonths')  !*** stop after stop_count months
                                  !*** need not be end of month
        iyear_end_run  = iyear   
        imonth_end_run = imonth + stop_count 
        iday_end_run   = iday           

        call reduce_months (imonth_end_run, iyear_end_run )

      case ('nday', 'ndays')    !*** stop after stop_count days
                                !*** identical to 'eod'

        if (end_run_at_midnight) then
          iyear_end_run  = iyear
          imonth_end_run = imonth 
          iday_end_run   = iday + stop_count
        else
          iyear_end_run  = iyear
          imonth_end_run = imonth 
          iday_end_run   = iday + stop_count - 1
        endif
 
      case ('nstep', 'nsteps')   !*** stop after stop_count steps

        ndays_temp     = stop_count/steps_per_day
        iday_end_run   = iday + ndays_temp 
        iyear_end_run  = iyear
        imonth_end_run = imonth 
 
      case ('date')

        call date2ymd (stop_count, iyear_end_run,
     &                 imonth_end_run, iday_end_run)
 
      case default
        call exit_POP('invalid stop_option: '//stop_option
     &,                normal_exit=.false.)
      end select
 
!-----------------------------------------------------------------------
!
!     if necessary, adjust iyear_end_run, imonth_end_run, iday_end_run
!
!-----------------------------------------------------------------------

      if (is_leapyear(iyear_end_run)) then
        leapyear_test = .true.
      else
        leapyear_test = .false.
      endif

      if (imonth_end_run == 2 .and. stop_option == 'eom' ) then

        if (end_run_at_midnight) then
          imonth_end_run = 3
          iday_end_run   = 1
        else if (leapyear_test) then
          imonth_end_run = 2
          iday_end_run   = 29
        else 
          imonth_end_run = 2
          iday_end_run   = 28
        endif
 
      else if (imonth_end_run == 2 .and. iday_end_run == 29) then
 
        if (.not. leapyear_test) then
          if (end_run_at_midnight) then
            imonth_end_run = 3
            iday_end_run   = 1
          else
            imonth_end_run = 2
            iday_end_run   = 28
          endif
        endif

      else
 
        if (imonth_end_run == 2 .and. leapyear_test) then
          days_in_month_temp = 29
        else
          days_in_month_temp = days_in_month(imonth_end_run)
        endif
 
        do while (iday_end_run > days_in_month_temp)
 
          iday_end_run   = iday_end_run - days_in_month_temp
          imonth_end_run = imonth_end_run + 1
 
          call reduce_months (imonth_end_run, iyear_end_run )
 
          if (allow_leapyear .and. is_leapyear(iyear_end_run)) then
            leapyear_test = .true.
          else
            leapyear_test = .false.
          endif
 
          if (imonth_end_run == 2 .and. is_leapyear(iyear_end_run)) then
            days_in_month_temp = 29
          else
            days_in_month_temp = days_in_month(imonth_end_run)
          endif
 
        enddo

      endif
 

      call ymd2eday (iyear_end_run, imonth_end_run, iday_end_run
     &,              elapsed_days_end_run)
 
      if (stop_iopt /= stop_opt_never)
     &elapsed_days_max = elapsed_days_end_run +
     &                   (dtt+dt_tol)/seconds_in_day

      if (elapsed_days_end_run < elapsed_days ) then
        call int_to_char (4,iyear_end_run, cyear_end_run)
        cmonth_end_run = cmonths(imonth_end_run)
        cday_end_run   = cdays  (iday_end_run  )
        if (my_task == master_task) then
            write(stdout,1100) 
            write(stdout,1110) ' Starting date: ', cyear,cmonth,cday
            if (stop_iopt /= stop_opt_never) then
              write(stdout,1110) ' Ending date  : ', cyear_end_run
     &,                            cmonth_end_run, cday_end_run
            else
              write(stdout,*)    ' No ending date. Model relies on'
     &                         //' external signal for stopping.'
            endif
        endif
        call exit_POP('invalid end date',normal_exit=.false.)
      endif
 
!-----------------------------------------------------------------------
!
!     print various time manager options to log (stdout)
!
!-----------------------------------------------------------------------

      call write_time_manager_options

!-----------------------------------------------------------------------

 1100 format (/,1x, ' Cannot end at a date earlier than starting date.')
 1110 format (1x,a, 2x, a4,'-',a2,'-',a2)
 
!-----------------------------------------------------------------------

      end subroutine init_time_manager2

!***********************************************************************

      subroutine time_manager (lcoupled, liceform)

 
!-----------------------------------------------------------------------
!
!     this routine updates various time-related variables and is
!     called once at the beginning of each timestep
!
!-----------------------------------------------------------------------

      logical (kind=log_kind), intent(in) ::
     &  lcoupled
     &, liceform


!-----------------------------------------------------------------------
!
!     save previous values of tsecond, isec, imin, ihour, etc
!
!-----------------------------------------------------------------------

      tsecond_old = tsecond

      iyear_last        = iyear
      imonth_last       = imonth
      iday_last         = iday
      iday_of_year_last = iday_of_year
      ihour_last        = ihour

      eod_last          = eod
      eom_last          = eom
      eoy_last          = eoy
 
      midnight_last     = midnight
 
!-----------------------------------------------------------------------
!
!     set logical switches to default values
!
!-----------------------------------------------------------------------

      call reset_switches

!-----------------------------------------------------------------------
!
!     increment the timestep counters
!
!-----------------------------------------------------------------------

      nsteps_run   = nsteps_run   + 1
      nsteps_total = nsteps_total + 1
 
      if (tmix_iopt == tmix_avgfit) then
         nsteps_this_interval = nsteps_this_interval + 1
         if (nsteps_this_interval > nsteps_per_interval) 
     &       nsteps_this_interval = 1
      endif
 

!-----------------------------------------------------------------------
!
!     activate logical switches which depend on nsteps_run, nsteps_total
!
!-----------------------------------------------------------------------

      call set_switches

!-----------------------------------------------------------------------
!
!     determine size of this timestep
!
!-----------------------------------------------------------------------

      if (avg_ts .or. back_to_back) then
        stepsize = p5*dtt
      else
        stepsize =    dtt
      endif

!-----------------------------------------------------------------------
!
!     compute seconds_this_year and seconds_the_day for this timestep
!
!-----------------------------------------------------------------------

      if (nsteps_total == 1 .or.  new_dtt_value) then
         seconds_this_year  = seconds_this_year  + stepsize
         seconds_this_day   = seconds_this_day   + stepsize
      else
         seconds_this_year  = seconds_this_year_next
         seconds_this_day   = seconds_this_day_next
      endif


!-----------------------------------------------------------------------
!
!     determine the size of the next timestep
!
!-----------------------------------------------------------------------

      if ( avg_ts_next .or. back_to_back_next ) then
        stepsize_next = p5*dtt
      else
        stepsize_next =    dtt
      endif

!-----------------------------------------------------------------------
!
!     compute seconds_this_year and seconds_the_day for next timestep
!
!-----------------------------------------------------------------------

      seconds_this_year_next =  seconds_this_year + stepsize_next
      seconds_this_day_next  =  seconds_this_day  + stepsize_next

!-----------------------------------------------------------------------
!
!     adjust seconds counters if necessary
!
!-----------------------------------------------------------------------

      if (nsteps_total == 1 .or.  new_dtt_value) then
        call reduce_seconds (seconds_this_day     , seconds_this_year
     &,                      adjust_year)
        call reduce_seconds (seconds_this_day_next
     &,                      seconds_this_year_next, adjust_year_next)
      else
        adjust_year = adjust_year_next
        call reduce_seconds (seconds_this_day_next
     &,                      seconds_this_year_next, adjust_year_next)
      endif

!-----------------------------------------------------------------------
!
!     compute present year, month, day, hour, minute, and second
!
!-----------------------------------------------------------------------

      call model_date

!-----------------------------------------------------------------------
!
!     compute decimal days, months, and years
!
!-----------------------------------------------------------------------

      call get_tday
 

      if (ihour /= ihour_last) newhour = .true.

!-----------------------------------------------------------------------
!
!     reset thour00_begin_this_year to be the current time if new year.
!
!-----------------------------------------------------------------------

      if (eoy) thour00_begin_this_year = thour00
 
!-----------------------------------------------------------------------
!
!     does this timestep follow a coupled timestep? If so, use
!       matsuno timestep for balanced budgets.  For restarts to work
!       properly, this check must precede set_time_flag_all 
!       *** applies only to first timestep after restart ***
!
!-----------------------------------------------------------------------
 
       if (tmix_iopt == tmix_matsuno) then
         if (nsteps_run == 1 .and. .not. first_step .and. 
     &       check_time_flag_last(coupled_ts) ) then
             matsuno_ts = .true.
             leapfrogts = .false.
         endif
       endif
 


!-----------------------------------------------------------------------
!
!     set all user-defined time flags
!
!-----------------------------------------------------------------------

      call set_time_flag_all


!-----------------------------------------------------------------------
!
!     ice formation or sample qflux this timestep?
!     this section must follow call set_time_flag_all
!
!-----------------------------------------------------------------------
      
      if (liceform) then
         if (lcoupled) then

           if (check_time_flag(coupled_ts) ) then
             if (tmix_iopt == tmix_matsuno .or.
     &           tmix_iopt == tmix_avgfit       ) then
                 ice_ts = .true.
             else
                 call exit_POP ('cannot use tmix_avg or tmix_avgbb '
     &           //'with lcoupled and liceform',normal_exit=.false.)
             endif
           
             if (avg_ts) call exit_POP ('cannot have coupled timestep '
     &            //'be an averaging timestep',normal_exit=.false.)
           endif

           if ( tmix_iopt == tmix_avgfit ) then
             if ( mod(nsteps_this_interval+1,nsteps_per_interval)
     &            == 0 )  ice_ts = .true.
           endif
 
         else  ! .not. lcoupled
 
           if (tmix_iopt == tmix_matsuno) then
              if (mod(nsteps_total,time_mix_freq) == time_mix_freq-1)
     &            ice_ts = .true.
           else if (tmix_iopt == tmix_avgfit  .or.
     &              tmix_iopt == tmix_avg          ) then
                  ice_ts = .true.
           else
              call exit_POP ('tmix_avgbb option is inconsistent with '
     &                   //  'ice_ts',normal_exit=.false.)
           endif
 
 
         endif
 
         if (ice_ts) sample_qflux_ts = .true.
 
      endif

 
 
!-----------------------------------------------------------------------
!
!     does this timestep follow a coupled timestep? If so, use
!       matsuno timestep for balanced budgets.  For all timesteps
!       after the first step of a restart, this check must follow
!       set_time_flag_all 
!
!-----------------------------------------------------------------------
 
       if (tmix_iopt == tmix_matsuno) then
        if (check_time_flag_last(coupled_ts) .and. nsteps_run > 1 ) then
          matsuno_ts = .true.
          leapfrogts = .false.
        endif
       endif
 
!-----------------------------------------------------------------------
!
!     stop after this timestep?
!
!-----------------------------------------------------------------------

      if (stop_option == 'nstep' .or. stop_option == 'nsteps') then
        if (nsteps_run == stop_count) 
     &    call set_time_flag(stop_now,.true.)

      else if (stop_iopt /= stop_opt_never .and. eod) then
 
        if (iyear == iyear_end_run .and. imonth == imonth_end_run
     &                             .and. iday == iday_end_run) then

          call set_time_flag(stop_now,.true.)
 
          if (stop_option == 'eoy' .and. .not. eoy) then
            call set_time_flag(stop_now,.false.)
          endif
          if (stop_option == 'eom' .and. .not. eom) then
            call set_time_flag(stop_now,.false.)
          endif
 
        elseif (elapsed_days > elapsed_days_max ) then

          call set_time_flag(stop_now,.true.)

        endif

        if (stop_option == 'eoy' .and.  eoy  .and.
     &      elapsed_years_this_run == stop_count)
     &    call set_time_flag(stop_now,.true.)

        if (stop_option == 'eom' .and.  eom  .and.
     &      elapsed_months_this_run == stop_count)
     &    call set_time_flag(stop_now,.true.)
 
      endif


      new_dtt_value = .false.

 
!-----------------------------------------------------------------------
!     report ocn model time daily
!-----------------------------------------------------------------------

      if (eod) then
       if (my_task == master_task) then
           write(stdout,1000) iyear, cmonth3, iday
     &,                       seconds_this_day
           call shr_sys_flush(stdout)
       endif
      endif
1000  format ('(time_manager)', ' ocn date ', i4.4, '-', a3, '-',
     &        i2.2,', ', 1pe12.6, ' sec') 
!-----------------------------------------------------------------------

      end subroutine time_manager

!***********************************************************************

      subroutine reset_switches

!-----------------------------------------------------------------------
!
!     set most logical switches to default values
!
!-----------------------------------------------------------------------

      eod                = .false.  ! not end-of-day
      eom                = .false.  ! not end-of-month
      eoy                = .false.  ! not end-of-year

      newday             = .false.  ! not new day
      newhour            = .false.  ! not new hour

      leapfrogts         = .true.   ! a leapfrog timestep
      f_euler_ts         = .false.  ! not a forward Euler timestep
      matsuno_ts         = .false.  ! not an Euler-backward timestep
      avg_ts             = .false.  ! not a time-averaging timestep
      avg_ts_next        = .false.  ! not the step before avg step
      back_to_back       = .false.  ! not a second time-averaging step
      back_to_back_next  = .false.  ! not the step before 2nd avg step
      ice_ts             = .false.  ! not an ice timestep
      sample_qflux_ts    = .false.  ! do not sample qflux 

      call reset_time_flag_all

!-----------------------------------------------------------------------

      end subroutine reset_switches

!***********************************************************************

      subroutine set_switches              

!-----------------------------------------------------------------------
!
!     determine if logical switches should be set to non-defalut values
!         this timestep
!
!     the switches set in this subroutine must depend ONLY on
!         nsteps_run, or nsteps_total
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     if first step, take Euler step
!
!-----------------------------------------------------------------------

      if (first_step) then 
        leapfrogts = .false.
        f_euler_ts = .true.
        newday     = .true.
        first_step = .false.
      endif

!-----------------------------------------------------------------------
!
!     set avg_ts flags    (avg or avgbb options)
!
!-----------------------------------------------------------------------

      if (tmix_iopt == tmix_avg .or. tmix_iopt == tmix_avgbb) then
        if (mod(nsteps_total+1,time_mix_freq) == 0)
     &     avg_ts_next  = .true.
        if (mod(nsteps_total  ,time_mix_freq) == 0) 
     &     avg_ts       = .true.
      endif
            
 
!-----------------------------------------------------------------------
!
!     set back-to-back flags  (avgbb option)   
!
!-----------------------------------------------------------------------
 
      if (tmix_iopt == tmix_avgbb) then
        if (avg_ts)
     &     back_to_back_next  = .true.
        if (nsteps_total > 1 .and.
     &     mod(nsteps_total-1,time_mix_freq) == 0) 
     &     back_to_back       = .true.
      endif
 
!-----------------------------------------------------------------------
!
!     set avg_ts flags (avgfit option)
!
!-----------------------------------------------------------------------
 
      if (tmix_iopt == tmix_avgfit) then
        if      (nsteps_this_interval == 1) then
           avg_ts_next = .true.
        else if (nsteps_this_interval == 2) then
           avg_ts      = .true.
        else if (mod(nsteps_this_interval+1,time_mix_freq) == 0) then
           avg_ts_next = .true.
        else if (mod(nsteps_this_interval  ,time_mix_freq) == 0) then
           avg_ts      = .true.
        endif
 
!-----------------------------------------------------------------------
!     don't allow an averaging step in the first step of an interval
!-----------------------------------------------------------------------

        if (nsteps_this_interval == nsteps_per_interval) then
            avg_ts_next = .false.   
        endif
 
      endif

!-----------------------------------------------------------------------
!
!     use Euler backward timestep this timestep?
!
!-----------------------------------------------------------------------

      if (tmix_iopt == tmix_matsuno .and. nsteps_total /= 1) then
        if (mod(nsteps_total,time_mix_freq) == 0 .or.
     &      time_mix_freq == 1) then
          matsuno_ts = .true.
          leapfrogts = .false.
        endif
      endif

!-----------------------------------------------------------------------

      end subroutine set_switches


!***********************************************************************

      function init_time_flag(flag_name, default,
     &                        freq_opt, freq )

!-----------------------------------------------------------------------
!
!     creates a user-defined time flag with optional default values
!     and frequency.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      character (*), intent(in) ::
     &  flag_name               ! name for this flag

      logical (kind=log_kind), intent(in), optional ::
     &  default                 ! default state for this flag

      integer (kind=int_kind), intent(in), optional ::
     &  freq_opt                ! optional freq option for setting flag
     &, freq                    ! freq in above units  for setting flag

!-----------------------------------------------------------------------
!
!     output
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  init_time_flag          ! integer index into time flag array
                                !   used as a flag id

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  n                      ! dummy loop index
     &, isearch                ! index of flag during search

!-----------------------------------------------------------------------
!
!     search to determine if flag already exists
!
!-----------------------------------------------------------------------

      isearch = 0
      flag_search: do n=1,num_time_flags
        if (trim(time_flags(n)%name) == flag_name) then
          isearch = n
          exit flag_search
        endif
      end do flag_search

!-----------------------------------------------------------------------
!
!     if no flag defined, define new flag and initialize
!
!-----------------------------------------------------------------------

      if (isearch == 0) then  ! no flag exists - define new flag

        num_time_flags = num_time_flags + 1
        isearch = num_time_flags

        time_flags(isearch)%name = flag_name

        time_flags(isearch)%has_default         = .false.
        time_flags(isearch)%default             = .false.
        time_flags(isearch)%freq_opt            = freq_opt_never
        time_flags(isearch)%freq                = 0
        time_flags(isearch)%value               = .false.
        time_flags(isearch)%old_value           = .false.
      endif

!-----------------------------------------------------------------------
!
!     set default if requested
!
!     NOTE: If flag previously defined and optional arguments are 
!           present, this will override any previous definition of 
!           optional arguments. user must make sure calls do not 
!           contain optional arguments or else that the last call to 
!           this routine for a specific flag contains desired values.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     set default value of flag
!
!-----------------------------------------------------------------------

      if (present(default)) then
        time_flags(isearch)%has_default = .true.
        time_flags(isearch)%default     = default
        time_flags(isearch)%value       = default
        time_flags(isearch)%old_value   = default
      endif

!-----------------------------------------------------------------------
!
!     define optional frequency for setting flag
!
!-----------------------------------------------------------------------

      if (present(freq_opt)) then
        time_flags(isearch)%freq_opt = freq_opt
        time_flags(isearch)%freq     = freq
      endif


!-----------------------------------------------------------------------
!
!     print time_flag information for debugging purposes
!
!-----------------------------------------------------------------------
 
      if (debug_time_management .and. my_task == master_task) then
        write(stdout,*) ' initialize time_flag(',isearch,')'
 
        write(stdout,*)  '   name              = ' 
     &, trim(time_flags(isearch)%name)
 
        write(stdout,*) '   has_default        = '  
     &, time_flags(isearch)%has_default      
 
        write(stdout,*) '   default            = '  
     &, time_flags(isearch)%default          
 
        write(stdout,*) '   freq_opt           = '  
     &, time_flags(isearch)%freq_opt         
 
        write(stdout,*) '   freq               = '  
     &, time_flags(isearch)%freq             
 
        write(stdout,*) '   value              = '  
     &, time_flags(isearch)%value            
 
        write(stdout,*) '   old_value          = '  
     &, time_flags(isearch)%old_value        
 
 
        write(stdout,*) '   '
      endif

!-----------------------------------------------------------------------
!
!     return array index as integer flag id
!
!-----------------------------------------------------------------------

      init_time_flag = isearch

!-----------------------------------------------------------------------

      end function init_time_flag

!***********************************************************************

      subroutine set_time_flag(flag_id, value)

!-----------------------------------------------------------------------
!
!     sets the time flag given by flag_id to the value
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  flag_id                ! index of flag array identifying flag

      logical (kind=log_kind), intent(in) ::
     &  value                  ! value requested for flag

!-----------------------------------------------------------------------
!
!     check for proper flag id and then set flag
!
!-----------------------------------------------------------------------

      if (flag_id < 1 .or. flag_id > num_time_flags)
     &  call exit_POP('set_time_flag: invalid flag_id'
     &,                normal_exit=.false.)

      time_flags(flag_id)%value = value

!-----------------------------------------------------------------------

      end subroutine set_time_flag

!***********************************************************************

      subroutine set_time_flag_last(flag_id, old_value)

!-----------------------------------------------------------------------
!
!     sets the old value of time flag given by flag_id to old_value
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  flag_id                ! index of flag array identifying flag

      logical (kind=log_kind), intent(in) ::
     &  old_value              ! old value requested for flag

!-----------------------------------------------------------------------
!
!     check for proper flag id and then set flag
!
!-----------------------------------------------------------------------

      if (flag_id < 1 .or. flag_id > num_time_flags)
     &  call exit_POP('(set_time_flag_last) invalid flag_id'
     &,                normal_exit=.false.)

      time_flags(flag_id)%old_value = old_value

!-----------------------------------------------------------------------

      end subroutine set_time_flag_last

!***********************************************************************

      subroutine reset_time_flag(flag_id)

!-----------------------------------------------------------------------
!
!     sets the time flag given by flag_id to default value
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  flag_id                ! index of flag array identifying flag

!-----------------------------------------------------------------------
!
!     check for proper flag id and then set flag
!
!-----------------------------------------------------------------------

      if (flag_id < 1 .or. flag_id > num_time_flags)
     &  call exit_POP('(reset_time_flag) invalid flag_id'
     &,                normal_exit=.false.)

      if (time_flags(flag_id)%has_default) then
        time_flags(flag_id)%value = time_flags(flag_id)%default
      endif

!-----------------------------------------------------------------------

      end subroutine reset_time_flag

!***********************************************************************

      subroutine reset_time_flag_all

!-----------------------------------------------------------------------
!
!     sets all time flags to default value (if exists)
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n  ! dummy index

!-----------------------------------------------------------------------

      do n=1,num_time_flags
        if (time_flags(n)%has_default) then
            time_flags(n)%value = time_flags(n)%default
        endif
      end do

!-----------------------------------------------------------------------

      end subroutine reset_time_flag_all

!***********************************************************************

      function check_time_flag(flag_id)

!-----------------------------------------------------------------------
!
!     returns the current value of time flag given by flag_id
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  flag_id                ! index of flag array identifying flag

!-----------------------------------------------------------------------
!
!     output
!
!-----------------------------------------------------------------------

      logical (kind=log_kind) ::
     &  check_time_flag

!-----------------------------------------------------------------------
!
!     check for proper flag id and then return flag value
!
!-----------------------------------------------------------------------

      if (flag_id < 1 .or. flag_id > num_time_flags)
     &  call exit_POP('(check_time_flag) invalid flag_id'
     &,                normal_exit=.false.)

      check_time_flag = time_flags(flag_id)%value

!-----------------------------------------------------------------------

      end function check_time_flag

!***********************************************************************

      function check_time_flag_last(flag_id)

!-----------------------------------------------------------------------
!
!     returns the value of time flag given by flag_id at previous
!     time step
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  flag_id                ! index of flag array identifying flag

!-----------------------------------------------------------------------
!
!     output
!
!-----------------------------------------------------------------------

      logical (kind=log_kind) ::
     &  check_time_flag_last

!-----------------------------------------------------------------------
!
!     check for proper flag id and then return old flag value
!
!-----------------------------------------------------------------------

      if (flag_id < 1 .or. flag_id > num_time_flags)
     &  call exit_POP('(check_time_flag_last) invalid flag_id'
     &,                normal_exit=.false.)

      check_time_flag_last = time_flags(flag_id)%old_value

!-----------------------------------------------------------------------

      end function check_time_flag_last

!***********************************************************************

      subroutine set_time_flag_all

!-----------------------------------------------------------------------
!
!     sets all time flags based on frequency options
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n  ! dummy index

!-----------------------------------------------------------------------
!
!     if it is time, set time flag and save value from old time
!
!-----------------------------------------------------------------------

      do n=1,num_time_flags

        if (time_flags(n)%freq_opt /= freq_opt_never) then

          time_flags(n)%old_value = time_flags(n)%value
          time_flags(n)%value = time_to_do(time_flags(n)%freq_opt, 
     &                                     time_flags(n)%freq)

        endif

      end do

!-----------------------------------------------------------------------

      end subroutine set_time_flag_all

!***********************************************************************

      function time_to_do (in_freq_opt, in_freq)

!-----------------------------------------------------------------------
!
!     this function determines whether it is time to take a
!     particular action based on input frequency options.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  in_freq_opt,           ! frequency option for this action
     &  in_freq                ! frequency in above intervals for action

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      logical (kind=log_kind) ::
     &  time_to_do             ! true if current timestep matches input
                               ! frequency conditions

!-----------------------------------------------------------------------

      time_to_do = .false.

      select case (in_freq_opt)

      case (freq_opt_nyear)
        if (eoy .and. mod(elapsed_years_init_date,in_freq) == 0)
     &      time_to_do = .true.

      case (freq_opt_nmonth)
        if (eom .and. mod(elapsed_months_init_date,in_freq) == 0)
     &      time_to_do = .true.

      case (freq_opt_nday)
        if (eod) then
          if (midnight) then
            if (mod(elapsed_days_init_date  ,in_freq) == 0) 
     &          time_to_do = .true.
          else
            if (mod(elapsed_days_init_date+1,in_freq) == 0) 
     &          time_to_do = .true.
          endif
        endif

      case (freq_opt_nhour)
        if (newhour .and. mod(ihour,in_freq) == 0) time_to_do = .true.

      case (freq_opt_nsecond)
        if (mod(isecond,in_freq) == 0) time_to_do = .true.

      case (freq_opt_nstep)
        if (mod(nsteps_total,in_freq) == 0) time_to_do = .true.

      case default
      end select

!-----------------------------------------------------------------------

      end function time_to_do

!***********************************************************************

      function time_to_start (in_start_opt, in_start)

!-----------------------------------------------------------------------
!
!     this function determines whether it is time to start a
!     particular function based on input start options.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  in_start_opt,           ! start option for this action
     &  in_start                ! start after value

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      logical (kind=log_kind) ::
     &  time_to_start          ! true if current timestep matches input
                               ! start conditions

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  eday_loc               ! temporary value for elapsed days

!-----------------------------------------------------------------------
!
!     check start conditions - do not start if called from initial
!     (nsteps_run == 0) and the condition matches exactly - the 
!     start will instead be triggered during the time step.  This 
!     avoids looking for restarts that do not yet exist.
!
!-----------------------------------------------------------------------

      time_to_start = .false.

      select case (in_start_opt)

      case(start_opt_nstep)
        if (nsteps_total > in_start) then
          time_to_start = .true.
        else if (nsteps_total == in_start .and. nsteps_run /= 0) then
          time_to_start = .true.
        endif

      case(start_opt_nday)
        if (elapsed_days_init_date > in_start) then
          time_to_start = .true.
        else if (elapsed_days_init_date == in_start .and. 
     &           nsteps_run /= 0) then
          time_to_start = .true.
        endif

      case(start_opt_nyear)
        if (elapsed_years_init_date > in_start) then
          time_to_start = .true.
        else if (elapsed_years_init_date == in_start .and. 
     &           nsteps_run /= 0) then
          time_to_start = .true.
        else if (elapsed_years_init_date == in_start .and. 
     &           elapsed_days_this_year > 1) then
          time_to_start = .true.
        endif

      case(start_opt_date)
        call date2eday (in_start, eday_loc)
        if (elapsed_days > eday_loc) then
          time_to_start = .true.
        else if (elapsed_days == eday_loc .and. nsteps_run /= 0) then
          time_to_start = .true.
        endif

      case default
        call exit_POP('unknown start option in time_to_start'
     &,                normal_exit=.false.)
      end select

!-----------------------------------------------------------------------

      end function time_to_start

!***********************************************************************

      subroutine model_date

!-----------------------------------------------------------------------
!
!     determine  iyear, imonth, iday, ihour, iminute, isecond, as
!       well as elapsed days, months, and years
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) ::
     &  rhour                   ! number of hours   elapsed today
     &, rminute                 ! number of minutes beyond the hour
     &, rsecond                 ! number of seconds beyond the minute
     &, rhour_next              ! rhour   for next timestep
     &, rminute_next            ! rminute for next timestep
     &, rsecond_next            ! rsecond for next timestep
     &, seconds_today           ! number of seconds elapsed today
 
      integer (kind=int_kind) ::
     &  day_inc                 ! change in the number of days elapsed
                                !   between timesteps

      logical (kind=log_kind), save ::
     &  increment_elapsed_months_next = .false.
     &, increment_elapsed_years_next  = .false.
 
!-----------------------------------------------------------------------
!
!     determine iyear
!
!-----------------------------------------------------------------------

      if (adjust_year) then
        iyear = iyear + 1
        days_in_prior_year = days_in_year
        if (allow_leapyear) call leap_adjust
        adjust_year_next = .false.
      endif

!-----------------------------------------------------------------------
!
!     determine iday_of_year, imonth, iday, ihour, iminute, isecond
!                                           rhour, rminute, rsecond
!
!-----------------------------------------------------------------------

      if (nsteps_run == 1) then
        call ymd_hms   ( seconds_this_year, seconds_this_day
     &,                  iday_of_year
     &,                  imonth, iday   , iday_last
     &,                  ihour , iminute, isecond
     &,                  rhour , rminute, rsecond
     &,                  midnight       , adjust_year)
      else
        iday_of_year  = iday_of_year_next
        imonth        = imonth_next
        iday          = iday_next
        ihour         = ihour_next
        iminute       = iminute_next
        isecond       = isecond_next
        rhour         = rhour_next
        rminute       = rminute_next
        rsecond       = rsecond_next
        midnight      = midnight_next
      endif

!-----------------------------------------------------------------------
!
!     determine iday_of_year, imonth, iday, etc, for next timestep
!
!-----------------------------------------------------------------------
 
      call ymd_hms(seconds_this_year_next
     &,            seconds_this_day_next
     &,            iday_of_year_next
     &,            imonth_next  , iday_next   , iday
     &,            ihour_next   , iminute_next, isecond_next
     &,            rhour_next   , rminute_next, rsecond_next
     &,            midnight_next, adjust_year_next          )

!-----------------------------------------------------------------------
!
!     end of day?
!
!-----------------------------------------------------------------------

      if (iday_of_year_next /= iday_of_year) then
        if (.not. midnight_next)                eod = .true.
        if (stepsize_next + dt_tol > seconds_in_day) eod = .true.
      endif

      if (midnight) eod = .true.

!-----------------------------------------------------------------------
!
!     newday?   (a timestep can be both eod and newday for dt > 24hrs)
!
!-----------------------------------------------------------------------
 
      if (iday_of_year > iday_of_year_last .and. .not. midnight) 
     &               newday = .true.
      if (eod_last ) newday = .true.

!-----------------------------------------------------------------------
!
!     end of month?
!
!-----------------------------------------------------------------------

      if (eod) then
 
        if (eom_next) then
          eom      = .true.
          eom_next = .false.
        else
 
          if (imonth_next > imonth_last  .or.
     &        imonth_next == 1 .and. imonth_last == 12) then
 
            if (iday <= days_in_month(imonth_last) .and. midnight_next
     &          .and. iday_next == 1 ) then
              eom      = .false.
              eom_next = .true.

            elseif (iday <= days_in_month(imonth_last)  .and.
     &                                iday_next >= 1 )  then
              eom      = .true.
              eom_next = .false.

            elseif (midnight .and. midnight_next .and.
     &                             midnight_last) then
              eom      = .false.
              eom_next = .true.
            else
              eom      = .true.
              eom_next = .false.
            endif
          endif
        endif
 
      endif ! eod

!-----------------------------------------------------------------------
!
!     elapsed months (integer) 
!
!-----------------------------------------------------------------------

      if((eom .and. midnight) .or. increment_elapsed_months_next) then
        elapsed_months           = elapsed_months           + 1
        elapsed_months_this_run  = elapsed_months_this_run  + 1
        elapsed_months_init_date = elapsed_months_init_date + 1
        if (increment_elapsed_months_next) 
     &      increment_elapsed_months_next = .false.
      else if (eom) then
        increment_elapsed_months_next = .true.
      else 
        increment_elapsed_months_next = .false.
      endif
 
      if (eom_last) then
        eom                      = .false.
      endif

!-----------------------------------------------------------------------
!
!     end of year?
!
!-----------------------------------------------------------------------

      if (eom .and. imonth_next == 1 .and. imonth_last == 12)
     &  eoy = .true.       

!-----------------------------------------------------------------------
!
!     adjust elapsed years and elapsed days in the year (integer)
!
!-----------------------------------------------------------------------

      if((eoy .and. midnight) .or. increment_elapsed_years_next) then
 
        elapsed_years           = elapsed_years           + 1
        elapsed_years_this_run  = elapsed_years_this_run  + 1
        elapsed_years_init_date = elapsed_years_init_date + 1
 
        call ymd2eday (iyear , 1, 1, elapsed_days_jan1)
        elapsed_days_this_year  = elapsed_days - elapsed_days_jan1
 
        if (increment_elapsed_years_next) 
     &      increment_elapsed_years_next = .false.
      else if (eoy) then
        increment_elapsed_years_next = .true.
      else 
        increment_elapsed_years_next = .false.
      endif
 
      if (eoy_last) then
        eoy = .false.
      endif

!-----------------------------------------------------------------------
!
!     character values for iyear, imonth, iday
!
!-----------------------------------------------------------------------
      
      if (iyear  /= iyear_last ) call int_to_char (4,iyear, cyear)
 
      if (imonth /= imonth_last) then
        cmonth  = cmonths   (imonth)
        cmonth3 = month3_all(imonth)
      endif

      if (iday   /= iday_last) cday = cdays(iday)                

!-----------------------------------------------------------------------
!
!     elapsed number of days (integer)
!
!-----------------------------------------------------------------------

      if (iday_of_year >= iday_of_year_last) then
        day_inc = iday_of_year - iday_of_year_last
      else
        day_inc = iday_of_year - iday_of_year_last + days_in_prior_year
      endif

      elapsed_days           = elapsed_days           + day_inc
      elapsed_days_this_run  = elapsed_days_this_run  + day_inc
      elapsed_days_this_year = elapsed_days_this_year + day_inc
      elapsed_days_init_date = elapsed_days_init_date + day_inc
 
!-----------------------------------------------------------------------
!
!     has a valid date been selected?
!
!-----------------------------------------------------------------------

      if (.not. valid_ymd_hms()) then
        call exit_POP('invalid ymd_hms',normal_exit=.false.)
      endif

!-----------------------------------------------------------------------
 
      end subroutine model_date
 
!***********************************************************************

      subroutine get_tday

!-----------------------------------------------------------------------
!
!     Compute decimal day, month, year, etc
!
!-----------------------------------------------------------------------

      frac_day= seconds_this_day/seconds_in_day
 
      tsecond = elapsed_days_init_date*seconds_in_day + seconds_this_day
      tday    = tsecond/seconds_in_day
      tmonth  = real(elapsed_months_init_date,kind=dbl_kind) +
     &          (real(iday,kind=dbl_kind)-c1+frac_day)/
     &          days_in_month(imonth)
      tyear   = elapsed_years_init_date +
     &          seconds_this_year/seconds_in_year
 
      thour   = tday*24.0_dbl_kind
 

!-----------------------------------------------------------------------
!
!     define tday00 and thour00 for use in forcing routines 
!     and in the output diagnostics files
!     txxx00 is the time in xxx since 0000-01-01
!
!-----------------------------------------------------------------------

      tyear00   = elapsed_years + seconds_this_year/seconds_in_year
      tday00    = elapsed_days + frac_day
      thour00   = tday00*24.0_dbl_kind
      tsecond00 = tday00*seconds_in_day


!-----------------------------------------------------------------------

      end subroutine get_tday

!***********************************************************************

      subroutine ymd_hms   (
     &           seconds_this_year_loc , seconds_this_day_loc
     &,          iday_of_year_loc
     &,          imonth_loc  , iday_loc   , iday_compare
     &,          ihour_loc   , iminute_loc, isecond_loc
     &,          rhour_loc   , rminute_loc, rsecond_loc
     &,          midnight_loc, adjust_year_loc                )

!-----------------------------------------------------------------------
!
!     compute iday_of_year, iyear, imonth, iday, ihour, iminute, isecond
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer  (kind=int_kind), intent(in) ::
     &  iday_compare          ! day to compare to check day change

      real (kind=dbl_kind), intent(inout) ::
     &  seconds_this_year_loc ! number of seconds in year
     &, seconds_this_day_loc  ! number of seconds in day

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      integer  (kind=int_kind), intent(out) ::
     &  imonth_loc            !  local value of imonth
     &, iday_loc              !    etc
     &, ihour_loc
     &, iminute_loc
     &, isecond_loc
     &, iday_of_year_loc

      real (kind=dbl_kind), intent(out) ::
     &  rhour_loc             ! real values for hour,minute,second
     &, rminute_loc
     &, rsecond_loc

      logical (kind=log_kind), intent(out) ::
     &  midnight_loc          ! midnight flag
 
      logical (kind=log_kind), intent(inout) ::
     &    adjust_year_loc     ! year adjustment flag
 
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  itest
     &, nm
     &, ntest

      real (kind=dbl_kind) ::
     &  test_seconds
     &, rtest
     &, r_ntest

!-----------------------------------------------------------------------
!
!     determine day number   [1,days_in_year]
!
!-----------------------------------------------------------------------
 
      rtest = seconds_this_year_loc/seconds_in_day
      itest =  int (rtest)
      ntest = nint (rtest)
      r_ntest = ntest

      if (is_near(rtest, r_ntest, dt_tol)) then
        iday_of_year_loc = ntest + 1
      else
        iday_of_year_loc = itest + 1
      endif
 
!-----------------------------------------------------------------------
!
!     determine month number [1,12]                
!
!-----------------------------------------------------------------------
 
      imonth_loc = 12

      do nm = 1,11
        if (iday_of_year_loc >  days_in_prior_months(nm)  .and.
     &      iday_of_year_loc <= days_in_prior_months(nm+1) )
     &      imonth_loc = nm
      enddo

!-----------------------------------------------------------------------
!
!     determine day-of-month number [1,31]
!
!-----------------------------------------------------------------------
 
      iday_loc = iday_of_year_loc - days_in_prior_months(imonth_loc)
 
!-----------------------------------------------------------------------
!
!     determine integer hour, minute, and second
!
!-----------------------------------------------------------------------
 
      call hms (seconds_this_day_loc
     &,         ihour_loc, iminute_loc, isecond_loc
     &,         rhour_loc, rminute_loc, rsecond_loc)
 
!-----------------------------------------------------------------------
!
!     midnight?
!
!-----------------------------------------------------------------------
 
      if (ihour_loc == 0 .and. iminute_loc == 0 .and.
     &                         isecond_loc == 0) then
        midnight_loc = .true.
      else
        midnight_loc = .false.
      endif

!-----------------------------------------------------------------------
!
!     if midnight, increment iday 
!
!-----------------------------------------------------------------------
 
       if (iday_loc == iday_compare .and. midnight_loc)
     &   iday_loc =  iday_loc + 1

!-----------------------------------------------------------------------
!
!     if necessary, adjust month value and year-adjustment indicator
!
!-----------------------------------------------------------------------
 
      if (iday_loc > days_in_month(imonth_loc)) then
        iday_loc = iday_loc - days_in_month(imonth_loc)
        imonth_loc = imonth_loc + 1
        if (imonth_loc == 13 ) then
          imonth_loc = 1
          adjust_year_loc = .true.
        endif
      endif

      iday_of_year_loc = days_in_prior_months(imonth_loc) + iday_loc

!-----------------------------------------------------------------------

      end subroutine ymd_hms

!***********************************************************************

      subroutine hms (seconds_loc
     &,               ihour_loc  , iminute_loc, isecond_loc
     &,               rhour_loc  , rminute_loc, rsecond_loc)

!-----------------------------------------------------------------------
!
!     Determines present hour, minute, and second
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), intent(inout) ::
     &  seconds_loc         ! elapsed seconds in current day 
 
!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      integer(kind=log_kind), intent(out) ::
     &  ihour_loc           ! hour in current day
     &, iminute_loc         ! minute in current hour
     &, isecond_loc         ! seconds in current minute

       real (kind=dbl_kind), intent(out) ::
     &  rhour_loc           ! real values for the above quantities
     &, rminute_loc
     &, rsecond_loc
 
!-----------------------------------------------------------------------
!
!     compute present hour, minute, and second
!
!-----------------------------------------------------------------------

      rhour_loc   = seconds_loc/seconds_in_hour
      ihour_loc   = rhour_loc

      rminute_loc = (rhour_loc - ihour_loc)*minutes_in_hour
      iminute_loc =  rminute_loc

      rsecond_loc = (rminute_loc - iminute_loc)*seconds_in_minute
      isecond_loc = nint (rsecond_loc)
 
!-----------------------------------------------------------------------
!
!     corrections to second, minute, and/or hour
!
!-----------------------------------------------------------------------

      if (isecond_loc == 60) then
        isecond_loc   =  0
        iminute_loc   =  iminute_loc+ 1
      endif

      if (iminute_loc == 60) then
        iminute_loc   =  0
        ihour_loc     =  ihour_loc + 1
      endif

      if (ihour_loc   == 24) then
        ihour_loc     =  0
      endif

!-----------------------------------------------------------------------
!
!     if h:m:s == 0:00:00, then adjust seconds 
!
!-----------------------------------------------------------------------

      if (ihour_loc == 0  .and. iminute_loc == 0  .and. 
     &    isecond_loc == 0) seconds_loc = c0
 
!-----------------------------------------------------------------------

      end subroutine hms

!***********************************************************************

      subroutine reduce_months (imonth_loc, iyear_loc)

!-----------------------------------------------------------------------
!
!     reduce imonth such that it never exceeds 12;
!     increment iyear accordingly
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent (inout) ::
     &  imonth_loc            ! current value of imonth
     &, iyear_loc             ! current value of iyear

!-----------------------------------------------------------------------
 
      do while (imonth_loc > 12)
        imonth_loc = imonth_loc - 12
        iyear_loc  = iyear_loc  + 1
      enddo
 
!-----------------------------------------------------------------------

      end subroutine reduce_months

!***********************************************************************
 
      subroutine reduce_seconds (seconds_this_day_loc,
     &                           seconds_this_year_loc, adjust_year_loc)

!-----------------------------------------------------------------------
!
!     reduce seconds_this_day and seconds_this year, if either
!       exceeds their bounds (eg due to roundoff)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), intent(inout) ::
     &  seconds_this_day_loc    ! current value of seconds_this_day
     &, seconds_this_year_loc   ! current value of seconds_this_year
 
      logical (kind=log_kind), intent(out) ::
     &  adjust_year_loc         ! flag to signal year adjustment

      integer (kind=int_kind) ::
     &  ns
     &, ns_end

!-----------------------------------------------------------------------
!
!     if seconds_this_day exceeds the number of seconds in a day, then
!       reset seconds_this_day
!
!-----------------------------------------------------------------------

      if (seconds_this_day_loc >= seconds_in_day) then

        ns_end = nint(seconds_this_day_loc/seconds_in_day)
        do ns = 1, ns_end
          if (seconds_this_day_loc + dt_tol >= seconds_in_day)
     &      seconds_this_day_loc = seconds_this_day_loc
     &                           - seconds_in_day
        enddo
 
      endif

!-----------------------------------------------------------------------
!
!     if seconds_this_year exceeds the number of seconds in a year, then
!       reset seconds_this_year
!
!-----------------------------------------------------------------------
    
      if (seconds_this_year_loc >= seconds_in_year - stepsize .and.
     &   (seconds_this_year_loc >= seconds_in_year .or.
     &   is_near(seconds_this_year_loc,seconds_in_year,dt_tol_year)))
     &   then
 
        seconds_this_year_loc = seconds_this_year_loc - seconds_in_year

        if (is_near(seconds_this_year_loc, c0, dt_tol)) then
          seconds_this_year_loc = c0
          seconds_this_day_loc  = c0
        endif
 
        adjust_year_loc  = .true.
      else
        adjust_year_loc  = .false.
      endif

!-----------------------------------------------------------------------
 
      end subroutine reduce_seconds

!***********************************************************************

      subroutine leap_adjust

!---------------------------------------------------------------------  
!
!     Set leap-year related variables
!
!---------------------------------------------------------------------  

      integer (kind=int_kind) :: nm  ! dummy month index

!-----------------------------------------------------------------------
!
!     is iyear a leap year?
!
!-----------------------------------------------------------------------
 
      leapyear = is_leapyear (iyear)
 
!-----------------------------------------------------------------------
!
!     adjust the number of days in February and in the year
!
!-----------------------------------------------------------------------

      if (leapyear) then
        days_in_month(2)  = 29
        days_in_year      = days_in_leap_year
      else
        days_in_month(2)  = 28
        days_in_year      = days_in_norm_year
      endif

      seconds_in_year     = days_in_year*seconds_in_day
      hours_in_year       = days_in_year*24.0_dbl_kind
 
!-----------------------------------------------------------------------
!
!     reset the values of days_in_prior_months(imonth)
!
!-----------------------------------------------------------------------
 
      call prior_days (days_in_prior_months, days_in_month)
 
!-----------------------------------------------------------------------

      end subroutine leap_adjust

!***********************************************************************

      subroutine date2ymd (date,year,month,day)

!-----------------------------------------------------------------------
!
!     Decode the calendar date
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  date            ! Calendar date (integer) in yyyymmdd format

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(out) ::
     &  year                   ! Calendar year
     &, month                  ! Calendar month
     &, day                    ! Calendar day

!-----------------------------------------------------------------------

      if (.not. valid_date(date))call exit_POP('(date2ymd)invalid date'
     &,                normal_exit=.false.)

      year  = int(     date       /10000)
      month = int( mod(date,10000)/  100)
      day   =      mod(date,  100)

!-----------------------------------------------------------------------

      end subroutine date2ymd

!***********************************************************************

      subroutine ymd2date (year,month,day,date)

!-----------------------------------------------------------------------
!
!     Encode the calendar date
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  year                   ! Calendar year
     &, month                  ! Calendar month
     &, day                    ! Calendar day

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(out) ::
     &  date            ! Calendar date (integer) in yyyymmdd format

!-----------------------------------------------------------------------

      date = 10000*year + 100*month + day

!-----------------------------------------------------------------------

      end subroutine ymd2date

!***********************************************************************

      subroutine eday2ymd (eday,year,month,day)

!-----------------------------------------------------------------------
!
!     Determine the year, month, and day number from elapsed days
!     since 0000-01-01
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  eday                   ! elapsed day since 0000-01-01

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(out) ::
     &  year,                  ! calendar year
     &  month,                 ! calendar month
     &  day                    ! calendar day

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(0:3) ::
     &  days_each_year         ! days in year for 4-year cycle

      integer (kind=int_kind), dimension(12)  ::
     &  tdays_in_prior_months  ! temporary days in prior months
     &, tdays_in_month         ! temporary days in each month

      integer (kind=int_kind) ::
     &  nm                     ! dummy month index
     &, test_day
     &, nnorm                  ! normal year counters
     &, nnorm_new              ! normal year counters
     &, nleap                  ! leap year counters
     &, nleap_new              ! leap year counters
     &, cycleindex             ! year index for 4-year cycle
     &, days                   ! day counter
     &, ny                     ! year index
     &, max_ny                 ! max possible number of years

!-----------------------------------------------------------------------
!
!     If leap years are not allowed, then compute number of elapsed
!     years and the number of days in the most recent year
!
!-----------------------------------------------------------------------

      if (.not. allow_leapyear) then
          nnorm = eday/days_in_norm_year + 1
          nleap = 0
          days  = eday -nnorm*days_in_norm_year
          tdays_in_prior_months = days_in_prior_months
      else

!-----------------------------------------------------------------------
!
!      Compute number of elapsed leap years and "normal" years, and
!      the number of days elapsed in the most recent year
!
!      First, initialize arrays used to determine date
!
!-----------------------------------------------------------------------

          days_each_year    = days_in_norm_year
          days_each_year(0) = days_in_leap_year

          tdays_in_month    = days_in_month
          tdays_in_month(2) = 29               ! year 0 value

          call prior_days (tdays_in_prior_months, tdays_in_month)

          days      = 0
          nleap     = 0
          nnorm     = 0
          nleap_new = 0
          nnorm_new = 0

          max_ny    = eday/days_in_norm_year + 1

!-----------------------------------------------------------------------
!
!     Determine the number of elapsed years and the day number of the
!     present year [1,days_in_norm_year]
!
!-----------------------------------------------------------------------

          year_loop: do ny = 0, max_ny

            cycleindex = mod(ny,4)
            year  = nleap + nnorm

            if (cycleindex == 0 .and.
     &         (mod(ny,100) /= 0 .or. mod(ny,400) == 0)) then
                nleap_new = nleap + 1
                tdays_in_month(2) = 29
            else
                nnorm_new = nnorm + 1
                tdays_in_month(2) = 28
            endif

!-----------------------------------------------------------------------
!
!       Update Tdays_in_prior_months for the most recent year
!
!-----------------------------------------------------------------------

           call prior_days (tdays_in_prior_months, tdays_in_month)

           test_day = eday - nnorm*days_in_norm_year
     &                     - nleap*days_in_leap_year

           if (test_day <= days_each_year(cycleindex) ) then
             days = test_day
             exit year_loop
           endif

           nnorm = nnorm_new
           nleap = nleap_new

         enddo year_loop

      endif ! (.not. allow_leapyear)

!-----------------------------------------------------------------------
!
!     Was the number of days this year properly determined?
!
!-----------------------------------------------------------------------

      if (days <= 0 .or. days > days_in_leap_year ) then
        if (my_task == master_task) then
          write(stdout,*   ) 'days = ', days
          write(stdout,1100) 'eday2ymd', 'days undetermined'
        endif
        call exit_POP('eday2ymd',normal_exit=.false.)
      endif

!-----------------------------------------------------------------------
!
!     Determine the day- and month-numbers for this year
!
!-----------------------------------------------------------------------

      month = 0
      day   = 0

      month_loop: do nm = 1,11

        test_day = days - tdays_in_prior_months(nm+1)
        if (test_day < 0) then
          day   = days - tdays_in_prior_months(nm) + 1
          month = nm
          exit month_loop
        endif
      enddo month_loop

      if (month == 0) then
        day   = days - tdays_in_prior_months(12) + 1
        month = 12
        if (day == 32) then
          day   = 1
          month = 1
          year  = year + 1
        endif
      endif

      if (day == 0) day = 1

!-----------------------------------------------------------------------

 1100 format (/, 2x, 'Fatal error (',a,'): ', a)

!-----------------------------------------------------------------------

      end subroutine eday2ymd

!***********************************************************************

      subroutine ymd2eday (year, month, day, eday)

!-----------------------------------------------------------------------
!
!     converts calendar date (year, month, day) to elapsed days since
!     0000-01-01.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  year                   ! calendar year
     &, month                  ! calendar month
     &, day                    ! calendar day

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(out) ::
     &  eday                   ! elapsed days since 0000-01-01

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(0:3) ::
     &  days_each_year         ! days in year for 4-year cycle

      integer (kind=int_kind), dimension(12)  ::
     &  tdays_in_prior_months  ! temporary days in prior months
     &, tdays_in_month         ! temporary days in each month

      integer (kind=int_kind) ::
     &  nm                     ! dummy month index
     &, num_leapyears          ! leap year counters

!---------------------------------------------------------------------  
!
!     If leap years are not allowed, eday computation is straightforward
!
!---------------------------------------------------------------------  

      if (.not. allow_leapyear) then
        eday = year * days_in_norm_year
     &              + days_in_prior_months(month) + day - 1
 
!---------------------------------------------------------------------  
!
!     If leap years are allowed, compute the number of days elapsed
!     in prior months for *this* year and the number of elapsed
!     leap years prior to this year
!
!---------------------------------------------------------------------  

      else

        tdays_in_month    = days_in_month

        num_leapyears  = 1 +  year/4  - year/100 + year/400

        if (is_leapyear(year)) then
          tdays_in_month(2) = 29
          num_leapyears     = num_leapyears - 1
        else
          tdays_in_month(2) = 28
        endif

        call prior_days (tdays_in_prior_months, tdays_in_month)

!---------------------------------------------------------------------  
!
!       Compute elapsed days for this date
!
!---------------------------------------------------------------------  

        eday = (      num_leapyears ) * days_in_leap_year
     &        + ( year-num_leapyears ) * days_in_norm_year
     &        + tdays_in_prior_months(month) + day - 1

      endif   ! (.not. allow_leapyear)

!---------------------------------------------------------------------  

      end subroutine ymd2eday

!***********************************************************************

      subroutine date2eday (date,eday)

!-----------------------------------------------------------------------
!
!     Determine number of elapsed days since 0000-01-01 from the
!     calendar date in (integer) yyyymmdd format
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  date                   ! date in yyyymmdd format

      integer (kind=int_kind), intent(out) ::
     &  eday                   ! elapsed days since 0000-01-01

      integer (kind=int_kind) ::
     &  year, month, day       ! year month day indices

!-----------------------------------------------------------------------
!
!     use existing routines for the conversion
!
!-----------------------------------------------------------------------

      if (.not. valid_date(date)) 
     &  call exit_POP('(date2eday) invalid date',normal_exit=.false.)

      call date2ymd (date, year, month, day)
      call ymd2eday (year, month, day, eday)

!-----------------------------------------------------------------------

      end subroutine date2eday

!***********************************************************************

      subroutine eday2date (eday,date)

!-----------------------------------------------------------------------
!
!     Determine calendar date in (integer) yyyymmdd format from the
!     number of elapsed days since 0000-01-01.
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  eday                   ! elapsed days since 0000-01-01

      integer (kind=int_kind), intent(out) ::
     &  date                   ! date in yyyymmdd format

      integer (kind=int_kind) ::
     &  year, month, day       ! year month day indices

!-----------------------------------------------------------------------
!
!     use existing routines for the conversion
!
!-----------------------------------------------------------------------

      call eday2ymd (eday, year, month, day)
      call ymd2date (year, month, day, date)

!-----------------------------------------------------------------------

      end subroutine eday2date

!***********************************************************************

      subroutine prior_days (days_in_prior_months_loc,days_in_month_loc)

!-----------------------------------------------------------------------
!
!     define or reset the total number of days in prior months;
!     if leap years are allowed, this routine will be called once per 
!     year 
!
!-----------------------------------------------------------------------
 
      integer (kind=int_kind), dimension(12), intent(in) ::
     &  days_in_month_loc      ! current num of days in each month

      integer (kind=int_kind), dimension(12), intent(out) ::
     &  days_in_prior_months_loc  ! number of days in prior months
 
      integer (kind=int_kind) ::
     &  nm                      !   local month index
 
!-----------------------------------------------------------------------

      days_in_prior_months_loc(1) = 0

      do nm=2,12
        days_in_prior_months_loc(nm) =
     &  days_in_prior_months_loc(nm-1) + days_in_month_loc(nm-1)
      enddo

!-----------------------------------------------------------------------

      end subroutine prior_days

!***********************************************************************
 
      subroutine time_stamp (date_string, time_string)

!-----------------------------------------------------------------------
!
!     write a character string containing the date stamp
!        mm/dd/yyyy hh:mm:ss
!
!-----------------------------------------------------------------------

      character (*), intent(inout), optional ::
     &  date_string            ! a string to fill with date stamp
     &, time_string            ! a string to fill with time stamp
 
      character (1), parameter ::
     &  time_separator=':'

!-----------------------------------------------------------------------

      if (present(date_string)) then
        date_string = ' '
        write (date_string,1000) imonth, date_separator,
     &                           iday  , date_separator,
     &                           iyear
      endif

      if (present(time_string)) then
        time_string = ' '
        write (time_string,2000) ihour  , time_separator,
     &                           iminute, time_separator,
     &                           isecond
      endif

 1000 format (2(i2.2,a),i4.4)
 2000 format (2(i2.2,a),i2.2)
 
!-----------------------------------------------------------------------

      end subroutine time_stamp

      subroutine ccsm_date_stamp (date_string, ymds)

!-----------------------------------------------------------------------
!
!     write a character string containing the date stamp
!        yyyy-mm-dd, yyyy-mm, or yyyy
!
!-----------------------------------------------------------------------

      character (*) :: 
     &  date_string            ! a string to fill with date stamp
     &, ymds                   ! a string indicating date stamp format
 
      character (4) :: ccsm_cyear
      character (2) :: ccsm_cmonth
      character (2) :: ccsm_cday  
      character (5) :: ccsm_csecond
 
      integer (kind=int_kind) ::
     &  iyear_stamp
     &, imonth_stamp
     &, iday_stamp   
     &, itotal_second
 
      date_string = ' '

!---------------------------------------------------------------------
!     set ixxxx_stamp variables to conform to the ccsm standard
!---------------------------------------------------------------------
      if (midnight) then
        if (eoy) then
           iyear_stamp  = iyear_last
           imonth_stamp = 12
           iday_stamp   = 31
        elseif (eom) then
           iyear_stamp  = iyear
           imonth_stamp = imonth_last
           iday_stamp   = iday_last
        elseif (eod) then 
           iyear_stamp  = iyear
           imonth_stamp = imonth
           iday_stamp   = iday_last
        endif
      else
        iyear_stamp  = iyear
        imonth_stamp = imonth
        iday_stamp   = iday
      endif
 
      select case (trim(ymds))
        case ('ymds')
!---------------------------------------------------------------------
!       use unmodified ixxx variables if printing ymds information
!---------------------------------------------------------------------
        itotal_second = isecond + 60*iminute + 3600*ihour
        call int_to_char (4,iyear        , ccsm_cyear  )
        call int_to_char (2,imonth       , ccsm_cmonth )
        call int_to_char (2,iday         , ccsm_cday   )
        call int_to_char (5,itotal_second, ccsm_csecond)
        write (date_string,1000) ccsm_cyear, ccsm_cmonth, ccsm_cday
     &,                          ccsm_csecond

        case ('ymd')
        call int_to_char (4,iyear_stamp  , ccsm_cyear )
        call int_to_char (2,imonth_stamp , ccsm_cmonth)
        call int_to_char (2,iday_stamp   , ccsm_cday  )
        write (date_string,1000) ccsm_cyear, ccsm_cmonth, ccsm_cday

        case ('ym')
        call int_to_char (4,iyear_stamp  , ccsm_cyear )
        call int_to_char (2,imonth_stamp , ccsm_cmonth)
        write (date_string,1000) ccsm_cyear, ccsm_cmonth

        case ('y')
        call int_to_char (4,iyear_stamp  , ccsm_cyear)
        write (date_string,1000) ccsm_cyear
        
        case default 
        call exit_POP('(ccsm_date_stamp)',normal_exit=.false.)
 
      end select
 

 1000 format (a4,:,'-',a2:,'-',a2,:,'-',a5)
 
!-----------------------------------------------------------------------

      end subroutine ccsm_date_stamp

!***********************************************************************

      function is_near (test_value, target, tol)

!-----------------------------------------------------------------------
!
!     determine if test_value is "near" the target value
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), intent(in) ::
     &  test_value          ! value to test
     &, target              ! value to test against
     &, tol                 ! tolerance for determining nearness

      logical (kind=log_kind) :: 
     &  is_near             ! result (T or F) of nearness test

!-----------------------------------------------------------------------

      if (abs(test_value - target) <= tol) then
        is_near = .true.
      else
        is_near = .false.
      endif

!-----------------------------------------------------------------------

      end function is_near

!***********************************************************************

      function is_leapyear (iyear_loc)

!-----------------------------------------------------------------------
!
!     determine if test_year is a leapyear
!
!     assumptions:
!           o  year = 0 is the first year of the integration
!           o  standard calendar has 28 days in February, a leap year
!              has 29
!
!     algorithm: a year is a leap year if it is:
!           o  divisible by 4,
!           o  NOT divisible by 100, except if
!           o  also divisible by 400
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  iyear_loc              ! input year to test for leapyear
 
      logical (kind=log_kind) :: is_leapyear  ! result (T or F)           
 
!-----------------------------------------------------------------------
!
!     check for leap year
!
!-----------------------------------------------------------------------

      is_leapyear = .false.

      if (allow_leapyear .and. mod(iyear_loc,4) == 0  .and.
     &   (mod(iyear_loc,100) /= 0 .or. mod(iyear_loc,400) == 0 ) )
     &        is_leapyear = .true.

!-----------------------------------------------------------------------

      end function is_leapyear

!***********************************************************************

      function valid_date (date)

!-----------------------------------------------------------------------
!
!     Determine if a valid year, month & day can be decoded
!     from the calendar date in (integer) yyyymmdd format
!
!-----------------------------------------------------------------------

      logical (kind=log_kind) :: valid_date

      integer (kind=int_kind), intent(in) ::
     &  date                   ! calendar date in yyyymmdd format

      integer (kind=int_kind) ::
     &  year, month, day       ! year month day indices

!-----------------------------------------------------------------------
!
!     check a variety of possible error conditions
!
!-----------------------------------------------------------------------

      year = int(     date       /10000)
      month= int( mod(date,10000)/  100)
      day  =      mod(date,  100)

      valid_date = .true.

      if (year  <  0) valid_date = .false.
      if (month <  1) valid_date = .false.
      if (month > 12) valid_date = .false.
      if (day   <  1) valid_date = .false.
      if (day   > days_in_month(month)) valid_date = .false.

!-----------------------------------------------------------------------

      end function valid_date

!***********************************************************************
 
      function valid_ymd_hms()

!-----------------------------------------------------------------------
!
!     determine if the computed values of iyear,imonth,iday,
!     ihour, iminute, and isecond are within reasonable bounds
!
!-----------------------------------------------------------------------
      
      logical (kind=log_kind) ::
     &   valid_ymd_hms
     &,  valid_year
     &,  valid_month
     &,  valid_day_b
     &,  valid_day_e
     &,  valid_hour
     &,  valid_minute
     &,  valid_second
     &,  valid_eday_run
     &,  valid_eday_year
     &,  valid_feb_day

!-----------------------------------------------------------------------
!
!     set default condition of true for all flags
!
!-----------------------------------------------------------------------

      valid_ymd_hms     = .true.
      valid_year        = .true.
      valid_month       = .true.
      valid_day_b       = .true.
      valid_day_e       = .true.
      valid_hour        = .true.
      valid_minute      = .true.
      valid_second      = .true.
      valid_eday_run    = .true.
      valid_eday_year   = .true.
      valid_feb_day     = .true.
 
!-----------------------------------------------------------------------
!
!     check a variety of possible error conditions
!
!-----------------------------------------------------------------------

      if (iyear < 0) then
        valid_ymd_hms = .false.
        valid_year    = .false.
      endif

      if (imonth < 0 .or. imonth > 12) then
        valid_ymd_hms = .false.
        valid_month   = .false.
      endif

      if (iday < 1) then
        valid_ymd_hms = .false.
        valid_day_b   = .false.
      endif
 
      if (valid_ymd_hms) then   ! prevents out-of-range reference
        if (iday > days_in_month(imonth)) then
          valid_ymd_hms = .false.
          valid_day_e   = .false.
        endif
      endif
 
      if (ihour < 0 .or. ihour > 24) then
        valid_ymd_hms = .false.
        valid_hour    = .false.
      endif
 
      if (iminute < 0 .or. iminute > 60) then
        valid_ymd_hms = .false.
        valid_minute  = .false.
      endif
 
      if (isecond < 0 .or. isecond > 60) then
        valid_ymd_hms = .false.
        valid_second  = .false.
      endif

      if (elapsed_days_this_year < 0) then
        valid_ymd_hms   = .false.
        valid_eday_year = .false.
      endif
 
      if (elapsed_days_init_date < 0) then
        valid_ymd_hms   = .false.
        valid_eday_run  = .false.
      endif
 
      if (.not. allow_leapyear .and. imonth == 2 .and. iday == 29) then
        valid_ymd_hms = .false.
        valid_feb_day = .false.
      endif
 
!-----------------------------------------------------------------------
!
!     if errors detected, write out message and quit
!
!-----------------------------------------------------------------------

      if (.not. valid_ymd_hms) then
        if (my_task == master_task) then
 
          write(stdout,1000) ' invalid date value:'
 
          if (.not. valid_year)
     &      write(stdout,1100) ' iyear   ', iyear,' ( must be > 0 )'
 
          if (.not. valid_month)
     &      write(stdout,1100) ' imonth  ', imonth
     &,                        ' ( must be in the range [1,12] )'
 
          if (.not. valid_day_b)
     &      write(stdout,1100) ' iday    ', iday
     &,                        ' ( must be greater than 1 )'
 
          if (.not. valid_day_e)
     &      write(stdout,1101) ' iday    ', iday
     &,                        ' ( must be less than '
     &,                        days_in_month(imonth), ' )'
 
          if (.not. valid_hour)
     &      write(stdout,1100) ' ihour   ', ihour
     &,                        ' ( must be in the range [0,23] )'
 
          if (.not. valid_minute)
     &      write(stdout,1100) ' iminute ', iminute
     &,                        ' ( must be in the range [0,59] )'
 
          if (.not. valid_second)
     &      write(stdout,1100) ' isecond ', isecond
     &,                        ' ( must be in the range [0,59] )'
 
          if (.not. valid_eday_run)
     &      write(stdout,1100) ' elapsed_days_init_date '
     &,                          elapsed_days_init_date
     &,                        ' ( must be > 0 ) '
 
          if (.not. valid_eday_year)
     &      write(stdout,1100) ' elapsed_days_this_year '
     &,                          elapsed_days_this_year
 
          if (.not. valid_feb_day)
     &      write(stdout,*) ' Error: initial date = ', imonth,'/'
     &,                      iday,' but no leap years are allowed.'
 
          write(stdout,1102) 

        endif ! (my_task == master_task)                     

        call exit_POP('invalid date values',normal_exit=.false.)
 
      endif   ! valid_ymd_hms
 
!-----------------------------------------------------------------------
!
!     format statements
!
!-----------------------------------------------------------------------

 1000 format (//, 2x, '*************************************',/,2x,a)
 1100 format (2x, a24, ' = ', i8, 2x, a)
 1101 format (2x, a24, ' = ', i8, 2x, a, a4, a)
 1102 format (/, 1x, 'Please report this error to POP developers',//,2x
     &,              '*************************************')

!-----------------------------------------------------------------------

      end function valid_ymd_hms

!***********************************************************************


      subroutine write_time_manager_options

!-----------------------------------------------------------------------
!
!     writes all time manager options to stdout
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: k          ! vert level index
     &, ind
     &, nn

      character (1) ::
     &  suffix
 
      character (2) ::
     &  cmonth_end_run
     &, cday_end_run
     &, cmonth0
     &, cday0
 
      character (3) ::
     &  mix_step
 
      character (4) ::
     &   cyear0
     &,  cyear_end_run

      character (char_len_long) ::
     &  mix_steps
 
!-----------------------------------------------------------------------
!
!     write only from master task
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then

!-----------------------------------------------------------------------
!
!       write start/current time data
!
!-----------------------------------------------------------------------

        call int_to_char (4,iyear0 , cyear0)
        cmonth0 = cmonths(imonth0)
        cday0   = cdays  (iday0)

        call int_to_char (4,iyear_end_run  , cyear_end_run )
        cmonth_end_run = cmonths(imonth_end_run)
        cday_end_run   = cdays  (iday_end_run)

        write (stdout,1000)
        write (stdout,1001)
        write (stdout,1002)
        write (stdout,1001)
        write (stdout,1000)

        write (stdout,1001)
        write (stdout,*) 'Run id: ',trim(runid)

        write (stdout,1001)
        write (stdout,1010)
        write (stdout,1020) cmonth0, cday0, cyear0
        write (stdout,1030) ihour0
        write (stdout,1040) iminute0
        write (stdout,1050) isecond0

        write (stdout,1001)
        write (stdout,1011)
        write (stdout,1020) cmonth, cday, cyear
        write (stdout,1030) ihour
        write (stdout,1040) iminute
        write (stdout,1050) isecond
        if (nsteps_total /=   0)
     &  write (stdout,1060) nsteps_total

        write (stdout,1001)
 
        if (end_run_at_midnight) then
            write (stdout,1013)  'at 00:00:00 on'
        else if (dtt > seconds_in_day) then
            write (stdout,1013)  'at the end of the day on or after'
        else
            write (stdout,1013)  'at the end of the day on'
        endif
 
        if (stop_count == 1) then
           suffix = ' '
        else
           suffix = 's'
        endif
 
        write (stdout,1020)cmonth_end_run,cday_end_run,cyear_end_run
 
        select case (stop_option)        
 
        case ('never')
          write (stdout,1014) 'upon receipt of stop signal' //
     &                         ' from external source (eg, cpl)'
        case ('nyear')
          write(stdout,1014)'after running for ',stop_count,' year' 
     &,                      suffix
        case ('nmonth','nmonths')
          write(stdout,1014)'after running for ',stop_count,' month'
     &,                      suffix
        case ('nday','ndays')
          write(stdout,1014)'after running for ',stop_count,' day'
     &,                      suffix
        case ('eoy')
          write(stdout,1014)'at the end of the year after '
     &,                      stop_count, ' year'
     &,                      suffix
        case ('eom')
          write(stdout,1014)'at the end of the month after'
     &,                      stop_count, ' month'
     &,                      suffix
        case ('eod')
          write (stdout,1014) 'at the end of the day'
        case ('nstep','nsteps')
          write (stdout,1014) 'after ', stop_count,' timestep'
     &,                        suffix
        case ('date')
          write (stdout,1014) 'after reaching the specified date'
        case default
        end select
        write (stdout,1014) 'unless a stop signal is received' //
     &                       ' from external source (eg, cpl)'

        write (stdout,1001)
        write (stdout,1012)
        write (stdout,1070) tyear
        write (stdout,1080) tmonth
        write (stdout,1090) tday
        write (stdout,1100) thour
        write (stdout,1110) tsecond

!-----------------------------------------------------------------------
!
!       timestep information
!
!-----------------------------------------------------------------------

        write (stdout,1001)
        if (dt_option == 'auto_dt') then
          write (stdout,1130) ' enabled'
        else
          write (stdout,1130) 'disabled'
        endif

        write (stdout,1140) dt_count
 
        if (tmix_iopt == tmix_avgfit) then
             write(stdout, 1143) fullsteps_per_day
             write(stdout, 1144) halfsteps_per_day
             write(stdout, 1145) nsteps_per_day
             write (stdout,1001)
        endif
 
        write (stdout,1150) time_mix_freq
 
        select case (tmix_iopt)
 
        case (tmix_avg)
          write (stdout,1151) 'avg -- time averaging'
 
        case (tmix_avgbb)
          write (stdout,1151) 'avgbb -- time averaging'
     &                       // 'with back-to-back averaging steps'
 
        case (tmix_avgfit)
        write (stdout,1151) 'avgfit -- time averaging'
     &                     //' with timestep chosen to fit exactly'
     &                     //' into one day or coupling interval  '
        if (time_mix_freq > fullsteps_per_interval + 1) then
            if (fit_freq == 1) then
              write (stdout,1155) 'number', '2', 'day'
            else
              write (stdout,1155) 'number', '2', 'interval'
            endif
        else
            ind = 1
            mix_steps = '2'
 
            do nn = 3, nsteps_per_interval
               if (mod(nn,time_mix_freq) == 0) then
                   write(mix_step,'(i2)' )  nn
                   mix_steps = trim(mix_steps)//',' // trim(mix_step)
               endif
            enddo
 
            if (fit_freq == 1) then
              write (stdout,1155) 'numbers', trim(mix_steps), 'day'
            else
              write (stdout,1155) 'numbers', trim(mix_steps), 'interval'
            endif
        endif
 
        case (tmix_matsuno)
          write (stdout,1170) time_mix_freq
        end select

        write (stdout,1001)
        write (stdout,1180) dtt
        write (stdout,1190) dtu
        write (stdout,1200) dtp

        write (stdout,1001)
        if (laccel) then
          write (stdout,1210) ' enabled'
          write (stdout,1220)
          write (stdout,1230)
          do k=1,km
            write (stdout,1240) k, dttxcel(k)
          end do
        else
          write (stdout,1210) 'disabled'
        endif

!-----------------------------------------------------------------------
!
!       other options
!
!-----------------------------------------------------------------------

        write (stdout,1001)
        if (allow_leapyear) then
          write (stdout,1250)
        else
          write (stdout,1251)
        endif

        write (stdout,1001)
        if (impcor) then
          write (stdout,1260) ' ON'
        else
          write (stdout,1260) 'OFF'
        endif

!-----------------------------------------------------------------------
!
!     end of writes
!
!-----------------------------------------------------------------------

      endif ! (my_task == master_task)

!-----------------------------------------------------------------------
!
!     format statements
!
!-----------------------------------------------------------------------

1000  format (72('-'))
1001  format (' ')
1002  format ('time management options')
1010  format ('This simulation started from')
1011  format ('This run        started from')
1020  format ('       date(month-day-year):',2x,2(a2,'-'),a4)
1030  format ('                       hour:',2x,i2)
1040  format ('                     minute:',2x,i2)
1050  format ('                     second:',2x,i2)
1060  format ('                       step:',2x,i10)

1013  format ('This run will terminate ',/,a  )                
1014  format (a, :i7, a,a)

1012  format ('Starting elapsed time in    ')
1070  format ('                      years:',2x,f16.3)
1080  format ('                     months:',2x,f16.3)
1090  format ('                       days:',2x,f16.3)
1100  format ('                      hours:',2x,f16.3)
1110  format ('                    seconds:',2x,f16.3)

1130  format ('Automatic time step option (auto_dt) ',a8)
1140  format ('dt_count      = ', 1pe12.6)
1143  format ('There are ', i3, ' full  steps each day')
1144  format ('There are ', i3, ' half  steps each day')
1145  format ('There are ', i3, ' total steps each day')
1150  format ('time_mix_freq = ', i6     )
1151  format ('Time mixing option: ', a)
1155  format ('Averaging time steps are at step ',a,  ': ', a,
     &        ' every ',a )
1170  format ('Matsuno time steps every ',i6,' steps')

1180  format ('Surface tracer     time step = ',1pe12.6,' seconds')
1190  format ('Surface momentum   time step = ',1pe12.6,' seconds')
1200  format ('Surface barotropic time step = ',1pe12.6,' seconds')

1210  format ('Tracer acceleration ',a8)
1220  format ('   k      accel factor')
1230  format ('  ---     ------------')
1240  format (2x,i3,7x,f8.3)

1250  format ('Leap years allowed')
1251  format ('Leap years not allowed')
1260  format ('Implicit treatment of Coriolis terms (impcor): ',a3)

!-----------------------------------------------------------------------

      end subroutine write_time_manager_options

 
!***********************************************************************

      subroutine int_to_char (string_length,int_in,char_out)

!-----------------------------------------------------------------------
!
!     converts an integer into an n-digit character and pads the
!     spaces with zeroes
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &  int_in                 ! input integer to convert
     &, string_length          ! length of output character

      character(*), intent(out) :: 
     &  char_out               ! output character string containg int

      integer (kind=int_kind) :: n   ! dummy counter
      character (len=8)       :: fmt ! variable format
 

!-----------------------------------------------------------------------

      if (string_length >= 1 .and. string_length <= 9) then
         write(fmt,1100) string_length
      else if (string_length >= 10 .and. string_length <= 99) then
         write(fmt,1200) string_length
      else
         call exit_POP ('string_length too long in sub. int_to_char'
     &,                normal_exit=.false.)
      endif
 
1100  format ('(i',i1,')')
1200  format ('(i',i2,')')
 
      write(char_out,fmt) int_in

      do n=1,string_length
        if (char_out(n:n) == ' ') char_out(n:n) = '0'
      end do

!-----------------------------------------------------------------------

      end subroutine int_to_char



!***********************************************************************

      end module time_management

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
