      module types
!     ============
 
!-----------------------------------------------------------------------
!   This module contains f90 derived type information
!
!   CVS:$Id: types.F,v 1.1.4.4 2001/12/21 21:24:39 njn01 Exp $
!   CVS:$Name: ccsm2_0_beta46 $
!-----------------------------------------------------------------------
 
      use kinds_mod
      
      implicit none
      save
 
 
!-----------------------------------------------------------------------
!     define parameters
!-----------------------------------------------------------------------
      integer (kind=int_kind)
     &, parameter                 ::
     &     max_dim_ids =   7                ! max no. of dimension ids
                                            !(temporary -- use 
                                            !  allocatable arrays instead
     &,    char_long   =   256              ! max length of long_name
 
!-----------------------------------------------------------------------
!     define f90 types used in netcdf-generating code
!-----------------------------------------------------------------------
 
      type output_file                      ! output file information
        character (char_len_long) :: name   
        character (char_len)      :: root          
        character (char_len)      :: suffix 
        character (char_len)      :: type   ! 'binary' or 'netcdf',eg
        integer   (kind=int_kind) :: id     !  binary unit no. or netcdf id
        logical   (kind=log_kind) :: lncdf 
        logical   (kind=log_kind)           !  for netcdf files only   
     &                            :: define_on = .false.  
      end type output_file
 
 
      type req_attributes                    ! req'd CSM/netcdf attributes
        character (char_len)      :: short_name
        character (char_long)     :: long_name
        character (char_len)      :: units
        integer   (kind=int_kind) :: ndims   ! number of dimensions
        integer   (kind=int_kind) :: id        
        integer   (kind=int_kind) :: type      
      end type req_attributes
 
 
      type dim_attributes
        character (char_len)      :: name    
        integer   (kind=int_kind) :: len
        integer   (kind=int_kind) :: id
      end type dim_attributes
 
 
      type coord_attributes
        type      (req_attributes):: req
        character (char_len)      :: positive
        character (char_len)      :: edges
        integer   (kind=int_kind) :: len
        integer   (kind=int_kind)
     &,   dimension(max_dim_ids)  :: dim_ids
        real      (kind=dbl_kind ):: valid_max
        real      (kind=dbl_kind) :: valid_min
      end type coord_attributes
 
 
      type scalar_attributes
        type      (req_attributes):: req
        real      (kind=dbl_kind ):: value
      end type scalar_attributes
 
 
      type var_attributes
        type      (req_attributes):: req
        character (char_len)      :: flux_direction
        character (len=4)         :: grid_code
        character (char_len)      :: mask_code
        character (char_len)      :: cell_methods
        character (char_len)      :: coordinates    !describes nonrectilinear grid
        real      (kind=real_kind )   
     &                            :: add_offset
     &,                              scale_factor
        integer   (kind=int_kind)
     &,   dimension(max_dim_ids)  :: dim_ids
      end type var_attributes
 
      type time_attributes
        type      (dim_attributes):: dim
        character (char_len)      :: units
        character (char_len)      :: bounds
        character (char_len)      :: calendar
        integer   (kind=int_kind) :: type
        integer   (kind=int_kind) :: vid
      end type time_attributes
 
      type time_boundaries
        type      (req_attributes):: req
        integer   (kind=int_kind)
     &,   dimension(max_dim_ids)  :: dim_ids
      end type time_boundaries
  
!-----------------------------------------------------------------------
!     defined-type variables
!-----------------------------------------------------------------------
      type (time_attributes)      :: time_info
 
      type (time_boundaries)      :: time_bound
 
      type (dim_attributes)
     &                            :: dim_nlon
     &,                              dim_nlat
     &,                              dim_z_t
     &,                              dim_z_w
     &,                              dim_tracers 
     &,                              dim_nchar
     &,                              dim_time  
     &,                              dim_d2    
 
      real (kind=dbl_kind)        :: time_bound_lower
     &,                              time_bound_upper
 
      logical   (kind=log_kind)   :: 
     &  debug_ncdf
 
      integer   (kind=int_kind)   :: 
     &  ier_ncdf
     &, fid
 
 
 
      integer (kind=int_kind)     :: int_fill
     &                               = -2147483647
      real (kind=real_kind)       :: flt_fill
     &                               = 9.9692099683868690e+36
      real (kind=dbl_kind)        :: dbl_fill 
     &                               = 9.9692099683868690e+36_dbl_kind
 
      integer (kind=int_kind)     :: xtype_unlim   = 0
     &,                              xtype_int     = 4
     &,                              xtype_float   = 5
     &,                              xtype_double  = 6
     &,                              xtype_global  = 0
     &,                              xtype_unknown = -999
     &,                              no_error_ncdf = 0
 
 
 
 
 
!     ================
      end module types
!     ================

