!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module xdisplay

!***********************************************************************
!
!     This is a dummy version of an unsupported module containing 
!     routines for opening an xdisplay and viewing a field while 
!     the simulation is running.  The actual sort-of-working version
!     relies on an unsupported library called fix (fortran interface 
!     to X).
!
!     The purpose of this dummy version is to allow the makefiles 
!     to be portable to systems which may not have X windows.  If 
!     you have X windows on your system and you want to try this, 
!     then overwrite this file, xdisplay.F, with 
!
!         cp ../input_templates/xdisplay.F.unsupported ./xdisplay.F
!
!     and do 
!
!         cp ../input_templates/fix_64.C.unsupported ./fix_64.C
!
!
!     CVS:$Id: xdisplay.F,v 1.2.2.1 2001/03/13 22:55:18 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use io
      use global_reductions
      use grid
      use exit_mod

      implicit none
      save

!-----------------------------------------------------------------------
!
!     module variables
!
!-----------------------------------------------------------------------

      logical (kind=log_kind) ::
     &  lxdisplay             ! logical flag to turn on xdisplay

      integer (kind=int_kind) ::
     &  nstep_xdisplay,       ! display image every nstep timesteps
     &  nzoom,                ! zoom factor for image
     &  nxpix, nypix,         ! image size in pixels
     &  xid                   ! id for X window

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_xdisplay

!-----------------------------------------------------------------------
!
!     this routine initializes an xwindow for displaying a 2-d field
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      namelist /xdisplay_nml/lxdisplay, nstep_xdisplay

      integer (kind=int_kind) :: 
     &  nml_error             ! namelist i/o error flag

!-----------------------------------------------------------------------
!
!     read input namelist to see if xdisplay required
!
!-----------------------------------------------------------------------

      lxdisplay = .false.
      nstep_xdisplay = 1

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=xdisplay_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading xdisplay_nml')
      endif

      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,1010)
        write(stdout,1020)
        write(stdout,1010)
        write(stdout,1000)

        if (lxdisplay) then
          write(stdout,1030)
          write(stdout,1040) nstep_xdisplay
        else
          write(stdout,1050)
        endif

 1000   format(72('-'))
 1010   format(' ')
 1020   format(' Xdisplay options')
 1030   format(' Xdisplay enabled')
 1040   format(' Display changes every ',i6,' steps.')
 1050   format(' Xdisplay disabled')

      endif

      call broadcast_scalar(lxdisplay, master_task)

      if (.not. lxdisplay) return

!-----------------------------------------------------------------------
!
!     if xdisplay requested, exit with error
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        write(stdout,*) 'X Display Not Available - exiting'
      endif

      stop 'X display requested when only dummy routines linked'

!-----------------------------------------------------------------------

      end subroutine init_xdisplay

!***********************************************************************

      subroutine clear_display
 
      stop 'X display requested when only dummy routines linked'

      end subroutine clear_display

!***********************************************************************

      subroutine display(FIELD1,FIELD2)

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) ::
     &  FIELD1, FIELD2       ! two fields to stack and display

      stop 'X display requested when only dummy routines linked'

      end subroutine display

!***********************************************************************


      end module xdisplay

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
