#!/usr/bin/env perl

use strict;
use Cwd;
use lib ".";
use Tk;
use Env qw(LOGNAME HOME);

#--------------------------------------------------------------------------------------------
# default settings of variables needed for submit
#--------------------------------------------------------------------------------------------

my $platform = 'aix';
my $lab = 'ncar';
my $res = 'T42_gx1v3';

my $csmroot = "";
my $casename = "";
my $csmdata = "";
my $exeroot = "";
my $status = "";

my $config = "";
my %status = {};
my %desc = (
	    "B" => "(Config B) active [atm,lnd,ocn,ice] ",
	    "F" => "(Config F) active [atm,lnd] + prescribed-clim[ice] + data[ocn]",
	    "J" => "(Config J) prescribed-clim[ice] + data[atm,lnd,ocn]",
	    "K" => "(Config K) active [atm,lnd] + data [ocn,ice]",
	    "G" => "(Config G) active [ocn,ice] + l[atm] + data[lnd]",
	    "C" => "(Config C) active [ocn] + data [atm,lnd,ice]",
	    "D" => "(Config D) active [ice] + data [atm,lnd,ocn]",
	    "H" => "(Config H) active [atm] + data [lnd,ocn,ice]",
	    "I" => "(Config I) active [lnd] + data [atm,ocn,ice]",
	    "M" => "(Config M) active MixedLayerOcean [ice] + l[atm] + data [lnd,ocn]",
	    "A" => "(Config A) data [atm,lnd,ocn,ice]"
	   );

my $submit_status = 0;
my $key;

my(@color1) = qw/-background peachpuff /;
my(@color2) = qw/-background peachpuff -foreground red/;
my(@color3) = qw/-background peachpuff -foreground blue/;
my(@colorRed) = qw/-background red -foreground white/;
my(@colorGreen) = qw/-background green -foreground black/;
my(@colorYellow) = qw/-background yellow -foreground black/;
my(@colorBlue) = qw/-background blue -foreground white/;

#--------------------------------------------------------------------------------------------
# Read ~/.ccsm_gui
#--------------------------------------------------------------------------------------------

my $text;

(my $run_dir = cwd()) =~ /(.*)\//;
$run_dir = $1;

my $file = "$HOME/.ccsm_gui";
if (-e $file) {
  open( CCSMGUI, "$file" ) || die "Cannot open file: $file";
  while (<CCSMGUI>) {
    $text = $_;
  }
} else {
  system ("touch $file");
  open( CCSMGUI, "$file" ) || die "Cannot open file: $file";
}
close(CCSMGUI);
print ("input is $text\n");

my @text = split(/ /,$text);
while ( defined($text[0]) ) {
  $_ = shift( @text );
  if (/-lab/) {
    $lab = shift(@text);
  } elsif (/-res/) {
    $res = shift(@text)
  } elsif (/-platform/) {
    $platform = shift(@text)
  } elsif (/-config/) {
    $config = shift(@text);
  } elsif (/-csmroot/) {
    $csmroot = shift(@text);
  } elsif (/-exeroot/) {
    $exeroot = shift(@text);
  } elsif (/-csmdata/) {
    $csmdata = shift(@text);
  } elsif (/-case/) {
    $casename = shift(@text);
  }
}

#--------------------------------------------------------------------------------------------
# Main window
#--------------------------------------------------------------------------------------------

my $mw = MainWindow->new();
$mw->title("CCSM startup script creation",);
$mw->configure(@color1);

#--------------------------------------------------------------------------------------------
# Platform 
#--------------------------------------------------------------------------------------------

my(@pl) = qw/-side top -pady 15 -padx 15 -fill x/;
my $f1 = $mw->Frame(@color1)->pack(@pl);

my(@pl) = qw/-side left -anchor w/;
$f1->Label(-text=>"Select Platform",
	   @color3
	   ) ->pack(@pl);

my(@pl) = qw/-side left -anchor w -padx 20 -pady 10 -fill x -expand 1/;
$f1->Menubutton(-textvariable=>\$platform,
		@color2,
		-relief=>'raised',
		-indicatoron => 1,
		-menuitems=>[['command'=> "aix",-command => sub {$platform = "aix"}],
			     ['command'=> "sgi",-command => sub {$platform = "sgi"}],
			    ]
	       ) ->pack(@pl);

my(@pl) = qw/-side left -anchor w/;
$f1->Label(-text=>"Select Resolution",
	   @color3,
	  )->pack(@pl);

my(@pl) = qw/-side right -anchor w -padx 20 -pady 10 -fill x -expand 1/;
$f1->Menubutton(-textvariable=>\$res,
		@color2,
		-relief=>'raised',
		-indicatoron => 1,
		-menuitems=>[['command'=> "T31_gx3"  , -command => sub {$res = "T31_gx3"}],
			     ['command'=> "T42_gx1v3", -command => sub {$res = "T42_gx1v3"}],
			     ['command'=> "T62_gx1v3 (G and M only)", -command => sub {$res = "T62_gx1v3"}],
			    ]
	       ) ->pack(@pl);

#--------------------------------------------------------------------------------------------
# Necessary entries
#--------------------------------------------------------------------------------------------

my(@pl0) = qw/-side top -pady 3 -padx 20 -fill x/;
my(@pl1) = qw/-side left -anchor w/;
my(@pl2) = qw/-side left -anchor w -padx 20 -pady 3 -fill x -expand 1/;

my $entry = $mw->Frame(@color1)->pack(@pl0);
$entry->Label(-text=>"Enter ccsm root directory (CSMROOT)         ",
	      @color3)->pack(@pl1);
my $entry_csmroot = $entry->Entry(-textvariable=>\$csmroot,
				  -selectbackground=>"blue")->pack(@pl2);

my $entry = $mw->Frame(@color1)->pack(@pl0);
$entry->Label(-text=>"Enter case name for run (CASE)              ",
	      @color3)->pack(@pl1);
my $entry_case = $entry->Entry(-textvariable=>\$casename,
			       -width=>45,
			       -selectbackground=>"blue")->pack(@pl2);

my $entry = $mw->Frame(@color1)->pack(@pl0);
$entry->Label(-text=>"Enter ccsm data directory (CSMDATA)         ",
	      @color3)->pack(@pl1);
my $entry_csmdata = $entry->Entry(-textvariable=>\$csmdata,
				  -selectbackground=>"blue")->pack(@pl2);

my $entry = $mw->Frame(@color1)->pack(@pl0);
$entry->Label(-text=>"Enter ccsm executable directory (EXEROOT)   ",
	      @color3)->pack(@pl1);
my $entry_exeroot = $entry->Entry(-textvariable=>\$exeroot,
				  -selectbackground=>"blue")->pack(@pl2);

#--------------------------------------------------------------------------------------------
# Tests 
#--------------------------------------------------------------------------------------------

my $test_label = $mw->Frame(@color1
			   )->pack(@pl0);

$test_label->Label(-text=>"Select ONE of the following model test configurations",
		   @color3, -borderwidth=>6,
		   )->pack(@pl1);

my(@pl) = qw/-side top -padx 25 -pady 2 -anchor w /;

my $test_case = $mw->Frame(@color3
			  )->pack(@pl);

foreach $key (keys(%desc)) {
    $test_case->Radiobutton(-text => $desc{$key},
			    @color3,
			    -variable => \$config,
			    -value => $key,
			    -relief =>'flat',
			    -selectcolor => "red",
			    )->pack(@pl);
}

#--------------------------------------------------------------------------------------------
# Generate test script or exit
#--------------------------------------------------------------------------------------------

my(@plf) = qw/-side top -pady 3 -fill x/;

my(@pl) = qw/-side left -anchor w -padx 40 -pady 10 -fill x -expand 1/;
my $gen = $mw->Frame(@color1)->pack(@plf);

$gen->Button(-text=>"GENERATE RUN SCRIPT DIRECTORY in $run_dir",
	     @colorYellow,
	     -command => sub {$submit_status = 0; submit()}
	     )->pack(@pl);

my(@pl) = qw/-side left -anchor w -padx 40 -pady 20 -fill x -expand 1/;
my $final = $mw->Frame(@color1)->pack(@plf);

$final->Button(-text=>"HELP",@colorBlue,
	       -command => \&print_help
	       )->pack(@pl);

$final->Button(-text=>"EXIT",
	       @colorRed,
	       -command => sub {exit}
	       )->pack(@pl);

#------------------------------------------------------------------

sub submit {

  my @input_args= ();

  # platform
  push (@input_args, "-platform $platform");

  # resolution
  push (@input_args, "-res $res");

  # configuration
  if ($config eq "") {
    $status = "Must select one run configuration";
    print_status($status);
    return;
  } elsif ($res eq "T62_gx1v3" && ($config !~ /G|M/)) {
    $status = "Must set configuration to G or M for T62_gx1v3 resolution";
    print ("out1: configuration is $config,  resolution is $res\n");
    print_status($status);
    return;
  } elsif ($res ne "T62_gx1v3" && ($config eq "G" || $config eq "M")) {
    $status = "Configurations G and M only supported at T62_gx1v3 \
              \n Reset either Configuration or Resolution\n";
    print ("out2: configuration is $config,  resolution is $res\n");
    print_status($status);
    return;
  } 
  push (@input_args, "-config $config");

  # csmroot directory
  if ( $csmroot eq "") {
    $status = "ERROR:\n\n Must enter CCSM root directory";
    print_status($status);
    $entry_csmroot->selectionRange(0,'end');
    $entry_csmroot->focus;
    return;
  } 
  push (@input_args, "-csmroot $csmroot");

  # case name
  if ( $casename eq "") {
    $status = "ERROR:\n\n Must specify case name";
    print_status($status);
    $entry_case->selectionRange(0,'end');
    $entry_case->focus;
    return;
  } elsif ( -d "${run_dir}/${casename}" ) {
    $status = "ERROR:\n\n Following case script directory aready exists \
                  \n ${run_dir}/${casename} \
                  \n Reenter Case Name\n";		
    print_status($status);
    $entry_case->selectionRange(0,'end');
    $entry_case->focus;
    return;
  }
  push (@input_args, "-case $casename");

  # csmdata directory
  if ( $csmdata eq "") {
    $status = "ERROR:\n\n Must enter CCSM input data root directory\n";
    print_status($status);
    $entry_csmdata->selectionRange(0,'end');
    $entry_csmdata->focus;
    return;
  }
  push (@input_args, "-csmdata $csmdata");

  # executable directory
  if ( $exeroot eq "") {
    $status = "ERROR:\n\n Must enter CCSM executable directory\n";
    print_status($status);
    $entry_exeroot->selectionRange(0,'end');
    $entry_exeroot->focus;
    return;
  } 
  push (@input_args, "-exeroot $exeroot");

  # generate run script
  open( CCSMGUI, "+>>$file" ) || die "Cannot open file: $file";
  system ("date >> $file") && die "cannot write to $file";
  print CCSMGUI "@input_args \n";
  close(CCSMGUI);
  if (($res ne "T42_gx1v3") || (($res eq "T42_gx1v3") && ($config ne "B"))) {
    $status = "WARNING: \n\n You have created script \
               \n ${casename}.run \
               \n in the directory \
               \n ${run_dir}/${casename} \
               \n to run a configuration that has no control simulation. \
               \n We do not guarantee the scientific validity of the results \n";

  } else {
    $status = "\n You have successfully created a script \
               \n ${casename}.run \
               \n in the directory \
               \n ${run_dir}/${casename} \n";
  }
  system("./generate_ccsm_run.csh @input_args") && {$status="ERROR EXIT"};
  print_status($status);		
  return;		
}

sub print_help {
  my $tl = $mw->Toplevel(-title=>'HELP');
  my $text = $tl->Scrolled("Text", 
			   -scrollbars=>'se',
		   -background=>"peachpuff", -foreground=>"midnightblue"
			  )->pack();
  $text->delete("1.0","end");
  if (!open(FH,"HELP")) {
    $text->insert('end',"ERROR: could not open HELP");
  }
  while (<FH>) {
    $text->insert("end",$_);
  }
  close(FH);
  $tl->Button(-text=>"QUIT", @color1, -command =>sub{$tl->withdraw}
	    )->pack(-side=>'top',-expand=>1);
}

sub print_status {
    my $status = shift(@_);
    my $tl = $mw->Toplevel(-title=>'STATUS');
    my(@pl) = qw/-side top -pady 3 -fill x/;
    $tl->Label(-text=>" $status", -background=>"peachpuff", -foreground=>"blue"
	       )->pack(-side=>'top', -expand=>1, -padx=>15, -pady=>15);
    $tl->Button(-text=>"CLOSE", -command =>sub{$tl->withdraw}
		)->pack(-side=>'top',-expand=>1, -padx=>15, -pady=>15);
}

MainLoop;

