#!/bin/csh -f
#
# Generate the ccsm tests
#

#-----------------------------------------------------------------------
# Parse the input (set LAB PLATFORM CSMROOT CONFIG SUBMIT)
#-------------------------------------------------

if ($#argv == 0 ) then
    echo "ERROR: arguments to script not specified correctly"
    exit 1
endif

set CASE = ""
set PLATFORMS = ""
set CONFIGS = ""
set RESOLUTIONS = ""
set CSMROOT = ""
set EXEROOT = ""
set CSMDATA = ""    

while ( 1 )
  if ( $#argv < 1 ) break;
  set i = $argv[1];
  switch ( $i )
    case "-platform"
      shift argv
      set PLATFORMS = $argv[1]
      echo "CCSM Platform: $PLATFORMS"
      breaksw
    case "-res"
      shift argv
      set RESOLUTIONS = $argv[1]
      echo "CCSM Resolution: $RESOLUTIONS"
      breaksw
    case "-case"
      shift argv
      set CASE = $argv[1]
      echo "CCSM Case name: $CASE"
      breaksw
    case "-csmroot"
      shift argv
      set CSMROOT = $argv[1]
      echo "CCSM Root Directory: $CSMROOT"
      breaksw
    case "-exeroot"
      shift argv
      set EXEROOT = $argv[1]
      echo "CCSM Executable Directory: $EXEROOT"
      breaksw
    case "-csmdata"
      shift argv
      set CSMDATA = $argv[1]
      echo "CCSM Data Root Directory: $CSMDATA"
      breaksw
    case "-config"
      shift argv
      set CONFIGS = $argv[1]
      echo "CCSM Run configuration: $CONFIGS"
      breaksw
    default:
      echo "Parse_the_input.sh: Unknown argument: $argv[1]"
      exit;
  endsw
  shift argv
end

if ( ( $CONFIGS[1] == "" || $PLATFORMS[1] == "")) exit 2

#-----------------------------------------------------------------------
# Define the GRIDS, TASK and THREAD relations
#-------------------------------------------------

set MODEL_L     =          ( cpl atm ocn ice lnd  atm  ocn  ice  lnd  atm )
set SETUP_L     =          ( cpl atm ocn ice lnd datm docn dice dlnd latm )

set NTASK_aix_T63_gx1v3 =  (  1   1   1   1   1    1    1    1    1    1  )
set NTHRD_aix_T63_gx1v3 =  (  4   1   1   1   1    1    1    1    1    1  )
                                                                          
set NTASK_sgi_T63_gx1v3 =  (  1   1   1   1   1    1    1    1    1    1  )
set NTHRD_sgi_T63_gx1v3 =  (  4   1   1   1   1    1    1    1    1    1  )
                                                                          
set NTASK_cpq_T63_gx1v3 =  (  1   1   1   1   1    1    1    1    1    1  )
set NTHRD_cpq_T63_gx1v3 =  (  4   1   1   1   1    1    1    1    1    1  )
                                                   
set NTASK_aix_T42_gx1v3 =  (  1   8  40  16   3    1    1    1    1    1  )
set NTHRD_aix_T42_gx1v3 =  (  4   4   1   1   4    1    1    1    1    1  )
                                                                          
set NTASK_sgi_T42_gx1v3 =  (  1   1   8   4   1    1    1    1    1    1  )
set NTHRD_sgi_T42_gx1v3 =  (  4   8   1   1   6    1    1    1    1    1  )
                                                                         
set NTASK_cpq_T42_gx1v3 =  (  1   4   8   8   1    1    1    1    1    1  )
set NTHRD_cpq_T42_gx1v3 =  (  4   4   1   1   1    1    1    1    1    1  )
                                                   
set NTASK_aix_T31_gx3  =   (  1  12   4   4   8    1    1    1    1    1  )
set NTHRD_aix_T31_gx3  =   (  4   4   1   1   1    1    1    1    1    1  )
                                                                          
set NTASK_sgi_T31_gx3  =   (  1   1   8   4   1    1    1    1    1    1  )
set NTHRD_sgi_T31_gx3  =   (  4   8   1   1   6    1    1    1    1    1  )
                                                                          
set NTASK_cpq_T31_gx3  =   (  1   4   8   8   1    1    1    1    1    1  )
set NTHRD_cpq_T31_gx3  =   (  4   4   1   1   1    1    1    1    1    1  )

#-----------------------------------------------------------------------
# Generate startup script
#----------------------------------------------------------

set MSSNAME = `echo $LOGNAME | tr '[a-z]' '[A-Z]'`  

foreach PLATFORM ( $PLATFORMS )
 foreach CONFIG   ( $CONFIGS )
  foreach RES      ( $RESOLUTIONS )

   # parse the resolution string
   set ATM_GRID = `echo $RES | sed 's/_.*//'`
   set OCN_GRID = `echo $RES | sed 's/.*_//'`
   
   if ($RES == T62_gx1v3)   set RESname = Sp
   if ($RES == T42_gx1v3)   set RESname = Hi
   if ($RES == T31_gx3)     set RESname = Lo
   
   #   
   # copy the template scripts into run script directory
   #

   setenv DIR ../$CASE
   if !(-d  $DIR) mkdir -p $DIR
   cp -rp gui_templates/* $DIR  || exit 1

   #
   # Define the arrays for a CCSM run
   # 
   set AG = $ATM_GRID
   set OG = $OCN_GRID

   set GRIDS_L =  (${AG}_$OG $AG $OG $AG  $OG $AG  $OG  $OG  $AG  $AG )

   if ($CONFIG == X ) set COMPS = ( atm  lnd  ice  ocn  cpl )
   if ($CONFIG == B ) set COMPS = ( atm  lnd  ice  ocn  cpl )
   if ($CONFIG == F ) set COMPS = ( atm  lnd  ice docn  cpl )
   if ($CONFIG == G ) set COMPS = (latm dlnd  ice  ocn  cpl )
   if ($CONFIG == A ) set COMPS = (datm dlnd dice docn  cpl )
   if ($CONFIG == C ) set COMPS = (datm dlnd dice  ocn  cpl )
   if ($CONFIG == D ) set COMPS = (datm dlnd  ice docn  cpl )
   if ($CONFIG == E ) set COMPS = ( atm  lnd dice  ocn  cpl )
   if ($CONFIG == H ) set COMPS = ( atm dlnd dice docn  cpl )
   if ($CONFIG == I ) set COMPS = (datm  lnd dice docn  cpl )
   if ($CONFIG == J ) set COMPS = (datm dlnd  ice  ocn  cpl )
   if ($CONFIG == K ) set COMPS = ( atm  lnd dice docn  cpl )
   if ($CONFIG == M ) set COMPS = (latm dlnd  ice docn  cpl )

   foreach n (MODELS SETUPS NTASKS NTHRDS GRIDS)
    set $n = ""
   end
 
   set task_geo = "" # text string for task_geometry command
   set node_num = 0  # node number: 0-143
   set proc_num = 0  # counter that cycles over proc_max
   set proc_max = ( 4 2 1 1)  # number of tasks/node for NTHRDS = (1 2 3 4)

   @ TASKS = 0
   @ NODES = 0
   foreach c (1 2 3 4 5 )  # loop over components
    foreach n (1 2 3 4 5 6 7 8 9 10 ) # loop over various possible configs
     if ( $COMPS[$c] ==  $SETUP_L[$n] ) then
       set MODELS = ( $MODELS $MODEL_L[$n] )
       set SETUPS = ( $SETUPS $SETUP_L[$n] )
       if ($RES == T31_gx3) then
        if ($PLATFORM == aix) set TASK =  $NTASK_aix_T31_gx3[$n]
        if ($PLATFORM == aix) set THRD =  $NTHRD_aix_T31_gx3[$n]
        if ($PLATFORM == sgi) set TASK =  $NTASK_sgi_T31_gx3[$n]
        if ($PLATFORM == sgi) set THRD =  $NTHRD_sgi_T31_gx3[$n]
        if ($PLATFORM == cpq) set TASK =  $NTASK_cpq_T31_gx3[$n]
        if ($PLATFORM == cpq) set THRD =  $NTHRD_cpq_T31_gx3[$n]
       else
        if ($PLATFORM == aix) set TASK =  $NTASK_aix_T42_gx1v3[$n]
        if ($PLATFORM == aix) set THRD =  $NTHRD_aix_T42_gx1v3[$n]
        if ($PLATFORM == sgi) set TASK =  $NTASK_sgi_T42_gx1v3[$n]
        if ($PLATFORM == sgi) set THRD =  $NTHRD_sgi_T42_gx1v3[$n]
        if ($PLATFORM == cpq) set TASK =  $NTASK_cpq_T42_gx1v3[$n]
        if ($PLATFORM == cpq) set THRD =  $NTHRD_cpq_T42_gx1v3[$n]
       endif
       set NTASKS = ( $NTASKS $TASK )
       set NTHRDS = ( $NTHRDS $THRD )
       set GRIDS  = ( $GRIDS  $GRIDS_L[$n] )
       @ TASKS = $TASKS + $TASK
       @ NODES = $NODES + $TASK * $THRD
       #
       # generate the TASK_GEOmetry layout
       #
       @ COUNT = 0
       if ($PLATFORM == aix) then
         if ($THRD > 4 ) echo "ERROR, NTHRDS maximum is 4, you've set NTHRDS = ( $NTHRDS )"
         if ($THRD > 4 ) exit 1
         @ proc_num = 0  # enforce 2nd req 
         while ($COUNT < $TASK)
           if ($proc_num == 0) set task_geo = "$task_geo($node_num" 
           if ($proc_num != 0) set task_geo = "$task_geo,$node_num" 
           @ proc_num++ 
           if ($proc_num == $proc_max[$THRD]) set task_geo = "$task_geo)"
           if ($proc_num == $proc_max[$THRD]) @ proc_num = 0
           @ node_num++ 
           @ COUNT++
         end # end COUNT loop
         if ($proc_num != 0) set task_geo = "$task_geo)"  # enforce 2nd req 
       endif
     endif
    end
   end
   @ NODES = $NODES / 4
   @ IRIX_TASKS = $TASKS + 4


   #   
   # modify the master template run script
   #

   set AIX_QUEUE  = csl_reg

   set IRIX_QUEUE = ded_32
   set IRIX_TASKS = 36
   foreach conf (A C D H I  )
     if ($CONFIG == $conf) set IRIX_QUEUE = ded_28
     if ($CONFIG == $conf) set IRIX_TASKS = 32
   end # conf
   
   set WALL = 9600
   if ($CONFIG == B || $CONFIG == F || $CONFIG == G  || $CONFIG == M ) set WALL = 21600

   sed "s,test.a1.run,$CASE.run.startup,g;\
        s,# @ wall_clock_limit = .*,## @ wall_clock_limit = ${WALL},;\
        s,node = .*,node = $NODES,;\
        s/task_geometry *=.*/task_geometry = {$task_geo}/g;\
        s,class = .*,class = $AIX_QUEUE,;\
        s,QSUB -q .*,QSUB -q $IRIX_QUEUE -l mpp_p=${IRIX_TASKS},;\
        s,# @ job_name = .*,# @ job_name = $CASE,g;"\
             $DIR/test.a1.run >&! tmp1
  
   sed "s,^ *setenv *CASE    .*#,setenv CASE     $CASE      #,;\
        s,^ *setenv *CASESTR .*#,setenv CASESTR '$AG $OG $COMPS'#,;\
        s,^ *setenv *GRID    .*#,setenv GRID     $RES       #,;\
        s,^ *setenv *RUNTYPE .*#,setenv RUNTYPE  startup    #,;\
        s,^ *setenv *CSMDATA .*#,setenv CSMDATA  $CSMDATA   #,; \
        s,^ *setenv *CSMROOT .*#,setenv CSMROOT  $CSMROOT   #,"\
            tmp1 >&! tmp2  || exit 2
  
  
   sed "s,T42_gx1v3,$RES,;\
        s,^ *setenv *EXEROOT .*#,setenv EXEROOT  $EXEROOT   #,;\
        s,^ *set *MODELS .*#,set MODELS = ( `printf "%5s%5s%5s%5s%5s" $MODELS` )  # generic model names,;\
        s,^ *set *SETUPS .*#,set SETUPS = ( `printf "%5s%5s%5s%5s%5s" $SETUPS` )  # setup script name   ,;\
        s,^ *set *NTASKS .*#,set NTASKS = ( `printf "%5i%5i%5i%5i%5i" $NTASKS` )  # use NTASK = 1 for data model,;\
        s,^ *set *NTHRDS .*#,set NTHRDS = ( `printf "%5i%5i%5i%5i%5i" $NTHRDS` )  # use NTHRD = 1 for data model,;" \
            tmp2 >&! $DIR/$CASE.run  || exit 2
   

   #	    
   # modify the test.a1.har script
   # 	    
   sed "s,# @ job_name = .*,# @ job_name = $CASE,g;"\
             $DIR/test.a1.har >&! tmp3
   mv tmp3  $DIR/$CASE.har

   #
   # Modify cpl.setup.csh for a 1 month startup 
   #
   #  1st, rm unwanted variables from test.a1/cpl.setup.csh,
   #  then change coupler output options to monthly settings
   #

   sed "/diag_n      = /d;/rest_n      = /d;\
        /stop_n      = /d;/hist_n      = /d;\
        /hist_tavg   = /d;/info_bcheck = /d" $DIR/cpl.setup.csh >&! tmpa
  
   sed "s/^ *rest_freq   *= *.*/  rest_freq    = 'monthly' /;\
        s/^ *diag_freq   *= *.*/  diag_freq    = 'monthly' /;\
        s/^ *hist_freq   *= *.*/  hist_freq    = 'monthly' /;\
        s/^ *stop_option *= *.*/  stop_option  = 'newmonth'/; " \
          tmpa >&! $DIR/cpl.setup.csh || exit 8
   #
   # Turn off rm of restart file in dice
   #

   sed "/data_dir/p;s/data_dir.*/mss_rmlf   =  0/g" $DIR/dice.setup.csh >&! tmpa
   rm $DIR/dice.setup.csh
   mv tmpa  $DIR/dice.setup.csh
 
   #
   # Make case dependent changes.
   #

   if ($CONFIG =~ F* || $CONFIG =~ J*) then
     sed "s/set *PRESCRIBED_ICE *= *.*/set PRESCRIBED_ICE = .true./g" $DIR/ice.setup.csh >&! tmpi || exit 8
     sed "s/set *PRESCRIBED_ICE_CLIM  *= *.*/set PRESCRIBED_ICE_CLIM  = .true./g" tmpi >&! $DIR/ice.setup.csh || exit 8
   endif

   if ($CONFIG =~ M* ) then
     sed "s/set *OCEANMIXED_ICE *= *.*/set OCEANMIXED_ICE      = .true./g" $DIR/ice.setup.csh >&! tmpi || exit 8
     cat tmpi >&! $DIR/ice.setup.csh || exit 8
     sed "s/^ *data_year0 *= *.*/  data_year0 =  16 /g" $DIR/dlnd.setup.csh >&! tmpi || exit 8
     sed "s/^ *data_nyear *= *.*/  data_nyear =  16 /g" tmpi >&! $DIR/dlnd.setup.csh || exit 8
     sed "s/^ *data_nyear(1) *= *.*/ data_nyear(1) =    1 /;\
          s/^ *data_nyear(2) *= *.*/ data_nyear(2) =    1 /;\
          s/^ *data_nyear(3) *= *.*/ data_nyear(3) =    1 /" $DIR/latm.setup.csh >&! tmpi || exit 8
     sed "s/^ *ncpl *= *.*/ ncpl          =   24 /g"         tmpi >&! $DIR/latm.setup.csh || exit 8
   endif
 
   #
   # cleanup tmp files and test.a1 files
   #

   rm tmp*
   rm $DIR/test.a1*
 
   #
   # Make the scripts executable.
   #
 
   chmod +x $DIR/*.run* $DIR/*.csh*
 
   echo 0 >! $DIR/RESUBMIT

  end # foreach RESOLUTIONS

  echo Notes: Created Case: $DIR 

 end # foreach CONFIG

end # foreach PLATFORM

exit 0
